/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Master/detail empty selection.
 *
 * @since 2.7
 */
NX.define('Nexus.masterdetail.EmptySelection', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this,
        text = self.emptyText;

    if (!text) {
      text = 'Please select a ' + self.entityType + ' or create a new capability';
    }

    Ext.apply(self, {
      cls: 'nx-masterdetail-EmptySelection',
      title: 'Empty Selection',
      iconCls: self.iconCls,
      html: '<span class="nx-masterdetail-EmptySelection-text">' + text + '</span>'
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Master/detail view.
 *
 * @since 2.7
 */
NX.define('Nexus.masterdetail.MasterDetail', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  /**
   * @property
   */
  master: undefined,

  /**
   * @property
   */
  detail: undefined,

  /**
   * @property
   */
  emptySelection: undefined,

  /**
   * @constructor
   */
  constructor: function (master, detail, emptySelection) {
    var self = this;

    NX.assert(master, 'Missing master');
    NX.assert(detail, 'Missing detail');
    NX.assert(emptySelection, 'Missing emptySelection');

    self.master = master;
    self.detail = detail;
    self.emptySelection = emptySelection;

    self.constructor.superclass.constructor.apply(self, arguments);
  },

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    self.detailPanel = NX.create('Ext.Panel', {
      region: 'south',
      minHeight: 25, // height of panel title
      split: true,
      autoDestroy: false,

      layout: 'card',
      defaults: {
        border: false
      },
      items: [
        self.emptySelection,  // 0
        self.detail           // 1
      ],
      activeItem: 0,
      listeners: {
        // resize detail panel height to 75% of parent on first render
        afterrender: {
          single: true,
          fn: function () {
            self.detailPanel.setHeight(self.getHeight() * 0.6);
          }
        }
      }
    });

    Ext.apply(self, {
      cls: 'nx-masterdetail-MasterDetail',
      layout: 'border',
      border: false,

      items: [
        self.master,
        self.detailPanel
      ]
    });

    self.master.getSelectionModel().on('selectionchange', self.selectionChanged, self);

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  /**
   * Update detail panel when grid selection changes.
   * @param sm    grid selection model
   * @private
   */
  selectionChanged: function (sm) {
    var self = this,
        cardLayout = self.detailPanel.getLayout(),
        selections = sm.getSelections();

    if (selections.length === 0) {
      cardLayout.setActiveItem(0);
    }
    else {
      cardLayout.setActiveItem(1);
      self.detail.updateRecord(selections[0].data);
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability data store.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilityStore', {
  extend: 'Ext.data.JsonStore',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: ['Nexus.siesta'],

  /**
   * @constructor
   */
  constructor: function (config) {
    var self = this,
        ST = Ext.data.SortTypes;

    config = config || {};

    Ext.apply(config, {
      url: Nexus.siesta.basePath + '/capabilities',
      id: 'capability.id',

      fields: [
        { name: 'id', mapping: 'capability.id' },
        { name: 'description', sortType: ST.asUCString },
        { name: 'notes', mapping: 'capability.notes', sortType: ST.asUCString },
        { name: 'enabled', mapping: 'capability.enabled' },
        { name: 'active' },
        { name: 'error' },
        { name: 'typeName' },
        { name: 'typeId', mapping: 'capability.typeId'},
        { name: 'stateDescription' },
        { name: 'status' },
        { name: 'properties', mapping: 'capability.properties' },
        { name: '$capability', mapping: 'capability' },
        { name: '$tags',
          convert: function (newValue, record) {
            return self.convertTags(record.tags);
          }
        }
      ],

      sortInfo: {
        field: 'typeName',
        direction: 'ASC'
      },

      listeners: {
        load: {
          fn: function () {
            self.tagKeys = self.calculateTagKeys();
            this.logDebug('Loaded ' + self.getCount() + ' capabilities');
          },
          scope: self
        }
      }
    });

    self.constructor.superclass.constructor.call(self, config);
  },

  /**
   * Returns the url of a capability given its ID.
   */
  urlOf: function (id) {
    return this.url + '/' + id;
  },

  /**
   * Calculates an array of all available tag keys.
   * @private
   */
  calculateTagKeys: function () {
    var self = this,
        allTagKeys = [];

    self.each(function (record) {
      if (record.data.$tags) {
        for (var key in record.data.$tags) {
          if (allTagKeys.indexOf(key) < 0) {
            allTagKeys.push(key);
          }
        }
      }
    });

    return allTagKeys.sort();
  },

  /**
   * Converts tags collection to an map like object.
   * @private
   */
  convertTags: function (recordTags) {
    var tags = {};

    if (recordTags) {
      Ext.each(recordTags, function (tag) {
        tags[tag.key] = tag.value;
      });
      return tags;
    }
  },

  sameTagKeysAs: function (another) {
    return JSON.stringify(this.tagKeys) === JSON.stringify(another);
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Capabilities  self.mediator().
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitiesMediator', {

  singleton: true,

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: [
    'Nexus.capabilities.Icons',
    'Nexus.capabilities.CapabilityStore',
    'Nexus.capabilities.CapabilityTypeStore',
    'Nexus.capabilities.factory.ComboFactory'
  ],

  /**
   * @property
   */
  capabilityStore: undefined,

  /**
   * @property
   */
  capabilityTypeStore: undefined,

  /**
   * @private
   */
  comboFactory: undefined,

  /**
   * @constructor
   */
  constructor: function () {
    var self = this;

    self.capabilityStore = NX.create('Nexus.capabilities.CapabilityStore');
    self.capabilityTypeStore = NX.create('Nexus.capabilities.CapabilityTypeStore');
    self.comboFactory = Nexus.capabilities.factory.ComboFactory;
  },

  /**
   * Creates a capability via REST.
   */
  addCapability: function (capability, successHandler, failureHandler) {
    var self = this;

    self.logDebug('Adding capability: ' + Ext.encode(capability));

    Ext.Ajax.request({
      url: self.capabilityStore.url,
      method: 'POST',
      scope: self,
      suppressStatus: true,
      jsonData: capability,
      success: successHandler,
      failure: failureHandler
    });
  },

  /**
   * Updates a capability via REST.
   */
  updateCapability: function (capability, successHandler, failureHandler) {
    var self = this;

    self.logDebug('Updating capability: ' + Ext.encode(capability));

    Ext.Ajax.request({
      url: self.capabilityStore.urlOf(capability.id),
      method: 'PUT',
      scope: self,
      suppressStatus: true,
      jsonData: capability,
      success: successHandler,
      failure: failureHandler
    });
  },

  /**
   * Enables a capability via REST.
   */
  enableCapability: function (capability, successHandler, failureHandler) {
    var self = this;

    self.logDebug('Enabling capability: ' + capability.id);

    Ext.Ajax.request({
      url: self.capabilityStore.urlOf(capability.id) + "/enable",
      method: 'PUT',
      scope: self,
      suppressStatus: true,
      success: successHandler,
      failure: failureHandler
    });
  },

  /**
   * Disables a capability via REST.
   */
  disableCapability: function (capability, successHandler, failureHandler) {
    var self = this;

    self.logDebug('Disabling capability: ' + capability.id);

    Ext.Ajax.request({
      url: self.capabilityStore.urlOf(capability.id) + "/disable",
      method: 'PUT',
      scope: self,
      suppressStatus: true,
      success: successHandler,
      failure: failureHandler
    });
  },

  /**
   * Deletes a capability via REST.
   */
  deleteCapability: function (capability, successHandler, failureHandler) {
    var self = this;

    self.logDebug('Deleting capability: ' + capability.id);

    Ext.Ajax.request({
      url: self.capabilityStore.urlOf(capability.id),
      method: 'DELETE',
      scope: self,
      suppressStatus: true,
      success: successHandler,
      failure: failureHandler
    });
  },

  /**
   * Returns a description of capability suitable to be displayed.
   */
  describeCapability: function (capability) {
    var description = capability.typeName;
    if (capability.description) {
      description += ' - ' + capability.description;
    }
    return description;
  },

  /**
   * Shows a message.
   */
  showMessage: function (title, message) {
    Nexus.messages.show(title, message);
  },

  /**
   * Refreshes data stores.
   */
  refresh: function () {
    var self = this;

    self.logDebug('Refreshing stores');

    self.capabilityStore.reload();
    self.capabilityTypeStore.reload();
    self.comboFactory.evictCache();
  },

  /**
   * Handles an REST response, eventually marking fields as invalid.
   * @param response REST response
   * @param [options] response options
   * @param [title] dialog error
   * @param [form] containing fields that should be marked in case of a validation error
   */
  handleError: function (response, options, title, form) {
    var handled = false,
        remainingMessages = [],
        message;

    if (response.siestaValidationError) {
      handled = true;
      Ext.each(response.siestaValidationError, function (error) {
        var marked = false,
            field;

        if (form) {
          field = form.findField('property.' + error.id);
          if (!field) {
            field = form.findField(error.id);
          }
          if (field) {
            marked = true;
            field.markInvalid(error.message);
          }
        }
        if (!marked) {
          remainingMessages.push(error.message);
        }
      });
    }
    if (response.siestaError) {
      handled = true;
      remainingMessages.push(response.siestaError.message);
    }
    if (!handled) {
      if (response.responseText) {
        message = Sonatype.utils.parseHTMLErrorMessage(response.responseText);
      }
      if (!message) {
        message = title + ' (' + response.statusText + ')';
        title = undefined;
      }
      remainingMessages.push(message);
    }
    if (remainingMessages.length > 0) {
      Ext.Msg.show({
        title: title || 'Operation failed',
        msg: remainingMessages.join('\n'),
        buttons: Ext.Msg.OK,
        icon: Ext.MessageBox.ERROR,
        closeable: false
      });
    }
  },

  /**
   * Calculates status label of a capability.
   */
  getStatusLabel: function (capability) {
    var enabled = capability.enabled,
        active = capability.active,
        error = capability.error;

    if (enabled && error) {
      return 'Error';
    }
    if (enabled && active) {
      return 'Active';
    }
    if (enabled && !active) {
      return 'Passive';
    }

    return 'Disabled';
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for icons used by capabilities.
 *
 * @since capabilities 2.2.2
 */
NX.define('Nexus.capabilities.Icons', {
  extend: 'Nexus.util.IconContainer',
  singleton: true,

  /**
   * @constructor
   */
  constructor: function () {
    var self = this;

    // helper to build an icon config with variants, where variants live in directories, foo.png x16 -> x16/foo.png
    function iconConfig(fileName, variants) {
      var config = {};
      if (variants === undefined) {
        variants = ['x32', 'x16'];
      }
      Ext.each(variants, function (variant) {
        config[variant] = variant + '/' + fileName;
      });
      return config;
    }

    self.constructor.superclass.constructor.call(self, {
      stylePrefix: 'nx-capabilities-icon-',

      icons: {
        capability:           iconConfig('brick.png'),
        capability_add:       iconConfig('brick_add.png'),
        capability_delete:    iconConfig('brick_delete.png'),
        capability_new:       iconConfig('brick_edit.png'),
        capability_active:    iconConfig('brick_valid.png'),
        capability_passive:   iconConfig('brick_error.png'),
        capability_disabled:  iconConfig('brick_grey.png'),
        capability_error:     iconConfig('brick_error.png'),
        cross:                'cross.png',
        cross_grey:           'cross_grey.png',
        filter:               'filter.png',
        filter_grey:          'filter_grey.png',
        warning:              'error.png',
        magnifier:            'magnifier.png',
        magnifier_grey:       'magnifier_grey.png',
        refresh:              'arrow_refresh.png',
        selectionEmpty:       '@warning',
        enable:               '@capability_active',
        disable:              '@capability_disabled'
      }
    });
  },

  /**
   * Return the icon for the given capability.
   * @param capability
   * @return {*} Icon; never null/undefined.
   */
  iconFor: function (capability) {
    var self = this,
        typeName = capability.typeName,
        enabled = capability.enabled,
        active = capability.active,
        error = capability.error,
        iconName;

    if (!typeName) {
      iconName = 'capability_new';
    }
    else if (enabled && error) {
      iconName = 'capability_error';
    }
    else if (enabled && active) {
      iconName = 'capability_active';
    }
    else if (enabled && !active) {
      iconName = 'capability_passive';
    }
    else {
      iconName = 'capability_disabled';
    }
    return self.get(iconName);
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Capabilities Mediator mixin.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitiesMediatorMixin', {

  requires: [
    'Nexus.capabilities.CapabilitiesMediator'
  ],

  mediator: function () {
    return Nexus.capabilities.CapabilitiesMediator;
  }

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability type data store.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilityTypeStore', {
  extend: 'Ext.data.JsonStore',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: ['Nexus.siesta'],

  /**
   * @constructor
   */
  constructor: function (config) {
    var self = this,
        ST = Ext.data.SortTypes;

    config = config || {};

    Ext.apply(config, {
      url: Nexus.siesta.basePath + '/capabilities/types',
      id: 'id',

      fields: [
        { name: 'id' },
        { name: 'name' },
        { name: 'about' },
        { name: 'formFields' }
      ],

      sortInfo: {
        field: 'id',
        direction: 'ASC'
      },

      listeners: {
        load: {
          fn: function () {
            this.logDebug('Loaded ' + self.getCount() + ' types');
          },
          scope: self
        }
      }
    });

    self.constructor.superclass.constructor.call(self, config);
  },

  /**
   * Returns capability type given its ID.
   */
  getTypeById: function (id) {
    var record = this.getById(id);

    if (record) {
      return record.data;
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability Status View.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilityStatus', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilityStatus',
      title: 'Status',
      autoScroll: true,
      html: ''
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  /**
   * Sets the current selected capability.
   * @param capability selected
   */
  setCapability: function (capability) {
    var self = this,
        status = 'Capability does not provide any status.';

    if (capability.status) {
      status = capability.status;
    }
    self.html = status;
    if (self.body) {
      self.body.update(status);
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability view.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilityView', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.Icons',
    'Nexus.capabilities.CapabilitySummary',
    'Nexus.capabilities.CapabilitySettings',
    'Nexus.capabilities.CapabilityStatus',
    'Nexus.capabilities.CapabilityAbout'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this,
        icons = Nexus.capabilities.Icons;

    self.messageTpl = NX.create('Ext.XTemplate',
        '<div class="nx-capabilities-CapabilitySummary-message">',
        '  <div>{icon}{html}</div>',
        '</div>',
        {
          compiled: true,

          message: function (capability) {
            var self = this,
                icons = Nexus.capabilities.Icons;

            if (capability.enabled && !capability.active) {
              return self.apply({
                icon: icons.get('warning').img,
                html: '<b>' + capability.stateDescription + '</b>.'
              });
            }
            return '';
          }
        });

    self.summaryView = NX.create('Nexus.capabilities.CapabilitySummary');
    self.settingsView = NX.create('Nexus.capabilities.CapabilitySettings');
    self.statusView = NX.create('Nexus.capabilities.CapabilityStatus');
    self.aboutView = NX.create('Nexus.capabilities.CapabilityAbout');

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilityView',
      header: true,
      border: false,
      layout: 'fit',
      items: {
        xtype: 'tabpanel',
        title: 'Capability',
        iconCls: icons.get('capability').cls,
        items: [
          self.summaryView,
          self.settingsView,
          self.statusView,
          self.aboutView
        ],
        activeTab: 0,
        layoutOnTabChange: true
      }
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  /**
   * Update the capability record.
   * @param capability
   */
  updateRecord: function (capability) {
    var self = this,
        icons = Nexus.capabilities.Icons;

    self.setTitle(
        self.mediator().describeCapability(capability) + self.messageTpl.message(capability),
        icons.iconFor(capability).cls);

    self.summaryView.setCapability(capability);
    self.settingsView.setCapability(capability);
    self.statusView.setCapability(capability);
    self.aboutView.setCapability(capability);
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Capability Settings View.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitySettings', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.CapabilitySettingsFieldSet'
  ],

  /**
   * Current selected capability.
   * @private
   */
  currentRecord: undefined,

  /**
   * Settings field set.
   * @private
   */
  settings: undefined,

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    self.settings = NX.create('Nexus.capabilities.CapabilitySettingsFieldSet', {
      border: false,
      parentPanel: self
    });

    self.formPanel = NX.create('Ext.FormPanel', {
      border: false,
      items: self.settings,
      buttonAlign: 'left',
      buttons: [
        {
          text: 'Save',
          formBind: true,
          scope: self,
          handler: function () {
            self.updateCapability();
          }
        },
        {
          xtype: 'link-button',
          text: 'Discard',
          formBind: false,
          scope: self,
          handler: function () {
            self.setCapability(self.currentRecord);
          }
        }
      ]
    });

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilitySettings',
      title: 'Settings',
      autoScroll: true,
      items: [
        self.formPanel
      ],
      listeners: {
        activate: {
          fn: function () {
            self.setCapability(self.currentRecord);
          },
          scope: self
        }
      }
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  /**
   * Sets the current selected capability.
   * @param capability selected
   */
  setCapability: function (capability) {
    var self = this,
        sp = Sonatype.lib.Permissions,
        editable = sp.checkPermission('nexus:capabilities', sp.EDIT);

    self.currentRecord = capability;

    self.settings.importCapability(self.formPanel.getForm(), capability);

    self.doLayout();
    self.togglePermission(self.items, editable);
  },

  /**
   * Updates capability in Nexus.

   * @private
   */
  updateCapability: function () {
    var self = this,
        form = self.formPanel.getForm(),
        capability;

    if (!form.isValid()) {
      return;
    }

    capability = Ext.apply(self.settings.exportCapability(form), {
      id: self.currentRecord.id,
      notes: self.currentRecord.notes
    });

    self.mediator().updateCapability(capability,
        function () {
          form.items.each(function (item) {
            item.clearInvalid();
          });
          self.mediator().showMessage('Capability saved', self.mediator().describeCapability(self.currentRecord));
          self.mediator().refresh();
        },
        function (response, options) {
          self.mediator().handleError(response, options, 'Capability could not be saved', form);
        }
    );
  },

  /**
   * Enables/disables fields marked with "requiresPermission".
   * @private
   */
  togglePermission: function (items, enabled) {
    var self = this,
        iterable;

    if (items) {
      iterable = items.items;
      if (!iterable) {
        iterable = items;
      }
      Ext.each(iterable, function (item) {
        if (item) {
          if (item.requiresPermission) {
            if (enabled) {
              item.enable();
            }
            else {
              item.disable();
            }
          }
          self.togglePermission(item.items, enabled);
        }
      });
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability grouping data store.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitiesGridStore', {
  extend: 'Ext.data.GroupingStore',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.Icons',
    'Nexus.capabilities.CapabilityStore'
  ],

  /**
   * @constructor
   */
  constructor: function (config) {
    var self = this,
        config = config || {},
        ST = Ext.data.SortTypes,
        fields = [
          { name: 'id' },
          { name: 'description', sortType: ST.asUCString },
          { name: 'notes', sortType: ST.asUCString },
          { name: 'enabled' },
          { name: 'active' },
          { name: 'error' },
          { name: 'typeName' },
          { name: 'typeId'},
          { name: 'stateDescription' },
          { name: 'status' },
          { name: 'properties' },
          { name: '$capability' }
        ];

    self.tagKeys = config.capabilityStore.tagKeys;

    if (self.tagKeys) {
      Ext.each(self.tagKeys, function (key) {
        fields.push({ name: 'tag$' + key, sortType: ST.asUCString });
      });
    }

    Ext.apply(config, {
      reader: NX.create('Ext.data.JsonReader', {
        fields: fields
      }),

      autoDestroy: true,

      sortInfo: {
        field: 'typeName',
        direction: 'ASC'
      }
    });

    self.constructor.superclass.constructor.call(self, config);

    self.loadCapabilities();
  },

  /**
   * Loads all capabilities from configured capability store into this store.
   */
  loadCapabilities: function () {
    var self = this,
        recordType = this.recordType,
        newRecords = [];

    self.capabilityStore.each(function (record) {
      var newRecord,
          overrideConfig = {};

      if (record.data.$tags) {
        for (var key in record.data.$tags) {
          overrideConfig['tag$' + key] = record.data.$tags[key];
        }
      }
      newRecord = new recordType(Ext.apply(Ext.apply({}, record.data), overrideConfig), record.id);
      newRecords.push(newRecord);
    });

    self.loadRecords({records: newRecords}, {});
  },

  /**
   * Returns the column model based on discovered tags.
   */
  getColumnModel: function () {
    var self = this,
        icons = Nexus.capabilities.Icons,
        columns = [];

    columns.push(
        {
          width: 30,
          resizable: false,
          sortable: false,
          fixed: true,
          hideable: false,
          menuDisabled: true,
          renderer: function (value, metaData, record) {
            return icons.iconFor(record.data).img;
          }
        },
        {
          id: 'typeName',
          width: 175,
          header: 'Type',
          dataIndex: 'typeName',
          sortable: true
        }
    );

    if (self.tagKeys) {
      Ext.each(self.tagKeys, function (key) {
        columns.push(
            {
              id: 'tag$' + key,
              width: 100,
              header: key,
              dataIndex: 'tag$' + key,
              sortable: true
            }
        );
      });
    }

    columns.push(
        {
          id: 'description',
          width: 250,
          header: 'Description',
          dataIndex: 'description',
          sortable: true
        },
        {
          id: 'notes',
          width: 175,
          header: 'Notes',
          dataIndex: 'notes',
          sortable: true
        }
    );

    return NX.create('Ext.grid.ColumnModel', {columns: columns});
  },

  /**
   * Extract tag key from name. Returns null if name is not a tag name.
   */
  getTagKeyFrom: function (name) {
    if (name.startsWith('tag$')) {
      return name.substring('tag$'.length);
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability About View.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilityAbout', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilityAbout',
      title: 'About',
      autoScroll: true,
      html: ''
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  /**
   * Sets the current selected capability.
   * @param capability selected
   */
  setCapability: function (capability) {
    var self = this,
        about = '',
        capabilityType = self.mediator().capabilityTypeStore.getTypeById(capability.typeId);

    if (capabilityType) {
      about = capabilityType.about;
    }
    self.html = about;
    if (self.body) {
      self.body.update(about);
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Capabilities master grid.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitiesGrid', {
  extend: 'Ext.grid.GridPanel',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.Icons',
    'Nexus.capabilities.CapabilitiesGridStore',
    'Nexus.capabilities.CreateCapabilityWindow',
    'Nexus.capabilities.CapabilitiesGridFilterBox'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this,
        icons = Nexus.capabilities.Icons;

    self.buttonAdd = NX.create('Ext.Button', {
      text: 'New',
      iconCls: icons.get('capability_add').cls,
      tooltip: 'Add a new capability',
      handler: function () {
        self.addCapability();
      },
      disabled: true
    });

    self.buttonDuplicate = NX.create('Ext.Button', {
      text: 'Duplicate',
      iconCls: icons.get('capability_add').cls,
      tooltip: 'Duplicates selected capability',
      handler: function () {
        var selections = self.getSelectionModel().getSelections();
        if (selections.length > 0) {
          self.duplicateCapability(selections[0].data);
        }
      },
      disabled: true
    });

    self.buttonDelete = NX.create('Ext.Button', {
      text: 'Delete',
      iconCls: icons.get('capability_delete').cls,
      tooltip: 'Delete selected capability',
      handler: function (button) {
        var selections = self.getSelectionModel().getSelections();
        if (selections.length > 0) {
          self.deleteCapability(selections[0].data, button.btnEl);
        }
      },
      disabled: true
    });

    self.gridStore = NX.create('Nexus.capabilities.CapabilitiesGridStore', {
      capabilityStore: self.mediator().capabilityStore
    });

    self.filterBox = NX.create('Nexus.capabilities.CapabilitiesGridFilterBox', {
      filteredGrid: self,
      width: 200
    });

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilityGrid',
      ds: self.gridStore,
      stripeRows: true,
      border: false,

      loadMask: {
        msg: 'Loading...',
        msgCls: 'loading-indicator'
      },

      view: NX.create('Ext.grid.GroupingView', {
        emptyText: 'No capabilities defined',
        emptyTextWhileFiltering: 'No capabilities matched criteria: {criteria}',
        deferEmptyText: false,
        getRowClass: function (record) {
          var capability = record.data;
          if (capability.enabled && !capability.active) {
            return 'red-flag';
          }
        },
        forceFit: true,
        groupTextTpl: '{[values.text.replace("undefined","(none)").replace("null","(none)")]}'
      }),

      sm: NX.create('Ext.grid.RowSelectionModel', {
        singleSelect: true,
        listeners: {
          selectionchange: {
            fn: self.selectionChanged,
            scope: self
          }
        }
      }),

      colModel: self.gridStore.getColumnModel(),
      autoExpandColumn: 'notes',

      tbar: [
        {
          text: 'Refresh',
          tooltip: 'Refresh capabilities',
          iconCls: icons.get('refresh').cls,
          handler: function () {
            self.refresh();
          }
        },
        self.buttonAdd,
        self.buttonDuplicate,
        self.buttonDelete,
        '->',
        self.filterBox
      ],

      listeners: {
        destroy: {
          fn: function () {
            self.mediator().capabilityStore.removeListener('beforeload', self.rememberSelection, self);
            self.mediator().capabilityStore.removeListener('load', self.reconfigureGrid, self);
            self.mediator().capabilityTypeStore.removeListener('beforeload', self.disableAddButton, self);
            self.mediator().capabilityTypeStore.removeListener('load', self.maybeEnableAddButton, self);
          },
          scope: self
        },
        render: {
          fn: function () {
            self.mediator().capabilityStore.on('beforeload', self.rememberSelection, self);
            self.mediator().capabilityStore.on('load', self.reconfigureGrid, self);

            self.mediator().capabilityTypeStore.on('beforeload', self.disableAddButton, self);
            self.mediator().capabilityTypeStore.on('load', self.maybeEnableAddButton, self);
          },
          scope: self
        }
      }

    });

    self.constructor.superclass.initComponent.apply(self, arguments);

    self.refresh();

    self.on('rowcontextmenu', self.showMenu, self);
  },

  /**
   * @private
   */
  selectionChanged: function (sm) {
    var self = this,
        sp = Sonatype.lib.Permissions;

    self.buttonDuplicate.disable();
    self.buttonDelete.disable();
    if (sm.getCount() !== 0) {
      if (sp.checkPermission('nexus:capabilities', sp.CREATE)) {
        self.buttonDuplicate.enable();
      }
      if (sp.checkPermission('nexus:capabilities', sp.DELETE)) {
        self.buttonDelete.enable();
      }
    }
  },

  /**
   * @private
   */
  selectedRecords: undefined,

  /**
   * Remember what records are selected.
   * @private
   */
  rememberSelection: function () {
    this.selectedRecords = this.getSelectionModel().getSelections();
  },

  /**
   * Recall selection from previous remembered selection.
   * @private
   */
  recallSelection: function () {
    var self = this,
        toSelect = [];

    if (self.selectedRecords === undefined || self.selectedRecords.length === 0) {
      return;
    }

    Ext.each(self.selectedRecords, function (record) {
      record = self.getStore().getById(record.id);
      if (!Ext.isEmpty(record)) {
        toSelect.push(record);
      }
    });

    self.getSelectionModel().selectRecords(toSelect);
  },

  /**
   * Refreshes grid and selects a capability by id.
   * @param capabilityId to be selected
   * @private
   */
  refreshAndSelect: function (capabilityId) {
    var self = this;

    self.getSelectionModel().clearSelections();
    self.refresh();
    self.mediator().capabilityStore.on('load',
        function () {
          var record = self.getStore().getById(capabilityId);
          if (!Ext.isEmpty(record)) {
            self.getSelectionModel().selectRecords([record]);
          }
        },
        self, {single: true}
    );
  },

  /**
   * Reconfigures the grid based on information available is grid store.
   * @private
   */
  reconfigureGrid: function () {
    var self = this;

    if (!self.mediator().capabilityStore.sameTagKeysAs(self.gridStore.tagKeys)) {
      self.gridStore = NX.create('Nexus.capabilities.CapabilitiesGridStore', {
        capabilityStore: self.mediator().capabilityStore
      });
      self.gridStore.on('load', self.recallSelection, self);

      self.reconfigure(self.gridStore, self.gridStore.getColumnModel());
    }
    else {
      self.gridStore.loadCapabilities();
    }
  },

  /**
   * Refreshes data stores.
   * @private
   */
  refresh: function () {
    this.mediator().refresh();
  },

  /**
   * Deletes a capability.
   * @param capability to be deleted
   * @private
   */
  deleteCapability: function (capability, animEl) {
    var self = this;

    Ext.Msg.show({
      title: 'Confirm deletion?',
      msg: self.mediator().describeCapability(capability),
      buttons: Ext.Msg.YESNO,
      animEl: animEl,
      icon: Ext.MessageBox.QUESTION,
      closeable: false,
      scope: self,
      fn: function (buttonName) {
        if (buttonName === 'yes' || buttonName === 'ok') {
          self.mediator().deleteCapability(capability,
              function () {
                self.mediator().showMessage('Capability deleted', self.mediator().describeCapability(capability));
                self.refresh();
              },
              function (response, options) {
                self.mediator().handleError(response, options, 'Capability could not be deleted');
                if (response.status === 404) {
                  self.refresh();
                }
              }
          );
        }
      }
    });
  },

  /**
   * Enables a capability.
   * @param capability to be enabled
   * @private
   */
  enableCapability: function (capability) {
    var self = this;

    self.mediator().enableCapability(capability,
        function () {
          self.mediator().showMessage('Capability enabled', self.mediator().describeCapability(capability));
          self.refresh();
        },
        function (response, options) {
          self.mediator().handleError(response, options, 'Capability could not be enabled');
          if (response.status === 404) {
            self.refresh();
          }
        }
    );
  },

  /**
   * Disables a capability.
   * @param capability to be disabled
   * @private
   */
  disableCapability: function (capability) {
    var self = this;

    self.mediator().disableCapability(capability,
        function () {
          self.mediator().showMessage('Capability disabled', self.mediator().describeCapability(capability));
          self.refresh();
        },
        function (response, options) {
          self.mediator().handleError(response, options, 'Capability could not be disabled');
          if (response.status === 404) {
            self.refresh();
          }
        }
    );
  },

  /**
   * Opens create window.
   * @private
   */
  addCapability: function () {
    NX.create(
        'Nexus.capabilities.CreateCapabilityWindow', this.refreshAndSelect.createDelegate(this)
    ).show();
  },

  /**
   * Opens create window with pre-configured values copied from provided capability.
   * @param capability to copy values from
   * @private
   */
  duplicateCapability: function (capability) {
    if (capability) {
      NX.create(
          'Nexus.capabilities.CreateCapabilityWindow', this.refreshAndSelect.createDelegate(this)
      ).show().importCapability(capability);
    }
  },

  /**
   * @private
   * Grid row where context menu was last activated.
   */
  contextMenuRow: undefined,

  /**
   * Shows context menu.
   * @private
   */
  showMenu: function (grid, index, e) {
    var self = this,
        sp = Sonatype.lib.Permissions,
        icons = Nexus.capabilities.Icons,
        row = grid.view.getRow(index),
        capability = self.store.getAt(index).data,
        menu;

    self.hideMenu();

    self.contextMenuRow = row;

    Ext.fly(row).addClass('x-node-ctx');

    menu = new Ext.menu.Menu({
      items: [
        {
          text: 'Refresh',
          iconCls: icons.get('refresh').cls,
          scope: self,
          handler: self.refresh.createDelegate(self)
        }
      ]
    });

    if (sp.checkPermission('nexus:capabilities', sp.EDIT)) {
      menu.add('-');
      if (capability.enabled) {
        menu.add({
          text: 'Disable',
          iconCls: icons.get('disable').cls,
          scope: self,
          handler: self.disableCapability.createDelegate(self, [capability])
        });
      }
      else {
        menu.add({
          text: 'Enable',
          iconCls: icons.get('enable').cls,
          scope: self,
          handler: self.enableCapability.createDelegate(self, [capability])
        });
      }
    }

    if (sp.checkPermission('nexus:capabilities', sp.CREATE)) {
      menu.add('-');
      menu.add({
        text: 'Duplicate',
        iconCls: icons.get('capability_add').cls,
        scope: self,
        handler: self.duplicateCapability.createDelegate(self, [capability])
      });
    }

    if (sp.checkPermission('nexus:capabilities', sp.DELETE)) {
      if (!sp.checkPermission('nexus:capabilities', sp.CREATE)) {
        menu.add('-');
      }
      menu.add({
        text: 'Delete',
        iconCls: icons.get('capability_delete').cls,
        scope: self,
        handler: self.deleteCapability.createDelegate(self, [capability])
      });
    }

    e.stopEvent();

    self.getSelectionModel().selectRow(index, false);

    menu.on('hide', self.hideMenu, self);
    menu.showAt(e.getXY());
  },

  /**
   * Hides context menu.
   * @private
   */
  hideMenu: function () {
    var self = this;

    if (self.contextMenuRow) {
      Ext.fly(self.contextMenuRow).removeClass('x-node-ctx');
      this.contextMenuRow = null;
    }
  },

  /**
   * Disables add button.
   * @private
   */
  disableAddButton: function () {
    var self = this;

    self.buttonAdd.disable();
  },

  /**
   * Enables add button if user has create permission adn there is at least one capability type available.
   * @private
   */
  maybeEnableAddButton: function () {
    var self = this,
        sp = Sonatype.lib.Permissions,
        canCreate = sp.checkPermission('nexus:capabilities', sp.CREATE);

    if (canCreate && self.mediator().capabilityTypeStore.getCount() > 0) {
      self.buttonAdd.enable();
    }
  }

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Window presented to user to create a new capability.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CreateCapabilityWindow', {
  extend: 'Ext.Window',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.Icons',
    'Nexus.capabilities.CapabilitySettingsFieldSet'
  ],

  /**
   * @constructor
   */
  constructor: function (callbackOnSuccess) {
    var self = this;

    self.callbackOnSuccess = callbackOnSuccess;

    self.constructor.superclass.constructor.call(self);
  },

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    self.aboutPanel = NX.create('Ext.Panel', {
      border: false,
      frame: false,
      html: ''
    });

    self.settings = NX.create('Nexus.capabilities.CapabilitySettingsFieldSet', {
      title: 'Settings'
    });

    /**
     * @private
     */
    self.formPanel = NX.create('Ext.form.FormPanel', {
      border: false,
      monitorValid: true,
      labelWidth: 175,
      items: [
        {
          xtype: 'fieldset',
          autoHeight: true,
          collapsed: false,
          border: false,
          items: {
            xtype: 'combo',
            fieldLabel: 'Type',
            itemCls: 'required-field',
            helpText: "Type of configured capability",
            name: 'typeId',
            store: self.mediator().capabilityTypeStore,
            displayField: 'name',
            valueField: 'id',
            forceSelection: true,
            editable: false,
            mode: 'local',
            triggerAction: 'all',
            emptyText: 'Select...',
            selectOnFocus: true,
            allowBlank: false,
            anchor: '96%',
            listeners: {
              select: self.handleCapabilityTypeSelected.createDelegate(self)
            }
          }
        },
        {
          xtype: 'fieldset',
          title: 'About',
          autoHeight: false,
          autoScroll: true,
          collapsible: true,
          collapsed: false,
          items: self.aboutPanel,
          listeners: {
            collapse: function () {
              // HACK: Fix window mask when about panel collapses
              self.fixMask();
            }
          }
        },
        self.settings
      ],

      buttons: [
        {
          text: 'Add',
          formBind: true,
          scope: self,
          handler: self.handleAdd
        },
        {
          xtype: 'link-button',
          text: 'Cancel',
          formBind: false,
          scope: self,
          handler: self.handleCancel
        }
      ],

      keys: [
        {
          // Close dialog on ESC
          key: Ext.EventObject.ESC,
          scope: self,
          fn: self.handleCancel
        }
      ]
    });

    Ext.apply(self, {
      cls: 'nx-capabilities-CreateCapabilityWindow',
      title: 'Create new capability',

      // FIXME: icon isn't properly aligned ATM, disable it for now its distracting
      //iconCls: icons.get('warning').cls,

      width: 640,
      autoHeight: true,
      border: false,

      modal: true,
      constrain: true,
      closable: false,
      resizable: true,

      items: [
        self.formPanel
      ],

      listeners: {
        show: function () {
          // auto-focus comment/description if we are showing it
          //self.formPanel.find('name', 'typeId')[0].focus(false, 100);
        }
      }
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  importCapability: function (capability) {
    var self = this,
        form = self.formPanel.getForm();

    self.settings.importCapability(form, capability);
    self.renderAbout(capability.typeId);
    self.doLayout();
  },

  /**
   * @private
   */
  callbackOnSuccess: undefined,

  /**
   * @private
   */
  aboutPanel: undefined,

  /**
   * @private
   */
  settings: undefined,

  /**
   * HACK: To fix the modal dialog mask.  Unsure how better to do this, needs to be done when the dialog resizes.
   *
   * @private
   */
  fixMask: function () {
    var self = this;
    self.setPosition(self.getPosition());
  },

  /**
   * @private
   */
  handleCapabilityTypeSelected: function (combo) {
    var self = this;

    self.renderAbout(combo.getValue());
    self.settings.setCapabilityType(combo.getValue());
    self.doLayout();

    // HACK: Fix window mask when capability type changes
    self.fixMask();
  },

  /**
   * @private
   */
  renderAbout: function (capabilityTypeId) {
    var self = this,
        about = '',
        capabilityType = self.mediator().capabilityTypeStore.getTypeById(capabilityTypeId);

    if (capabilityType) {
      about = capabilityType.about;
    }
    self.aboutPanel.html = about;
    if (self.aboutPanel.body) {
      self.aboutPanel.body.update(about);
    }
  },

  /**
   * @private
   */
  handleAdd: function () {
    var self = this,
        form = self.formPanel.getForm(),
        mask = NX.create('Ext.LoadMask', self.body, {
          msg: 'Please wait...'
        }),
        capability;

    mask.show();

    capability = self.settings.exportCapability(form);

    self.mediator().addCapability(capability,
        function (response) {
          self.mediator().showMessage(
              'Capability added', self.mediator().describeCapability({typeName: self.settings.capabilityType.name})
          );
          mask.hide();
          self.close();
          if (self.callbackOnSuccess) {
            var responseObj = Ext.decode(response.responseText);
            if (responseObj && responseObj.capability) {
              self.callbackOnSuccess(responseObj.capability.id);
            }
          }
        },
        function (response, options) {
          self.mediator().handleError(response, options, 'Capability could not be created', self.formPanel.getForm());
          mask.hide();
        }
    );
  },

  /**
   * @private
   */
  handleCancel: function () {
    this.close();
  }

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Capabilities master/detail view.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitiesView', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.Icons',
    'Nexus.masterdetail.MasterDetail',
    'Nexus.masterdetail.EmptySelection',
    'Nexus.capabilities.CapabilitiesGrid',
    'Nexus.capabilities.CapabilityView'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this,
        icons = Nexus.capabilities.Icons;

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilitiesView',
      layout: 'border',
      items: NX.create('Nexus.masterdetail.MasterDetail',
          NX.create('Nexus.capabilities.CapabilitiesGrid', {
            region: 'center'
          }),
          NX.create('Nexus.capabilities.CapabilityView'),
          NX.create('Nexus.masterdetail.EmptySelection', {
            iconCls: icons.get('selectionEmpty').cls,
            entityType: 'capability'
          })
      )
    });

    self.constructor.superclass.initComponent.apply(self, arguments);
  }

}, function () {
  var type = this,
      sp = Sonatype.lib.Permissions;

  NX.log.debug('Adding global view: ' + type.$className);

  // install panel into main NX navigation
  Sonatype.Events.on('nexusNavigationInit', function (panel) {
    panel.add({
      enabled: sp.checkPermission('nexus:capabilities', sp.READ),
      sectionId: 'st-nexus-config',
      title: 'Capabilities',
      tabId: 'capabilities',
      tabCode: type
    });

    NX.log.debug('Registered global view: ' + type.$className);
  });
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Capability Summary View.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitySummary', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.Icons'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this,
        sp = Sonatype.lib.Permissions,
        editable = sp.checkPermission('nexus:capabilities', sp.EDIT);

    self.templatePanel = NX.create('Ext.Panel', {
      cls: 'nx-capabilities-CapabilitySummary-template',
      border: false
    });

    self.notesPanel = NX.create('Ext.FormPanel', {
      border: false,
      hideLabels: true,

      items: [
        {
          xtype: 'textarea',
          htmlDecode: true,
          helpText: "Optional notes about configured capability",
          name: 'notes',
          anchor: '-20px',
          allowBlank: true,
          disabled: !editable
        }
      ],

      buttonAlign: 'left',
      buttons: [
        {
          text: 'Save',
          formBind: true,
          scope: self,
          handler: function () {
            self.updateCapability();
          }
        },
        {
          xtype: 'link-button',
          text: 'Discard',
          formBind: false,
          scope: self,
          handler: function () {
            self.setCapability(self.currentRecord);
          }
        }
      ]
    });

    Ext.apply(self, {
      cls: 'nx-capabilities-CapabilitySummary',
      title: 'Summary',
      autoScroll: true,
      items: [
        self.templatePanel,
        {
          xtype: 'fieldset',
          title: 'Notes',
          autoScroll: true,
          collapsed: false,
          hideLabels: true,
          items: self.notesPanel
        }
      ]
    });

    self.initTemplate();

    self.constructor.superclass.initComponent.apply(self, arguments);
  },

  initTemplate: function () {
    var self = this,
        icons = Nexus.capabilities.Icons;

    self.mainTpl = NX.create('Ext.XTemplate',
        '<div class="nx-capabilities-CapabilitySummary-body">',
        '{[ this.status(values) ]}',
        '{[ this.properties(values) ]}',
        '</div>',
        {
          compiled: true,

          status: function (capability) {
            return self.statusTpl.apply(capability);
          },

          properties: function (capability) {
            var properties;

            properties = [
              { name: 'Type', value: capability.typeName }
            ];

            if (capability.description) {
              properties.push({ name: 'Description', value: capability.description });
            }

            if (capability.$tags) {
              for (var key in capability.$tags) {
                properties.push({ name: key, value: capability.$tags[key] });
              }
            }

            return self.propertiesTpl.apply({
              properties: properties
            });
          }

        });

    self.statusTpl = NX.create('Ext.XTemplate',
        '<div class="nx-capabilities-CapabilitySummary-status">',
        '<div class="nx-capabilities-CapabilitySummary-status-icon">',
        '{[ this.statusIcon(values) ]}',
        '</div>',
        '<div class="nx-capabilities-CapabilitySummary-status-label">',
        '{[ this.statusLabel(values) ]}',
        '</div>',
        '</div>',
        {
          compiled: true,

          statusIcon: function (capability) {
            return icons.iconFor(capability).variant('x32').img;
          },

          statusLabel: self.mediator().getStatusLabel.createDelegate(self.mediator())
        });

    self.propertiesTpl = NX.create('Ext.XTemplate',
        '<div class="nx-capabilities-CapabilitySummary-properties">',
        '<table>',
        '<tpl for="properties">',
        '<tr class="nx-capabilities-CapabilitySummary-properties-entry">',
        '<td class="nx-capabilities-CapabilitySummary-properties-name">{name}</td>',
        '<td class="nx-capabilities-CapabilitySummary-properties-value">{value}</td>',
        '</tr>',
        '</tpl>',
        '</tr>',
        '</table>',
        '</div>',
        {
          compiled: true
        });

  },

  /**
   * Sets the current selected capability.
   * @param capability selected
   */
  setCapability: function (capability) {
    var self = this;

    self.currentRecord = capability;
    self.mainTpl.overwrite(self.templatePanel.body, capability);
    self.notesPanel.getForm().setValues(capability);
  },

  /**
   * @private
   */
  currentRecord: undefined,

  /**
   * Updates capability in Nexus.
   * @private
   */
  updateCapability: function () {
    var self = this,
        form = self.notesPanel.getForm(),
        values = form.getFieldValues(),
        capability = Ext.apply(self.currentRecord.$capability, {notes: values.notes});

    self.mediator().updateCapability(capability,
        function () {
          self.mediator().showMessage('Capability saved', self.mediator().describeCapability(self.currentRecord));
          self.mediator().refresh();
        },
        function (response, options) {
          self.mediator().handleError(response, options, 'Capability could not be saved', form);
        }
    );
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX*/

/**
 * 'datefield' factory.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.factory.DateFieldFactory', {

  singleton: true,

  supports: ['datefield', 'date'],

  /**
   * Creates a datefield.
   * @param formField capability type form field to create datefield for
   * @returns {*} created datefield (never null)
   */
  create: function (formField) {
    var item = {
      xtype: 'datefield',
      htmlDecode: true,
      fieldLabel: formField.label,
      itemCls: formField.required ? 'required-field' : '',
      helpText: formField.helpText,
      allowBlank: formField.required ? false : true,
      regex: formField.regexValidation ? new RegExp(formField.regexValidation) : null,
      value: new Date(),
      anchor: '96%'
    };
    if (formField.initialValue) {
      item.value = new Date(Number(formField.initialValue));
    }
    return item;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype*/

/**
 * 'combo' factory.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.factory.ComboFactory', {

  singleton: true,

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  supports: ['combo', 'combobox', 'repo', 'repo-or-group', 'repo-target'],

  /**
   * Map of stores / store url.
   * @private
   */
  stores: {},

  /**
   * Creates a combo.
   * @param formField capability type form field to create combo for
   * @returns {*} created combo (never null)
   */
  create: function (formField) {
    var self = this,
        ST = Ext.data.SortTypes,
        store,
        item = NX.create('Ext.form.ComboBox', {
          xtype: 'combo',
          fieldLabel: formField.label,
          itemCls: formField.required ? 'required-field' : '',
          helpText: formField.helpText,
          name: formField.id,
          displayField: 'name',
          valueField: 'id',
          editable: false,
          forceSelection: true,
          mode: 'local',
          triggerAction: 'all',
          emptyText: 'Select...',
          selectOnFocus: true,
          allowBlank: formField.required ? false : true,
          anchor: '96%'
        });

    if (formField.initialValue) {
      item.value = formField.initialValue;
    }
    if (formField.storePath) {
      store = self.stores[formField.storePath];
      if (!store) {
        store = NX.create('Ext.data.JsonStore', {
          url: Sonatype.config.contextPath + formField.storePath,
          id: formField.idMapping || 'id',
          root: formField.storeRoot,

          fields: [
            { name: 'id', mapping: formField.idMapping || 'id' },
            { name: 'name', mapping: formField.nameMapping || 'name', sortType: ST.asUCString }
          ],

          sortInfo: {
            field: 'name',
            direction: 'ASC'
          },

          autoLoad: true
        });
        self.stores[formField.storePath] = store;
        self.logDebug("Caching store for " + store.url);
      }
      item.store = store;
    }
    return item;
  },

  /**
   * Evicts all cached stores (they will be recreated on demand).
   */
  evictCache: function () {
    var self = this;

    self.logDebug('Evicted all cached stores');
    self.stores = {};
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX*/

/**
 * 'numberfield' factory.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.factory.NumberFieldFactory', {

  singleton: true,

  supports: ['numberfield', 'number'],

  /**
   * Creates a numberfield.
   * @param formField capability type form field to create numberfield for
   * @returns {*} created numberfield (never null)
   */
  create: function (formField) {
    var item = {
      xtype: 'numberfield',
      fieldLabel: formField.label,
      itemCls: formField.required ? 'required-field' : '',
      helpText: formField.helpText,
      allowBlank: formField.required ? false : true,
      regex: formField.regexValidation ? new RegExp(formField.regexValidation) : null,
      anchor: '96%'
    };
    if (formField.initialValue) {
      item.value = Number(formField.initialValue);
    }
    return item;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX*/

/**
 * 'textfield' factory.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.factory.TextFieldFactory', {

  singleton: true,

  supports: ['textfield', 'string', 'password'],

  /**
   * Creates a textfield.
   * @param formField capability type form field to create textfield for
   * @returns {*} created textfield (never null)
   */
  create: function (formField) {
    var item = {
      xtype: 'textfield',
      htmlDecode: true,
      fieldLabel: formField.label,
      itemCls: formField.required ? 'required-field' : '',
      helpText: formField.helpText,
      allowBlank: formField.required ? false : true,
      regex: formField.regexValidation ? new RegExp(formField.regexValidation) : null,
      anchor: '96%'
    };
    if (formField.type === 'password') {
      item.inputType = 'password';
    }
    if (formField.initialValue) {
      item.value = formField.initialValue;
    }
    return item;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX*/

/**
 * 'checkbox' factory.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.factory.CheckboxFactory', {

  singleton: true,

  supports: ['checkbox'],

  /**
   * Creates a checkbox.
   * @param formField capability type form field to create checkbox for
   * @returns {*} created checkbox (never null)
   */
  create: function (formField) {
    var item = {
      xtype: 'checkbox',
      fieldLabel: formField.label,
      helpText: formField.helpText
    };
    if (formField.initialValue) {
      item.checked = Boolean('true' === formField.initialValue);
    }
    return item;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX*/

/**
 * 'textarea' factory.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.factory.TextAreaFactory', {

  singleton: true,

  supports: ['textarea', 'text-area'],

  /**
   * Creates a textarea.
   * @param formField capability type form field to create textarea for
   * @returns {*} created textarea (never null)
   */
  create: function (formField) {
    var item = {
      xtype: 'textarea',
      htmlDecode: true,
      fieldLabel: formField.label,
      itemCls: formField.required ? 'required-field' : '',
      helpText: formField.helpText,
      allowBlank: formField.required ? false : true,
      regex: formField.regexValidation ? new RegExp(formField.regexValidation) : null,
      anchor: '96%'
    };
    if (formField.initialValue) {
      item.value = formField.initialValue;
    }
    return item;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Capability Settings FieldSet.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitySettingsFieldSet', {
  extend: 'Ext.form.FieldSet',

  mixins: [
    'Nexus.LogAwareMixin',
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  requires: [
    'Nexus.capabilities.factory.CheckboxFactory',
    'Nexus.capabilities.factory.ComboFactory',
    'Nexus.capabilities.factory.DateFieldFactory',
    'Nexus.capabilities.factory.NumberFieldFactory',
    'Nexus.capabilities.factory.TextAreaFactory',
    'Nexus.capabilities.factory.TextFieldFactory'
  ],

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    Ext.apply(self, {
      autoHeight: false,
      autoScroll: true,
      collapsed: false,
      labelWidth: 120,
      labelSeparator: '',
      items: []
    });

    self.constructor.superclass.initComponent.apply(self, arguments);

    self.factories = NX.create('Ext.util.MixedCollection');

    self.addFactory(Nexus.capabilities.factory.CheckboxFactory);
    self.addFactory(Nexus.capabilities.factory.ComboFactory);
    self.addFactory(Nexus.capabilities.factory.DateFieldFactory);
    self.addFactory(Nexus.capabilities.factory.NumberFieldFactory);
    self.addFactory(Nexus.capabilities.factory.TextAreaFactory);
    self.addFactory(Nexus.capabilities.factory.TextFieldFactory);
  },

  /**
   * @property
   */
  capabilityType: undefined,

  /**
   * Renders fields for a capability type.
   * @param capabilityTypeId id of capability type to rendered
   */
  setCapabilityType: function (capabilityTypeId) {
    var self = this,
        item;

    self.capabilityType = self.mediator().capabilityTypeStore.getTypeById(capabilityTypeId);

    self.removeAll();

    if (self.capabilityType) {
      self.add({
        xtype: 'checkbox',
        fieldLabel: 'Enabled',
        helpText: 'This flag determines if the capability is currently enabled. To disable this capability for a period of time, de-select this checkbox.',
        name: 'enabled',
        allowBlank: false,
        checked: true,
        editable: true
      });

      if (self.capabilityType.formFields) {
        Ext.each(self.capabilityType.formFields, function (formField) {
          var factory = self.factories.get(formField.type);
          if (!factory) {
            factory = self.factories.get('string');
          }
          if (factory) {
            item = Ext.apply(factory.create(formField), {
              requiresPermission: true,
              name: 'property.' + formField.id,
              factory: factory
            });
            if (item.xtype === 'combo' && item.store) {
              item.store.on('load', function () {
                if (item.store) {
                  item.setValue(item.getValue());
                }
              }, self, {single: true});
            }
            self.add(item);
          }
        });
      }
    }
  },

  /**
   * Exports form as a capability.
   * @param form to be exported
   * @returns {Object} capability
   */
  exportCapability: function (form) {
    var self = this,
        values = form.getFieldValues(),
        value,
        capability = {
          typeId: self.capabilityType.id,
          enabled: values.enabled,
          properties: []
        };

    if (self.capabilityType && self.capabilityType.formFields) {
      Ext.each(self.capabilityType.formFields, function (formField) {
        value = values['property.' + formField.id];
        if (value) {
          capability.properties[capability.properties.length] = {
            key: formField.id,
            value: String(value)
          };
        }
      });
    }

    return capability;
  },

  /**
   * Imports capability into a form.
   * @param form to set values into
   * @param capability to import
   */
  importCapability: function (form, capability) {
    var self = this,
        data = Ext.apply({}, capability);

    self.setCapabilityType(capability.typeId);

    if (self.capabilityType && self.capabilityType.formFields) {
      Ext.each(self.capabilityType.formFields, function (formField) {
        data['property.' + formField.id] = '';
      });
    }

    if (capability.properties) {
      Ext.each(capability.properties, function (property) {
        data['property.' + property.key] = property.value;
      });
    }

    form.setValues(data);
  },

  /**
   * @private
   */
  factories: undefined,

  /**
   * @private
   */
  addFactory: function (factory) {
    var self = this;

    Ext.each(factory.supports, function (supported) {
      self.factories.add(supported, factory);
    });
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * A grid filter box that will also search tag keys for matches.
 * A tag will be a match if matches teh search string and tag has a value.
 *
 * @since 2.7
 */
NX.define('Nexus.capabilities.CapabilitiesGridFilterBox', {
  extend: 'Nexus.ext.GridFilterBox',

  mixins: [
    'Nexus.capabilities.CapabilitiesMediatorMixin'
  ],

  /**
   * @override
   */
  matches: function (filterValue, record, fieldName, fieldValue) {
    var self = this;
    return (fieldValue && self.filteredStore.getTagKeyFrom(fieldName) && self.filterFn(fieldName, filterValue) )
        || Nexus.capabilities.CapabilitiesGridFilterBox.superclass.matches(filterValue, record, fieldName, fieldValue)
        || self.filterFn(self.mediator().getStatusLabel(record.data), filterValue);
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,NX*/
define('nexus-capabilities-plugin-boot', [
  'Nexus/capabilities/CapabilitiesView'
], function () {
  NX.log.debug('Main nexus-capabilities-plugin modules loaded');
});
