/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for icons used by logging.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.Icons', {
  extend: 'Nexus.util.IconContainer',
  singleton: true,

  /**
   * @constructor
   */
  constructor: function () {
    var me = this;

    // helper to build an icon config with variants, where variants live in directories, foo.png x16 -> x16/foo.png
    function iconConfig(fileName, variants) {
      var config = {};
      if (variants === undefined) {
        variants = ['x32', 'x16'];
      }
      Ext.each(variants, function (variant) {
        config[variant] = variant + '/' + fileName;
      });
      return config;
    }

    me.constructor.superclass.constructor.call(me, {
      stylePrefix: 'nx-logging-icon-',

      icons: {
        arrow_refresh: 'arrow_refresh.png',
        download: iconConfig('download.png'),
        book: iconConfig('book.png'),
        book_add: iconConfig('book_add.png'),
        book_delete: iconConfig('book_delete.png'),
        bookmark_red: iconConfig('bookmark_red.png'),
        arrow_undo_orange: iconConfig('arrow_undo_orange.png'),

        logging: '@book',
        logger: '@book',

        loggers_refresh: '@arrow_refresh',
        loggers_add: '@book_add',
        loggers_remove: '@book_delete',
        loggers_reset: '@arrow_undo_orange',

        log_refresh: '@arrow_refresh',
        log_download: '@download',
        log_mark: '@bookmark_red'
      }
    });
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Loggers store.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.store.Logger', {
  extend: 'Ext.data.Store',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: [
    'Nexus.siesta'
  ],

  /**
   * @constructor
   */
  constructor: function () {
    var me = this;

    Ext.apply(me, {
      storeId: 'nx-logging-store-logger',
      autoDestroy: true,
      restful: true,

      sortInfo: { field: 'name', direction: 'ASC' },

      proxy: NX.create('Ext.data.HttpProxy', {
        url: Nexus.siesta.basePath + '/logging/loggers'
      }),

      reader: NX.create('Ext.data.JsonReader', {
        root: '',
        idProperty: 'name',
        fields: [
          'name',
          'level'
        ]
      }),

      writer: NX.create('Ext.data.JsonWriter', {
        encode: false,
        render: function (params, baseParams, data) {
          params.jsonData = data;
        }
      })
    });

    me.constructor.superclass.constructor.call(me);

    Ext.apply(me.reader, {
      // HACK: without this create will fail as it will look for a successProperty in response
      getSuccess: function (obj) {
        return Ext.isDefined(obj);
      }
    });
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Log panel.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.view.Log', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  xtype: 'nx-logging-view-log',

  title: 'Log',
  id: 'nx-logging-view-log',
  cls: 'nx-logging-view-log',
  layout: 'fit',

  /**
   * @override
   */
  initComponent: function () {
    var me = this,
        icons = Nexus.logging.Icons,
        sp = Sonatype.lib.Permissions;

    Ext.apply(me, {
      id: 'nx-logging-view-log',

      items: {
        xtype: 'textarea',
        readOnly: true,
        hideLabel: true,
        emptyText: 'Refresh to display log',
        anchor: '100% 100%'
      },

      tbar: [
        {
          id: 'nx-logging-button-refresh-log',
          text: 'Refresh',
          tooltip: 'Refresh log',
          iconCls: icons.get('log_refresh').cls
        },
        {
          id: 'nx-logging-button-download-log',
          text: 'Download',
          tooltip: 'Download log file',
          iconCls: icons.get('log_download').cls
        },
        '-',
        {
          id: 'nx-logging-button-mark',
          text: 'Mark',
          tooltip: 'Add a mark in Nexus log file',
          iconCls: icons.get('log_mark').cls,
          disabled: !sp.checkPermission('nexus:logconfig', sp.EDIT)
        },
        '->',
        {
          xtype: 'combo',
          id: 'nx-logging-combo-refresh-period',
          triggerAction: 'all',
          lazyRender: true,
          mode: 'local',
          emptyText: 'Select...',
          editable: false,
          value: 0,
          store: NX.create('Ext.data.ArrayStore', {
            id: 0,
            fields: [
              'seconds',
              'text'
            ],
            data: [
              [0, 'Refresh manually'],
              [20, 'Refresh every 20 seconds'],
              [60, 'Refresh every minute'],
              [120, 'Refresh every 2 minutes'],
              [300, 'Refresh every 5 minutes']
            ]
          }),
          valueField: 'seconds',
          displayField: 'text'
        },
        {
          xtype: 'combo',
          id: 'nx-logging-combo-refresh-size',
          triggerAction: 'all',
          lazyRender: true,
          mode: 'local',
          emptyText: 'Select...',
          editable: false,
          width: 90,
          value: 25,
          store: NX.create('Ext.data.ArrayStore', {
            id: 0,
            fields: [
              'kb',
              'text'
            ],
            data: [
              [25, 'Last 25KB'],
              [50, 'Last 50KB'],
              [100, 'Last 100KB']
            ]
          }),
          valueField: 'kb',
          displayField: 'text'
        }
      ]
    });

    me.constructor.superclass.initComponent.apply(me, arguments);

    me.retrieveLogTask = {
      interval: 0,
      started: false,

      changeInterval: function (millis) {
        me.retrieveLogTask.interval = millis;
        me.retrieveLogTask.stop();
        me.retrieveLogTask.start();
      },

      start: function () {
        if (me.retrieveLogTask.run) {
          if (me.retrieveLogTask.interval > 0) {
            Ext.TaskMgr.start(me.retrieveLogTask);
            me.retrieveLogTask.started = true;
            me.logDebug('Started refreshing log every ' + me.retrieveLogTask.interval / 1000 + ' seconds');
          }
          else {
            me.retrieveLogTask.run();
          }
        }
      },

      stop: function () {
        if (me.retrieveLogTask.started) {
          Ext.TaskMgr.stop(me.retrieveLogTask);
          me.retrieveLogTask.started = false;
          me.logDebug('Stopped refreshing log');
        }
      }
    };
  },

  /**
   * Display log content.
   * @param {String} text log content
   */
  showLog: function (text) {
    var textarea = this.down('textarea');
    textarea.setValue(text);
    // scroll to the bottom
    textarea.getEl().dom.scrollTop = 1000000;
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Logger level combo.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.view.LoggerLevel', {
  extend: 'Ext.form.ComboBox',
  xtype: 'nx-logging-combo-logger-level',

  /**
   * @override
   */
  initComponent: function () {
    var me = this;

    Ext.apply(me, {
      triggerAction: 'all',
      lazyRender: true,
      mode: 'local',
      emptyText: 'Select...',
      editable: false,

      store: NX.create('Ext.data.ArrayStore', {
        id: 0,
        fields: [
          'level',
          'text'
        ],
        data: [
          ['TRACE', 'TRACE'],
          ['DEBUG', 'DEBUG'],
          ['INFO', 'INFO'],
          ['WARN', 'WARN'],
          ['ERROR', 'ERROR'],
          ['OFF', 'OFF'],
          ['DEFAULT', 'DEFAULT']
        ]
      }),
      valueField: 'level',
      displayField: 'text'
    });

    me.constructor.superclass.initComponent.apply(me, arguments);
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Add new logger window.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.view.Add', {
  extend: 'Ext.Window',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: [
    'Nexus.logging.Icons'
  ],

  xtype: 'nx-logging-view-add',
  cls: 'nx-logging-view-add',

  title: 'Add logger',

  autoShow: true,
  constrain: true,
  resizable: false,
  width: 500,
  border: false,
  modal: true,

  /**
   * @override
   */
  initComponent: function() {
    var me = this,
        icons = Nexus.logging.Icons,
        loggerNamePattern = /^[a-zA-Z0-9_.]+$/;

    Ext.apply(me, {
      items: [
        {
          xtype: 'panel',
          border: false,
          cls: 'nx-logging-view-add-description',
          html: icons.get('loggers_add').variant('x32').img + '<div>Add a logger with a specific level.</div>'
        },
        {
          xtype: 'form',
          itemId: 'form',
          border: false,
          monitorValid: true,
          layoutConfig: {
            labelSeparator: ''
          },
          items: [
            {
              xtype: 'textfield',
              fieldLabel: 'Logger',
              itemCls: 'required-field',
              helpText: 'Enter a logger name',
              name: 'name',
              allowBlank: false,
              validateOnBlur: false, // allow cancel to be clicked w/o validating this to be non-blank
              anchor: '96%',
              validator: function(value) {
                return loggerNamePattern.test(value);
              },
              invalidText: 'This field should only contain letters, numbers, _ and .'
            },
            {
              xtype: 'nx-logging-combo-logger-level',
              fieldLabel: 'Level',
              itemCls: 'required-field',
              //helpText: 'Select logger level',
              name: 'level',
              value: 'INFO',
              width: 80
            }
          ],

          buttonAlign: 'right',
          buttons: [
            { text: 'Cancel', xtype: 'link-button', handler: me.close, scope: me },
            { text: 'Save', formBind: true, id: 'nx-logging-button-add-save' }
          ]
        }
      ],

      keys: [
        {
          // Save on ENTER
          key: Ext.EventObject.ENTER,
          scope: me,
          fn: function() {
            // fire event only if form is valid
            if (me.getComponent('form').getForm().isValid()) {
              var btn = Ext.getCmp('nx-logging-button-add-save');
              btn.fireEvent('click', btn);
            }
          }
        },
        {
          // Close on ESC
          key: Ext.EventObject.ESC,
          scope: me,
          fn: me.close
        }
      ],

      listeners: {
        show: function(component) {
          component.find('name', 'name')[0].focus(false, 100);
        }
      }
    });

    me.constructor.superclass.initComponent.apply(me, arguments);
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Logging main panel.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.view.Panel', {
  extend: 'Ext.Panel',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: [
    'Nexus.logging.view.Loggers',
    'Nexus.logging.view.Log',
    'Nexus.logging.Icons'
  ],

  xtype: 'nx-logging-view-panel',

  title: 'Logging',

  border: false,
  layout: {
    type: 'vbox',
    align: 'stretch'
  },

  /**
   * @override
   */
  initComponent: function () {
    var me = this,
        icons = Nexus.logging.Icons,
        sp = Sonatype.lib.Permissions,
        items = [],
        tabs = [],
        text = '';

    if (sp.checkPermission('nexus:logconfig', sp.READ)) {
      tabs.push({ xtype: 'nx-logging-view-loggers' });
      text = 'changing logging configuration';
    }
    if (sp.checkPermission('nexus:logs', sp.READ)) {
      tabs.push({ xtype: 'nx-logging-view-log' });
      if (!Ext.isEmpty(text)) {
        text += ' and ';
      }
      text += 'viewing the current log';
    }

    items.push({
      xtype: 'panel',
      border: false,
      cls: 'nx-logging-view-panel-description',
      html: icons.get('logging').variant('x32').img +
          '<div>' + (!Ext.isEmpty(text) ? 'Allows ' + text + '. ' : ''),
      height: 60,
      flex: 0
    });

    if (tabs.length > 0) {
      items.push({
        xtype: 'tabpanel',
        border: false,
        plain: true,
        layoutOnTabChange: true,
        flex: 1,
        items: tabs,
        activeItem: 0
      });
    }

    Ext.apply(me, {
      items: items
    });

    me.constructor.superclass.initComponent.apply(me, arguments);
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Mark log window.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.view.Mark', {
  extend: 'Ext.Window',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: [
    'Nexus.logging.Icons'
  ],

  xtype: 'nx-logging-view-mark',
  cls: 'nx-logging-view-mark',

  title: 'Mark log',

  autoShow: true,
  constrain: true,
  resizable: false,
  width: 500,
  border: false,
  modal: true,

  /**
   * @override
   */
  initComponent: function () {
    var me = this,
        icons = Nexus.logging.Icons;

    Ext.apply(me, {
      items: [
        {
          xtype: 'panel',
          border: false,
          cls: 'nx-logging-view-mark-description',
          html: icons.get('log_mark').variant('x32').img + '<div>Mark the log with a unique message for reference.</div>'
        },
        {
          xtype: 'form',
          itemId: 'form',
          border: false,
          monitorValid: true,
          layoutConfig: {
            labelSeparator: ''
          },
          items: [
            {
              xtype: 'textfield',
              fieldLabel: 'Message',
              itemCls: 'required-field',
              helpText: 'Message to be included in the log',
              name: 'message',
              allowBlank: false,
              validateOnBlur: false, // allow cancel to be clicked w/o validating this to be non-blank
              anchor: '96%'
            }
          ],

          buttonAlign: 'right',
          buttons: [
            { text: 'Cancel', xtype: 'link-button', formBind: false, handler: me.close, scope: me },
            { text: 'Save', formBind: true, id: 'nx-logging-button-mark-save' }
          ]
        }
      ],

      keys: [
        {
          // Save on ENTER
          key: Ext.EventObject.ENTER,
          scope: me,
          fn: function() {
            // fire event only if form is valid
            if (me.getComponent('form').getForm().isValid()) {
              var btn = Ext.getCmp('nx-logging-button-mark-save');
              btn.fireEvent('click', btn);
            }
          }
        },
        {
          // Close on ESC
          key: Ext.EventObject.ESC,
          scope: me,
          fn: me.close
        }
      ],

      listeners: {
        show: function (component) {
          component.find('name', 'message')[0].focus(false, 100);
        }
      }
    });

    me.constructor.superclass.initComponent.apply(me, arguments);
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * Loggers grid.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.view.Loggers', {
  extend: 'Ext.grid.EditorGridPanel',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  requires: [
    'Nexus.logging.Icons',
    'Nexus.logging.store.Logger',
    'Nexus.logging.view.LoggerLevel'
  ],

  xtype: 'nx-logging-view-loggers',

  title: 'Loggers',

  stripeRows: true,
  border: false,
  autoScroll: true,
  clicksToEdit: 1,

  viewConfig: {
    emptyText: 'No loggers defined.',
    emptyTextWhileFiltering: 'No loggers matched criteria: {criteria}',
    deferEmptyText: false
  },

  loadMask: {
    msg: 'Loading...',
    msgCls: 'loading-indicator'
  },

  autoExpandColumn: 'name',

  /**
   * @override
   */
  initComponent: function () {
    var me = this,
        icons = Nexus.logging.Icons,
        sp = Sonatype.lib.Permissions;

    Ext.apply(me, {
      store: NX.create('Nexus.logging.store.Logger'),

      columns: [
        {
          width: 30,
          resizable: false,
          sortable: false,
          fixed: true,
          hideable: false,
          menuDisabled: true,
          renderer: function (value, metaData, record) {
            return icons.get('logger').img;
          }
        },
        {
          id: 'name',
          header: 'Name',
          dataIndex: 'name',
          sortable: true
        },
        {
          id: 'level',
          header: 'Level',
          dataIndex: 'level',
          sortable: true,
          width: 80,
          tooltip: 'Double click to edit',
          editor: {
            xtype: 'nx-logging-combo-logger-level',
            listeners: {
              select: function () {
                // Automatically save when logger level is selected to avoid need for user to press enter or click away
                // in order for value to be saved
                me.stopEditing(false);
              }
            }
          }
        }
      ],

      tbar: [
        {
          id: 'nx-logging-button-refresh-loggers',
          text: 'Refresh',
          tooltip: 'Refresh loggers',
          iconCls: icons.get('loggers_refresh').cls
        },
        {
          id: 'nx-logging-button-add-logger',
          text: 'Add',
          tooltip: 'Add new logger',
          iconCls: icons.get('loggers_add').cls,
          disabled: !sp.checkPermission('nexus:logconfig', sp.EDIT)
        },
        {
          id: 'nx-logging-button-remove-loggers',
          text: 'Remove',
          tooltip: 'Remove selected logger',
          iconCls: icons.get('loggers_remove').cls,
          disabled: true
        },
        '-',
        {
          id: 'nx-logging-button-reset-loggers',
          text: 'Reset',
          tooltip: 'Reset loggers to their default levels',
          iconCls: icons.get('loggers_reset').cls,
          disabled: !sp.checkPermission('nexus:logconfig', sp.EDIT)
        },
        '->',
        {
          xtype: 'nx-grid-filter-box',
          filteredGrid: me
        }
      ]
    });

    me.constructor.superclass.initComponent.apply(me, arguments);
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Logging controller.
 *
 * @since 2.7
 */
NX.define('Nexus.logging.controller.Logging', {
  extend: 'Nexus.controller.Controller',

  requires: [
    'Nexus.siesta',
    'Nexus.logging.Icons',
    'Nexus.logging.view.Panel',
    'Nexus.logging.view.Add',
    'Nexus.logging.view.Mark',
    'Nexus.util.DownloadHelper'
  ],

  init: function () {
    var me = this;

    me.control({
      '#logging': {
        afterrender: me.bindToLoggersGrid,
        activate: me.onLoggingActivate,
        deactivate: me.onLoggingDeactivate,
        beforedestroy: me.onLoggingDeactivate
      },
      '#nx-logging-button-refresh-loggers': {
        click: me.loadLoggers
      },
      '#nx-logging-button-add-logger': {
        click: me.showAddLogger
      },
      '#nx-logging-button-remove-loggers': {
        click: me.removeLogger
      },
      '#nx-logging-button-reset-loggers': {
        click: me.resetLoggers
      },
      '#nx-logging-button-mark': {
        click: me.showMarkLog
      },
      '#nx-logging-button-add-save': {
        click: me.addLogger
      },
      '#nx-logging-button-mark-save': {
        click: me.markLog
      },
      '#nx-logging-view-log': {
        activate: me.onLogTabActivate,
        deactivate: me.onLogTabDeactivate
      },
      '#nx-logging-button-refresh-log': {
        click: me.refreshLog
      },
      '#nx-logging-combo-refresh-period': {
        select: me.changeRefreshPeriod
      },
      '#nx-logging-combo-refresh-size': {
        select: me.changeRefreshSize
      },
      '#nx-logging-button-download-log': {
        click: me.downloadLog
      }
    });

    me.addNavigationMenu();
  },

  /**
   * Adds a menu entry in left navigation bar under config section.
   * @private
   */
  addNavigationMenu: function () {
    // install panel into main NX navigation
    Sonatype.Events.on('nexusNavigationInit', function (panel) {
      var sp = Sonatype.lib.Permissions;

      if (sp.checkPermission('nexus:logconfig', sp.READ) || sp.checkPermission('nexus:logs', sp.READ)) {
        panel.add({
          enabled: true,
          sectionId: 'st-nexus-config',
          title: 'Logging',
          tabId: 'logging',
          tabCode: function () {
            return Ext.create({
              xtype: 'nx-logging-view-panel',
              id: 'logging'
            });
          }
        });
      }
    });
  },

  /**
   * Binds to grid after it has been rendered.
   * @param {Nexus.logging.view.Panel} loggingPanel logging panel to be controlled
   * @private
   */
  bindToLoggersGrid: function (loggingPanel) {
    var me = this,
        grid = loggingPanel.down('nx-logging-view-loggers'),
        store = grid.getStore();

    me.controlSelection(grid);
    store.on('write', me.onSuccessfulWrite, me);
    store.load();
  },

  /**
   * Reloads logger store.
   * @param {Ext.Button} button refresh button that triggered the action (from loggers grid toolbar)
   */
  loadLoggers: function (button) {
    var loggersGrid = button.up('nx-logging-view-loggers'),
        store = loggersGrid.getStore();

    store.load();
  },

  /**
   * Configures handling of grid selection events.
   * @param {Nexus.logging.view.Loggers} loggersGrid logging panel to be controlled
   * @private
   */
  controlSelection: function (loggersGrid) {
    var sp = Sonatype.lib.Permissions,
        removeBtn;

    if (sp.checkPermission('nexus:logconfig', sp.EDIT)) {
      removeBtn = loggersGrid.getTopToolbar().down('#nx-logging-button-remove-loggers');

      loggersGrid.getSelectionModel().on('selectionchange', function (sm, selection) {
        if (sm.hasSelection() && selection.record.get('name') !== 'ROOT') {
          removeBtn.enable();
        }
        else {
          removeBtn.disable();
        }
      });
    }
  },

  /**
   * Shows success messages after records has been successfully written.
   * @private
   */
  onSuccessfulWrite: function (store, action, result, res, rs) {
    if (Ext.isDefined(rs)) {
      Ext.each(rs, function (record) {
        var name = record.get('name');

        if (action === Ext.data.Api.actions.create) {
          Nexus.messages.show('Logging', 'Logger added: ' + name);
        }
        else if (action === Ext.data.Api.actions.update) {
          Nexus.messages.show('Logging', 'Logger updated: ' + name);
        }
        else if (action === Ext.data.Api.actions.destroy) {
          Nexus.messages.show('Logging', 'Logger removed: ' + name);
        }
      });
      store.load();
    }
  },

  /**
   * Opens add logger window.
   * @param {Ext.Button} button add button that triggered the action (from loggers grid toolbar)
   * @private
   */
  showAddLogger: function (button) {
    var win = Ext.create({xtype: 'nx-logging-view-add'});

    win.grid = button.up('nx-logging-view-loggers');
    win.show();
  },

  /**
   * Adds a logger to logger store.
   * @param {Ext.Button} button save button that triggered the action (from add window)
   * @private
   */
  addLogger: function (button) {
    var me = this,
        win = button.up('nx-logging-view-add'),
        store = win.grid.getStore(),
        form = win.down('form').getForm(),
        values = form.getFieldValues(),
        record = store.getById(values.name);

    if (Ext.isDefined(record)) {
      Ext.Msg.show({
        title: 'Confirm',
        msg: 'Logger "' + values.name
            + '" is already configured. Would you like to update its level to "' + values.level + '"?',
        buttons: Ext.Msg.YESNO,
        icon: Ext.MessageBox.QUESTION,
        closeable: false,
        scope: me,
        fn: function (buttonName) {
          if (buttonName === 'yes') {
            record.set('level', values['level']);
            store.save();
            win.grid.getSelectionModel().select(store.indexOf(record), 1);
            win.close();
          }
        }
      });
    }
    else {
      record = new store.recordType(values);
      store.addSorted(record);
      win.grid.getSelectionModel().select(store.indexOf(record), 1);
      win.close();
    }
  },

  /**
   * Remove selected logger (if any)
   * @param {Ext.Button} button delete button that triggered the action (from loggers grid toolbar)
   * @private
   */
  removeLogger: function (button) {
    var loggersGrid = button.up('nx-logging-view-loggers'),
        sm = loggersGrid.getSelectionModel(),
        store = loggersGrid.getStore(),
        icons = Nexus.logging.Icons,
        name;

    // if we have a logger selected, confirm before removal
    if (sm.hasSelection()) {
      name = sm.selection.record.get('name');
      Ext.Msg.show({
        title: 'Remove logger',
        msg: 'Remove "' + name + '" logger ?',
        buttons: Ext.Msg.OKCANCEL,
        icon: icons.get('loggers_remove').variant('x32').cls,
        fn: function (btn) {
          if (btn === 'ok') {
            store.remove(sm.selection.record);
          }
        }
      });
    }
  },

  /**
   * Resets all loggers to their default levels.
   *
   * @param {Ext.Button} button reset button that triggered the action (from loggers grid toolbar)
   * @private
   */
  resetLoggers: function (button) {
    var loggersGrid = button.up('nx-logging-view-loggers'),
        store = loggersGrid.getStore(),
        icons = Nexus.logging.Icons;

    Ext.Msg.show({
      title: 'Reset loggers',
      msg: 'Reset loggers to their default levels?',
      buttons: Ext.Msg.OKCANCEL,
      icon: icons.get('loggers_reset').variant('x32').cls,
      fn: function (btn) {
        if (btn === 'ok') {
          Ext.Ajax.request({
            url: Nexus.siesta.basePath + '/logging/loggers',
            method: 'DELETE',
            suppressStatus: true,
            callback: function () {
              store.load();
            },
            success: function () {
              Nexus.messages.show('Logging', 'Loggers had been reset');
            },
            failure: function (response) {
              Nexus.messages.show('Logging', 'Failed to reset loggers: ' + me.parseExceptionMessage(response));
            }
          });
        }
      }
    });
  },

  /**
   * Opens mark log window.
   * @private
   */
  showMarkLog: function () {
    Ext.create({xtype: 'nx-logging-view-mark'}).show();
  },

  /**
   * Adds a marker to the log by calling PUT on /service/siesta/logging/log/mark REST resource.
   * @param {Ext.Button} button save button that triggered the action (from mark log window)
   * @private
   */
  markLog: function (button) {
    var me = this,
        win = button.up('nx-logging-view-mark'),
        form = win.down('form').getForm(),
        values = form.getFieldValues();

    win.close();

    Ext.Ajax.request({
      url: Nexus.siesta.basePath + '/logging/log/mark',
      method: 'PUT',
      suppressStatus: true,
      jsonData: values,
      success: function () {
        Nexus.messages.show('Logging', 'Log has been marked with: ' + values.message);
        // refresh the log view
        me.retrieveLog(Ext.getCmp('nx-logging-view-log'));
      },
      failure: function (response) {
        Nexus.messages.show('Logging', 'Failed to mark log file: ' + me.parseExceptionMessage(response));
      }
    });
  },

  /**
   * Retrieves the log.
   * @param {Ext.Button} button the refresh button that triggered the action (from log panel toolbar)
   * @private
   */
  refreshLog: function (button) {
    this.retrieveLog(button.up('nx-logging-view-log'));
  },

  /**
   * Opens a new browser window pointing to GET /service/siesta/logging/log.
   * @private
   */
  downloadLog: function () {
    Nexus.util.DownloadHelper.downloadUrl(Nexus.siesta.basePath + '/logging/log');
  },

  /**
   * Starts log retrieving task on log tab activation.
   * @param {Ext.Panel} logPanel that was activated
   * @private
   */
  onLogTabActivate: function (logPanel) {
    var me = this,
        task = logPanel.retrieveLogTask;

    task.run = function () {
      me.retrieveLog(logPanel);
    };
    task.start();
  },

  /**
   * Stops log retrieving task on log tab deactivation.
   * @param {Ext.Panel} logPanel that was deactivated
   * @private
   */
  onLogTabDeactivate: function (logPanel) {
    var task = logPanel.retrieveLogTask;

    task.stop();
    delete task.run;
  },

  /**
   * Starts log retrieving task on logging panel activation if log tab is active.
   * @param {Ext.Panel} panel logging panel that was activated
   * @private
   */
  onLoggingActivate: function (panel) {
    var logPanel = panel.down('nx-logging-view-log');

    if (Ext.isDefined(logPanel)) {
      logPanel.retrieveLogTask.start();
    }
  },

  /**
   * Stops log retrieving task on this panel activation if log tab is active.
   * @param {Ext.Panel} panel logging panel that was deactivated
   * @private
   */
  onLoggingDeactivate: function (panel) {
    var logPanel = panel.down('nx-logging-view-log');

    if (Ext.isDefined(logPanel)) {
      logPanel.retrieveLogTask.stop();
    }
  },

  /**
   * Changes retrieving task recurrence interval when value is changed via refresh period combobox from log panel toolbar.
   * @param {Ext.form.ComboBox} combo refresh period combobox
   * @param {Ext.data.Record} record selected record in combobox
   * @private
   */
  changeRefreshPeriod: function (combo, record) {
    var millis = record.get('seconds') * 1000,
        logPanel = combo.up('nx-logging-view-log'),
        task = logPanel.retrieveLogTask;

    task.changeInterval(millis);
  },

  /**
   * Triggers log retrieval when value is changed via refresh size combobox from log panel toolbar.
   * @param {Ext.form.ComboBox} combo refresh size combobox
   * @private
   */
  changeRefreshSize: function (combo) {
    this.retrieveLog(combo.up('nx-logging-view-log'));
  },

  /**
   * Retrieves log from /service/internal/logs/nexus.log and shows it in log panel.
   * @param {Nexus.logging.view.Log} logPanel where teh log should be shown
   * @private
   */
  retrieveLog: function (logPanel) {
    var me = this,
        size = logPanel.getTopToolbar().down('#nx-logging-combo-refresh-size').getValue(),
        mask;

    mask = NX.create('Ext.LoadMask', logPanel.body, {
      msg: 'Loading...'
    });

    mask.show();

    me.logDebug('Retrieving last ' + size + 'kb from log');

    Ext.Ajax.request({
      url: Nexus.siesta.basePath + '/logging/log',
      method: 'GET',
      headers: {
        'accept': 'text/plain'
      },
      params: {
        bytesCount: -1024 * size
      },
      scope: me,
      suppressStatus: true,
      success: function (response) {
        mask.hide();
        logPanel.showLog(response.responseText);
      },
      failure: function (response) {
        mask.hide();
        logPanel.showLog(
            'Failed to retrieve log due to "' + me.parseExceptionMessage(response) + '".'
        );
      }
    });
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,NX*/
define('nexus-logging-plugin-boot', [
  'Nexus/logging/controller/Logging'
], function () {
  NX.log.debug('Main nexus-logging-plugin modules loaded');

  NX.create('Nexus.logging.controller.Logging').init();
});
