/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for icons used by wonderland plugin.
 *
 * @since 2.7
 */
NX.define('Nexus.wonderland.Icons', {
  extend: 'Nexus.util.IconContainer',
  singleton: true,

  /**
   * @constructor
   */
  constructor: function () {
    var me = this;

    // helper to build an icon config with variants, where variants live in directories, foo.png x16 -> x16/foo.png
    function iconConfig(fileName, variants) {
      var config = {};
      if (variants === undefined) {
        variants = [ 'x32', 'x16' ]
      }
      Ext.each(variants, function(variant) {
        config[variant] = variant + '/' + fileName;
      });
      return config;
    }

    me.constructor.superclass.constructor.call(me, {
      stylePrefix: 'nx-wonderland-icon-',

      icons: {
        lock: iconConfig('lock.png')
      }
    });
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * Authenticate current user button.
 */
NX.define('Nexus.wonderland.AuthenticateButton', {
  extend: 'Ext.Button',

  requires: [
    'Nexus.siesta',
    'Nexus.wonderland.Icons',
    'Nexus.wonderland.view.AuthenticateWindow'
  ],

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  xtype: 'nx-wonderland-button-authenticate',
  cls: 'nx-wonderland-button-authenticate',

  /**
   * @cfg message
   * @type String
   */
  message: 'You have requested an operation which requires validation of your credentials.',

  /**
   * @override
   */
  initComponent: function () {
    var me = this,
        icons = Nexus.wonderland.Icons;

    Ext.apply(me, {
      cls: 'x-btn-text-icon',
      iconCls: icons.get('lock').cls,
      scope: me,
      handler: me.showWindow
    });

    me.constructor.superclass.initComponent.apply(me, arguments);

    // register events
    me.addEvents(
        /**
         * @event authenticated
         * @param {Button} self
         * @param {String} authentication ticket
         */
        'authenticated'
    );
  },

  /**
   * @private
   */
  showWindow: function () {
    var me = this;

    var win = NX.create('Nexus.wonderland.view.AuthenticateWindow', {
      message: me.message,
      animateTarget: me.getEl(),
      listeners: {
        // HACK: propagate event from window to button, controller does not have context of the button
        'authenticated': function(window, authticket) {
          me.fireEvent('authenticated', me, authticket);
        }
      }
    });

    win.show();
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * Window to authenticate current user.
 */
NX.define('Nexus.wonderland.view.AuthenticateWindow', {
  extend: 'Ext.Window',

  requires: [
    'Nexus.wonderland.Icons'
  ],

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  xtype: 'nx-wonderland-view-authenticate',

  title: 'Authenticate',
  id: 'nx-wonderland-view-authenticate',
  cls: 'nx-wonderland-view-authenticate',

  border: false,
  closable: false,
  modal: true,
  height: 190,
  width: 350,
  resizable: false,

  /**
   * @cfg message
   * @type String
   */
  message: 'Authentication required',

  /**
   * @override
   */
  initComponent: function () {
    var me = this,
        icons = Nexus.wonderland.Icons;

    me.passwordField = NX.create('Ext.form.TextField', {
      fieldLabel: 'Password',
      inputType: 'password',
      name: 'password',
      width: 170,
      allowBlank: false,
      validateOnBlur: false // allow cancel to be clicked w/o validating this to be non-blank
    });

    Ext.apply(me, {
      items: [
        {
          xtype: 'form',
          height: 160,
          monitorValid: true,
          layoutConfig: {
            labelSeparator: ''
          },
          labelAlign: 'right',

          items: [
            {
              xtype: 'component',
              cls: 'nx-wonderland-view-authenticate-description',
              html: icons.get('lock').variant('x32').img + '<div>' + me.message +
                  '</div><br style="clear:both;"/>' // clear to get remaining form elements aligned correctly
            },
            {
              xtype: 'textfield',
              fieldLabel: 'Username',
              name: 'username',
              width: 170,
              readOnly: true,
              preventMark: true
            },
            me.passwordField
          ],

          buttons: [
            {
              text: 'Confirm',
              formBind: true,
              scope: me,
              handler: me.fireValidate
            },
            {
              xtype: 'link-button',
              text: 'Cancel',
              formBind: false,
              scope: me,
              handler: me.close
            }
          ],

          keys: [
            {
              // Validate on ENTER
              key: Ext.EventObject.ENTER,
              scope: me,
              fn: me.fireValidate
            },
            {
              // Close dialog on ESC
              key: Ext.EventObject.ESC,
              scope: me,
              fn: me.close
            }
          ]
        }
      ],

      listeners: {
        /**
         * Set username to current user and auto-focus password field.
         */
        show: function (component) {
          component.find('name', 'username')[0].setValue(Nexus.currentUser().username);
          component.find('name', 'password')[0].focus(false, 100);
          me.passwordField.markInvalid('This field is required');
        }
      }
    });

    me.constructor.superclass.initComponent.apply(me, arguments);

    // register events
    me.addEvents(
        /**
         * @event validate-credentials
         * @param {Window} self
         * @param {String} username
         * @param {String} password
         */
        'validate-credentials',

        /**
         * @event authenticated
         * @param {Window} self
         * @param {String} authentication ticket
         */
        'authenticated'
    );
  },

  /**
   * @private
   */
  fireValidate: function () {
    this.fireEvent(
        'validate-credentials',
        this,
        Nexus.currentUser().username,
        this.passwordField.getValue()
    );
  },

  /**
   * @public
   */
  markInvalid: function() {
    this.passwordField.markInvalid('Invalid password');
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus, Sonatype*/

/**
 * File created window.
 *
 * @since 2.8
 */
NX.define('Nexus.wonderland.view.FileCreated', {
  extend: 'Ext.Window',
  xtype: 'nx-wonderland-view-file-created',
  mixins: [
    'Nexus.LogAwareMixin'
  ],
  requires: [
    'Nexus.wonderland.Icons',
    'Nexus.wonderland.AuthenticateButton'
  ],

  /**
   * @cfg Icon to show
   */
  fileIcon: undefined,

  /**
   * @cfg Type of file shown
   */
  fileType: undefined,

  /**
   * @cfg Download button id
   */
  downloadButtonId: undefined,

  cls: 'nx-wonderland-view-file-created',

  autoShow: true,
  constrain: true,
  resizable: false,
  width: 500,
  border: false,
  modal: true,

  /**
   * @override
   */
  initComponent: function () {
    var me = this;

    Ext.apply(me, {
      title: me.title || me.fileType + ' Created',
      items: [
        {
          xtype: 'component',
          border: false,
          cls: 'nx-wonderland-view-file-created-description',
          html: me.fileIcon.img + '<div>' + me.fileType + ' has been created.' +
              '<br/><br/>You can reference this file on the filesystem or download the file from your browser.</div>'
        },
        {
          xtype: 'form',
          itemId: 'form',
          border: false,
          monitorValid: true,
          layoutConfig: {
            labelSeparator: '',
            labelWidth: 40
          },
          labelAlign: 'right',
          items: [
            {
              xtype: 'textfield',
              fieldLabel: 'Name',
              helpText: me.fileType + ' file name',
              name: 'name',
              readOnly: true,
              grow: true,
              style: {
                border: 0,
                background: 'none'
              }
            },
            {
              xtype: 'textfield',
              fieldLabel: 'Size',
              helpText: 'Size of ' + me.fileType + ' file in bytes',  // FIXME: Would like to render in bytes/kilobytes/megabytes
              name: 'size',
              readOnly: true,
              grow: true,
              style: {
                border: 0,
                background: 'none'
              }
            },
            {
              xtype: 'textfield',
              fieldLabel: 'Path',
              helpText: me.fileType + ' file location',
              name: 'file',
              readOnly: true,
              selectOnFocus: true,
              anchor: '96%'
            },
            {
              xtype: 'hidden',
              name: 'truncated'
            }
          ],

          buttonAlign: 'right',
          buttons: [
            { text: 'Close', xtype: 'link-button', handler: me.close, scope: me },
            { text: 'Download', xtype: 'nx-wonderland-button-authenticate', formBind: true, id: me.downloadButtonId }
          ]
        }
      ],

      keys: [
        {
          // Download on ENTER
          key: Ext.EventObject.ENTER,
          scope: me,
          fn: function () {
            var btn = Ext.getCmp(me.downloadButtonId);
            btn.fireEvent('click', btn);
          }
        },
        {
          // Close on ESC
          key: Ext.EventObject.ESC,
          scope: me,
          fn: me.close
        }
      ]
    });

    Nexus.wonderland.view.FileCreated.superclass.initComponent.apply(me, arguments);
  },

  /**
   * Set form values.
   *
   * @public
   */
  setValues: function (values) {
    this.down('form').getForm().setValues(values);
  },

  /**
   * Get form values.
   *
   * @public
   */
  getValues: function () {
    return this.down('form').getForm().getValues();
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Wonderland controller.
 *
 * @since 2.7
 */
NX.define('Nexus.wonderland.controller.Wonderland', {
  extend: 'Nexus.controller.Controller',

  requires: [
    'Nexus.siesta'
  ],

  init: function () {
    var me = this;

    me.control({
      '#nx-wonderland-view-authenticate': {
        'validate-credentials': me.validateCredentials
      }
    });
  },

  /**
   * @private
   *
   * @param {AuthenticateWindow} authwin
   * @param {String} username
   * @param {String} password
   */
  validateCredentials: function (authwin, username, password) {
    var me = this, mask;

    mask = new Ext.LoadMask(authwin.getEl(), {
      msg: 'Authenticating'
    });
    mask.show();

    Ext.Ajax.request({
      url: Nexus.siesta.basePath + '/wonderland/authenticate',
      method: 'POST',
      suppressStatus: [
        403 // Used to signal from the server unauthenticated state, don't display an error box for this response
      ],

      jsonData: {
        u: Sonatype.utils.base64.encode(username),
        p: Sonatype.utils.base64.encode(password)
      },

      scope: me,

      callback: function () {
        mask.hide();
      },

      success: function (response) {
        me.logDebug('Authenticated');

        var authTicket = Ext.decode(response.responseText);
        me.logDebug('Ticket: ' + authTicket.t);

        // fire event and close window
        authwin.fireEvent('authenticated', me, authTicket.t);
        authwin.close();
      },

      failure: function (response) {
        if (response.status === 403) {
          authwin.markInvalid();
        }
      }
    });
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,NX*/
define('nexus-wonderland-plugin-boot', [
  'Nexus/wonderland/controller/Wonderland'
],
function () {
  NX.log.debug('Module loaded: nexus-wonderland-plugin-boot');

  NX.create('Nexus.wonderland.controller.Wonderland').init();
});

