/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX */

define('NX/assert', [], function() {

  Ext.ns('NX');

  /**
   * Exception thrown when assertion is triggered.
   *
   * @param message
   * @constructor
   *
   * @since 2.4
   */
  function AssertError(message) {
    this.name = 'AssertError';
    this.message = message || 'Assertion failure';
  }

  AssertError.prototype = new Error();
  AssertError.prototype.constructor = AssertError;

  /**
   * Assertion helpers.
   */
  Ext.apply(NX, {

    /**
     * Assert a condition.  When the condition expression is false an exception is thrown.
     *
     * @param expression    Condition expression
     * @param message       Exception message
     *
     * @since 2.4
     */
    assert : function(expression, message) {
      if (!expression) {
        throw new AssertError(message);
      }
    }

  });

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX,Nexus */

define('NX/log', ['NX/base'], function() {
  Ext.ns('NX');

  /**
   * Logging helper.
   *
   * @singleton
   */
  NX.log = (function () {
      var logger;

      // FIXME: Really would like a better/more-powerful logging API here
      // FIXME: Including the ability to remote back to the server to capture UI events
      //
      // http://log4javascript.org/index.html
      // http://www.gscottolson.com/blackbirdjs
      // http://log4js.berlios.de

      logger = {
          enabled: true,

          levels: {
              trace:  false,
              debug:  false,
              group:  false,
              info:   true,
              warn:   true,
              error:  true
          },

          isEnabled: function(level) {
              return this.enabled && (this.levels[level] || NX.global.location.search === '?debug');
          }

          // FIXME: Add ExtJS 4.x compatible(ish) log() helper
          //log: function(/*[options],[message]*/) {
          //}
      };

      /**
       * @private
       * @param target {Object} The logging implementation
       * @param name {String} The logger to use
       * @param isLoggerEnabled {Function} A function that returns true when the logger for target/name is enabled, false otherwise.
       * @return {Function} The logger.
       */
      function createLogger(target, name, isLoggerEnabled) {
        var log;
        if (Ext.isDefined(target)) {
          if (Ext.isFunction(target[name])) { // most browser have proper console.log, console.debug, ... as functions
            log = function() {
              if (isLoggerEnabled()) {
                target[name].apply(target, arguments);
              }
            };
          } else if (Ext.isDefined(target[name])) { // IE9: console.log is reported as an object, not a function, and does not have #apply
            log = function() {
              if (isLoggerEnabled()) {
                var args = [];
                Ext.each(arguments, function(item) {
                  args.push(item);
                });
                target[name](args.join(' '));
              }
            };
          } else if (Ext.isDefined(target.log)) { // if e.g. console.debug is not available, use console.log
            log = createLogger(target, 'log', isLoggerEnabled);
          } else { // nothing we can do, don't log.
            log = Ext.emptyFn;
          }
        }
        return log;
      }

      function safeProxy(target, name) {
        return createLogger(target, name, function() {
          return logger.enabled;
        });
      }

      function levelAwareProxy(target, name) {
        return createLogger(target, name, function() {
          return logger.isEnabled(name);
        });
      }

      if ( !NX.global.console ) { // IE without webdev has no console object, so we create an empty one
        NX.global.console = {
          log : Ext.emptyFn
        };
      } else if ( !NX.global.console.log ) { // We rely on existence of console.log below
        NX.global.console.log = Ext.emptyFn;
      }

      Ext.each([
          'trace',
          //'log', skipping; as we may want to make an Ext.log compatible method
          'debug',
          'group',
          'info',
          'warn',
          'error'
      ], function (name) {
          logger[name] = levelAwareProxy(NX.global.console, name);
      });

      Ext.each([
          'groupEnd'
      ], function (name) {
          logger[name] = safeProxy(NX.global.console, name);
      });

      return logger;
  }());

  NX.log.debug('Logging initialized');

  // Compatibility
  Ext.ns('Nexus');
  Nexus.Log = NX.log;
  Nexus.log = NX.log.debug;

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX */

define('NX/define', ['NX/base', 'NX/assert', 'NX/log'], function() {
  Ext.ns('NX');

  /**
   * Used as the super-class for all NX.define classes which not not provide a class to extend.
   * Do not put anything special in here, since not everything will use this as its super-class.
   *
   * @constructor
   *
   * @private
   */
  NX.Base = function (config) {
      Ext.apply(this, config);
  };

  /**
   * Define a new class, in Extjs-4 compatible manner.
   *
   * @static
   *
   * @param {String} className        The name of the class to define.
   * @param {Object} data             Configuration for the class.
   * @param {Function} [createdFn]    Function to execute when class has been defined.
   *
   * @cfg {String} extend             Super-class name.
   * @cfg {*} statics                 Static members.
   * @cfg {Array} mixins              Mixin class names, does not support object configuration like extjs4 ATM.
   * @cfg {*} requirejs               Array of requirejs module dependencies, custom... defs not extjs4 compatible.
   * @cfg {*} requires                Array of class names (created by Ext.define) which are dependencies.
   * @cfg {boolean} requireSuper      Flag to enable/disable automatic dependency on super-class;
   *                                  Tries to auto-detect but just inc-ase you can enable/disable it.
   * @cfg {String} xtype              Automatically register class as Ext xtype component.
   * @cfg {boolean} singleton         Defines a singleton instead of a class.
   *                                  Given class name will reference singleton object and createdFn 'this' will be the singleton object.
   */
  NX.define = function (className, data, createdFn) {
      data = data || {};

      var i,
          nameSpace,
          simpleClassName,
          superName,
          type,
          requireSuper,
          superClass,
          statics,
          mixins,
          mixin,
          requiredClassNames,
          requiredModulePaths,
          moduleName,
          singleton,
          xtype,
          tmp;

      // Find the namespace (if any) for the new class
      i = className.lastIndexOf('.');
      if (i !== -1) {
          nameSpace = className.substring(0, i);
          simpleClassName = className.substring(i + 1);
      }
      else {
          simpleClassName = className;
      }

      requireSuper = data.requireSuper;
      delete data.requireSuper;

      // Determine the super-class
      if (data.extend !== undefined) {
          superName = data.extend;
          delete data.extend;

          // require super module if there is not already a defined class of that name for legacy support
          if (requireSuper === undefined) {
              requireSuper = !NX.isobj(superName);
          }
      }
      else {
          superName = 'NX.Base';
          requireSuper = false;
      }

      statics = data.statics;
      delete data.statics;

      mixins = data.mixins;
      delete data.mixins;

      singleton = data.singleton;
      delete data.singleton;

      xtype = data.xtype;
      delete data.xtype;

      // Extract requirejs dependencies
      requiredModulePaths = NX.arrayify(data.requirejs);
      delete data.requirejs;

      // Extract class dependencies (which were defined using Ext.define)
      requiredClassNames = NX.arrayify(data.requires);
      delete data.requires;

      // Require super if enabled
      if (requireSuper === true) {
          requiredClassNames.push(superName);
      }

      // Require mixins
      if (mixins !== undefined) {
          for(i=0; i<mixins.length; i++) {
              requiredClassNames.push(mixins[i]);
          }
      }

      // append translated dependency classes on to required modules
      for (i=0; i < requiredClassNames.length; i++) {
          tmp = requiredClassNames[i].replaceAll('.', '/');
          requiredModulePaths.push(tmp);
      }

      // Translate class name into module name
      moduleName = className.replaceAll('.', '/');

      if (requiredModulePaths.length !== 0) {
          NX.log.debug('Defining module:', moduleName, 'depends:', requiredModulePaths);
      }
      else {
          NX.log.debug('Defining module:', moduleName);
      }

      define(moduleName, requiredModulePaths, function()
      {
          NX.log.debug('Defining class:', className, 'super:', superName);

          // Sanity check required classes exist
          Ext.each(requiredClassNames, function(className) {
              NX.assert(NX.isobj(className), 'Missing required class: ' + className);
          });

          // Create namespace if required
          if (nameSpace) {
              Ext.namespace(nameSpace);
          }

          // Get a reference to the super-class
          superClass = NX.obj(superName);

          // When no constructor given in configuration (its always there due to picking up from Object.prototype), use a synthetic version
          if (data.constructor === Object.prototype.constructor) {
              data.constructor = function () {
                  superClass.prototype.constructor.apply(this, arguments);
              };
          }

          // Create the sub-class
          type = Ext.extend(superClass, data);

          // Remember class name (full and simple)
          type.$className = className;
          type.$simpleClassName = simpleClassName;
          type.prototype.$className = className;
          type.prototype.$simpleClassName = simpleClassName;

          // replace toString if its the default Object.toString
          if (type.prototype.toString === Object.prototype.toString) {
              type.prototype.toString = function() {
                  return '[object ' + className + ']';
              };
          }

          // Handle mixins
          if (mixins !== undefined) {
              for(i=0; i<mixins.length; i++) {
                  mixin = mixins[i]; // name
                  NX.log.debug('Applying mixin:', mixin);
                  mixin = NX.obj(mixin); // class ref
                  Ext.applyIf(type.prototype, mixin.prototype);
              }
          }

          // Apply any static members
          if (statics !== undefined) {
              Ext.apply(type, statics);
          }

          // Register xtype for class
          if (xtype !== undefined) {
              Ext.reg(xtype, type);
          }


          // When singleton; type becomes new instance
          // NOTE: `type` is passed into the creation callback function later on, so the variable must hold the actual instance for singletons
          if (singleton !== undefined) {
              type = new type();
          }

          // Assign to global namespace
          NX.obj(nameSpace)[simpleClassName] = type;


          // Call post-define hook
          if (createdFn !== undefined) {
              // Scope to created type, empty args seems to be required here
              createdFn.call(type, []);
          }

          return type;
      });
  };

  // Compatibility
  Ext.define = NX.define;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('NX/bootstrap', [
  'NX/base',
  'NX/define',
  'NX/log'
]);
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX,global */

define('NX/base', ['require'], function(require) {
  Ext.ns('NX');

  /**
   * Basic framework helpers.
   *
   * @singleton
   */
  Ext.apply(NX, {
      /**
       * Global object reference; never undefined.
       *
       * @type {Object}
       */
      global: (function() {
          if (window !== undefined) {
              return window;
          }
          if (global !== undefined) {
              return global;
          }
          throw new Error('Unable to determine global object');
      }()), // assign return value of function

      /**
       * Reference an object by its global name.
       *
       * @param path
       * @return {*}
       * @throws Error    No object at path
       */
      obj: function (path) {
          var context = NX.global;
          if (typeof path === 'string') {
            Ext.each(path.split('.'), function (part) {
                context = context[part];
                if (context === undefined) {
                    throw new Error('No object at path: ' + path + '; part is undefined: ' + part);
                }
            });
          }
          return context;
      },

      /**
       * Check if an object exists by its global name.
       *
       * @param path
       * @return {boolean}    True if it exists, false if it does not.
       */
      isobj: function(path) {
          var context = NX.global;
          Ext.each(path.split('.'), function (part) {
              context = context[part];
              if (context === undefined) {
                  return false; // break
              }
          });
          return context !== undefined;
      },

      /**
       * Turns input into an array of strings.
       *
       * @param input
       * @return {Array}
       */
      arrayify: function (input) {
          var i, list = [];

          if (Ext.isArray(input)) {
              for (i = 0; i < input.length; i++) {
                  if (Ext.isString(input[i])) {
                      list.push(input[i]);
                  }
                  else {
                      throw new Error('Invalid entry: ' + input[i]);
                  }
              }
          }
          else if (Ext.isString(input)) {
              list.push(input);
          }
          else if (input !== undefined) {
              throw new Error('Invalid value: ' + input);
          }

          return list;
      },

      /**
       * Helper to construct an object from a constructor.
       *
       * @param {Function} constructor
       * @param {Array} args
       * @return {*}
       */
      construct: function (constructor, args) {
          function F() {
              return constructor.apply(this, args);
          }

          F.prototype = constructor.prototype;
          return new F();
      },

      /**
       * Create a new instance of a class.
       *
       * @param name
       * @return {*}
       */
      create: function (name /*, [arg1,arg2,...,argN] */) {
          var args,
              type;

          // pull out varargs, strip off class name
          args = Array.prototype.slice.call(arguments);
          args.shift();

          NX.log.debug('Creating instance:', name);

          // Get a reference to the class
          type = NX.obj(name);

          // Require the class if its not defined
          if (type === undefined) {
              type = require(name.replaceAll('.', '/'));
          }

          return NX.construct(type, args);
      }

  });
});

