/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('ext/layout/absolutelayout', ['extjs'], function(Ext) {
  /**
   * Override the absolute layout to optionally omit the defined extraCls (x-abs-layout-item).
   * The class forces the 'left' CSS position property, which will disable positioning items
   * from the right side.
   */
  Ext.layout.AbsoluteLayout.override({
    renderItem : function(c, position, target) {
      Ext.layout.AbsoluteLayout.superclass.renderItem.call(this, c, position, target);
      if (c.noExtraClass) {
        c.removeClass(this.extraCls);
      }
    }
  });
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/form/TextField',['extjs', 'Nexus/util'], function(Ext, Nexus) {
Ext.override(Ext.form.TextField, {
  /**
   * @cfg {Boolean} htmlDecode
   * <tt>true</tt> to decode html entities in the value given to
   * Ext.form.TextField.setValue and Ext.form.TextField.setRawValue
   * before setting the actual value.
   * <p/>
   * This is needed for displaying the 'literal' value in the text field when it was received by the server,
   * for example in the repository name. The REST layer will encode to html entities, which will be correct
   * for html rendering, but text fields without this configuration will display '&quot;test&quot;' instead
   * of the originally sent '"test"'.
   */
  htmlDecode : false,

  /**
   * @cfg {Boolean} htmlConvert
   * <tt>true</tt> to decode html entities in the value given to
   * Ext.form.TextField.set(Raw)Value
   * before setting the actual value, and encode html entities again
   * in the call to Ext.form.TextField.get(Raw)Value.
   * <p/>
   * This is needed for displaying the 'literal' value in the text field when it was received by the server
   * (see htmlDecode configuration doc), and display to the user correctly before round-tripping to the server again
   * (e.g. in a grid field).
   * <p/>
   * when this config is set, the value has to be html-decoded again before sending it to the server, because the REST layer
   * will encode the string again.
   * <p/>
   * Default value is false.
   */
  htmlConvert : false,

  setRawValueOrig : Ext.form.TextField.prototype.setRawValue,
  setValueOrig : Ext.form.TextField.prototype.setValue,
  getRawValueOrig : Ext.form.TextField.prototype.getRawValue,
  getValueOrig : Ext.form.TextField.prototype.getValue,

  setRawValue : function(value) {
    if ( this.htmlDecode || this.htmlConvert )
    {
      value = Nexus.util.Format.htmlDecode(value);
    }
    this.setRawValueOrig(value);
  },
  setValue : function(value) {
    if ( this.htmlDecode || this.htmlConvert )
    {
      value = Nexus.util.Format.htmlDecode(value);
    }
    this.setValueOrig(value);
  },
  getRawValue : function() {
    var value = this.getRawValueOrig();
    if ( this.htmlConvert )
    {
      value = Ext.util.Format.htmlEncode(value);
    }
    return value;
  },
  getValue : function() {
    var value = this.getValueOrig();
    if ( this.htmlConvert )
    {
      value = Ext.util.Format.htmlEncode(value);
    }
    return value;
  }
});
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/form/VTypes',['extjs'], function(Ext) {
Ext.apply(Ext.form.VTypes, {
  password : function(val, field) {
    if (field.initialPasswordField !== undefined && field.initialPasswordField !== null && field.initialPasswordField !== 'undefined')
    {
      var pwd = field.ownerCt.find('name', field.initialPasswordField)[0].getValue();
      return (val === pwd);
    }
    return true;
  },

  passwordText : 'Passwords do not match.'
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('ext/form/formpanel', ['extjs'], function(Ext) {
  Ext.override(Ext.FormPanel, {
    buttonAlign : 'center'
  });
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('ext/form/displayfield',['extjs'], function(Ext) {
Ext.form.TimestampDisplayField = Ext.extend(Ext.form.DisplayField, {
  setValue : function(v) {
    if (typeof v !== 'number') {
      this.setRawValue(v);
      return this;
    }

    // java give the timestamp in miliseconds, extjs consumes it in seconds
    var toSecs = Math.round(v / 1000);
    v = new Date.parseDate(toSecs, 'U').toString();
    this.setRawValue(v);
    return this;
  }
});

Ext.reg('timestampDisplayField', Ext.form.TimestampDisplayField);

Ext.form.ByteDisplayField = Ext.extend(Ext.form.DisplayField, {
  setValue : function(v) {
    if (typeof v !== 'number') {
      this.setRawValue(v);
      return this;
    }

    if (v < 1024)
    {
      v = v + ' Bytes';
    }
    else if (v < 1048576)
    {
      v = (v / 1024).toFixed(2) + ' KB';
    }
    else if (v < 1073741824)
    {
      v = (v / 1048576).toFixed(2) + ' MB';
    }
    else
    {
      v = (v / 1073741824).toFixed(2) + ' GB';
    }
    this.setRawValue(v);
    return this;
  }
});

Ext.reg('byteDisplayField', Ext.form.ByteDisplayField);

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/form/field',['extjs','sonatype'], function(Ext, Sonatype) {
Ext.override(Ext.form.Field, {
  /*
   * Override default form field rendering to include help text quick tip on
   * question mark rendered after field label.
   * Specifying 'helpText' will cause a help icon and text to be added as a QuickTip.
   * Specifying 'persistent' in conjunction with 'helpTest' will create a ToolTip that remains visible, useful for embedding links in the text.
   */
  afterRenderOrig : Ext.form.Field.prototype.afterRender,
  afterRender : function() {
    var helpClass = null, wrapDiv = null, helpMark = null, self = this;
    if ( this.helpMarker === true )
    {
      wrapDiv = this.getEl().up('div');
      helpClass = 'form-label-helpmark';
    }
    else if (this.getXType() === 'combo' || this.getXType() === 'uxgroupcombo' || this.getXType() === 'datefield' || this.getXType() === 'timefield')
    {
      wrapDiv = this.getEl().up('div.x-form-field-wrap');
      helpClass = 'form-label-helpmark-combo';
    }
    else if (this.getXType() === 'checkbox')
    {
      wrapDiv = this.getEl().up('div.x-form-check-wrap');
      helpClass = 'form-label-helpmark-check';
    }
    else if (this.getXType() === 'textarea')
    {
      wrapDiv = this.getEl().up('div.x-form-element');
      helpClass = 'form-label-helpmark-textarea';
    }
    else if ( this.getXType() === 'textentrylist' )
    {
      wrapDiv = this.getEl().up();
      helpClass = 'form-label-helpmark';
    }
    else
    {
      wrapDiv = this.getEl().up('div.x-form-element');
      helpClass = 'form-label-helpmark';
    }

    // @todo: afterText doesn't work with combo boxes!
    if (this.afterText)
    {
      if ( !wrapDiv ) {
        wrapDiv = this.getEl().wrap({cls:'x-form-element'});
      }

      wrapDiv.createChild({
        tag : 'span',
        cls : 'form-label-after-field',
        html : this.afterText
      });
    }

    if (this.helpText)
    {
      if ( !wrapDiv ) {
        wrapDiv = this.getEl().wrap({cls:'x-form-element'});
      }

      helpMark = wrapDiv.createChild({
        tag : 'img',
        src : Sonatype.config.resourcePath + '/static/icons/glyph_help.png',
        width : 16,
        height : 16,
        cls : helpClass
      });

      // Use a ToolTip instead of QuickTip if we want the tip to remain in place.
      // In this case the entire html fo the ToolTip will be set to the provided
      // 'helpText'.
      if(this.persistent)
      {
        var help = new Ext.ToolTip({
          html :  self.helpText,
          anchor: 'left',
          hideDelay: 10000
        });
        help.initTarget(helpMark);
      }
      else
      {
        Ext.QuickTips.register({
          target : helpMark,
          title : '',
          text : this.helpText,
          enabled : true
        });
      }
    }

    // original method
    this.afterRenderOrig.apply(this, arguments);
  }

});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('ext/form/textarea',['extjs'], function(Ext) {
Ext.override(Ext.form.TextArea, {
  wordWrap : true,
  onRenderOriginal : Ext.form.TextArea.prototype.onRender,
  onRender : function(ct, position) {
    this.onRenderOriginal.apply(this, arguments);

    this.el.setOverflow('auto');
    if (this.wordWrap === false)
    {
      if (!Ext.isIE)
      {
        this.el.set({
          wrap : 'off'
        });
      }
      else
      {
        this.el.dom.wrap = "off";
      }
    }
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/form/basicform',['extjs'], function(Ext) {
Ext.override(Ext.form.BasicForm, {
  clearInvalid : function() {
    // same as before, but ignore items without clearInvalid (== non-form-items)
    this.items.each(function(f) {
      if (f.clearInvalid) {
        f.clearInvalid();
      }
    });
  },
  /**
   * Override findField to look for enabled field and return that, otherwise
   * return first found
   */
  findField : function(id) {
    var
          findMatchingField,
          field = this.items.get(id),
          fallbackField = null;

    if (!Ext.isObject(field)) {
      //searches for the field corresponding to the given id. Used recursively for composite fields
      findMatchingField = function(f) {
        if (f.isFormField) {
          if (f.dataIndex === id || f.id === id || f.getName() === id) {
            fallbackField = f;
            if (!f.disabled) {
              field = f;
              return false;
            }
          } else if (f.isComposite) {
            return f.items.each(findMatchingField);
          } else if (f instanceof Ext.form.CheckboxGroup && f.rendered) {
            return f.eachItem(findMatchingField);
          }
        }
      };

      this.items.each(findMatchingField);
    }
    return field || fallbackField;
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/all',
      ['extjs', 'ext/ux', 'ext/data', 'ext/dd', 'ext/form',
        'ext/layout', 'ext/lib', 'ext/tree', 'ext/component',
        'ext/tooltip', 'ext/util', 'ext/element' ],
      function(Ext) {
        Ext.BLANK_IMAGE_URL = "static/ext-3.4.1.1/resources/images/default/s.gif";
        return Ext;
      }
);

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/tooltip',['extjs'], function(Ext){
// some special tooltip config to reuse same tooltip for whole grid
Ext.override(Ext.ToolTip, {
  onTargetOver : function(e) {
    if (this.disabled || e.within(this.target.dom, true))
    {
      return;
    }
    var t = e.getTarget(this.delegate);
    if (t)
    {
      this.triggerElement = t;
      this.clearTimer('hide');
      this.targetXY = e.getXY();
      this.delayShow();
    }
  },
  onMouseMove : function(e) {
    var t = e.getTarget(this.delegate);
    if (t)
    {
      this.targetXY = e.getXY();
      if (t === this.triggerElement)
      {
        if (!this.hidden && this.trackMouse)
        {
          this.setPagePosition(this.getTargetXY());
        }
      }
      else
      {
        this.hide();
        this.lastActive = new Date(0);
        this.onTargetOver(e);
      }
    }
    else if (!this.closable && this.isVisible())
    {
      this.hide();
    }
  },
  hide : function() {
    this.clearTimer('dismiss');
    this.lastActive = new Date();
    delete this.triggerElement;
    Ext.ToolTip.superclass.hide.call(this);
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global require*/
define('ext/data/connection',['extjs'], function(Ext) {
Ext.override(Ext.data.Connection, {
  extendFormUploadURL : function(url) {
    // hack for IE: if a non success response is received, we can't
    // access the response data, because an error page is loaded from local disc
    // for most response codes, making the containing iframe protected because it has
    // a different domain (CORS)
    if (Ext.isIE) {
      if (url.indexOf('?') >= 0) {
        url += '&forceSuccess=true';
      }
      else {
        url += '?forceSuccess=true';
      }
    }
    return url;
  },
  doFormUploadOrig : Ext.data.Connection.prototype.doFormUpload,
  doFormUpload : function(o, ps, url) {
    this.doFormUploadOrig.call(this, o, ps, this.extendFormUploadURL(url));
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global require*/

define('ext/data/store',['extjs'], function(Ext){
// Extension to the store to allow for multi field sorting
// from http://www.extjs.com/forum/showthread.php?t=48324
Ext.override(Ext.data.Store, {
  /**
   * Sort by multiple fields in the specified order.
   *
   * @param {Array}
        *          An Array of field sort specifications, or, if ascending sort
   *          is required on all columns, an Array of field names. A field
   *          specification looks like: { field: 'orderNumber', direction:
   *          'ASC' }
   */
  sortByFields : function(fields) {
    // Collect sort type functions,
    // Convert string field names to field+direction spec objects.
    var st = [], i, fn;
    for (i = 0; i < fields.length; i=i+1)
    {
      if (typeof fields[i] === 'string')
      {
        fields[i] = {
          field : fields[i],
          direction : 'ASC'
        };
      }
      st.push(this.fields.get(fields[i].field).sortType);
    }

    fn = function(r1, r2) {
      var result, i, v1, v2;
      for (i = 0; !result && i < fields.length; i=i+1)
      {
        v1 = st[i](r1.data[fields[i].field]);
        v2 = st[i](r2.data[fields[i].field]);
        result = (v1 > v2) ? 1 : ((v1 < v2) ? -1 : 0);
        if (fields[i].direction === 'DESC') {
          result = -result;
        }
      }
      return result;
    };
    this.data.sort('ASC', fn);
    if (this.snapshot && this.snapshot !== this.data)
    {
      this.snapshot.sort('ASC', fn);
    }
    this.fireEvent("datachanged", this);
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/menu', ['extjs', 'Nexus/error/ErrorHandler'], function(Ext, ErrorHandler){
  // HACK we cannot replace the menu class to check cfg.id, because initComponent has a static reference to Ext.menu.Menu -> recursion
  Ext.override(Ext.menu.Menu, {
    initComponentOrig : Ext.menu.Menu.prototype.initComponent,
    initComponent : function() {
      // HACK check if id was generated by Ext
      if (this.id.indexOf('ext-') !== 0 &&
            // weed out legitimate, ExtJS-internal use
            (this.id.indexOf('-hctx') !== this.id.length - 5 && this.id.indexOf('-hcols-menu') !== this.id.length - 11 )) {
        ErrorHandler.handleError(new Error('Menu config with fixed id leads to duplicate menu items: ' + this.id));
      }
      this.initComponentOrig.apply(this, arguments);
    }
  });

  return Ext.menu.Menu;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/component',['extjs'], function(Ext){
Ext.override(Ext.Component, {
  stateful : false,
  saveState : function() {}
});
});

/*global define*/
define('ext/ux/gridvalidator',['extjs'], function(Ext){
// handy grid validation from
// http://www.extjs.com/forum/showthread.php?t=21158
Ext.namespace('Ext.ux', 'Ext.ux.plugins');
/**
 * EditorGrid validation plugin Adds validation functions to the grid
 *
 * @author Jozef Sakalos, aka Saki
 * @version 0.1 Usage: grid = new Ext.grid.EditorGrid( { plugins:new
 *          Ext.ux.plugins.GridValidator(), ... } )
 */
Ext.ux.plugins.GridValidator = function(config) {
  // initialize plugin
  this.init = function(grid) {
    Ext.apply(grid, {
      /**
       * Checks if a grid cell is valid
       *
       * @param {Integer}
            *          col Cell column index
       * @param {Integer}
            *          row Cell row index
       * @return {Boolean} true = valid, false = invalid
       */
      isCellValid : function(col, row) {
        if (!this.colModel.isCellEditable(col, row))
        {
          return true;
        }
        var ed = this.colModel.getCellEditor(col, row);
        if (!ed)
        {
          return true;
        }
        var record = this.store.getAt(row);
        if (!record)
        {
          return true;
        }
        var field = this.colModel.getDataIndex(col);
        ed.field.setValue(record.data[field]);
        return ed.field.isValid(true);
      },
      /**
       * Checks if grid has valid data
       *
       * @param {Boolean}
            *          editInvalid true to automatically start editing of the
       *          first invalid cell
       * @return {Boolean} true = valid, false = invalid
       */
      isValid : function(editInvalid) {
        var cols = this.colModel.getColumnCount();
        var rows = this.store.getCount();
        var r, c;
        var valid = true;
        for (r = 0; r < rows; r++)
        {
          for (c = 0; c < cols; c++)
          {
            valid = this.isCellValid(c, r);
            if (!valid)
            {
              break;
            }
          }
          if (!valid)
          {
            break;
          }
        }
        if (editInvalid && !valid)
        {
          this.getSelectionModel().selectRow(r);
          this.startEditing(r, c);
        }
        return valid;
      }
    });
  };
};
});

/**
 * Ext.ux.form.LovCombo, List of Values Combo
 *
 * @author    Ing. Jozef Sakáloš
 * @copyright (c) 2008, by Ing. Jozef Sakáloš
 * @date      16. April 2008
 * @version   $Id: lovcombo.js 285 2008-06-06 09:22:20Z jozo $
 *
 * @license lovcombo.js is licensed under the terms of the Open Source
 * LGPL 3.0 license. Commercial use is permitted to the extent that the 
 * code/component(s) do NOT become part of another Open Source or Commercially
 * licensed development library or toolkit without explicit permission.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 */
 
/*global define*/

define('ext/ux/lovcombo',['extjs'], function(Ext){

// add RegExp.escape if it has not been already added
if('function' !== typeof RegExp.escape) {
	RegExp.escape = function(s) {
		if('string' !== typeof s) {
			return s;
		}
		// Note: if pasting from forum, precede ]/\ with backslash manually
		return s.replace(/([.*+?\^=!:${}()|\[\]\/\\])/g, '\\$1');
	}; // eo function escape
}

// create namespace
Ext.ns('Ext.ux.form');
 
/**
 *
 * @class Ext.ux.form.LovCombo
 * @extends Ext.form.ComboBox
 */
Ext.ux.form.LovCombo = Ext.extend(Ext.form.ComboBox, {

	// {{{
    // configuration options
	/**
	 * @cfg {String} checkField name of field used to store checked state.
	 * It is automatically added to existing fields.
	 * Change it only if it collides with your normal field.
	 */
	 checkField:'checked'

	/**
	 * @cfg {String} separator separator to use between values and texts
	 */
    ,separator:','

	/**
	 * @cfg {String/Array} tpl Template for items. 
	 * Change it only if you know what you are doing.
	 */
	// }}}
    // {{{
    ,initComponent:function() {
        
		// template with checkbox
		if(!this.tpl) {
			this.tpl = 
				 '<tpl for=".">'
				+'<div class="x-combo-list-item">'
				+'<img src="' + Ext.BLANK_IMAGE_URL + '" '
				+'class="ux-lovcombo-icon ux-lovcombo-icon-'
				+'{[values.' + this.checkField + '?"checked":"unchecked"' + ']}">'
				+'<div class="ux-lovcombo-item-text">{' + (this.displayField || 'text' )+ '}</div>'
				+'</div>'
				+'</tpl>'
			;
		}
 
        // call parent
        Ext.ux.form.LovCombo.superclass.initComponent.apply(this, arguments);

		// install internal event handlers
		this.on({
			 scope:this
			,beforequery:this.onBeforeQuery
			,blur:this.onRealBlur
		});

		// remove selection from input field
		this.onLoad = this.onLoad.createSequence(function() {
			if(this.el) {
				var v = this.el.dom.value;
				this.el.dom.value = '';
				this.el.dom.value = v;
			}
		});
 
    } // e/o function initComponent
    // }}}
	// {{{
	/**
	 * Disables default tab key bahavior
	 * @private
	 */
	,initEvents:function() {
		Ext.ux.form.LovCombo.superclass.initEvents.apply(this, arguments);

		// disable default tab handling - does no good
		this.keyNav.tab = false;

	} // eo function initEvents
	// }}}
	// {{{
	/**
	 * clears value
	 */
	,clearValue:function() {
		this.value = '';
		this.setRawValue(this.value);
		this.store.clearFilter();
		this.store.each(function(r) {
			r.set(this.checkField, false);
		}, this);
		if(this.hiddenField) {
			this.hiddenField.value = '';
		}
		this.applyEmptyText();
	} // eo function clearValue
	// }}}
	// {{{
	/**
	 * @return {String} separator (plus space) separated list of selected displayFields
	 * @private
	 */
	,getCheckedDisplay:function() {
		var re = new RegExp(this.separator, "g");
		return this.getCheckedValue(this.displayField).replace(re, this.separator + ' ');
	} // eo function getCheckedDisplay
	// }}}
	// {{{
	/**
	 * @return {String} separator separated list of selected valueFields
	 * @private
	 */
	,getCheckedValue:function(field) {
		field = field || this.valueField;
		var
          c = [],
          // store may be filtered so get all records
          snapshot = this.store.snapshot || this.store.data;

		snapshot.each(function(r) {
			if(r.get(this.checkField)) {
				c.push(r.get(field));
			}
		}, this);

		return c.join(this.separator);
	} // eo function getCheckedValue
	// }}}
	// {{{
	/**
	 * beforequery event handler - handles multiple selections
	 * @param {Object} qe query event
	 * @private
	 */
	,onBeforeQuery:function(qe) {
		qe.query = qe.query.replace(new RegExp(this.getCheckedDisplay() + '[ ' + this.separator + ']*'), '');
	} // eo function onBeforeQuery
	// }}}
	// {{{
	/**
	 * blur event handler - runs only when real blur event is fired
	 */
	,onRealBlur:function() {
		this.list.hide();
		var
          rv = this.getRawValue(),
          rva = rv.split(new RegExp(RegExp.escape(this.separator) + ' *')),
          va = [],
          snapshot = this.store.snapshot || this.store.data;

		// iterate through raw values and records and check/uncheck items
		Ext.each(rva, function(v) {
			snapshot.each(function(r) {
				if(v === r.get(this.displayField)) {
					va.push(r.get(this.valueField));
				}
			}, this);
		}, this);
		this.setValue(va.join(this.separator));
		this.store.clearFilter();
	} // eo function onRealBlur
	// }}}
	// {{{
	/**
	 * Combo's onSelect override
	 * @private
	 * @param {Ext.data.Record} record record that has been selected in the list
	 * @param {Number} index index of selected (clicked) record
	 */
	,onSelect:function(record, index) {
        if(this.fireEvent('beforeselect', this, record, index) !== false){

			// toggle checked field
			record.set(this.checkField, !record.get(this.checkField));

			// display full list
			if(this.store.isFiltered()) {
				this.doQuery(this.allQuery);
			}

			// set (update) value and fire event
			this.setValue(this.getCheckedValue());
            this.fireEvent('select', this, record, index);
        }
	} // eo function onSelect
	// }}}
	// {{{
	/**
	 * Sets the value of the LovCombo
	 * @param {Mixed} v value
	 */
	,setValue:function(v) {
		if(v) {
			v = String(v);
			if(this.valueField) {
				this.store.clearFilter();
				this.store.each(function(r) {
					var checked = !(!v.match(
						 '(^|' + this.separator + ')' + RegExp.escape(r.get(this.valueField))
						+'(' + this.separator + '|$)'))
					;

					r.set(this.checkField, checked);
				}, this);
				this.value = this.getCheckedValue();
				this.setRawValue(this.getCheckedDisplay());
				if(this.hiddenField) {
					this.hiddenField.value = this.value;
				}
			}
			else {
				this.value = v;
				this.setRawValue(v);
				if(this.hiddenField) {
					this.hiddenField.value = v;
				}
			}
			if(this.el) {
				this.el.removeClass(this.emptyClass);
			}
		}
		else {
			this.clearValue();
		}
	} // eo function setValue
	// }}}
	// {{{
	/**
	 * Selects all items
	 */
	,selectAll:function() {
        this.store.each(function(record){
            // toggle checked field
            record.set(this.checkField, true);
        }, this);

        //display full list
        this.doQuery(this.allQuery);
        this.setValue(this.getCheckedValue());
    } // eo full selectAll
	// }}}
	// {{{
	/**
	 * Deselects all items. Synonym for clearValue
	 */
    ,deselectAll:function() {
		this.clearValue();
    } // eo full deselectAll 
	// }}}

}); // eo extend
 
// register xtype
Ext.reg('lovcombo', Ext.ux.form.LovCombo); 
});

/*global define*/
define('ext/ux/CheckColumn', ['extjs'], function(Ext) {

  /*
  This file is part of Ext JS 3.4

  Copyright (c) 2011-2012 Sencha Inc

  Contact:  http://www.sencha.com/contact

  Commercial Usage
  Licensees holding valid commercial licenses may use this file in accordance with the Commercial
  Software License Agreement provided with the Software or, alternatively, in accordance with the
  terms contained in a written agreement between you and Sencha.

  If you are unsure which license is appropriate for your use, please contact the sales department
  at http://www.sencha.com/contact.

  Build date: 2012-07-26 18:41:31 (${git.current.hash})
  */
  Ext.ns('Ext.ux.grid');

  /**
   * @class Ext.ux.grid.CheckColumn
   * @extends Ext.grid.Column
   * <p>A Column subclass which renders a checkbox in each column cell which toggles the truthiness of the associated data field on click.</p>
   * <p><b>Note. As of ExtJS 3.3 this no longer has to be configured as a plugin of the GridPanel.</b></p>
   * <p>Example usage:</p>
   * <pre><code>
  var cm = new Ext.grid.ColumnModel([{
         header: 'Foo',
         ...
      },{
         xtype: 'checkcolumn',
         header: 'Indoor?',
         dataIndex: 'indoor',
         width: 55
      }
  ]);

  // create the grid
  var grid = new Ext.grid.EditorGridPanel({
      ...
      colModel: cm,
      ...
  });
   * </code></pre>
   * In addition to toggling a Boolean value within the record data, this
   * class toggles a css class between <tt>'x-grid3-check-col'</tt> and
   * <tt>'x-grid3-check-col-on'</tt> to alter the background image used for
   * a column.
   */
  Ext.ux.grid.CheckColumn = Ext.extend(Ext.grid.Column, {

      /**
       * @private
       * Process and refire events routed from the GridView's processEvent method.
       */
      processEvent : function(name, e, grid, rowIndex, colIndex){
          if (name == 'mousedown' && !this.readOnly) {
              var record = grid.store.getAt(rowIndex);
              record.set(this.dataIndex, !record.data[this.dataIndex]);
              return false; // Cancel row selection.
          } else {
              return Ext.grid.ActionColumn.superclass.processEvent.apply(this, arguments);
          }
      },

      renderer : function(v, p, record){
          p.css += ' x-grid3-check-col-td';
          return String.format('<div class="x-grid3-check-col{0}">&#160;</div>', v ? '-on' : '');
      },

      // Deprecate use as a plugin. Remove in 4.0
      init: Ext.emptyFn
  });

  // register ptype. Deprecate. Remove in 4.0
  Ext.preg('checkcolumn', Ext.ux.grid.CheckColumn);

  // backwards compat. Remove in 4.0
  Ext.grid.CheckColumn = Ext.ux.grid.CheckColumn;

  // register Column xtype
  Ext.grid.Column.types.checkcolumn = Ext.ux.grid.CheckColumn;
});

/*
 * <sonatypeNote> this javascript was found here
 * http://extjs.com/forum/showthread.php?t=28115 </sonatypeNote>
 * MultiSelectTreePanel v 1.1 This work is derivative of Ext-JS 2.2. Much of the
 * code is modified versions of default code. Refer to Ext-JS 2.2 licencing for
 * more information. http://extjs.com/license Any and all original code is made
 * available as is for whatever purpose you see fit. Should be a largely drop in
 * replacement for ordinary TreePanel when you require multiselect with drag and
 * drop. Overrides most of the methods and events to pass a nodelist rather than
 * a single node. Note that the code is provided as-is and should be considered
 * experimental and likely to contain bugs, especially when combined with other
 * extensions or modifications to the default library. It has been tested
 * against Ext-JS 2.2 and 2.2.1 with: Firefox 3, Opera 9.5+, Safari 3.1, MSIE
 * 6,7,8rc1 (+5.5 seems to work too) Usage: Add the following CSS to make the
 * floating "drag" version of the tree indent prettily.. .x-dd-drag-ghost
 * .x-tree-node-indent,.x-dd-drag-ghost .x-tree-ec-icon {display: inline
 * !important;} If you are using Ext-JS 2.2.1 or earlier you need to add this
 * override! (reported as a bug) Ext.override(Ext.tree.TreeDropZone, {
 * completeDrop : function(de){ var ns = de.dropNode, p = de.point, t =
 * de.target; if(!Ext.isArray(ns)){ ns = [ns]; } var n, node, ins = false; if (p !=
 * 'append'){ ins = true; node = (p == 'above') ? t : t.nextSibling; } for(var i =
 * 0, len = ns.length; i < len; i++){ n = ns[i]; if (ins){
 * t.parentNode.insertBefore(n, node); }else{ t.appendChild(n); }
 * if(Ext.enableFx && this.tree.hlDrop){ n.ui.highlight(); } } ns[0].ui.focus();
 * t.ui.endDrop(); this.tree.fireEvent("nodedrop", de); } }); Instantiate like a
 * normal tree (except DD stuff is enabled by default) var tree = new
 * Ext.ux.MultiSelectTreePanel({ autoScroll:true, width:400, height:500,
 * animate:true, containerScroll: true, enableDD: true, root: new
 * Ext.tree.AsyncTreeNode({ text: 'A Book', draggable:false, id:'node0' }),
 * loader: new Ext.tree.TreeLoader({ dataUrl:'bookdata.json' }) });
 * tree.render("target"); When listening for DND events look for dragdata.nodes
 * instead of dragdata.node Use ctrl-click to select multiple nodes. Use
 * shift-click to select a range of nodes. Changelog v1.0 Initial Release v1.1 -
 * reinstated enableDD, enableDrag, enableDrop config params. *NEED TO INCLUDE
 * THIS NOW* - consolidated compareDocumentPosition code into compareNodeOrder
 * (only works with rendered nodes) - cleaned up select function by above and
 * creating selectNode function. - cleaned up DDGhost generation code to be less
 * hacky (still not ideal) - included onContainerOver and onContainerDrop code
 * (awaiting ExtJS fix) - fixed several lingering postdrag selection bugs -
 * fixed key events to respect shift/ctrl keys Enjoy
 */

define('ext/ux/multiselecttree',['extjs'], function(Ext){

Ext.ux.FixedMultiSelectionModel = Ext.extend(Ext.tree.MultiSelectionModel, {
      // disabled tracking of mouse clicks because it doubles up drag
      // selection...
      onNodeClick : function(node, e) {
        if (e.shiftKey)
          e.preventDefault();
        // this.select(node);
      },

      // private
      // for comparing node order... (taken from quirksmode.org and googlecode)
      compareNodeOrder : document.compareDocumentPosition ? function(node1, node2) {
        // W3C DOM lvl 3 method (Gecko)
        return 3 - (node1.ui.elNode.compareDocumentPosition(node2.ui.elNode) & 6);
      } : (typeof document.documentElement.sourceIndex !== "undefined" ? function(node1, node2) {
        // IE source index method
        return node1.ui.elNode.sourceIndex - node2.ui.elNode.sourceIndex;
      } : function(node1, node2) {
        if (node1 == node2)
          return 0;
        // Safari doesn't support compareDocumentPosition or sourceIndex
        // from
        // http://code.google.com/p/doctype/wiki/ArticleNodeCompareDocumentOrder
        var range1 = document.createRange();
        range1.selectNode(a.ui.elNode);
        range1.collapse(true);

        var range2 = document.createRange();
        range2.selectNode(b.ui.elNode);
        range2.collapse(true);

        return range1.compareBoundaryPoints(Range.START_TO_END, range2);
      }),

      // private
      sortSelNodes : function() {
        if (this.selNodes.length > 1)
        {
          if (!this.selNodes[0].ui.elNode)
            return;
          this.selNodes.sort(this.compareNodeOrder);
        }
      },

      // private single point for selectNode
      selectNode : function(node, push) {
        if (!this.isSelected(node))
        {
          this.selNodes.push(node);
          this.selMap[node.id] = node;
          node.ui.onSelectedChange(true);
        }
      },

      // overwritten from MultiSelectionModel to fix unselecting...
      select : function(node, e, keepExisting) {
        // Add in setting an array as selected... (for multi-selecting D&D
        // nodes)
        if (node instanceof Array)
        {
          for (var c = 0; c < node.length; c++)
          {
            this.selectNode(node[c]);
          }
          this.sortSelNodes();
          this.fireEvent("selectionchange", this, this.selNodes, this.lastSelNode);
          return node;
        }
        // Shift Select to select a range
        // NOTE: Doesn't change lastSelNode
        // EEK has to be a prettier way to do this
        if (e && e.shiftKey && this.selNodes.length > 0)
        {
          this.lastSelNode = this.lastSelNode || this.selNodes[0];
          var before = this.compareNodeOrder(this.lastSelNode, node) > 0;
          // if (this.lastSelNode == node) {
          // check dom node ordering (from ppk of quirksmode.org)
          this.clearSelections(true);
          var cont = true;
          var inside = false;
          var parent = this.lastSelNode;
          // ummm... yeah don't read this bit...
          do
          {
            for (var next = parent; next != null; next = (before ? next.previousSibling : next.nextSibling))
            {
              // hack to make cascade work the way I want it to
              inside = inside || (before && (next == node || next.contains(node)));
              if (next.isExpanded())
              {
                next.cascade(function(n) {
                      if (cont != inside)
                      {
                        this.selectNode(n);
                      }
                      cont = (cont && n != node);
                      return true;
                    }, this);
              }
              else
              {
                this.selectNode(next);
                cont = (next != node);
              }
              if (!cont)
                break;
            }
            if (!cont)
              break;
            while ((parent = parent.parentNode) != null)
            {
              if (before)
              {
                this.selectNode(parent);
              }
              cont = (cont && parent != node);
              if (before && parent.previousSibling)
              {
                parent = parent.previousSibling;
                break;
              }
              if (!before && parent.nextSibling)
              {
                parent = parent.nextSibling;
                break;
              }
            }
            if (!cont)
              break;
          } while (parent != null);
          this.selectNode(node);
          // sort the list
          this.sortSelNodes();
          this.fireEvent("selectionchange", this, this.selNodes, node);
          e.preventDefault();
          return node;
        }
        else if (keepExisting !== true)
        {
          this.clearSelections(true);
        }
        if (this.isSelected(node))
        {
          // handle deselect of node...
          if (keepExisting === true)
          {
            this.unselect(node);
            if (this.lastSelNode === node)
            {
              this.lastSelNode = this.selNodes[0];
            }
            return node;
          }
          this.lastSelNode = node;
          return node;
        }
        // save a resort later on...
        this.selectNode(node);
        this.sortSelNodes();
        this.lastSelNode = node;
        this.fireEvent("selectionchange", this, this.selNodes, this.lastSelNode);
        return node;
      },
      // returns selected nodes precluding children of other selected nodes...
      // used for multi drag and drop...
      getUniqueSelectedNodes : function() {
        var ret = [];
        for (var c = 0; c < this.selNodes.length; c++)
        {
          var parent = this.selNodes[c];
          ret.push(parent);
          // nodes are sorted(?) so skip over subsequent nodes inside this one..
          while ((c + 1) < this.selNodes.length && parent.contains(this.selNodes[c + 1]))
            c++;
        }
        return ret;
      },

      // check for descendents when nodes are removed...
      unselect : function(node, subnodes) {
        if (subnodes)
        {
          for (var c = this.selNodes.length - 1; c >= 0; c--)
          {
            if (this.selNodes[c].isAncestor(node))
            {
              Ext.ux.FixedMultiSelectionModel.superclass.unselect.call(this, this.selNodes[c]);
            }
          }
        }
        return Ext.ux.FixedMultiSelectionModel.superclass.unselect.call(this, node);
      },

      /**
       * Selects the node above the selected node in the tree, intelligently
       * walking the nodes
       * 
       * @return TreeNode The new selection
       */
      selectPrevious : function(keepExisting) {
        var s = this.selNodes[0];
        if (!s)
        {
          return null;
        }
        var ps = s.previousSibling;
        if (ps)
        {
          if (!ps.isExpanded() || ps.childNodes.length < 1)
          {
            return this.select(ps, null, keepExisting);
          }
          else
          {
            var lc = ps.lastChild;
            while (lc && lc.isExpanded() && lc.childNodes.length > 0)
            {
              lc = lc.lastChild;
            }
            return this.select(lc, null, keepExisting);
          }
        }
        else if (s.parentNode && (this.tree.rootVisible || !s.parentNode.isRoot))
        {
          return this.select(s.parentNode, null, keepExisting);
        }
        return null;
      },

      /**
       * Selects the node above the selected node in the tree, intelligently
       * walking the nodes
       * 
       * @return TreeNode The new selection
       */
      selectNext : function(keepExisting) {
        var s = this.selNodes[this.selNodes.length - 1];
        if (!s)
        {
          return null;
        }
        if (s.firstChild && s.isExpanded())
        {
          return this.select(s.firstChild, null, keepExisting);
        }
        else if (s.nextSibling)
        {
          return this.select(s.nextSibling, null, keepExisting);
        }
        else if (s.parentNode)
        {
          var newS = null;
          s.parentNode.bubble(function() {
                if (this.nextSibling)
                {
                  newS = this.getOwnerTree().selModel.select(this.nextSibling, null, keepExisting);
                  return false;
                }
              });
          return newS;
        }
        return null;
      },

      onKeyDown : function(e) {
        var s = this.selNode || this.lastSelNode;
        // undesirable, but required
        var sm = this;
        if (!s)
        {
          return;
        }
        var k = e.getKey();
        switch (k)
        {
          case e.DOWN :
            e.stopEvent();
            this.selectNext(e.shiftKey || e.ctrlKey);
            break;
          case e.UP :
            e.stopEvent();
            this.selectPrevious(e.shiftKey || e.ctrlKey);
            break;
          case e.RIGHT :
            e.preventDefault();
            if (s.hasChildNodes())
            {
              if (!s.isExpanded())
              {
                s.expand();
              }
              else if (s.firstChild)
              {
                this.select(s.firstChild, e, e.shiftKey || e.ctrlKey);
              }
            }
            break;
          case e.LEFT :
            e.preventDefault();
            if (s.hasChildNodes() && s.isExpanded())
            {
              s.collapse();
            }
            else if (s.parentNode && (this.tree.rootVisible || s.parentNode != this.tree.getRootNode()))
            {
              this.select(s.parentNode, e, e.shiftKey || e.ctrlKey);
            }
            break;
        }
      }

    });
/*
 * Enhanced to support dragging multiple nodes... for extension refer to
 * data.nodes instead of data.node
 */
Ext.ux.MultiSelectTreeDragZone = Ext.extend(Ext.tree.TreeDragZone, {
      onBeforeDrag : function(data, e) {
        if (data.nodes && data.nodes.length > 0)
        {
          for (var c = 0; c < data.nodes.length; c++)
          {
            n = data.nodes[c];
            if (n.draggable === false || n.disabled)
              return false
          }
          return true;
        }
        else if (data.node)
        {
          if (data.node.draggable === false || data.node.disabled)
            return false
        }
        return false;

      },
      // v1.0
      // fixed to handle multiSelectionModel
      // Data now calls SelectionModel.select instead of waiting for the click
      // event
      // Creates Ghost inline rather than calling TreeNodeUI.
      //
      // v1.1
      // cleanup to have ghost generation slightly less hacky... still hacky
      // though...
      // fixes problems with using extra tag nesting in a custom TreeNodeUI.
      getDragData : function(e) {
        // use tree selection model..
        var selModel = this.tree.getSelectionModel();
        // get event target
        var target = Ext.dd.Registry.getHandleFromEvent(e);
        // if no target (die)
        if (target == null)
          return;
        if (target.node.isSelected() && e.ctrlKey)
        {
          selModel.unselect(target.node);
          return;
        }
        var selNodes = [];
        if (!selModel.getSelectedNodes)
        {
          // if not multiSelectionModel.. just use the target...
          selNodes = [target.node];
        }
        else
        {
          // if target not selected select it...
          if (!target.node.isSelected() || e.shiftKey)
          {
            selModel.select(target.node, e, e.ctrlKey);
          }
          // get selected nodes - nested nodes...
          selNodes = selModel.getUniqueSelectedNodes();
        }
        // if no nodes selected stop now...
        if (!selNodes || selNodes.length < 1)
          return;
        var dragData = {
          nodes : selNodes
        };
        // create a container for the proxy...
        var div = document.createElement('ul'); // create the multi element drag
                                                // "ghost"
        // add classes to keep is pretty...
        div.className = 'x-tree-node-ct x-tree-lines';
        // add actual dom nodes to div (instead of tree nodes)
        var height = 0;
        for (var i = 0, len = selNodes.length; i < len; i++)
        {
          // add entire node to proxy
          // normally this is done by TreeNodeUI.appendDDGhost(), but overriding
          // that class requires
          // also overriding TreeLoader etc. Ext.extend() is an option though...
          var clonenode = selNodes[i].ui.wrap.cloneNode(true);
          // fix extra indenting by removing extra spacers
          // should really modify UI rendering code to render a duplicate
          // subtree but this is simpler...
          // count current indent nodes from ui indentNode... (add 1 for elbow)
          var subtract = selNodes[i].ui.indentNode.childNodes.length + 1;
          // avoid indent alterations if possible..
          if (subtract > 0)
          {
            // relies on node ui using the same tag for all elems...
            var subNodes = Ext.query(selNodes[i].ui.indentNode.nodeName + ".x-tree-node-indent", clonenode);
            for (var c = 0, clen = subNodes.length; c < clen; c++)
            {
              var inode = subNodes[c];
              var current = inode.childNodes.length;
              if (current <= subtract)
              {
                inode.innerHTML = "";
                // remove elbow icon as well..
                if (current < subtract)
                  inode.parentNode.removeChild(subNodes[c].nextSibling);
              }
              else
              {
                for (var r = 0; r < subtract; r++)
                {
                  subNodes[c].removeChild(subNodes[c].firstChild);
                }
              }
            }
          }
          div.appendChild(clonenode);
          Ext.fly(clonenode).removeClass(['x-tree-selected', 'x-tree-node-over']);
        }
        dragData.ddel = div;
        return dragData;
      },
      // fix from TreeDragZone (references dragData.node instead of
      // dragData.nodes)
      onInitDrag : function(e) {
        var data = this.dragData;
        this.tree.eventModel.disable();
        this.proxy.update("");
        this.proxy.ghost.dom.appendChild(data.ddel);
        this.tree.fireEvent("startdrag", this.tree, data.nodes, e);
      },
      // Called from TreeDropZone (looks like hack for handling multiple tree
      // nodes)
      getTreeNode : function() {
        return this.dragData.nodes;
      },
      // fix from TreeDragZone (refers to data.node instead of data.nodes)
      // Don't know what this does, so leaving as first node.
      getRepairXY : function(e, data) {
        return data.nodes[0].ui.getDDRepairXY();
      },

      // fix from TreeDragZone (refers to data.node instead of data.nodes)
      onEndDrag : function(data, e) {
        this.tree.eventModel.enable.defer(100, this.tree.eventModel);
        this.tree.fireEvent("enddrag", this.tree, data.nodes || [data.node], e);
      },

      // fix from TreeDragZone (refers to dragData.node instead of
      // dragData.nodes)
      onValidDrop : function(dd, e, id) {
        this.tree.fireEvent("dragdrop", this.tree, this.dragData.nodes, dd, e);
        this.hideProxy();
      },

      // fix for invalid Drop
      beforeInvalidDrop : function(e, id) {
        // this scrolls the original position back into view
        var sm = this.tree.getSelectionModel();
        // sm.clearSelections();
        // sm.select(this.dragData.nodes, e, true);
      }

    });

/*
 * MultiSelectTreeDropZone Contains following fixups - modified functions to
 * handle multiple nodes in dd operation isValidDropPoint afterRepair - modified
 * getDropPoint such that isValidDropPoint can simulate leaf style below
 * inserting. Overriding isValidDropPoint affects getDropPoint affects
 * onNodeOver and onNodeDrop Refer to data.nodes instead of data.node for
 * events..
 */
Ext.ux.MultiSelectTreeDropZone = Ext.extend(Ext.tree.TreeDropZone, {

      // fix from TreeDropZone (referred to data.node instead of data.nodes)
      isValidDropPoint : function(n, pt, dd, e, data) {
        if (!n || !data)
        {
          return false;
        }
        var targetNode = n.node;
        var dropNodes = data.nodes ? data.nodes : [data.node];
        // default drop rules
        if (!(targetNode && targetNode.isTarget && pt))
        {
          return false;
        }
        if (pt == "append" && targetNode.allowChildren === false)
        {
          return false;
        }
        if ((pt == "above" || pt == "below") && (targetNode.parentNode && targetNode.parentNode.allowChildren === false))
        {
          return false;
        }
        // don't allow dropping a treenode inside itself...
        for (var c = 0; c < dropNodes.length; c++)
        {
          if (dropNodes[c] && (targetNode == dropNodes[c] || dropNodes[c].contains(targetNode)))
          {
            return false;
          }
        }
        // reuse the object
        var overEvent = this.dragOverData;
        overEvent.tree = this.tree;
        overEvent.target = targetNode;
        overEvent.data = data;
        overEvent.point = pt;
        overEvent.source = dd;
        overEvent.rawEvent = e;
        overEvent.dropNode = dropNodes;
        overEvent.cancel = false;
        var result = this.tree.fireEvent("nodedragover", overEvent);
        return overEvent.cancel === false && result !== false;
      },

      // override to allow insert "below" when leaf != true...
      getDropPoint : function(e, n, dd, data) {
        var tn = n.node;
        if (tn.isRoot)
        {
          return this.isValidDropPoint(n, "append", dd, e, data) ? "append" : false;
        }
        var dragEl = n.ddel;
        var t = Ext.lib.Dom.getY(dragEl), b = t + dragEl.offsetHeight;
        var y = Ext.lib.Event.getPageY(e);
        var noAppend = tn.allowChildren === false || tn.isLeaf() || !this.isValidDropPoint(n, "append", dd, e, data);
        if (!this.appendOnly && tn.parentNode.allowChildren !== false)
        {
          var noBelow = false;
          if (!this.allowParentInsert)
          {
            noBelow = tn.hasChildNodes() && tn.isExpanded();
          }
          var q = (b - t) / (noAppend ? 2 : 3);
          if (y >= t && y < (t + q) && this.isValidDropPoint(n, "above", dd, e, data))
          {
            return "above";
          }
          else if (!noBelow && (noAppend || y >= b - q && y <= b) && this.isValidDropPoint(n, "below", dd, e, data))
          {
            return "below";
          }
        }
        return noAppend ? false : "append";
      },

      // Override because it calls getDropPoint and isValidDropPoint
      onNodeOver : function(n, dd, e, data) {
        var pt = this.getDropPoint(e, n, dd, data);
        var node = n.node;

        if (!this.expandProcId && pt == "append" && node.hasChildNodes() && !n.node.isExpanded())
        {
          this.queueExpand(node);
        }
        else if (pt != "append")
        {
          this.cancelExpand();
        }

        var returnCls = this.dropNotAllowed;
        if (pt)
        {
          var el = n.ddel;
          var cls;
          if (pt == "above")
          {
            returnCls = n.node.isFirst() ? "x-tree-drop-ok-above" : "x-tree-drop-ok-between";
            cls = "x-tree-drag-insert-above";
          }
          else if (pt == "below")
          {
            returnCls = n.node.isLast() ? "x-tree-drop-ok-below" : "x-tree-drop-ok-between";
            cls = "x-tree-drag-insert-below";
          }
          else
          {
            returnCls = "x-tree-drop-ok-append";
            cls = "x-tree-drag-append";
          }
          if (this.lastInsertClass != cls)
          {
            Ext.fly(el).replaceClass(this.lastInsertClass, cls);
            this.lastInsertClass = cls;
          }
        }
        return returnCls;
      },

      // Override because it calls getDropPoint and isValidDropPoint
      onNodeDrop : function(n, dd, e, data) {
        var point = this.getDropPoint(e, n, dd, data);
        var targetNode = n.node;
        targetNode.ui.startDrop();
        if (point === false)
        {
          targetNode.ui.endDrop();
          return false;
        }

        var dropNode = data.node || (dd.getTreeNode ? dd.getTreeNode(data, targetNode, point, e) : null);
        var dropEvent = {
          tree : this.tree,
          target : targetNode,
          data : data,
          point : point,
          source : dd,
          rawEvent : e,
          dropNode : dropNode,
          cancel : !dropNode,
          dropStatus : false
        };
        var retval = this.tree.fireEvent("beforenodedrop", dropEvent);
        if (retval === false || dropEvent.cancel === true || !dropEvent.dropNode)
        {
          targetNode.ui.endDrop();
          return dropEvent.dropStatus;
        }

        targetNode = dropEvent.target;
        if (point == "append" && !targetNode.isExpanded())
        {
          targetNode.expand(false, null, function() {
                this.completeDrop(dropEvent);
              }.createDelegate(this));
        }
        else
        {
          this.completeDrop(dropEvent);
        }
        return true;
      },

      // fix from TreeDropZone (referred to data.node instead of data.nodes)
      afterRepair : function(data) {
        if (data && Ext.enableFx)
        {
          var nl = data.nodes ? data.nodes : [data.node];
          for (var c = 0, len = nl.length; c < len; c++)
          {
            nl[c].ui.highlight();
          }
        }
        this.hideProxy();
      },

      // handle allowContainerDrop (appends nodes to the root node)
      onContainerDrop : function(dd, e, data) {
        if (this.allowContainerDrop && this.isValidDropPoint({
              ddel : this.tree.getRootNode().ui.elNode,
              node : this.tree.getRootNode()
            }, "append", dd, e, data))
        {
          var targetNode = this.tree.getRootNode();
          targetNode.ui.startDrop();
          var dropNode = data.node || (dd.getTreeNode ? dd.getTreeNode(data, targetNode, "append", e) : null);
          var dropEvent = {
            tree : this.tree,
            target : targetNode,
            data : data,
            point : "append",
            source : dd,
            rawEvent : e,
            dropNode : dropNode,
            cancel : !dropNode,
            dropStatus : false
          };
          var retval = this.tree.fireEvent("beforenodedrop", dropEvent);
          if (retval === false || dropEvent.cancel === true || !dropEvent.dropNode)
          {
            targetNode.ui.endDrop();
            return dropEvent.dropStatus;
          }

          targetNode = dropEvent.target;
          if (!targetNode.isExpanded())
          {
            targetNode.expand(false, null, function() {
                  this.completeDrop(dropEvent);
                }.createDelegate(this));
          }
          else
          {
            this.completeDrop(dropEvent);
          }
          return true;
        }
        return false;
      },

      // handle allowContaineDrop (treat as a drop to the root node)
      onContainerOver : function(dd, e, data) {
        if (this.allowContainerDrop && this.isValidDropPoint({
              ddel : this.tree.getRootNode().ui.elNode,
              node : this.tree.getRootNode()
            }, "append", dd, e, data))
        {
          return this.dropAllowed;
        }
        return this.dropNotAllowed;
      }

    });

/*
 * MultiSelectTreePanel sets up using FixedMultiSelectionModel and initing with
 * extended DragZone and DropZone by default
 */

Ext.ux.MultiSelectTreePanel = Ext.extend(Ext.tree.TreePanel, {

      getSelectionModel : function() {
        if (!this.selModel)
        {
          this.selModel = new Ext.ux.FixedMultiSelectionModel();
        }
        return this.selModel;
      },

      initEvents : function() {
        if ((this.enableDD || this.enableDrop) && !this.dropZone)
        {
          this.dropZone = new Ext.ux.MultiSelectTreeDropZone(this, this.dropConfig || {
                ddGroup : this.ddGroup || "TreeDD",
                appendOnly : this.ddAppendOnly === true
              });
        }
        if ((this.enableDD || this.enableDrag) && !this.dragZone)
        {
          this.dragZone = new Ext.ux.MultiSelectTreeDragZone(this, {
                ddGroup : this.ddGroup || "TreeDD",
                scroll : this.ddScroll
              });
        }
        Ext.ux.MultiSelectTreePanel.superclass.initEvents.apply(this, arguments);

        // This is temporary. Should really Ext.extend on TreeNode.removeChild()
        // and call getOwnerTree().removeNode(node) or similar...

        this.on("remove", function(tree, parent, node) {
              tree.getSelectionModel().unselect(node, true);
            });
      }
    });

Ext.reg('multiselecttreepanel', Ext.ux.MultiSelectTreePanel);
});

/*global define*/
define('ext/ux/RowExpander', ['extjs'], function(Ext) {

Ext.ns('Ext.ux.grid');

/**
 * @class Ext.ux.grid.RowExpander
 * @extends Ext.util.Observable
 * Plugin (ptype = 'rowexpander') that adds the ability to have a Column in a grid which enables
 * a second row body which expands/contracts.  The expand/contract behavior is configurable to react
 * on clicking of the column, double click of the row, and/or hitting enter while a row is selected.
 *
 * @ptype rowexpander
 */
Ext.ux.grid.RowExpander = Ext.extend(Ext.util.Observable, {
    /**
     * @cfg {Boolean} expandOnEnter
     * <tt>true</tt> to toggle selected row(s) between expanded/collapsed when the enter
     * key is pressed (defaults to <tt>true</tt>).
     */
    expandOnEnter : true,
    /**
     * @cfg {Boolean} expandOnDblClick
     * <tt>true</tt> to toggle a row between expanded/collapsed when double clicked
     * (defaults to <tt>true</tt>).
     */
    expandOnDblClick : true,

    header : '',
    width : 20,
    sortable : false,
    fixed : true,
    hideable: false,
    menuDisabled : true,
    dataIndex : '',
    id : 'expander',
    lazyRender : true,
    enableCaching : true,

    constructor: function(config){
        Ext.apply(this, config);

        this.addEvents({
            /**
             * @event beforeexpand
             * Fires before the row expands. Have the listener return false to prevent the row from expanding.
             * @param {Object} this RowExpander object.
             * @param {Object} Ext.data.Record Record for the selected row.
             * @param {Object} body body element for the secondary row.
             * @param {Number} rowIndex The current row index.
             */
            beforeexpand: true,
            /**
             * @event expand
             * Fires after the row expands.
             * @param {Object} this RowExpander object.
             * @param {Object} Ext.data.Record Record for the selected row.
             * @param {Object} body body element for the secondary row.
             * @param {Number} rowIndex The current row index.
             */
            expand: true,
            /**
             * @event beforecollapse
             * Fires before the row collapses. Have the listener return false to prevent the row from collapsing.
             * @param {Object} this RowExpander object.
             * @param {Object} Ext.data.Record Record for the selected row.
             * @param {Object} body body element for the secondary row.
             * @param {Number} rowIndex The current row index.
             */
            beforecollapse: true,
            /**
             * @event collapse
             * Fires after the row collapses.
             * @param {Object} this RowExpander object.
             * @param {Object} Ext.data.Record Record for the selected row.
             * @param {Object} body body element for the secondary row.
             * @param {Number} rowIndex The current row index.
             */
            collapse: true
        });

        Ext.ux.grid.RowExpander.superclass.constructor.call(this);

        if(this.tpl){
            if(typeof this.tpl == 'string'){
                this.tpl = new Ext.Template(this.tpl);
            }
            this.tpl.compile();
        }

        this.state = {};
        this.bodyContent = {};
    },

    getRowClass : function(record, rowIndex, p, ds){
        p.cols = p.cols-1;
        var content = this.bodyContent[record.id];
        if(!content && !this.lazyRender){
            content = this.getBodyContent(record, rowIndex);
        }
        if(content){
            p.body = content;
        }
        return this.state[record.id] ? 'x-grid3-row-expanded' : 'x-grid3-row-collapsed';
    },

    init : function(grid){
        this.grid = grid;

        var view = grid.getView();
        view.getRowClass = this.getRowClass.createDelegate(this);

        view.enableRowBody = true;


        grid.on('render', this.onRender, this);
        grid.on('destroy', this.onDestroy, this);
    },

    // @private
    onRender: function() {
        var grid = this.grid;
        var mainBody = grid.getView().mainBody;
        mainBody.on('mousedown', this.onMouseDown, this, {delegate: '.x-grid3-row-expander'});
        if (this.expandOnEnter) {
            this.keyNav = new Ext.KeyNav(this.grid.getGridEl(), {
                'enter' : this.onEnter,
                scope: this
            });
        }
        if (this.expandOnDblClick) {
            grid.on('rowdblclick', this.onRowDblClick, this);
        }
    },
    
    // @private    
    onDestroy: function() {
        if(this.keyNav){
            this.keyNav.disable();
            delete this.keyNav;
        }
        /*
         * A majority of the time, the plugin will be destroyed along with the grid,
         * which means the mainBody won't be available. On the off chance that the plugin
         * isn't destroyed with the grid, take care of removing the listener.
         */
        var mainBody = this.grid.getView().mainBody;
        if(mainBody){
            mainBody.un('mousedown', this.onMouseDown, this);
        }
    },
    // @private
    onRowDblClick: function(grid, rowIdx, e) {
        this.toggleRow(rowIdx);
    },

    onEnter: function(e) {
        var g = this.grid;
        var sm = g.getSelectionModel();
        var sels = sm.getSelections();
        for (var i = 0, len = sels.length; i < len; i++) {
            var rowIdx = g.getStore().indexOf(sels[i]);
            this.toggleRow(rowIdx);
        }
    },

    getBodyContent : function(record, index){
        if(!this.enableCaching){
            return this.tpl.apply(record.data);
        }
        var content = this.bodyContent[record.id];
        if(!content){
            content = this.tpl.apply(record.data);
            this.bodyContent[record.id] = content;
        }
        return content;
    },

    onMouseDown : function(e, t){
        e.stopEvent();
        var row = e.getTarget('.x-grid3-row');
        this.toggleRow(row);
    },

    renderer : function(v, p, record){
        p.cellAttr = 'rowspan="2"';
        return '<div class="x-grid3-row-expander">&#160;</div>';
    },

    beforeExpand : function(record, body, rowIndex){
        if(this.fireEvent('beforeexpand', this, record, body, rowIndex) !== false){
            if(this.tpl && this.lazyRender){
                body.innerHTML = this.getBodyContent(record, rowIndex);
            }
            return true;
        }else{
            return false;
        }
    },

    toggleRow : function(row){
        if(typeof row == 'number'){
            row = this.grid.view.getRow(row);
        }
        this[Ext.fly(row).hasClass('x-grid3-row-collapsed') ? 'expandRow' : 'collapseRow'](row);
    },

    expandRow : function(row){
        if(typeof row == 'number'){
            row = this.grid.view.getRow(row);
        }
        var record = this.grid.store.getAt(row.rowIndex);
        var body = Ext.DomQuery.selectNode('tr:nth(2) div.x-grid3-row-body', row);
        if(this.beforeExpand(record, body, row.rowIndex)){
            this.state[record.id] = true;
            Ext.fly(row).replaceClass('x-grid3-row-collapsed', 'x-grid3-row-expanded');
            this.fireEvent('expand', this, record, body, row.rowIndex);
        }
    },

    collapseRow : function(row){
        if(typeof row == 'number'){
            row = this.grid.view.getRow(row);
        }
        var record = this.grid.store.getAt(row.rowIndex);
        var body = Ext.fly(row).child('tr:nth(1) div.x-grid3-row-body', true);
        if(this.fireEvent('beforecollapse', this, record, body, row.rowIndex) !== false){
            this.state[record.id] = false;
            Ext.fly(row).replaceClass('x-grid3-row-expanded', 'x-grid3-row-collapsed');
            this.fireEvent('collapse', this, record, body, row.rowIndex);
        }
    }
});

Ext.preg('rowexpander', Ext.ux.grid.RowExpander);

//backwards compat
Ext.grid.RowExpander = Ext.ux.grid.RowExpander;

});
define('ext/ux/browsebutton',['extjs'], function(Ext){
/*
 * Public Domain
 * http://www.sencha.com/forum/showthread.php?29032-Ext.ux.form.BrowseButton
 */

/*global Ext*/
Ext.namespace('Ext.ux.form');

// @author 4Him
//if(typeof Ext.isIE8 !== 'boolean') { // Ext 2.x (at least 2.1) doesn't know IE8. Let's tell him about it...
//    Ext.isIE8 = Ext.isIE && navigator.userAgent.toLowerCase().indexOf("msie 8")>-1;
//}

/**
 * @class Ext.ux.form.BrowseButton
 * @extends Ext.Button
 * Ext.Button that provides a customizable file browse button.
 * Clicking this button, pops up a file dialog box for a user to select the file to upload.
 * This is accomplished by having a transparent <input type="file"> box above the Ext.Button.
 * When a user thinks he or she is clicking the Ext.Button, they're actually clicking the hidden input "Browse..." box.
 * Note: this class can be instantiated explicitly or with xtypes anywhere a regular Ext.Button can be except in 2 scenarios:
 * - Panel.addButton method both as an instantiated object or as an xtype config object.
 * - Panel.buttons config object as an xtype config object.
 * These scenarios fail because Ext explicitly creates an Ext.Button in these cases.
 * Browser compatibility:
 * Internet Explorer 6:
 * - no issues
 * Internet Explorer 7:
 * - no issues
 * Internet Explorer 8:
 * - no issues
 * Firefox 3 - Windows:
 * - pointer cursor doesn't display when hovering over the button.
 * @author loeppky - based on the work done by MaximGB in Ext.ux.UploadDialog (http://extjs.com/forum/showthread.php?t=21558)
 * The follow the curosr float div idea also came from MaximGB. With patches for IE8 by 4Him.
 * @see http://extjs.com/forum/showthread.php?t=29032
 * @constructor
 * Create a new BrowseButton.
 * @param {Object} config Configuration options
 */

/*global Ext*/
Ext.ux.form.BrowseButton = Ext.extend(Ext.Button, {
    /*
     * Config options:
     */
    /**
     * @cfg {String} inputFileName
     * Name to use for the hidden input file DOM element.  Deaults to "file".
     */
    inputFileName: 'file',

    /**
     * @cfg {Boolean} debug
     * Toggle for turning on debug mode.
     * Debug mode doesn't make clipEl transparent so that one can see how effectively it covers the Ext.Button.
     * In addition, clipEl is given a green background and floatEl a red background to see how well they are positioned.
     */
    debug: window.location.search === '?debug',

    /*
     * Private constants:
     */
    /**
     * @property FLOAT_EL_WIDTH
     * @type Number
     * The width (in pixels) of floatEl.
     * It should be less than the width of the IE "Browse" button's width (65 pixels), since IE doesn't let you resize it.
     * We define this width so we can quickly center floatEl at the mouse cursor without having to make any function calls.
     * @private
     */
    FLOAT_EL_WIDTH: 60,
    
    /**
     * @property FLOAT_EL_HEIGHT
     * @type Number
     * The heigh (in pixels) of floatEl.
     * It should be less than the height of the "Browse" button's height.
     * We define this height so we can quickly center floatEl at the mouse cursor without having to make any function calls.
     * @private
     */
    FLOAT_EL_HEIGHT: 18,
    
    
    /*
     * Private properties:
     */
    /**
     * @property buttonCt
     * @type Ext.Element
     * Element that contains the actual Button DOM element.
     * We store a reference to it, so we can easily grab its size for sizing the clipEl.
     * @private
     */
    buttonCt: null,
    /**
     * @property clipEl
     * @type Ext.Element
     * Element that contains the floatEl.
     * This element is positioned to fill the area of Ext.Button and has overflow turned off.
     * This keeps floadEl tight to the Ext.Button, and prevents it from masking surrounding elements.
     * @private
     */
    clipEl: null,
    /**
     * @property floatEl
     * @type Ext.Element
     * Element that contains the inputFileEl.
     * This element is size to be less than or equal to the size of the input file "Browse" button.
     * It is then positioned wherever the user moves the cursor, so that their click always clicks the input file "Browse" button.
     * Overflow is turned off to preven inputFileEl from masking surrounding elements.
     * @private
     */
    floatEl: null,
    /**
     * @property inputFileEl
     * @type Ext.Element
     * Element for the hiden file input.
     * @private
     */
    inputFileEl: null,
    /**
     * @property originalHandler
     * @type Function
     * The handler originally defined for the Ext.Button during construction using the "handler" config option.
     * We need to null out the "handler" property so that it is only called when a file is selected.
     * @private
     */
    originalHandler: null,
    /**
     * @property originalScope
     * @type Object
     * The scope originally defined for the Ext.Button during construction using the "scope" config option.
     * While the "scope" property doesn't need to be nulled, to be consistent with originalHandler, we do.
     * @private
     */
    originalScope: null,
    /**
     * @property BROWSERS_OFFSET
     * @type Object
     * The browsers specific offsets used to position the clipping element for better overlay tightness. For
     * Ext 3, Ext 2 offsets are used unless there is an Ext 3 entry.
     * @private
     * @author 4Him
     */
    BROWSERS_OFFSETS: {
        Ext2: {
            IE10:  {left: -8, top: -16, width: 16,  height: 22},
            IE9:   {left: -8, top: -16, width: 16,  height: 22},
            IE8:   {left: -8, top: -16, width: 16,  height: 22},
            IE:    {left: -8, top: -3,  width: 16,  height: 6},
            Opera: {left: -8, top: -3,  width: -18, height: -1},
            Gecko: {left: -8, top: -6,  width: 16,  height: 10},
            Safari:{left: -4, top: -2,  width: 6,   height: 6},
            Chrome:{left: -4, top: -2,  width: 6,   height: 6}
        },
        Ext3: {
            IE10:  {left: -7,           width: 10},
            IE9:   {left: -7,           width: 10},
            IE8:   {left: -7,           width: 10},
            IE:    {left: -3,           width: 6},
            Gecko: {                    width: 11}
        }
    },
    /**
     * @property isExt2x
     * @type boolean
     * Whether we are currently using Ext 2.x
     * @private
     * @author 4Him
     */
    isExt2x: false, // HACK: jdillon we are only in 3 now fuck 2
    
    
    /*
     * Protected Ext.Button overrides
     */
    /**
     * @see Ext.Button.initComponent
     */
    initComponent: function(){
        Ext.ux.form.BrowseButton.superclass.initComponent.call(this);

        // FIXME: This should never have fucked with the handler, should have registered a new event

        // Store references to the original handler and scope before nulling them.
        // This is done so that this class can control when the handler is called.
        // There are some cases where the hidden file input browse button doesn't completely cover the Ext.Button.
        // The handler shouldn't be called in these cases.  It should only be called if a new file is selected on the file system.  
        this.originalHandler = this.handler;
        this.originalScope = this.scope;
        this.handler = null;
        this.scope = null;
    },
    
    /**
     * @see Ext.Button.onRender
     */
    onRender: function(ct, position){
        Ext.ux.form.BrowseButton.superclass.onRender.call(this, ct, position); // render the Ext.Button
        // Patch for compatibility with 3.x (@author 4Him, based on dario's 05-10-2009 post)
        if(this.isExt2x) {
            this.buttonCt = this.el.child('.x-btn-center em');
        } else {
            this.buttonCt = this.el.child('.x-btn-mc em');
        }

        this.buttonCt.position('relative'); // this is important!
        var styleCfg = {
            position: 'absolute',
            overflow: 'hidden',
            top: '0px', // default
            left: '0px' // default
        };

        // browser specifics for better overlay tightness - modified by 4Him
        for(var browser in this.BROWSERS_OFFSETS.Ext2) {
            if(Ext['is'+browser]) {
                Ext.apply(styleCfg, {
                    left: this.getBrowserOffset(browser, 'left')+'px',
                    top: this.getBrowserOffset(browser, 'top')+'px'
                });
                break;
            }
        }
        this.clipEl = this.buttonCt.createChild({
            tag: 'div',
            style: styleCfg
        });
        this.setClipSize();
        this.clipEl.on({
            'mousemove': this.onButtonMouseMove,
            'mouseover': this.onButtonMouseMove,
            scope: this
        });
        
        this.floatEl = this.clipEl.createChild({
            tag: 'div',
            style: {
                position: 'absolute',
                width: this.FLOAT_EL_WIDTH + 'px',
                height: this.FLOAT_EL_HEIGHT + 'px',
                overflow: 'hidden'
            }
        });
        
        
        if (this.debug) {
            this.clipEl.applyStyles({
                'background-color': 'green'
            });
            this.floatEl.applyStyles({
                'background-color': 'red'
            });
        } else {
            // We don't set the clipEl to be transparent, because IE 6/7 occassionaly looses mouse events for transparent elements.
            // We have listeners on the clipEl that can't be lost as they're needed for realligning the input file element.
            this.floatEl.setOpacity(0.0);
        }
        
        // Cover cases where someone tabs to the button:
        // Listen to focus of the button so we can translate the focus to the input file el.
        var buttonEl = this.el.child(this.buttonSelector);
        buttonEl.on('focus', this.onButtonFocus, this);
        // In IE, it's possible to tab to the text portion of the input file el.  
        // We want to listen to keyevents so that if a space is pressed, we "click" the input file el.
        if (Ext.isIE) {
            this.el.on('keydown', this.onButtonKeyDown, this);
        }
        
        this.createInputFile();
    },
    
    
    /*
     * Private helper methods:
     */
    /**
     * Returns an offset based on this.BROWSERS_OFFSET 
     * If currently using Ext 3.x, tries to find a value for 3.x and if there is none for 3.x, it
     * returns a value for 2.x
     * @param {string} which the desired offset. Can be one of the following: 'left', 'top', 'width', 'height'
     * @param {string} browser the browser for which to return the offset
     * @return {int} the desired offset
     * @author 4Him
     */
    getBrowserOffset: function(browser, which) {
        if(!this.isExt2x && this.BROWSERS_OFFSETS.Ext3[browser] && this.BROWSERS_OFFSETS.Ext3[browser][which]) {
            return this.BROWSERS_OFFSETS.Ext3[browser][which];
        } else {
            return this.BROWSERS_OFFSETS.Ext2[browser][which];
        }
    },

    /**
     * Sets the size of clipEl so that is covering as much of the button as possible.
     * @private
     */
    setClipSize: function(){
        if (this.clipEl) {
            var width = this.buttonCt.getWidth();
            var height = this.buttonCt.getHeight();

            // The button container can have a width and height of zero when it's rendered in a hidden panel.
            // This is most noticable when using a card layout, as the items are all rendered but hidden,
            // (unless deferredRender is set to true). 
            // In this case, the clip size can't be determined, so we attempt to set it later.
            // This check repeats until the button container has a size. 
            if (width === 0 || (height === 0 && (!Ext.isIE8 && !Ext.isIE9 && !Ext.isIE10)) ) {  // ugly hack (Ext.isIE8) by 4Him
                this.setClipSize.defer(100, this);
            } else {
                // Loop by 4Him
                for(var browser in this.BROWSERS_OFFSETS.Ext2) {
                    if(Ext['is'+browser]) {
                        width = width + this.getBrowserOffset(browser, 'width');
                        height = height + this.getBrowserOffset(browser, 'height');
                        break;
                    }
                }

                this.clipEl.setSize(width, height);
            }
        }
    },
    
    /**
     * Creates the input file element and adds it to inputFileCt.
     * The created input file elementis sized, positioned, and styled appropriately.
     * Event handlers for the element are set up, and a tooltip is applied if defined in the original config.
     * @private
     */
    createInputFile: function(){
        // When an input file gets detached and set as the child of a different DOM element,
        // straggling <em> elements get left behind.  
        // I don't know why this happens but we delete any <em> elements we can find under the floatEl to prevent a memory leak.
        this.floatEl.select('em').each(function(el){
            el.remove();
        });
        this.inputFileEl = this.floatEl.createChild({
            tag: 'input',
            type: 'file',
            size: 1, // must be > 0. It's value doesn't really matter due to our masking div (inputFileCt).  
            name: this.inputFileName || Ext.id(this.el),
            tabindex: this.tabIndex,
            // Use the same pointer as an Ext.Button would use.  This doesn't work in Firefox.
            // This positioning right-aligns the input file to ensure that the "Browse" button is visible.
            style: {
                position: 'absolute',
                cursor: 'pointer',
                right: '0px',
                top: '0px'
            }
        });
        this.inputFileEl = this.inputFileEl.child('input') || this.inputFileEl;
        // IE8 needs opacity on the 'file input' element - @author 4Him
        if(Ext.isIE8 || Ext.isIE9) {
            this.inputFileEl.setOpacity(0.0);
        }
        
        // setup events
        this.inputFileEl.on({
            'click': this.onInputFileClick,
            'change': this.onInputFileChange,
            'focus': this.onInputFileFocus,
            'select': this.onInputFileFocus,
            'blur': this.onInputFileBlur,
            scope: this
        });
        
        // add a tooltip
        if (this.tooltip) {
            if (typeof this.tooltip === 'object') {
                Ext.QuickTips.register(Ext.apply({
                    target: this.inputFileEl
                }, this.tooltip));
            } else {
                this.inputFileEl.dom[this.tooltipType] = this.tooltip;
            }
        }
    },
    
    /**
     * Redirecting focus to the input file element so the user can press space and select files.
     * @param {Event} e focus event.
     * @private
     */
    onButtonFocus: function(e){
        if (this.inputFileEl) {
            this.inputFileEl.focus();
            e.stopEvent();
        }
    },
    
    /**
     * Handler for the IE case where once can tab to the text box of an input file el.
     * If the key is a space, we simply "click" the inputFileEl.
     * @param {Event} e key event.
     * @private
     */
    onButtonKeyDown: function(e){
        if (this.inputFileEl && e.getKey() === Ext.EventObject.SPACE) {
            this.inputFileEl.dom.click();
            e.stopEvent();
        }
    },
    
    /**
     * Handler when the cursor moves over the clipEl.
     * The floatEl gets centered to the cursor location.
     * @param {Event} e mouse event.
     * @private
     */
    onButtonMouseMove: function(e){
        var xy = e.getXY();
        xy[0] -= this.FLOAT_EL_WIDTH / 2;
        xy[1] -= this.FLOAT_EL_HEIGHT / 2;
        this.floatEl.setXY(xy);
    },
    
    /**
     * Add the visual enhancement to the button when the input file recieves focus. 
     * This is the tip for the user that now he/she can press space to select the file.
     * @private
     */
    onInputFileFocus: function(e){
        if (!this.isDisabled) {
            this.el.addClass("x-btn-over");
        }
    },
    
    /**
     * Removes the visual enhancement from the button.
     * @private
     */
    onInputFileBlur: function(e){
        this.el.removeClass("x-btn-over");
    },
    
    /**
     * Handler when inputFileEl's "Browse..." button is clicked.
     * @param {Event} e click event.
     * @private
     */
    onInputFileClick: function(e){
        e.stopPropagation();
    },
    
    /**
     * Handler when inputFileEl changes value (i.e. a new file is selected).
     * @private
     */
    onInputFileChange: function(){
        if (this.originalHandler) {
            this.originalHandler.call(this.originalScope, this);
        }
    },
    
    
    /*
     * Public methods:
     */
    /**
     * Detaches the input file associated with this BrowseButton so that it can be used for other purposed (e.g. uplaoding).
     * The returned input file has all listeners and tooltips applied to it by this class removed.
     * @param {Boolean} noCreate whether to create a new input file element for this BrowseButton after detaching.
     * True will prevent creation.  Defaults to false.
     * @return {Ext.Element} the detached input file element.
     */
    detachInputFile: function(noCreate){
        var result = this.inputFileEl;
        
        if (typeof this.tooltip === 'object') {
            Ext.QuickTips.unregister(this.inputFileEl);
        } else {
            this.inputFileEl.dom[this.tooltipType] = null;
        }
        this.inputFileEl.removeAllListeners();
        this.inputFileEl = null;
        
        if (!noCreate) {
            this.createInputFile();
        }
        return result;
    },
    
    /**
     * @return {Ext.Element} the input file element attached to this BrowseButton.
     */
    getInputFile: function(){
        return this.inputFileEl;
    },
    
    /**
     * @see Ext.Button.disable
     */
    disable: function(){
        Ext.ux.form.BrowseButton.superclass.disable.call(this);
        this.inputFileEl.dom.disabled = true;
    },
    
    /**
     * @see Ext.Button.enable
     */
    enable: function(){
        Ext.ux.form.BrowseButton.superclass.enable.call(this);
        this.inputFileEl.dom.disabled = false;
    }
});

/**
 * HACK:
 * Some layouts (e.g. license detail page) expect a BoxComponent (setters for size and position).
 * The documentation states that Ext.Button is a BoxComponent, but the source actually extends only
 * Ext.Component! Instead of "fixing" ext js (and potentially breaking a button somewhere in the Nexus UI)
 * we will add the BoxComponent methods only to the browse button.
 */
// FIXME this is fixed in ExtJS3 AFAIK, no need to enhance the prototype here
Ext.applyIf(Ext.ux.form.BrowseButton.prototype, Ext.BoxComponent.prototype);

Ext.reg('browsebutton', Ext.ux.form.BrowseButton);

});

/*!
 * Ext JS Library 3.4.0
 * Copyright(c) 2006-2011 Sencha Inc.
 * licensing@sencha.com
 * http://www.sencha.com/license
 */

/*global define*/
define('ext/ux/statusbar', ['extjs'], function(Ext){

/**
 * @class Ext.ux.StatusBar
 * <p>Basic status bar component that can be used as the bottom toolbar of any {@link Ext.Panel}.  In addition to
 * supporting the standard {@link Ext.Toolbar} interface for adding buttons, menus and other items, the StatusBar
 * provides a greedy status element that can be aligned to either side and has convenient methods for setting the
 * status text and icon.  You can also indicate that something is processing using the {@link #showBusy} method.</p>
 * <pre><code>
new Ext.Panel({
    title: 'StatusBar',
    // etc.
    bbar: new Ext.ux.StatusBar({
        id: 'my-status',

        // defaults to use when the status is cleared:
        defaultText: 'Default status text',
        defaultIconCls: 'default-icon',

        // values to set initially:
        text: 'Ready',
        iconCls: 'ready-icon',

        // any standard Toolbar items:
        items: [{
            text: 'A Button'
        }, '-', 'Plain Text']
    })
});

// Update the status bar later in code:
var sb = Ext.getCmp('my-status');
sb.setStatus({
    text: 'OK',
    iconCls: 'ok-icon',
    clear: true // auto-clear after a set interval
});

// Set the status bar to show that something is processing:
sb.showBusy();

// processing....

sb.clearStatus(); // once completeed
</code></pre>
 * @extends Ext.Toolbar
 * @constructor
 * Creates a new StatusBar
 * @param {Object/Array} config A config object
 */
Ext.ux.StatusBar = Ext.extend(Ext.Toolbar, {
    /**
     * @cfg {String} statusAlign
     * The alignment of the status element within the overall StatusBar layout.  When the StatusBar is rendered,
     * it creates an internal div containing the status text and icon.  Any additional Toolbar items added in the
     * StatusBar's {@link #items} config, or added via {@link #add} or any of the supported add* methods, will be
     * rendered, in added order, to the opposite side.  The status element is greedy, so it will automatically
     * expand to take up all sapce left over by any other items.  Example usage:
     * <pre><code>
// Create a left-aligned status bar containing a button,
// separator and text item that will be right-aligned (default):
new Ext.Panel({
    title: 'StatusBar',
    // etc.
    bbar: new Ext.ux.StatusBar({
        defaultText: 'Default status text',
        id: 'status-id',
        items: [{
            text: 'A Button'
        }, '-', 'Plain Text']
    })
});

// By adding the statusAlign config, this will create the
// exact same toolbar, except the status and toolbar item
// layout will be reversed from the previous example:
new Ext.Panel({
    title: 'StatusBar',
    // etc.
    bbar: new Ext.ux.StatusBar({
        defaultText: 'Default status text',
        id: 'status-id',
        statusAlign: 'right',
        items: [{
            text: 'A Button'
        }, '-', 'Plain Text']
    })
});
</code></pre>
     */
    /**
     * @cfg {String} defaultText
     * The default {@link #text} value.  This will be used anytime the status bar is cleared with the
     * <tt>useDefaults:true</tt> option (defaults to '').
     */
    /**
     * @cfg {String} defaultIconCls
     * The default {@link #iconCls} value (see the iconCls docs for additional details about customizing the icon).
     * This will be used anytime the status bar is cleared with the <tt>useDefaults:true</tt> option (defaults to '').
     */
    /**
     * @cfg {String} text
     * A string that will be <b>initially</b> set as the status message.  This string
     * will be set as innerHTML (html tags are accepted) for the toolbar item.
     * If not specified, the value set for <code>{@link #defaultText}</code>
     * will be used.
     */
    /**
     * @cfg {String} iconCls
     * A CSS class that will be <b>initially</b> set as the status bar icon and is
     * expected to provide a background image (defaults to '').
     * Example usage:<pre><code>
// Example CSS rule:
.x-statusbar .x-status-custom {
    padding-left: 25px;
    background: transparent url(images/custom-icon.gif) no-repeat 3px 2px;
}

// Setting a default icon:
var sb = new Ext.ux.StatusBar({
    defaultIconCls: 'x-status-custom'
});

// Changing the icon:
sb.setStatus({
    text: 'New status',
    iconCls: 'x-status-custom'
});
</code></pre>
     */

    /**
     * @cfg {String} cls
     * The base class applied to the containing element for this component on render (defaults to 'x-statusbar')
     */
    cls : 'x-statusbar',
    /**
     * @cfg {String} busyIconCls
     * The default <code>{@link #iconCls}</code> applied when calling
     * <code>{@link #showBusy}</code> (defaults to <tt>'x-status-busy'</tt>).
     * It can be overridden at any time by passing the <code>iconCls</code>
     * argument into <code>{@link #showBusy}</code>.
     */
    busyIconCls : 'x-status-busy',
    /**
     * @cfg {String} busyText
     * The default <code>{@link #text}</code> applied when calling
     * <code>{@link #showBusy}</code> (defaults to <tt>'Loading...'</tt>).
     * It can be overridden at any time by passing the <code>text</code>
     * argument into <code>{@link #showBusy}</code>.
     */
    busyText : 'Loading...',
    /**
     * @cfg {Number} autoClear
     * The number of milliseconds to wait after setting the status via
     * <code>{@link #setStatus}</code> before automatically clearing the status
     * text and icon (defaults to <tt>5000</tt>).  Note that this only applies
     * when passing the <tt>clear</tt> argument to <code>{@link #setStatus}</code>
     * since that is the only way to defer clearing the status.  This can
     * be overridden by specifying a different <tt>wait</tt> value in
     * <code>{@link #setStatus}</code>. Calls to <code>{@link #clearStatus}</code>
     * always clear the status bar immediately and ignore this value.
     */
    autoClear : 5000,

    /**
     * @cfg {String} emptyText
     * The text string to use if no text has been set.  Defaults to
     * <tt>'&nbsp;'</tt>).  If there are no other items in the toolbar using
     * an empty string (<tt>''</tt>) for this value would end up in the toolbar
     * height collapsing since the empty string will not maintain the toolbar
     * height.  Use <tt>''</tt> if the toolbar should collapse in height
     * vertically when no text is specified and there are no other items in
     * the toolbar.
     */
    emptyText : '&nbsp;',

    // private
    activeThreadId : 0,

    // private
    initComponent : function(){
        if(this.statusAlign === 'right'){
            this.cls += ' x-status-right';
        }
        Ext.ux.StatusBar.superclass.initComponent.call(this);
    },

    // private
    afterRender : function(){
        Ext.ux.StatusBar.superclass.afterRender.call(this);

        var right = this.statusAlign === 'right';
        this.currIconCls = this.iconCls || this.defaultIconCls;
        this.statusEl = new Ext.Toolbar.TextItem({
            cls: 'x-status-text ' + (this.currIconCls || ''),
            text: this.text || this.defaultText || ''
        });

        if(right){
            this.add('->');
            this.add(this.statusEl);
        }else{
            this.insert(0, this.statusEl);
            this.insert(1, '->');
        }
        this.doLayout();
    },

    /**
     * Sets the status {@link #text} and/or {@link #iconCls}. Also supports automatically clearing the
     * status that was set after a specified interval.
     * @param {Object/String} config A config object specifying what status to set, or a string assumed
     * to be the status text (and all other options are defaulted as explained below). A config
     * object containing any or all of the following properties can be passed:<ul>
     * <li><tt>text</tt> {String} : (optional) The status text to display.  If not specified, any current
     * status text will remain unchanged.</li>
     * <li><tt>iconCls</tt> {String} : (optional) The CSS class used to customize the status icon (see
     * {@link #iconCls} for details). If not specified, any current iconCls will remain unchanged.</li>
     * <li><tt>clear</tt> {Boolean/Number/Object} : (optional) Allows you to set an internal callback that will
     * automatically clear the status text and iconCls after a specified amount of time has passed. If clear is not
     * specified, the new status will not be auto-cleared and will stay until updated again or cleared using
     * {@link #clearStatus}. If <tt>true</tt> is passed, the status will be cleared using {@link #autoClear},
     * {@link #defaultText} and {@link #defaultIconCls} via a fade out animation. If a numeric value is passed,
     * it will be used as the callback interval (in milliseconds), overriding the {@link #autoClear} value.
     * All other options will be defaulted as with the boolean option.  To customize any other options,
     * you can pass an object in the format:<ul>
     *    <li><tt>wait</tt> {Number} : (optional) The number of milliseconds to wait before clearing
     *    (defaults to {@link #autoClear}).</li>
     *    <li><tt>anim</tt> {Number} : (optional) False to clear the status immediately once the callback
     *    executes (defaults to true which fades the status out).</li>
     *    <li><tt>useDefaults</tt> {Number} : (optional) False to completely clear the status text and iconCls
     *    (defaults to true which uses {@link #defaultText} and {@link #defaultIconCls}).</li>
     * </ul></li></ul>
     * Example usage:<pre><code>
// Simple call to update the text
statusBar.setStatus('New status');

// Set the status and icon, auto-clearing with default options:
statusBar.setStatus({
    text: 'New status',
    iconCls: 'x-status-custom',
    clear: true
});

// Auto-clear with custom options:
statusBar.setStatus({
    text: 'New status',
    iconCls: 'x-status-custom',
    clear: {
        wait: 8000,
        anim: false,
        useDefaults: false
    }
});
</code></pre>
     * @return {Ext.ux.StatusBar} this
     */
    setStatus : function(o){
        o = o || {};

        if(typeof o === 'string'){
            o = {text:o};
        }
        if(o.text !== undefined){
            this.setText(o.text);
        }
        if(o.iconCls !== undefined){
            this.setIcon(o.iconCls);
        }

        if(o.clear){
            var c = o.clear,
                wait = this.autoClear,
                defaults = {useDefaults: true, anim: true};

            if(typeof c === 'object'){
                c = Ext.applyIf(c, defaults);
                if(c.wait){
                    wait = c.wait;
                }
            }else if(typeof c === 'number'){
                wait = c;
                c = defaults;
            }else if(typeof c === 'boolean'){
                c = defaults;
            }

            c.threadId = this.activeThreadId;
            this.clearStatus.defer(wait, this, [c]);
        }
        return this;
    },

    /**
     * Clears the status {@link #text} and {@link #iconCls}. Also supports clearing via an optional fade out animation.
     * @param {Object} config (optional) A config object containing any or all of the following properties.  If this
     * object is not specified the status will be cleared using the defaults below:<ul>
     * <li><tt>anim</tt> {Boolean} : (optional) True to clear the status by fading out the status element (defaults
     * to false which clears immediately).</li>
     * <li><tt>useDefaults</tt> {Boolean} : (optional) True to reset the text and icon using {@link #defaultText} and
     * {@link #defaultIconCls} (defaults to false which sets the text to '' and removes any existing icon class).</li>
     * </ul>
     * @return {Ext.ux.StatusBar} this
     */
    clearStatus : function(o){
        o = o || {};

        if(o.threadId && o.threadId !== this.activeThreadId){
            // this means the current call was made internally, but a newer
            // thread has set a message since this call was deferred.  Since
            // we don't want to overwrite a newer message just ignore.
            return this;
        }

        var text = o.useDefaults ? this.defaultText : this.emptyText,
            iconCls = o.useDefaults ? (this.defaultIconCls || '') : '';

        if(o.anim){
            // animate the statusEl Ext.Element
            this.statusEl.el.fadeOut({
                remove: false,
                useDisplay: true,
                scope: this,
                callback: function(){
                    this.setStatus({
	                    text: text,
	                    iconCls: iconCls
	                });

                    this.statusEl.el.show();
                }
            });
        }else{
            // hide/show the el to avoid jumpy text or icon
            this.statusEl.hide();
	        this.setStatus({
	            text: text,
	            iconCls: iconCls
	        });
            this.statusEl.show();
        }
        return this;
    },

    /**
     * Convenience method for setting the status text directly.  For more flexible options see {@link #setStatus}.
     * @param {String} text (optional) The text to set (defaults to '')
     * @return {Ext.ux.StatusBar} this
     */
    setText : function(text){
        this.activeThreadId+=1;
        this.text = text || '';
        if(this.rendered){
            this.statusEl.setText(this.text);
        }
        return this;
    },

    /**
     * Returns the current status text.
     * @return {String} The status text
     */
    getText : function(){
        return this.text;
    },

    /**
     * Convenience method for setting the status icon directly.  For more flexible options see {@link #setStatus}.
     * See {@link #iconCls} for complete details about customizing the icon.
     * @param {String} iconCls (optional) The icon class to set (defaults to '', and any current icon class is removed)
     * @return {Ext.ux.StatusBar} this
     */
    setIcon : function(cls){
        this.activeThreadId+=1;
        cls = cls || '';

        if(this.rendered){
	        if(this.currIconCls){
	            this.statusEl.removeClass(this.currIconCls);
	            this.currIconCls = null;
	        }
	        if(cls.length > 0){
	            this.statusEl.addClass(cls);
	            this.currIconCls = cls;
	        }
        }else{
            this.currIconCls = cls;
        }
        return this;
    },

    /**
     * Convenience method for setting the status text and icon to special values that are pre-configured to indicate
     * a "busy" state, usually for loading or processing activities.
     * @param {Object/String} config (optional) A config object in the same format supported by {@link #setStatus}, or a
     * string to use as the status text (in which case all other options for setStatus will be defaulted).  Use the
     * <tt>text</tt> and/or <tt>iconCls</tt> properties on the config to override the default {@link #busyText}
     * and {@link #busyIconCls} settings. If the config argument is not specified, {@link #busyText} and
     * {@link #busyIconCls} will be used in conjunction with all of the default options for {@link #setStatus}.
     * @return {Ext.ux.StatusBar} this
     */
    showBusy : function(o){
        if(typeof o === 'string'){
            o = {text:o};
        }
        o = Ext.applyIf(o || {}, {
            text: this.busyText,
            iconCls: this.busyIconCls
        });
        return this.setStatus(o);
    }
});
Ext.reg('statusbar', Ext.ux.StatusBar);

});

/*!
 * Ext JS Library 3.4.0
 * Copyright(c) 2006-2011 Sencha Inc.
 * licensing@sencha.com
 * http://www.sencha.com/license
 */
/*global define*/
define('ext/ux/TabCloseMenu',['extjs'], function(Ext){
  /**
   * @class Ext.ux.TabCloseMenu
   * @extends Object
   * Plugin (ptype = 'tabclosemenu') for adding a close context menu to tabs. Note that the menu respects
   * the closable configuration on the tab. As such, commands like remove others and remove all will not
   * remove items that are not closable.
   *
   * @constructor
   * @param {Object} config The configuration options
   * @ptype tabclosemenu
   */
  Ext.ux.TabCloseMenu = Ext.extend(Object, {
    /**
     * @cfg {String} closeTabText
     * The text for closing the current tab. Defaults to <tt>'Close Tab'</tt>.
     */
    closeTabText: 'Close Tab',

    /**
     * @cfg {String} closeOtherTabsText
     * The text for closing all tabs except the current one. Defaults to <tt>'Close Other Tabs'</tt>.
     */
    closeOtherTabsText: 'Close Other Tabs',

    /**
     * @cfg {Boolean} showCloseAll
     * Indicates whether to show the 'Close All' option. Defaults to <tt>true</tt>.
     */
    showCloseAll: true,

    /**
     * @cfg {String} closeAllTabsText
     * <p>The text for closing all tabs. Defaults to <tt>'Close All Tabs'</tt>.
     */
    closeAllTabsText: 'Close All Tabs',

    constructor : function(config){
      Ext.apply(this, config || {});
    },

    //public
    init : function(tabs){
      this.tabs = tabs;
      tabs.on({
        scope: this,
        contextmenu: this.onContextMenu,
        destroy: this.destroy
      });
    },

    destroy : function(){
      Ext.destroy(this.menu);
      delete this.menu;
      delete this.tabs;
      delete this.active;
    },

    // private
    onContextMenu : function(tabs, item, e){
      this.active = item;
      var m = this.createMenu(),
            disableAll = true,
            disableOthers = true,
            closeAll = m.getComponent('closeall'),
            closableTabs = 0;

      m.getComponent('close').setDisabled(!item.closable);
      tabs.items.each(function(){
        if (this.closable && this != item) {
          disableAll = false;
          disableOthers = false;
          return false;
        }
      });
      m.getComponent('closeothers').setDisabled(disableOthers);
      if(closeAll){
        closeAll.setDisabled(disableAll);
      }

      e.stopEvent();
      m.showAt(e.getPoint());
    },

    createMenu : function(){
      if(!this.menu){
        var items = [{
          itemId: 'close',
          text: this.closeTabText,
          scope: this,
          handler: this.onClose
        }];
        if(this.showCloseAll){
          items.push('-');
        }
        items.push({
          itemId: 'closeothers',
          text: this.closeOtherTabsText,
          scope: this,
          handler: this.onCloseOthers
        });
        if(this.showCloseAll){
          items.push({
            itemId: 'closeall',
            text: this.closeAllTabsText,
            scope: this,
            handler: this.onCloseAll
          });
        }
        this.menu = new Ext.menu.Menu({
          items: items
        });
      }
      return this.menu;
    },

    onClose : function(){
      this.tabs.remove(this.active);
    },

    onCloseOthers : function(){
      this.doClose(true);
    },

    onCloseAll : function(){
      this.doClose(false);
    },

    doClose : function(excludeActive){
      var items = [];
      this.tabs.items.each(function(item){
        if(item.closable){
          if(!excludeActive || item != this.active){
            items.push(item);
          }
        }
      }, this);
      Ext.each(items, function(item){
        this.tabs.remove(item);
      }, this);
    }
  });

  Ext.preg('tabclosemenu', Ext.ux.TabCloseMenu);
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/util',
      [
        'ext/util/event'
      ],
      function(Ext) {
        return Ext;
      });
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/form',
      [
        'ext/form/basicform',
        'ext/form/displayfield',
        'ext/form/field',
        'ext/form/formpanel',
        'ext/form/textarea',
        'ext/form/TextField',
        'ext/form/VTypes'
      ],
      function() {});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/lib',
      [
        'ext/lib/ajax'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/data',
      [
        'ext/data/connection', 'ext/data/store'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/lib/ajax',['extjs'], function(Ext){
Ext.lib.Ajax.setHeader = function(o) {
  var combinedHeaders, prop;

  // Sonatype: Safari and IE don't always overwrite headers correctly, so need
  // to merge default and provide headers before writing
  if (this.hasDefaultHeaders && this.hasHeaders)
  {
    combinedHeaders = Ext.applyIf(this.headers, this.defaultHeaders);

    for (prop in combinedHeaders)
    {
      if (combinedHeaders.hasOwnProperty(prop))
      {
        o.conn.setRequestHeader(prop, combinedHeaders[prop]);
      }
    }

    this.headers = {};
    this.hasHeaders = false;
  }
  else if (this.hasDefaultHeaders)
  {
    for (prop in this.defaultHeaders)
    {
      if (this.defaultHeaders.hasOwnProperty(prop))
      {
        o.conn.setRequestHeader(prop, this.defaultHeaders[prop]);
      }
    }
  }
  else if (this.hasHeaders)
  {
    for (prop in this.headers)
    {
      if (this.headers.hasOwnProperty(prop))
      {
        o.conn.setRequestHeader(prop, this.headers[prop]);
      }
    }
    this.headers = {};
    this.hasHeaders = false;
  }
};

Ext.lib.Ajax.handleTransactionResponse = function(o, callback, isAbort) {
  if (!callback)
  {
    this.releaseObject(o);
    return;
  }

  var httpStatus, responseObject;

  try
  {
    if (o.conn.status !== undefined && o.conn.status !== 0)
    {
      httpStatus = o.conn.status;
    }
    else
    {
      httpStatus = 13030;
    }
  }
  catch (e)
  {

    httpStatus = 13030;
  }

  if ((httpStatus >= 200 && httpStatus < 300) || httpStatus === 1223)
  {
    responseObject = this.createResponseObject(o, callback.argument);
    if (callback.success)
    {
      if (!callback.scope)
      {
        callback.success(responseObject);
      }
      else
      {

        callback.success.apply(callback.scope, [responseObject]);
      }
    }
  } else {
    switch (httpStatus) {
      case 12002 :
      case 12029 :
      case 12030 :
      case 12031 :
      case 12152 :
      case 13030 :
        responseObject = this.createExceptionObject(o.tId, callback.argument, (isAbort || false));
        if (callback.failure)
        {
          if (!callback.scope)
          {
            callback.failure(responseObject);
          }
          else
          {
            callback.failure.apply(callback.scope, [responseObject]);
          }
        }
        break;
      default :
        responseObject = this.createResponseObject(o, callback.argument);
        if (callback.failure)
        {
          if (!callback.scope)
          {
            callback.failure(responseObject);
          }
          else
          {
            callback.failure.apply(callback.scope, [responseObject]);
          }
        }
    }
  }

  this.releaseObject(o);
  responseObject = null;
};
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/dd/dropzone',['extjs'], function(Ext) {
Ext.dd.DropZone = function(el, config) {
  Ext.dd.DropZone.superclass.constructor.call(this, el, config);
};

Ext.extend(Ext.dd.DropZone, Ext.dd.DropTarget, {
  getTargetFromEvent : function(e) {
    return Ext.dd.Registry.getTargetFromEvent(e);
  },
  onNodeEnter : function(n, dd, e, data) {},
  onNodeOver : function(n, dd, e, data) {
    return this.dropAllowed;
  },
  onNodeOut : function(n, dd, e, data) {},
  onNodeDrop : function(n, dd, e, data) {
    return false;
  },
  onContainerOver : function(dd, e, data) {
    return this.dropNotAllowed;
  },
  onContainerDrop : function(dd, e, data) {
    return false;
  },
  notifyEnter : function(dd, e, data) {
    return this.dropNotAllowed;
  },
  notifyOver : function(dd, e, data) {
    var n = this.getTargetFromEvent(e);
    if (!n)
    {
      if (this.lastOverNode)
      {
        this.onNodeOut(this.lastOverNode, dd, e, data);
        this.lastOverNode = null;
      }
      return this.onContainerOver(dd, e, data);
    }
    if (this.lastOverNode !== n)
    {
      if (this.lastOverNode)
      {
        this.onNodeOut(this.lastOverNode, dd, e, data);
      }
      this.onNodeEnter(n, dd, e, data);
      this.lastOverNode = n;
    }
    return this.onNodeOver(n, dd, e, data);
  },
  notifyOut : function(dd, e, data) {
    if (this.lastOverNode)
    {
      this.onNodeOut(this.lastOverNode, dd, e, data);
      this.lastOverNode = null;
    }
  },
  notifyDrop : function(dd, e, data) {
    if (this.lastOverNode)
    {
      this.onNodeOut(this.lastOverNode, dd, e, data);
      this.lastOverNode = null;
    }
    var n = this.getTargetFromEvent(e);
    return n ? this.onNodeDrop(n, dd, e, data) : this.onContainerDrop(dd, e, data);
  },
  triggerCacheRefresh : function() {
    Ext.dd.DDM.refreshCache(this.groups);
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/dd/droptarget', ['extjs'], function(Ext) {
  Ext.override(Ext.dd.DropTarget, {
    constructor : function(el, config) {
      this.el = Ext.get(el);

      Ext.apply(this, config);

      if (this.containerScroll) {
        Ext.dd.ScrollManager.register(this.el);
      }

      Ext.dd.DropTarget.superclass.constructor.call(this, this.el.dom, this.ddGroup || this.group,
        // Sonatype: allow config to pass thru to Ext.dd.DDTarget constructor so padding may
        // be passed from Ext.tree.TreePanel config when it need to have the whole drop panel
        // set as the drop zone because this is not done by the library
        config || {
          isTarget : true
        });
    }
  });
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/dd',
      [
        'ext/dd/droptarget', 'ext/dd/dropzone'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/element', ['extjs'], function(Ext) {
  Ext.override(Ext.Element, {
    setHeightOrig : Ext.Element.prototype.setHeight,
    setHeight : function(height)
    {
      // height is NaN for IE8 because syncHeight() will call this with
      // Math.max(0, "100%" - 27) == NaN
      if ( Ext.isIE8 && typeof height === 'number' && isNaN(height) ) {
        height = 0;
      }

      return this.setHeightOrig.call(this, height);
    }
  });
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/ux',
      ['extjs',
       'ext/ux/browsebutton',
       'ext/ux/gridvalidator',
       'ext/ux/lovcombo',
       'ext/ux/multiselecttree',
        'ext/ux/TabCloseMenu',
        'ext/ux/statusbar',
        'ext/ux/CheckColumn',
        'ext/ux/RowExpander'
      ],
      function(Ext) {
        Ext.namespace('Ext.ux');
        return Ext;
      });
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/tree/treedropzone',['extjs'], function(Ext){
Ext.override(Ext.tree.TreeDropZone, {
  completeDrop : function(de) {
    var ns = de.dropNode, p = de.point, t = de.target, i, len, n, node, ins = false;

    if (!Ext.isArray(ns))
    {
      ns = [ns];
    }
    if (p !== 'append')
    {
      ins = true;
      node = (p === 'above') ? t : t.nextSibling;
    }
    for (i = 0, len = ns.length; i < len; i=i+1)
    {
      n = ns[i];
      if (ins)
      {
        t.parentNode.insertBefore(n, node);
      }
      else
      {
        t.appendChild(n);
      }
      if (Ext.enableFx && this.tree.hlDrop)
      {
        n.ui.highlight();
      }
    }
    ns[0].ui.focus();
    t.ui.endDrop();
    this.tree.fireEvent("nodedrop", de);
  }

});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
// allow eval statement because it's used for uiProvider attribute, not sure where that is set so we cannot drop it
// without potentially breaking things
/*jslint evil:true*/

define('ext/tree/sonatype',['extjs'], function(Ext){
Ext.tree.SonatypeTreeLoader = function(config) {
  config.requestMethod = "GET";

  Ext.tree.SonatypeTreeLoader.superclass.constructor.call(this, config);
};

Ext.extend(Ext.tree.SonatypeTreeLoader, Ext.tree.TreeLoader, {
      load : function(node, callback) {
        if (this.clearOnLoad)
        {
          while (node.firstChild)
          {
            node.removeChild(node.firstChild);
          }
        }
        if (this.doPreload(node))
        { // preloaded json children
          if (typeof callback === "function")
          {
            callback();
          }
        }
        else if (typeof(this.dataUrl) !== 'undefined' || typeof(this.url) !== 'undefined')
        { // diff
          this.requestData(node, callback);
        }
      },

      // override to request data according ot Sonatype's Nexus REST service
      requestData : function(node, callback) {
        if (this.fireEvent("beforeload", this, node, callback) !== false)
        {
          this.transId = Ext.Ajax.request({
                method : this.requestMethod,
                url : node.id + '?isLocal', // diff
                success : this.handleResponse,
                failure : this.handleFailure,
                options : {
                  dontForceLogout : true
                },
                scope : this,
                argument : {
                  callback : callback,
                  node : node
                },
                // disableCaching: false, //diff
                params : this.getParams(node)
              });
        }
        else
        {
          // if the load is cancelled, make sure we notify
          // the node that we are done
          if (typeof callback === "function")
          {
            callback();
          }
        }
      },

      getParams : function(node) {
        var buf = [], bp = this.baseParams, key;
        for (key in bp)
        {
          if (typeof bp[key] !== "function")
          {
            buf.push(encodeURIComponent(key), "=", encodeURIComponent(bp[key]), "&");
          }
        }
        // buf.push("node=", encodeURIComponent(node.id)); //diff
        return buf.join("");
      },

      processResponse : function(response, node, callback) {
        try
        {
          var i, len, data = Ext.decode(response.responseText).data;
          if (this.jsonRoot) {
            data = data[this.jsonRoot];
          }
          
          node.beginUpdate();
          for (i = 0, len = data.length; i < len; i=i+1)
          {
            if (this.createNode(data[i]))
            {
              node.appendChild(this.createNode(data[i]));
            }
          }
          node.endUpdate();
          if (typeof callback === "function")
          {
            callback(this, node);
          }
        }
        catch (e)
        {
          this.handleFailure(response);
        }
      },

      // FIXME this method is only copy/paste? need to compare
      createNode : function(attr) {
        // apply baseAttrs, nice idea Corey!
        if (this.baseAttrs)
        {
          Ext.applyIf(attr, this.baseAttrs);
        }
        if (this.nodeTextAttribute){
          attr.text = attr[this.nodeTextAttribute];
        }
        if (this.nodeForceLeaf){
          attr.leaf = true;
        }
        if (!attr.id)
        { // diff
          attr.id = attr.resourceURI; // diff
        } // diff
        if (this.applyLoader !== false)
        {
          attr.loader = this;
        }
        if (typeof attr.uiProvider === 'string')
        {
          attr.uiProvider = this.uiProviders[attr.uiProvider] || eval(attr.uiProvider);
        }

        attr.singleClickExpand = true; // diff

        return (attr.leaf ? new Ext.tree.TreeNode(attr) : new Ext.tree.AsyncTreeNode(attr));
      }
    });

// create constructor for new class
Ext.tree.SonatypeTreeSorter = function(el, config) {
  Ext.tree.SonatypeTreeSorter.superclass.constructor.call(this, el, config);
};

Ext.extend(Ext.tree.SonatypeTreeSorter, Ext.tree.TreeSorter, {
      disableSort : function(tree) {
        tree.un("beforechildrenrendered", this.doSort, this);
        tree.un("append", this.updateSort, this);
        tree.un("insert", this.updateSort, this);
        tree.un("textchange", this.updateSortParent, this);
      },
      enableSort : function(tree) {
        tree.on("beforechildrenrendered", this.doSort, this);
        tree.on("append", this.updateSort, this);
        tree.on("insert", this.updateSort, this);
        tree.on("textchange", this.updateSortParent, this);

        this.doSort(tree.root);
      }
    });
 });
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/tree/treeloader',['extjs'], function(Ext){
Ext.tree.TreeLoader.override({
  requestData : function(node, callback) {
    if (this.fireEvent("beforeload", this, node, callback) !== false)
    {
      this.transId = Ext.Ajax.request({
        method : this.requestMethod,
        url : this.dataUrl || this.url,
        success : this.handleResponse,
        failure : this.handleFailure,
        timeout : this.timeout || 1000,
        scope : this,
        argument : {
          callback : callback,
          node : node
        },
        params : this.getParams(node)
      });
    }
    else
    {
      // if the load is cancelled, make sure we notify
      // the node that we are done
      if (typeof callback === "function")
      {
        callback();
      }
    }
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/layout',
      [
        'ext/layout/absolutelayout'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/util/event', ['extjs', 'Nexus/log', 'Nexus/error/ErrorHandler'], function(Ext, Log, ErrorHandler){
  Ext.util.Event = Ext.extend(Ext.util.Event, {
    createListener : function() {
      var
            listener = Ext.util.Event.superclass.createListener.apply(this, arguments),
            fireFn = listener.fireFn;

      listener.fireFn = function() {
        try {
          return fireFn.apply(this, arguments);
        } catch(e) {
          ErrorHandler.handleError(e);
        }
      };

      return listener;
    }
  });
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('ext/tree',
      [
        'ext/tree/sonatype',
        'ext/tree/treedropzone',
        'ext/tree/treeloader'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,NX, Nexus*/

/**
 * Mixin to make classes logging aware.
 *
 * @since 2.4
 */
NX.define('Nexus.LogAwareMixin', {

    statics: {
        /**
         * True to include class-names in log message (default); false to omit it.
         *
         * @property
         */
        includeName: true,

        /**
         * True to use simple class-names (default); false to use full class-names.
         *
         * @property
         */
        simpleName: true,

        /**
         * True to include level in log message (default); false to omit it.
         *
         * @property
         */
        includeLevel: true
    },

    /**
     * @private
     *
     * @param {String} level
     * @param {Array} args
     */
    logFormat: function (level, args) {
        var name,
            config = Nexus.LogAwareMixin; // config pulled from static properties

        // maybe prepend class-name
        if (config.includeName === true) {
            name = this.$className;
            if (config.simpleName === true) {
                name = this.$simpleClassName;
            }
            args.unshift(name + ':');
        }

        // maybe prepend level
        if (config.includeLevel === true) {
            args.unshift('[' + level.toUpperCase() + ']');
        }

        return args;
    },

    /**
     * @private
     *
     * @param {String} level
     * @param {Array} args
     */
    logx: function (level, args) {
        var fn;

        NX.assert(NX.log.levels[level] !== undefined, 'Invalid log level: ' + level);
        NX.assert(args.length !== 0, 'Missing log message detail');

        args = this.logFormat(level, args);

        fn = NX.log[level];
        fn.apply(NX.log, args);
    },

    /**
     * @protected
     */
    logDebug: function () {
        this.logx('debug', Array.prototype.slice.call(arguments));
    },

    /**
     * @protected
     */
    logInfo: function () {
        this.logx('info', Array.prototype.slice.call(arguments));
    },

    /**
     * @protected
     */
    logWarn: function () {
        this.logx('warn', Array.prototype.slice.call(arguments));
    },

    /**
     * @protected
     */
    logError: function () {
        this.logx('error', Array.prototype.slice.call(arguments));
    },

    /**
     * @protected
     */
    logGroup: function () {
        NX.log.group.apply(NX.log, this.logFormat('group', Array.prototype.slice.call(arguments)));
    },

    /**
     * @protected
     */
    logGroupEnd: function () {
        NX.log.groupEnd();
    }

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/form/LabelField',['extjs', 'nexus'], function(Ext, Nexus) {
Ext.namespace('Nexus.form');

/**
 * A field using a label to display the value.
 */
Nexus.form.LabelField = Ext.extend( Ext.form.Field, {
  onRender : function(ct, position){
    this.defaultAutoCreate = {
      tag : 'label',
      // a label inside a form element inherits 'clear: left' -> broken layout
      style : 'clear: none;',
      html : this.value
    };
    Nexus.form.LabelField.superclass.onRender.call(this, ct, position);
  },

  // overriding value getter and setter to make this behave more like a field
  // (a label's el.dom has no 'value' attribute which is what the superclass tries
  // to manipulate)
  getValue : function() {
    return this.getRawValue();
  },
  setValue : function(value) {
    this.setRawValue(value);
  },
  getRawValue : function() {
    return this.value;
  },
  setRawValue : function(value) {
    this.value = value;
    if (this.rendered) {
      this.el.dom.innerHTML = this.value;
    }
  }
});

Ext.reg('labelfield', Nexus.form.LabelField);
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/form/searchfield',['extjs', 'nexus'], function(Ext, Nexus) {
Ext.namespace('Nexus.form');

Nexus.form.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
      initComponent : function() {
        Ext.app.SearchField.superclass.initComponent.call(this);
        this.on('specialkey', function(f, e) {
              if (e.getKey() === e.ENTER)
              {
                this.onTrigger2Click();
              }
            }, this);
        if (this.searchPanel)
        {
          this.searchPanel.searchField = this;
        }
      },

      validationEvent : false,
      validateOnBlur : false,
      trigger1Class : 'x-form-clear-trigger',
      trigger2Class : 'x-form-search-trigger',
      hideTrigger1 : true,
      width : 180,
      paramName : 'q',

      onTrigger1Click : function() {
        if (this.getRawValue())
        {
          this.el.dom.value = '';
          this.triggers[0].hide();
          this.hasSearch = false;
        }
        if (this.searchPanel.stopSearch)
        {
          this.searchPanel.stopSearch(this.searchPanel);
        }
      },

      onTrigger2Click : function() {
        var v = this.getRawValue();
        if (v.length < 1)
        {
          this.onTrigger1Click();
          return;
        }
        // var o = {start: 0};
        this.searchPanel.startSearch(this.searchPanel, true);
      },

      /**
       * Override TwinTriggerField#afterRender, because position calculation for IE was always off by 1 (or even many)
       * pixels by default.
       */
      afterRender : function(){
        Ext.form.TriggerField.superclass.afterRender.call(this);
        var y;
        if (Ext.isIE && !this.hideTrigger) {
          if (Ext.isIE8||Ext.isIE6) { // IE6 is also discovered for IE9
            this.el.position();
            this.el.setY(this.el.getY()+1);
          } else if(Ext.isIE7 && this.el.getY() !== (y = this.trigger.getY())){
            this.el.position();
            this.el.setY(y);
          }
        }
      }
    });

Ext.reg('nexussearchfield', Nexus.form.SearchField);

// FIXME: legacy
Ext.app.SearchField = Nexus.form.SearchField;


});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/form/action',['extjs', 'nexus', 'Nexus/messagebox', 'Sonatype/utils'], function(Ext, Nexus, messagebox, Sonatype) {

Ext.namespace('Nexus.form');

/**
 * @class Ext.form.Action.sonatypeSubmit
 * @extends Ext.form.Action A custom sonatype form serializer that submits JSON
 *          text to the Sonatype service and processes the returned response.
 *          ToDo: Define the error response format and variations. How to expose
 *          this to callbacks and event handlers? Other data may be placed into
 *          the response for processing the the Ext.form.BasicForm's callback or
 *          event handler methods. The object decoded from this JSON is
 *          available in the result property. Note: no form.errorReader
 *          accepted. JSON error format (when defined) is the only format
 *          accepted. Additional option params: serviceDataObj: reference object
 *          that matches the service's data object for this action fpanel:
 *          FormPanel dataModifiers: optional functions to modify or collect
 *          data values Additional values: after submit the output object is
 *          available on action.output. This is the native object that was
 *          serialized and sent to the server.
 */
Ext.form.Action.sonatypeSubmit = function(form, options) {
  if (options.autoValidation === undefined)
  {
    options.autoValidation = true;
  }
  Ext.form.Action.sonatypeSubmit.superclass.constructor.call(this, form, options);
};

  function setUnpacked(obj, flatName, value) {
    var ctx = obj, id = flatName;

    Ext.each( flatName.split('.'), function(part) {
      ctx = obj;
      id = part;
      obj = obj[part];
    });

    ctx[id] = value;
  }


Ext.extend(Ext.form.Action.sonatypeSubmit, Ext.form.Action, {
  /**
   * @cfg {boolean} clientValidation Determines whether a Form's fields are
   *      validated in a final call to
   *      {@link Ext.form.BasicForm#isValid isValid} prior to submission. Pass
   *      <tt>false</tt> in the Form's submit options to prevent this. If not
   *      defined, pre-submission field validation is performed.
   */
  type : 'sonatypeSubmit',

  // private
  run : function() {
    var
          sJsonOutput,
          o = this.options,
          method = this.getMethod(),
          isPost = method === 'POST';

    if (o.clientValidation === false || this.form.isValid())
    {
      sJsonOutput = this.serializeForm(this.options.fpanel, this.form);

      Ext.Ajax.request(Ext.apply(this.createCallback(o), {
        jsonData : sJsonOutput,
        url : this.getUrl(!isPost),
        method : method,
        params : isPost ? this.getParams() : null,
        isUpload : this.form.fileUpload
      }));
    }
    else if (o.clientValidation !== false)
    { // client validation failed
      this.failureType = Ext.form.Action.CLIENT_INVALID;
      this.form.afterAction(this, false);
    }
  },

  // override connection failure because server validation errors come back with
  // 400 code
  failure : function(response) {
    this.response = response;
    if (response.status === 400 && this.options.autoValidation)
    { // validation error
      this.success(response);
      return;
    }

    this.failureType = Ext.form.Action.CONNECT_FAILURE;
    this.form.afterAction(this, false);
  },

  // private
  success : function(response) {
    var
          i, errorObj,
          result = this.processResponse(response),
          remainingErrors = [];

    // if a 204 response, we arent looking at errors, it should go through ok
    if (result === true || result.data || response.status === 200 || response.status === 204 ||
      // 1223 is the IE way to deal with 204 http://goo.gl/xzJt1
          response.status === 1223)
    {
      this.form.afterAction(this, true);
      return;
    }

    var contentType = response.getResponseHeader("Content-Type");

    // response will be a json serialized org.sonatype.sisu.siesta.common.error.ErrorXO
    if( contentType === "application/vnd.siesta-error-v1+json") {
      result = { errors: [{ id:'*', msg : result.message }]};
    }

    // response will be a json serialized array of org.sonatype.sisu.siesta.common.validation.ValidationErrorXO
    // convert error message field (ExtJs expects a field named "msg")
    if( contentType === "application/vnd.siesta-validation-errors-v1+json") {
        result = { errors : result };
        for (i = 0; i < result.errors.length; i=i+1) {
          result.errors[i].msg = result.errors[i].message;
        }
    }

    if (result.errors !== null && result.errors !== undefined)
    {
      if (this.options.validationModifiers)
      {
        for (i = 0; i < result.errors.length; i=i+1)
        {
          if (this.options.validationModifiers[result.errors[i].id])
          {
            if (typeof(this.options.validationModifiers[result.errors[i].id]) === 'function')
            {
              (this.options.validationModifiers[result.errors[i].id])(result.errors[i], this.options.fpanel);
            }
            else
            {
              errorObj = result.errors[i];
              errorObj.id = this.options.validationModifiers[result.errors[i].id];
              remainingErrors[remainingErrors.length] = errorObj;
            }
          }
          else
          {
            remainingErrors[remainingErrors.length] = result.errors[i];
          }
        }

        result.errors = remainingErrors;
      }

      if (result.errors.length === 1 && result.errors[0].id === '*')
      {
        messagebox.show({
          title : 'Configuration Error',
          msg : result.errors[0].msg,
          buttons : messagebox.OK,
          icon : messagebox.ERROR
        });
        return;
      }

      this.form.markInvalid(result.errors);
      this.failureType = Ext.form.Action.SERVER_INVALID;
    }
    this.form.afterAction(this, false);

    // if we came in here on 400 error, and we properly marked field as error,
    // hide dialog
    if (result.errors)
    {
      messagebox.hide();
    }
  },

  // private
  handleResponse : function(response) {
    try
    {
      return Ext.decode(response.responseText); // throws SyntaxError
    }
    catch (e)
    {
      return false;
    }
  },

  // private
  serializeForm : function(fpanel, form) {
    // note: srcObj (form.sonatypeLoadedData) is not modified only walked
    var
          output = Sonatype.utils.cloneObj(this.options.serviceDataObj),
          resultOutput = this.serializeFormHelper(fpanel, output, this.options.serviceDataObj, '');

    if(this.options.noEnvelope){
      this.output = resultOutput || output;
    } else {
      this.output = {
        "data" : resultOutput || output
      };
    }

    Nexus.Log.debug(this.options.method + ' ' + this.options.url + ' ', this.output);

    return Ext.encode(this.output);
  },

  // serializeHelper(object fpanel, object accObj, object srcObj, string
  // sPrepend, [string sVal])
  // Leave off sVal arg to call on root data obj
  // Walks the data object sent from the server originally, and plucks field
  // values from form
  // applying modifier functions if specified. Handles collapsed fleldsets by
  // returning null
  // as object value to server.
  // Invariant: srcObj is not modified! If this changes, call with a cloned copy
  serializeFormHelper : function(fpanel, accObj, srcObj, sPrepend, sVal) {
    var i, value, nextPrepend, fieldValue, fieldSet, flatName, field, fieldId, obj, context;
    if (sVal)
    { // non-root case
      nextPrepend = sPrepend + sVal + '.';
      value = srcObj[sVal]; // @todo: "value" name here is whack because, it's
      // not the field value
    }
    else
    { // root case
      nextPrepend = sPrepend;
      value = srcObj;
    }

    obj = accObj;
    context = accObj;

    // this function goes through flatName and 'unpacks' the objects in the path
    // and sets the value
    // was written as "eval('accObj.' + flatName + '=' + fieldValue)" before
    if (this.options.dataModifiers && this.options.dataModifiers.rootData)
    {
      return (this.options.dataModifiers.rootData)(fieldValue, fpanel);
    }
    else if (Ext.type(value) === 'object')
    {
      if (sVal)
      { // only write object serialization for non-root objects
        fieldSet = Ext.value(
              Ext.getCmp(fpanel.id + '_' + sPrepend + sVal),
              fpanel.find('name', 'fieldset_' + sPrepend + sVal)[0]
        );

        if (fieldSet && fieldSet.collapsed)
        {
          setUnpacked(accObj, sPrepend + sVal, null);
          return; // skip recursive calls for children form items
        }
      }

      for (i in value)
      {
        if ( value.hasOwnProperty(i)) {
          this.serializeFormHelper(fpanel, accObj, value, nextPrepend, i);
        }
      }
    }
    else
    { // only non-root case should ever get in here
      flatName = sPrepend + sVal;
      field = fpanel.form.findField(flatName);

      if (field && !Ext.isEmpty(field.getValue(), false))
      { // getValue normalizes undefined to '', but it's still false
        fieldValue = field.getValue();
      }

      // data mod function gets the field value if the field exists. o/w null is
      // passed
      fieldValue = (this.options.dataModifiers && this.options.dataModifiers[flatName]) ?
            (this.options.dataModifiers[flatName])(fieldValue, fpanel) :
            fieldValue;

      setUnpacked(accObj, flatName, fieldValue);
    }
  }

  // @note: this was going to be used for processing hierarchical error
  // messaging
  // // walks the error response and flattens each field into a flat object by
  // fieldname
  // flattenErrors : function(accObj, srcObj, sPrepend, sVal){
  // var value, nextPrepend;
  // if (sVal){ //non-root case
  // nextPrepend = sPrepend + sVal + '.';
  // value = srcObj[sVal];
  // }
  // else { //root case
  // nextPrepend = sPrepend;
  // value = srcObj;
  // }
  //
  // if (Ext.type(value) === 'object'){
  // for (var i in value){
  // this.flattenErrors(fpanel, accObj, value, nextPrepend, i);
  // }
  // }
  // else { //only non-root case should ever get in here
  // var flatName = sPrepend + sVal;
  // if(!Ext.isEmpty(value, true)){
  // accObj[flatName] = value;
  // }
  // }
  // }
});

/**
 * @class Ext.form.Action.sonatypeLoad
 * @extends Ext.form.Action A class which handles loading of data from Sonatype
 *          service into the Fields of an Ext.form.BasicForm. Expected repsonse
 *          format { data: { clientName: "Fred. Olsen Lines", portOfLoading:
 *          "FXT", portOfDischarge: "OSL" } } Other data may be placed into the
 *          response for processing the Ext.form.BasicForm Form's callback or
 *          event handler methods. The object decoded from this JSON is
 *          available in the result property. Needed Improvements: create a
 *          standard way to for callbacks to assess server respone's success.
 *          The regular loader gave access to this in the data.success field
 *          that it required in its data format. We could access
 *          this.response.HTTPcode (?). Should we push that down in an acessible
 *          way, so every callback doesn't need to understand our service
 *          response codes. Notes No form.reader may be used here. JSON data
 *          format is assumed this loader Additional options params: fpanel the
 *          FromPanel containing this form dataModifiers (optional)
 */
Ext.form.Action.sonatypeLoad = function(form, options) {
  Ext.form.Action.sonatypeLoad.superclass.constructor.call(this, form, options);
};

Ext.extend(Ext.form.Action.sonatypeLoad, Ext.form.Action, {
  // private
  type : 'sonatypeLoad',

  // private
  run : function() {
    Ext.Ajax.request(Ext.apply(this.createCallback(this.options), {
      method : this.getMethod(),
      url : this.getUrl(false),
      params : this.getParams(),
      suppressStatus : this.options.suppressStatus
    }));
  },

  // private
  // note: service response object "data" value expected here in result.data
  success : function(response) {
    var flatData, result = this.processResponse(response);

    data = result.data;
    if(this.options.noEnvelope){
      data = result;
    }
    if (result === true || !data)
    {
      this.failureType = Ext.form.Action.LOAD_FAILURE;
      this.form.afterAction(this, false);
      return;
    }
    this.form.clearInvalid();
    flatData = this.translateDataToFieldValues(this.options.fpanel, data);
    this.form.setValues(flatData);
    this.form.afterAction(this, true);
  },

  // private
  // called from in Ext.form.Action.processResponse
  handleResponse : function(response) {
    return Ext.decode(response.responseText);
  },

  // private
  // takes result.data and returns flattened data to pass to
  // this.form.setValues()
  translateDataToFieldValues : function(fpanel, data) {
    var flat = {};
    this.translateHelper(fpanel, flat, data, '');
    return flat;
  },

  // translateHelper(object accObj, object srcObj, string sPrepend, [string
  // sVal])
  // Leave off sVal arg to call on root data obj
  translateHelper : function(fpanel, accObj, srcObj, sPrepend, sVal) {
    var value, nextPrepend, hasNonEmptyChildren = false, i, thisChildNotEmpty, fieldSet, flatName;
    if (sVal)
    { // non-root case
      nextPrepend = sPrepend + sVal + '.';
      value = srcObj[sVal];
    }
    else
    { // root case
      nextPrepend = sPrepend;
      value = srcObj;
    }

    if (this.options.dataModifiers && this.options.dataModifiers.rootData)
    {
      accObj = this.options.dataModifiers.rootData(value, srcObj, fpanel);
    }
    else if (Ext.type(value) === 'object')
    {
      for (i in value)
      {
        if ( value.hasOwnProperty(i)) {
          thisChildNotEmpty = this.translateHelper(fpanel, accObj, value, nextPrepend, i);
          hasNonEmptyChildren = hasNonEmptyChildren || thisChildNotEmpty;
        }
      }

      if (sVal) { // only write object serialization for non-root objects
        if (hasNonEmptyChildren)
        {
          fieldSet = Ext.getCmp(fpanel.id + '_' + sPrepend + sVal);
          if (!fieldSet)
          {
            fieldSet = fpanel.find('name', 'fieldset_' + sPrepend + sVal)[0];
          }
          if (fieldSet)
          {
	      if ( fieldSet.rendered ) {
		  fieldSet.expand();
	      } else {
		  fieldSet.collapsed = false;
	      }
          }
        }
        accObj['.' + sPrepend + sVal] = hasNonEmptyChildren;
      }
      return hasNonEmptyChildren;
    }
    else
    { // only non-root case should ever get in here
      flatName = sPrepend + sVal;
      // note: all vaues passed to modifier funcs, even if the value is
      // undefined, null, or empty!
      // Modifier funcs should ALWAYS return a value, even if it's
      // unmodified.
      value = (this.options.dataModifiers && this.options.dataModifiers[flatName]) ? this.options.dataModifiers[flatName](value, srcObj, fpanel) : value;
      if (Ext.isEmpty(value, true))
      {
        return false;
      }

      // flatName is correct here, because we're selecting field ids with it
      accObj[flatName] = value;
      return true;
    }
  }
});

Ext.form.Action.ACTION_TYPES.sonatypeLoad = Ext.form.Action.sonatypeLoad;
Ext.form.Action.ACTION_TYPES.sonatypeSubmit = Ext.form.Action.sonatypeSubmit;

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define, top*/
define('Nexus/form/roleselector',['extjs', 'nexus'], function(Ext, Nexus) {
var ns = Ext.namespace('Nexus.form');
/**
 * A RoleManager is used to display assigned roles and privileges (optional) in a grid, with a toolbar
 * that has options to add roles/privileges (which will open up a new window that will show a specialized role/privilege list
 * grid that will allow for filtering/paging the lists and selection for addition) and to remove roles/privileges ( again opening
 * up a new window, where the user can simply deselect available items ) 
 *  
 * usePrivileges - boolean flag, if true will use the privileges along with the roles 
 * selectedRoleIds - role ids that are already assigned 
 * selectedPrivilegeIds - privilege ids that are already assigned
 * userId - userId which is used to properly retrieve the data for UI (most specifically the external roles)
 */

ns.RoleManager = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          header : false,
          trackMouseOver : false,
          loadMask : true,
          usePrivileges : true,
          selectedRoleIds : [],
          selectedPrivilegeIds : [],
          onlySelected : true,
          userId : null,
          hideHeaders : true,
          doValidation : true,
          style : 'border:1px solid #B5B8C8',
          readOnly : false,
          viewConfig : {
            forceFit : true
          }
        };
  //apply the config and defaults to 'this'
  Ext.apply(this, config, defaultConfig);

  //proxy for the store, to assign jsonData
  this.storeProxy = new Ext.data.HttpProxy({
        method : 'POST',
        url : Sonatype.config.servicePath + '/rolesAndPrivs',
        jsonData : {
          data : {
            onlySelected : this.onlySelected,
            noPrivileges : !this.usePrivileges,
            noRoles : false,
            selectedRoleIds : this.selectedRoleIds,
            selectedPrivilegeIds : this.selectedPrivilegeIds,
            userId : this.userId
          }
        }
      });

  //our remote resource, that will be supplying the paginated content
  this.store = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        totalProperty : 'totalCount',
        remoteSort : true,
        proxy : this.storeProxy,
        fields : [{
              name : 'id'
            }, {
              name : 'type'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'description'
            }, {
              name : 'external'
            }]
      });

  this.store.setDefaultSort('name', 'asc');

  //setup the columns in the grid
  this.columns = [];

  this.columns.push({
        id : 'name',
        width : 100,
        header : 'Name',
        dataIndex : 'name',
        sortable : true,
        renderer : function(value, metaData, record, rowIndex, colIndex, store) {
          switch (record.get('type'))
          {
            case 'role' :
              metaData.css = 'roleRow';
              break;
            case 'privilege' :
              metaData.css = 'privilegeRow';
              break;
          }
          return String.format('<img class="placeholder" src="{0}"/>', Ext.BLANK_IMAGE_URL) + (record.get('external') ? ('<b>' + value + '</b>') : value);
        }
      });

  this.addButton = new Ext.Toolbar.Button({
        text : 'Add',
        handler : this.addHandler,
        scope : this,
        disabled : this.readOnly
      });

  this.removeButton = new Ext.Toolbar.Button({
        text : 'Remove',
        handler : this.removeHandler,
        scope : this,
        disabled : true
      });

  this.tbar = ['<b>Role' + (this.usePrivileges ? '/Privilege' : '') + ' Management</b>', '->', '-', this.addButton, '-', this.removeButton];

  ns.RoleManager.superclass.constructor.call(this, {});

  this.getView().scrollOffset = 1;

  this.getSelectionModel().on('selectionchange', this.selectionChangeHandler, this);
};

Ext.extend(ns.RoleManager, Ext.grid.GridPanel, {
      selectionChangeHandler : function(selectionModel) {
        if (selectionModel.getCount() > 0 && !this.readOnly)
        {
          this.removeButton.enable();
        }
        else
        {
          this.removeButton.disable();
        }
      },
      addHandler : function() {
        this.roleSelectorWindow = new Ext.Window({
              modal : true,
              layout : 'fit',
              width : 800,
              items : [{
                    xtype : 'roleselector',
                    name : 'roleSelector',
                    height : 400,
                    width : 750,
                    usePrivileges : this.usePrivileges,
                    hiddenRoleIds : this.getHiddenRoleIds(),
                    hiddenPrivilegeIds : this.getSelectedPrivilegeIds(),
                    title : 'Add Roles' + (this.usePrivileges ? ' and Privileges' : '')
                  }],
              buttonAlign : 'center',
              buttons : [{
                    handler : this.addOk,
                    text : 'OK',
                    scope : this
                  }, {
                    handler : this.addCancel,
                    text : 'Cancel',
                    scope : this
                  }]
            });
        this.roleSelectorWindow.show();
      },
      removeHandler : function() {
        var
              i,
              records = this.getSelectionModel().getSelections();

        if (records && records.length > 0)
        {
          for (i = 0; i < records.length; i=i+1)
          {
            if (records[i].get('type') === 'role')
            {
              this.selectedRoleIds.remove(records[i].get('id'));
            }
            else if (records[i].get('type') === 'privilege')
            {
              this.selectedPrivilegeIds.remove(records[i].get('id'));
            }
          }
          this.reloadStore();
          this.validate();
        }
      },
      addOk : function() {
        var roleSelector = this.roleSelectorWindow.find('name', 'roleSelector')[0];
        this.addSelectedRoleId(roleSelector.getSelectedRoleIds());
        this.addSelectedPrivilegeId(roleSelector.getSelectedPrivilegeIds());
        this.roleSelectorWindow.close();
        this.reloadStore();
      },
      addCancel : function() {
        this.roleSelectorWindow.close();
      },
      getIdFromObject : function(object) {
        if (typeof(object) !== 'string' && object.source === 'default')
        {
          if (object.id)
          {
            return object.id;
          }
          else if (object.roleId)
          {
            return object.roleId;
          }
        }
        else if (typeof(object) === 'string')
        {
          return object;
        }
        return null;
      },
      append : function (roleIds, to) {
        var i, roleId;

        if (roleIds)
        {
          if (!Ext.isArray(roleIds))
          {
            roleIds = [roleIds];
          }

          for (i = 0; i < roleIds.length; i=i+1)
          {
            roleId = this.getIdFromObject(roleIds[i]);
            if (roleId)
            {
              to.push(roleId);
            }
          }
        }
      },
      setHiddenRoleIds : function(roleIds, reload) {
        this.hiddenRoleIds = [];
        
        this.append(roleIds, this.hiddenRoleIds);
        
        this.validate();

        if (reload)
        {
          this.reloadStore();
        }
      },
      getHiddenRoleIds : function() {
        // NEXUS-4371: merge selected and explicitly hidden roles
        var hidden = [];
        this.append(this.hiddenRoleIds, hidden);
        this.append(this.getSelectedRoleIds(), hidden);
        return hidden;
      },
      setSelectedRoleIds : function(roleIds, reload) {
        this.selectedRoleIds = [];
        
        this.append(roleIds, this.selectedRoleIds);

        this.noRolesOnStart = this.selectedRoleIds.length === 0;

        this.validate();

        if (reload)
        {
          this.reloadStore();
        }
      },
      setSelectedPrivilegeIds : function(privilegeIds, reload) {
        this.selectedPrivilegeIds = [];
        
        this.append(privilegeIds, this.selectedPrivilegeIds);

        this.validate();

        if (reload)
        {
          this.reloadStore();
        }
      },
      addSelectedRoleId : function(roleIds, reload) {
        var i, roleId;

        if (roleIds)
        {
          if (!Ext.isArray(roleIds))
          {
            roleIds = [roleIds];
          }

          for (i = 0; i < roleIds.length; i=i+1)
          {
            roleId = this.getIdFromObject(roleIds[i]);
            if (roleId)
            {
              this.selectedRoleIds.push(roleId);
            }
          }

          this.validate();

          if (reload)
          {
            this.reloadStore();
          }
        }
      },
      addSelectedPrivilegeId : function(privilegeIds, reload) {
        var i, privilegeId;

        if (privilegeIds)
        {
          if (!Ext.isArray(privilegeIds))
          {
            privilegeIds = [privilegeIds];
          }

          for (i = 0; i < privilegeIds.length; i=i+1)
          {
            privilegeId = this.getIdFromObject(privilegeIds[i]);
            if (privilegeId)
            {
              this.selectedPrivilegeIds.push(privilegeId);
            }
          }

          this.validate();

          if (reload)
          {
            this.reloadStore();
          }
        }
      },
      reloadStore : function() {
        this.storeProxy.conn.jsonData.data.selectedRoleIds = this.selectedRoleIds;
        this.storeProxy.conn.jsonData.data.selectedPrivilegeIds = this.selectedPrivilegeIds;
        this.storeProxy.conn.jsonData.data.userId = this.userId;
        this.store.load({
              params : {
                start : 0
              }
            });
      },
      getSelectedRoleIds : function() {
        return this.selectedRoleIds;
      },
      getSelectedPrivilegeIds : function() {
        return this.selectedPrivilegeIds;
      },
      getRoleNameFromId : function(id) {
        var rec = this.store.getById(id);

        if (rec)
        {
          return rec.get('name');
        }

        return id;
      },
      showErrorMarker: function(msg) {
        var elp = this.getEl();
        if (!this.errorEl) {
            this.errorEl = elp.createChild({
                cls: "x-form-invalid-msg"
            });
            this.errorEl.setWidth(elp.getWidth(true));
            this.errorEl.setStyle("border: 0 solid #fff");
        }
        this.errorEl.update(msg);
        elp.setStyle({
            "background-color": "#fee",
            border: "1px solid #dd7870"
        });
        Ext.form.Field.msgFx.normal.show(this.errorEl, this);
      },
      markInvalid: function(msg) {
       this.showErrorMarker(msg);
      },
      validate : function() {
        if (this.doValidation && !this.userId && this.selectedRoleIds.length === 0 && this.selectedPrivilegeIds.length === 0)
        {
          var msg = "You must select at least 1 role" + (this.usePrivileges ? " or privilege" : "");
          this.showErrorMarker(msg);
          return false;
        }

        this.clearValidation();
        return true;
      },
      clearValidation : function() {
        if (this.errorEl)
        {
          this.getEl().setStyle({
                'background-color' : '#FFFFFF',
                border : '1px solid #B5B8C8'
              });
          Ext.form.Field.msgFx.normal.hide(this.errorEl, this);
        }
      },
      setUserId : function(userId) {
        this.userId = userId;
        this.storeProxy.conn.jsonData.data.userId = this.userId;
      },
      disable : function() {
        this.readOnly = true;
        this.addButton.disable();
        this.removeButton.disable();
      },
      enable : function() {
        this.readOnly = false;
        this.addButton.enable();
        //dont blatantly enable, unless something is selected
        this.selectionChangeHandler(this.getSelectionModel());
      }
    });

Ext.reg('rolemanager', ns.RoleManager);

/**
 * A ns.RoleSelectorGrid is used to display the roles and privileges (optional) in a grid with checkboxes for simple selection
 * The grid supports pagination
 *  
 * usePrivileges - boolean flag, if true will show the privileges along with the roles 
 * selectedRoleIds - role ids that should show as selected 
 * selectedPrivilegeIds - privilege ids that should show as selected
 */
ns.RoleSelectorGrid = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          title : 'Manage Roles' + (this.usePrivileges ? ' and Privileges' : ''),
          trackMouseOver : false,
          loadMask : true,
          usePrivileges : true,
          hiddenRoleIds : [],
          hiddenPrivilegeIds : [],
          selectedRoleIds : [],
          selectedPrivilegeIds : []
        }, columns;
  //apply the config and defaults to 'this'
  Ext.apply(this, config, defaultConfig);

  //we have a predefined proxy here, so that we can post json data
  //and it is stored in this. so that we can easily change the json
  //params
  this.storeProxy = new Ext.data.HttpProxy({
        method : 'POST',
        url : Sonatype.config.servicePath + '/rolesAndPrivs',
        jsonData : {
          data : {
            name : null,
            noPrivileges : !this.usePrivileges,
            noRoles : false,
            onlySelected : false,
            selectedRoleIds : [],
            selectedPrivilegeIds : [],
            hiddenRoleIds : this.hiddenRoleIds,
            hiddenPrivilegeIds : this.hiddenPrivilegeIds
          }
        }
      });

  //our remote resource, that will be supplying the paginated content
  this.store = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        totalProperty : 'totalCount',
        remoteSort : true,
        proxy : this.storeProxy,
        fields : [{
              name : 'id'
            }, {
              name : 'type'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'description'
            }, {
              name : 'external'
            }],
        listeners : {
          load : {
            fn : function(store, records, options) {
              this.applySelection();
            },
            scope : this
          }
        }
      });

  this.store.setDefaultSort('name', 'asc');

  //add a checkbox selection model to the grid
  this.sm = new Ext.grid.CheckboxSelectionModel({
        listeners : {
          rowselect : {
            fn : function(sm, idx, rec) {
              var i, found = false;
              if (rec.get('type') === 'role')
              {
                for (i = 0; i < this.selectedRoleIds.length; i=i+1)
                {
                  if (this.selectedRoleIds[i] === rec.get('id'))
                  {
                    found = true;
                    break;
                  }
                }

                if (!found)
                {
                  this.selectedRoleIds.push(rec.get('id'));
                }
              }
              else
              {
                for (i = 0; i < this.selectedPrivilegeIds.length; i=i+1)
                {
                  if (this.selectedPrivilegeIds[i] === rec.get('id'))
                  {
                    found = true;
                    break;
                  }
                }

                if (!found)
                {
                  this.selectedPrivilegeIds.push(rec.get('id'));
                }
              }
            },
            scope : this
          },
          rowdeselect : {
            fn : function(sm, idx, rec) {
              var i, found = false;
              if (rec.get('type') === 'role')
              {
                if (rec.get('external'))
                {
                  sm.selectRecords([rec], true);
                }
                for (i = 0; i < this.selectedRoleIds.length; i=i+1)
                {
                  if (this.selectedRoleIds[i] === rec.get('id'))
                  {
                    this.selectedRoleIds.remove(this.selectedRoleIds[i]);
                    break;
                  }
                }
              }
              else
              {
                for (i = 0; i < this.selectedPrivilegeIds.length; i=i+1)
                {
                  if (this.selectedPrivilegeIds[i] === rec.get('id'))
                  {
                    this.selectedPrivilegeIds.remove(this.selectedPrivilegeIds[i]);
                    break;
                  }
                }
              }
            },
            scope : this
          }
        }
      });

  columns = [this.sm];

  columns.push({
        id : 'name',
        width : 300,
        header : 'Name',
        dataIndex : 'name',
        sortable : true,
        renderer : function(value, metaData, record, rowIndex, colIndex, store) {
          switch (record.get('type'))
          {
            case 'role' :
              metaData.css = 'roleRow';
              break;
            case 'privilege' :
              metaData.css = 'privilegeRow';
              break;
          }
          return String.format('<img class="placeholder" src="{0}"/>', Ext.BLANK_IMAGE_URL) + (record.get('external') ? ('<b>' + value + '</b>') : value);
        }
      }, {
        id : 'description',
        width : 400,
        header : 'Description',
        dataIndex : 'description',
        sortable : true
      });

  this.autoExpandColumn = 'description';

  //columns in the grid
  this.cm = new Ext.grid.ColumnModel({
        columns : columns
      });

  this.textFilter = new Ext.form.TextField({
        emptyText : 'Enter filter text...',
        listeners : {
          specialkey : {
            fn : this.filterTextSpecialkeyListener,
            scope : this
          }
        }
      });

  this.selectedFilter = new Ext.Button({
        text : 'Selected Only',
        enableToggle : true,
        tooltip : 'Add filter that will only show items selected',
        pressed : false
      });

  this.rolesFilter = new Ext.Button({
        text : 'Roles',
        enableToggle : true,
        tooltip : 'Add filter that will show roles',
        pressed : true
      });

  this.privilegesFilter = new Ext.Button({
        text : 'Privileges',
        enableToggle : true,
        tooltip : 'Add filter that will show privileges',
        pressed : true
      });

  // toolbar at top
  this.tbar = ['Filter: ', ' ', this.textFilter, '-', this.selectedFilter, '-'];

  if (this.usePrivileges)
  {
    this.tbar.push(this.rolesFilter, '-');
    this.tbar.push(this.privilegesFilter, '-');
  }

  this.tbar.push('->', '-', {
        text : 'Apply Filter',
        tooltip : 'Apply the filter parameter(s)',
        handler : this.applyFilter,
        scope : this
      }, '-', {
        text : 'Reset Filter',
        tooltip : 'Reset the filter to default selections',
        handler : this.resetFilter,
        scope : this
      }, '-');

  // paging bar on the bottom
  this.bbar = new Ext.PagingToolbar({
        pageSize : 25,
        store : this.store,
        displayInfo : true,
        displayMsg : 'Displaying roles' + (this.usePrivileges ? ' and privileges' : '') + ' {0} - {1} of {2}',
        emptyMsg : 'No roles' + (this.usePrivileges ? ' or privileges' : '') + ' to display'
      });

  //constructor call, adding the panel setup here
  ns.RoleSelectorGrid.superclass.constructor.call(this, {});
};

Ext.extend(ns.RoleSelectorGrid, Ext.grid.GridPanel, {
      filterTextSpecialkeyListener : function(f, e) {
        if (e.getKey() === e.ENTER)
        {
          this.applyFilter();
        }
      },
      //implement local onRender to load the first page of store
      onRender : function() {
        ns.RoleSelectorGrid.superclass.onRender.apply(this, arguments);
        this.store.load({
              params : {
                start : 0,
                limit : 25
              }
            });
      },
      applySelection : function() {
        //suspend the events here, we dont want to update our selected items in mem, only in the grid
        this.getSelectionModel().suspendEvents();
        this.getSelectionModel().clearSelections();
        this.getSelectionModel().resumeEvents();

        var i, records = this.store.getRange();

        for (i = 0; i < records.length; i=i+1)
        {
          if (records[i].get('type') === 'role' && this.selectedRoleIds.indexOf(records[i].get('id')) !== -1)
          {
            this.getSelectionModel().selectRecords([records[i]], true);
          }
          else if (records[i].get('type') === 'privilege' && this.selectedPrivilegeIds.indexOf(records[i].get('id')) !== -1)
          {
            this.getSelectionModel().selectRecords([records[i]], true);
          }
        }
      },
      setSelectedRoleIds : function(roleIds, reload) {
        var i;

        this.selectedRoleIds = [];

        if (!roleIds)
        {
          if (!Ext.isArray(roleIds))
          {
            roleIds = [roleIds];
          }

          for (i = 0; i < roleIds.length; i=i+1)
          {
            if (typeof(roleIds[i]) !== 'string')
            {
              if (roleIds[i].id)
              {
                this.selectedRoleIds.push(roleIds[i].id);
              }
              else if (roleIds[i].roleId)
              {
                this.selectedRoleIds.push(roleIds[i].roleId);
              }
            }
            else
            {
              this.selectedRoleIds.push(roleIds[i]);
            }
          }
        }

        if (reload)
        {
          this.applyFilter();
        }
        else
        {
          this.applySelection();
        }
      },
      setSelectedPrivilegeIds : function(privilegeIds, reload) {
        var i;

        this.selectedPrivilegeIds = [];

        if (privilegeIds)
        {
          if (!Ext.isArray(privilegeIds))
          {
            privilegeIds = [privilegeIds];
          }

          for (i = 0; i < privilegeIds.length; i=i+1)
          {
            if (typeof(privilegeIds[i]) !== 'string')
            {
              this.selectedPrivilegeIds.push(privilegeIds[i].id);
            }
            else
            {
              this.selectedPrivilegeIds.push(privilegeIds[i]);
            }
          }
        }

        if (reload)
        {
          this.applyFilter();
        }
        else
        {
          this.applySelection();
        }
      },
      getSelectedRoleIds : function() {
        return this.selectedRoleIds;
      },
      getSelectedPrivilegeIds : function() {
        return this.selectedPrivilegeIds;
      },
      applyFilter : function() {
        this.storeProxy.conn.jsonData.data.name = this.textFilter.getValue();

        if (this.selectedFilter.pressed)
        {
          this.storeProxy.conn.jsonData.data.selectedRoleIds = this.selectedRoleIds;
          this.storeProxy.conn.jsonData.data.selectedPrivilegeIds = this.selectedPrivilegeIds;
          this.storeProxy.conn.jsonData.data.onlySelected = true;
        }
        else
        {
          this.storeProxy.conn.jsonData.data.selectedRoleIds = [];
          this.storeProxy.conn.jsonData.data.selectedPrivilegeIds = [];
          this.storeProxy.conn.jsonData.data.onlySelected = false;
        }

        this.storeProxy.conn.jsonData.data.noRoles = !this.rolesFilter.pressed;

        this.storeProxy.conn.jsonData.data.noPrivileges = !this.privilegesFilter.pressed || !this.usePrivileges;

        this.store.load({
              params : {
                start : 0,
                limit : 25
              }
            });
      },
      resetFilter : function() {
        this.textFilter.setValue(null);
        this.selectedFilter.toggle(false);
        this.rolesFilter.toggle(true);
        this.privilegesFilter.toggle(true);

        this.storeProxy.conn.jsonData.data.name = null;
        this.storeProxy.conn.jsonData.data.onlySelected = false;
        this.storeProxy.conn.jsonData.data.selectedRoleIds = [];
        this.storeProxy.conn.jsonData.data.selectedPrivilegeIds = [];
        this.storeProxy.conn.jsonData.data.noRoles = false;
        this.storeProxy.conn.jsonData.data.noPrivileges = !this.usePrivileges;

        this.store.load({
              params : {
                start : 0,
                limit : 25
              }
            });
      }
    });

top.RoleManager = ns.RoleManager;
top.RoleSelectorGrid = ns.RoleSelectorGrid;

Ext.reg('roleselector', ns.RoleSelectorGrid);
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/error',
      [
        'Nexus/error/ErrorHandler'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

//
// Based on: http://www.sencha.com/forum/showthread.php?54602-Link-Button-or-Hyper-link&p=260201&viewfull=1#post260201
//

/**
 * A button which is rendered as a link.
 *
 * @class Nexus.ext.LinkButton
 * @extends Ext.Button
 * @namespace Nexus
 */
define('Nexus/ext/linkbutton', ['extjs', 'nexus'], function(Ext, Nexus) {
  Ext.namespace('Nexus.ext');

  var LinkButton = Ext.extend(Ext.Button, {
    template : new Ext.Template(
          '<table cellspacing="0" class="x-btn {3}"><tbody class="{4}">',
          '<tr>',
          '<td class="x-linkbtn-tl"><i> </i></td>',
          '<td class="x-linkbtn-tc"></td>',
          '<td class="x-linkbtn-tr"><i> </i></td>',
          '</tr>',
          '<tr>',
          '<td class="x-linkbtn-ml"><i> </i></td>',
          '<td class="x-linkbtn-mc">',
          '<em class="{4}" unselectable="on">',
          '<a href="{5}" style="display:block" target="{6}" class="x-linkbtn-text {2}" style="text-decoration: none; color: black; padding-left: 3px; padding-right: 3px;">{0}</a>',
          '</em>',
          '</td>',
          '<td class="x-linkbtn-mr"><i> </i></td>',
          '</tr>',
          '<tr>',
          '<td class="x-linkbtn-bl"><i> </i></td>',
          '<td class="x-linkbtn-bc"></td>',
          '<td class="x-linkbtn-br"><i> </i></td>',
          '</tr>',
          '</tbody></table>' ).compile(),

    buttonSelector : 'a:first',

    // FIXME: Consider removing the href/target muck exposed here, as this class is intended to behave just like a normal button, just look like a link.
    // FIXME: If the button handler really wants to change the location, it can/should do so programmatically.

    /**
     * @cfg String href
     * The URL to create a link for.
     */
    /**
     * @cfg String target
     * The target for the 'a' element.
     */
    /**
     * @cfg Object
     * A set of parameters which are always passed to the URL specified in the href
     */
    baseParams : {},

    params : {},

    getTemplateArgs : function() {
      return Ext.Button.prototype.getTemplateArgs.apply(this).concat([this.getHref(), this.target]);
    },

    onClick : function(e) {
      if (e.button !== 0) {
        return;
      }
      // if disabled, stop the event
      if (this.disabled) {
        e.stopEvent();
      }
      // else if there is a handler, stop the event and call the handler
      else if (this.handler) {
        e.stopEvent();
        this.handler.call(this.scope || this, this, e);
      }
      // else let the event propagate
    },

    // private
    getHref : function() {
      var
            result = this.href,
            p = Ext.urlEncode(Ext.apply(Ext.apply({}, this.baseParams), this.params));

      if (p.length) {
        result += ((this.href.indexOf('?') === -1) ? '?' : '&') + p;
      }
      return result;
    },

    /**
     * Sets the href of the link dynamically according to the params passed, and any {@link #baseParams} configured.
     * @param {Object} p Parameters to use in the href URL.
     */
    setParams : function(p) {
      this.params = p;
      this.el.child(this.buttonSelector, true).href = this.getHref();
    }
  });

  Nexus.ext.LinkButton = LinkButton;
  Ext.ComponentMgr.registerType('link-button', Nexus.ext.LinkButton);
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * A search box.
 *
 * @since 2.7
 */
NX.define('Nexus.ext.SearchBox', {
  extend: 'Ext.Container',

  requires: [ 'Nexus.util.Icons' ],

  xtype: 'nx-search-box',

  /**
   * @cfg {Number} delay between keystrokes and actual filtering (defaults to 200).
   */
  searchDelay: 200,

  /**
   * @override
   */
  initComponent: function () {
    var self = this,
        icons = Nexus.util.Icons;

    self.searchField = NX.create('Ext.form.TextField', {
      enableKeyEvents: true,
      cls: 'nx-searchbox',
      style: {
        paddingLeft: '22px',
        paddingRight: '22px'
      },
      listeners: {
        keyup: {
          fn: function (field, e) {
            clearTimeout(self.searchTimeout);
            if (e.keyCode === 27) {
              self.clearSearch();
            }
            else {
              self.searchTimeout = function () {
                if (self.searchField.getValue() && self.searchField.getValue().length > 0) {
                  self.fireEvent('search', self.searchField.getValue());
                }
                else {
                  self.fireEvent('searchcleared');
                }
              }.defer(self.searchDelay, self);
            }
          },
          scope: self
        }
      }
    });

    self.clearButton = NX.create('Ext.Button', {
      xtype: 'button',
      iconCls: icons.get('glyph_circle_remove').cls,
      scope: self,
      handleMouseEvents: false,
      hidden: true,
      handler: self.clearSearch,
      style: {
        position: 'absolute',
        top: '3px',
        right: '3px',
        opacity: 0.5
      }
    });

    Ext.apply(self, {
      layout: 'fit',
      items: [
        self.searchField,
        self.clearButton,
        {
          xtype: 'button',
          // FIXME: Pick one of these icons
          //iconCls: icons.get('glyph_filter').cls,
          iconCls: icons.get('glyph_search').cls,
          disabled: true,
          scope: self,
          style: {
            position: 'absolute',
            top: '3px',
            right: (self.width - 20) + 'px',
            opacity: 0.25
          }
        }
      ]
    });

    self.addEvents(

        /**
         * Fires when a search value is available.
         * @event search
         * @param {String} search value
         */
        'search',

        /**
         * Fires when search has been cleared (no value available).
         * @event searchcleared
         */
        'searchcleared'
    );

    Nexus.ext.SearchBox.superclass.initComponent.call(self, arguments);

    self.on('search', function (value) {
      if (value && value.length > 0) {
        self.clearButton.show();
      }
    });

    self.on('searchcleared', function () {
      self.clearButton.hide();
    });
  },

  /**
   * Clears the search.
   */
  clearSearch: function () {
    var self = this;

    self.searchField.setValue(undefined);
    self.fireEvent('searchcleared');
  },

  /**
   * Sets teh value of search box and performs the search.
   */
  search: function (value) {
    var self = this;

    self.searchField.setValue(value);
    self.fireEvent('search');
  },

  /**
   * Returns the current search value or undefined if search is undefined or empty.
   */
  getSearchValue: function () {
    var self = this;
    if (self.searchField.getValue() && self.searchField.getValue().length > 0) {
      return self.searchField.getValue();
    }
    return undefined;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('Nexus/ext/FormPanel',['extjs', 'sonatype', 'nexus', 'Nexus/messagebox'], function(Ext, Sonatype, Nexus, mbox) {

Ext.namespace('Nexus.ext');

/*
 * Generic object editor (intended to be subclassed). When used with
 * Sonatype.panels.GridViewer, instanced of this editor panel will never be
 * reused. When the form is submitted, all child panels related to this grid
 * record are re-created, so the editor does not have to worry about altering
 * its state after submit (as opposed to having to disable certain fields after
 * the new record is saved, which we had to do previously). Config options:
 * cancelButton: if set to "true", the form will display a "Cancel" button, so
 * the invoker can subscribe to a "cancel" event and do the necessary cleanup
 * (e.g. close the panel). By default, the form will display a "Reset" button
 * instead, which reloads the form when clicked. Note a special dataModifier of
 * 'rootData' will work with entire contents of json response rather than on a
 * field by field basis. Using this modifier will then render other modifiers
 * useless dataModifiers: { // data modifiers on form submit/load load: { attr1:
 * func1, attr2: func2 }, save: { ... } } dataStores: an array of data stores
 * this editor depends on. It will make sure all stores are loaded before the
 * form load request is sent. The stores should be configured with auto load
 * off. listeners: { // custom events offered by the editor panel cancel: { fn:
 * function( panel ) { // do cleanup, remove the form from the container, //
 * delete the temporary grid record, etc. }, scope: this }, load: { fn:
 * function( form, action, receivedData ) { // do extra work for data load if
 * needed }, scope: this }, submit: { fn: function( form, action, receivedData ) { //
 * update the grid record and do other stuff if needed var rec = this.payload;
 * rec.beginEdit(); rec.set( 'attr1', receivedData.attr1 ); rec.set( 'attr2',
 * receivedData.attr2 ); rec.commit(); rec.endEdit(); }, scope: this } }
 * payload: the grid record being edited referenceData: a reference data object
 * that's used as a template on form submit uri: base URL for JSON requests. It
 * will be used for POST requests when creating a new object, or for PUT (with
 * payload.id appended) if the record does not a resourceURI attribute
 */
Nexus.ext.FormPanel = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          region : 'center',
          width : '100%',
          height : '100%',
          autoScroll : true,
          border : false,
          frame : true,
          collapsible : false,
          collapsed : false,
          labelWidth : 200,
          layoutConfig : {
            labelSeparator : ''
          }
        };

  Ext.apply(this, config, defaultConfig);

  this.checkPayload();
  if (this.isNew && (this.cancelButton === null || this.cancelButton === undefined))
  {
    this.cancelButton = true;
  }

  Nexus.ext.FormPanel.superclass.constructor.call(this, {
    buttons : (config.readOnly || this.readOnly) ? [] : [{
      text : 'Save',
      handler : this.saveHandler,
      scope : this
    }, {
      handler : this.cancelButton ? this.cancelHandler : this.resetHandler,
      scope : this,
      text : this.cancelButton ? 'Cancel' : 'Reset'
    }]
  });

  this.on('afterlayout', this.initData, this, {
    single : true
  });
  this.on('afterlayout', this.registerRequiredQuicktips, this, {
    single : true
  });
  this.form.on('actioncomplete', this.actionCompleteHandler, this);
  this.form.on('actionfailed', this.actionFailedHandler, this);

  this.addEvents({
    cancel : true,
    load : true,
    submit : true
  });
};

Ext.extend(Nexus.ext.FormPanel, Ext.FormPanel, {
  convertDataValue : function(value, store, idProperty, nameProperty) {
    if (value)
    {
      var rec = store.getAt(store.find(idProperty, value));
      if (rec)
      {
        return rec.data[nameProperty];
      }
    }
    return '';
  },
  checkPayload : function() {
    this.isNew = false;
    if (this.payload)
    {
      if (this.payload.id && this.payload.id.substring(0, 4) === 'new_')
      {
        this.isNew = true;
      }
    }
  },

  checkStores : function() {
    var i, store;
    if (this.dataStores)
    {
      for (i = 0; i < this.dataStores.length; i=i+1)
      {
        store = this.dataStores[i];
        if (!store.lastOptions)
        {
          return false;
        }
      }
    }
    return true;
  },

  dataStoreLoadHandler : function(store, records, options) {
    if (this.checkStores())
    {
      this.loadData();
    }
  },

  registerRequiredQuicktips : function(formPanel, fLayout) {
    // register required field quicktip, but have to wait for elements to
    // show up in DOM
    var temp = function() {
      var els = Ext.select('.required-field .x-form-item-label, .required-field .x-panel-header-text', this.getEl());
      els.each(function(el, els, i) {
        Ext.QuickTips.register({
          target : el,
          cls : 'required-field',
          title : '',
          text : 'Required Field',
          enabled : true
        });
      });
    }.defer(300, formPanel);
  },

  cancelHandler : function(button, event) {
    this.fireEvent('cancel', this);
  },

  resetHandler : function(button, event) {
    this.loadData();
  },

  initData : function() {
    var i, store;

    if (this.dataStores)
    {
      for (i = 0; i < this.dataStores.length; i=i+1)
      {
        store = this.dataStores[i];
        store.on('load', this.dataStoreLoadHandler, this);
        if (store.autoLoad !== true)
        {
          store.load();
        }
      }
    }
    else
    {
      this.loadData();
    }
  },

  loadData : function() {
    if (this.isNew)
    {
      this.form.reset();
    }
    else
    {
      this.form.doAction('sonatypeLoad', {
        url : this.getActionURL(),
        method : 'GET',
        fpanel : this,
        dataModifiers : this.dataModifiers ? this.dataModifiers.load : {},
        scope : this
      });
    }
  },

  isValid : function() {
    return this.form.isValid();
  },

  saveHandler : function(button, event) {
    if (this.isValid())
    {
      this.form.doAction('sonatypeSubmit', {
        method : this.getSaveMethod(),
        url : this.getActionURL(),
        waitMsg : this.isNew ? 'Creating a new record...' : 'Updating records...',
        fpanel : this,
        validationModifiers : this.validationModifiers,
        dataModifiers : this.dataModifiers ? this.dataModifiers.submit : {},
        serviceDataObj : this.referenceData,
        isNew : this.isNew
        // extra option to send to callback, instead of conditioning on
        // method
      });
    }
  },

  actionFailedHandler : function(form, action) {
    if (action.failureType === Ext.form.Action.CLIENT_INVALID)
    {
      mbox.alert('Missing or Invalid Fields', 'Please change the missing or invalid fields.').setIcon(mbox.WARNING);
    }
    else if (action.failureType === Ext.form.Action.CONNECT_FAILURE || action.response)
    {
      Sonatype.utils.connectionError(action.response, 'There is an error communicating with the server.');
    }
    else if (action.failureType === Ext.form.Action.LOAD_FAILURE)
    {
      mbox.alert('Load Failure', 'The data failed to load from the server.').setIcon(Nexus.MessageBox.ERROR);
    }
  },

  // (Ext.form.BasicForm, Ext.form.Action)
  actionCompleteHandler : function(form, action) {
    var receivedData = action.handleResponse(action.response).data, i, r, rec, store;
    if (!receivedData)
    {
      receivedData = {};
    }

    if (action.type === 'sonatypeSubmit')
    {
      this.fireEvent('submit', form, action, receivedData);

      if (this.isNew && this.payload.autoCreateNewRecord)
      {
        store = this.payload.store;
        store.remove(this.payload);

        if (Ext.isArray(receivedData))
        {
          for (i = 0; i < receivedData.length; i=i+1)
          {
            r = receivedData[i];
            rec = new store.reader.recordType(r, r.resourceURI);
            this.addSorted(store, rec);
          }
        }
        else
        {
          rec = new store.reader.recordType(receivedData, receivedData.resourceURI);
          rec.autoCreateNewRecord = true;
          this.addSorted(store, rec);
        }
      }
      this.isNew = false;
      this.payload.autoCreateNewRecord = false;
    }
    else if (action.type === 'sonatypeLoad')
    {
      this.fireEvent('load', form, action, receivedData);
    }
  },

  addSorted : function(store, rec) {
    store.addSorted(rec);
  },

  getActionURL : function() {
    var uri;
    if (this.isNew) {
      uri = this.uri;
    } else if (this.payload.data && this.payload.data.resourceURI) {
      // if resourceURI is supplied, return it
      uri = this.payload.data.resourceURI;
    } else {
      // otherwise construct a uri
      uri = this.uri + '/' + this.payload.id;
    }

    return uri;
  },

  getSaveMethod : function() {
    return this.isNew ? 'POST' : 'PUT';
  },

  optionalFieldsetExpandHandler : function(panel) {
    panel.items.each(function(item, i, len) {
      if (item.rendered && item.getEl().up('div.required-field', 3))
      {
        item.allowBlank = false;
      }
      else if (item.isXType('fieldset', true))
      {
        this.optionalFieldsetExpandHandler(item);
      }
    }, this);
  },

  optionalFieldsetCollapseHandler : function(panel) {
    panel.items.each(function(item, i, len) {
      if (item.rendered && item.getEl().up('div.required-field', 3))
      {
        item.allowBlank = true;
      }
      else if (item.isXType('fieldset', true))
      {
        this.optionalFieldsetCollapseHandler(item);
      }
    }, this);
  }
});

Sonatype.ext.FormPanel = Nexus.ext.FormPanel;

return Nexus.ext.FormPanel;

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('Nexus/ext/TextEntryList',['extjs', 'nexus', 'Nexus/config'], function(Ext, Nexus, Config){
Ext.namespace('Nexus.ext');

/*
 * A widget that will allow custom strings to be entered (and removed) into a
 * listbox.
 */
Nexus.ext.TextEntryList = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          layout : 'form'
        };

  Ext.apply(this, config, defaultConfig);

  this.textEntryField = new Ext.form.Field({
        xtype : 'textfield',
        helpText : config.entryHelpText,
        name : 'entryName',
        width : 300,
        helpMarker : true
      });

  this.addEntryButton = new Ext.Button({
        xtype : 'button',
        text : 'Add',
        style : 'padding-left: 7px',
        minWidth : 100,
        id : 'button-add',
        handler : this.addNewEntry,
        scope : this
      });

  this.removeEntryButton = new Ext.Button({
        xtype : 'button',
        text : 'Remove',
        style : 'padding-left: 6px',
        minWidth : 100,
        id : 'button-remove',
        handler : this.removeEntry,
        scope : this
      });

  this.removeAllEntriesButton = new Ext.Button({
        xtype : 'button',
        text : 'Remove All',
        style : 'padding-left: 6px; margin-top: 5px',
        minWidth : 100,
        id : 'button-remove-all',
        handler : this.removeAllEntries,
        scope : this
      });

  this.entryList = new Ext.tree.TreePanel({
        xtype : 'treepanel',
        id : 'entry-list', // note: unique ID is assinged
        // before instantiation
        name : 'entry-list',
        title : config.listLabel,
        // cls: 'required-field',
        border : true, // note: this seem to have no effect w/in form panel
        bodyBorder : true, // note: this seem to have no effect w/in form
        // panel
        // note: this style matches the expected behavior
        bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
        style : 'padding: 0 20px 0 0',
        width : 320,
        height : 150,
        animate : true,
        lines : false,
        autoScroll : true,
        containerScroll : true,
        // @note: root node must be instantiated uniquely for each instance of
        // treepanel
        // @ext: can TreeNode be registerd as a component with an xtype so this
        // new root node
        // may be instantiated uniquely for each form panel that uses this
        // config?
        rootVisible : false,
        enableDD : false,
        root : new Ext.tree.TreeNode({
              text : 'root'
            })

      });

  Nexus.ext.TextEntryList.superclass.constructor.call(this, {
        autoScroll : true,
        border : false,
        collapsible : false,
        collapsed : false,
        labelWidth : 175,
        layoutConfig : {
          labelSeparator : ''
        },
        items : [{
              xtype : 'panel',
              fieldLabel : config.entryLabel,
              layout : 'column',
              items : [
                {
                    xtype : 'panel',
                    width : 320,
                    items : [this.textEntryField]
                  }, {
                    xtype : 'panel',
                    width : 120,
                    items : [this.addEntryButton]
                  }]
            }, {
              xtype : 'panel',
              fieldLabel : config.listLabel,
              layout : 'column',
              autoHeight : true,
              items : [this.entryList, {
                    xtype : 'panel',
                    width : 120,
                    items : [this.removeEntryButton, this.removeAllEntriesButton]
                  }]
            }]
      });

};

Ext.extend(Nexus.ext.TextEntryList, Ext.Panel, {

      addEntryNode : function(treePanel, entry) {
        var id = Ext.id();

        treePanel.root.appendChild(new Ext.tree.TreeNode({
                  id : id,
                  text : entry,
                  payload : entry,
                  allowChildren : false,
                  draggable : false,
                  leaf : true,
                  nodeType : 'entry',
                  icon : Config.extPath + '/resources/images/default/tree/leaf.gif'
                }));
      },

      addNewEntry : function() {
        var nodes, i, entry = this.textEntryField.getRawValue();


        if (entry)
        {
          nodes = this.entryList.root.childNodes;
          for (i = 0; i < nodes.length; i=i+1)
          {
            if (entry === nodes[i].attributes.payload)
            {
              this.textEntryField.markInvalid('This entry already exists');
              return;
            }
          }

          this.addEntryNode(this.entryList, entry);
          this.textEntryField.setRawValue('');
        }
      },

      removeEntry : function() {

        var selectedNode = this.entryList.getSelectionModel().getSelectedNode();
        if (selectedNode)
        {
          this.entryList.root.removeChild(selectedNode);
        }
      },

      removeAllEntries : function() {
        var treeRoot = this.entryList.root;

        while (treeRoot.lastChild)
        {
          treeRoot.removeChild(treeRoot.lastChild);
        }
      },

      setEntries : function(stringArray) {
        var entry, i;
        // first clear all
        this.removeAllEntries();

        for (i = 0; i < stringArray.length; i=i+1)
        {
          entry = stringArray[i];
          this.addEntryNode(this.entryList, entry);
        }

        return stringArray; // return stringArray, even if empty to comply with
                            // sonatypeLoad data modifier requirement
      },

      getEntries : function() {
        var
              i,
              outputArr = [],
              nodes = this.entryList.root.childNodes;

        for (i = 0; i < nodes.length; i=i+1)
        {
          outputArr[i] = nodes[i].attributes.payload;
        }

        return outputArr;
      }

    });

Ext.reg('textentrylist', Nexus.ext.TextEntryList);
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * A grid filter box.
 *
 * @since 2.7
 */
NX.define('Nexus.ext.GridFilterBox', {
  extend: 'Nexus.ext.StoreFilterBox',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  xtype: 'nx-grid-filter-box',

  /**
   * @cfg {Ext.grid.GridPanel} grid that should filtered
   */
  filteredGrid: undefined,

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    Ext.apply(self, {
      filteredStore: self.findGridStore(self.filteredGrid),
      filteredFields: self.findGridColumns(self.filteredGrid)
    });

    Nexus.ext.GridFilterBox.superclass.initComponent.call(self, arguments);

    self.on('render', function () {
      self.logDebug('Binding to grid ' + self.filteredGrid);
      self.filteredGrid.on('reconfigure', self.onGridReconfigured, self);
    });

    self.on('destroy', function () {
      self.logDebug('Unbinding from grid ' + self.filteredGrid);
      self.filteredGrid.removeListener('reconfigure', self.onGridReconfigured, self);
    });
    self.on('beforeFiltering', function () {
      if (self.filteredGrid.view.emptyTextBackup) {
        self.filteredGrid.view.emptyText = self.filteredGrid.view.emptyTextBackup;
      }
      if (self.filteredGrid.view.emptyTextWhileFiltering && self.filteredStore.getCount() > 0) {
        if (!self.filteredGrid.view.emptyTextBackup) {
          self.filteredGrid.view.emptyTextBackup = self.filteredGrid.view.emptyText;
        }
        self.filteredGrid.view.emptyText = self.filteredGrid.view.emptyTextWhileFiltering.replaceAll(
            '{criteria}', self.getSearchValue()
        );
      }
    });
    self.on('searchcleared', function () {
      if (self.filteredGrid.view.emptyTextBackup) {
        self.filteredGrid.view.emptyText = self.filteredGrid.view.emptyTextBackup;
      }
    });
  },

  /**
   * @private
   * Handles reconfiguration of grid.
   * @param grid that was reconfigured
   * @param store new store
   * @param columnModel new column model
   */
  onGridReconfigured: function (grid, store, columnModel) {
    var self = this;
    self.logDebug('Grid ' + self.filteredGrid + ' reconfigured, binding to new store');
    self.reconfigureStore(store, self.extractColumnsWithDataIndexFromModel(columnModel));
  },

  /**
   * @private
   * Finds the data store of a grid.
   * @param grid to find store of
   * @returns {Ext.data.Store} store or undefined if store could not be found
   */
  findGridStore: function (grid) {
    return grid.getStore() || grid.gridStore || grid.store || grid.ds;
  },

  /**
   * @private
   * Find the columns of a grid.
   * @param grid to find columns of
   * @returns {Array} columns or undefined if columns could not be determined
   */
  findGridColumns: function (grid) {
    var self = this,
        colModel = grid.getColumnModel() || grid.cm || grid.colModel,
        columns;

    if (colModel) {
      columns = self.extractColumnsWithDataIndexFromModel(colModel);
    }
    if (!columns) {
      columns = self.extractColumnsWithDataIndex(grid.columns);
    }
    return columns;
  },

  /**
   * @private
   * Returns the dataIndex property of all grid columns.
   * @returns {Array} of fields names
   */
  extractColumnsWithDataIndexFromModel: function (columnModel) {
    var self = this,
        columns;

    if (columnModel) {
      columns = columnModel.getColumnsBy(function () {
        return true;
      });
      return self.extractColumnsWithDataIndex(columns);
    }
  },

  /**
   * @private
   * Returns the dataIndex property of all grid columns.
   * @returns {Array} of fields names
   */
  extractColumnsWithDataIndex: function (columns) {
    var filterFieldNames = [];

    if (columns) {
      Ext.each(columns, function (column) {
        if (column.dataIndex) {
          filterFieldNames.push(column.dataIndex);
        }
      });
    }

    if (filterFieldNames.length > 0) {
      return filterFieldNames;
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Target Edit/Create panel layout and controller
 */
/*global define*/
define('Nexus/ext/twinpanelcontroller', ['extjs', 'nexus', 'Nexus/config'], function(Ext, Nexus, Config) {

Ext.namespace('Nexus.ext');

Nexus.ext.TwinPanelController = function(cfg) {
  Ext.apply(this, cfg || {});

  this.addOneButton = new Ext.Button({
        xtype : 'button',
        handler : this.addOne,
        scope : this,
        tooltip : 'Add',
        icon : Config.extPath + '/resources/images/default/grid/page-prev.gif',
        cls : 'x-btn-icon'
      });

  this.addAllButton = new Ext.Button({
        xtype : 'button',
        handler : this.addAll,
        scope : this,
        tooltip : 'Add All',
        icon : Config.extPath + '/resources/images/default/grid/page-first.gif',
        cls : 'x-btn-icon'
      });

  this.removeOneButton = new Ext.Button({
        xtype : 'button',
        handler : this.removeOne,
        scope : this,
        tooltip : 'Remove',
        icon : Config.extPath + '/resources/images/default/grid/page-next.gif',
        cls : 'x-btn-icon'
      });

  this.removeAllButton = new Ext.Button({
        xtype : 'button',
        handler : this.removeAll,
        scope : this,
        tooltip : 'Remove All',
        icon : Config.extPath + '/resources/images/default/grid/page-last.gif',
        cls : 'x-btn-icon'
      });

  Nexus.ext.TwinPanelController.superclass.constructor.call(this, {
        layout : 'table',
        style : 'padding-top: ' + (this.halfSize ? 40 : 100) + 'px; padding-right: 10px; padding-left: 10px',
        width : 45,
        defaults : {
          style : 'margin-bottom: 3px'
        },
        layoutConfig : {
          columns : 1
        },
        items : [this.addOneButton, this.addAllButton, this.removeOneButton, this.removeAllButton]
      });

};

Ext.extend(Nexus.ext.TwinPanelController, Ext.Panel, {
      disable : function() {
        this.addOneButton.disable();
        this.addAllButton.disable();
        this.removeOneButton.disable();
        this.removeAllButton.disable();
      },
      enable : function() {
        this.addOneButton.enable();
        this.addAllButton.enable();
        this.removeOneButton.enable();
        this.removeAllButton.enable();
      },
      addOne : function() {
        this.moveItems(2, 0, false);
      },

      addAll : function() {
        this.moveItems(2, 0, true);
      },

      removeOne : function() {
        this.moveItems(0, 2, false);
      },

      removeAll : function() {
        this.moveItems(0, 2, true);
      },

      moveItems : function(fromIndex, toIndex, moveAll) {
        var
              i, node, selectedNodes,
              fromPanel = this.ownerCt.getComponent(fromIndex),
              toPanel = this.ownerCt.getComponent(toIndex),
              fromRoot = fromPanel.root,
              toRoot = toPanel.root,

              dragZone = fromPanel.dragZone,
              dropZone = toPanel.dropZone,
              fn = toPanel.dropConfig.onContainerOver.createDelegate(dropZone, [dragZone, null], 0),
              checkIfDragAllowed = function(node) {
                var dropTarget = fn({
                  node : node
                });
                return (!node.disabled) && dropTarget === dropZone.dropAllowed;
              };

        if (fromPanel && toPanel)
        {
          if (toPanel.sorter && toPanel.sorter.disableSort)
          {
            toPanel.sorter.disableSort(toPanel);
          }
          if (fromPanel.sorter && fromPanel.sorter.disableSort)
          {
            fromPanel.sorter.disableSort(fromPanel);
          }
          if (moveAll)
          {
            while (fromRoot.firstChild) {
              toRoot.appendChild(fromRoot.firstChild);
            }
          }
          else
          {
            selectedNodes = fromPanel.getSelectionModel().getSelectedNodes();
            if (selectedNodes)
            {
              for (i = 0; i < selectedNodes.length; i=i+1)
              {
                node = selectedNodes[i];
                if (checkIfDragAllowed(node))
                {
                  toRoot.appendChild(node);
                }
              }
            }
          }
          if (toPanel.sorter && toPanel.sorter.enableSort)
          {
            toPanel.sorter.enableSort(toPanel);
          }
          if (fromPanel.sorter && fromPanel.sorter.enableSort)
          {
            fromPanel.sorter.enableSort(fromPanel);
          }
        }
      }
    });

Ext.reg('twinpanelcontroller', Nexus.ext.TwinPanelController);

Nexus.ext.TwinPanelChooser = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          doubleWide : false,
          displayField : 'name',
          validateLeftItems : false,
          validateLeftItemsText : 'Invalid items selected.',
          nodeIcon : Config.extPath + '/resources/images/default/tree/leaf.gif'
        };
  Ext.apply(this, config, defaultConfig);

  Nexus.ext.TwinPanelChooser.superclass.constructor.call(this, {
        layout : 'column',
        autoHeight : true,
        style : 'padding: 10px 0 10px 0',
        listeners : {
          beforedestroy : {
            fn : function() {
              if (this.store)
              {
                this.loadStore();
                this.store.un('load', this.loadStore, this);
              }
            },
            scope : this
          }
        },

        items : [{
              xtype : 'multiselecttreepanel',
              name : 'targettree',
              // id: '_staging-profiles-target-groups-tree', //note: unique ID
              // is assinged before instantiation
              title : this.titleLeft,
              cls : this.required ? 'required-field' : null,
              border : true, // note: this seem to have no effect w/in form panel
              bodyBorder : true, // note: this seem to have no effect w/in form panel
              // note: this style matches the expected behavior
              bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
              width : this.doubleWide ? 400 : 225,
              height : this.halfSize ? 150 : 300,
              animate : true,
              lines : false,
              autoScroll : true,
              containerScroll : true,
              // @note: root node must be instantiated uniquely for each instance of treepanel
              // @ext: can TreeNode be registerd as a component with an xtype so this new root
              // node may be instantiated uniquely for each form panel that uses this config?
              rootVisible : false,
              root : new Ext.tree.TreeNode({
                    text : 'root'
                  }),
              enableDD : true,
              ddScroll : true,
              dropConfig : {
                allowContainerDrop : true,
                onContainerDrop : function(source, e, data) {
                  var i;

                  if (data.nodes)
                  {
                    for (i = 0; i < data.nodes.length; i=i+1)
                    {
                      this.tree.root.appendChild(data.nodes[i]);
                    }
                  }
                  return true;
                },
                onContainerOver : function(source, e, data) {
                  return this.dropAllowed;
                },
                // passign padding to make whole treePanel the drop zone. This is dependent
                // on a sonatype fix in the Ext.dd.DropTarget class. This is necessary
                // because treepanel.dropZone.setPadding is never available in
                // time to be useful.
                padding : [0, 0, (this.halfSize ? 124 : 274), 0]
              },
              // added Field values to simulate form field validation
              invalidText : 'Select one or more items',
              validate : function() {
                return (this.root.childNodes.length > 0);
              },
              invalid : false,
              listeners : {
                'append' : {
                  fn : function(tree, parentNode, insertedNode, i) {
                    this.clearInvalid();
                  },
                  scope : this
                },
                'remove' : {
                  fn : function(tree, parentNode, removedNode) {
                    if (tree.root.childNodes.length < 1 && this.required)
                    {
                      this.markTreeInvalid(tree, null);
                    }
                    else
                    {
                      this.clearInvalid();
                    }
                  },
                  scope : this
                }
              }
            }, {
              xtype : 'twinpanelcontroller',
              name : 'twinpanel',
              halfSize : this.halfSize
            }, {
              xtype : 'multiselecttreepanel',
              name : 'sourcetree',
              // id: id + '_staging-profiles-available-groups-tree', //note:
              // unique ID is assinged before instantiation
              title : this.titleRight,
              border : true, // note: this seem to have no effect w/in form
                              // panel
              bodyBorder : true, // note: this seem to have no effect w/in form
                                  // panel
              // note: this style matches the expected behavior
              bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
              width : this.doubleWide ? 400 : 225,
              height : (this.halfSize ? 150 : 300) + (Ext.isGecko ? 15 : 0),
              animate : true,
              lines : false,
              autoScroll : true,
              containerScroll : true,
              // @note: root node must be instantiated uniquely for each
              // instance of treepanel
              // @ext: can TreeNode be registerd as a component with an xtype so
              // this new root node
              // may be instantiated uniquely for each form panel that uses this
              // config?
              rootVisible : false,
              root : new Ext.tree.TreeNode({
                    text : 'root'
                  }),
              enableDD : true,
              ddScroll : true,
              dropConfig : {
                allowContainerDrop : true,
                onContainerDrop : function(source, e, data) {
                  var i;

                  if (data.nodes)
                  {
                    for (i = 0; i < data.nodes.length; i=i+1)
                    {
                      this.tree.root.appendChild(data.nodes[i]);
                    }
                  }
                  return true;
                },
                onContainerOver : function(source, e, data) {
                  return this.dropAllowed;
                },
                // passign padding to make whole treePanel the drop zone. This
                // is dependent
                // on a sonatype fix in the Ext.dd.DropTarget class. This is
                // necessary
                // because treepanel.dropZone.setPadding is never available in
                // time to be useful.
                padding : [0, 0, (this.halfSize ? 124 : 274), 0]
              }
            }]
      });

  if (this.store)
  {
    this.loadStore();
    this.store.on('load', this.loadStore, this);
  }
};

Ext.extend(Nexus.ext.TwinPanelChooser, Ext.Panel, {
      disable : function() {
        this.find('name', 'twinpanel')[0].disable();
        this.find('name', 'sourcetree')[0].dragZone.lock();
        this.find('name', 'targettree')[0].dragZone.lock();
      },
      enable : function() {
        this.find('name', 'twinpanel')[0].enable();
        this.find('name', 'sourcetree')[0].dragZone.unlock();
        this.find('name', 'targettree')[0].dragZone.unlock();
      },
      createNode : function(root, rec) {
        root.appendChild(new Ext.tree.TreeNode({
              id : rec.id,
              text : rec.data[this.displayField],
              payload : rec,
              allowChildren : false,
              draggable : true,
              leaf : true,
              disabled : rec.data.readOnly,
              icon : this.nodeIcon
            }));
      },

      loadStore : function() {
        if (this.store)
        {
          var root = this.getComponent(2).root;
          while (root.lastChild)
          {
            root.removeChild(root.lastChild);
          }
          this.store.each(function(rec) {
                this.createNode(root, rec);
              }, this);
        }
      },

      clearInvalid : function() {
        var tree = this.getComponent(0);
        if (tree.invalid)
        {
          // remove error messaging
          tree.getEl().child('.x-panel-body').setStyle({
                'background-color' : '#FFFFFF',
                border : '1px solid #B5B8C8'
              });
          Ext.form.Field.msgFx.normal.hide(tree.errorEl, tree);
        }
      },

      markTreeInvalid : function(tree, errortext) {
        if (!tree)
        {
          tree = this.getComponent(0);
        }
        var
              elp = tree.getEl(),
              oldErrorText = tree.invalidText;

        if (elp)
        {
          if (!tree.errorEl)
          {
            tree.errorEl = elp.createChild({
                  cls : 'x-form-invalid-msg'
                });
            tree.errorEl.setWidth(elp.getWidth(true)); // note removed -20 like on form fields
          }
          tree.invalid = true;
          if (errortext)
          {
            tree.invalidText = errortext;
          }
          tree.errorEl.update(tree.invalidText);
          tree.invalidText = oldErrorText;
          elp.child('.x-panel-body').setStyle({
                'background-color' : '#fee',
                border : '1px solid #dd7870'
              });
          Ext.form.Field.msgFx.normal.show(tree.errorEl, tree);
        }
      },

      validate : function() {
        var
              valid,
              leftTree = this.getComponent(0);

        if (this.containsInvalidNode())
        {
          this.markTreeInvalid(leftTree, this.validateLeftItemsText);
          return false;
        }

        if (!this.required) {
          return true;
        }

        valid = leftTree.validate.call(leftTree);
        if (!valid)
        {
          this.markTreeInvalid(leftTree, null);
        }
        return valid;
      },

      containsInvalidNode : function() {
        if (!this.validateLeftItems)
        {
          return false;
        }
        var
              i,
              root = this.getComponent(0).root,
              nodes = root.childNodes;

        for (i = 0; i < nodes.length; i=i+1)
        {
          if (nodes[i].attributes.payload.invalid === true)
          {
            return true;
          }
        }

        return false;
      },

      getValue : function() {
        var
              i,
              output = [],
              nodes = this.getComponent(0).root.childNodes;

        for (i = 0; i < nodes.length; i=i+1)
        {
          if (!nodes[i].disabled)
          {
            output.push(this.valueField ? nodes[i].attributes.payload.data[this.valueField] : nodes[i].attributes.payload.data);
          }
        }

        return output;
      },

      setValue : function(arr) {
        if (!Ext.isArray(arr))
        {
          arr = [arr];
        }

        var
              i, j,
              name, valueId, displayValue, found = false, readOnly, node, nodeValue, rec,
              leftRoot = this.getComponent(0).root,
              rightRoot = this.getComponent(2).root,
              nodes = rightRoot.childNodes;

        while (leftRoot.lastChild)
        {
          leftRoot.removeChild(leftRoot.lastChild);
        }

        this.loadStore();

        for (i = 0; i < arr.length; i=i+1)
        {
          valueId = arr[i];
          name = valueId;
          readOnly = false;
          if (typeof(valueId) !== 'string')
          {
            name = valueId[this.displayField];
            readOnly = valueId.readOnly;
            valueId = valueId[this.valueField];
          }
          for (j = 0; j < nodes.length; j=j+1)
          {
            node = nodes[j];
            nodeValue = this.valueField ? node.attributes.payload.data[this.valueField] : node.attributes.payload.id;
            if (nodeValue === valueId)
            {
              leftRoot.appendChild(node);
              if (readOnly)
              {
                node.disable();
              }
              found = true;
              break;
            }
          }
          if (!found)
          {
            rec = {
              id : valueId,
              data : {
                readOnly : readOnly
              }
            };
            rec.data[this.valueField] = valueId;
            displayValue = valueId;
            if (this.validateLeftItems)
            {
              displayValue += ' - (Invalid)';
            }
            rec.data[this.displayField] = displayValue;
            rec.invalid = true;
            this.createNode(leftRoot, rec);
          }
        }

        this.doLayout();
      }
    });

Ext.reg('twinpanelchooser', Nexus.ext.TwinPanelChooser);
return Nexus;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * A store filter box.
 *
 * @since 2.7
 */
NX.define('Nexus.ext.StoreFilterBox', {
  extend: 'Nexus.ext.SearchBox',

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  xtype: 'nx-store-filter-box',

  /**
   * @cfg {Ext.data.Store} store that should filtered
   */
  filteredStore: undefined,

  /**
   * @cfg {Array} array of field ids that should be used for filtering
   */
  filteredFields: undefined,

  /**
   * @cfg {Function} to be used for filtering (defaults to string contains)
   */
  filterFn: function (valueToBeMatched, filterValue) {
    var stringValue;
    if (valueToBeMatched) {
      stringValue = valueToBeMatched.toString();
      if (stringValue) {
        return stringValue.toLowerCase().indexOf(filterValue.toLowerCase()) != -1;
      }
    }
    return false;
  },

  /**
   * @override
   */
  initComponent: function () {
    var self = this;

    Nexus.ext.StoreFilterBox.superclass.initComponent.call(self, arguments);

    self.addEvents(

        /**
         * Fires before filtering the store.
         * @event beforeFiltering
         * @param {String} search value
         */
        'beforefiltering'
    );

    self.bindToStore(self.filteredStore);

    self.on('destroy', function () {
      self.unbindFromStore(self.filteredStore);
    });
    self.on('search', function () {
      self.filterStore();
    });
    self.on('searchcleared', function () {
      self.filterStore();
    });
  },

  /**
   * Filters the store on current search box value.
   */
  filterStore: function () {
    var self = this,
        filterValue = self.getSearchValue();

    self.filteredStore.clearFilter();
    if (filterValue) {
      self.logDebug(
          'Filtering ' + self.filteredStore + ' on [' + filterValue + '] using fields: ' + self.filteredFields
      );
      self.fireEvent('beforefiltering');
      self.filteredStore.filterBy(function (record) {
        for (var i = 0; i < self.filteredFields.length; i++) {
          var filteredField = self.filteredFields[i];
          if (filteredField) {
            if (self.matches(filterValue, record, filteredField, record.data[filteredField])) {
              return true;
            }
          }
        }
        return false;
      }, self);
    }
    else {
      self.logDebug('Filtering cleared on ' + self.filteredStore);
    }
  },

  /**
   * @protected
   * Returns true if the field value is defined and matches the filtering function.
   * @param filterValue to match
   * @param record record that was used to extract the value to be matched
   * @param fieldName filter field name that was used to extract the value to be matched
   * @param fieldValue to me matched
   */
  matches: function (filterValue, record, fieldName, fieldValue) {
    var self = this;
    return self.filterFn(fieldValue, filterValue);
  },

  /**
   * Unbinds form current store and register itself to provided store.
   * @param store to register itself to
   */
  reconfigureStore: function (store, filteredFields) {
    var self = this;
    if (self.filteredStore !== store) {
      self.unbindFromStore(self.filteredStore);
      self.bindToStore(store);
    }
    self.filteredFields = filteredFields;
    self.filterStore();
  },

  /**
   * @private
   * Remove itself as listener from provided store.
   * @param store to remove itself from
   */
  unbindFromStore: function (store) {
    var self = this;
    if (store) {
      self.logDebug('Unbinding from store ' + self.filteredStore);
      store.removeListener('load', self.filterStore, self);
    }
  },

  /**
   * @private
   * Register itself as listener of load events on provided store.
   * @param store to register itself to
   */
  bindToStore: function (store) {
    var self = this;
    self.filteredStore = store;
    if (store) {
      self.logDebug('Binding to store ' + self.filteredStore);
      store.on('load', self.filterStore, self);
    }
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/ui',[
  'ext/all',
  'nexus',
  'Nexus/all',
  'Sonatype/all',
  'repoServer'
],
	function(Ext, Nexus) {
	    return Nexus;
	}
);

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus, NX*/

/*
 * A helper panel creating a tabbed container inside itself if more than one
 * component is added.
 */
NX.define('Nexus.panels.AutoTabPanel', {
  extend : 'Ext.Panel',

  requirejs : ['Sonatype/view'],

  constructor : function(cfg) {
    var
          config = cfg || {},
          defaultConfig = {
            layout : 'card',
            activeItem : 0,
            deferredRender : false,
            autoScroll : false,
            frame : false,
            border : false,
            activeTab : 0,
            hideMode : 'offsets',
            tools : [
              {
                id : 'refresh',
                qtip : 'Refresh data',
                hidden : true,
                handler : function(evt, toolEl, panel) {
                  var active;
                  if (panel.tabPanel) {
                    active = panel.tabPanel.getActiveTab();
                  } else {
                    active = panel.getComponent(0);
                  }
                  if (active && active.refreshContent) {
                    active.refreshContent();
                  }
                }
              }
            ]
          };
    Ext.apply(this, config, defaultConfig);
    Sonatype.panels.AutoTabPanel.superclass.constructor.call(this, {
      collapseMode : 'mini'
    });
  },

  add : function(c) {
    if (this.items && this.items.length > 0) {
      if (!this.tabPanel) {
        this.initTabPanel();
      }

      if (!c.title && c.tabTitle) {
        c.title = c.tabTitle;
      }
      return this.tabPanel.add(c);
    }
    else {
      return Sonatype.panels.AutoTabPanel.superclass.add.call(this, c);
    }
  },
  insert : function(index, c) {
    if (this.items && this.items.length > 0) {
      if (!this.tabPanel) {
        this.initTabPanel();
      }

      if (!c.title && c.tabTitle) {
        c.title = c.tabTitle;
      }
      return this.tabPanel.insert(index, c);
    }
    else {
      return Sonatype.panels.AutoTabPanel.superclass.insert.call(this, index, c);
    }
  },
  initTabPanel : function() {
    var first = this.getComponent(0);
    this.remove(first, false);
    first.setTitle(first.tabTitle);

    this.tabPanel = new Ext.TabPanel({
      activeItem : this.activeTab === -1 ? null : this.activeTab,
      deferredRender : false,
      enableTabScroll : true,
      autoScroll : false,
      frame : false,
      border : false,
      layoutOnTabChange : true,
      items : [first],
      hideMode : 'offsets',
      listeners : {
        tabchange : function(panel, tab) {
          var tool = this.tools && this.tools.refresh;
          if (tool) {
            if (tab.refreshContent) {
              tool.show();
            } else {
              tool.hide();
            }
          }
        },
        scope : this
      }
    });

    Sonatype.panels.AutoTabPanel.superclass.add.call(this, this.tabPanel);
    if (this.getLayout() && this.getLayout().setActiveItem) {
      this.getLayout().setActiveItem(this.tabPanel);
    }
  }
}, function() {
  Ext.ns('Sonatype.panels').AutoTabPanel = Nexus.panels.AutoTabPanel;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
/*jslint evil:true*/
define('Nexus/panels/TreePanel',['extjs', 'Nexus/config', 'sonatype'], function(Ext, Config, Sonatype) {
var ns = Ext.namespace('Sonatype.panels');

ns.TreePanel = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          titleColumn : 'name',
          nodeIconClass : null,
          useNodeIconClassParam : null,
          nodeClass : null,
          useNodeClassParam : null,
          nodePathPrepend : '',
          appendPathToRoot : true,
          leafClickEvent : null,
          resetRootNodeText : true,
          autoExpandRoot : true,
          appendAttributeToId : null
        },
        root;
  Ext.apply(this, config, defaultConfig);

  this.tbar = [{
    text : 'Refresh',
    iconCls : 'st-icon-refresh',
    cls : 'x-btn-text-icon',
    scope : this,
    handler : this.refreshHandler
  }];

  if (this.toolbarInitEvent)
  {
    Sonatype.Events.fireEvent(this.toolbarInitEvent, this, this.tbar);
  }

  ns.TreePanel.superclass.constructor.call(this, {
    anchor : '0 -2',
    bodyStyle : 'background-color:#FFFFFF',
    animate : true,
    lines : false,
    autoScroll : true,
    containerScroll : true,
    rootVisible : true,
    enableDD : false,
    loader : new Ext.tree.TreeLoader({
      nodePathPrepend : this.nodePathPrepend,
      appendPathToRoot : this.appendPathToRoot,
      nodeIconClass : this.nodeIconClass,
      useNodeIconClassParam : this.useNodeIconClassParam,
      nodeClass : this.nodeClass,
      useNodeClassParam : this.useNodeClassParam,
      appendAttributeToId : this.appendAttributeToId,
      requestMethod : 'GET',
      url : this.url,
      listeners : {
        loadexception : this.treeLoadExceptionHandler,
        scope : this
      },
      requestData : function(node, callback) {
        if (this.fireEvent("beforeload", this, node, callback) !== false)
        {
          this.transId = Ext.Ajax.request({
            method : this.requestMethod,
            // Sonatype: nodes contain a relative request path
            url : this.url + ((this.appendPathToRoot || node.attributes.path !== '/') ? (this.nodePathPrepend + node.attributes.path) : ''),
            success : this.handleResponse,
            params : this.baseParams,
            failure : this.handleFailure,
            scope : this,
            argument : {
              callback : callback,
              node : node
            }
          });
        }
        else
        {
          if (typeof callback === "function")
          {
            callback();
          }
        }
      },
      createNode : function(attr) {
        if (this.baseAttrs)
        {
          Ext.applyIf(attr, this.baseAttrs);
        }
        if (this.applyLoader !== false)
        {
          attr.loader = this;
        }
        if (typeof attr.uiProvider === 'string')
        {
          attr.uiProvider = this.uiProviders[attr.uiProvider] || eval(attr.uiProvider);
        }

        // Sonatype: node name is supplied as 'nodeName' instead of
        // 'text'
        if (!attr.text && attr.nodeName)
        {
          attr.text = attr.nodeName;
        }
        if (!attr.id)
        {
          attr.id = (this.url + attr.path).replace(/\//g, '_');
          if ( this.appendAttributeToId )
          {
            attr.id += attr[this.appendAttributeToId];
          }
        }

        if (!attr.singleClickExpand)
        {
          attr.singleClickExpand = true;
        }

        if (!this.nodeIconClass)
        {
          if (!this.useNodeIconClassParam || attr[this.useNodeIconClassParam])
          {
            attr.iconCls = this.nodeIconClass;
          }
        }

        if (!this.nodeClass)
        {
          if (!this.useNodeClassParam || attr[this.useNodeClassParam])
          {
            attr.cls = this.nodeClass;
          }
        }

        attr.rootUrl = this.url;

        if (attr.nodeType)
        {
          return new Ext.tree.TreePanel.nodeTypes[attr.nodeType](attr);
        }

        return attr.leaf ? new Ext.tree.TreeNode(attr) : new Ext.tree.AsyncTreeNode(attr);
      },
      processResponse : function(response, node, callback) {
        var i, len, n, o, json = response.responseText;
        try
        {
          o = Ext.decode(json);
          if (o.data)
          {
            o = o.data;
            node.beginUpdate();

            // Sonatype:
            // - tree response contains the current node, not just an
            // array of children
            // - node name is supplied as 'nodeName' instead of 'text'
            if (!node.isRoot)
            {
              node.setText(o.nodeName);
              Ext.apply(node.attributes, o, {});
            }
            for (i = 0, len = o.children.length; i < len; i=i+1)
            {
              n = this.createNode(o.children[i]);
              if (n)
              {
                node.appendChild(n);
              }
            }

            node.endUpdate();
          }

          if (typeof callback === "function")
          {
            callback(this, node);
          }
        }
        catch (e)
        {
          this.handleFailure(response);
        }
      }
    }),
    listeners : {
      click : {
        fn : this.nodeClickHandler,
        scope : this
      },
      contextMenu : {
        fn : this.nodeContextMenuHandler,
        scope : this
      }
    }
  });

  this.sorter = new Ext.tree.TreeSorter(this, {
    folderSort : true
  });

  if (!this.getRootNode())
  {

    root = new Ext.tree.AsyncTreeNode({
      text : this.payload ? this.payload.data[this.titleColumn] : '/',
      path : '/',
      singleClickExpand : true,
      expanded : this.autoExpandRoot
    });

    this.setRootNode(root);
  }
};

Ext.extend(ns.TreePanel, Ext.tree.TreePanel, {
  nodeClickHandler : function(node, e) {
    if (e.target.nodeName === 'A') {
      return; // no menu on links
    }

    if (this.nodeClickEvent)
    {
      Sonatype.Events.fireEvent(this.nodeClickEvent, node, this.nodeClickPassthru);
    }
    else if (this.leafClickEvent)
    {
      Sonatype.Events.fireEvent(this.leafClickEvent, node, this.leafClickPassthru);
    }
  },

  nodeContextMenuHandler : function(node, e) {
    if (e.target.nodeName === 'A') {
      return; // no menu on links
    }

    if (this.nodeContextMenuEvent)
    {

      var menu = new Sonatype.menu.Menu({
        payload : node,
        scope : this,
        items : []
      });

      Sonatype.Events.fireEvent(this.nodeContextMenuEvent, menu, node);
      if (!menu.items.first()) {
        return;
      }

      e.stopEvent();
      menu.showAt(e.getXY());
    }
  },

  refreshHandler : function(button, e) {
    if (this.resetRootNodeText)
    {
      this.root.setText(this.payload ? this.payload.data[this.titleColumn] : '/');
    }
    this.root.attributes.localStorageUpdated = false;
    if (this.root.reload)
    {
      this.root.reload();
    }
  },

  treeLoadExceptionHandler : function(treeLoader, node, response) {
    if (response.status === 503)
    {
      if (Sonatype.MessageBox.isVisible())
      {
        Sonatype.MessageBox.hide();
      }
      node.setText(node.text + ' (Out of Service)');
    }
    else if (response.status === 404)
    {
      if (Sonatype.MessageBox.isVisible())
      {
        Sonatype.MessageBox.hide();
      }
      node.setText(node.text + (node.isRoot ? ' (Not Available)' : ' (Not Found)'));
    }
    else if (response.status === 401)
    {
      if (Sonatype.MessageBox.isVisible())
      {
        Sonatype.MessageBox.hide();
      }
      node.setText(node.text + ' (Access Denied)');
    }
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/all',
      [
        'Nexus/config',
        'Nexus/error',
        'Nexus/ext',
        'Nexus/form',
        'Nexus/log',
        'Nexus/messages',
        'Nexus/messagebox',
        'Nexus/navigation',
        'Nexus/panels',
        'Nexus/grid',
        'Nexus/profile',
        'Nexus/util'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/messages', ['extjs', 'nexus'], function() {
    /**
     * Helper to show transient messages to users.
     *
     * This is based on the Ext.examples.msg helper in the 2.3 sources.
     */
    Nexus.messages = function () {
        var msgCt;

        // FIXME: Sort out a better way to do this so we can include icons and other muck if needed

        function createBox(t, s) {
            return [
                '<div class="nx-msg">',
                '<div class="x-box-tl"><div class="x-box-tr"><div class="x-box-tc"></div></div></div>',
                '<div class="x-box-ml"><div class="x-box-mr"><div class="x-box-mc"><h3>', t, '</h3>', s, '</div></div></div>',
                '<div class="x-box-bl"><div class="x-box-br"><div class="x-box-bc"></div></div></div>',
                '</div>'
            ].join('');
        }

        return {
            show: function (title, format) {
                var s, m;

                if (!msgCt) {
                    msgCt = Ext.DomHelper.insertFirst(document.body, {id: 'nx-msg-div'}, true);
                }
                msgCt.alignTo(document, 't-t');
                s = String.format.apply(String, Array.prototype.slice.call(arguments, 1));
                m = Ext.DomHelper.append(msgCt, {html: createBox(title, s)}, true);
                m.slideIn('t').pause(2).ghost("t", {remove: true});
            }
        };
    }();
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/config',['extjs', 'Nexus/messagebox', 'Sonatype/init', 'Nexus/configuration/Ajax'], function(Ext, mbox, Sonatype) {

  // ********* Set ExtJS options
  // *************************************************


  // set Sonatype defaults for Ext widgets
  Ext.form.Field.prototype.msgTarget = 'under';

  mbox.minWidth = 200;

  Ext.state.Manager.setProvider(new Ext.state.CookieProvider());

  Sonatype.config = (function() {
    var
          host = window.location.protocol + '//' + window.location.host,
          path = window.location.pathname,
          contextPath = path.substr(0, path.lastIndexOf('/')),
          servicePathSnippet = '/service/local',
          servicePath = contextPath + servicePathSnippet,
          browsePathSnippet = '/content',
          contentPath = contextPath + browsePathSnippet,
          repoBrowsePathSnippet = browsePathSnippet + '/repositories',
          groupBrowsePathSnippet = browsePathSnippet + '/groups',
          repoServicePathSnippet = servicePathSnippet + '/repositories',
          groupServicePathSnippet = servicePathSnippet + '/repo_groups';

    return {
      isDebug : window.location.search === '?debug',
      host : host,
      contextPath : contextPath,
      servicePath : servicePath,
      // @deprecated use contextPath
      resourcePath : contextPath,
      extPath : contextPath + '/static/ext-3.4.1.1',
      contentPath : contentPath,
      cssPath : '/styles',
      jsPath : '/js',
      browsePathSnippet : browsePathSnippet,

      installedServers : {
        repoServer : true
      },

      repos : {
        snippets : {
          repoBrowsePathSnippet : repoBrowsePathSnippet,
          groupBrowsePathSnippet : groupBrowsePathSnippet,
          repoServicePathSnippet : repoServicePathSnippet,
          groupServicePathSnippet : groupServicePathSnippet
        },
        urls : {
          login : servicePath + '/authentication/login',
          logout : servicePath + '/authentication/logout',
          globalSettings : servicePath + '/global_settings',
          globalSettingsState : servicePath + '/global_settings/current',
          restApiSettings: servicePath + '/rest_api_settings',
          repositories : servicePath + '/repositories',
          allRepositories : servicePath + '/all_repositories',
          repositoryStatuses : servicePath + '/repository_statuses',
          repoTemplates : servicePath + '/templates/repositories',
          repoTemplate : {
            virtual : servicePath + '/templates/repositories/default_virtual_m2_m1',
            hosted : servicePath + '/templates/repositories/default_hosted_release', // default
            hosted_release : servicePath + '/templates/repositories/default_hosted_release',
            hosted_snapshot : servicePath + '/templates/repositories/default_hosted_snapshot',
            proxy : servicePath + '/templates/repositories/default_proxy_release', // default
            proxy_release : servicePath + '/templates/repositories/default_proxy_release',
            proxy_snapshot : servicePath + '/templates/repositories/default_proxy_snapshot'
          },

          metadata : servicePath + '/metadata',
          cache : servicePath + '/data_cache',
          groups : servicePath + '/repo_groups',
          routes : servicePath + '/repo_routes',
          configs : servicePath + '/configs',
          configCurrent : servicePath + '/configs/current',
          status : servicePath + '/status?perms=1',
          schedules : servicePath + '/schedules',
          scheduleRun : servicePath + '/schedule_run',
          scheduleTypes : servicePath + '/schedule_types',
          upload : servicePath + '/artifact/maven/content',
          redirect : servicePath + '/artifact/maven/redirect',
          trash : servicePath + '/wastebasket',
          plexusUsersAllConfigured : servicePath + '/plexus_users/allConfigured',
          plexusUsersDefault : servicePath + '/plexus_users/default',
          plexusUsers : servicePath + '/plexus_users',
          userLocators : servicePath + '/components/userLocators',
          searchUsers : servicePath + '/user_search',
          plexusUser : servicePath + '/plexus_user',
          userToRoles : servicePath + '/user_to_roles',
          users : servicePath + '/users',
          usersReset : servicePath + '/users_reset',
          usersForgotId : servicePath + '/users_forgotid',
          usersForgotPassword : servicePath + '/users_forgotpw',
          usersChangePassword : servicePath + '/users_changepw',
          usersSetPassword : servicePath + '/users_setpw',
          roles : servicePath + '/roles',
          plexusRoles : servicePath + '/plexus_roles',
          plexusRolesAll : servicePath + '/plexus_roles/all',
          externalRolesAll : servicePath + '/external_role_map/all',
          privileges : servicePath + '/privileges',
          repoTargets : servicePath + '/repo_targets',
          repoContentClasses : servicePath + '/components/repo_content_classes',
          realmComponents : servicePath + '/components/realm_types',
          repoTypes : servicePath + '/components/repo_types',
          repoMirrors : servicePath + '/repository_mirrors',
          repoPredefinedMirrors : servicePath + '/repository_predefined_mirrors',
          privilegeTypes : servicePath + '/privilege_types',
          smtpSettingsState : servicePath + '/check_smtp_settings'
        }
      },

      content : {
        groups : contentPath + '/groups',
        repositories : contentPath + '/repositories'
      }
    };
  }());

  return Sonatype.config;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('Nexus/profile/Summary',['extjs', 'Nexus/profile/UserProfile', 'Nexus/ext/FormPanel', 'Nexus/config'], function(Ext, profile){

Ext.namespace('Nexus.profile');
Nexus.profile.Summary = function(config) {
  var
        cfg = config || {},
        defaultConfig,
        items,
        // external => user is not from XML realm but LDAP, Crowd, ...
        isExternalUser = Sonatype.user.curr.loggedInUserSource !== 'default',
        ht = Sonatype.repoServer.resources.help.users,
        WHITE_SPACE_PREFIX = /^\s/;

  if ( !cfg.username ){
    throw 'No username in config';
  }

  this.FIELD_WIDTH = 250;

  items = [
    {
      // NEXUS-5294 Disable Save for external users
      xtype : 'panel',
      cls : 'x-form-invalid-msg',
      border : false,
      html : 'Externally configured users are read-only (' + Sonatype.user.curr.loggedInUserSource + ')',
      hidden : !isExternalUser,
      width : this.FIELD_WIDTH * 2
    },
    {
      xtype : 'textfield',
      fieldLabel : 'User ID',
      itemCls : 'required-field',
      labelStyle : 'margin-left: 15px; width: 185px;',
      helpText : ht.userId,
      name : 'userId',
      disabled : true,
      allowBlank : false,
      width : this.FIELD_WIDTH
    },
    {
      xtype : 'textfield',
      fieldLabel : 'First Name',
      labelStyle : 'margin-left: 15px; width: 185px;',
      helpText : ht.firstName,
      name : 'firstName',
      allowBlank : false,
      itemCls : 'required-field',
      width : this.FIELD_WIDTH,
      disabled : isExternalUser,
      validator : function(v) {
        if (v && v.length !== 0 && (!v.match(WHITE_SPACE_PREFIX))) {
          return true;
        }
        else {
          return 'First Name cannot start with whitespace.';
        }
      }
    },
    {
      xtype : 'textfield',
      fieldLabel : 'Last Name',
      labelStyle : 'margin-left: 15px; width: 185px;',
      helpText : ht.lastName,
      name : 'lastName',
      allowBlank : false,
      itemCls : 'required-field',
      width : this.FIELD_WIDTH,
      disabled : isExternalUser,
      validator : function(v) {
        if (v && v.length !== 0 && (!v.match(WHITE_SPACE_PREFIX))) {
          return true;
        }
        else {
          return 'Last Name cannot start with whitespace.';
        }
      }
    },
    {
      xtype : 'textfield',
      fieldLabel : 'Email',
      itemCls : 'required-field',
      labelStyle : 'margin-left: 15px; width: 185px;',
      helpText : ht.email,
      name : 'email',
      allowBlank : false,
      disabled : isExternalUser,
      width : this.FIELD_WIDTH
    },
    {
      xtype : 'labelfield',
      labelStyle : 'margin-left: 15px; width: 185px;',
      fieldLabel : 'Password',
      value : 'Change Password',
      width : this.FIELD_WIDTH,
      style : 'text-decoration: underline; color: blue; cursor: pointer;',
      hidden : isExternalUser,
      hideLabel : isExternalUser,
      listeners : {
        'render' : function(component) {
          component.getEl().on('click', function() {
            Sonatype.utils.changePassword();
          });
        }
      }
    }
  ];

  defaultConfig = {
    minWidth : 650,
    labelWidth : 50,
    items : items
  };

  Ext.apply(this, cfg, defaultConfig);

  // URI template, payload.id will be appended
  this.uri = Sonatype.config.servicePath + '/user_account';

  // this is a template for the data to be sent from the form fields
  this.referenceData = {
    userId : '',
    firstName : '',
    lastName : '',
    email : ''
  };

  // defining payload.id is a must, see Sonatype.ext.FormPanel#getActionUrl
  // in short, `payload.id` will be used to extend `this.uri` if `payload.data.resourceUri` is not set.
  // otherwise, `payload.data.resourceUri` is used as is.
  this.payload = {
    id : this.username
  };

  // NEXUS-5294 Disable Save for external users
  this.isValid = function() {
    return !isExternalUser && Nexus.profile.Summary.superclass.isValid.apply(this, arguments);
  };

  // mandatory call
  this.checkPayload();

  Nexus.profile.Summary.superclass.constructor.call(this, cfg);
};

Ext.extend(Nexus.profile.Summary, Sonatype.ext.FormPanel, {
  isValid : function() {
    return this.form.isValid();
  }
});

/* don't add this view to the admin tabs in security/users, but it would work like this:
Sonatype.Events.addListener('userAdminViewInit', function(views) {
  views.push({
    name : 'Summary', // title of the tab

    // class definition of the panel, constructor will be called with {username:$selectedUsername}
    item : Nexus.profile.Summary
  });
});
*/

profile.register('Summary', Nexus.profile.Summary, ['user']);

  /* FIXME remove this when UserProfile is stable
  var testPanel = function(){
    var config = {
      html : "This is a panel"
    };
    testPanel.superclass.constructor.call(this, config);
  };

  Nexus.profile.show = true;

  Ext.extend(testPanel, Ext.Panel, {
    refreshContent : function() {
      alert('refreshing');
    },
    shouldShow : function() {
      return Nexus.profile.show;
    }
  })

  Nexus.profile.register('test', testPanel);
   */

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('Nexus/profile/UserProfile',['extjs', 'Sonatype/view'], function(Ext, Sonatype) {
Ext.namespace('Nexus.profile');

var Nexus = window.Nexus;

/**
 * The user profile tab.
 *
 * @constructor
 */
Nexus.profile.UserProfile = function(cfg) {
  var
        _this = this,
        config = cfg || {},
        defaultConfig = {
          autoScroll : true,
          minWidth : 270,
          layout : 'absolute'
        },
        views = [];

  Ext.apply(this, config, defaultConfig);

  // this may happen on opening a bookmark
  if (!Sonatype.user.curr || !Sonatype.user.curr.isLoggedIn) {
    // initialize empty tab (we cannot remove it without this call)
    Nexus.profile.UserProfile.superclass.constructor.call(this, {
      title : 'Profile'
    });

    // suppress the next change TO this tab, because we don't have valid user data yet
    Sonatype.view.mainTabPanel.on("beforetabchange", function(tabPanel, newTab, oldTab) {
      if (newTab === _this) {
        Sonatype.view.mainTabPanel.remove(_this);
        return false;
      }
    }, _this, {
      single : true
    });

    if ( !Sonatype.view.justLoggedOut ) {
      Sonatype.view.afterLoginToken = window.location.hash.substring(1);
      Sonatype.repoServer.RepoServer.loginHandler();
    }

    return;
  }

  Sonatype.Events.fireEvent('userProfileInit', views);

  this.content = new Nexus.profile.UserProfile.Content({
    cls : 'user-profile-dynamic-content',
    border : false,
    x : 20,
    y : 20,
    anchor : '-20 -20'
  });

  this.selector = new Ext.form.ComboBox({
    id: 'user-profile-selector',
    x : 30,
    y : 11,
    editable : false,
    triggerAction : 'all',
    listeners : {
      'select' : {
        fn : function(combo, record, index) {
          this.bookmark = record.get(combo.displayField);
          this.content.display(record.get(combo.valueField), this);
          Sonatype.utils.updateHistory(this);
        },
        scope : this
      },
      'render' : {
        fn : function(combo) {
          if ( this.bookmark !== null ) {
            return;
          }
          var rec = combo.store.getAt(0);
          this.bookmark = rec.get(combo.displayField);

          combo.setValue(rec.get(combo.displayField));
          this.content.display(rec.get(combo.valueField), this);
        },
        scope : this
      }
    },
    store : (function() {
      // [ (v.item,v.name) for v in views]
      var viewArray = [];
      Ext.each(views, function(v) {
        var content = new v.item({username : Sonatype.user.curr.username, border : false, frame : false});
        if (!content.shouldShow || content.shouldShow()) {
          viewArray.push([content, v.name]);
        }
      });
      return viewArray;
    }())
  });

  this.refreshContent = function() {
    var tab = this.content.getActiveTab();
    if (tab.refreshContent) {
      tab.refreshContent();
      this.content.doLayout();
    }
  };

  this.refreshButton = new Ext.Button({
    tooltip : 'Refresh',
    style : 'position: absolute; right:25px; top:25px;',
    iconCls : 'st-icon-refresh',
    cls : 'x-btn-icon',
    scope : this,
    handler : this.refreshContent,
    noExtraClass : true
  });

  Nexus.profile.UserProfile.superclass.constructor.call(this, {
    title : 'Profile',
    items : [
      this.content,
      this.selector,
      this.refreshButton
    ]
  });

  this.bookmark = null;

  this.getBookmark = function() {
    return this.bookmark;
  };

  this.applyBookmark = function(token) {
    var
          rec, idx,
          combo = this.selector;

    token = decodeURIComponent(token);

    // FIXME should use combo.findRecord?
    idx = combo.store.find(combo.displayField, token);

    if (idx !== -1) {
      this.bookmark = token;
      rec = combo.store.getAt(idx);
      combo.setValue(rec.get(combo.valueField));
      this.content.display(rec.get(combo.valueField), this);
    }
  };

};

Ext.extend(Nexus.profile.UserProfile, Ext.Panel);

/**
 * The inner content panel of the user profile tab.
 * This is what changes on combo box selection.
 *
 * @constructor
 */
Nexus.profile.UserProfile.Content = function(config)
{
  Ext.apply(this, config || {}, {
    plain : true,
    autoScroll : true,
    border : true,
    layoutOnTabChange: true,
    listeners : {
      'tabchange' : function() {
        // hide tabStrip
        if (!this.headerAlreadyHidden) {
          this.header.hide();
          this.headerAlreadyHidden = true;
        }
      },
      scope : this
    }
  });

  Nexus.profile.UserProfile.Content.superclass.constructor.call(this);

  this.display = function(panel, profile) {
    this.add(panel);
    this.setActiveTab(panel);
    profile.refreshButton.setVisible(panel.refreshContent !== undefined);
  };
};
Ext.extend(Nexus.profile.UserProfile.Content, Ext.TabPanel);

/**
 * Weirdo hack to get bookmarking to work. navigation-* is usually the link in the left nav panel,
 * but the user profile menu does not have that.
 */
Nexus.profile.OpenAction = function(config) {
  config.initialConfigNavigation = {
    tabId : 'profile',
    tabCode : Nexus.profile.UserProfile,
    title : 'Profile'
  };
  Nexus.profile.OpenAction.superclass.constructor.call(this, config);
};
Ext.extend(Nexus.profile.OpenAction, Ext.Panel);

Nexus.profile.OpenAction.registered = new Nexus.profile.OpenAction({
  id : 'navigation-profile'
});

/**
 * @param {string} name The name displayed in the combo box selector.
 * @param {Object} panelCls The class definition of the panel to show as content. The constructor will be called with {username:$currentUsername} and may override frame and border settings.
 * @param {Array} views (optional) List of profile views to add the panel to. Currently 'user' and 'admin' are support. If omitted, the panel will be added to all views.
 *
 * @static
 * @member Nexus.profile
 */
Nexus.profile.register = function(name, panelCls, views) {
  if (views === undefined || views.indexOf('user') !== -1) {
    Sonatype.Events.addListener('userProfileInit', function(views) {
      views.push({
        name : name,
        item : panelCls
      });
    });
  }

  if (views === undefined || views.indexOf('admin') !== -1) {
    Sonatype.Events.addListener('userAdminViewInit', function(views) {
      views.push({
        name : name,
        item : panelCls
      });
    });
  }
};

  return Nexus.profile;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/util', ['extjs', 'nexus', 'Nexus/util/Format', 'Nexus/util/FormFieldUtil', 'Nexus/util/observable'], function(Ext, Nexus){
  return Nexus;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Nexus, define*/
define('Nexus/log', [], function() {
  return Nexus.Log;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/panels',
      [
        'Nexus/panels/AutoTabPanel',
        'Sonatype/panels/GridViewer',
        'Nexus/panels/TreePanel'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/grid',
      [
        'Nexus/grid/checkcolumn'
      ],
      function() {});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/form',
      [
        'Nexus/form/action',
        'Nexus/form/LabelField',
        'Nexus/form/roleselector',
        'Nexus/form/searchfield'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/error/ErrorHandler', ['extjs', 'nexus', 'Nexus/log'], function(Ext, Nexus, Log) {
  Ext.namespace('Nexus.error');

  Nexus.error.ErrorHandler = function() {
    return {
      init : function() {
        if ( !window.onerror ) {
          window.onerror = Nexus.error.handle;
        } else if ( window.onerror.createSequence ) {
          window.onerror = window.onerror.createSequence(Nexus.error.handle);
        } // else we don't have error display (e.g. with nexus-ui-testsuite siesta tests)
      },
      getFormattedMessage : function(args) {
        var
              lines = ["The following error has occurred:"],
              err = args[0],
              x;
        if (args[0] instanceof Error) { // Error object thrown in try...catch
          lines[lines.length] = "Message: (" + err.name + ") " + err.message;
          lines[lines.length] = "Error number: " + (err.number & 0xFFFF); //Apply binary arithmetic for IE number, firefox returns message string in element array element 0
          lines[lines.length] = "Description: " + err.description;
          lines[lines.length] = "Stacktrace: ";
          lines[lines.length] = err.stack;
        } else if ((args.length === 3) && (typeof(args[2]) === "number")) { // Check the signature for a match with an unhandled exception
          lines[lines.length] = "Message: " + args[0];
          lines[lines.length] = "URL: " + args[1];
          lines[lines.length] = "Line Number: " + args[2];
        } else {
          lines = ["An unknown error has occurred."];
          lines[lines.length] = "The following information may be useful:";
          for (x = 0; x < args.length; x+=1) {
            lines[lines.length] = Ext.encode(args[x]);
          }
        }
        return lines.join("\n");
      },
      displayError : function(args) {
        // purposely creating a new window for each exception (to handle concurrent exceptions)
        var errWindow = new Ext.Window({
          autoScroll : true,
          bodyStyle : {padding : 5},
          height : 150,
          html : this.getFormattedMessage(args).replace(/\n/g, "<br />").replace(/\t/g, " &nbsp; &nbsp;"),
          title : "An error has occurred",
          width : 400
        });
        errWindow.show();
        errWindow.alignTo(Ext.getCmp('st-main-tab-panel').el, 'tr-tr');
      },
      handleError : function() {
        var args = [];
        for (var x = 0; x < arguments.length; x++) {
          args[x] = arguments[x];
        }
        try {
          Log.error(this.getFormattedMessage(args));

          if ('?debug' === window.location.search) {
            this.displayError(args);
          }
        }
        catch (e) {
          // don't introduce even more errors when displaying errors
        }
        // let handling continue to bubble up to firebug console etc.
        return false;
      }
    };
  }();

  Nexus.error.handle = Nexus.error.ErrorHandler.handleError.createDelegate(Nexus.error.ErrorHandler);

  Nexus.error.ErrorHandler.init();

  return Nexus.error.ErrorHandler;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * A simple component to display an image.
 *
 * @class Ext.Image
 * @extends Ext.BoxComponent
 * @namespace Ext
 */
Ext.define('Nexus.Image', {
    extend: 'Ext.BoxComponent',
    xtype: 'image',

    /**
     * @cfg {String} src Image source
     */
    constructor: function (config) {
        var self = this;

        Ext.apply(self, {
            autoEl: {
                tag: 'img',
                src: config.src
            }
        });

        // Call super constructor
        self.constructor.superclass.constructor.apply(self, arguments);
    }
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/profile',
      [
        'Nexus/profile/UserProfile',
        'Nexus/profile/Summary'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
/**
 * DEPRECATED This does not seem to be working anymore, need to check whether plugins use this and then remove.
 */
define('Nexus/grid/checkcolumn', ['extjs', 'nexus'], function(Ext, Nexus) {
  Ext.namespace('Nexus.grid');

  Nexus.grid.CheckColumn = function(config) {
    Ext.apply(this, config);
    if (!this.id) {
      this.id = Ext.id();
    }
    this.renderer = this.renderer.createDelegate(this);
  };

  Nexus.grid.CheckColumn.prototype = {
    init : function(grid) {
      this.grid = grid;
      this.grid.on('render', function() {
        var view = this.grid.getView();
        view.mainBody.on('mousedown', this.onMouseDown, this);
      }, this);
    },

    onMouseDown : function(e, t) {
      if (t.className && t.className.indexOf('x-grid3-cc-' + this.id) !== -1) {
        e.stopEvent();
        var
              index = this.grid.getView().findRowIndex(t),
              record = this.grid.store.getAt(index);

        record.set(this.dataIndex, !record.data[this.dataIndex]);
      }
    },

    renderer : function(v, p, record) {
      p.css += ' x-grid3-check-col-td';
      return '<div class="x-grid3-check-col' + (v ? '-on' : '') + ' x-grid3-cc-' + this.id + '">&#160;</div>';
    }
  };

  // legacy: use Ext namespace as well
  Ext.grid.CheckColumn = Nexus.grid.CheckColumn;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/ext',
      [
        'Nexus/ext/FormPanel',
        'Nexus/ext/TextEntryList',
        'Nexus/ext/linkbutton',
        'Nexus/ext/twinpanelcontroller'
      ],
      function() {});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/repository/AbstractRepoPanel',['extjs', 'Sonatype/all', 'nexus', 'Nexus/repository/action'], function(Ext, Sonatype, Nexus, Action){

/*
 * Repository panel superclass
 *
 * config options: { id: the is of this panel instance [required] title: title
 * of this panel (shows in tab) }
 */
Sonatype.repoServer.AbstractRepoPanel = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {},
        sp = Sonatype.lib.Permissions;

  Ext.apply(this, config, defaultConfig);

  this.ctxRecord = null;
  this.reposGridPanel = null;

  this.repoActions = {
    downloadRemoteItem : new Action({
      condition : function(repo, content) {
        return content && content.isLeaf() && repo.repoType === 'proxy';
      },
      action : {
        text : 'Download From Remote',
        scope : this,
        handler : this.downloadFromRemoteHandler
      }}),
    downloadItem : new Action({
      condition : function(repo, content) {
        return content && content.isLeaf();
      },
      action : {
        text : 'Download',
        scope : this,
        handler : this.downloadHandler
      }
    }),
    viewRemote : new Action(
          {
            condition : function(repo, content) {
              return content && !content.isRoot && !content.isLeaf() &&
                    repo.repoType === 'proxy';
            },
            action : {
              text : 'View Remote',
              scope : this,
              handler : this.downloadFromRemoteHandler
            }}),
    clearCache : new Action({
      condition : function(repo, content) {
        return sp.checkPermission('nexus:cache', sp.DELETE) && repo.repoType !== 'virtual' && repo.userManaged;
      },
      action : {
        text : 'Expire Cache',
        handler : this.clearCacheHandler,
        scope : this
      }
    }),
    rebuildMetadata : new Action({
      condition : function(repo) {
        return sp.checkPermission('nexus:metadata', sp.DELETE) &&
              (repo.format === 'maven2' || repo.format === 'maven1') &&
              (repo.repoType === 'hosted' || repo.repoType === 'group') && repo.userManaged;
      },
      action : {
        text : 'Rebuild Metadata',
        handler : this.rebuildMetadataHandler,
        scope : this
      }
    }),
    putInService : new Action({
      condition : function(repo) {
        return sp.checkPermission('nexus:repostatus', sp.EDIT) &&
              repo.repoType !== 'group' &&
              repo.status && repo.status.localStatus !== 'IN_SERVICE';
      },
      action : {
        text : 'Put in Service',
        scope : this,
        handler : this.putInServiceHandler
      }
    }),
    putOutOfService : new Action({
      condition : function(repo) {
        return sp.checkPermission('nexus:repostatus', sp.EDIT) &&
              repo.repoType !== 'group' &&
              repo.status && repo.status.localStatus === 'IN_SERVICE';
      },
      action : {
        text : 'Put Out of Service',
        scope : this,
        handler : this.putOutOfServiceHandler
      }
    }),
    allowProxy : new Action({
      condition : function(repo) {
        return sp.checkPermission('nexus:repostatus', sp.EDIT) &&
              repo.repoType === 'proxy' && repo.status && repo.status.proxyMode !== 'ALLOW';
      },
      action : {
        text : 'Allow Proxy',
        scope : this,
        handler : this.proxyStatusHandlerFactory('ALLOW', 'ALLOW')
      }
    }),
    blockProxy : new Action({
      condition : function(repo) {
        return sp.checkPermission('nexus:repostatus', sp.EDIT) &&
              repo.repoType === 'proxy' && repo.status && repo.status.proxyMode === 'ALLOW';
      },
      action : {
        text : 'Block Proxy',
        scope : this,
        handler : this.proxyStatusHandlerFactory('BLOCKED_MANUAL', 'BLOCKED')
      }
    }),
    deleteRepoItem : new Action({
      condition : function(repo) {
        return true;
      },
      action : {
        text : 'Delete',
        scope : this,
        handler : this.deleteRepoItemHandler
      }
    })
  };

  Sonatype.repoServer.AbstractRepoPanel.superclass.constructor.call(this, {});
};

Ext.extend(Sonatype.repoServer.AbstractRepoPanel, Ext.Panel, {
      hasSelection : function() {
        return this.ctxRecord || this.reposGridPanel.getSelectionModel().hasSelection();
      },

  /*
      viewHandler : function() {
        if (this.ctxRecord || this.reposGridPanel.getSelectionModel().hasSelection())
        {
          var rec = this.ctxRecord || this.reposGridPanel.getSelectionModel().getSelected();
          this.viewRepo(rec);
        }
      },
      */

      repoActionAjaxSuccessHandler : function(response, options) {
        var statusResp = Ext.decode(response.responseText);
        this.updateRepoStatuses(statusResp.data, options.repoRecord);
      },

      repoActionAjaxFailureHandlerFactory : function(msg) {
        return function(response, options) {
          Sonatype.utils.connectionError(response, msg);
        };
      },

      clearCacheHandler : function(rec) {
        var url = Sonatype.config.repos.urls.cache + rec.data.resourceURI.slice(Sonatype.config.host.length + Sonatype.config.servicePath.length);

        // make sure to provide /content path for repository root requests like
        // ../repositories/central
        if (/.*\/repositories\/[^\/]*$/i.test(url) || /.*\/repo_groups\/[^\/]*$/i.test(url))
        {
          url += '/content';
        }

        Ext.Ajax.request({
              url : url,
              failure : this.repoActionAjaxFailureHandlerFactory("The server did not clear the repository's cache."),
              scope : this,
              method : 'DELETE'
            });
      },

      rebuildMetadataHandler : function(rec) {
        var url = Sonatype.config.repos.urls.metadata + rec.data.resourceURI.slice(Sonatype.config.host.length + Sonatype.config.servicePath.length);

        // make sure to provide /content path for repository root requests like
        // ../repositories/central
        if (/.*\/repositories\/[^\/]*$/i.test(url) || /.*\/repo_groups\/[^\/]*$/i.test(url))
        {
          url += '/content';
        }

        Ext.Ajax.request({
              url : url,
              failure :  this.repoActionAjaxFailureHandlerFactory('The server did not rebuild metadata in the repository.'),
              scope : this,
              method : 'DELETE'
            });
      },

      putInServiceHandler : function(rec) {
        Ext.Ajax.request({
              url : rec.data.resourceURI + '/status',
              jsonData : {
                data : {
                  id : rec.data.id,
                  repoType : rec.data.repoType,
                  localStatus : 'IN_SERVICE'
                }
              },
              success : this.repoActionAjaxSuccessHandler,
              failure : this.repoActionAjaxFailureHandlerFactory('The server did not put the repository into service.'),
              repoRecord : rec,
              scope : this,
              method : 'PUT'
            });
      },

      putOutOfServiceHandler : function(rec) {
        Ext.Ajax.request({
              url : rec.data.resourceURI + '/status',
              jsonData : {
                data : {
                  id : rec.data.id,
                  repoType : rec.data.repoType,
                  localStatus : 'OUT_OF_SERVICE'
                }
              },
              success : this.repoActionAjaxSuccessHandler,
              failure : this.repoActionAjaxFailureHandlerFactory('The server did not put the repository out of service.'),
              repoRecord : rec,
              scope : this,
              method : 'PUT'
            });
      },

      proxyStatusHandlerFactory : function(state, stateName) {
        return function(rec) {
          if (rec.data.status) {
            Ext.Ajax.request({
              url : rec.data.resourceURI + '/status',
              jsonData : {
                data : {
                  id : rec.data.id,
                  repoType : rec.data.repoType,
                  localStatus : rec.data.status.localStatus,
                  remoteStatus : rec.data.status.remoteStatus,
                  proxyMode : state
                }
              },
              success : this.repoActionAjaxSuccessHandler,
              failure : this.repoActionAjaxFailureHandlerFactory('The server did not update the proxy repository status to ' + stateName + '.'),
              scope : this,
              repoRecord : rec,
              method : 'PUT'
            });
          }
        };
      },

      statusConverter : function(status, parent) {
        if (!parent.status) {
          return '<I>retrieving...</I>';
        }

        var
              available, unknown, reason,
              remoteStatus = (String(status.remoteStatus)).toLowerCase(),
              sOut = (status.localStatus === 'IN_SERVICE') ? 'In Service' : 'Out of Service';

        available = remoteStatus === 'available';
        unknown = remoteStatus === 'unknown';

        // unavailable: $reason
        reason = remoteStatus.indexOf('unavailable:') === 0 ? '<br/><I>' + Ext.util.Format.htmlEncode(status.remoteStatus.substr(12)) + '</I>' : null;

        if (parent.repoType === 'proxy')
        {
          if (status.proxyMode.search(/BLOCKED/) === 0)
          {
            sOut += status.proxyMode === 'BLOCKED_AUTO' ? ' - Remote Automatically Blocked' : ' - Remote Manually Blocked';
            if (available) {
              sOut += ' and Available';
            } else {
              sOut += ' and Unavailable';
            }
          }
          else
          { // allow
            if (status.localStatus === 'IN_SERVICE')
            {
              if (!available && unknown)
              {
                sOut += unknown ? ' - <I>checking remote...</I>' : ' - Attempting to Proxy and Remote Unavailable';
              }
            }
            else
            { // Out of service
              sOut += available ? ' - Remote Available' : ' - Remote Unavailable';
            }
          }
        }

        if (reason !== null) {
          sOut += reason;
        }

        return sOut;
      },

      updateRepoStatuses : function(status, rec) {
        var i, status2, rec2;
        rec.beginEdit();
        rec.data.status = status;
        rec.set('displayStatus', this.statusConverter(status, rec.data));
        rec.commit();
        rec.endEdit();

        if (status.dependentRepos)
        {
          for (i = 0; i < status.dependentRepos.length; i+=1)
          {
            status2 = status.dependentRepos[i];
            rec2 = rec.store.getById(Sonatype.config.host + Sonatype.config.repos.urls.repositories + '/' + status2.id);
            if (rec2)
            {
              rec2.beginEdit();
              rec2.data.status = status2;
              rec2.set('displayStatus', this.statusConverter(status2, rec2.data));
              rec2.commit();
              rec2.endEdit();
            }
          }
        }

        Sonatype.Events.fireEvent('nexusRepositoryStatus', status);
      },

      restToContentUrl : function(node, repoRecord) {
        return repoRecord.data.resourceURI + '/content' + node.data.relativePath;
      },

      restToRemoteUrl : function(node, repoRecord) {
        return repoRecord.data.remoteUri + node.data.relativePath;
      },

      downloadHandler : function(node, item, event) {
        event.stopEvent();
        window.open(this.restToContentUrl(node, node.attributes.repoRecord));
      },

      downloadFromRemoteHandler : function(node, item, event) {
        event.stopEvent();
        window.open(this.restToRemoteUrl(node, node.attributes.repoRecord));
      },

      deleteRepoItemHandler : function(node) {
        var url = Sonatype.config.repos.urls.repositories + node.id.slice(Sonatype.config.host.length + Sonatype.config.repos.urls.repositories.length);
        // make sure to provide /content path for repository root requests like
        // ../repositories/central
        if (/.*\/repositories\/[^\/]*$/i.test(url))
        {
          url += '/content';
        }
        Sonatype.MessageBox.show({
              animEl : node.getOwnerTree().getEl(),
              title : 'Delete Repository Item?',
              msg : 'Delete the selected "' + node.data.relativePath + '" ' + (node.isLeaf() ? 'file' : 'folder') + '?',
              buttons : Sonatype.MessageBox.YESNO,
              scope : this,
              icon : Sonatype.MessageBox.QUESTION,
              fn : function(btnName) {
                if (btnName === 'yes' || btnName === 'ok')
                {
                  Ext.Ajax.request({
                        url : url,
                        callback : this.deleteRepoItemCallback,
                        success : function(response, options) {
                          options.contentNode.parentNode.removeChild(options.contentNode);
                        },
                        failure : function(response, options) {
                          Sonatype.MessageBox.alert('Error', response.status === 401 ? "You don't have permission to delete artifacts in this repository" : "The server did not delete the file/folder from the repository");
                        },
                        scope : this,
                        contentNode : node,
                        method : 'DELETE'
                      });
                }
              }
            });
      },

      onRepositoryMenuInit : function(menu, repoRecord) {
        var
              repo = repoRecord.data,
              actions = this.repoActions;

        if (repoRecord.id.substring(0, 4) === 'new_' || !repo.userManaged) {
          return;
        }

        actions.clearCache.maybeAddToMenu(menu, repo);
        actions.rebuildMetadata.maybeAddToMenu(menu, repo);

        menu.add('-');

        actions.blockProxy.maybeAddToMenu(menu, repo);
        actions.allowProxy.maybeAddToMenu(menu, repo);
        actions.putOutOfService.maybeAddToMenu(menu, repo);
        actions.putInService.maybeAddToMenu(menu, repo);

        menu.add('-');
      },

      onRepositoryContentMenuInit : function(menu, repoRecord, contentRecord) {
        var
              repo = repoRecord.data,
              actions = this.repoActions;

        contentRecord.data.resourceURI = contentRecord.data.resourceURI || contentRecord.data.id;

        actions.clearCache.maybeAddToMenu(menu, repo, contentRecord);
        actions.rebuildMetadata.maybeAddToMenu(menu, repo, contentRecord);

        if (contentRecord.isLeaf())
        {
          menu.add('-');
          actions.downloadRemoteItem.maybeAddToMenu(menu, repo, contentRecord);
          actions.downloadItem.maybeAddToMenu(menu, repo, contentRecord);
        }

        if (!contentRecord.isRoot && repoRecord.data.repoType !== 'group')
        {
          actions.viewRemote.maybeAddToMenu(menu, repo, contentRecord);
          menu.add('-');
          actions.deleteRepoItem.maybeAddToMenu(menu, repo, contentRecord);
        }
      }
    });

Sonatype.repoServer.DefaultRepoHandler = new Sonatype.repoServer.AbstractRepoPanel();
Sonatype.Events.addListener('repositoryMenuInit', Sonatype.repoServer.DefaultRepoHandler.onRepositoryMenuInit, Sonatype.repoServer.DefaultRepoHandler);
Sonatype.Events.addListener('repositoryContentMenuInit', Sonatype.repoServer.DefaultRepoHandler.onRepositoryContentMenuInit, Sonatype.repoServer.DefaultRepoHandler);

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('Nexus/repository/action', ['Nexus/log'], function(Log) {

  var Action = function(cfg) {
    var
          _action = this,
          conditions = [];

    if (cfg.condition) {
      conditions.push(cfg.condition);
    }

    this.maybeAddToMenu = function(menu, repo, content) {
      if (_action.isApplicable(repo, content)) {
        menu.add(cfg.action);
      }
    };

    this.addCondition = function(condition) {
      if (typeof(condition) === 'function') {
        conditions.push(condition);
      } else {
        Log.debug('Tried to add non-function condition, ignoring');
      }
    };

    this.isApplicable = function(repo, content) {
      var i, condition;
      for (i = 0; i < conditions.length; i = i + 1) {
        condition = conditions[i];
        if (!condition(repo, content)) {
          return false;
        }
      }
      return true;
    };
  };

  return Action;
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/navigation',
      [
        'Nexus/navigation/menu',
        'Nexus/navigation/panel',
        'Nexus/navigation/section'
      ],
      function() {});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

//
// FIXME: Need to refactor this class into a icon manager and accessor.
// FIXME: The manager will be the single place where all icons are registered.
// FIXME: The accessors will be the place where plugins ask for icons, and also where they register custom icons.
// FIXME: ATM we have lots of duplicated icons in plugins, though they all share the same resource namespace under static/icons.
// FIXME: This is duplication but could also lead to plugins contributing icons which are different and messing up their intended usage.
//
// FIXME: Also need to include the iconConfig stuff here which is duplicated all over now
//

/**
 * Support for icon containers.  An icon container is a helper to manage image/icon resources used by an application.
 *
 * The container automatically creates CSS styles for icons and installs them into the browser when loading.
 *
 * A simple example of an icon container with a single icon, and how to reference the icon.
 *
 *      @example
 *      var myIcons = NX.create('Nexus.util.IconContainer', {
 *          icons: {
 *              hello:  'hello.png'
 *          }
 *      });
 *
 *      // returns an icon object
 *      var icon = myIcons.get('hello);
 *
 *      // the name of the CSS class of the icon
 *      var cssClass = icon.cls;
 *
 *      // renders a <img> html snippet for the icon
 *      var imgHtml = icon.img;
 *
 * Icons can reference other icons, this helps avoid loading duplicate assents into the browser, and still allows
 * for flexible icon naming.  This allows applications to define meaningful names, and then manage which asset
 * belongs to that name inside of the icon container:
 *
 *      @example
 *      var myIcons = NX.create('Nexus.util.IconContainer', {
 *          icons: {
 *              rulePassed:         'tick.png',
 *              event_rulesPassed:  '@rulePassed',
 *              event_rulePassed:   '@rulePassed'
 *             }
 *      });
 *
 * Here 'event_rulesPassed' and 'event_rulePassed' are aliases to the 'rulePassed' icon.  Only 1 img/css asset for 'rulePassed'
 * is installed into the browser.
 *
 * For icons which might have many different sizes, and icon definition can be configured to know about different variants:
 *
 *      @example
 *      var myIcons = NX.create('Nexus.util.IconContainer', {
 *          icons: {
 *              rulePassed: {
 *                  x16:    'tick.png',
 *                  x32:    'tick-32x32.png',
 *                  _:      '^x16'
 *              },
 *              event_rulesPassed:   '@rulePassed',
 *              event_rulePassed:    '@rulePassed'
 *             }
 *      });
 *
 * This defines 5 icons, and 2 img+css (the assets loaded into the browser).
 *
 * The first bits for rulePassed define 2 icons 'rulePassedx16' and 'rulePassedx32' pointing at tick.png and tick-32x32.png respectively.
 * The special _ key is the default which will be used to set the 'rulePassed' icon.
 * The special syntax here with '^x16' means that he default icon is really the 'x16' variant and
 * ATM this only works for the default '_' variant.
 *
 * Icon objects have a variant() method which can be used to access a variant:
 *
 *      @example
 *      var icon = MyIcons.get('rulePassed');
 *      var bigger = icon.variant('x32');
 *
 * If the variant does not exist then it returns the same icon.
 *
 * @since 2.4
 */
NX.define('Nexus.util.IconContainer', {
    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requirejs: [
        'Nexus/config'
    ],

    /**
     * Array of styles (Strings) for each defined icon.
     *
     * @private
     * @type {Array}
     */
    styles: undefined,

    /**
     * Reference to installed stylesheet.
     *
     * @private
     * @type {Stylesheet}
     */
    stylesheet: undefined,

    /**
     * Defined icons.
     *
     * @private
     */
    icons: undefined,

    /**
     * Base-path for images.
     *
     * This defaults to the base resource path of Nexus + '/static/icons'.
     *
     * @public
     * @property
     * @type {String}
     */
    basePath: undefined,

    /**
     * @constructor
     *
     * @param config
     *
     * @cfg {string} stylePrefix    Optional icon style prefix.
     * @cfg {*} icons               At least one {name: fileName} icon configuration is required.
     * @cfg {String} basePath       Optional base path for images.
     */
    constructor: function (config) {
        var self = this,
            config = config || {},
            icons;

        // assign values to fields to avoid using the prototype's storage
        self.styles = [];
        self.icons = {};

        // apply defaults to configuration
        Ext.applyIf(config, {
            basePath: Sonatype.config.resourcePath + '/static/icons',
            stylePrefix: 'nx-icons-'
        });

        // verify, capture and strip out 'icons' from configuration
        NX.assert(config.icons !== undefined, 'At least one icon definition must be configured');
        icons = config.icons;
        delete config.icons;

        // apply configuration
        Ext.apply(self, config);

        self.logGroup('Loading icons');

        Ext.iterate(icons, function (key, value, obj) {
            self.loadIcon(key, value);
        });

        // TODO: Need to optimize this further, so that there is a shared stylesheet to avoid >31 limitation on IE{8,9,?}

        // Install all icons styles into a single stylesheet for this container
        self.stylesheet = Ext.util.CSS.createStyleSheet(self.styles.join(' '));

        // TODO: Pre-load all icons into browser

        self.logGroupEnd();
    },

    /**
     * Returns the full path to an icon file.
     *
     * @private
     *
     * @return {string}
     */
    iconPath: function (file) {
        if (!file.startsWith('/')) {
            file = '/' + file;
        }
        return this.basePath + file;
    },

    /**
     * Returns the icon name of a variant.
     *
     * @private
     *
     * @return {String}
     */
    variantName: function(name, variant) {
        // TODO: Do we want this or maybe name + '_' + variant ?
        return name + variant;
    },

    /**
     * Load icons from configuration.
     *
     * @private
     *
     * @param {String} name     Icon name.
     * @param {*} config        String fileName/alias or object for icon with variants.
     */
    loadIcon: function(name, config) {
        var self = this;

        // if config is a string, then its a simple fileName/alias just define it
        if (Ext.isString(config)) {
            self.defineIcon(name, config);
        }
        // else its a complex icon definition with variants
        else if (Ext.isObject(config)) {
            // strip off the default icon configuration.
            var defaultIconFileName = config._;
            delete config._;

            // define icons for each variant, remember last icon name we created for default
            var lastIcon;
            Ext.iterate(config, function(key, value) {
                lastIcon = self.defineIcon(self.variantName(name, key), value, name);
            });

            // complain if there were no variants configured
            NX.assert(lastIcon !== undefined, 'No icon variants defined');

            // handle default icon
            if (defaultIconFileName !== undefined) {
                // if the fileName starts with '^' then its an alias back reference to a variant
                if (defaultIconFileName.startsWith('^')) {
                    defaultIconFileName = '@' + self.variantName(name, defaultIconFileName.substring(1));
                }
                // otherwise could be a filePath or alias too
                self.defineIcon(name, defaultIconFileName);
            }
            else {
                // if no default configuration, then the default icon an alias to the last variant defined
                self.defineIcon(name, '@' + lastIcon.name);
            }
        }
    },

    /**
     * Define an icon from a fileName (or @alias).
     *
     * @private
     *
     * @param {string} name         Icon name.
     * @param {string} fileName     Icon file name (or @alias).
     * @param {string} [baseName]   Base icon name when used with variants.
     * @return {*}                  Icon helper.
     */
    defineIcon: function (name, fileName, baseName) {
        var self = this,
            alias,
            iconPath,
            cls,
            icon;

        // TODO: fileName, really a config, if a string, is this below, if an object is variant icon container

        // Puke early if icon already defined, this is likely a mistake
        NX.assert(self.icons[name] === undefined, 'Icon already defined with name: ' + name);

        // If fileName is an alias, then resolve it
        if (fileName.startsWith('@')) {
            alias = fileName.substring(1, fileName.length);
            icon = self.icons[alias];
            NX.assert(icon !== undefined, 'Invalid alias; No icon defined with name: ' + alias);

            self.logDebug('Defining icon:', name, 'aliased to:', alias);
        }
        else {
            // else define a new icon
            iconPath = self.iconPath(fileName);

            cls = self.stylePrefix + name;

            self.logDebug('Defining icon:', name, 'cls:', cls, 'path:', iconPath);

            // append style for icon, will be appended to icon containers stylesheet
            self.styles.push('.' + cls + ' { background: url(' + iconPath + ') no-repeat !important; }');

            /**
             * Icon.
             */
            icon = {
                /**
                 * Symbolic name for icon.
                 *
                 * @type {String}
                 */
                name: name,

                /**
                 * The base symbolic name for an icon with variants.
                 *
                 * @type {String}
                 */
                baseName: baseName,

                /**
                 * Short icon file-name.
                 *
                 * @type {String}
                 */
                fileName: fileName,

                /**
                 * Full icon path.
                 *
                 * @type {String}
                 */
                path: iconPath,

                /**
                 * HTML <img> representation.
                 *
                 * @type {String}
                 */
                img: '<img src="' + iconPath + '">',

                /**
                 * Icon CSS class.
                 *
                 * @type {String}
                 */
                cls: cls,

                /**
                 * Return an icon variant, or the same icon if the variant does not exist.
                 *
                 * @param {String} variantName
                 * @return {*} Icon; never null
                 */
                variant: function(variantName) {
                    // if baseName is not set, then we have no variants configured, return same icon
                    if (baseName === undefined) {
                        self.logWarn('Icon has no variants:', name);
                        return this;
                    }

                    // else look for a variant
                    var icon = self.find(baseName, variantName);
                    if (icon !== undefined) {
                        return icon;
                    }

                    self.logWarn('No such icon variant:', variantName, 'for icon:', baseName);

                    // still nothing, return same icon
                    return this;
                }
            }
        }

        self.icons[name] = icon;

        return icon;
    },

    /**
     * Lookup an icon by name.
     *
     * @public
     *
     * @param name      The name of the icon.
     * @param variant   Optional icon variant name.
     * @return {*}      Icon; or null
     */
    find: function (name, variant) {
        var self = this;

        if (variant !== undefined) {
            // TODO: Probably should verify that variant is a string, as that is what we expect ATM
            name = name + variant;
        }

        return self.icons[name];
    },

    /**
     * Lookup an icon by name.  If the named icon is not defined an exception will be thrown.
     *
     * @public
     *
     * @param name      The name of the icon.
     * @param [variant] Optional icon variant name.
     * @return {*}      Icon; never null/undefined.
     */
    get: function (name, variant) {
        var self = this,
            icon = self.find(name, variant);

        if (variant !== undefined) {
            NX.assert(icon !== undefined, 'No icon defined for name: ' + name + ' variant: ' + variant);
        }
        else {
            NX.assert(icon !== undefined, 'No icon defined for name: ' + name);
        }

        return icon;
    }

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for common icons.
 *
 * @since 2.7
 */
NX.define('Nexus.util.Icons', {
  extend: 'Nexus.util.IconContainer',
  singleton: true,

  /**
   * @constructor
   */
  constructor: function () {
    var self = this;

    self.constructor.superclass.constructor.call(self, {
      stylePrefix: 'nx-icon-',

      icons: {
        glyph_circle_remove:  'glyph_circle_remove.png',
        glyph_filter:         'glyph_filter.png',
        glyph_search:         'glyph_search.png'
      }
    });
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global top, define*/
define('Nexus/util/FormFieldUtil',['extjs', 'Sonatype/view', 'nexus'], function(Ext, Sonatype, Nexus) {

Ext.namespace('Nexus.util');
Nexus.util.FormFieldUtil = {
  generateForm : function(panelId, fieldSetName, fieldNamePrefix, typeStore, repoStore, groupStore, repoOrGroupStore,
        customTypes, width, repoTargetStore) {
    var allTypes = [];

    if (!width) {
      width = 300;
    }

    allTypes[0] = {
      xtype : 'fieldset',
      id : panelId + '_emptyItem',
      checkboxToggle : false,
      title : fieldSetName,
      anchor : Sonatype.view.FIELDSET_OFFSET,
      collapsible : false,
      autoHeight : true,
      layoutConfig : {
        labelSeparator : ''
      }
    };

    // Now add the dynamic content
    typeStore.each(function(item, i, len) {
      var j, items = [], curRec;
      if (item.data.formFields.length > 0) {
        for (j = 0; j < item.data.formFields.length; j = j + 1) {
          curRec = item.data.formFields[j];
          // Note that each item is disabled initially, this is because
          // the select handler for the capabilityType
          // combo box handles enabling/disabling as necessary, so each
          // inactive card isn't also included in the form
          if (curRec.type === 'string' || curRec.type === 'password') {
            items[j] = {
              xtype : 'textfield',
              htmlDecode : true,
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              width : width,
              regex : curRec.regexValidation ? new RegExp(curRec.regexValidation) : null
            };
            if (curRec.type === 'password') {
              items[j].inputType = 'password';
            }
            if (curRec.initialValue) {
              items[j].value = curRec.initialValue;
            }
          }
          else if (curRec.type === 'number') {
            items[j] = {
              xtype : 'numberfield',
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              width : width,
              regex : curRec.regexValidation ? new RegExp(curRec.regexValidation) : null
            };
            if (curRec.initialValue) {
              items[j].value = Number(curRec.initialValue);
            }
          }
          else if (curRec.type === 'text-area') {
            items[j] = {
              xtype : 'textarea',
              htmlDecode : true,
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              anchor : '-20',
              height : '138',
              regex : curRec.regexValidation ? new RegExp(curRec.regexValidation) : null
            };
            if (curRec.initialValue) {
              items[j].value = curRec.initialValue;
            }
          }
          else if (curRec.type === 'checkbox') {
            items[j] = {
              xtype : 'checkbox',
              fieldLabel : curRec.label,
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              disabled : true
            };
            if (curRec.initialValue) {
              items[j].checked = Boolean('true' === curRec.initialValue);
            }
          }
          else if (curRec.type === 'date') {
            items[j] = {
              xtype : 'datefield',
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              value : new Date()
            };
            if (curRec.initialValue) {
              items[j].value = new Date(Number(curRec.initialValue));
            }
          }
          else if (curRec.type === 'repo') {
            items[j] = {
              xtype : 'combo',
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              store : repoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              width : width,
              minListWidth : width
            };
            if (curRec.initialValue) {
              items[j].value = curRec.initialValue;
            }
          }
          else if (curRec.type === 'repo-target') {
            items[j] = {
              xtype : 'combo',
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              store : repoTargetStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              width : width,
              minListWidth : width
            };
            if (curRec.initialValue) {
              items[j].value = curRec.initialValue;
            }
          }
          else if (curRec.type === 'group') {
            items[j] = {
              xtype : 'combo',
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              store : groupStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              width : width,
              minListWidth : width
            };
            if (curRec.initialValue) {
              items[j].value = curRec.initialValue;
            }
          }
          else if (curRec.type === 'repo-or-group') {
            items[j] = {
              xtype : 'combo',
              fieldLabel : curRec.label,
              itemCls : curRec.required ? 'required-field' : '',
              helpText : curRec.helpText,
              name : fieldNamePrefix + curRec.id,
              store : repoOrGroupStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : curRec.required ? false : true,
              disabled : true,
              width : width,
              minListWidth : width
            };
            if (curRec.initialValue) {
              items[j].value = curRec.initialValue;
            }
          }
          else if (customTypes && customTypes[curRec.type]) {
            items[j] = customTypes[curRec.type].createItem.call(this, curRec, fieldNamePrefix, width);
          }

          allTypes[allTypes.length] = {
            xtype : 'fieldset',
            id : panelId + '_' + item.data.id,
            checkboxToggle : false,
            title : fieldSetName,
            anchor : Sonatype.view.FIELDSET_OFFSET,
            collapsible : false,
            autoHeight : true,
            labelWidth : 175,
            layoutConfig : {
              labelSeparator : ''
            },
            items : items
          };
        }
      }
      else {
        allTypes[allTypes.length] = {
          xtype : 'fieldset',
          id : panelId + '_' + item.data.id,
          checkboxToggle : false,
          title : fieldSetName,
          anchor : Sonatype.view.FIELDSET_OFFSET,
          collapsible : false,
          autoHeight : true,
          labelWidth : 175,
          layoutConfig : {
            labelSeparator : ''
          }
        };
      }
    }, this);

    return allTypes;
  }, createForm : function(panelId, fieldSetName, fieldNamePrefix, typeStore, stores, customTypes, width) {
    var allTypes = [];

    if (!width) {
     width = 300;
    }

    storeRecordConstructor = Ext.data.Record.create([{
       name : 'id'
     }, {
       name : 'name',
       sortType : Ext.data.SortTypes.asUCString
    }]);

    allTypes[0] = {
     xtype : 'fieldset',
     id : panelId + '_emptyItem',
     checkboxToggle : false,
     title : fieldSetName,
     anchor : Sonatype.view.FIELDSET_OFFSET,
     collapsible : false,
     autoHeight : true,
     layoutConfig : {
       labelSeparator : ''
     }
    };

    // Now add the dynamic content
    typeStore.each(function(item, i, len) {
     var j, items = [], curRec;
     if (item.data.formFields.length > 0) {
       for (j = 0; j < item.data.formFields.length; j = j + 1) {
         curRec = item.data.formFields[j];
         // Note that each item is disabled initially, this is because
         // the select handler for the capabilityType
         // combo box handles enabling/disabling as necessary, so each
         // inactive card isn't also included in the form
         if (curRec.type === 'string' || curRec.type === 'password') {
           curItem = items[items.length] = {
             xtype : 'textfield',
             htmlDecode : true,
             fieldLabel : curRec.label,
             itemCls : curRec.required ? 'required-field' : '',
             helpText : curRec.helpText,
             name : fieldNamePrefix + curRec.id,
             allowBlank : curRec.required ? false : true,
             disabled : true,
             width : width,
             regex : curRec.regexValidation ? new RegExp(curRec.regexValidation) : null
           };
           if (curRec.type === 'password') {
             curItem.inputType = 'password';
           }
           if (curRec.initialValue) {
             curItem.value = curRec.initialValue;
           }
         }
         else if (curRec.type === 'number') {
           curItem = items[items.length] = {
             xtype : 'numberfield',
             fieldLabel : curRec.label,
             itemCls : curRec.required ? 'required-field' : '',
             helpText : curRec.helpText,
             name : fieldNamePrefix + curRec.id,
             allowBlank : curRec.required ? false : true,
             disabled : true,
             width : width,
             regex : curRec.regexValidation ? new RegExp(curRec.regexValidation) : null
           };
           if (curRec.initialValue) {
             curItem.value = Number(curRec.initialValue);
           }
         }
         else if (curRec.type === 'text-area') {
           curItem = items[items.length] = {
             xtype : 'textarea',
             htmlDecode : true,
             fieldLabel : curRec.label,
             itemCls : curRec.required ? 'required-field' : '',
             helpText : curRec.helpText,
             name : fieldNamePrefix + curRec.id,
             allowBlank : curRec.required ? false : true,
             disabled : true,
             anchor : '-20',
             height : '138',
             regex : curRec.regexValidation ? new RegExp(curRec.regexValidation) : null
           };
           if (curRec.initialValue) {
             curItem.value = curRec.initialValue;
           }
         }
         else if (curRec.type === 'checkbox') {
           curItem = items[items.length] = {
             xtype : 'checkbox',
             fieldLabel : curRec.label,
             helpText : curRec.helpText,
             name : fieldNamePrefix + curRec.id,
             disabled : true
           };
           if (curRec.initialValue) {
             curItem.checked = Boolean('true' === curRec.initialValue);
           }
         }
         else if (curRec.type === 'date') {
           curItem = items[items.length] = {
             xtype : 'datefield',
             fieldLabel : curRec.label,
             itemCls : curRec.required ? 'required-field' : '',
             helpText : curRec.helpText,
             name : fieldNamePrefix + curRec.id,
             allowBlank : curRec.required ? false : true,
             disabled : true,
             value : new Date()
           };
           if (curRec.initialValue) {
             curItem.value = new Date(Number(curRec.initialValue));
           }
         }
         else if (curRec.type === 'combobox' || curRec.type === 'repo' || curRec.type === 'repo-or-group' || curRec.type === 'repo-target') {
           if(curRec.storePath){
             store = stores[curRec.storePath];
             if(!store) {
               storeReader = new Ext.data.JsonReader({
                 root : curRec.storeRoot,
                 idProperty : curRec.idMapping ? curRec.idMapping : 'id',
                 fields: [
                   {name: 'id', mapping: curRec.idMapping ? curRec.idMapping : 'id'},
                   {name: 'name', mapping: curRec.nameMapping ? curRec.nameMapping : 'name', sortType : Ext.data.SortTypes.asUCString},
                 ]
               });
               store = new Ext.data.Store({
                 url : Sonatype.config.contextPath + curRec.storePath,
                 reader : storeReader,
                 sortInfo : {
                   field : 'name',
                   direction : 'ASC'
                 },
                 autoLoad : true
               });
               store[curRec.storePath] = store
             }
             curItem = items[items.length] = {
               xtype : 'combo',
               fieldLabel : curRec.label,
               itemCls : curRec.required ? 'required-field' : '',
               helpText : curRec.helpText,
               name : fieldNamePrefix + curRec.id,
               store : store,
               displayField : 'name',
               valueField : 'id',
               editable : false,
               forceSelection : true,
               mode : 'local',
               triggerAction : 'all',
               emptyText : 'Select...',
               selectOnFocus : true,
               allowBlank : curRec.required ? false : true,
               disabled : true,
               width : width,
               minListWidth : width
             };
             if (curRec.initialValue) {
               curItem.value = curRec.initialValue;
             }
           }
         }
         else if (customTypes && customTypes[curRec.type]) {
           items[items.length] = customTypes[curRec.type].createItem.call(this, curRec, fieldNamePrefix, width);
         }

         allTypes[allTypes.length] = {
           xtype : 'fieldset',
           id : panelId + '_' + item.data.id,
           checkboxToggle : false,
           title : fieldSetName,
           anchor : Sonatype.view.FIELDSET_OFFSET,
           collapsible : false,
           autoHeight : true,
           labelWidth : 175,
           layoutConfig : {
             labelSeparator : ''
           }
         };
         if (items.length > 0) {
           allTypes[allTypes.length - 1].items = items;
         }
       }
     }
     else {
       allTypes[allTypes.length] = {
         xtype : 'fieldset',
         id : panelId + '_' + item.data.id,
         checkboxToggle : false,
         title : fieldSetName,
         anchor : Sonatype.view.FIELDSET_OFFSET,
         collapsible : false,
         autoHeight : true,
         labelWidth : 175,
         layoutConfig : {
           labelSeparator : ''
         }
       };
     }
    }, this);

    return allTypes;
 }, "exportForm" : function(formPanel, panelIdSuffix, formFieldPrefix, customTypes) {
    var outputArr = [], i = 0, formFieldPanel = formPanel.findById(formPanel.id + panelIdSuffix);

    // These are dynamic fields here, so some pretty straightforward generic
    // logic below
    formFieldPanel.getLayout().activeItem.items.each(function(item, i, len) {
      var value;

      if (item.xtype === 'datefield') {
        // long representation is used, not actual date
        // force to a string, as that is what the current api requires
        value = String(item.getValue().getTime());
      }
      else if (item.xtype === 'textfield') {
        value = item.getValue();
      }
      else if (item.xtype === 'numberfield') {
        // force to a string, as that is what the current api requires
        value = String(item.getValue());
      }
      else if (item.xtype === 'textarea') {
        value = item.getValue();
      }
      else if (item.xtype === 'checkbox') {
        value = String(item.getValue());
      }
      else if (item.xtype === 'combo') {
        value = item.getValue();
      }
      else if (customTypes && customTypes[item.xtype]) {
        value = customTypes[item.xtype].retrieveValue.call(item, item);
      }

      outputArr[i] = {
        key : item.getName().substring(formFieldPrefix.length),
        value : value
      };
      i = i + 1;
    }, formFieldPanel.getLayout().activeItem);

    return outputArr;
  }, "importForm" : function(jsonObject, formPanel, formFieldPrefix, customTypes) {
    var i, j, formFields, formField;
    // Maps the incoming json properties to the generic component
    for (i = 0; i < jsonObject.properties.length; i = i + 1) {
      formFields = formPanel.find('name', formFieldPrefix + jsonObject.properties[i].key);
      for (j = 0; j < formFields.length; j = j + 1) {
        formField = formFields[j];

        if (formField !== null) {
          if (!formField.disabled && !Ext.isEmpty(jsonObject.properties[i].value)) {
            if (formField.xtype === 'datefield') {
              formField.setValue(new Date(Number(jsonObject.properties[i].value)));
            }
            else if (formField.xtype === 'textfield') {
              formField.setValue(jsonObject.properties[i].value);
            }
            else if (formField.xtype === 'numberfield') {
              formField.setValue(Number(jsonObject.properties[i].value));
            }
            else if (formField.xtype === 'textarea') {
              formField.setValue(jsonObject.properties[i].value);
            }
            else if (formField.xtype === 'checkbox') {
              formField.setValue(Boolean('true' === jsonObject.properties[i].value));
            }
            else if (formField.xtype === 'combo') {
              formField.setValue(jsonObject.properties[i].value);
            }
            else if (customTypes && customTypes[formField.xtype]) {
              customTypes[formField.xtype].setValue.call(formField, formField, jsonObject.properties[i].value);
            }
            break;
          }
        }
      }
    }
  }
};

// FIXME legacy
top.FormFieldGenerator = Nexus.util.FormFieldUtil.generateForm;
top.FormFieldFactory = Nexus.util.FormFieldUtil.createForm;
top.FormFieldImporter = Nexus.util.FormFieldUtil.importForm;
top.FormFieldExporter = Nexus.util.FormFieldUtil.exportForm;

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/util/Format',['extjs', 'nexus'], function(Ext, Nexus) {
  Ext.namespace('Nexus.util');
/**
 * @static
 */
Nexus.util.Format = {
  /**
   * Decode the given string as HTML. Strips all HTML tags before rendering as XSS protection.
   *
   * @param input The input string. May contain HTML entities (e.g. &#26412;&amp;)
   * @return {string} The string as rendered in a browser, with all tags stripped.
   */
  htmlDecode : function(input) {
    if (input === null || input === '') {
      return input;
    }
    var el = document.createElement('div');
    el.innerHTML = Ext.util.Format.stripTags(input);
    return el.childNodes[0].nodeValue;
  }
};

  return Nexus.util.Format;
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('Nexus/util/observable', ['extjs', 'nexus'], function(Ext, Nexus) {
  Ext.namespace('Nexus.util');
  Nexus.util.Observable = function() {
    this.addEvents({
      /*
       * Fired when the main Nexus navigation panel is being built.
       * Subscribers can use this event to add items to the navigation panel.
       * A Sonatype.navigation.NavigationPanel instance is passed as a
       * parameter. The subscriber can use the "add" method on it to append
       * new sections or individual items to existing sections (using a
       * "sectionId" config property). init: function() {
       * Sonatype.Events.addListener( 'nexusNavigationInit', this.naviHandler,
       * this ); }, naviHandler: function( navigationPanel ) { // add a new
       * section with some items navigationPanel.add( { title: 'My Section',
       * id: 'my-nexus-section', items: [ { title: 'Open New Tab', tabId:
       * 'my-unique-tab-id', tabCode: My.package.ClassName, // JavaScript
       * class implementing Ext.Panel, tabTitle: 'My Tab' // optional tab
       * title (if different from the link title) }, { title: 'Open Another
       * Tab', tabId: 'my-second-tab-id', tabCode: My.package.AnotherClass,
       * enabled: this.isSecondTabEnabled() // condition to show the link or
       * not }, { title: 'Pop-up Dialog', handler: this.popupHandler, // click
       * handler scope: this // handler execution scope } ] } ); // add a link
       * to an existing section navigationPanel.add( { sectionId:
       * 'st-nexus-docs', title: 'Download Nexus', href:
       * 'http://nexus.sonatype.org/using/download.html' } ); }, See
       * Sonatype.repoServer.RepoServer.addNexusNavigationItems() for more
       * examples.
       */
      'nexusNavigationInit' : true,

      /*
       * Fired when a repository context action menu is initialized.
       * Subscribers can add action items to the menu. A menu object and
       * repository record are passed as parameters. If clicked, the action
       * handler receives a repository record as parameter. init: function() {
       * Sonatype.Events.addListener( 'repositoryMenuInit', this.onRepoMenu,
       * this ); }, onRepoMenu: function( menu, repoRecord ) { if (
       * repoRecord.get( 'repoType' ) == 'proxy' ) { menu.add(
       * this.actions.myProxyAction ); } },
       */
      'repositoryMenuInit' : true,

      /*
       * Fired when an repository item (e.g. artifact or folder) context
       * action menu is initialized. Subscribers can add action items to the
       * menu. A menu object, a repository and item records are passed as
       * parameters. If clicked, the action handler receives an item record as
       * parameter. init: function() { Sonatype.Events.addListener(
       * 'repositoryContentMenuInit', this.onArtifactMenu, this ); },
       * onRepoMenu: function( menu, repoRecord, contentRecord ) { if (
       * repoRecord.get( 'repoType' ) == 'proxy' ) { menu.add(
       * this.actions.myProxyContentAction ); } },
       */
      'repositoryContentMenuInit' : true,

      /*
       * Fired when a user action menu is initialized (most likely an admin
       * function). Subscribers can add action items to the menu. A menu
       * object and a user record are passed as parameters. If clicked, the
       * action handler receives an user record as parameter. init: function() {
       * Sonatype.Events.addListener( 'userMenuInit', this.onUserMenu, this ); },
       * onUserMenu: function( menu, userRecord ) { if ( userRecord.get(
       * 'userId' ) != 'anonymous' ) { menu.add( this.actions.myUserAction ); } },
       */
      'userMenuInit' : true,

      /*
       * Fired when a privilege formPanel initializes Subscribers can mangle
       * the PrivilegeEditor as they see fit init: function() {
       * Sonatype.Events.addListener( 'privilegeEditorInit',
       * this.onPrivilegeEditorInit, this ); }, onPrivilegeEditorInit:
       * function( editor ) { editor.mangle(); },
       */
      'privilegeEditorInit' : true,

      /*
       * Fired when a privilege panel initializes Subscribers can mangle the
       * PrivilegePanel as they see fit init: function() {
       * Sonatype.Events.addListener( 'privilegePanelInit',
       * this.onPrivilegePanelInit, this ); }, onPrivilegePanelInit: function(
       * panel ) { panel.mangle(); },
       */
      'privilegePanelInit' : true
    });
  };
  Ext.extend(Nexus.util.Observable, Ext.util.Observable);
  return Nexus;
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Nexus, Sonatype*/
(function() {

  var
        utils = Ext.namespace('Sonatype.utils'),
        passwordPlaceholder = '|$|N|E|X|U|S|$|',
        idPattern = /^[a-zA-Z0-9_\-\.]+$/;

  Ext.define('Nexus.util.Strings', {
          statics : {
            lowercase : function(str) {
              if (Ext.isEmpty(str))
              {
                return str;
              }
              str = str.toString();
              return str.toLowerCase();
            },
            sortFn : function(r1, r2) {
              var v1 = Nexus.util.Strings.lowercase(r1), v2 = Nexus.util.Strings.lowercase(r2);
              return v1 > v2 ? 1 : (v1 < v2 ? -1 : 0);
            },
            lowercaseFirstChar : function(str) {
              if (Ext.isEmpty(str))
              {
                return str;
              }
              str = str.toString();
              return str.charAt(0).toLowerCase() + str.slice(1);
            },
            upperFirstCharLowerRest : function(str) {
              if (Ext.isEmpty(str))
              {
                return str;
              }
              str = str.toString();
              return str.charAt(0).toUpperCase() + str.slice(1).toLowerCase();
            },
            uppercase : function(str) {
              if (Ext.isEmpty(str))
              {
                return str;
              }
              str = str.toString();
              return str.toUpperCase();
            },
            capitalize : function(str) {
              if (Ext.isEmpty(str))
              {
                return str;
              }
              str = str.toString();
              return str.charAt(0).toUpperCase() + str.slice(1);
            },
            returnEmptyStr : function() {
              return '';
            },
            returnValidStr : function(str) {
              if (str)
              {
                return str;
              }

              return Nexus.util.Strings.returnEmptyStr();
            },
            validateNoSpaces : function(value) {
              if (value && value.indexOf(' ') !== -1)
              {
                return 'Spaces are not allowed in ID';
              }

              return true;
            },

            validateId : function(value) {
              if (idPattern.test(value))
              {
                return true;
              }

              return 'Only letters, digits, underscores(_), hyphens(-), and dots(.) are allowed in ID';
            },
            convert : {
              stringContextToBool : function(str) {
                return (str.toLowerCase() === 'true');
              },
              passwordToString : function(str) {
                if (passwordPlaceholder === str)
                {
                  return null;
                }

                if (str)
                {
                  return str;
                }

                return Nexus.util.Strings.returnEmptyStr();
              }
            }

          }
        }, function() {
          // Add string util methods to Sonatype.utils for backward compat
          Ext.applyIf(Sonatype.utils, Nexus.util.Strings);
        }
  );
}());

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Helper to facilitate browser-based file downloads.
 *
 * @since 2.7
 */
NX.define('Nexus.util.DownloadHelper', {
  singleton: true,

  mixins: [
    'Nexus.LogAwareMixin'
  ],

  /**
   * @private
   *
   * ExtJS component identifier for nested iframe.
   */
  windowId: 'nx-download-frame',

  /**
   * @private
   *
   * Window names in IE are very picky, using '_' instead of '-' so that its its a valid javascript identifier.
   */
  windowName: 'nx_download_frame',

  /**
   * Get the hidden download frame.
   *
   * @private
   * @returns {Ext.Element}
   */
  getFrame: function() {
    var me = this, frame;

    // create the download frame if needed
    frame = Ext.get(me.windowId);
    if (!frame) {
      frame = Ext.getBody().createChild({
        tag: 'iframe',
        cls: 'x-hidden',
        id: me.windowId,
        name: me.windowName
      });
      me.logDebug('Created download-frame: ' + frame);
    }

    return frame;
  },

  /**
   * @public
   * @param {String} url URL to download
   * @return {Boolean} false if download was blocked, else true
   */
  downloadUrl: function (url) {
    var me = this,
        frame,
        win;

    me.logDebug('Downloading URL: ' + url);

    // resolve the download frame
    frame = me.getFrame();

    // TODO: Consider changing this to a dynamic form or 'a' link and automatically submit/click
    // TODO: ... to make use of html5 download attribute and avoid needing to _open_ more windows
    // TODO: Form method could be handy to GET/POST w/params vs link to just GET?

    if (XMLHttpRequest.tokenName) {
      url += (url.indexOf('?') > 0 ? '&' : '?') + XMLHttpRequest.tokenName + '=' + XMLHttpRequest.tokenValue();
    }
    // open new window in hidden download-from to initiate download
    win = NX.global.open(url, me.windowName);
    if (win == null) {
      alert('Download window pop-up was blocked!');
      return false;
    }
    else {
      return true;
    }
  }
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/configuration/Ajax', ['extjs'], function(Ext) {
  Ext.Ajax.defaultHeaders = {
    'accept' : 'application/json,application/vnd.siesta-error-v1+json,application/vnd.siesta-validation-errors-v1+json',

    // HACK: Setting request header to allow analytics to tell if the request came from the UI or not
    // HACK: This has some issues, will only catch ajax requests, etc... but may be fine for now
    'X-Nexus-UI': 'true'
  };

  Ext.Ajax.on('beforerequest', function (connection, options) {
    if (options.isUpload && options.url && XMLHttpRequest.tokenName) {
      options.url = options.url
          + (options.url.indexOf('?') > 0 ? '&' : '?') + XMLHttpRequest.tokenName + '=' + XMLHttpRequest.tokenValue();
    }
  });

  Ext.Ajax.on('requestexception', function(connection, response) {
    if ( response && Ext.isFunction(response.getResponseHeader) ) { // timeouts/socket closed response does not have this method(?)
      if (XMLHttpRequest.tokenName) {
        var tokenValue = response.getResponseHeader(XMLHttpRequest.tokenName);
        if (tokenValue) {
          XMLHttpRequest.tokenValue = function () {
            return tokenValue;
          };
        }
      }
      var contentType = response.getResponseHeader('Content-Type');
      if ( contentType === 'application/vnd.siesta-error-v1+json') {
        response.siestaError = Ext.decode(response.responseText);
      } else if ( contentType === 'application/vnd.siesta-validation-errors-v1+json') {
        response.siestaValidationError = Ext.decode(response.responseText);
      }
    }
  });

  Ext.Ajax.on('requestcomplete', function (connection, response) {
    if (response && Ext.isFunction(response.getResponseHeader)) {
      if (XMLHttpRequest.tokenName) {
        var tokenValue = response.getResponseHeader(XMLHttpRequest.tokenName);
        if (tokenValue) {
          XMLHttpRequest.tokenValue = function () {
            return tokenValue;
          };
        }
      }
    }
  });

  // Set default HTTP headers
  Ext.lib.Ajax.defaultPostHeader = 'application/json; charset=utf-8';
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/messagebox',['extjs', 'nexus', 'sonatype'], function(Ext, Nexus, Sonatype) {
  Sonatype.MessageBox = Ext.MessageBox;

  return Ext.MessageBox;
});



/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/navigation/menu', ['extjs', 'sonatype', 'ext/menu'], function(Ext, Sonatype, Menu){

Ext.namespace('Sonatype.menu');

Sonatype.menu.Menu = Ext.extend(Menu, {
  add : function(c) {
    if (!c) {
      return null;
    }

    var i, item, param, arr = null, a = arguments;

    if (a.length > 1)
    {
      arr = a;
    }
    else if (Ext.isArray(c))
    {
      arr = c;
    }
    if (arr)
    {
      for (i = 0; i < arr.length; i=i+1)
      {
        this.add(arr[i]);
      }
      return;
    }

    item = Sonatype.menu.Menu.superclass.add.call(this, c);
    param = c.payload || this.payload;
    if (c.handler && param)
    {
      // create a delegate to pass the payload object to the handler
      item.setHandler(c.handler.createDelegate(c.scope ? c.scope : this.scope, [param], 0));
    }
    return item;
  }
});
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/navigation/panel',['extjs', 'sonatype'], function(Ext, Sonatype){
Ext.namespace('Sonatype.navigation');

Sonatype.navigation.NavigationPanel = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  this.delayedItems = {};

  Sonatype.navigation.NavigationPanel.superclass.constructor.call(this, {
    cls : 'st-server-panel',
    autoScroll : true,
    border : false,
    items : []
  });
};

Ext.extend(Sonatype.navigation.NavigationPanel, Ext.Panel, {
  insert : function(sectionIndex, container) {
    if (!sectionIndex) {
      sectionIndex = 0;
    }
    if (!container) {
      return;
    }

    var panel;

    // check if this is an attempt to add a navigation item to an existing
    // section
    if (container.sectionId)
    {
      panel = this.findById(container.sectionId);
      if (panel)
      {
        return panel.insert(sectionIndex, container);
      }
      else
      {
        if (!this.delayedItems[container.sectionId])
        {
          this.delayedItems[container.sectionId] = [];
        }
        this.delayedItems[container.sectionId].push(container);
        return null;
      }
    }

    panel = new Sonatype.navigation.Section(container);
    panel = Sonatype.navigation.NavigationPanel.superclass.insert.call(this, sectionIndex, panel);
    if (panel.id && this.delayedItems[panel.id])
    {
      panel.add(this.delayedItems[panel.id]);
      this.delayedItems[panel.id] = null;
    }
    return panel;
  },
  add : function(c) {
    var i, arr = null, a = arguments, panel, newPanel;

    if (a.length > 1)
    {
      arr = a;
    }
    else if (Ext.isArray(c))
    {
      arr = c;
    }
    if (arr !== null)
    {
      for (i = 0; i < arr.length; i=i+1)
      {
        this.add(arr[i]);
      }
      return;
    }

    if (!c) {
      return;
    }

    // check if this is an attempt to add a navigation item to an existing
    // section
    if (c.sectionId)
    {
      panel = this.findById(c.sectionId);
      if (panel)
      {
        newPanel = panel.add(c);
        panel.sort();
        return newPanel;
      }
      else
      {
        if (!this.delayedItems[c.sectionId])
        {
          this.delayedItems[c.sectionId] = [];
        }
        this.delayedItems[c.sectionId].push(c);
        return null;
      }
    }

    panel = new Sonatype.navigation.Section(c);
    panel = Sonatype.navigation.NavigationPanel.superclass.add.call(this, panel);
    if (panel.id && this.delayedItems[panel.id])
    {
      panel.add(this.delayedItems[panel.id]);
      this.delayedItems[panel.id] = null;
      panel.sort();
    }
    return panel;
  }
});
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Nexus/navigation/section',['extjs', 'sonatype', 'Sonatype/strings'], function(Ext, Sonatype, Strings){
Ext.namespace('Sonatype.navigation');

Sonatype.navigation.Section = function(cfg) {
  var
        config = cfg || {},
        defaultConfig = {
          collapsible : true,
          titleCollapse : true,
          collapsed : false
        };

  if (config.items)
  {
    config.items = this.transformItem(config.items);
  }
  if (!config.items || config.items.length === 0)
  {
    config.hidden = true;
  }

  Ext.apply(this, config, defaultConfig);

  Sonatype.navigation.Section.superclass.constructor.call(this, {
        cls : 'st-server-sub-container',
        layout : 'fit',
        frame : true,
        autoHeight : true,
        hideMode : 'offsets',
        animCollapse : false,
        listeners : {
          collapse : this.collapseExpandHandler,
          expand : this.collapseExpandHandler,
          scope : this
        }
      });
};

Ext.extend(Sonatype.navigation.Section, Ext.Panel, {
      collapseExpandHandler : function() {
        if (this.layout && this.layout.layout)
        {
          this.ownerCt.doLayout();
        }
      },

      transformItem : function(c) {
        if (!c) {
          return null;
        }

        var i, item, c2;

        if (Ext.isArray(c))
        {
          c2 = [];
          for (i = 0; i < c.length; i=i+1)
          {
            item = this.transformItem(c[i]);
            if (item)
            {
              c2.push(item);
            }
          }
          return c2;
        }

        if (!c.xtype)
        {
          if (c.href)
          {
            // regular external link
            return {
              sortable_title : c.title,
              autoHeight : true,
              html : '<ul class="group-links"><li><a href="' + c.href + '" target="' + c.href + '"' +
                    (c.style ? ' style="' + c.style + '"' : '') +
                    '>' + c.title + '</a></li></ul>'
            };
          }
          else if (c.tabCode || c.handler)
          {

            if (Sonatype.view.supportedNexusTabs)
            {
              Sonatype.view.supportedNexusTabs[c.tabId] = true;
            }
            // panel open action
            return (c.enabled === false) ? null : {
              sortable_title : c.title,
              autoHeight : true,
              id : 'navigation-' + c.tabId,
              initialConfigNavigation : c,
              listeners : {
                render : {
                  fn : function(panel) {
                    panel.body.on('click', Ext.emptyFn, null, {
                          delegate : 'a',
                          preventDefault : true
                        });
                    panel.body.on('mousedown', function(e, target) {
                          e.stopEvent();
                          if (c.handler)
                          {
                            c.handler();
                          }
                          else
                          {
                            Sonatype.view.mainTabPanel.addOrShowTab(c.tabId, c.tabCode, {
                                  title : c.tabTitle || c.title
                                });
                          }
                        }, c.scope, {
                          delegate : 'a'
                        });
                  },
                  scope : this
                }
              },
              html : '<ul class="group-links"><li><a href="#">' + c.title + '</a></li></ul>'
            };
          }
        }
        return c;
      },

      add : function(c) {
        var
              i, arr = null, a = arguments;
        if (a.length > 1)
        {
          arr = a;
        }
        else if (Ext.isArray(c))
        {
          arr = c;
        }
        if (arr !== null)
        {
          for (i = 0; i < arr.length; i=i+1)
          {
            this.add(arr[i]);
          }
          return;
        }

        c = this.transformItem(c);
        if (!c) {
          return;
        }

        if (this.hidden)
        {
          this.show();
        }
        return Sonatype.navigation.Section.superclass.add.call(this, c);
      },

      sort : function(asOrder) {
        if(!this.items)
        {
            return;
        }
          
        var _fSorter = function(obj1, obj2) {
           var fieldName = "sortable_title";
           return Strings.sortFn(obj1[fieldName], obj2[fieldName]);
        };
        this.items.sort(asOrder || 'ASC', _fSorter);
      }
    });

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Controller.
 *
 * @since 2.7
 */
NX.define('Nexus.controller.Controller', {
  mixins: [
    'Nexus.LogAwareMixin'
  ],

  control: function (config) {
    var me = this;
    Ext.iterate(config, function (key) {
      if (key.startsWith('#')) {
        // closure to register control events on given object
        function register(obj) {
          var events = config['#' + obj.id];

          Ext.iterate(events, function (event) {
            obj.on(event, events[event], me);
            me.logDebug('Registered for event "' + event + '" on ' + obj.id);
          });
        }

        var id = key.substring(1),
            obj = Ext.ComponentMgr.get(id);

        // If component already exists, register events on it
        if (Ext.isObject(obj)) {
          register(obj);
        }
        else {
          // else when the component is created we will register events then
          Ext.ComponentMgr.onAvailable(id, register, me);
        }
      }
    });
  },

  /**
   * Parses an error message out of an Ajax response.
   * @param response Ajax response
   * @param {String} [defaultMessage] to be used if a message could not be extracted from response
   * @returns {String} parsed message
   */
  parseExceptionMessage: function (response, defaultMessage) {
    var message;
    if (response.siestaError) {
      message = response.siestaError.message;
    }
    if (Ext.isEmpty(message) && response.responseText) {
      message = Sonatype.utils.parseHTMLErrorMessage(response.responseText);
    }
    if (Ext.isEmpty(message)) {
      message = response.statusText;
    }
    if (Ext.isEmpty(message)) {
      message = defaultMessage;
    }
    if (Ext.isEmpty(message)) {
      message = 'Could not be determined';
    }
    return message;
  }

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX */

define('NX/assert', [], function() {

  Ext.ns('NX');

  /**
   * Exception thrown when assertion is triggered.
   *
   * @param message
   * @constructor
   *
   * @since 2.4
   */
  function AssertError(message) {
    this.name = 'AssertError';
    this.message = message || 'Assertion failure';
  }

  AssertError.prototype = new Error();
  AssertError.prototype.constructor = AssertError;

  /**
   * Assertion helpers.
   */
  Ext.apply(NX, {

    /**
     * Assert a condition.  When the condition expression is false an exception is thrown.
     *
     * @param expression    Condition expression
     * @param message       Exception message
     *
     * @since 2.4
     */
    assert : function(expression, message) {
      if (!expression) {
        throw new AssertError(message);
      }
    }

  });

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX,Nexus */

define('NX/log', ['NX/base'], function() {
  Ext.ns('NX');

  /**
   * Logging helper.
   *
   * @singleton
   */
  NX.log = (function () {
      var logger;

      // FIXME: Really would like a better/more-powerful logging API here
      // FIXME: Including the ability to remote back to the server to capture UI events
      //
      // http://log4javascript.org/index.html
      // http://www.gscottolson.com/blackbirdjs
      // http://log4js.berlios.de

      logger = {
          enabled: true,

          levels: {
              trace:  false,
              debug:  false,
              group:  false,
              info:   true,
              warn:   true,
              error:  true
          },

          isEnabled: function(level) {
              return this.enabled && (this.levels[level] || NX.global.location.search === '?debug');
          }

          // FIXME: Add ExtJS 4.x compatible(ish) log() helper
          //log: function(/*[options],[message]*/) {
          //}
      };

      /**
       * @private
       * @param target {Object} The logging implementation
       * @param name {String} The logger to use
       * @param isLoggerEnabled {Function} A function that returns true when the logger for target/name is enabled, false otherwise.
       * @return {Function} The logger.
       */
      function createLogger(target, name, isLoggerEnabled) {
        var log;
        if (Ext.isDefined(target)) {
          if (Ext.isFunction(target[name])) { // most browser have proper console.log, console.debug, ... as functions
            log = function() {
              if (isLoggerEnabled()) {
                target[name].apply(target, arguments);
              }
            };
          } else if (Ext.isDefined(target[name])) { // IE9: console.log is reported as an object, not a function, and does not have #apply
            log = function() {
              if (isLoggerEnabled()) {
                var args = [];
                Ext.each(arguments, function(item) {
                  args.push(item);
                });
                target[name](args.join(' '));
              }
            };
          } else if (Ext.isDefined(target.log)) { // if e.g. console.debug is not available, use console.log
            log = createLogger(target, 'log', isLoggerEnabled);
          } else { // nothing we can do, don't log.
            log = Ext.emptyFn;
          }
        }
        return log;
      }

      function safeProxy(target, name) {
        return createLogger(target, name, function() {
          return logger.enabled;
        });
      }

      function levelAwareProxy(target, name) {
        return createLogger(target, name, function() {
          return logger.isEnabled(name);
        });
      }

      if ( !NX.global.console ) { // IE without webdev has no console object, so we create an empty one
        NX.global.console = {
          log : Ext.emptyFn
        };
      } else if ( !NX.global.console.log ) { // We rely on existence of console.log below
        NX.global.console.log = Ext.emptyFn;
      }

      Ext.each([
          'trace',
          //'log', skipping; as we may want to make an Ext.log compatible method
          'debug',
          'group',
          'info',
          'warn',
          'error'
      ], function (name) {
          logger[name] = levelAwareProxy(NX.global.console, name);
      });

      Ext.each([
          'groupEnd'
      ], function (name) {
          logger[name] = safeProxy(NX.global.console, name);
      });

      return logger;
  }());

  NX.log.debug('Logging initialized');

  // Compatibility
  Ext.ns('Nexus');
  Nexus.Log = NX.log;
  Nexus.log = NX.log.debug;

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX */

define('NX/define', ['NX/base', 'NX/assert', 'NX/log'], function() {
  Ext.ns('NX');

  /**
   * Used as the super-class for all NX.define classes which not not provide a class to extend.
   * Do not put anything special in here, since not everything will use this as its super-class.
   *
   * @constructor
   *
   * @private
   */
  NX.Base = function (config) {
      Ext.apply(this, config);
  };

  /**
   * Define a new class, in Extjs-4 compatible manner.
   *
   * @static
   *
   * @param {String} className        The name of the class to define.
   * @param {Object} data             Configuration for the class.
   * @param {Function} [createdFn]    Function to execute when class has been defined.
   *
   * @cfg {String} extend             Super-class name.
   * @cfg {*} statics                 Static members.
   * @cfg {Array} mixins              Mixin class names, does not support object configuration like extjs4 ATM.
   * @cfg {*} requirejs               Array of requirejs module dependencies, custom... defs not extjs4 compatible.
   * @cfg {*} requires                Array of class names (created by Ext.define) which are dependencies.
   * @cfg {boolean} requireSuper      Flag to enable/disable automatic dependency on super-class;
   *                                  Tries to auto-detect but just inc-ase you can enable/disable it.
   * @cfg {String} xtype              Automatically register class as Ext xtype component.
   * @cfg {boolean} singleton         Defines a singleton instead of a class.
   *                                  Given class name will reference singleton object and createdFn 'this' will be the singleton object.
   */
  NX.define = function (className, data, createdFn) {
      data = data || {};

      var i,
          nameSpace,
          simpleClassName,
          superName,
          type,
          requireSuper,
          superClass,
          statics,
          mixins,
          mixin,
          requiredClassNames,
          requiredModulePaths,
          moduleName,
          singleton,
          xtype,
          tmp;

      // Find the namespace (if any) for the new class
      i = className.lastIndexOf('.');
      if (i !== -1) {
          nameSpace = className.substring(0, i);
          simpleClassName = className.substring(i + 1);
      }
      else {
          simpleClassName = className;
      }

      requireSuper = data.requireSuper;
      delete data.requireSuper;

      // Determine the super-class
      if (data.extend !== undefined) {
          superName = data.extend;
          delete data.extend;

          // require super module if there is not already a defined class of that name for legacy support
          if (requireSuper === undefined) {
              requireSuper = !NX.isobj(superName);
          }
      }
      else {
          superName = 'NX.Base';
          requireSuper = false;
      }

      statics = data.statics;
      delete data.statics;

      mixins = data.mixins;
      delete data.mixins;

      singleton = data.singleton;
      delete data.singleton;

      xtype = data.xtype;
      delete data.xtype;

      // Extract requirejs dependencies
      requiredModulePaths = NX.arrayify(data.requirejs);
      delete data.requirejs;

      // Extract class dependencies (which were defined using Ext.define)
      requiredClassNames = NX.arrayify(data.requires);
      delete data.requires;

      // Require super if enabled
      if (requireSuper === true) {
          requiredClassNames.push(superName);
      }

      // Require mixins
      if (mixins !== undefined) {
          for(i=0; i<mixins.length; i++) {
              requiredClassNames.push(mixins[i]);
          }
      }

      // append translated dependency classes on to required modules
      for (i=0; i < requiredClassNames.length; i++) {
          tmp = requiredClassNames[i].replaceAll('.', '/');
          requiredModulePaths.push(tmp);
      }

      // Translate class name into module name
      moduleName = className.replaceAll('.', '/');

      if (requiredModulePaths.length !== 0) {
          NX.log.debug('Defining module:', moduleName, 'depends:', requiredModulePaths);
      }
      else {
          NX.log.debug('Defining module:', moduleName);
      }

      define(moduleName, requiredModulePaths, function()
      {
          NX.log.debug('Defining class:', className, 'super:', superName);

          // Sanity check required classes exist
          Ext.each(requiredClassNames, function(className) {
              NX.assert(NX.isobj(className), 'Missing required class: ' + className);
          });

          // Create namespace if required
          if (nameSpace) {
              Ext.namespace(nameSpace);
          }

          // Get a reference to the super-class
          superClass = NX.obj(superName);

          // When no constructor given in configuration (its always there due to picking up from Object.prototype), use a synthetic version
          if (data.constructor === Object.prototype.constructor) {
              data.constructor = function () {
                  superClass.prototype.constructor.apply(this, arguments);
              };
          }

          // Create the sub-class
          type = Ext.extend(superClass, data);

          // Remember class name (full and simple)
          type.$className = className;
          type.$simpleClassName = simpleClassName;
          type.prototype.$className = className;
          type.prototype.$simpleClassName = simpleClassName;

          // replace toString if its the default Object.toString
          if (type.prototype.toString === Object.prototype.toString) {
              type.prototype.toString = function() {
                  return '[object ' + className + ']';
              };
          }

          // Handle mixins
          if (mixins !== undefined) {
              for(i=0; i<mixins.length; i++) {
                  mixin = mixins[i]; // name
                  NX.log.debug('Applying mixin:', mixin);
                  mixin = NX.obj(mixin); // class ref
                  Ext.applyIf(type.prototype, mixin.prototype);
              }
          }

          // Apply any static members
          if (statics !== undefined) {
              Ext.apply(type, statics);
          }

          // Register xtype for class
          if (xtype !== undefined) {
              Ext.reg(xtype, type);
          }


          // When singleton; type becomes new instance
          // NOTE: `type` is passed into the creation callback function later on, so the variable must hold the actual instance for singletons
          if (singleton !== undefined) {
              type = new type();
          }

          // Assign to global namespace
          NX.obj(nameSpace)[simpleClassName] = type;


          // Call post-define hook
          if (createdFn !== undefined) {
              // Scope to created type, empty args seems to be required here
              createdFn.call(type, []);
          }

          return type;
      });
  };

  // Compatibility
  Ext.define = NX.define;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('NX/bootstrap', [
  'NX/base',
  'NX/define',
  'NX/log'
]);
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define,Ext,NX,global */

define('NX/base', ['require'], function(require) {
  Ext.ns('NX');

  /**
   * Basic framework helpers.
   *
   * @singleton
   */
  Ext.apply(NX, {
      /**
       * Global object reference; never undefined.
       *
       * @type {Object}
       */
      global: (function() {
          if (window !== undefined) {
              return window;
          }
          if (global !== undefined) {
              return global;
          }
          throw new Error('Unable to determine global object');
      }()), // assign return value of function

      /**
       * Reference an object by its global name.
       *
       * @param path
       * @return {*}
       * @throws Error    No object at path
       */
      obj: function (path) {
          var context = NX.global;
          if (typeof path === 'string') {
            Ext.each(path.split('.'), function (part) {
                context = context[part];
                if (context === undefined) {
                    throw new Error('No object at path: ' + path + '; part is undefined: ' + part);
                }
            });
          }
          return context;
      },

      /**
       * Check if an object exists by its global name.
       *
       * @param path
       * @return {boolean}    True if it exists, false if it does not.
       */
      isobj: function(path) {
          var context = NX.global;
          Ext.each(path.split('.'), function (part) {
              context = context[part];
              if (context === undefined) {
                  return false; // break
              }
          });
          return context !== undefined;
      },

      /**
       * Turns input into an array of strings.
       *
       * @param input
       * @return {Array}
       */
      arrayify: function (input) {
          var i, list = [];

          if (Ext.isArray(input)) {
              for (i = 0; i < input.length; i++) {
                  if (Ext.isString(input[i])) {
                      list.push(input[i]);
                  }
                  else {
                      throw new Error('Invalid entry: ' + input[i]);
                  }
              }
          }
          else if (Ext.isString(input)) {
              list.push(input);
          }
          else if (input !== undefined) {
              throw new Error('Invalid value: ' + input);
          }

          return list;
      },

      /**
       * Helper to construct an object from a constructor.
       *
       * @param {Function} constructor
       * @param {Array} args
       * @return {*}
       */
      construct: function (constructor, args) {
          function F() {
              return constructor.apply(this, args);
          }

          F.prototype = constructor.prototype;
          return new F();
      },

      /**
       * Create a new instance of a class.
       *
       * @param name
       * @return {*}
       */
      create: function (name /*, [arg1,arg2,...,argN] */) {
          var args,
              type;

          // pull out varargs, strip off class name
          args = Array.prototype.slice.call(arguments);
          args.shift();

          NX.log.debug('Creating instance:', name);

          // Get a reference to the class
          type = NX.obj(name);

          // Require the class if its not defined
          if (type === undefined) {
              type = require(name.replaceAll('.', '/'));
          }

          return NX.construct(type, args);
      }

  });
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

define('nexus', ['extjs'], function(Ext){

    Ext.namespace('Nexus');

    /**
     * Return the current user.
     *
     * @return {*}
     * @static
     */
    Nexus.currentUser = function () {
        return Sonatype.user.curr;
    };

    return window.Nexus;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define, Ext*/

// just give extjs file a name
define('extjs',[], function() {
  return Ext;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
 /*global define*/
define('sonatype', ['extjs'], function(Ext) {
  // Define all second level namespaces
  Ext.namespace('Sonatype', 'Sonatype.state', 'Sonatype.ext', 'Sonatype.lib', 'Sonatype.utils', 'Sonatype.config', 'Sonatype.user', 'Sonatype.resources', 'Sonatype.repoServer', 'Sonatype.repoServer.resources');

  return Sonatype;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Sonatype/utils',['../extjs', 'Nexus/config', 'Nexus/util/Format', 'Sonatype/strings', 'sonatype'], function(Ext, Config, format, Strings, Sonatype) {

  var ns = Ext.namespace('Sonatype.utils');

  Ext.applyIf(ns, {
    edition : '',
    editionShort : '',
    version : '',
    attributionsURL : '',
    purchaseURL : '',
    userLicenseURL : '',

    // deep copy of an object. All references independent from object passed in.
    cloneObj : function(o) {
      if (typeof(o) !== 'object' || o === null)
      {
        return o;
      }

      var i, newObj = {};

      for (i in o)
      {
        if (o.hasOwnProperty(i)) {
          newObj[i] = ns.cloneObj(o[i]);
        }
      }

      return newObj;
    },

    // (Array : arr, string : child, [string seperator])
    // array to join, name of element of contained object, seperator (defaults
    // to ", ")
    joinArrayObject : function(arr, child, seperator) {
      var i, sOut = '', sep = seperator || ', ';

      for (i = 0; i < arr.length; i=i+1)
      {
        if ((arr[i])[child])
        {
          sOut += (arr[i])[child] + sep;
        }
      }

      if (sOut !== "")
      {
        return sOut.substring(0, sOut.length - sep.length);
      }
      else
      {
        return sOut;
      }
    },

    updateGlobalTimeout : function() {
      Ext.Ajax.request({
            method : 'GET',
            scope : this,
            url : Config.repos.urls.restApiSettings,
            callback : function(options, success, response) {
              if (success)
              {
                var dec = Ext.decode(response.responseText);
                Ext.Ajax.timeout = dec.data.uiTimeout * 1000;
              }
              else
              {
                Ext.Ajax.timeout = 60000;
                ns.connectionError(response, 'Error retrieving rest timeout');
              }
            }
          });
    },

    parseHTMLErrorMessage : function(text) {
      var
            n1 = text.toLowerCase().indexOf('<p>') + 3,
            n2 = text.toLowerCase().indexOf('</p>');

      if (n2 > n1) {
        return text.substring(n1, n2);
      }

      return '';
    },

    /**
     * Show a message box with content according to the response.
     *
     * If the login window is visible, this method will look at the X-Nexus-Reason error
     * and offer a password change if that header content is 'expired'.
     *
     * Used options:
     * # options.decodeErrorResponse: Try to parse the responseText as a JSON error response, or a HTML error page.
     * # options.hideErrorStatus: If a message could be taken out of the responseText, HTTP status code and reason will
     *                            not be included in the message.
     *
     * @param response The response object of the request.
     * @param message The message to show.
     * @param offerRestart Whether to offer a reload of the UI.
     * @param options The options object of the request.
     * @param showResponseText Show the plain responseText for 'Bad Request' errors.
     *
     */
    connectionError : function(response, message, offerRestart, options, showResponseText) {
      // prime options object if necessary
      options = options || {};

      var
            i, errorResponse,
            serverMessage = '',
            r = response.responseText,
            displayMessage;

      if (r)
      {
        if (options.decodeErrorResponse) {
          var contentType = response.getResponseHeader("Content-Type");
          if( contentType === "application/vnd.siesta-error-v1+json") {
            // response will be a json serialized org.sonatype.sisu.siesta.common.error.ErrorXO
            serverMessage = Ext.decode(r).message;
          } else if( contentType === "application/vnd.siesta-validation-errors-v1+json") {
            // response will be a json serialized array of org.sonatype.sisu.siesta.common.validation.ValidationErrorXO
            validationErrors = Ext.decode(r);
            for (i = 0; i < validationErrors.length; i=i+1)
            {
              if(serverMessage.length > 0) {
                serverMessage += '<br /><br />';
              }
              serverMessage += validationErrors[i].message;
            }
          } else if(r.toLowerCase().indexOf('"errors"') > -1) {
            // last, check if we have an json serialized restlet1x org.sonatype.plexus.rest.resource.error.ErrorResponse
            errorResponse = Ext.decode(r);
            for (i = 0; i < errorResponse.errors.length; i=i+1)
            {
              if(serverMessage.length > 0) {
                serverMessage += '<br /><br />';
              }
              serverMessage += errorResponse.errors[i].msg;
            }
          }
        }
        else
        {
          serverMessage = ns.parseHTMLErrorMessage(r);
        }
      }

      if (Sonatype.repoServer.RepoServer.loginWindow.isVisible() && (response.status == 403 || response.status == 401))
      {
        if (Sonatype.repoServer.RepoServer.loginWindow.isVisible())
        {
          var nexusReason = response.getResponseHeader['X-Nexus-Reason'];
          if (nexusReason && nexusReason.substring(0, 7) == 'expired')
          {
            Sonatype.repoServer.RepoServer.loginWindow.hide();
            ns.changePassword(Sonatype.repoServer.RepoServer.loginForm.find('name', 'username')[0].getValue());
          }
          else
          {
            Ext.MessageBox.show({
                  title : 'Login Error',
                  msg : 'Incorrect username, password or no permission to use the Nexus User Interface.<br />Try again.' + serverMessage,
                  buttons : Ext.MessageBox.OK,
                  icon : Ext.MessageBox.ERROR,
                  animEl : 'mb3',
                  fn : this.focusPassword
                });
          }
        }
      }
      else
      {
        if (message == null)
        {
          message = '';
        }
        if (serverMessage)
        {
          message += serverMessage;
        }

        var responseStatus = response.status;
        var responseStatusText = response.statusText;

        // the prompt is in the following format

        // <message> (optional)
        //
        // <response text> (if 400)
        //
        // <detail> (include status code if valid)
        //
        // <retry message> (optional)

        displayMessage = '';

        // caller provided message + serverMessage
        // status == -1 is request timed out (?), don't show message then
        if (message && responseStatus != -1) {
          displayMessage = message;
        }

        // show response text if requested (and makes sense, status 400)
        if (responseStatus == 400 && showResponseText) {
          if (displayMessage.length > 0) {
            displayMessage += '<br/><br/>';
          }
          displayMessage += response.responseText;
        }
        // otherwise show statusLine if not requested to hide and we have a serverMessage
        // (we want to always provide some kind of server message, either error msg or status line)
        else {
          if (!(options.hideErrorStatus && serverMessage)) {
            if (displayMessage.length > 0) {
              displayMessage += '<br/><br/>';
            }
            // hide -1 status
            if (responseStatus && responseStatus != -1) {
              displayMessage += 'Nexus returned an error: ERROR ' + responseStatus + ': ' + responseStatusText;
            }
            else {
              displayMessage += 'There was an error communicating with the server: request timed out.';
            }
          }
        }

        if (offerRestart) {
          if (displayMessage.length > 0) {
            displayMessage += '<br/><br/>';
          }
          displayMessage += 'Click OK to reload the console or CANCEL if you wish to retry the same action in a little while.';
        }

        Ext.MessageBox.show({
          title: "Error",
          msg: displayMessage,
          buttons: offerRestart ? Ext.MessageBox.OKCANCEL : Sonatype.MessageBox.OK,
          icon: Ext.MessageBox.ERROR,
          animEl: 'mb3',
          fn: function (button) {
            if (offerRestart && button == "ok") {
              window.location.reload();
            }
          }
        });
      }
    },
    /**
     * Call this after the error signing in dialog appears. Otherwise focus
     * doesn't get put in the password field correctly.
     */
    focusPassword : function() {
      if (Sonatype.repoServer.RepoServer.loginWindow.isVisible())
      {
        Sonatype.repoServer.RepoServer.loginForm.find('name', 'password')[0].focus(true);
      }
    },
    /**
     * Base64 encode / decode http://www.webtoolkit.info/
     */
    base64 : function() {
      // private property
      var _keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";

      // private method for UTF-8 encoding
      var _utf8_encode = function(string) {
        string = string.replace(/\r\n/g, "\n");
        var utftext = "";

        for (var n = 0; n < string.length; n++)
        {
          var c = string.charCodeAt(n);

          if (c < 128)
          {
            utftext += String.fromCharCode(c);
          }
          else if ((c > 127) && (c < 2048))
          {
            utftext += String.fromCharCode((c >> 6) | 192);
            utftext += String.fromCharCode((c & 63) | 128);
          }
          else
          {
            utftext += String.fromCharCode((c >> 12) | 224);
            utftext += String.fromCharCode(((c >> 6) & 63) | 128);
            utftext += String.fromCharCode((c & 63) | 128);
          }
        }

        return utftext;
      };

      // private method for UTF-8 decoding
      var _utf8_decode = function(utftext) {
        var string = "";
        var i = 0;
        var c = c1 = c2 = 0;

        while (i < utftext.length)
        {
          c = utftext.charCodeAt(i);

          if (c < 128)
          {
            string += String.fromCharCode(c);
            i++;
          }
          else if ((c > 191) && (c < 224))
          {
            c2 = utftext.charCodeAt(i + 1);
            string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
            i += 2;
          }
          else
          {
            c2 = utftext.charCodeAt(i + 1);
            c3 = utftext.charCodeAt(i + 2);
            string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
            i += 3;
          }
        }

        return string;
      };

      return {
        // public method for encoding
        encode : function(input) {
          var output = "";
          var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
          var i = 0;

          input = _utf8_encode(input);

          while (i < input.length)
          {
            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);

            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;

            if (isNaN(chr2))
            {
              enc3 = enc4 = 64;
            }
            else if (isNaN(chr3))
            {
              enc4 = 64;
            }

            output = output + _keyStr.charAt(enc1) + _keyStr.charAt(enc2) + _keyStr.charAt(enc3) + _keyStr.charAt(enc4);
          }

          return output;
        },

        // public method for decoding
        decode : function(input) {
          var output = "";
          var chr1, chr2, chr3;
          var enc1, enc2, enc3, enc4;
          var i = 0;

          input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

          while (i < input.length)
          {
            enc1 = _keyStr.indexOf(input.charAt(i++));
            enc2 = _keyStr.indexOf(input.charAt(i++));
            enc3 = _keyStr.indexOf(input.charAt(i++));
            enc4 = _keyStr.indexOf(input.charAt(i++));

            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;

            output = output + String.fromCharCode(chr1);

            if (enc3 != 64)
            {
              output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64)
            {
              output = output + String.fromCharCode(chr3);
            }
          }

          output = _utf8_decode(output);

          return output;
        }
      }
    }(),

    defaultToNo : function() {
      // @note: this handler selects the "No" button as the default
      // @todo: could extend Ext.MessageBox to take the button to select as
      // a param
      Ext.MessageBox.getDialog().on('show', function() {
            this.focusEl = this.buttons[2]; // ack! we're offset dependent here
            this.focus();
          }, Ext.MessageBox.getDialog(), {
            single : true
          });
    },

    getCookie : function(cookieName) {
      var c = document.cookie + ";";
      var re = /\s?(.*?)=(.*?);/g;
      var matches;
      while ((matches = re.exec(c)) != null)
      {
        if (matches[1] == cookieName)
        {
          return matches[2];
        }
      }
      return null;
    },

    setCookie : function(cookieName, value) {
      document.cookie = cookieName + "=" + value + "; path=" + Sonatype.config.resourcePath
    },

    clearCookie : function(cookieName) {
      document.cookie = cookieName + "=null; expires=Thu, 01-Jan-70 00:00:01 GMT" + "; path=" + Sonatype.config.resourcePath
    },

    recoverUsername : function() {
      var w = new Ext.Window({
            title : 'Username Recovery',
            closable : true,
            autoWidth : false,
            width : 300,
            autoHeight : true,
            modal : true,
            constrain : true,
            resizable : false,
            draggable : false,
            items : [{
                  xtype : 'form',
                  labelAlign : 'right',
                  labelWidth : 60,
                  frame : true,
                  defaultType : 'textfield',
                  monitorValid : true,
                  items : [{
                        xtype : 'panel',
                        style : 'padding-left: 70px; padding-bottom: 10px',
                        html : 'Please enter the e-mail address you used to register your account and we will send you your username.'
                      }, {
                        fieldLabel : 'E-mail',
                        name : 'email',
                        width : 200,
                        allowBlank : false
                      }],
                  buttons : [{
                        text : 'E-mail Username',
                        formBind : true,
                        scope : this,
                        handler : function() {
                          var email = w.find('name', 'email')[0].getValue();

                          Ext.Ajax.request({
                                scope : this,
                                method : 'POST',
                                jsonData : {
                                  data : {
                                    email : email
                                  }
                                },
                                url : Sonatype.config.repos.urls.usersForgotId + '/' + email,
                                success : function(response, options) {
                                  w.close();
                                  Ext.MessageBox.show({
                                        title : 'Username Recovery',
                                        msg : 'Username request completed successfully.' + '<br /><br />' + 'Check your mailbox, the username reminder should arrive in a few minutes.',
                                        buttons : Ext.MessageBox.OK,
                                        icon : Ext.MessageBox.INFO,
                                        animEl : 'mb3'
                                      });
                                },
                                failure : function(response, options) {
                                  var errorOptions = {
                                    hideErrorStatus : true
                                  };
                                  ns.connectionError(response, 'There is a problem retrieving your username.', false, errorOptions)
                                }
                              });
                        }
                      }, {
                        text : 'Cancel',
                        formBind : false,
                        scope : this,
                        handler : function() {
                          w.close();
                        }
                      }]
                }]
          });

      w.show();
    },

    recoverPassword : function() {
      var w = new Ext.Window({
            title : 'Password Recovery',
            closable : true,
            autoWidth : false,
            width : 300,
            autoHeight : true,
            modal : true,
            constrain : true,
            resizable : false,
            draggable : false,
            items : [{
                  xtype : 'form',
                  labelAlign : 'right',
                  labelWidth : 60,
                  frame : true,
                  defaultType : 'textfield',
                  monitorValid : true,
                  items : [{
                        xtype : 'panel',
                        style : 'padding-left: 70px; padding-bottom: 10px',
                        html : 'Please enter your username and e-mail address below. We will send you a new password.'
                      }, {
                        fieldLabel : 'Username',
                        name : 'username',
                        width : 200,
                        allowBlank : false
                      }, {
                        fieldLabel : 'E-mail',
                        name : 'email',
                        width : 200,
                        allowBlank : false
                      }],
                  buttons : [{
                        text : 'Reset Password',
                        formBind : true,
                        scope : this,
                        handler : function() {
                          var username = w.find('name', 'username')[0].getValue();
                          var email = w.find('name', 'email')[0].getValue();

                          Ext.Ajax.request({
                                scope : this,
                                method : 'POST',
                                jsonData : {
                                  data : {
                                    userId : username,
                                    email : email
                                  }
                                },
                                url : Sonatype.config.repos.urls.usersForgotPassword,
                                success : function(response, options) {
                                  w.close();
                                  Ext.MessageBox.show({
                                        title : 'Reset Password',
                                        msg : 'Password request completed successfully.' + '<br /><br />' + 'Check your mailbox, your new password should arrive in a few minutes.',
                                        buttons : Ext.MessageBox.OK,
                                        icon : Ext.MessageBox.INFO,
                                        animEl : 'mb3'
                                      });
                                },
                                failure : function(response, options) {
                                  var errorOptions = {
                                    hideErrorStatus : true
                                  };
                                  ns.connectionError(response, 'There is a problem resetting your password.', false, errorOptions)
                                }
                              });
                        }
                      }, {
                        text : 'Cancel',
                        formBind : false,
                        scope : this,
                        handler : function() {
                          w.hide();
                          w.close();
                        }
                      }]
                }]
          });

      w.show();
    },

    changePassword : function(expiredUsername) {
      var w = new Ext.Window({
            id : 'change-password-window',
            title : 'Change Password',
            closable : true,
            autoWidth : false,
            width : 350,
            autoHeight : true,
            modal : true,
            constrain : true,
            resizable : false,
            draggable : false,
            items : [{
                  xtype : 'form',
                  labelAlign : 'right',
                  labelWidth : 110,
                  frame : true,
                  defaultType : 'textfield',
                  monitorValid : true,
                  items : [{
                        xtype : 'panel',
                        style : 'padding-left: 70px; padding-bottom: 10px',
                        html : (expiredUsername ? 'Your password has expired, you need to reset it. ' : '') + 'Please enter your current password and then the new password twice to confirm.'
                      }, {
                        fieldLabel : 'Current Password',
                        inputType : 'password',
                        name : 'currentPassword',
                        width : 200,
                        allowBlank : false,
                        validateOnBlur : false
                      }, {
                        fieldLabel : 'New Password',
                        inputType : 'password',
                        name : 'newPassword',
                        width : 200,
                        allowBlank : false,
                        validateOnBlur : false
                      }, {
                        fieldLabel : 'Confirm Password',
                        inputType : 'password',
                        name : 'confirmPassword',
                        width : 200,
                        allowBlank : false,
                        validateOnBlur : false,
                        validator : function(s) {
                          var firstField = this.ownerCt.find('name', 'newPassword')[0];
                          if (firstField && firstField.getRawValue() != s)
                          {
                            return "Passwords don't match";
                          }
                          return true;
                        }
                      }],
                  buttons : [{
                        id : 'change-password-button',
                        text : 'Change Password',
                        formBind : true,
                        scope : this,
                        handler : function() {
                          var currentPassword = w.find('name', 'currentPassword')[0].getValue();
                          var newPassword = w.find('name', 'newPassword')[0].getValue();

                          Ext.Ajax.request({
                                scope : this,
                                method : 'POST',
                                cbPassThru : {
                                  newPassword : newPassword
                                },
                                jsonData : {
                                  data : {
                                    userId : expiredUsername ? expiredUsername : Sonatype.user.curr.username,
                                    oldPassword : currentPassword,
                                    newPassword : newPassword
                                  }
                                },
                                url : Sonatype.config.repos.urls.usersChangePassword,
                                success : function(response, options) {
                                  if (expiredUsername)
                                  {
                                    ns.doLogin(w, expiredUsername, newPassword);
                                    w.close();
                                  }
                                  else
                                  {
                                    w.close();
                                    Ext.MessageBox.show({
                                          title : 'Password Changed',
                                          msg : 'Password change request completed successfully.',
                                          buttons : Ext.MessageBox.OK,
                                          icon : Ext.MessageBox.INFO,
                                          animEl : 'mb3'
                                        });
                                  }
                                },
                                failure : function(response, options) {
                                  ns.connectionError(response, 'There is a problem changing your password.', false, { hideErrorStatus: true })
                                }
                              });
                        }
                      }, {
                        text : 'Cancel',
                        formBind : false,
                        scope : this,
                        handler : function() {
                          w.close();
                        }
                      }]
                }]
          });

      w.show();
    },

    getAuthToken : function(username, password) {
      return ns.base64.encode(username + ':' + password);
    },

    refreshTask : (function() {
      var config, running;

      running = null;

      config = {
        run : function() {
          Ext.Ajax.request({
            method : 'GET',
            options : {
              ignore401 : true
            },
            url : Sonatype.config.repos.urls.status,
            callback : function(options, success, response) {
              // do nothing, we only request to refresh session
              // but stop the task if something goes wrong (forbidden, not found, connection refused etc.)
              if (!success) {
                ns.refreshTask.stop();
              }
            }
          });
        },
        interval : 15 * 60 * 1000
      };

      // catch the case of page reload, which will leave the user logged in but not go through the login handler
      Sonatype.Events.addListener('nexusSettings', function () {
        ns.refreshTask.start();
      }, this);

      // public API
      return {
        stop : function() {
          if (running !== null) {
            Ext.TaskMgr.stop(running);
            running = null;
          }
        },
        start : function() {
          if (running === null && Sonatype.utils.settings.keepAlive) {
            running = Ext.TaskMgr.start(config);
          }
        }};
    })(),

    doLogin : function(activeWindow, username, password) {
      if (activeWindow)
      {
        activeWindow.getEl().mask("Logging you in...");
      }

      Ext.Ajax.request({
            method : 'GET',
            cbPassThru : {
              username : username
            },
            headers : {
              'Authorization' : 'Basic ' + ns.getAuthToken(username, password)
            }, // @todo: send HTTP basic auth data
            url : Sonatype.config.repos.urls.login,
            success : function(response, options) {
              if (activeWindow)
              {
                activeWindow.getEl().unmask();
              }

              if (Sonatype.repoServer.RepoServer.loginWindow.isVisible())
              {
                Sonatype.view.loginSuccessfulToken = Sonatype.view.afterLoginToken;
                Sonatype.repoServer.RepoServer.loginWindow.hide();
                Sonatype.repoServer.RepoServer.loginForm.getForm().reset();
              }

              ns.loadNexusStatus();
            },
            failure : function(response, options) {
              Ext.namespace('Sonatype.utils').refreshTask.stop();
              if (activeWindow)
              {
                activeWindow.getEl().unmask();
              }

              if (Sonatype.repoServer.RepoServer.loginWindow.isVisible())
              {
                Sonatype.repoServer.RepoServer.loginForm.find('name', 'password')[0].focus(true);
              }

              Ext.Ajax.request({
                  scope : this,
                  method : 'GET',
                  url : Sonatype.config.repos.urls.logout,
                  callback : function(options, success, response) {
                    Sonatype.view.justLoggedOut = true;
                  }
                });

            }

          });
    },

    parseFormattedAppName : function(formattedAppName) {
      return formattedAppName;
    },

    /**
     * Loads Nexus settings.
     */
    loadNexusSettings: function () {
      Ext.Ajax.request({
        method: 'GET',
        suppressStatus: true,
        url: Sonatype.config.contextPath + '/service/siesta/wonderland/settings',
        callback: function (options, success, response) {
          var respObj;

          ns.settings = {};
          if (success) {
            respObj = Ext.decode(response.responseText);

            Ext.each(respObj, function(entry){
              ns.settings[entry.key] = entry.value;
            });
          }
          ns.settings.keepAlive = ns.settings.keepAlive === 'true';
          Sonatype.Events.fireEvent('nexusSettings');
        }
      });
    },

    loadNexusStatus : function() {
      Sonatype.user.curr = ns.cloneObj(Sonatype.user.anon);

      Ext.Ajax.request({
            method : 'GET',
            options : {
              ignore401 : true
            },
            url : Sonatype.config.repos.urls.status,
            callback : function(options, success, response) {
              var baseUrlMismatch = false;

              Sonatype.view.historyDisabled = true;

              if (success)
              {
                var respObj = Ext.decode(response.responseText);

                ns.edition = respObj.data.editionLong;
                ns.editionShort = respObj.data.editionShort;
                ns.version = respObj.data.version;
                ns.attributionsURL = respObj.data.attributionsURL;
                ns.purchaseURL = respObj.data.purchaseURL;
                ns.userLicenseURL = respObj.data.userLicenseURL;
                ns.licenseInstalled = respObj.data.licenseInstalled;
                ns.licenseExpired = respObj.data.licenseExpired;
                ns.trialLicense = respObj.data.trialLicense;

                ns.formattedAppName = Ext.namespace('Sonatype.utils').parseFormattedAppName(respObj.data.formattedAppName);

                Ext.get('logo').update('<span>' + ns.formattedAppName + '</span>');

                Sonatype.view.headerPanel.doLayout();

                Sonatype.user.curr.repoServer = respObj.data.clientPermissions.permissions;
                Sonatype.user.curr.isLoggedIn = respObj.data.clientPermissions.loggedIn;
                Sonatype.user.curr.username = respObj.data.clientPermissions.loggedInUsername;
                Sonatype.user.curr.loggedInUserSource = respObj.data.clientPermissions.loggedInUserSource;

                var baseUrl = respObj.data.baseUrl;
                baseUrlMismatch = (baseUrl.toLowerCase() != window.location.href.substring(0, baseUrl.length).toLowerCase());
              }
              else
              {
                Ext.namespace('Sonatype.utils').edition = '';
                Ext.namespace('Sonatype.utils').licenseInstalled = null;
                Ext.namespace('Sonatype.utils').licenseExpired = null;
                Ext.namespace('Sonatype.utils').trialLicense = null;
                Ext.namespace('Sonatype.utils').anonDisabled = (response.status === 401);

                Sonatype.user.curr.repoServer = null;
                Sonatype.user.curr.isLoggedIn = null;
                Sonatype.user.curr.username = null;
                Sonatype.user.curr.loggedInUserSource = null;

              }

              var availSvrs = Sonatype.config.installedServers;
              for (var srv in availSvrs)
              {
                if (availSvrs[srv] && typeof(Sonatype[srv]) != 'undefined')
                {
                  Sonatype[srv][Strings.capitalize(srv)].statusComplete(respObj);
                }
              }

              Sonatype.Events.fireEvent('initHeadLinks');

              Sonatype.view.serverTabPanel.doLayout();

              if (baseUrlMismatch && Sonatype.lib.Permissions.checkPermission('nexus:settings', Sonatype.lib.Permissions.READ))
              {
                ns.postWelcomePageAlert('<b>WARNING:</b> ' + 'Base URL setting of <a href="' + baseUrl + '">' + baseUrl + '</a> ' + 'does not match your actual URL! ' + 'If you\'re running Apache mod_proxy, here\'s '
                    + '<a href="http://links.sonatype.com/products/nexus/oss/docs-mod-proxy">' + 'more information</a> on configuring Nexus with it.' );
              }

              Sonatype.Events.fireEvent('nexusStatus');

              Sonatype.view.historyDisabled = false;
              var token = Ext.History.getToken();
              if (Sonatype.view.loginSuccessfulToken)
              {
                token = Sonatype.view.loginSuccessfulToken;
                Sonatype.view.loginSuccessfulToken = null;
              }
              ns.onHistoryChange(token);
              ns.updateHistory();
              Sonatype.view.justLoggedOut = false;

              if (Sonatype.lib.Permissions.checkPermission('nexus:settings', Sonatype.lib.Permissions.READ)) {
                ns.loadNexusSettings();
              }
            }
          });
    },

    postWelcomePageAlert : function(msg) {
      // we want to insert the toolbar first, rather than add to the page in
      // random spot
      Sonatype.view.welcomeTab.insert(0, {
            xtype : 'panel',
            html : '<div class="x-toolbar-warning-box"><div class="x-form-invalid-msg" style="width: 95%;">' + msg + '</div></div>'
          });
      Sonatype.view.welcomeTab.doLayout();
    },

    onHistoryChange : function(token) {
      if (!token && window.location.hash)
      {
        token = window.location.hash.substring(1);
      }

      Sonatype.view.historyDisabled = true;
      var eventResult = Sonatype.Events.fireEvent('historyChanged',token);
      Sonatype.view.historyDisabled = false;

      //if event returns true, that means no event listener handled the data and stopped the process, so handle by default means
      if (token && Sonatype.user.curr.repoServer && Sonatype.user.curr.repoServer.length && eventResult)
      {
        Sonatype.view.historyDisabled = true;

        var toks = token.split(Sonatype.view.HISTORY_DELIMITER);

        var tabId = toks[0];
        var tabPanel = Sonatype.view.mainTabPanel.getComponent(tabId);
        if (tabPanel)
        {
          Sonatype.view.mainTabPanel.setActiveTab(tabId);
        }
        else
        {
          var navigationPanel = Ext.getCmp('navigation-' + tabId);
          if (navigationPanel)
          {
            var c = navigationPanel.initialConfigNavigation;
            if (c)
            {
              tabPanel = Sonatype.view.mainTabPanel.addOrShowTab(c.tabId, c.tabCode, {
                    title : c.tabTitle ? c.tabTitle : c.title
                  });
            }
          }
          else if (Sonatype.view.supportedNexusTabs[tabId] && !Sonatype.view.justLoggedOut && !Sonatype.user.curr.isLoggedIn)
          {
            Sonatype.view.historyDisabled = false;
            Sonatype.view.afterLoginToken = token;
            Sonatype.repoServer.RepoServer.loginHandler();
            return;
          }
        }
        Sonatype.view.historyDisabled = false;

        if (tabPanel && tabPanel.applyBookmark && toks.length > 1)
        {
          tabPanel.applyBookmark(toks[1]);
        }
      }
    },

    updateHistory : function(tab) {
      var bookmark = ns.getBookmark(tab);
      if (!bookmark)
      {
        return;
      }
      var oldBookmark = Ext.History.getToken();
      if (bookmark != oldBookmark)
      {
        Ext.History.add(bookmark);
      }
    },

    replaceHistory : function(tab) {
      var bookmark = ns.getBookmark(tab);
      if (!bookmark)
      {
        return;
      }
      location.replace(window.location.href.replace(/\#.*$/, '') + '#' + bookmark);
    },

    getBookmark : function(tab) {
      if (Sonatype.view.historyDisabled)
        return;

      var bookmark = null;

      if (tab)
      {
        if (tab.ownerCt != Sonatype.view.mainTabPanel || tab != Sonatype.view.mainTabPanel.getActiveTab())
          return;

        bookmark = tab.id;
      }
      else
      {
        // this is to handle very special where we are passing a bookmark on
        // redirect, in IE this is not handled very well at all, so we trick it
        var oldBookmark = Ext.History.getToken();
        if (oldBookmark && oldBookmark.indexOf('force') == 0)
        {
          bookmark = oldBookmark.substring('force'.length);
        }
        else
        {
          tab = Sonatype.view.mainTabPanel.getActiveTab();
          bookmark = tab.id;
        }
      }

      if (tab && tab.getBookmark)
      {
        var b2 = tab.getBookmark();
        if (b2)
        {
          bookmark += Sonatype.view.HISTORY_DELIMITER + b2;
        }
      }

      return bookmark;
    },

    openWindow : function(url) {
      window.open(url);
    }
  });

  return Sonatype;
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Sonatype/view',['../extjs', 'Sonatype/init', 'nexus', 'Nexus/log', 'Sonatype/utils', 'ext/ux/TabCloseMenu', 'Sonatype/strings'], function(Ext, Sonatype, Nexus, Log, Utils, TabCloseMenu, Strings) {
Sonatype.view = {
  FIELD_OFFSET_WITH_SCROLL : String((3 + 16 + 3 + 30) * (-1)), // (help icon margin) + (help icon) + margin + (scrollbar)
  FIELD_OFFSET : String((3 + 16) * (-1)), // extra padding on right of icon not needed
  FIELDSET_OFFSET : String((3 + 18) * (-1)), // (extra room between border and scrollbar) + (scrollbar)
  FIELDSET_OFFSET_WITH_SCROLL : String((3 + 18 + 3 + 30) * (-1)), // (extra room between border and scrollbar) + (scrollbar) + margin + (scrollbar)
  HISTORY_DELIMITER : ';',

  headLinks : new Sonatype.headLinks(),

  init : function() {
    var viewport, size, srv, availSvrs;

    Sonatype.Events.addListener('initHeadLinks', function() {
          Sonatype.view.headLinks.updateLinks();
        });

    Sonatype.Events.fireEvent('initHeadLinks');

    // Ext.get('login-link').on('click',
    // Sonatype.repoServer.RepoServer.loginHandler,
    // Sonatype.repoServer.RepoServer);
    // this.updateLoginLinkText();

    Ext.get('header').show();
    Ext.get('welcome-tab').show();

    Sonatype.view.welcomeTab = new Ext.Panel({
          contentEl : 'welcome-tab',
          title : 'Welcome',
          id : 'welcome'
        });

    Sonatype.view.headerPanel = new Ext.Panel({
          contentEl : 'header',
          region : 'north',
          margins : '0 0 0 0',
          border : false,
          autoHeight : true
        });

    Ext.Ajax.on({
          "requestexception" : {
            fn : function(conn, response, options) {
              var suppressed = options.suppressStatus;

              if (Sonatype.config.repos.urls.status === options.url)
              {
                // never show the connection error message for the status resource
                // (if anonymous is disabled, UI would show this all the time, until you're logged in)
                return;
              } else if ( suppressed === true ) {
                // 'true' means suppress all error handling
                return;
              }

              if (suppressed === undefined) {
                suppressed = [];
              } else if (!Ext.isArray(suppressed)) {
                suppressed = [suppressed];
              }

              // NEXUS-5228 some plugins use numbers in their options, some use strings (suppressStatus : '404')
              // and rely on type conversion. That should be fixed because
              // a) it was relying on broken js behavior (implicit conversion),
              // b) it did not work at all when multiple status codes should be suppressed
              //     (['400', '404'] would still show error messages)
              Ext.each(suppressed, function(item, idx, array) {
                if (Ext.type(item) !== 'number') {
                  Nexus.Log.warn('The "suppressStatus" config option is not an instance of Number. Trying to convert.' );
                  array[idx] = parseInt(item, 10);
                }
              });

              if (suppressed.indexOf(response.status) < 0)
              {
                Sonatype.utils.connectionError(response, options.defaultErrorMessage, null, options, true);
              }
            },
            scope : this
          }
        });

    viewport = new Ext.Viewport({
          layout : 'border',
          items : [Sonatype.view.headerPanel, {
                region : 'west',
                title : 'Sonatype&trade;',
                collapsible : true,
                // collapseMode: 'mini',
                // border: false,
                split : false,
                width : 185,
                minSize : 185,
                maxSize : 185,
                layout : 'fit',
                margins : '0 5 5 5',
                items : [{
                  xtype : 'tabpanel',
                  id : 'st-server-tab-panel',
                  border : false,
                  tabPosition : 'top',
                  layoutOnTabChange : true
                    // tabs added by servers
                  }]
              }, new Sonatype.view.MainTabPanel({
                    id : 'st-main-tab-panel',
                    region : 'center',
                    margins : '0 5 5 0',
                    deferredRender : false,
                    resizeTabs : true,
                    enableTabScroll : true,
                    minTabWidth : 140,
                    autoScroll : false, // default
                    defaults : {
                      closable : true
                    },
                    activeTab : 0,
                    layoutOnTabChange : true,
                    // border: false,
                    // bodyBorder: true,
                    items : [Sonatype.view.welcomeTab]
                  })]
        });

    // Export useful values to Sonatype.view namespace
    Sonatype.view.viewport = viewport;
    Sonatype.view.serverTabPanel = viewport.findById('st-server-tab-panel');
    Sonatype.view.mainTabPanel = viewport.findById('st-main-tab-panel');
    Sonatype.view.supportedNexusTabs = {};

    size = Sonatype.view.serverTabPanel.getSize();
    Sonatype.view.serverTabPanel.setHeight(size.height - 20);

    // allow each included sonatype server to setup its tab and events
    availSvrs = Sonatype.config.installedServers;
    for (srv in availSvrs)
    {
      if (availSvrs.hasOwnProperty(srv)) {
        if (availSvrs[srv] && typeof(Sonatype[srv]) !== 'undefined')
        {
          Sonatype[srv][Strings.capitalize(srv)].initServerTab();
        }
      }
    }

    Sonatype.view.serverTabPanel.setActiveTab('st-nexus-tab');

    Ext.History.addListener('change', Sonatype.utils.onHistoryChange);
    Sonatype.view.mainTabPanel.addListener('tabchange', function(panel, tab) {
          Sonatype.utils.updateHistory(tab);
        });
  }

  // updateLoginLinkText : function(){
  // var loginEl = Ext.get('login-link');
  // var usernameEl = Ext.get('username');
  //    
  // if (Sonatype.user.curr.isLoggedIn){
  // usernameEl.update(Sonatype.user.curr.username + ' | ').show();
  // loginEl.update('Log Out');
  // }
  // else {
  // loginEl.update('Log In');
  // usernameEl.update('');
  // }
  // }
};


Sonatype.view.MainTabPanel = Ext.extend(Ext.TabPanel, {
      plugins: new Ext.ux.TabCloseMenu(),

      addOrShowTab : function(id, PanelClass, panelClassParams) {
        var  tab = this.getComponent(id);

        if (tab)
        {
          this.setActiveTab(tab);
        }
        else
        {
          tab = new PanelClass(Ext.apply({
                id : id
              }, panelClassParams||{}));
          this.add(tab);
        }

        this.setActiveTab(tab);

        return tab;
      }
    });

Sonatype.Events.addListener('nexusNavigationInit', function(panel) {
      Sonatype.view.viewport.doLayout();
      Sonatype.utils.updateGlobalTimeout();
    });


  return Sonatype;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define, Nexus*/
define('Sonatype/strings', ['Nexus/util/Strings'], function() {
  return Nexus.util.Strings;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Nexus, Sonatype, Ext*/
/*
 * A viewer panel offering a grid on top and a details pane at the bottom.
 * Config options:
 *
 * autoCreateNewRecord: if set, the "add" menu action handler
 * will automatically create a new blank record and insert it into the grid,
 * before invoking the subscriber supplied handler. The record will be assigned
 * the same "autoCreateNewRecord=true" flag, which is tracked by the editor
 * component, causing it to re-create the record from the server response when
 * the form is submitted.
 *
 * addMenuInitEvent: this parameter causes the panel to
 * create an "Add" button on the taskbar and fire the event so the subscribers
 * can append actions to the "add" menu.
 * columns: a column config, combining the
 * properties of Ext.data.Record and Ext.grid.ColumnModel. All items are used in
 * the Record constructor. Additionally, items with a 'header' property are used
 * in the ColumnsModel config, with the Record's 'name' becoming ColumnModel's
 * 'dataIndex'.
 *
 * dataAutoLoad: the 'autoLoad' property for the data store
 * (defaults to true). dataId: the id property in the record set (defaults to
 * 'resourceURI').
 * dataRoot: the root property of the response (defaults to
 * 'data').
 *
 * dataSortInfo: the 'sortInfo' property for the data store.
 *
 * deleteButton: creates a "Delete" button on the toolbar. The delete handler
 * will submit a DELETE request to requestURI of the selected record.
 *
 * rowClickEvent: event name to fire when a row is clicked.
 *
 * rowFocusChangedEvent: event name to fire when a row's focus changes. Called
 * every time a row is clicked.
 * rowClickHandler: a specific handler to be called
 * on rowClick
 *
 * rowContextClickEvent: event name to fire when a row is
 * right-clicked.
 *
 * rowContextClickHandler: a specific handler to be called when a
 * row is right-clicked
 *
 * mouseOverEvent: event name to fire when mouse over event
 * is fired
 *
 * mouseOverHandler: a specific handler to call when mouse over grid
 *
 * singleSelect: the 'singleSelect' property for the grid's selection model
 * (defaults to true).
 *
 * titleColumn: the name of the column to get a record title
 * from (defaults to 'name').
 *
 * url: the URl to load the data from.
 */
NX.define('Sonatype.panels.GridViewer', {
  requires : ['Nexus.panels.AutoTabPanel','Nexus.ext.GridFilterBox'],
  requirejs : ['Sonatype/view', 'Nexus/config', 'Nexus/navigation', 'Sonatype/utils'],
  extend : 'Ext.Panel',

  statics : {
    removeDanglingSeparators : function(menu) {
      var item, seenSeparator;
      if (!menu.items) {
        return menu;
      }
      item = menu.items.first();
      while (item && !item.text) {
        menu.remove(item); // clean up if the first element is a separator
        item = menu.items.first();
      }
      item = menu.items.last();
      while (item && !item.text) {
        menu.remove(item); // clean up if the last element is a separator
        item = menu.items.last();
      }

      menu.items.each(function(item) {
        if (!item.text) {
          if (seenSeparator) {
            menu.remove(item);
          } else {
            seenSeparator = true;
          }
        } else {
          seenSeparator = false;
        }
      });
    }
  },

  constructor : function(cfg) {
    var
          fields = [], columns = [], i, store, toolbar, c,
          config = cfg || {},
          defaultConfig = {
            dataAutoLoad : true,
            dataId : 'resourceURI',
            dataBookmark : 'id',
            dataRoot : 'data',
            dataSortInfo : {
              field : 'name',
              direction : 'asc'
            },
            titleColumn : 'name',
            singleSelect : true,
            collapsibleDetails : false
          };
    Ext.apply(this, config, defaultConfig);

    if (config.columns) {
      for (i = 0; i < config.columns.length; i = i + 1) {
        c = config.columns[i];
        fields.push({
          name : c.name,
          mapping : c.mapping,
          type : c.type,
          sortType : c.sortType,
          sortDir : c.sortDir,
          convert : c.convert,
          dateFormat : c.dateFormat,
          defaultValue : c.defaultValue
        });

        if (c.header !== null && c.header !== undefined) {
          if (c.autoExpand) {
            if (!c.id) {
              c.id = Ext.id();
            }
            this.autoExpandColumn = c.id;
          }
          c.dataIndex = c.name;
          columns.push(c);
        }
      }
    }

    this.dataStore = new Ext.data.JsonStore({
      root : this.dataRoot,
      id : this.dataId,
      fields : fields,
      url : this.url,
      autoLoad : this.dataAutoLoad && (!this.dataStores || this.dataStores.length === 0),
      sortInfo : this.dataSortInfo,
      isLoaded : false,
      listeners : {
        add : this.recordAddHandler,
        remove : this.recordRemoveHandler,
        update : this.recordUpdateHandler,
        load : {
          fn : function() {
            this.dataStore.isLoaded = true;
          },
          single : true,
          scope : this
        },
        scope : this
      }
    });

    this.gridPanel = new Ext.grid.GridPanel({
      region : this.collapsibleDetails ? 'center' : 'north',
      collapsible : this.collapsible || false,
      split : true,
      height : this.collapsibleDetails ? null : Sonatype.view.mainTabPanel.getInnerHeight() / 3,
      minHeight : this.collapsibleDetails ? null : 100,
      maxHeight : this.collapsibleDetails ? null : 500,
      frame : false,
      autoScroll : true,
      selModel : new Ext.grid.RowSelectionModel({
        singleSelect : this.singleSelect,
        listeners : {
          // HACK rowselect events were always fired with ExtJS2, even on store refresh/rows added,
          // so we have to hack the same for ExtJS3 or rewrite how the refreshHandler/rowSelectHandler is working.
          beforerowselect : function(model) {
            model.silent = false;
          }
        }
      }),

      ds : this.dataStore,
      // sortInfo: { field: 'name', direction: "ASC"},
      loadMask : true,
      deferredRender : false,
      colModel : new Ext.grid.ColumnModel({
        defaultSortable : true,
        columns : columns
      }),
      autoExpandColumn : this.autoExpandColumn,
      disableSelection : false,

      viewConfig : {
        emptyText: this.emptyText,
        deferEmptyText: false,
        emptyTextWhileFiltering: this.emptyTextWhileFiltering
      },

      listeners : {
        rowcontextmenu : {
          fn : this.rowContextMenuHandler,
          scope : this
        },
        mouseover : {
          fn : this.mouseOverGridHandler,
          scope : this
        },
        cellclick : {
          fn : this.cellClickHandler,
          scope : this
        }
      }
    });

    this.gridPanel.getSelectionModel().on('selectionchange', this.rowSelectHandler, this);
    this.gridPanel.getView().on('rowsinserted', function() {
      var idx, previousSelection = this.gridPanel.getSelectionModel().previousSelection;
      if (previousSelection) {
        idx = this.gridPanel.store.findBy(function(rec) {
          return rec.id === previousSelection.id;
        });
        if (idx !== -1) {
          this.gridPanel.getSelectionModel().selectRecords([ this.gridPanel.store.getAt(idx) ]);
        }
      }
    }, this);
    this.gridPanel.getView().on('beforerefresh', function() {
      this.gridPanel.getSelectionModel().previousSelection = this.gridPanel.getSelectionModel().getSelected();
    }, this);

    this.refreshButton = new Ext.Button({
      text : 'Refresh',
      iconCls : 'st-icon-refresh',
      cls : 'x-btn-text-icon',
      scope : this,
      handler : this.refreshHandler
    });
    toolbar = this.tbar;
    this.tbar = [this.refreshButton];
    this.createAddMenu();
    this.createDeleteButton();
    if (toolbar) {
      this.tbar = this.tbar.concat(toolbar);
    }
    this.tbar.push(
        '->',
        NX.create('Nexus.ext.GridFilterBox', {
          filteredGrid: this.gridPanel
        })
    );

    this.cardPanel = new Ext.Panel({
      layout : 'card',
      region : this.collapsibleDetails ? 'south' : 'center',
      title : this.collapsibleDetails ? ' ' : null,
      split : true,
      height : this.collapsibleDetails ? Sonatype.view.mainTabPanel.getInnerHeight() / 4 : null,
      activeItem : 0,
      deferredRender : false,
      autoScroll : false,
      frame : false,
      collapsed : this.collapsibleDetails,
      collapsible : this.collapsibleDetails,
      items : [
        {
          xtype : 'panel',
          layout : 'fit',
          html : '<div class="little-padding">Select a record to view the details.</div>'
        }
      ]
    });

    Sonatype.panels.GridViewer.superclass.constructor.call(this, {
      layout : 'border',
      autoScroll : false,
      width : '100%',
      height : '100%',
      items : [this.gridPanel, this.cardPanel]
    });

    if (this.dataStores) {
      for (i = 0; i < this.dataStores.length; i = i + 1) {
        store = this.dataStores[i];
        store.on('load', this.dataStoreLoadHandler, this);
        if (store.autoLoad !== true) {
          store.load();
        }
      }
    }
  },

  addActionHandler : function(handler, item, e) {
    if (item.autoCreateNewRecord) {
      var rec = new this.dataStore.reader.recordType({
        name : 'New ' + item.text
      }, 'new_' + new Date().getTime());
      rec.autoCreateNewRecord = true;
      if (handler && (handler(rec, item, e) === false)) {
        return;
      }

      this.dataStore.insert(0, [rec]);
      this.gridPanel.getSelectionModel().selectRecords([rec], false);
    }
    else {
      handler(item, e);
    }
  },

  applyBookmark : function(bookmark) {
    if (!this.dataStore.lastOptions) {
      this.dataStore.on('load', function(store, recs, options) {
        this.selectBookmarkedItem(bookmark);
      }, this, {
        single : true
      });
    } else {
      this.selectBookmarkedItem(bookmark);
    }
  },

  cancelHandler : function(panel) {
    var rec = panel.payload;
    if (rec) {
      if (this.dataStore.getById(rec.id)) {
        this.dataStore.remove(rec);
      }
      else {
        this.recordRemoveHandler(this.dataStore, rec, -1);
      }
    }
  },

  checkStores : function() {
    var i, store;
    if (this.dataStores) {
      for (i = 0; i < this.dataStores.length; i = i + 1) {
        store = this.dataStores[i];
        if (!store.lastOptions) {
          return false;
        }
      }
    }
    return true;
  },

  clearAll : function() {
    this.clearCards();
    this.dataStore.removeAll();
  },

  clearCards : function() {
    this.cardPanel.items.each(function(item, i, len) {
      if (i > 0) {
        this.remove(item, true);
      }
    }, this.cardPanel);

    this.cardPanel.getLayout().setActiveItem(0);
  },

  convertDataValue : function(value, store, idProperty, nameProperty) {
    if (value) {
      var rec = store.getAt(store.data.indexOfKey(value));
      if (!rec) {
        rec = store.getAt(store.find(idProperty, value));
      }
      if (rec) {
        return rec.data[nameProperty];
      }
    }
    return '';
  },

  createAddMenu : function() {
    if (this.addMenuInitEvent) {
      var
            menu = new Sonatype.menu.Menu({
              payload : this,
              scope : this,
              items : []
            }), item;

      Sonatype.Events.fireEvent(this.addMenuInitEvent, menu);

      Sonatype.panels.GridViewer.removeDanglingSeparators(menu);

      if (!menu.items || menu.items.length === 0) {
        return; // quit if empty
      }

      menu.items.each(function(item, index, length) {
        if (item.handler) {
          item.setHandler(this.addActionHandler.createDelegate(this, [item.handler], 0));
        }
      }, this);

      this.toolbarAddButton = new Ext.Button({
        text : 'Add...',
        icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
        cls : 'x-btn-text-icon',
        menu : menu
      });
      this.tbar.push(this.toolbarAddButton);
    }
  },

  createChildPanel : function(rec, recreateIfExists) {
    var child, tab, id, panel;

    rec.data.showCtx = this.showRecordContextMenu(rec);
    if (this.collapsibleDetails) {
      this.cardPanel.expand();
    }

    id = this.id + rec.id;

    panel = this.cardPanel.findById(id);

    if (recreateIfExists) {
      if (panel) {
        this.cardPanel.remove(panel, true);
        panel = null;
      }
    }

    if (!panel) {
      panel = new Nexus.panels.AutoTabPanel({
        id : id,
        title : rec.data[this.titleColumn],
        activeTab : -1
      });

      if (this.rowClickHandler) {
        this.rowClickHandler(panel, rec);
      }

      if (this.rowClickEvent) {
        Sonatype.Events.fireEvent(this.rowClickEvent, panel, rec, this);
      }

      if (panel.items) {
        if (!panel.tabPanel) {
          // if the panel has a single child, and the child fires a cancel
          // event,
          // catch it to clean up automatically
          child = panel.getComponent(0);
          child.on('cancel', this.cancelHandler.createDelegate(this, [child]), this);
        }
        else {
          panel.tabPanel.items.sort('ASC', function(a, b) {
            if (a.tabTitle <= b.tabTitle) {
              return a.tabTitle < b.tabTitle ? -1 : 0;
            }
            return 1;
          });
          panel.tabPanel.on('beforetabchange', function(tabpanel, newtab, currenttab) {
            // don't want to set this unless user clicked
            if (currenttab) {
              this.selectedTabName = newtab.name;
            }
          }, this);

          if (this.selectedTabName) {
            tab = panel.find('name', this.selectedTabName)[0];

            if (tab) {
              panel.tabPanel.setActiveTab(tab.id);
            }
            else {
              panel.tabPanel.setActiveTab(0);
            }
          }
          else {
            panel.tabPanel.setActiveTab(0);
          }
        }

        this.cardPanel.add(panel);
      }
      else {
        panel.add({
          xtype : 'panel',
          layout : 'fit',
          html : '<div class="little-padding">No details available.</div>'
        });
        this.cardPanel.add(panel);
      }
    }
    else {
      if (this.selectedTabName) {
        tab = panel.find('name', this.selectedTabName)[0];

        if (tab && panel.tabPanel && tab.id !== panel.tabPanel.getActiveTab().id) {
          panel.tabPanel.setActiveTab(tab.id);
        }
      }
    }

    // row clicked (not just init)
    if (this.rowFocusChangedEvent) {
      Sonatype.Events.fireEvent(this.rowFocusChangedEvent, panel, rec, this);
    }

    this.cardPanel.getLayout().setActiveItem(panel);
    panel.doLayout();
  },

  createDeleteButton : function() {
    if (this.deleteButton) {
      this.toolbarDeleteButton = new Ext.Button({
        text : 'Delete',
        icon : Sonatype.config.resourcePath + '/static/images/icons/delete.png',
        cls : 'x-btn-text-icon',
        handler : this.deleteActionHandler,
        scope : this
      });
      this.tbar.push(this.toolbarDeleteButton);
    }
  },

  dataStoreLoadHandler : function(store, records, options) {
    if (this.checkStores() && this.dataAutoLoad) {
      this.dataAutoLoad = false;
      this.dataStore.reload();
    }
  },

  deleteRecord : function(rec) {
    Ext.Ajax.request({
      callback : function(options, success, response) {
        if (success) {
          this.dataStore.remove(rec);
        }
        else {
          Sonatype.utils.connectionError(response, 'Delete Failed!');
        }
      },
      scope : this,
      method : 'DELETE',
      url : rec.data.resourceURI
    });
  },

  deleteActionHandler : function(button, e) {
    if (this.gridPanel.getSelectionModel().hasSelection()) {
      var rec = this.gridPanel.getSelectionModel().getSelected();

      if (rec.id.substring(0, 4) === 'new_') {
        this.dataStore.remove(rec);
      }
      else {
        Sonatype.utils.defaultToNo();

        Sonatype.MessageBox.show({
          animEl : this.gridPanel.getEl(),
          title : 'Delete',
          msg : 'Delete ' + rec.data[this.titleColumn] + '?',
          buttons : Sonatype.MessageBox.YESNO,
          scope : this,
          icon : Sonatype.MessageBox.QUESTION,
          fn : function(btnName) {
            if (btnName === 'yes' || btnName === 'ok') {
              this.deleteRecord(rec);
            }
          }
        });
      }
    }
  },

  getBookmark : function() {
    var rec = this.gridPanel.getSelectionModel().getSelected();
    return rec ? rec.data[this.dataBookmark] : null;
  },

  recordAddHandler : function(store, recs, index) {
    if (recs.length === 1 && recs[0].autoCreateNewRecord && recs[0].id.substring(0, 4) !== 'new_') {
      this.createChildPanel(recs[0]);
    }
  },

  recordRemoveHandler : function(store, rec, index) {
    var
          resetActiveItem,
          id = this.id + rec.id,
          panel = this.cardPanel.findById(id);

    if (panel) {
      resetActiveItem = this.cardPanel.getLayout().activeItem === panel;
      this.cardPanel.remove(panel, true);

      if (resetActiveItem) {
        this.cardPanel.getLayout().setActiveItem(0);
      }
    }
  },

  recordUpdateHandler : function(store, rec, op) {
    if (op === Ext.data.Record.COMMIT) {
      this.createChildPanel(rec, true);
    }
  },

  refreshHandler : function(button, e) {
    this.clearCards();

    var i, store;

    if (this.dataStores) {
      this.dataAutoLoad = true;
      for (i = 0; i < this.dataStores.length; i = i + 1) {
        store = this.dataStores[i];
        store.lastOptions = null;
        store.reload();
      }
    }
    else {
      this.gridPanel.store.reload();
    }
  },

  mouseOverGridHandler : function(e, t) {
    if (this.mouseOverHandler) {
      this.mouseOverHandler(e, t);
    }

    if (this.mouseOverEvent) {
      Sonatype.Events.fireEvent(this.mouseOverEvent, e, t);
    }
  },

  cellClickHandler : function(grid, rowIndex, columnIndex, eventObject) {
    var internalColumnIndex = this.getDisplayedColumnByIndex(columnIndex);
    if (internalColumnIndex !== -1 && this.columns[internalColumnIndex].clickHandler) {
      this.columns[internalColumnIndex].clickHandler(grid, rowIndex, eventObject);
    }
  },

  rowSelectHandler : function(selectionModel) {
    var rec = selectionModel.getSelected();
    if (rec && (this.rowClickEvent || this.rowClickHandler)) {
      this.createChildPanel(rec);

      var bookmark = rec.data[this.dataBookmark];
      if (bookmark) {
        Sonatype.utils.updateHistory(this);
      }
    }
  },

  //our internal column index contains all data, even for 'columns' not displayed, that
  //may just be mapped to a store, this will weed those out and retrieve the needed index
  getDisplayedColumnByIndex : function(index) {
    var i, j;
    for (i = 0, j = 0; i < this.columns.length; i = i + 1) {
      if (this.columns[i].header) {
        if (index === j) {
          return i;
        }

        j = j + 1;
      }
    }
    return -1;
  },

  rowContextMenuHandler : function(grid, index, e) {
    if (e.target.nodeName === 'A') {
      return; // no menu on links
    }

    if (this.rowContextClickEvent || this.rowContextClickHandler) {
      var
            item,
            rec = grid.store.getAt(index),
            menu = new Sonatype.menu.Menu({
              payload : rec,
              scope : this,
              items : []
            });

      if (this.rowContextClickHandler) {
        this.rowContextClickHandler(menu, rec);
      }

      if (this.rowContextClickEvent) {
        Sonatype.Events.fireEvent(this.rowContextClickEvent, menu, rec);
      }

      Sonatype.panels.GridViewer.removeDanglingSeparators(menu);

      if (!menu.items || !menu.items.first()) {
        return;
      }

      e.stopEvent();
      menu.showAt(e.getXY());
    }
  },

  selectBookmarkedItem : function(bookmark) {
    var
          oldBookmark,
          recIndex = this.dataStore.findBy(function(rec, id) {
            return rec.data[this.dataBookmark] === bookmark;
          }, this);

    if (recIndex >= 0) {
      oldBookmark = this.getBookmark();
      if (bookmark !== oldBookmark) {
        this.gridPanel.getSelectionModel().selectRecords([this.dataStore.getAt(recIndex)]);
      }
    }
  },

  // Override if want to restrict the context menu
  // default: show context menu
  showRecordContextMenu : function(rec) {
    return true;
  }
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Sonatype/all',['sonatype', 'Sonatype/lib', 'Sonatype/utils', 'Sonatype/view', 'Nexus/panels', 'Nexus/ext'], function(Sonatype) {
  return Sonatype;
});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define, escape, unescape*/
/*jslint eqeq: true*/
///* jslint bitwise: true, eqeq: true*/
define('Sonatype/lib',['extjs', 'sonatype'], function(Ext, Sonatype) {

  Sonatype.lib.Permissions = {
    READ : 1, // 0001
    EDIT : 2, // 0010
    DELETE : 4, // 0100
    CREATE : 8, // 1000
    ALL : 15, // 1111
    NONE : 0, // 0000

    // returns bool indicating if value has all perms
    // all values are base 10 representations of the n-bit representation
    // Example: for 4-bit permissions: 3 (base to) represents 0011 (base 2)
    checkPermission : function(value, perm /* , perm... */) {
      var p = perm, perms;

      if (Sonatype.user.curr.repoServer)
      {
        Ext.each(Sonatype.user.curr.repoServer, function(item, i, arr) {
              if (item.id === value)
              {
                value = item.value;
                return false;
              }
            });
      }

      if (arguments.length > 2)
      {
        perms = [].slice.call(arguments, 2);
        Ext.each(perms, function(item, i, arr) {
              p = p | item;
            });
      }

      return ((p & value) == p);
    }
  };

  /*
   * Adapted from ExtJS v2.0.2 Ext.state.CookieProvider; removed inheritance
   * from Ext.state.Provider @cfg {String} path The path for which the cookie is
   * active (defaults to root '/' which makes it active for all pages in the
   * site) @cfg {Date} expires The cookie expiration date (defaults to 7 days
   * from now) @cfg {String} domain The domain to save the cookie for. Note that
   * you cannot specify a different domain than your page is on, but you can
   * specify a sub-domain, or simply the domain itself like 'extjs.com' to
   * include all sub-domains if you need to access cookies across different
   * sub-domains (defaults to null which uses the same domain the page is
   * running on including the 'www' like 'www.extjs.com') @cfg {Boolean} secure
   * True if the site is using SSL (defaults to false) @constructor Create a new
   * CookieProvider @param {Object} config The configuration object
   */
  Sonatype.lib.CookieProvider = function(config) {
    this.namePrefix = 'st-'; // added alternate default prefix
    this.path = "/";
    this.expires = new Date(new Date().getTime() + (1000 * 60 * 60 * 24 * 7)); // 7
                                                                                // days
    this.domain = null;
    this.secure = false;
    Ext.apply(this, config);
    this.state = this.readCookies();
  };

  Sonatype.lib.CookieProvider.prototype = {
    /**
     * Returns the current value for a key
     * 
     * @param {String}
     *          name The key name
     * @param {Mixed}
     *          defaultValue A default value to return if the key's value is not
     *          found
     * @return {Mixed} The state data
     */
    get : function(name, defaultValue) {
      return this.state[name] || defaultValue;
    },

    // private
    set : function(name, value) {
      if (!value)
      {
        this.clear(name);
        return;
      }
      this.setCookie(name, value);
      this.state[name] = value;
    },

    // private
    clear : function(name) {
      this.clearCookie(name);
      delete this.state[name];
    },

    // private
    readCookies : function() {
      var
            cookies = {},
            c = document.cookie + ";",
            re = /\s?(.*?)=(.*?);/g,
            matches, name, value;

      while ((matches = re.exec(c)) != null)
      {
        name = matches[1];
        value = matches[2];

        if (name && name.substring(0, 3) === this.namePrefix)
        {
          cookies[name.substr(3)] = this.decodeValue(value);
        }
      }
      return cookies;
    },

    // private
    setCookie : function(name, value) {
      document.cookie = this.namePrefix + name + "=" + this.encodeValue(value) + ((this.expires == null) ? "" : ("; expires=" + this.expires.toGMTString())) + ((this.path == null) ? "" : ("; path=" + this.path))
          + ((this.domain == null) ? "" : ("; domain=" + this.domain)) + ((this.secure == true) ? "; secure" : "");
    },

    // private
    clearCookie : function(name) {
      document.cookie = this.namePrefix + name + "=null; expires=Thu, 01-Jan-70 00:00:01 GMT" + ((this.path == null) ? "" : ("; path=" + this.path)) + ((this.domain == null) ? "" : ("; domain=" + this.domain)) + ((this.secure == true) ? "; secure" : "");
    },

    /**
     * Decodes a string previously encoded with {@link #encodeValue}.
     * 
     * @param {String}
     *          value The value to decode
     * @return {Mixed} The decoded value
     */
    decodeValue : function(cookie) {
      var all, values, i, len, kv, re, matches, type, v;
      re = /^(a|n|d|b|s|o)\:(.*)$/;
      matches = re.exec(unescape(cookie));
      if (!matches || !matches[1]) {
        return null; // non state cookie
      }
      type = matches[1];
      v = matches[2];

      switch (type)
      {
        case "n" :
          return parseFloat(v);
        case "d" :
          return new Date(Date.parse(v));
        case "b" :
          return (v === "1");
        case "a" :
                all = [];
                values = v.split("^");
          for (i = 0, len = values.length; i < len; i=i+1)
          {
            all.push(this.decodeValue(values[i]));
          }
          return all;
        case "o" :
          all = {};
          values = v.split("^");
          for (i = 0, len = values.length; i < len; i=i+1)
          {
            kv = values[i].split("=");
            all[kv[0]] = this.decodeValue(kv[1]);
          }
          return all;
        default :
          return v;
      }
    },

    /**
     * Encodes a value including type information. Decode with
     * {@link #decodeValue}.
     * 
     * @param {Mixed}
     *          value The value to encode
     * @return {String} The encoded value
     */
    encodeValue : function(v) {
      var enc, i, flat = "", key, len;
      if (typeof v === "number")
      {
        enc = "n:" + v;
      }
      else if (typeof v === "boolean")
      {
        enc = "b:" + (v ? "1" : "0");
      }
      else if (Ext.isDate(v))
      {
        enc = "d:" + v.toGMTString();
      }
      else if (Ext.isArray(v))
      {
        for (i = 0, len = v.length; i < len; i=i+1)
        {
          flat += this.encodeValue(v[i]);
          if (i !== len - 1) {
            flat += "^";
          }
        }
        enc = "a:" + flat;
      }
      else if (typeof v === "object")
      {
        for (key in v)
        {
          if (v.hasOwnProperty(key) && typeof v[key] !== "function" && v[key] !== undefined)
          {
            flat += key + "=" + this.encodeValue(v[key]) + "^";
          }
        }
        enc = "o:" + flat.substring(0, flat.length - 1);
      }
      else
      {
        enc = "s:" + v;
      }
      return escape(enc);
    }
  };

});
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('Sonatype/init',['extjs', 'sonatype', 'Nexus/util/observable', 'sonatype'], function(Ext, Sonatype, Nexus) {
  Ext.apply(Sonatype, {
          init : function() {
            Ext.QuickTips.init();
            Ext.apply(Ext.QuickTips.getQuickTip(), {
              showDelay : 250,
              hideDelay : 300,
              dismissDelay : 0
              // don't automatically hide quicktip
            });

            Ext.History.init();

            Ext.get('header').hide();
            Ext.get('welcome-tab').hide();

            window.Sonatype.state.CookieProvider = new window.Sonatype.lib.CookieProvider({
              expires : new Date(new Date().getTime() + (1000 * 60 * 60 * 24 * 365))
              // expires in 1 year
            });

            Sonatype.resources.help = {};

            // Default anonymous user permissions; 3-bit permissions: delete | edit | read
            Sonatype.user.anon = {
              username : '',
              isLoggedIn : false,
              repoServer : {}
            };

            Sonatype.user.curr = {
              username : '',
              isLoggedIn : false,
              repoServer : {}
            };
          }
        }
  );

  Sonatype.Events = new Nexus.util.Observable();

  // FIXME circular dependency sonatype -> sonatype/view -> sonatype/headlinks, but headlinks needs 'Sonatype'
  Sonatype.headLinks = Ext.emptyFn;

  Ext.apply(Sonatype.headLinks.prototype, {
    /**
     * Update the head links based on the current status of Nexus
     */
    updateLinks : function() {
      var
            right = Ext.get('head-link-r'),
            loggedIn = Sonatype.user.curr.isLoggedIn;

      if (loggedIn)
      {
        this.updateRightWhenLoggedIn(right);
      }
      else
      {
        this.updateRightWhenLoggedOut(right);
      }
    },

    updateRightWhenLoggedIn : function(linkEl) {
      linkEl.update(Sonatype.user.curr.username);
      linkEl.addClass('head-link-logged-in');
      linkEl.un('click', Sonatype.repoServer.RepoServer.loginHandler, Sonatype.repoServer.RepoServer);
      linkEl.on('click', Sonatype.repoServer.RepoServer.showProfileMenu);
    },
    updateRightWhenLoggedOut : function(linkEl) {
      linkEl.un('click', Sonatype.repoServer.RepoServer.showProfileMenu);
      linkEl.update('Log In');

      this.setClickLink(linkEl);
      linkEl.removeClass('head-link-logged-in');
    },
    setClickLink : function(el) {
      el.removeAllListeners();
      el.on('click', Sonatype.repoServer.RepoServer.loginHandler, Sonatype.repoServer.RepoServer);
    }
  });

  return Sonatype;
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define, Ext, Nexus, Sonatype, NX*/
NX.define('Sonatype.repoServer.RepositoryRoutingPanel', {
  extend : 'Nexus.ext.FormPanel',

  requirejs : ['Nexus/config'],

  statics : {
    discoveryUpdateIntervalStore : new Ext.data.ArrayStore({
      fields : ['intervalLabel', 'valueHrs'],
      data : [
        ['1 hr', '1'],
        ['2 hr', '2'],
        ['3 hr', '3'],
        ['6 hr', '6'],
        ['9 hr', '9'],
        ['12 hr', '12'],
        ['Daily', '24'],
        ['Weekly', '168']
      ]
    }),

    publishStatusStore : new Ext.data.ArrayStore({
      fields : ['text', 'value'],
      data : [
        ['Not published.', '-1'],
        ['Unknown.', '0'],
        ['Published.', '1']
      ]
    }),

    discoveryStatusStore : new Ext.data.ArrayStore({
      fields : ['text', 'value'],
      data : [
        ['Unsuccessful.', '-1'],
        ['In Progress.', '0'],
        ['Successful.', '1']
      ]
    })
  },

  refreshContent : function() {
    var self = this;

    self.actionMask = new Ext.LoadMask(self.el, {
      msg : 'Loading...',
      removeMask : true
    });
    self.actionMask.show();
    self.loadData();
  },

  constructor : function(cfg) {
    if (!cfg || !cfg.payload) {
      throw new Error("payload missing: need repository record");
    }

    // should be static, but Sonatype.config is not defined yet when statics are defined (before requirejs dep resolution)
    this.resourceUrl = new Ext.Template(Sonatype.config.repos.urls.repositories + "/{0}/routing").compile();

    var
          self = this,
          payload = cfg.payload,
          subjectIsNotM2Proxy = cfg.payload.data.repoType !== 'proxy' || cfg.payload.data.format !== 'maven2',
          defaultConfig = {
            frame : true,
            autoScroll : true,
            readOnly : true, // don't want save/cancel buttons
            url : self.resourceUrl.apply([cfg.payload.data.id])
          };


    self.payload = {
      data : {
        id : cfg.payload.data.id,
        resourceURI : self.resourceUrl.apply([cfg.payload.data.id])
      }
    };

    // don't use payload directly, this panel does not behave as expected by Nexus.ext.FormPanel
    delete cfg.payload;

    Ext.apply(this, cfg, defaultConfig);

    self.dataModifiers = {
      load : {
        'publishedStatus' : function(value) {
          self.publishedStatus = value;
          var store = Sonatype.repoServer.RepositoryRoutingPanel.publishStatusStore;
          return store.getAt(store.find('value', value)).get('text');
        },

        'discovery.discoveryLastStatus' : function(value) {
          self.discoveryStatus = value;
          var store = Sonatype.repoServer.RepositoryRoutingPanel.discoveryStatusStore;
          return store.getAt(store.find('value', value)).get('text');
        },

        'publishedUrl' : function(value) {
          self.whitelistUrl = value;
          return value;
        }
      }
    };

    self.items = [
      {
        xtype : 'fieldset',
        title : 'Publishing',
        name : 'publishingFieldset',
        anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
        layout : {
          type : 'hbox',
          align : 'stretchmax'
        },
        items : [
          {
            xtype : 'panel',
            layout : 'form',
            flex : 1,
            items : [
              {
                xtype : 'displayfield',
                fieldLabel : 'Status',
                name : 'publishedStatus'
              },
              {
                xtype : 'displayfield',
                fieldLabel : 'Message',
                name : 'publishedMessage'
              },
              {
                xtype : 'timestampDisplayField',
                fieldLabel : 'Published on',
                name : 'publishedTimestamp'
              }
            ]
          },
          {
            xtype : 'container',
            width : 80,
            layout : {
              type : 'vbox',
              pack : 'end'
            },
            items : [
              {
                xtype : 'link-button',
                text : 'Show prefix file',
                name : 'prefix-link',
                handler : function() {
                  window.open(self.whitelistUrl, '_blank');
                }
              }
            ]
          }
        ]
      },
      {
        xtype : 'fieldset',
        title : 'Discovery',
        anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
        checkboxToggle : true,
        name : 'discoveryFieldset',
        hidden : subjectIsNotM2Proxy,
        listeners : {
          expand : function() {
            this.enableDiscoveryHandler(true);
          },
          collapse : function() {
            this.disableDiscovery();
          },
          scope : this
        },
        items : [
          {
            xtype : 'hidden',
            name : 'discovery.discoveryEnabled',
            listeners : {
              change : function(field, enabled, oldValue) {
                var discoveryFieldset = this.find('name', 'discoveryFieldset')[0];
                if (enabled) {
                  discoveryFieldset.expand();
                } else {
                  discoveryFieldset.collapse();
                }
              }
            }
          },
          {
            xtype : 'displayfield',
            fieldLabel : 'Status',
            name : 'discovery.discoveryLastStatus'
          },
          {
            xtype : 'displayfield',
            fieldLabel : 'Message',
            name : 'discovery.discoveryLastMessage'
          },
          {
            xtype : 'timestampDisplayField',
            fieldLabel : 'Last run',
            name : 'discovery.discoveryLastRunTimestamp'
          },
          {
            xtype : 'combo',
            fieldLabel : 'Update interval',
            name : 'discovery.discoveryIntervalHours',
            store : Sonatype.repoServer.RepositoryRoutingPanel.discoveryUpdateIntervalStore,
            displayField : 'intervalLabel',
            valueField : 'valueHrs',
            emptyText : 'Select...',
            iconCls : 'no-icon', //use iconCls if placing within menu to shift to right side of menu
            mode : 'local',
            editable : false,
            allowBlank : false,
            selectOnFocus : true,
            forceSelection : true,
            triggerAction : 'all',
            typeAhead : true,
            listeners : {
              select : function(combo) {
                this.enableDiscovery(parseInt(combo.getValue(), 10));
              },
              scope : this
            },
            width : 150
          },
          {
            xtype : 'button',
            hideLabel : true,
            text : 'Update now',
            handler : this.forceRemoteDiscoveryHandler,
            scope : this
          }
        ]
      }
    ];

    Sonatype.repoServer.RepositoryRoutingPanel.superclass.constructor.apply(self, arguments);

    self.on('actioncomplete', self.onActionComplete);
  },

  enableDiscoveryHandler : function(checked) {
    // 'this' is the checkbox
    var
          combo = this.find('name', 'discovery.discoveryIntervalHours'),
          fieldset = this.find('name', 'discoveryFieldset');

    if (!(fieldset.length > 0 && combo.length > 0)) {
      throw new Error('could not find interval combo box or fieldset');
    }

    if (Ext.isEmpty(combo[0].getValue())) {
      // still in loadData(), do nothing
      return;
    }

    if (checked) {
      this.enableDiscovery(parseInt(combo[0].getValue(), 10));
      fieldset[0].expand();
    } else {
      this.disableDiscovery();
      fieldset[0].collapse();
    }
  },

  disableDiscovery : function() {
    if (!this.submitDiscoverySetting) {
      return;
    }

    var
          self = this,
          mask = new Ext.LoadMask(this.el, {
            msg : 'Disabling discovery...',
            removeMask : true
          });
    mask.show();
    Ext.Ajax.request({
      method : 'PUT',
      url : this.resourceUrl.apply([this.payload.data.id]) + '/config',
      jsonData : {
        data : {
          discoveryEnabled : false,
          discoveryIntervalHours : -1
        }
      },
      callback : function() {
        self.actionMask = mask;
        this.loadData();
      },
      scope : this
    });
  },

  enableDiscovery : function(interval) {
    if (!this.submitDiscoverySetting) {
      return;
    }
    if (interval === 0) {
      interval = 24;
    }

    var
          self = this,
          mask = new Ext.LoadMask(this.el, {
            msg : 'Updating discovery...'
          });
    mask.show();
    Ext.Ajax.request({
      method : 'PUT',
      url : this.resourceUrl.apply([this.payload.data.id]) + '/config',
      jsonData : {
        data : {
          discoveryEnabled : true,
          discoveryIntervalHours : interval
        }
      },
      callback : function() {
        self.actionMask = mask;
        self.loadData();
      }
    });
  },

  forceRemoteDiscoveryHandler : function() {
    var
          self = this,
          mask = new Ext.LoadMask(this.el, {
            msg : 'Updating...',
            removeMask : true
          });

    mask.show();
    Ext.Ajax.request({
      method : 'DELETE',
      url : this.resourceUrl.apply([this.payload.data.id]),
      callback : function() {
        self.actionMask = mask;
        self.loadData();
      },
      scope : this
    });
  },
  onActionComplete : function(form, action) {
    var
          self = this,
          panel = action.options.fpanel,
          tstampField = panel.find('name', 'discovery.discoveryLastRunTimestamp')[0];

    if (action.type === 'sonatypeLoad') {
      // @note: this is a work around to get proper use of the isDirty()
      // function of this field

      if (panel.find('name', 'discovery.discoveryEnabled')[0].getValue() === 'true') {
        panel.find('name', 'discoveryFieldset')[0].expand();
      } else {
        panel.find('name', 'discoveryFieldset')[0].collapse();
      }
      panel.submitDiscoverySetting = true;

      // FIXME this does not change the height of the fieldset
      panel.find('xtype', 'container')[0].setVisible(panel.publishedStatus !== -1);
      panel.find('name', 'publishedTimestamp')[0].setVisible(panel.publishedStatus !== -1);

      tstampField.setVisible(panel.discoveryStatus !== 0);

      panel.find('name', 'prefix-link')[0].setVisible(self.whitelistUrl !== undefined);
    }

    if ( self.actionMask !== undefined ) {
      self.actionMask.hide();
      self.actionMask = undefined;
    }
  }
}, function() {
  Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec) {
    var sp = Sonatype.lib.Permissions, newRecord, maven2FormatRepo, notShadowType;

    // Check whether record is new or received from Nexus
    // (GridViewer convention for new records: id starts with 'new_')
    newRecord = rec.id && rec.id.indexOf('new_') === 0;

    maven2FormatRepo = rec.data.format === 'maven2';
    notShadowType = rec.data.repoType !== 'virtual';

    if (!newRecord && maven2FormatRepo && notShadowType &&
          ( sp.checkPermission('nexus:repositories', sp.CREATE) || sp.checkPermission('nexus:repositories', sp.DELETE) || sp.checkPermission('nexus:repositories', sp.EDIT) )) {
      cardPanel.add(new Sonatype.repoServer.RepositoryRoutingPanel({
        tabTitle : 'Routing',
        name : 'routing',
        payload : rec
      }));
    }
  });
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, NX*/
NX.define('Sonatype.repoServer.UserMappingEditor', {
  extend : 'Sonatype.ext.FormPanel',
  requireSuper : false,

  requires : ['Sonatype.repoServer.UserEditPanel'],

  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {
      uri : Sonatype.config.repos.urls.plexusUser,
      dataModifiers : {
        load : {
          roles : function(arr, srcObj, fpanel) {
            var roleManager = fpanel.find('name', 'roleManager')[0];
            if (!roleManager) {
              // guard against rec.commit() without fpanel set - no need to translate then
              return arr;
            }
            if (this.lastLoadedId) {
              roleManager.setUserId(this.lastLoadedId);
            }
            else if (fpanel.payload && fpanel.payload.get) {
              roleManager.setUserId(fpanel.payload.get('userId'));
            }
            roleManager.setSelectedRoleIds(arr, true);
            return arr;
          }
        },
        submit : {
          roles : function(value, fpanel) {
            return fpanel.find('name', 'roleManager')[0].getSelectedRoleIds();
          }
        }
      },
      referenceData : {
        userId : '',
        source : '',
        roles : []
      }
    };
    Ext.apply(this, config, defaultConfig);

    var ht = Sonatype.repoServer.resources.help.users;

    this.COMBO_WIDTH = 300;

    var useridField;
    if (this.payload.id === 'new_mapping') {
      useridField = {
        xtype : 'trigger',
        triggerClass : 'x-form-search-trigger',
        fieldLabel : 'Enter a User ID',
        itemCls : 'required-field',
        labelStyle : 'margin-left: 15px; width: 185px;',
        name : 'userId',
        allowBlank : false,
        width : this.COMBO_WIDTH,
        listeners : {
          specialkey : {
            fn : function(f, e) {
              if (e.getKey() === e.ENTER) {
                this.loadUserId.createDelegate(this);
              }
            }
          },
          change : {
            fn : function(control, newValue, oldValue) {
              if (newValue != this.lastLoadedId) {
                this.loadUserId();
              }
            },
            scope : this
          }
        },
        onTriggerClick : this.loadUserId.createDelegate(this)
      };
    } else {
      useridField = {
        xtype : 'textfield',
        fieldLabel : 'User ID',
        itemCls : 'required-field',
        labelStyle : 'margin-left: 15px; width: 185px;',
        name : 'userId',
        disabled : true,
        allowBlank : false,
        width : this.COMBO_WIDTH,
        userFound : true
      };
    }

    Sonatype.repoServer.UserMappingEditor.superclass.constructor.call(this, {
      items : [
        {
          xtype : 'panel',
          layout : 'form',
          width : 600,
          items : [useridField, {
            xtype : 'textfield',
            fieldLabel : 'Realm',
            itemCls : 'required-field',
            labelStyle : 'margin-left: 15px; width: 185px;',
            name : 'source',
            disabled : true,
            allowBlank : false,
            width : this.COMBO_WIDTH
          }, {
            xtype : 'textfield',
            fieldLabel : 'First Name',
            itemCls : 'required-field',
            labelStyle : 'margin-left: 15px; width: 185px;',
            name : 'firstName',
            disabled : true,
            allowBlank : false,
            width : this.COMBO_WIDTH
          }, {
            xtype : 'textfield',
            fieldLabel : 'Last Name',
            itemCls : 'required-field',
            labelStyle : 'margin-left: 15px; width: 185px;',
            name : 'lastName',
            disabled : true,
            allowBlank : false,
            width : this.COMBO_WIDTH
          }, {
            xtype : 'textfield',
            fieldLabel : 'Email',
            itemCls : 'required-field',
            labelStyle : 'margin-left: 15px; width: 185px;',
            name : 'email',
            disabled : true,
            allowBlank : false,
            width : this.COMBO_WIDTH
          }, {
            id : "usermapping-rolemanager",
            xtype : 'rolemanager',
            name : 'roleManager',
            height : 200,
            width : 505,
            usePrivileges : false
          }]
        }
      ],
      listeners : {
        load : this.loadHandler,
        submit : this.submitHandler,
        scope : this
      }
    });
  },
  saveHandler : function(button, event) {
    if (this.isValid()) {
      var method = 'PUT';
      var roleManager = this.find('name', 'roleManager')[0];
      var roles = roleManager.getSelectedRoleIds();
      if (roles.length === 0) {
        if (roleManager.noRolesOnStart) {
          // if there weren't any nexus roles on load, and we're not saving
          // any - do nothing
          return;
        }
        else {
          method = 'DELETE';
        }
      }

      var url = Sonatype.config.repos.urls.userToRoles + '/' + this.form.findField('source').getValue() + '/'
            + this.form.findField('userId').getValue();

      this.form.doAction('sonatypeSubmit', {
        method : method,
        url : url,
        waitMsg : 'Updating records...',
        fpanel : this,
        dataModifiers : this.dataModifiers.submit,
        serviceDataObj : this.referenceData,
        validationModifiers : this.validationModifiers,
        isNew : this.isNew
        // extra option to send to callback, instead of conditioning on
        // method
      });
    }
  },

  // update roles if the user record with the same id is displayed in the grid
  // (auto-update doesn't work since the mapping resource does not return anything)
  submitHandler : function(form, action, receivedData) {
    var s = '', roles = [], store, rec, resourceURI,
          roleManager = this.find('name', 'roleManager')[0];

    if (this.payload.id === 'new_mapping' && this.payload.hostPanel) {
      store = this.payload.hostPanel.dataStore;
    }
    else if (this.payload.store) {
      store = this.payload.store;
    }

    if (store) {
      rec = store.getById(action.output.data.userId);
      if (!rec && this.payload.hostPanel && this.loadedUserData) {
        resourceURI = Sonatype.config.host + Sonatype.config.repos.urls.plexusUser + '/' + this.loadedUserData.userId;
        rec = new store.reader.recordType({
          //                  name : this.loadedUserData.name,
          email : this.loadedUserData.email,
          source : this.loadedUserData.source,
          userId : this.loadedUserData.userId,
          resourceURI : resourceURI,
          roles : roles,
          displayRoles : s
        }, resourceURI);
        rec.autoCreateNewRecord = true;
        store.addSorted(rec);
      }

      if (rec) {
        rec.beginEdit();
        rec.set('displayRoles', '<i>Updating...</i>');
        rec.commit();
        rec.endEdit();
        Ext.Ajax.request({
          url : Sonatype.config.host + Sonatype.config.repos.urls.plexusUser + '/' + this.loadedUserData.userId,
          success : function(response, options) {
            var i, s = '', receivedData = Ext.decode(response.responseText).data;

            for (i = 0; i < receivedData.roles.length; i += 1) {
              if (s) {
                s += ', ';
              }
              s += receivedData.roles[i].name;
            }

            rec.beginEdit();
            rec.set('roles', receivedData.roles);
            rec.set('displayRoles', s);
            rec.commit();
            rec.endEdit();
          },
          failure : function(response, options) {
            rec.beginEdit();
            rec.set('displayRoles', 'Update failed');
            rec.commit();
            rec.endEdit();
          }
        });
      }
    }
  },

  isValid : function() {
    return this.form.findField('userId').userFound && this.form.findField('source').getValue() && this.find('name',
          'roleManager')[0].validate();
  },

  loadHandler : function(form, action, receivedData) {
    this.loadedUserData = receivedData;
  },

  loadUserId : function() {
    var testField = this.form.findField('userId');
    testField.clearInvalid();
    testField.userFound = true;
    this.lastLoadedId = testField.getValue();
    var roleManager = this.find('name', 'roleManager')[0];
    roleManager.setUserId(testField.getValue());

    this.form.doAction('sonatypeLoad', {
      url : this.uri + '/' + testField.getValue(),
      method : 'GET',
      fpanel : this,
      testField : testField,
      suppressStatus : 404,
      dataModifiers : this.dataModifiers.load,
      scope : this
    });
  },

  actionFailedHandler : function(form, action) {
    if (action.response.status === 404) {
      if (action.options.testField) {
        action.options.testField.markInvalid('User record not found.');
        action.options.testField.userFound = false;
      }
      else {
        var s;
        if (this.payload && this.payload.data) {
          s = this.payload.data.source;
        }
        Sonatype.MessageBox.show({
          title : 'Error',
          msg : 'Unable to retrieve user details.' + '<br/><br/>' + 'Please make sure the ' + (s || 'selected')
                + ' realm is enabled<br/>on the server administration panel.',
          buttons : Sonatype.MessageBox.OK,
          icon : Sonatype.MessageBox.ERROR,
          animEl : 'mb3'
        });
      }
    }
    else {
      return Sonatype.repoServer.UserMappingEditor.superclass.actionFailedHandler.call(this, form, action);
    }
  },
  validationModifiers : {
    'roles' : function(error, panel) {
      Ext.getCmp('usermapping-rolemanager').markInvalid(error.msg);
    }
  }

}, function() {
  Sonatype.Events.addListener('userViewInit', function(cardPanel, rec) {
    var config = {
      payload : rec,
      tabTitle : 'Config'
    };

    if (rec.data.source !== 'default') {
      cardPanel.add(new Sonatype.repoServer.UserMappingEditor(config));
    }
  });
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, NX, Nexus*/
NX.define('Sonatype.repoServer.DefaultUserEditor', {
  extend : 'Sonatype.ext.FormPanel',
  requireSuper : false,

  requires : ['Sonatype.repoServer.UserEditPanel'],

  requirejs : ['Sonatype/all'],

  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {
      uri : Sonatype.config.repos.urls.users,
      labelWidth : 100,
      dataModifiers : {
        load : {
          roles : function(arr, srcObj, fpanel) {
            fpanel.find('name', 'roleManager')[0].setSelectedRoleIds(arr, true);
            return arr;
          }
        },
        submit : {
          roles : function(value, fpanel) {
            return fpanel.find('name', 'roleManager')[0].getSelectedRoleIds();
          },
          email : function(value, fpanel) {
            return Ext.util.Format.trim(value);
          }
        }
      }
    };
    Ext.apply(this, config, defaultConfig);

    // List of user statuses
    this.statusStore = new Ext.data.SimpleStore({
      fields : ['value', 'display'],
      data : [
        ['active', 'Active'],
        ['disabled', 'Disabled']
      ]
    });

    var ht = Sonatype.repoServer.resources.help.users;

    this.COMBO_WIDTH = 300;

    this.checkPayload();

    var items = [
      {
        xtype : 'textfield',
        fieldLabel : 'User ID',
        itemCls : 'required-field',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.userId,
        name : 'userId',
        disabled : !this.isNew,
        allowBlank : false,
        width : this.COMBO_WIDTH,
        validator : Nexus.util.Strings.validateId
      },
      {
        xtype : 'textfield',
        fieldLabel : 'First Name',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.firstName,
        name : 'firstName',
        allowBlank : false,
        itemCls : 'required-field',
        htmlDecode : true,
        width : this.COMBO_WIDTH,
        validator : function(v) {
          if (v && v.length !== 0 && v.match(Sonatype.repoServer.UserEditPanel.WHITE_SPACE_REGEX)) {
            return true;
          }

          return 'First Name cannot start with whitespace.';
        }
      },
      {
        xtype : 'textfield',
        fieldLabel : 'Last Name',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.lastName,
        name : 'lastName',
        allowBlank : false,
        itemCls : 'required-field',
        htmlDecode : true,
        width : this.COMBO_WIDTH,
        validator : function(v) {
          if (v && v.length !== 0 && v.match(Sonatype.repoServer.UserEditPanel.WHITE_SPACE_REGEX)) {
            return true;
          }
          return 'Last Name cannot start with whitespace.';
        }
      },
      {
        xtype : 'textfield',
        fieldLabel : 'Email',
        itemCls : 'required-field',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.email,
        name : 'email',
        allowBlank : false,
        width : this.COMBO_WIDTH
      },
      {
        xtype : 'combo',
        fieldLabel : 'Status',
        labelStyle : 'margin-left: 15px; width: 185px;',
        itemCls : 'required-field',
        helpText : ht.status,
        name : 'status',
        store : this.statusStore,
        displayField : 'display',
        valueField : 'value',
        editable : false,
        forceSelection : true,
        mode : 'local',
        triggerAction : 'all',
        emptyText : 'Select...',
        selectOnFocus : true,
        allowBlank : false,
        width : this.COMBO_WIDTH
      }
    ];

    if (this.isNew) {
      items.push({
        xtype : 'textfield',
        fieldLabel : 'New Password (optional)',
        inputType : 'password',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.password,
        name : 'password',
        allowBlank : true,
        width : this.COMBO_WIDTH
      });
      items.push({
        xtype : 'textfield',
        fieldLabel : 'Confirm Password',
        inputType : 'password',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.reenterPassword,
        name : 'confirmPassword',
        allowBlank : true,
        width : this.COMBO_WIDTH,
        validator : function(s) {
          var firstField = this.ownerCt.find('name', 'password')[0];
          if (firstField && firstField.getRawValue() !== s) {
            return "Passwords don't match";
          }
          return true;
        }
      });
    }

    items.push({
      xtype : 'rolemanager',
      name : 'roleManager',
      height : 200,
      width : 490,
      usePrivileges : false,
      style : 'margin-left: 15px;margin-top: 10px;border: 1px solid #B5B8C8;'
    });

    Sonatype.repoServer.DefaultUserEditor.superclass.constructor.call(this, {
      items : items,
      listeners : {
        submit : {
          fn : this.submitHandler,
          scope : this
        }
      }
    });
  },

  // FIXME parts of this look like c/p from UserMappingEditor
  combineRoles : function(val) {
    var s = '', i;
    if (val) {
      for (i = 0; i < val.length; i++) {
        if (s) {
          s += ', ';
        }
        s += this.find('name', 'roleManager')[0].getRoleNameFromId(val[i]);
      }
    }

    return s;
  },

  isValid : function() {
    return this.form.isValid() && this.find('name', 'roleManager')[0].validate();
  },

  saveHandler : function(button, event) {
    var password = this.form.getValues().password;
    this.referenceData = (this.isNew && password) ? Sonatype.repoServer.referenceData.userNew
          : Sonatype.repoServer.referenceData.users;

    return Sonatype.repoServer.DefaultUserEditor.superclass.saveHandler.call(this, button, event);
  },

  submitHandler : function(form, action, receivedData) {
    if (this.isNew) {
      receivedData.source = 'default';
      receivedData.displayRoles = this.combineRoles(receivedData.roles);
      return;
    }

    var rec = this.payload;
    rec.beginEdit();
    rec.set('firstName', receivedData.firstName);
    rec.set('lastName', receivedData.lastName);
    rec.set('email', receivedData.email);
    rec.set('status', receivedData.status);
    rec.set('displayRoles', this.combineRoles(receivedData.roles));
    rec.commit();
    rec.endEdit();
  },
  validationModifiers : {
    'roles' : function(error, panel) {
      panel.find('name', 'roleManager')[0].markInvalid(error.msg);
    }
  }
}, function() {

  Sonatype.Events.addListener('userViewInit', function(cardPanel, rec) {
    var config = {
      payload : rec,
      tabTitle : 'Config'
    };

    if (rec.data.source === 'default') {
      cardPanel.add(new Sonatype.repoServer.DefaultUserEditor(config));
    }
  });
});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
/**
 * The server settings view.
 *
 * @event serverConfigViewPostInit
 * Global event for plugins to change the server settings view. (Uses Sonatype.Events.)
 * @param {Ext.FormPanel} The form panel used as the server settings view.
 */
Ext.define('Sonatype.repoServer.ServerEditPanel', {
  extend : 'Ext.Panel',
  requires : 'Nexus.util.Strings',

  constructor : function(cfg) {

    Ext.apply(this, cfg || {}, {
      autoScroll : true
    });

    var   self = this,
          ht = Sonatype.repoServer.resources.help.server,
          formId = Ext.id(),
          tfStore = new Ext.data.SimpleStore({
            fields : ['value'],
            data : [
              ['True'],
              ['False']
            ]
          }),
          smtpConnectionSettings = new Ext.data.SimpleStore({
            fields : ['value', 'display'],
            data : [
              ['plain', 'Use plain SMTP'],
              ['ssl', 'Use Secure SMTP (SSL)'],
              ['tls', 'Use STARTTLS negotiation (TLS)']
            ]
          });

    // Simply a record to hold details of each service type
    this.realmTypeRecordConstructor = Ext.data.Record.create([
      {
        name : 'roleHint',
        sortType : Ext.data.SortTypes.asUCString
      },
      {
        name : 'description'
      }
    ]);

    this.realmTypeReader = new Ext.data.JsonReader({
      root : 'data',
      id : 'roleHint'
    }, this.realmTypeRecordConstructor);

    this.realmTypeDataStore = new Ext.data.Store({
      url : Sonatype.config.repos.urls.realmComponents,
      reader : this.realmTypeReader,
      listeners : {
        load : {
          fn : this.loadServerConfig,
          scope : this
        }
      }
    });

    this.formPanel = new Ext.FormPanel({
      region : 'center',
      id : formId,
      trackResetOnLoad : true,
      autoScroll : true,
      border : false,
      frame : true,
      collapsible : false,
      collapsed : false,
      labelWidth : 175,
      layoutConfig : {
        labelSeparator : ''
      },
      items : [
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          title : 'SMTP Settings',
          name : 'smtp-settings',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          collapsible : true,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },

          items : [
            {
              xtype : 'textfield',
              fieldLabel : 'Hostname',
              helpText : ht.smtphost,
              name : 'smtpSettings.host',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : false,
              itemCls : 'required-field'
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'Port',
              helpText : ht.smtpport,
              name : 'smtpSettings.port',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : false,
              itemCls : 'required-field'
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Username',
              helpText : ht.smtpuser,
              name : 'smtpSettings.username',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true
            },
            {
              xtype : 'textfield',
              inputType : 'password',
              fieldLabel : 'Password',
              helpText : ht.smtppass,
              name : 'smtpSettings.password',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true
            },
            {
              xtype : 'combo',
              fieldLabel : 'Connection',
              helpText : ht.connection,
              width : 210,
              store : smtpConnectionSettings,
              valueField : 'value',
              displayField : 'display',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              name : 'smtpSettings.connection',
              value : 'plain'
            },
            {
              xtype : 'textfield',
              fieldLabel : 'System Email',
              helpText : ht.smtpsysemail,
              name : 'smtpSettings.systemEmailAddress',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : false,
              itemCls : 'required-field'
            }
          ],
          buttons : [
            {
              xtype : 'button',
              scope : this,
              text : 'Test SMTP settings',
              handler : this.testSmtpBtnHandler
            }
          ],
          buttonAlign : 'left'
        },
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          title : 'HTTP Request Settings',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          collapsible : true,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },

          items : [
            {
              xtype : 'textfield',
              fieldLabel : 'User Agent Customization',
              helpText : ht.userAgentString,
              name : 'globalConnectionSettings.userAgentString',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Additional URL Parameters',
              helpText : ht.queryString,
              name : 'globalConnectionSettings.queryString',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'Request Timeout',
              helpText : ht.connectionTimeout,
              afterText : 'seconds',
              name : 'globalConnectionSettings.connectionTimeout',
              width : 50,
              allowBlank : false,
              itemCls : 'required-field',
              allowDecimals : false,
              allowNegative : false,
              maxValue : 36000
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'Request Retry Attempts',
              helpText : ht.retrievalRetryCount,
              name : 'globalConnectionSettings.retrievalRetryCount',
              width : 50,
              allowBlank : false,
              itemCls : 'required-field',
              allowDecimals : false,
              allowNegative : false,
              maxValue : 10
            }
          ]
        },
        // end http conn
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          id : formId + '_' + 'securitySettings',
          title : 'Security Settings',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          collapsible : true,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },
          items : [
            {
              xtype : 'twinpanelchooser',
              titleLeft : 'Selected Realms',
              titleRight : 'Available Realms',
              name : 'securityRealms',
              valueField : 'roleHint',
              displayField : 'description',
              store : this.realmTypeDataStore,
              required : true,
              halfSize : true
            },
            {
              xtype : 'fieldset',
              checkboxToggle : true,
              collapsed : true,
              id : formId + '_' + 'anonymousAccessSettings',
              name : 'anonymousAccessFields',
              title : 'Anonymous Access',
              anchor : Sonatype.view.FIELDSET_OFFSET,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              listeners : {
                'expand' : {
                  fn : function(panel) {
                    this.formPanel.isSecurityAnonymousAccessEnabled = true;
                    this.optionalFieldsetExpandHandler(panel);
                  },
                  scope : this
                },
                'collapse' : {
                  fn : function(panel) {
                    this.formPanel.isSecurityAnonymousAccessEnabled = false;
                    this.optionalFieldsetCollapseHandler(panel);
                  },
                  scope : this,
                  delay : 100
                }
              },

              items : [
                {
                  xtype : 'panel',
                  layout : 'fit',
                  html : '<div style="padding-bottom:10px">' + ht.anonymousAccess + '</div>'
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'Anonymous Username',
                  name : 'securityAnonymousUsername',
                  itemCls : 'required-field',
                  helpText : ht.anonUsername,
                  width : 100,
                  allowBlank : true,
                  anchor : Sonatype.view.FIELD_OFFSET
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'Anonymous Password',
                  name : 'securityAnonymousPassword',
                  itemCls : 'required-field',
                  inputType : 'password',
                  helpText : ht.anonPassword,
                  width : 100,
                  allowBlank : true,
                  minLength : 4,
                  minLengthText : "Password must be 4 characters or more",
                  maxLength : 25,
                  maxLengthText : "Password must be 25 characters or less",
                  anchor : Sonatype.view.FIELD_OFFSET
                }
              ]
            }
          ]
        },
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          collapsed : false,
          collapsible : true,
          id : formId + '_' + 'globalRestApiSettings',
          name : 'globalRestApiSettings',
          title : 'Application Server Settings',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },
          listeners : {
            'expand' : {
              fn : this.optionalFieldsetExpandHandler,
              scope : this
            },
            'collapse' : {
              fn : this.optionalFieldsetCollapseHandler,
              scope : this,
              delay : 100
            }
          },

          items : [
            {
              xtype : 'textfield',
              itemCls : 'required-field',
              fieldLabel : 'Base URL',
              helpText : ht.baseUrl,
              name : 'globalRestApiSettings.baseUrl',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true,
              validator : function(v) {
                if (!v.match(/^(?:http|https):\/\//i)) {
                  return 'Protocol must be http:// or https://';
                }

                var
                      forceCheckbox = this.ownerCt.find('name', 'globalRestApiSettings.forceBaseUrl')[0],
                      elp = this.getErrorCt();

                if (!this.allowBlank && forceCheckbox.checked && !Ext.isEmpty(v) && v.toLowerCase()
                      !== window.location.href.substring(0, v.length).toLowerCase()) {
                  if (!this.warningEl) {
                    if (!elp) {
                      return;
                    }
                    this.warningEl = elp.createChild({
                      cls : 'x-form-invalid-msg'
                    });
                    this.warningEl.setWidth(elp.getWidth(true));
                  }
                  this.warningEl.update('<b>WARNING:</b> ' + 'this Base URL setting does not match your actual URL!');
                  Ext.form.Field.msgFx[this.msgFx].show(this.warningEl, this);
                }
                else {
                  if (this.warningEl) {
                    Ext.form.Field.msgFx[this.msgFx].hide(this.warningEl, this);
                  }
                }
                return true;
              }
            },
            {
              xtype : 'checkbox',
              fieldLabel : 'Force Base URL',
              helpText : ht.forceBaseUrl,
              name : 'globalRestApiSettings.forceBaseUrl',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true,
              handler : function(checkbox, checked) {
                var baseUrlField = checkbox.ownerCt.find('name', 'globalRestApiSettings.baseUrl')[0];
                baseUrlField.validate();
              }
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'UI Timeout',
              helpText : ht.uiTimeout,
              afterText : 'seconds',
              name : 'globalRestApiSettings.uiTimeout',
              width : 50,
              allowBlank : false,
              itemCls : 'required-field',
              allowDecimals : false,
              allowNegative : false,
              maxValue : 36000
            }
          ]
        },
        {
          xtype : 'fieldset',
          checkboxToggle : true,
          collapsed : true,
          id : formId + '_' + 'remoteProxySettings.httpProxySettings',
          name : 'remoteProxySettings.httpProxySettings',
          title : 'Default HTTP Proxy Settings (optional)',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },
          listeners : {
            'expand' : {
              fn : Ext.createSequence(this.optionalFieldsetExpandHandler, function() {
                var fieldset = self.find('name', 'remoteProxySettings.httpsProxySettings')[0];
                // we need to use the DOM here because the 'checkboxToggle' is not a real Ext component
                fieldset.checkbox.dom.disabled = false;
              }),
              scope : this
            },
            'collapse' : {
              fn : Ext.createSequence(this.optionalFieldsetCollapseHandler, function() {
                var fieldset = self.find('name', 'remoteProxySettings.httpsProxySettings')[0];
                fieldset.collapse();
                // we need to use the DOM here because the 'checkboxToggle' is not a real Ext component
                fieldset.checkbox.dom.disabled = true;
              }),
              scope : this,
              delay : 100
            }
          },

          items : [
            {
              xtype : 'textfield',
              fieldLabel : 'Proxy Host',
              helpText : ht.proxyHostname,
              name : 'remoteProxySettings.httpProxySettings.proxyHostname',
              anchor : Sonatype.view.FIELD_OFFSET,
              itemCls : 'required-field',
              allowBlank : true,
              validator : function(v) {
                if (v.search(/:\//) === -1) {
                  return true;
                }

                return 'Specify hostname without the protocol, example "my.host.com"';
              }
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'Proxy Port',
              helpText : ht.proxyPort,
              name : 'remoteProxySettings.httpProxySettings.proxyPort',
              width : 50,
              itemCls : 'required-field',
              allowBlank : true,
              allowDecimals : false,
              allowNegative : false,
              maxValue : 65535
            },
            {
              xtype : 'fieldset',
              checkboxToggle : true,
              collapsed : true,
              id : formId + '_' + 'remoteProxySettings.httpProxySettings.authentication',
              name : 'remoteProxySettings.httpProxySettings.authentication',
              title : 'Authentication (optional)',
              anchor : Sonatype.view.FIELDSET_OFFSET,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              listeners : {
                'expand' : {
                  fn : this.optionalFieldsetExpandHandler,
                  scope : this
                },
                'collapse' : {
                  fn : this.optionalFieldsetCollapseHandler,
                  scope : this,
                  delay : 100
                }
              },
              items : [
                {
                  xtype : 'textfield',
                  fieldLabel : 'Username',
                  helpText : ht.username,
                  name : 'remoteProxySettings.httpProxySettings.authentication.username',
                  width : 100,
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'Password',
                  helpText : ht.password,
                  inputType : 'password',
                  name : 'remoteProxySettings.httpProxySettings.authentication.password',
                  width : 100,
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'NT LAN Host',
                  helpText : ht.ntlmHost,
                  name : 'remoteProxySettings.httpProxySettings.authentication.ntlmHost',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'NT LAN Manager Domain',
                  helpText : ht.ntlmDomain,
                  name : 'remoteProxySettings.httpProxySettings.authentication.ntlmDomain',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                }
              ]
            }, // end auth fieldset
            {
              xtype : 'textentrylist',
              name : 'nonProxyHosts',
              entryHelpText : ht.nonProxyHosts,
              entryLabel : 'Non Proxy Host',
              listLabel : 'Non Proxy Hosts'
            },
          ]
        },
        // end http proxy settings
        {
          xtype : 'fieldset',
          checkboxToggle : true,
          collapsed : true,
          id : formId + '_' + 'remoteProxySettings.httpsProxySettings',
          name : 'remoteProxySettings.httpsProxySettings',
          title : 'Default HTTPS Proxy Settings (optional, defaults to HTTP Proxy Settings )',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },
          listeners : {
            'expand' : {
              fn : this.optionalFieldsetExpandHandler,
              scope : this
            },
            'collapse' : {
              fn : this.optionalFieldsetCollapseHandler,
              scope : this,
              delay : 100
            }
          },

          items : [
            {
              xtype : 'textfield',
              fieldLabel : 'Proxy Host',
              helpText : ht.proxyHostname,
              name : 'remoteProxySettings.httpsProxySettings.proxyHostname',
              anchor : Sonatype.view.FIELD_OFFSET,
              itemCls : 'required-field',
              allowBlank : true,
              validator : function(v) {
                if (v.search(/:\//) === -1) {
                  return true;
                }

                return 'Specify hostname without the protocol, example "my.host.com"';
              }
            },
            {
              xtype : 'numberfield',
              fieldLabel : 'Proxy Port',
              helpText : ht.proxyPort,
              name : 'remoteProxySettings.httpsProxySettings.proxyPort',
              width : 50,
              itemCls : 'required-field',
              allowBlank : true,
              allowDecimals : false,
              allowNegative : false,
              maxValue : 65535
            },
            {
              xtype : 'fieldset',
              checkboxToggle : true,
              collapsed : true,
              id : formId + '_' + 'remoteProxySettings.httpsProxySettings.authentication',
              name : 'remoteProxySettings.httpsProxySettings.authentication',
              title : 'Authentication (optional)',
              anchor : Sonatype.view.FIELDSET_OFFSET,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              listeners : {
                'expand' : {
                  fn : this.optionalFieldsetExpandHandler,
                  scope : this
                },
                'collapse' : {
                  fn : this.optionalFieldsetCollapseHandler,
                  scope : this,
                  delay : 100
                }
              },
              items : [
                {
                  xtype : 'textfield',
                  fieldLabel : 'Username',
                  helpText : ht.username,
                  name : 'remoteProxySettings.httpsProxySettings.authentication.username',
                  width : 100,
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'Password',
                  helpText : ht.password,
                  inputType : 'password',
                  name : 'remoteProxySettings.httpsProxySettings.authentication.password',
                  width : 100,
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'NT LAN Host',
                  helpText : ht.ntlmHost,
                  name : 'remoteProxySettings.httpsProxySettings.authentication.ntlmHost',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  fieldLabel : 'NT LAN Manager Domain',
                  helpText : ht.ntlmDomain,
                  name : 'remoteProxySettings.httpsProxySettings.authentication.ntlmDomain',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                }
              ]
            } // end auth fieldset
          ]
        },
        // end https proxy settings
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          collapsed : false,
          collapsible : true,
          id : formId + '_' + 'systemNotificationSettings',
          name : 'systemNotificationSettings',
          title : 'System Notification Settings',
          anchor : Sonatype.view.FIELDSET_OFFSET,
          autoHeight : true,
          layoutConfig : {
            labelSeparator : ''
          },
          listeners : {
            'expand' : {
              fn : this.optionalFieldsetExpandHandler,
              scope : this
            },
            'collapse' : {
              fn : this.optionalFieldsetCollapseHandler,
              scope : this,
              delay : 100
            }
          },

          items : [
            {
              xtype : 'panel',
              layout : 'fit',
              html : '<div style="padding-bottom:10px">' + ht.systemNotification + '</div>'
            },
            {
              xtype : 'checkbox',
              fieldLabel : 'Enabled',
              helpText : ht.notificationsEnabled,
              name : 'systemNotificationSettings.enabled'
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Email Addresses',
              helpText : ht.notificationEmailAddresses,
              name : 'systemNotificationSettings.emailAddresses',
              anchor : Sonatype.view.FIELD_OFFSET,
              allowBlank : true
            },
            {
              xtype : 'rolemanager',
              name : 'systemNotificationRoleManager',
              height : 200,
              width : 505,
              usePrivileges : false,
              doValidation : false,
              style : 'margin-top: 10px;border: 1px solid #B5B8C8;'
            }
          ]
        } // end notification settings
      ],
      buttons : [
        {
          text : 'Save',
          handler : this.saveBtnHandler,
          disabled : true,
          scope : this
        },
        {
          text : 'Cancel',
          handler : this.cancelBtnHandler,
          scope : this
        }
      ]
    });

    this.formPanel.buttons[0].scope = this.formPanel;
    this.formPanel.save = this.save;

    Sonatype.repoServer.ServerEditPanel.superclass.constructor.call(this, {
      autoScroll : false,
      layout : 'border',
      items : [this.formPanel]
    });

    this.formPanel.on('beforerender', this.beforeRenderHandler, this.formPanel);
    this.formPanel.on('afterlayout', this.afterLayoutHandler, this, {
      single : true
    });
    this.formPanel.form.on('actioncomplete', this.actionCompleteHandler, this.formPanel);
    this.formPanel.form.on('actionfailed', this.actionFailedHandler, this.formPanel);

    Sonatype.Events.fireEvent('serverConfigViewPostInit', this.formPanel);
  },

  optionalFieldsetExpandHandler : function(panel) {
    panel.items.each(function(item, i, len) {
      if (item.isXType('fieldset', true)) {
        this.optionalFieldsetExpandHandler(item);
      }
      else if (item.getEl() && item.getEl().up('div.required-field', 3)) {
        item.allowBlank = false;
      }
      else {
        item.allowBlank = true;
      }
    }, this);
  },

  optionalFieldsetCollapseHandler : function(panel) {
    panel.items.each(function(item, i, len) {
      if (item.isXType('fieldset', true)) {
        this.optionalFieldsetCollapseHandler(item);
      }
      else {
        item.allowBlank = true;
      }
    }, this);
  },

  saveBtnHandler : function() {
    var allValid = this.form.isValid() && this.find('name', 'securityRealms')[0].validate() && this.find('name',
          'systemNotificationRoleManager')[0].validate();

    if (allValid) {
      this.save();
    }
  },

  save : function() {
    var form = this.form;

    form.doAction('sonatypeSubmit', {
      method : 'PUT',
      url : Sonatype.config.repos.urls.globalSettingsState,
      waitMsg : 'Updating server configuration...',
      fpanel : this,
      dataModifiers : {
        "routing.followLinks" : Nexus.util.Strings.convert.stringContextToBool,
        "routing.groups.stopItemSearchOnFirstFoundFile" : Nexus.util.Strings.convert.stringContextToBool,
        "routing.groups.mergeMetadata" : Nexus.util.Strings.convert.stringContextToBool,
        "securityRealms" : function(val, fpanel) {
          return fpanel.find('name', 'securityRealms')[0].getValue();
        },
        "systemNotificationSettings.roles" : function(val, fpanel) {
          return fpanel.find('name', 'systemNotificationRoleManager')[0].getSelectedRoleIds();
        },
        "securityAnonymousAccessEnabled" : function(val, fpanel) {
          return fpanel.isSecurityAnonymousAccessEnabled;
        },
        "remoteProxySettings.nonProxyHosts" : function(val, fpanel) {
          return fpanel.find('name', 'nonProxyHosts')[0].getEntries();
        },
        "smtpSettings.sslEnabled" : function(val, fpanel) {
          return fpanel.find('name', 'smtpSettings.connection')[0].value === 'ssl';
        },
        "smtpSettings.tlsEnabled" : function(val, fpanel) {
          return fpanel.find('name', 'smtpSettings.connection')[0].value === 'tls';
        }
      },
      serviceDataObj : Sonatype.repoServer.referenceData.globalSettingsState,
      success : Sonatype.utils.updateGlobalTimeout
    });
  },

  cancelBtnHandler : function() {
    Sonatype.view.mainTabPanel.remove(this.id, true);
  },

  beforeRenderHandler : function() {
    var sp = Sonatype.lib.Permissions;
    if (sp.checkPermission('nexus:settings', sp.EDIT)) {
      this.buttons[0].disabled = false;
    }
  },

  afterLayoutHandler : function() {
    this.realmTypeDataStore.load();

    function registerQuickTips() {
      var els = Ext.select('.required-field .x-form-item-label', this.getEl());
      els.each(function(el, els, i) {
        Ext.QuickTips.register({
          target : el,
          cls : 'required-field',
          title : '',
          text : 'Required Field',
          enabled : true
        });
      });
    }

    // register required field quicktip, but we have to wait for elements to
    // show up in DOM
    registerQuickTips.defer(300, this.formPanel);

  },

  loadServerConfig : function() {
    // if stores aren't loaded, abort, they will load again when done
    if (!this.realmTypeDataStore.lastOptions) {
      return;
    }

    var fpanel = this.formPanel;

    this.formPanel.getForm().doAction('sonatypeLoad', {
      url : Sonatype.config.repos.urls.globalSettingsState,
      method : 'GET',
      fpanel : fpanel,
      dataModifiers : {
        "routing.followLinks" : Nexus.util.Strings.capitalize,
        "routing.groups.stopItemSearchOnFirstFoundFile" : Nexus.util.Strings.capitalize,
        "routing.groups.mergeMetadata" : Nexus.util.Strings.capitalize,
        "securityRealms" : function(arr, srcObj, fpanel) {
          fpanel.find('name', 'securityRealms')[0].setValue(arr);
          return arr; // return arr, even if empty to comply with sonatypeLoad data modifier requirement
        },
        "systemNotificationSettings.roles" : function(arr, srcObj, fpanel) {
          fpanel.find('name', 'systemNotificationRoleManager')[0].setSelectedRoleIds(arr, true);
          return arr;
        },
        "securityAnonymousAccessEnabled" : function(arr, srcObj, fpanel) {
          fpanel.isSecurityAnonymousAccessEnabled = arr;
        },
        "remoteProxySettings.nonProxyHosts" : function(arr, srcObj, fpanel) {
          fpanel.find('name', 'nonProxyHosts')[0].setEntries(arr);
        },
        "smtpSettings.sslEnabled" : function(arr, srcObj, fpanel) {
          if (arr) {
            fpanel.find('name', 'smtpSettings.connection')[0].setValue('ssl');
          }
        },
        "smtpSettings.tlsEnabled" : function(arr, srcObj, fpanel) {
          if (arr) {
            fpanel.find('name', 'smtpSettings.connection')[0].setValue('tls');
          }
        }
      }
    });
  },

  testSmtpBtnHandler : function() {
    var data, w, fpanel = this.formPanel;

    data = {
      testEmail : '',
      host : fpanel.form.findField('smtpSettings.host').getValue(),
      port : fpanel.form.findField('smtpSettings.port').getValue(),
      username : fpanel.form.findField('smtpSettings.username').getValue(),
      password : fpanel.form.findField('smtpSettings.password').getValue(),
      systemEmailAddress : fpanel.form.findField('smtpSettings.systemEmailAddress').getValue(),
      sslEnabled : fpanel.form.findField('smtpSettings.connection').getValue() === 'ssl',
      tlsEnabled : fpanel.form.findField('smtpSettings.connection').getValue() === 'tls'
    };

    w = new Ext.Window({
      title : 'Validate SMTP settings',
      closable : true,
      autoWidth : false,
      width : 350,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      draggable : false,
      items : [
        {
          xtype : 'form',
          labelWidth : 60,
          frame : true,
          defaultType : 'textfield',
          monitorValid : true,
          items : [
            {
              xtype : 'panel',
              style : 'padding-left: 70px; padding-bottom: 10px',
              html : 'Please enter an email address which will receive the test email message.'
            },
            {
              fieldLabel : 'E-mail',
              name : 'email',
              width : 200,
              allowBlank : false
            }
          ],
          buttons : [
            {
              text : 'Validate',
              formBind : true,
              scope : this,
              handler : function() {
                var email = w.find('name', 'email')[0].getValue(),
                    mask = new Ext.LoadMask(w.el, {
                      msg : 'Validating SMTP settings...',
                      removeMask : true
                    });

                mask.show();

                this.runStmpConfigCheck(email, data, function(success) {
                  mask.hide();

                  if (success) {
                    w.close();
                  }
                });
              }
            },
            {
              text : 'Cancel',
              formBind : false,
              scope : this,
              handler : function() {
                w.close();
              }
            }
          ]
        }
      ]
    });

    w.show();
  },

  runStmpConfigCheck : function(testEmail, data, callback) {

    data.testEmail = testEmail;

    Ext.Ajax.request({
      method : 'PUT',
      url : Sonatype.config.repos.urls.smtpSettingsState,
      jsonData : {
        data : data
      },
      callback : function(options, success) {
        callback(success);
      },
      success : function() {
          Sonatype.MessageBox.show({
            title : 'SMTP configuration',
            msg : 'SMTP configuration validated successfully, check your inbox!',
            buttons : Sonatype.MessageBox.OK,
            icon : Sonatype.MessageBox.INFO
          });
      },
      failure : function(response) {
        Sonatype.utils.connectionError(response, 'Error on SMTP validation!', false, {
          hideErrorStatus : true,
          decodeErrorResponse : true
        });
      },
      scope : this
    });
  },

  // (Ext.form.BasicForm, Ext.form.Action)
  actionCompleteHandler : function(form, action) {
    if (action.type === 'sonatypeLoad') {
      // @note: this is a work around to get proper use of the isDirty()
      // function of this field
      if (action.options.fpanel.isSecurityAnonymousAccessEnabled) {
        action.options.fpanel.find('id', (action.options.fpanel.id + '_' + 'anonymousAccessSettings'))[0].expand();
      }
    }
  },

  // (Ext.form.BasicForm, Ext.form.Action)
  actionFailedHandler : function(form, action) {
    if (action.failureType === Ext.form.Action.CLIENT_INVALID) {
      Sonatype.MessageBox.alert('Missing or Invalid Fields',
            'Please change the missing or invalid fields.').setIcon(Sonatype.MessageBox.WARNING);
    }
    else if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
      Sonatype.utils.connectionError(action.response, 'There is an error communicating with the server.');
    }
    else if (action.failureType === Ext.form.Action.LOAD_FAILURE) {
      Sonatype.MessageBox.alert('Load Failure',
            'The data failed to load from the server.').setIcon(Sonatype.MessageBox.ERROR);
    }
  },

  anonymousCheckHandler : function(checkbox, checked) {
    this.ownerCt.find('name', 'securityAnonymousUsername')[0].setDisabled(!checked);
    this.ownerCt.find('name', 'securityAnonymousPassword')[0].setDisabled(!checked);
  }

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype*/
NX.define('Sonatype.repoServer.Documentation', {
  extend : 'Ext.Panel',
  requirejs : ['extjs', 'sonatype'],

  constructor : function(cfg) {
    Ext.apply(this, cfg || {});

    Sonatype.repoServer.Documentation.superclass.constructor.call(this,
          {
            layout : 'border',
            autoScroll : false,
            width : '100%',
            height : '100%',
            items : [
              {
                xtype : 'panel',
                region : 'center',
                layout : 'fit',
                html : this.getHelpText()
              }
            ]
          });
  },

    getHelpText : function() {
      return '<div class="little-padding">'
            + '<h1>Nexus Repository Manager</h1>'
            + '<br><a href="http://nexus.sonatype.org/" target="_new">Nexus Community Website</a>'
            + '<br><a href=" http://links.sonatype.com/products/nexus/pro/home " target="_new">Nexus Repository Manager Website</a>'
            + '<br><a href="http://links.sonatype.com/products/nexus/pro/trial" target="_new">Nexus Repository Manager Trial - including guide and example projects</a>'
            + '<br><a href="http://links.sonatype.com/products/nexus/pro/kb" target="_new">Knowledge Base</a>'
            + '<br><a href="http://links.sonatype.com/products/nexus/pro/training" target="_new">Training</a>'
            + '<br><a href="http://links.sonatype.com/products/nexus/oss/docs" target="_new">Repository Management With Nexus - free comprehensive book</a>'
            + '<br><a href="http://links.sonatype.com/products/maven/docs" target="_new">Maven: The Complete Reference and other books</a>'
            + '<br><a href="http://links.sonatype.com/products/nexus/oss/release-notes" target="_new">Release Notes</a>'
            + '</div>';
    }

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Sonatype, Ext*/
NX.define('Sonatype.repoServer.HelpAboutPanel', {
  extend : 'Ext.Panel',
  requireSuper : false,
  requirejs : ['Sonatype/all'],
  constructor : function(config) {
    Ext.apply(this, config || {});

    var helpItems = [
      {
        //add this one by default, so existing usage will be preserved
        text : this.getHelpText()
      }
    ];

    Sonatype.Events.fireEvent('aboutPanelContributions', helpItems);

    var i, helpText = '';

    for (i = 0; i < helpItems.length; i++) {
      helpText += helpItems[i].text;
    }

    Sonatype.repoServer.HelpAboutPanel.superclass.constructor.call(this, {
      layout : 'border',
      autoScroll : false,
      width : '100%',
      height : '100%',
      items : [
        {
          xtype : 'panel',
          region : 'center',
          layout : 'fit',
          html : helpText
        }
      ]
    });
  },

  getHelpText : function() {
    return '<div class="little-padding">'
          + 'Nexus Repository Manager'
          + '<br/>Copyright &copy; 2008-present Sonatype, Inc.'
          + '<br/>All rights reserved. Includes the third-party code listed at <a href="'
          + Sonatype.utils.attributionsURL + '" target="_new">' + Sonatype.utils.attributionsURL + '</a>.'
          + '<br/>'
          + '<br/>This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,'
          + '<br/>which accompanies this distribution and is available at <a href="http://www.eclipse.org/legal/epl-v10.html" target="_new">http://www.eclipse.org/legal/epl-v10.html</a>.'
          + '<br/>'
          + '<br/>Nexus Repository Manager is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks'
          + '<br/>of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the'
          + '<br/>Eclipse Foundation. All other trademarks are the property of their respective owners.'
          + '</div>';
  }

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Nexus, Ext, Sonatype*/
/*
 * User Edit/Create panel layout and controller
 */
NX.define('Sonatype.repoServer.UserEditPanel', {
  extend : 'Sonatype.panels.GridViewer',
  // FIXME: remove requireSuper once GridViewer uses NX.define
  requireSuper : false,

  requirejs : ['Sonatype/all'],

  statics : {
    WHITE_SPACE_REGEX : /^\S/
  },

  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {
      title : 'Users'
    };
    Ext.apply(this, config, defaultConfig);

    this.sp = Sonatype.lib.Permissions;

    this.actions = {
      resetPasswordAction : {
        text : 'Reset Password',
        scope : this,
        handler : this.resetPasswordHandler
      },
      changePasswordAction : {
        text : 'Set Password',
        scope : this,
        handler : this.changePasswordHandler
      }
    };

    this.displaySelector = new Ext.Button({
      text : 'All Configured Users',
      icon : Sonatype.config.resourcePath + '/static/images/icons/page_white_stack.png',
      cls : 'x-btn-text-icon',
      value : 'allConfigured',
      menu : {
        // no context menu,
        itemId : 'user-realm-selector-menu',
        items : [
          {
            text : 'All Users',
            checked : false,
            handler : this.showUsers,
            value : 'all',
            group : 'user-realm-selector',
            scope : this
          },
          {
            text : 'All Authorized Users',
            checked : false,
            handler : this.showUsers,
            value : 'effectiveUsers',
            group : 'user-realm-selector',
            scope : this
          }
        ]
      }
    });

    this.sourceStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'roleHint',
      autoLoad : true,
      url : Sonatype.config.repos.urls.userLocators,
      sortInfo : {
        field : 'description',
        direction : 'ASC'
      },
      fields : [
        {
          name : 'roleHint'
        },
        {
          name : 'description',
          sortType : Ext.data.SortTypes.asUCString
        }
      ],
      listeners : {
        load : {
          fn : this.loadSources,
          scope : this
        }
      }
    });

    this.searchField = new Ext.app.SearchField({
      searchPanel : this,
      disabled : true,
      width : 250,
      emptyText : 'Select a filter to enable username search',

      onTrigger2Click : function() {
        var v = this.getRawValue();
        this.searchPanel.startSearch(this.searchPanel);
      }
    });

    Sonatype.Events.on('userAddMenuInit', this.onAddMenuInit, this);
    Sonatype.Events.on('userMenuInit', this.onUserMenuInit, this);

    Sonatype.repoServer.UserEditPanel.superclass.constructor.call(this, {
      emptyText: 'No users defined',
      emptyTextWhileFiltering: 'No users matched criteria: {criteria}',

      addMenuInitEvent : 'userAddMenuInit',
      deleteButton : this.sp.checkPermission('security:users', this.sp.DELETE),
      rowClickEvent : 'userViewInit',
      rowContextClickEvent : 'userMenuInit',
      url : Sonatype.config.repos.urls.plexusUsersAllConfigured,
      dataAutoLoad : true,
      dataId : 'userId',
      dataBookmark : 'userId',
      dataSortInfo : {
        field : 'firstName',
        direction : 'asc'
      },
      columns : [
        {
          name : 'resourceURI',
          mapping : 'userId',
          convert : function(value, parent) {
            return parent.source === 'default' ? (Sonatype.config.repos.urls.users + '/' + value)
                  : (Sonatype.config.repos.urls.plexusUser + '/' + parent.source + '/' + value);
          }
        },
        {
          name : 'userId',
          sortType : Ext.data.SortTypes.asUCString,
          header : 'User ID',
          width : 100
        },
        {
          name : 'source',
          header : 'Realm',
          width : 50
        },
        {
          name : 'firstName',
          sortType : Ext.data.SortTypes.asUCString,
          header : 'First Name',
          width : 175
        },
        {
          name : 'lastName',
          sortType : Ext.data.SortTypes.asUCString,
          header : 'Last Name',
          width : 175
        },
        {
          name : 'email',
          header : 'Email',
          width : 175
        },
        {
          name: 'status',
          header: 'Status',
          width: 60,
          renderer: function(value, metaData, record, rowIndex, colIndex, store) {
            // edit panel displays data as capitalized, so we match it here
            return Ext.util.Format.capitalize(value);
          }
        },
        {
          name : 'roles'
        },
        {
          name : 'displayRoles',
          mapping : 'roles',
          convert : this.combineRoles.createDelegate(this),
          header : 'Roles',
          autoExpand : true
        }
      ],
      listeners : {
        beforedestroy : {
          fn : function() {
            Sonatype.Events.un('userAddMenuInit', this.onAddMenuInit, this);
            Sonatype.Events.un('userMenuInit', this.onUserMenuInit, this);
          },
          scope : this
        }
      },
      tbar : [' ', this.displaySelector, this.searchField]
    });

    this.titleColumn = 'userId';

  },

  combineRoles : function(val, parent) {
    var s = '', i;
    if (val) {
      for (i = 0; i < val.length; i++) {
        var roleName = val[i].name;
        if (s) {
          s += ', ';
        }
        s += roleName;
      }
    }

    return s;
  },

  changePasswordHandler : function(rec) {
    var userId = rec.get('userId');

    var w = new Ext.Window({
      id : 'set-password-window',
      title : 'Set Password',
      closable : true,
      autoWidth : false,
      width : 350,
      autoHeight : true,
      modal : true,
      constrain : true,
      resizable : false,
      draggable : false,
      items : [
        {
          xtype : 'form',
          labelAlign : 'right',
          labelWidth : 110,
          frame : true,
          defaultType : 'textfield',
          monitorValid : true,
          items : [
            {
              xtype : 'panel',
              style : 'padding-left: 70px; padding-bottom: 10px',
              html : 'Enter a new password for user ' + userId
            },
            {
              fieldLabel : 'New Password',
              inputType : 'password',
              name : 'newPassword',
              width : 200,
              allowBlank : false
            },
            {
              fieldLabel : 'Confirm Password',
              inputType : 'password',
              name : 'confirmPassword',
              width : 200,
              allowBlank : false,
              validator : function(s) {
                var firstField = this.ownerCt.find('name', 'newPassword')[0];
                if (firstField && firstField.getRawValue() != s) {
                  return "Passwords don't match";
                }
                return true;
              }
            }
          ],
          buttons : [
            {
              text : 'Set Password',
              formBind : true,
              scope : this,
              handler : function() {
                var newPassword = w.find('name', 'newPassword')[0].getValue();

                Ext.Ajax.request({
                  scope : this,
                  method : 'POST',
                  cbPassThru : {
                    userId : userId,
                    newPassword : newPassword
                  },
                  jsonData : {
                    data : {
                      userId : userId,
                      newPassword : newPassword
                    }
                  },
                  url : Sonatype.config.repos.urls.usersSetPassword,
                  success : function(response, options) {
                    w.close();
                    Sonatype.MessageBox.show({
                      id : 'password-changed-messagebox',
                      title : 'Password Changed',
                      msg : 'Password change request completed successfully.',
                      buttons : Sonatype.MessageBox.OK,
                      icon : Sonatype.MessageBox.INFO,
                      animEl : 'mb3'
                    });
                  },
                  failure : function(response, options) {
                    Sonatype.utils.connectionError(response, 'There is a problem changing the password.')
                  }
                });
              }
            },
            {
              text : 'Cancel',
              formBind : false,
              scope : this,
              handler : function() {
                w.close();
              }
            }
          ]
        }
      ],
      listeners : {
        show : function(c) {
          var field = c.find('name', 'newPassword')[0].focus(false, 100);
        }
      }
    });

    w.show();
  },

  refreshHandler : function(button, e) {
    this.clearCards();
    if (this.lastUrl) {
      this.searchByUrl();
    }
    else {
      this.dataStore.reload();
    }
  },

  resetPasswordHandler : function(rec) {
    if (rec.data.resourceURI != 'new') {
      Sonatype.utils.defaultToNo();

      Sonatype.MessageBox.show({
        animEl : this.gridPanel.getEl(),
        title : 'Reset user password?',
        msg : 'Reset the ' + rec.get('userId') + ' user password?',
        buttons : Sonatype.MessageBox.YESNO,
        scope : this,
        icon : Sonatype.MessageBox.QUESTION,
        fn : function(btnName) {
          if (btnName == 'yes' || btnName == 'ok') {
            Ext.Ajax.request({
              callback : this.resetPasswordCallback,
              cbPassThru : {
                resourceId : rec.id
              },
              scope : this,
              method : 'DELETE',
              url : Sonatype.config.repos.urls.usersReset + '/' + rec.data.userId
            });
          }
        }
      });
    }
  },

  resetPasswordCallback : function(options, isSuccess, response) {
    if (isSuccess) {
      Sonatype.MessageBox.alert('Password Reset', 'The password has been reset.');
    }
    else {
      Sonatype.utils.connectionError(response, 'The server did not reset the password.');
    }
  },

  searchByUrl : function() {
    this.clearAll();
    if (this.warningLabel) {
      this.warningLabel.destroy();
      this.warningLabel = null;
    }
    this.gridPanel.loadMask.show();
    Ext.Ajax.request({
      scope : this,
      url : this.lastUrl,
      callback : function(options, success, response) {
        this.gridPanel.loadMask.hide();
        if (success) {
          var r = Ext.decode(response.responseText);
          if (r.data) {
            this.dataStore.loadData(r);
          }
        }
      }
    });
  },

  showUsers : function(button, e) {
    this.displaySelector.setText(button.text);
    this.displaySelector.value = button.value;
    this.clearAll();

    if (button.value == 'allConfigured') {
      this.searchField.emptyText = 'Select a filter to enable username search';
      this.searchField.setValue('');
      this.searchField.disable();
      this.calculateSearchUrl(this);
      this.searchByUrl();
    }
    else {
      this.searchField.emptyText = 'Enter a username or leave blank to display all';
      this.searchField.setValue('');
      this.searchField.enable();
      this.calculateSearchUrl(this);
      if (!this.warningLabel) {
        this.warningLabel =
              this.getTopToolbar().addText('<span class="x-toolbar-warning">Click "Search" to refresh the view</span>');
      }
    }
  },

  startSearch : function(panel) {
    panel.calculateSearchUrl(panel);
    panel.searchByUrl();
  },

  calculateSearchUrl : function(panel) {
    var url = Sonatype.config.repos.urls.searchUsers;
    var prefix = '/' + panel.displaySelector.value;
    var suffix = '';

    // this is somewhat ugly, since effectiveUsers is a flag, not a locator
    if (panel.displaySelector.value == 'effectiveUsers') {
      prefix = '/all';
      suffix = '?effectiveUsers=true';
    }

    var v = '';
    if (panel.searchField.disabled) {
      url = Sonatype.config.repos.urls.plexusUsers;
    }
    else {
      v = panel.searchField.getValue();
    }

    if (v.length > 0) {
      panel.lastUrl = url + prefix + '/' + v + suffix;
    }
    else {
      panel.lastUrl = url + prefix + suffix;
    }
  },

  stopSearch : function(panel) {
    panel.searchField.setValue(null);
    panel.searchField.triggers[0].hide();
  },

  onAddMenuInit : function(menu) {
    menu.add('-');
    if (this.sp.checkPermission('security:users', this.sp.CREATE)) {
      menu.add({
        text : 'Nexus User',
        autoCreateNewRecord : true,
        handler : function(container, rec, item, e) {
          rec.beginEdit();
          rec.set('source', 'default');
          rec.commit();
          rec.endEdit();
        },
        scope : this
      });
    }
    menu.add({
      text : 'External User Role Mapping',
      handler : this.mapRolesHandler,
      scope : this
    });
  },

  onUserMenuInit : function(menu, userRecord) {
    if (userRecord.data.source == 'default' // &&
    // userRecord.data.userManaged
    // == true
          ) {

      if (userRecord.data.resourceURI.substring(0, 4) != 'new_') {
        if (this.sp.checkPermission('security:usersreset', this.sp.DELETE)) {
          menu.add(this.actions.resetPasswordAction);
        }

        if (this.sp.checkPermission('security:users', this.sp.EDIT)) {
          menu.add(this.actions.changePasswordAction);
        }
      }
    }
  },

  mapRolesHandler : function(button, e) {
    this.createChildPanel({
      id : 'new_mapping',
      hostPanel : this,
      data : {
        name : 'User Role Mapping'
      }
    }, true);
  },

  deleteRecord : function(rec) {
    if (rec.data.source == 'default') {
      return Sonatype.repoServer.UserEditPanel.superclass.deleteRecord.call(this, rec);
    }
    else {
      Ext.Ajax.request({
        callback : function(options, success, response) {
          if (success) {
            this.dataStore.remove(rec);
          }
          else {
            Sonatype.utils.connectionError(response, 'Delete Failed!');
          }
        },
        scope : this,
        suppressStatus : 404,
        method : 'DELETE',
        url : Sonatype.config.repos.urls.userToRoles + '/' + rec.data.source + '/' + rec.data.userId
      });
    }
  },

  deleteActionHandler : function(button, e) {
    if (this.gridPanel.getSelectionModel().hasSelection()) {
      var rec = this.gridPanel.getSelectionModel().getSelected();
      if (rec.data.source == 'default') {
        return Sonatype.repoServer.UserEditPanel.superclass.deleteActionHandler.call(this, button, e);
      }
      else {
        var roles = rec.data.roles;
        if (roles) {
          for (var i = 0; i < roles.length; i++) {
            if (roles[i].source == 'default') {
              Sonatype.utils.defaultToNo();

              Sonatype.MessageBox.show({
                animEl : this.gridPanel.getEl(),
                title : 'Delete',
                msg : 'Delete Nexus role mapping for ' + rec.data[this.titleColumn] + '?',
                buttons : Sonatype.MessageBox.YESNO,
                scope : this,
                icon : Sonatype.MessageBox.QUESTION,
                fn : function(btnName) {
                  if (btnName == 'yes' || btnName == 'ok') {
                    this.deleteRecord(rec);
                  }
                }
              });
              return;
            }
          }
        }
        Sonatype.MessageBox.show({
          animEl : this.gridPanel.getEl(),
          title : 'Delete',
          msg : 'This user does not have any Nexus roles mapped.',
          buttons : Sonatype.MessageBox.OK,
          scope : this,
          icon : Sonatype.MessageBox.WARNING
        });
      }
    }
  },

  loadSources : function(store, records, options) {
    var menu = this.displaySelector.menu;

    for (var i = 0; i < records.length; i++) {
      var rec = records[i];
      var v = rec.data.roleHint;
      if (v != 'mappedExternal') {
        menu.addMenuItem({
          text : v == 'default' ? 'Default Realm Users' : rec.data.description,
          value : v,
          checked : v == 'allConfigured',
          handler : this.showUsers,
          group : 'user-realm-selector',
          scope : this
        });
      }
    }
  }
},

function() {
  var views = [];
  var viewsCollected = false;

  // NXCM-4099 plugin-contributed tabs
  Sonatype.Events.addListener('userViewInit', function(cardPanel, rec) {
    if (!viewsCollected) {
      Sonatype.Events.fireEvent('userAdminViewInit', views);
      viewsCollected = true;
    }

    for (var i = 0; i < views.length; i++) {
      var view = views[i];

      // don't add views for 'new user':
      // if you add a Nexus user, you get a record without username
      // if you add a role mapping, you get a js object (without even #get(string))
      var username = undefined;

      if (rec.get) {
        username = rec.get("userId");
      }

      if (username === undefined) {
        return;
      }

      var content = new view.item({username : username});
      content.tabTitle = view.name;
      if (!content.shouldShow || content.shouldShow()) {
        cardPanel.add(content);
      }
    }
  });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, NX, Sonatype*/

/*
 * View Nexus serer XML configuration file
 */
NX.define('Sonatype.repoServer.ConfigViewPanel', {
  requirejs : ['Sonatype/all', 'Nexus/config'],

  extend : 'Ext.form.FormPanel',

  constructor : function(cfg) {
    Ext.apply(this, cfg || {});

    this.listeners = {
      // note: this isn't pre-render dependent, we just need an early event to
      // start this off
      'beforerender' : this.getConfigFile,
      scope : this
    };

    Sonatype.repoServer.ConfigViewPanel.superclass.constructor.call(this, {
      autoScroll : false,
      border : false,
      frame : false,
      collapsible : false,
      collapsed : false,
      tbar : [
        {
          text : 'Refresh',
          iconCls : 'st-icon-refresh',
          cls : 'x-btn-text-icon',
          tooltip : {
            text : 'Reloads the config file'
          },
          scope : this,
          handler : this.getConfigFile
        },
        {
          text : 'Download Config',
          icon : Sonatype.config.resourcePath + '/static/images/icons/page_white_put.png',
          cls : 'x-btn-text-icon',
          scope : this,
          handler : function() {
            Sonatype.utils.openWindow(Sonatype.config.repos.urls.configCurrent);
          }
        }
      ],
      items : [
        {
          xtype : 'textarea',
          id : 'config-text',
          readOnly : true,
          hideLabel : true,
          anchor : '100% 100%'
        }
      ]
    });

    this.configTextArea = this.findById('config-text');
  },

  getConfigFile : function() {
    Ext.Ajax.request({
      callback : this.renderResponse,
      scope : this,
      method : 'GET',
      headers : {
        'accept' : 'application/xml'
      },
      url : Sonatype.config.repos.urls.configCurrent
    });
  },

  renderResponse : function(options, success, response) {
    if (success) {
      this.configTextArea.setRawValue(response.responseText);
    }
    else {
      Ext.MessageBox.alert('The data failed to load from the server.');
    }
  }

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Sonatype, Ext*/
//define('repoServer/FileUploadPanel', ['extjs', 'Sonatype/all'], function(Ext, Sonatype) {
NX.define('Sonatype.repoServer.ArtifactUploadPanel', {
  extend : 'Ext.FormPanel',
  requirejs : ['Sonatype/all'],
  mixins : ['Nexus.LogAwareMixin'],

  constructor : function(cfg) {
    Ext.apply(this, cfg || {});

    var
          ht = Sonatype.repoServer.resources.help.artifact,
          packagingStore = new Ext.data.SimpleStore({
            fields : ['value'],
            data : [
              ['pom'],
              ['jar'],
              ['ejb'],
              ['war'],
              ['ear'],
              ['rar'],
              ['par'],
              ['maven-archetype'],
              ['maven-plugin']
            ]
          });

    this.fileInput = null;
    this.pomInput = null;

    this.uploadModeTitle = this.uploadModeTitle || 'Select GAV Definition Source';
    this.uploadModeLabel = this.uploadModeLabel || 'GAV Definition';
    this.uploadModeStore = this.uploadModeStore || new Ext.data.SimpleStore({
      fields : ['value', 'display'],
      data : [
        ['pom', 'From POM'],
        ['gav', 'GAV Parameters']
      ]
    });
    this.uploadModeHt = this.uploadModeHt || ht.gavDefinition;
    this.uploadModeExplanation = this.uploadModeExplanation ||
          'Select a source for the GAV definition. GAV can be specified either manually or from a POM file. These settings will be applied to all artifacts specified below.';

    this.extraItems = this.extraItems || {};

    Sonatype.repoServer.ArtifactUploadPanel.superclass.constructor.call(this, {
      region : 'center',
      id : this.id || 'uploadFormId',
      trackResetOnLoad : true,
      autoScroll : true,
      bodyStyle : 'overflow:auto;',
      border : false,
      frame : true,
      collapsible : false,
      collapsed : false,
      width : '100%',
      fileUpload : true,
      layoutConfig : {
        labelSeparator : ''
      },
      items : [
        {
          xtype : 'hidden',
          name : 'r',
          value : this.payload.id
        },
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          title : this.uploadModeTitle,
          collapsible : false,
          autoHeight : true,
          width : '95%',
          items : [
            {
              xtype : 'combo',
              lazyInit : false,
              fieldLabel : this.uploadModeLabel,
              itemCls : 'required-field',
              helpText : this.uploadModeHt,
              name : 'gavDefinition',
              store : this.uploadModeStore,
              valueField : 'value',
              displayField : 'display',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              selectOnFocus : true,
              emptyText : 'Select...',
              allowBlank : false,
              listeners : {
                select : {
                  fn : this.gavDefinitionSelectHandler,
                  scope : this
                }
              }
            },
            {
              xtype : 'panel',
              id : 'gav-definition-card-panel',
              header : false,
              layout : 'card',
              region : 'center',
              activeItem : 0,
              deferredRender : false,
              autoScroll : false,
              frame : false,
              items : [
                {
                  xtype : 'fieldset',
                  checkboxToggle : false,
                  border : false,
                  anchor : Sonatype.view.FIELDSET_OFFSET,
                  collapsible : false,
                  autoHeight : true,
                  layoutConfig : {
                    labelSeparator : ''
                  },
                  items : [
                    {
                      xtype : 'label',
                      html : this.uploadModeExplanation
                    }
                  ]
                },
                {
                  xtype : 'fieldset',
                  border : false,
                  checkboxToggle : false,
                  collapsible : false,
                  autoHeight : true,
                  style : 'margin-right:7px;',
                  items : [
                    {
                      hideLabel : true,
                      name : 'uploadPomButton',
                      xtype : 'browsebutton',
                      text : 'Select POM to Upload...',
                      style : 'margin-bottom: 5px;',
                      uploadPanel : this,
                      handler : function(b) {
                        b.uploadPanel.pomInput = b.detachInputFile();
                        var filename = b.uploadPanel.pomInput.getValue();
                        b.uploadPanel.updatePomFilename(b.uploadPanel, filename);
                      }
                    },
                    {
                      xtype : 'textfield',
                      fieldLabel : 'POM Filename',
                      name : 'pomnameField',
                      anchor : Sonatype.view.FIELD_OFFSET,
                      readOnly : true,
                      allowBlank : false,
                      itemCls : 'required-field',
                      disabled : true,
                      regex : /^(.*(\/|\\)pom.xml)|(.*\.pom)|(pom.xml)$/i,
                      regexText : 'POM file name must be pom.xml or file extension must be .pom'
                    },
                    {
                      xtype : 'label',
                      text : 'Note that if you only intend to upload a POM file, there is no need to add additional artifacts below, simply click the Upload Artifact(s) button.'
                    }
                  ]
                },
                {
                  xtype : 'fieldset',
                  border : false,
                  checkboxToggle : false,
                  collapsible : false,
                  style : 'margin-right:7px;',
                  autoHeight : true,
                  items : [
                    {
                      xtype : 'checkbox',
                      fieldLabel : 'Auto Guess',
                      checked : true,
                      name : 'autoguess',
                      helpText : ht.autoguess,
                      listeners : {
                        'check' : {
                          fn : function(checkbox, value) {
                            this.updateFilename(this);
                          },
                          scope : this
                        }
                      },
                      disabled : true
                    },
                    {
                      xtype : 'textfield',
                      fieldLabel : 'Group',
                      itemCls : 'required-field',
                      helpText : ht.groupId,
                      anchor : Sonatype.view.FIELD_OFFSET,
                      name : 'g',
                      allowBlank : false,
                      disabled : true,
                      validator : function(v) {
                        if (!/^[\w\.\-]+$/.test(v)) {
                          return 'Group ID is illegal, only letters, numbers, underscore(_), hyphon(-), and dot(.) are allowed.';
                        }
                        return true;
                      }
                    },
                    {
                      xtype : 'textfield',
                      fieldLabel : 'Artifact',
                      itemCls : 'required-field',
                      helpText : ht.artifactId,
                      anchor : Sonatype.view.FIELD_OFFSET,
                      name : 'a',
                      allowBlank : false,
                      disabled : true,
                      validator : function(v) {
                        if (!/^[\w\.\-]+$/.test(v)) {
                          return 'Artifact ID is illegal, only letters, numbers, underscore(_), hyphon(-), and dot(.) are allowed.';
                        }
                        return true;
                      }
                    },
                    {
                      xtype : 'textfield',
                      fieldLabel : 'Version',
                      itemCls : 'required-field',
                      helpText : ht.version,
                      anchor : Sonatype.view.FIELD_OFFSET,
                      name : 'v',
                      allowBlank : false,
                      uploadPanel : this,
                      validator : function(v) {
                        var
                              isSnapshotVersion = /-SNAPSHOT$/.test(v) || /LATEST$/.test(v)
                                    || /^(.*)-([0-9]{8}.[0-9]{6})-([0-9]+)$/.test(v),
                              isSnapshotRepo = this.uploadPanel.payload.data.repoPolicy === 'snapshot';

                        if (isSnapshotRepo) {
                          if (!isSnapshotVersion) {
                            return 'You cannot upload a release version into a snapshot repository';
                          }
                        } else {
                          if (isSnapshotVersion) {
                            return 'You cannot upload a snapshot version into a release repository';
                          }
                        }
                        if (!/^[\w\.\-]+$/.test(v)) {
                          return 'Version is illegal, only letters, numbers, underscore(_), hyphon(-), and dot(.) are allowed.';
                        }
                        return true;
                      },
                      disabled : true
                    },
                    {
                      xtype : 'combo',
                      fieldLabel : 'Packaging',
                      itemCls : 'required-field',
                      helpText : ht.packaging,
                      store : packagingStore,
                      displayField : 'value',
                      editable : true,
                      forceSelection : false,
                      mode : 'local',
                      triggerAction : 'all',
                      emptyText : 'Select...',
                      selectOnFocus : true,
                      allowBlank : false,
                      name : 'p',
                      width : 150,
                      listWidth : 150,
                      disabled : true
                    }
                  ]
                }
              ]
            }
          ]
        },

        {
          xtype : 'fieldset',
          id : 'artifacts-upload-fieldset',
          checkboxToggle : false,
          title : 'Select Artifact(s) for Upload',
          collapsible : false,
          autoHeight : true,
          style : 'margin-right:7px;width:95%;',
          items : [
            {
              hideLabel : true,
              xtype : 'browsebutton',
              text : 'Select Artifact(s) to Upload...',
              style : 'margin-bottom: 5px;',
              uploadPanel : this,
              name : 'uploadArtifactButton',
              handler : function(b) {
                b.uploadPanel.fileInput = b.detachInputFile();
                var
                      filename = b.uploadPanel.fileInput.getValue(),
                      endStr = '.pom';
                if (filename === "pom.xml" || (filename.indexOf(endStr) === filename.length - endStr.length )) {
                  Ext.Msg.show({
                    title : 'Error',
                    msg : "Select the POM file as part of the GAV Definition.",
                    buttons : Ext.Msg.OK,
                    icon : Ext.MessageBox.ERROR
                  });
                  return;
                }
                b.uploadPanel.updateFilename(b.uploadPanel, filename);
              }
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Filename',
              name : 'filenameField',
              readOnly : true,
              width : '95%',
              allowBlank : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Classifier',
              helpText : ht.classifier,
              name : 'classifier',
              width : '95%',
              allowBlank : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Extension',
              helpText : ht.extension,
              name : 'extension',
              width : '95%',
              allowBlank : true
            },
            {
              xtype : 'button',
              id : 'add-button',
              text : 'Add Artifact',
              handler : this.addArtifact,
              scope : this,
              disabled : true
            },
            {
              xtype : 'panel',
              layout : 'column',
              autoHeight : true,
              style : 'padding-top: 5px; padding-bottom: 5px;',
              items : [
                {
                  xtype : 'treepanel',
                  name : 'artifact-list',
                  title : 'Artifacts',
                  border : true,
                  bodyBorder : true,
                  bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
                  style : 'padding: 0px 10px 0px 105px',
                  width : 750,
                  height : 100,
                  animate : true,
                  lines : false,
                  autoScroll : true,
                  containerScroll : true,
                  rootVisible : false,
                  ddScroll : true,
                  enableDD : true,
                  root : new Ext.tree.TreeNode({
                    text : 'root',
                    draggable : false
                  }),
                  invalidText : 'Add one or more artifacts',
                  validate : function() {
                    var pomMode = this.find('name', 'gavDefinition')[0].getValue() === 'pom';

                    if (pomMode) {
                      return true;
                    }

                    return this.find('name', 'artifact-list')[0].root.childNodes.length > 0;
                  },
                  invalid : false,
                  listeners : {
                    'append' : {
                      fn : function(tree, parentNode, insertedNode, i) {
                        if (tree.invalid) {
                          this.clearInvalid();
                        }
                      },
                      scope : this
                    },
                    'remove' : {
                      fn : function(tree, parentNode, removedNode) {
                        if (tree.root.childNodes.length < 1 && tree.required) {
                          this.markTreeInvalid(null);
                        }
                        else {
                          this.clearInvalid();
                        }
                      },
                      scope : this
                    }
                  }
                },
                {
                  xtype : 'panel',
                  width : 120,
                  items : [
                    {
                      xtype : 'button',
                      text : 'Remove',
                      minWidth : 100,
                      id : 'button-remove',
                      handler : this.removeArtifact,
                      scope : this
                    },
                    {
                      xtype : 'button',
                      text : 'Remove All',
                      style : 'margin-top: 5px',
                      minWidth : 100,
                      id : 'button-remove-all',
                      handler : this.removeAllArtifacts,
                      scope : this
                    }
                  ]
                }
              ]
            },
            this.extraItems,
            {
              xtype : 'panel',
              id : 'end-button-card-panel',
              header : false,
              deferredRender : false,
              autoScroll : false,
              layout : 'fit',
              buttonAlign : 'center',
              frame : false,
              items : [
                {}
              ],
              buttons : [
                {
                  xtype : 'button',
                  id : 'upload-button',
                  text : 'Upload Artifact(s)',
                  handler : this.uploadArtifacts,
                  scope : this
                },
                {
                  xtype : 'button',
                  id : 'reset-all-button',
                  text : 'Reset',
                  handler : this.resetFields,
                  scope : this
                }
              ]
            }
          ]
        }
      ]
    });
  },

  clearInvalid : function() {
    var tree = this.find('name', 'artifact-list')[0];
    if (tree.invalid) {
      // remove error messaging
      tree.getEl().child('.x-panel-body').setStyle({
        'background-color' : '#FFFFFF',
        border : '1px solid #B5B8C8'
      });
      Ext.form.Field.msgFx.normal.hide(tree.errorEl, tree);
    }
  },
  markTreeInvalid : function(errortext) {
    var
          tree = this.find('name', 'artifact-list')[0],
          elp = tree.getEl();

    if (!tree.errorEl) {
      tree.errorEl = elp.createChild({
        cls : 'x-form-invalid-msg'
      });
      tree.errorEl.setWidth(elp.getWidth(true)); // note removed -20 like
      // on form fields
    }
    tree.invalid = true;
    var oldErrorText = tree.invalidText;
    if (errortext) {
      tree.invalidText = errortext;
    }
    tree.errorEl.update(tree.invalidText);
    tree.invalidText = oldErrorText;
    elp.child('.x-panel-body').setStyle({
      'background-color' : '#fee',
      border : '1px solid #dd7870'
    });
    Ext.form.Field.msgFx.normal.show(tree.errorEl, tree);
  },
  resetFields : function() {
    this.gavResponse = null;
    // reset the artifact panels
    var
          filenameField = this.find('name', 'filenameField')[0],
          classifierField = this.find('name', 'classifier')[0],
          extensionField = this.find('name', 'extension')[0],
          addArtifactBtn = this.find('id', 'add-button')[0],
          g = this.find('name', 'g')[0],
          a = this.find('name', 'a')[0],
          v = this.find('name', 'v')[0],
          autoGuess = this.find('name', 'autoguess')[0],
          pomField = this.find('name', 'pomnameField')[0],
          desc = this.find('name', 'description')[0];

    filenameField.reset();
    classifierField.reset();
    extensionField.reset();
    addArtifactBtn.setDisabled(true);

    // clear the artifacts fields
    this.removeAllArtifacts();

    // reset the gav panel
    g.reset();
    a.reset();
    v.reset();
    autoGuess.reset();

    // the pom panel
    pomField.reset();
    this.pomInput = null;
    this.fileInput = null;

    if (desc) {
      desc.reset();
    }
  },

  artifactWithClassifierAndExtensionExists : function(classifier, extension) {
    var
          i, currClassifier, currExtension,
          treePanel = this.find('name', 'artifact-list')[0];
    for (i = 0; i < treePanel.root.childNodes.length; i++) {
      currClassifier = treePanel.root.childNodes[i].attributes.payload.classifier;
      currExtension = treePanel.root.childNodes[i].attributes.payload.extension;

      if (classifier === currClassifier && extension === currExtension) {
        return true;
      }
    }
    return false;
  },

  addArtifact : function() {
    var
          treePanel = this.find('name', 'artifact-list')[0],
          filenameField = this.find('name', 'filenameField')[0],
          classifierField = this.find('name', 'classifier')[0],
          extensionField = this.find('name', 'extension')[0],
          classifier = classifierField.getValue(),
          extension = extensionField.getValue(),
          nodeText = filenameField.getValue();

    if (this.artifactWithClassifierAndExtensionExists(classifier, extension)) {
      Ext.Msg.show({
        title : 'Classifier and Extension Taken',
        msg : "Every artifact must have a unique classifier and extension. The specified classifier and extension is already taken.",
        buttons : Ext.Msg.OK,
        icon : Ext.MessageBox.WARNING
      });
      return;
    }
    if (!Ext.isEmpty(classifier)) {
      nodeText += ' c:' + classifier;
    }
    if (!Ext.isEmpty(extension)) {
      nodeText += ' e:' + extension;
    }

    if (this.fileInput) {
      treePanel.root.appendChild(new Ext.tree.TreeNode({
        id : filenameField.getValue(),
        text : nodeText,
        payload : {
          id : filenameField.getValue(),
          filename : filenameField.getValue(),
          fileInput : this.fileInput,
          classifier : classifier,
          extension : extension
        },
        allowChildren : false,
        draggable : false,
        leaf : true,
        icon : Sonatype.config.extPath + '/resources/images/default/tree/leaf.gif'
      }));
    }
    filenameField.setValue('');
    classifierField.setValue('');
    extensionField.setValue('');
    this.fileInput = null;
    this.find('id', 'add-button')[0].setDisabled(true);
  },
  removeArtifact : function() {
    var treePanel = this.find('name', 'artifact-list')[0];

    var selectedNode = treePanel.getSelectionModel().getSelectedNode();
    if (selectedNode) {
      treePanel.root.removeChild(selectedNode);
    }
  },
  removeAllArtifacts : function() {
    var
          treePanel = this.find('name', 'artifact-list')[0],
          treeRoot = treePanel.root;

    while (treeRoot.lastChild) {
      treeRoot.removeChild(treeRoot.lastChild);
    }
  },
  updateFilename : function(uploadPanel, filename) {
    var
          filenameField = uploadPanel.find('name', 'filenameField')[0],
          g = '', a = '', v = '', c = '', p = '', e = '';

    if (filename) {
      filenameField.setValue(filename);
    }
    else {
      filename = filenameField.getValue();
      if (!filename) {
        return;
      }
    }

    var cardPanel = uploadPanel.find('id', 'gav-definition-card-panel')[0];

    // match extension to guess the packaging
    var extensionIndex = filename.lastIndexOf('.');
    if (extensionIndex > 0) {
      p = filename.substring(extensionIndex + 1);
      e = filename.substring(extensionIndex + 1);
      filename = filename.substring(0, extensionIndex);

      if (e === 'asc') {
        var primaryExtensionIndex = filename.substring(0, extensionIndex).lastIndexOf('.');
        var primaryExtension = '';
        if (primaryExtensionIndex >= 0) {
          primaryExtension = filename.substring(primaryExtensionIndex + 1);
        }

        if (/^[a-z]*$/.test(primaryExtension)) {
          e = primaryExtension + '.' + e;
          filename = filename.substring(0, primaryExtensionIndex);
        }
      }
    }

    // match the path to guess the group
    if (filename.indexOf('\\') >= 0) {
      filename = filename.replace(/\\/g, '\/');
    }
    var slashIndex = filename.lastIndexOf('/');
    if (slashIndex) {
      g = filename.substring(0, slashIndex);

      filename = filename.substring(slashIndex + 1);
    }

    // separate the artifact name and version
    var versionIndex = filename.search(/\-[\d]/);
    if (versionIndex === -1) {
      versionIndex = filename.search(/-LATEST-/i);
      if (versionIndex === -1) {
        versionIndex = filename.search(/-CURRENT-/i);
      }
    }
    if (versionIndex >= 0) {
      a = filename.substring(0, versionIndex).toLowerCase();

      // guess the version
      filename = filename.substring(versionIndex + 1);
      var classifierIndex = filename.lastIndexOf('-');
      if (classifierIndex >= 0) {
        var classifier = filename.substring(classifierIndex + 1);
        if (classifier && !(/^SNAPSHOT$/i.test(classifier) || /^\d/.test(classifier)
              || /^LATEST$/i.test(classifier)
              || /^CURRENT$/i.test(classifier))) {
          c = classifier;
          filename = filename.substring(0, classifierIndex);
          // dont guess packaging when there is a classifier
          p = '';
          extensionIndex = c.indexOf('.');
          if (extensionIndex >= 0) {
            e = c.substring(extensionIndex + 1) + '.' + e;
            c = c.substring(0, extensionIndex);
          }
        }
      }
      v = filename;

      if (g) {
        // if group ends with version and artifact name, strip those parts
        // (useful if uploading from a local maven repo)
        var i = g.search(new RegExp('\/' + v + '$'));
        if (i > -1) {
          g = g.substring(0, i);
        }
        i = g.search(new RegExp('\/' + a + '$'));
        if (i > -1) {
          g = g.substring(0, i);
        }

        // strip extra path parts, leave only com.* or org.* or net.* or the
        // last element
        i = g.lastIndexOf('/com/');
        if (i === -1) {
          i = g.lastIndexOf('/org/');
          if (i === -1) {
            i = g.lastIndexOf('/net/');
            if (i === -1) {
              i = g.lastIndexOf('/');
            }
          }
        }
        g = g.substring(i + 1).replace(/\//g, '.').toLowerCase();
      }
    }
    else {
      g = '';
    }

    if (cardPanel.find('name', 'autoguess')[0].checked) {
      var groupField = cardPanel.find('name', 'g')[0];
      if (Ext.isEmpty(groupField.getValue())) {
        groupField.setRawValue(g);
      }
      var artifactField = cardPanel.find('name', 'a')[0];
      if (Ext.isEmpty(artifactField.getValue())) {
        artifactField.setRawValue(a);
      }
      var versionField = cardPanel.find('name', 'v')[0];
      if (Ext.isEmpty(versionField.getValue())) {
        versionField.setRawValue(v);
      }
      var packagingField = cardPanel.find('name', 'p')[0];
      if (Ext.isEmpty(packagingField.getValue())) {
        packagingField.setRawValue(p);
      }
    }
    uploadPanel.find('name', 'classifier')[0].setRawValue(c);
    uploadPanel.find('name', 'extension')[0].setRawValue(e);
    if (!a) {
      uploadPanel.form.clearInvalid();
    }
    uploadPanel.find('id', 'add-button')[0].setDisabled(false);
  },
  updatePomFilename : function(uploadPanel, filename) {
    var filenameField = uploadPanel.find('name', 'pomnameField')[0];
    if (filename) {
      filenameField.setValue(filename);
    }
  },
  gavDefinitionSelectHandler : function(combo, record, index) {
    var gavDefinitionPanel = this.findById('gav-definition-card-panel');
    // First disable all the items currently on screen, so they wont be
    // validated/submitted etc
    gavDefinitionPanel.getLayout().activeItem.items.each(function(item) {
      if (item.xtype !== 'browsebutton') {
        item.disable();
      }
    });
    this.clearInvalid();
    // Then find the proper card to activate (based upon the selected
    // schedule type)
    if (record.data.value === 'pom') {
      gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(1));
    }
    else if (record.data.value === 'gav') {
      gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(2));
    }
    else {
      gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(0));
    }
    // Then enable the fields in that card
    gavDefinitionPanel.getLayout().activeItem.items.each(function(item) {
      if (item.xtype !== 'browsebutton') {
        item.enable();
        if (item.readOnly) {
          item.getEl().dom.readOnly = true;
        }
      }
      else {
        item.setClipSize();
      }
    });
    this.doLayout();
  },
  uploadArtifacts : function() {
    var tree = this.find('name', 'artifact-list')[0];
    if (!tree.validate.call(this)) {
      this.markTreeInvalid(null);
    }
    else if (this.form.isValid()) {
      this.doUpload();
    }
  },
  doUpload : function() {
    var treePanel = this.find('name', 'artifact-list')[0];
    var hasArtifacts = treePanel.root.childNodes && treePanel.root.childNodes.length > 0;
    if (!this.pomInput && !hasArtifacts) {
      Sonatype.MessageBox.show({
        title : 'No Artifacts Selected',
        msg : 'The Artifacts list must contain at least one artifact to upload (or a POM file must be selected).',
        buttons : Sonatype.MessageBox.OK,
        icon : Sonatype.MessageBox.ERROR
      });
      return;
    }

    Sonatype.MessageBox.wait('Uploading ...');

    if (hasArtifacts) {
      // FIXME this.currentChildNode is always 0 here? side effect?
      this.currentChildNode = 0;
      this.createUploadForm(treePanel,
            treePanel.root.childNodes[this.currentChildNode].attributes.payload.fileInput,
            treePanel.root.childNodes[this.currentChildNode].attributes.payload.classifier,
            treePanel.root.childNodes[this.currentChildNode].attributes.payload.extension,
            this.currentChildNode === (treePanel.root.childNodes.length - 1));
    }
    else if (this.pomInput) {
      this.createUploadForm(treePanel, null, null, null, true);
    }
  },
  createUploadForm : function(treePanel, fileInput, classifier, extension, lastItem) {
    var repoId = this.payload.id;
    repoId = repoId.substring(repoId.lastIndexOf('/') + 1);
    var pomMode = this.find('name', 'gavDefinition')[0].getValue() === 'pom';

    if (this.gavResponse) {
      pomMode = false;
      this.form.findField('g').setValue(this.gavResponse.groupId);
      this.form.findField('a').setValue(this.gavResponse.artifactId);
      this.form.findField('v').setValue(this.gavResponse.version);
      this.form.findField('p').setValue(this.gavResponse.packaging);
    }

    var repoTag = {
      tag : 'input',
      type : 'hidden',
      name : 'r',
      value : repoId
    };

    var tmpForm = Ext.getBody().createChild({
      tag : 'form',
      cls : 'x-hidden',
      id : Ext.id(),
      children : pomMode ? [repoTag, {
        tag : 'input',
        type : 'hidden',
        name : 'hasPom',
        value : 'true'
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'c',
        value : classifier
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'e',
        value : extension
      }] : [repoTag, {
        tag : 'input',
        type : 'hidden',
        name : 'g',
        value : this.form.findField('g').getValue()
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'a',
        value : this.form.findField('a').getValue()
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'v',
        value : this.form.findField('v').getValue()
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'p',
        value : this.form.findField('p').getValue()
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'c',
        value : classifier
      }, {
        tag : 'input',
        type : 'hidden',
        name : 'e',
        value : extension
      }]
    });

    if (pomMode) {
      this.pomInput.appendTo(tmpForm);
    }
    if (fileInput) {
      fileInput.appendTo(tmpForm);
    }

    Ext.Ajax.request({
      url: Sonatype.config.repos.urls.upload,
      form : tmpForm,
      isUpload : true,
      cbPassThru : {
        treePanel : treePanel
      },
      callback : function(options, success, response) {
        tmpForm.remove();

        // This is a hack to get around the fact that upload submit
        // always returns
        // success = true
        var
              indexOfErrorMsg = response.responseText.toLowerCase().indexOf('<error>'),
              endIndexOfErrorMsg = response.responseText.toLowerCase().indexOf('</error>'),
              msg = 'Artifact upload failed.<br />',
              indexOfMsgContent = indexOfErrorMsg + 7,
              treePanel = options.cbPassThru.treePanel;

        if (indexOfErrorMsg === -1) {
          // get the json response and set the gavResponse
          this.gavResponse = Ext.decode(response.responseText);

          if (lastItem) {
            Sonatype.MessageBox.show({
              title : 'Upload Complete',
              msg : 'Artifact upload finished successfully',
              buttons : Sonatype.MessageBox.OK,
              icon : Sonatype.MessageBox.INFO
            });
            this.resetFields();
          }
          else {
            this.currentChildNode += 1;
            if (this.currentChildNode < treePanel.root.childNodes.length) {
              this.createUploadForm(treePanel,
                    treePanel.root.childNodes[this.currentChildNode].attributes.payload.fileInput,
                    treePanel.root.childNodes[this.currentChildNode].attributes.payload.classifier,
                    treePanel.root.childNodes[this.currentChildNode].attributes.payload.extension,
                    this.currentChildNode === treePanel.root.childNodes.length - 1);
            }
          }
        } else {
          this.logDebug('Upload failed: ' + response.responseText);

          this.gavResponse = null;

          if (endIndexOfErrorMsg > indexOfMsgContent) {
            msg += response.responseText.substring(indexOfMsgContent, endIndexOfErrorMsg);
          }
          else {
            msg += 'Check Nexus logs for more information.';
          }
          Sonatype.MessageBox.show({
            title : 'Upload Failed',
            msg : msg,
            buttons : Sonatype.MessageBox.OK,
            icon : Sonatype.MessageBox.ERROR
          });
        }
      },
      scope : this
    });
  }
}, function() {

  Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec) {
    var sp = Sonatype.lib.Permissions;

    if (rec.data.resourceURI && rec.data.userManaged && sp.checkPermission('nexus:artifact', sp.CREATE)
          && rec.data.repoType === 'hosted' && rec.data.repoPolicy && rec.data.repoPolicy.toUpperCase()
          === 'RELEASE') {

      Ext.Ajax.request({
        url : rec.data.resourceURI,
        scope : this,
        callback : function(options, success, response) {
          if (success) {
            var statusResp = Ext.decode(response.responseText);
            if (statusResp.data) {
              if (statusResp.data.writePolicy === 'ALLOW_WRITE' || statusResp.data.writePolicy
                    === 'ALLOW_WRITE_ONCE') {
                var uploadPanel = new Sonatype.repoServer.ArtifactUploadPanel({
                  payload : rec,
                  name : 'upload'
                });
                var card = cardPanel.add({
                  xtype : 'panel',
                  layout : 'fit',
                  tabTitle : 'Artifact Upload',
                  name : 'uploadPanel',
                  items : [uploadPanel]
                });

                card.on('show', function(p) {
                  // This is a hack to fix the width of the edit box
                  // in IE
                  p.doLayout();
                  p.find('name', 'filenameField')[0].setValue('.');
                  p.find('name', 'filenameField')[0].setValue('');

                  // another hack to make the whole browse button
                  // clickable
                  if (!p.browseButtonsUpdated) {
                    var i, b = p.find('xtype', 'browsebutton');
                    for (i = 0; i < b.length; i++) {
                      b[i].setClipSize();
                    }
                    p.browseButtonsUpdated = true;
                  }

                  var gavDefComboField = p.find('name', 'gavDefinition')[0];
                  // another hack to fix the combo box lists
                  gavDefComboField.syncSize();
                });
              }
              else {
                cardPanel.add({
                  xtype : 'panel',
                  tabTitle : 'Artifact Upload',
                  name : 'upload',
                  items : [
                    {
                      border : false,
                      html : '<div class="little-padding">' + 'Artifact deployment is disabled for '
                            + rec.data.name
                            + '.<br /><br />' + 'You can enable it in the "Access Settings" section of the '
                            + 'repository configuration.</div>'
                    }
                  ]
                });
              }

              cardPanel.doLayout();

              return;
            }
          }
          Sonatype.utils.connectionError(response, 'There was a problem obtaining repository status.');
        }
      });
    }
  });
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * /js/repoServer/referenceData.js Reference service data objects
 * used for reading form data and compiling resource state objects to send back
 * to the service.
 */

/*global NX*/
NX.define('Sonatype.repoServer.referenceData', {
  singleton : true,

  requirejs : ['Sonatype/all'],

  globalSettingsState : {
    securityAnonymousUsername : "",
    securityAnonymousPassword : "",
    securityEnabled : false,
    securityAnonymousAccessEnabled : false,
    securityRealms : [],
    smtpSettings : {
      host : "",
      port : 25,
      username : "",
      password : "",
      systemEmailAddress : "",
      sslEnabled : "",
      tlsEnabled : ""
    },
    globalConnectionSettings : {
      connectionTimeout : 0,
      retrievalRetryCount : 0,
      queryString : "",
      userAgentString : ""
    },
    remoteProxySettings : {
      httpProxySettings : {
        proxyHostname : "",
        proxyPort : 0,
        authentication : {
          username : "",
          password : "",
          ntlmHost : "",
          ntlmDomain : ""
        }
      },
      httpsProxySettings : {
        proxyHostname : "",
        proxyPort : 0,
        authentication : {
          username : "",
          password : "",
          ntlmHost : "",
          ntlmDomain : ""
        }
      },
      nonProxyHosts : []
    },
    globalRestApiSettings : {
      baseUrl : "",
      forceBaseUrl : false,
      uiTimeout : 30000
    },
    systemNotificationSettings : {
      enabled : false,
      emailAddresses : "",
      roles : []
    }
  },

  repositoryState : {
    virtual : {
      repoType : "",
      id : "",
      name : "",
      shadowOf : "",
      provider : "",
      providerRole : "",
      syncAtStartup : false,
      exposed : true
    },

    hosted : {
      repoType : "",
      id : "",
      name : "",
      writePolicy : "ALLOW_WRITE_ONCE",
      browseable : true,
      indexable : true,
      exposed : true,
      notFoundCacheTTL : 0,
      repoPolicy : "",
      provider : "",
      providerRole : "",
      overrideLocalStorageUrl : "",
      defaultLocalStorageUrl : "",
      downloadRemoteIndexes : true,
      checksumPolicy : ""
    },

    proxy : {
      repoType : "",
      id : "",
      name : "",
      browseable : true,
      indexable : true,
      notFoundCacheTTL : 0,
      artifactMaxAge : 0,
      metadataMaxAge : 0,
      itemMaxAge : 0,
      repoPolicy : "",
      provider : "",
      providerRole : "",
      overrideLocalStorageUrl : "",
      defaultLocalStorageUrl : "",
      downloadRemoteIndexes : true,
      autoBlockActive : true,
      fileTypeValidation : false,
      exposed : true,
      checksumPolicy : "",
      remoteStorage : {
        remoteStorageUrl : "",
        authentication : {
          username : "",
          password : "",
          ntlmHost : "",
          ntlmDomain : ""
        },
        connectionSettings : {
          connectionTimeout : 0,
          retrievalRetryCount : 0,
          queryString : "",
          userAgentString : ""
        }
      }
    } // end repositoryProxyState
  },

  group : {
    id : "",
    name : "",
    format : "",
    exposed : "",
    provider : "",
    repositories : []
    // note: internal record structure is the responsibility of data modifier
    // func
    // {
    // id:"central",
    // name:"Maven Central",
    // resourceURI:".../repositories/repoId" // added URI to be able to reach
    // repo
    // }
  },

  route : {
    id : "",
    ruleType : "",
    groupId : "",
    pattern : "",
    repositories : []
    // @todo: there's a discrepancy between routes list and state
    // representation of
    // the repo data inside routes data
  },

  schedule : {
    manual : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ]
    },
    once : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ],
      startDate : "",
      startTime : ""
    },
    hourly : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ],
      startDate : "",
      startTime : ""
    },
    daily : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ],
      startDate : "",
      recurringTime : ""
    },
    weekly : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ],
      startDate : "",
      recurringTime : "",
      recurringDay : []
    },
    monthly : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ],
      startDate : "",
      recurringTime : "",
      recurringDay : []
    },
    advanced : {
      id : "",
      name : "",
      enabled : "",
      typeId : "",
      alertEmail : "",
      schedule : "",
      properties : [
        {
          id : "",
          value : ""
        }
      ],
      cronCommand : ""
    }
  },

  upload : {
    r : "",
    g : "",
    a : "",
    v : "",
    p : "",
    c : "",
    e : ""
  },

  users : {
    userId : "",
    firstName : "",
    lastName : "",
    email : "",
    status : "",
    roles : []
  },

  userNew : {
    userId : "",
    firstName : "",
    lastName : "",
    email : "",
    status : "",
    password : "",
    roles : []
  },

  roles : {
    id : "",
    name : "",
    description : "",
    sessionTimeout : 0,
    roles : [],
    privileges : []
  },

  privileges : {
    target : {
      name : "",
      description : "",
      type : "",
      repositoryTargetId : "",
      repositoryId : "",
      repositoryGroupId : "",
      method : []
    }
  },

  repoTargets : {
    id : "",
    name : "",
    contentClass : "",
    patterns : []
  },

  contentClasses : {
    contentClass : "",
    name : ""
  },

  repoMirrors : [
    {
      id : "",
      url : ""
    }
  ]

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

NX.define('Sonatype.repoServer.ArtifactContainer', {
  extend : 'Nexus.panels.AutoTabPanel',

  constructor : function(cfg) {
    Ext.apply(this, cfg || {}, {
      initEventName : 'artifactContainerInit',
      updateEventName : 'artifactContainerUpdate'
    });

    Sonatype.repoServer.ArtifactContainer.superclass.constructor.call(this, {
      layoutOnTabChange : true
    });

    var items = [], i;

    Sonatype.Events.fireEvent(this.initEventName, items, null);

    items.sort(function(a, b) {
      if (a.preferredIndex === undefined && b.preferredIndex === undefined) {
        return 0;
      }

      if (a.preferredIndex === undefined) {
        return 1;
      }
      if (b.preferredIndex === undefined) {
        return -1;
      }
      if (a.preferredIndex < b.preferredIndex) {
        return -1;
      }
      if (a.preferredIndex > b.preferredIndex) {
        return 1;
      }
      return 0;
    });

    for (i = 0; i < items.length; i += 1) {
      this.add(items[i]);
    }
  },

  collapsePanel : function() {
    this.collapse();
    Sonatype.Events.fireEvent(this.updateEventName, this, null);
  },
  updateArtifact : function(data) {
    Sonatype.Events.fireEvent(this.updateEventName, this, data);
    if (data) {
      this.expand();
    }
  },
  hideTab : function(panel) {
    var i, nextPanel;

    panel.tabHidden = true;
    this.tabPanel.hideTabStripItem(panel);
    for (i = 0; i < this.tabPanel.items.getCount(); i += 1) {
      nextPanel = this.tabPanel.items.get(i);
      if (nextPanel.id !== panel.id && !nextPanel.tabHidden) {
        this.tabPanel.setActiveTab(nextPanel);
        return;
      }
    }

    // we haven't found anything, so collapse
    this.tabPanel.doLayout();
    this.collapse();
  },
  showTab : function(panel) {
    panel.tabHidden = false;
    this.tabPanel.unhideTabStripItem(panel);
  }

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Sonatype, Ext*/
NX.define('Sonatype.repoServer.Maven2InformationPanel', {
  extend : 'Ext.form.FormPanel',
  requirejs : ['Sonatype/init'],

  constructor : function(config) {
    Ext.apply(this, config || {}, {
      halfSize : false
    });

    this.sp = Sonatype.lib.Permissions;

    this.linkDivId = Ext.id();
    this.linkLabelId = Ext.id();

    Sonatype.repoServer.Maven2InformationPanel.superclass.constructor.call(this, {
      title : 'Maven',
      autoScroll : true,
      border : true,
      frame : true,
      collapsible : false,
      collapsed : false,
      items : [
        {
          xtype : 'displayfield',
          fieldLabel : 'Group',
          name : 'groupId',
          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
          allowBlank : true,
          readOnly : true
        },
        {
          xtype : 'displayfield',
          fieldLabel : 'Artifact',
          name : 'artifactId',
          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
          allowBlank : true,
          readOnly : true
        },
        {
          xtype : 'displayfield',
          fieldLabel : 'Version',
          name : 'baseVersion',
          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
          allowBlank : true,
          readOnly : true
        },
        {
          xtype : 'displayfield',
          fieldLabel : 'Classifier',
          name : 'classifier',
          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
          allowBlank : true,
          readOnly : true
        },
        {
          xtype : 'displayfield',
          fieldLabel : 'Extension',
          name : 'extension',
          anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
          allowBlank : true,
          readOnly : true
        },
        {
          xtype : 'textarea',
          fieldLabel : 'XML',
          anchor : Sonatype.view.FIELD_OFFSET,
          height : 120,
          name : 'dependencyXmlChunk',
          allowBlank : true,
          readOnly : true
        }
      ]
    });
  },

  showArtifact : function(data, artifactContainer) {
    this.data = data;
    if (data) {
      Ext.Ajax.request({
        url : this.data.resourceURI + '?describe=maven2&isLocal=true',
        callback : function(options, isSuccess, response) {
          if (isSuccess) {
            var infoResp = Ext.decode(response.responseText);

            // hide classifier if empty
            if (this.data.classifier) {
              this.find('name', 'classifier')[0].show();
            }
            else {
              this.find('name', 'classifier')[0].hide();
            }
            this.form.setValues(infoResp.data);
            artifactContainer.showTab(this);
          }
          else {
            if (response.status === 404) {
              artifactContainer.hideTab(this);
            }
            else {
              Sonatype.utils.connectionError(response, 'Unable to retrieve Maven information.');
            }
          }
        },
        scope : this,
        method : 'GET',
        suppressStatus : 404
      });
    }
    else {
      this.find('name', 'groupId')[0].setRawValue(null);
      this.find('name', 'artifactId')[0].setRawValue(null);
      this.find('name', 'baseVersion')[0].setRawValue(null);
      this.find('name', 'classifier')[0].setRawValue(null);
      this.find('name', 'extension')[0].setRawValue(null);
      this.find('name', 'dependencyXmlChunk')[0].setRawValue(null);
    }
  }
}, function() {

  Sonatype.Events.addListener('fileContainerInit', function(items) {
    items.push(new Sonatype.repoServer.Maven2InformationPanel({
      name : 'maven2InformationPanel',
      tabTitle : 'Maven',
      preferredIndex : 10
    }));
  });

  Sonatype.Events.addListener('fileContainerUpdate', function(artifactContainer, data) {
    var panel = artifactContainer.find('name', 'maven2InformationPanel')[0];

    if (data && data.leaf) {
      panel.showArtifact(data, artifactContainer);
    }
    else {
      panel.showArtifact(null, artifactContainer);
    }
  });

  Sonatype.Events.addListener('artifactContainerInit', function(items) {
    items.push(new Sonatype.repoServer.Maven2InformationPanel({
      name : 'maven2InformationPanel',
      tabTitle : 'Maven',
      preferredIndex : 10
    }));
  });

  Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, payload) {
    var panel = artifactContainer.find('name', 'maven2InformationPanel')[0];

    if (payload && payload.leaf) {
      panel.showArtifact(payload, artifactContainer);
    }
    else {
      panel.showArtifact(null, artifactContainer);
    }

  });

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Sonatype, Nexus, Ext*/
/*
 * Repository Groups Edit/Create panel layout and controller
 */
NX.define('Sonatype.repoServer.RepositoryGroupEditor', {
        extend : 'Nexus.ext.FormPanel',

        requirejs : ['Sonatype/all'],

        constructor : function(config) {
          var
                ht = Sonatype.repoServer.resources.help.groups,
                defaultConfig = {
                  dataModifiers : {
                    load : {
                      repositories : this.loadRepositories.createDelegate(this),
                      exposed : Nexus.util.Strings.capitalize
                    },
                    submit : {
                      repositories : this.saveRepositories.createDelegate(this),
                      exposed : Nexus.util.Strings.convert.stringContextToBool
                    }
                  },
                  validationModifiers : {
                    repositories : this.repositoriesValidationError.createDelegate(this)
                  },
                  referenceData : Sonatype.repoServer.referenceData.group,
                  uri : Sonatype.config.repos.urls.groups
                };
          Ext.apply(this, config || {}, defaultConfig);

          this.tfStore = new Ext.data.SimpleStore({
            fields : ['value'],
            data : [
              ['True'],
              ['False']
            ]
          });

          this.providerStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'provider',
            fields : [
              {
                name : 'description',
                sortType : Ext.data.SortTypes.asUCString
              },
              {
                name : 'format'
              },
              {
                name : 'provider'
              }
            ],
            sortInfo : {
              field : 'description',
              direction : 'asc'
            },
            url : Sonatype.config.repos.urls.repoTypes + '?repoType=group'
          });

          this.contentClassStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'contentClass',
            fields : [
              { name : 'contentClass' },
              { name : 'name' },
              { name : 'groupable' },
              { name : 'compatibleTypes' }
            ],
            url : Sonatype.config.repos.urls.repoContentClasses
          });

          this.repoStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [
              {
                name : 'id'
              },
              {
                name : 'resourceURI'
              },
              {
                name : 'format'
              },
              {
                name : 'name',
                sortType : Ext.data.SortTypes.asUCString
              }
            ],
            sortInfo : {
              field : 'name',
              direction : 'asc'
            },
            url : Sonatype.config.repos.urls.allRepositories
          });

          this.checkPayload();

          Sonatype.repoServer.RepositoryGroupEditor.superclass.constructor.call(this, {
            dataStores : [this.providerStore, this.repoStore, this.contentClassStore],
            items : [
              {
                xtype : 'textfield',
                fieldLabel : 'Group ID',
                itemCls : 'required-field',
                helpText : ht.id,
                name : 'id',
                width : 200,
                allowBlank : false,
                disabled : !this.isNew,
                validator : Nexus.util.Strings.validateId
              },
              {
                xtype : 'textfield',
                fieldLabel : 'Group Name',
                itemCls : 'required-field',
                helpText : ht.name,
                name : 'name',
                width : 200,
                allowBlank : false
              },
              {
                xtype : 'combo',
                fieldLabel : 'Provider',
                itemCls : 'required-field',
                helpText : ht.provider,
                name : 'provider',
                width : this.COMBO_WIDTH,
                store : this.providerStore,
                displayField : 'description',
                valueField : 'provider',
                editable : false,
                forceSelection : true,
                mode : 'local',
                triggerAction : 'all',
                emptyText : 'Select...',
                selectOnFocus : true,
                allowBlank : false,
                disabled : !this.isNew,
                listeners : {
                  select : {
                    fn : this.providerSelectHandler,
                    scope : this
                  }
                }
              },
              {
                xtype : 'textfield',
                fieldLabel : 'Format',
                itemCls : 'required-field',
                helpText : ht.format,
                name : 'format',
                width : 100,
                disabled : true,
                allowBlank : false
              },
              {
                xtype : 'combo',
                fieldLabel : 'Publish URL',
                itemCls : 'required-field',
                helpText : ht.exposed,
                name : 'exposed',
                width : 75,
                store : this.tfStore,
                displayField : 'value',
                editable : false,
                forceSelection : true,
                mode : 'local',
                triggerAction : 'all',
                emptyText : 'Select...',
                selectOnFocus : true,
                allowBlank : false,
                value : 'True'
              },
              {
                xtype : 'twinpanelchooser',
                titleLeft : 'Ordered Group Repositories',
                titleRight : 'Available Repositories',
                name : 'repositories',
                valueField : 'id',
                store : this.repoStore,
                validateLeftItems : true,
                validateLeftItemsText : 'Invalid Repository Found'
              }
            ],
            listeners : {
              submit : this.submitHandler,
              scope : this
            }
          });
        },

        isValid : function() {
          if (!this.form.isValid()) {
            return false;
          }
          var repoBox = this.find('name', 'repositories')[0];
          if (!repoBox.validate()) {
            return false;
          }
          return true;
        },

        loadRepositories : function(arr, srcObject, fpanel) {
          var i, repoBox = fpanel.find('name', 'repositories')[0];
          this.repoStore.filterBy(function(rec, id) {
            var compatibleClasses, contentClass = this.contentClassStore.getById(rec.data.format);

            // if we have the content class
            // and the content class is compatible with the repo type
            if (contentClass && rec.data.id !== srcObject.id) {
              compatibleClasses = contentClass.get('compatibleTypes');

              for (i = 0; i < compatibleClasses.length; i += 1) {
                if (compatibleClasses[i] === srcObject.format) {
                  return true;
                }
              }
            }
            return false;
          }, this);

          repoBox.setValue(arr);
        },

        providerSelectHandler : function(combo, rec, index) {
          combo.ownerCt.find('name', 'format')[0].setValue(rec.data.format);
          this.loadRepositories([], rec.data, this);
        },

        saveRepositories : function(value, fpanel) {
          var
                i, response = [],
                repoBox = fpanel.find('name', 'repositories')[0],
                repoIds = repoBox.getValue();

          for (i = 0; i < repoIds.length; i += 1) {
            response.push({
              id : repoIds[i]
            });
          }

          return response;
        },

        submitHandler : function(form, action, receivedData) {
          if (this.isNew) {
            if (!receivedData.resourceURI) {
              receivedData.displayStatus = Sonatype.utils.joinArrayObject(action.output.data.repositories, 'name');
              receivedData.resourceURI =
                    Sonatype.config.host + Sonatype.config.repos.urls.groups + '/' + receivedData.id;
            }
            receivedData.userManaged = true;
            return;
          }

          var rec = this.payload;
          rec.beginEdit();
          rec.set('name', action.output.data.name);
          rec.set('format', action.output.data.format);
          rec.set('exposed', action.output.data.exposed);
          rec.set('displayStatus', Sonatype.utils.joinArrayObject(action.output.data.repositories, 'name'));
          rec.commit();
          rec.endEdit();
        },

        // @override
        addSorted : function(store, rec) {
          var i, insertIndex, tempRec;
          for (i = 0; i < store.getCount(); i += 1) {
            tempRec = store.getAt(i);
            if (tempRec.get('repoType') !== 'group') {
              insertIndex = i;
              break;
            }
            if (tempRec.get('name').toLowerCase() > rec.get('name').toLowerCase()) {
              insertIndex = i;
              break;
            }
          }
          store.insert(insertIndex, [rec]);
        },
        repositoriesValidationError : function(error, fpanel) {
          var repos = fpanel.find('name', 'repositories')[0];
          repos.markTreeInvalid(null, error.msg);
        }
      },
      function() {

        Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec) {
          var sp = Sonatype.lib.Permissions, editor;

          if (rec.data.repoType === 'group' && rec.data.userManaged && sp.checkPermission('nexus:repogroups', sp.READ)
                && (sp.checkPermission('nexus:repogroups', sp.CREATE) || sp.checkPermission('nexus:repogroups',
                sp.EDIT))) {

            editor = new Sonatype.repoServer.RepositoryGroupEditor({
              tabTitle : 'Configuration',
              name : 'configuration',
              payload : rec
            });

            cardPanel.add(editor);

            if (rec.data.resourceURI) {
              // I have to say this is a poor solution, get the resource once, if
              // ok, show the tab and get again.
              // But I cannot think out a better, simpler solution.
              Ext.Ajax.request({
                url : Sonatype.config.repos.urls.groups + '/' + rec.get('id'),
                method : 'GET',
                scope : this,
                callback : function(options, isSuccess, response) {
                  if (!isSuccess) {
                    cardPanel.remove(editor);
                  }
                }
              });
            }
          }
        });

        Sonatype.Events.addListener('repositoryAddMenuInit', function(menu) {
          var sp = Sonatype.lib.Permissions;

          if (sp.checkPermission('nexus:repogroups', sp.CREATE)) {
            menu.add('-');
            menu.add({
              text : 'Repository Group',
              autoCreateNewRecord : true,
              handler : function(container, rec, item, e) {
                rec.beginEdit();
                rec.set('repoType', 'group');
                rec.set('exposed', true);
                rec.set('userManaged', true);
                rec.commit();
                rec.endEdit();
              },
              scope : this
            });
          }
        });

      });
/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('repoServer/UserBrowserPanel',['Sonatype/all'], function(){
Sonatype.repoServer.UserBrowsePanel = function(config) {
  var config = config || {};
  var defaultConfig = {
    titleColumn : 'name',
    isRole : false
  };
  Ext.apply(this, config, defaultConfig);

  this.roleTreeDataStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        baseParams : {
          isRole : this.isRole
        },
        proxy : new Ext.data.HttpProxy({
              url : Sonatype.config.servicePath + '/role_tree/' + this.parentId,
              method : 'GET'
            }),
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true,
        fields : [{
              name : 'id'
            }, {
              name : 'type'
            }, {
              name : 'name'
            }, {
              name : 'children'
            }],
        listeners : {
          load : {
            fn : this.roleTreeLoadHandler,
            scope : this
          }
        }
      });

  Sonatype.repoServer.UserBrowsePanel.superclass.constructor.call(this, {
        anchor : '0 -2',
        bodyStyle : 'background-color:#FFFFFF',
        animate : true,
        lines : false,
        autoScroll : true,
        containerScroll : true,
        rootVisible : true,
        enableDD : false,
        root : new Ext.tree.TreeNode({
              text : this.parentName,
              draggable : false
            }),
        tbar : [{
              text : 'Refresh',
              iconCls : 'st-icon-refresh',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.refreshHandler
            }]
      });

  new Ext.tree.TreeSorter(this, {
        folderSort : true
      });
};

Ext.extend(Sonatype.repoServer.UserBrowsePanel, Ext.tree.TreePanel, {
      refreshHandler : function(button, e) {
        this.roleTreeDataStore.reload();
      },
      roleTreeLoadHandler : function(store, records, options) {
        if (!this.getRootNode()) {
          // not rendered yet
          return;
        }
        this.idCounter = 0;
        while (this.getRootNode().lastChild)
        {
          this.getRootNode().removeChild(this.getRootNode().lastChild);
        }
        for (var i = 0; i < records.length; i++)
        {
          this.loadItemIntoTree(records[i].data.name, records[i].data.type, this.getRootNode(), records[i].data.children);
        }
      },
      loadItemIntoTree : function(name, type, parentNode, children) {
        var childNode = new Ext.tree.TreeNode({
              id : this.idCounter++,
              text : name,
              allowChildren : (children && children.length > 0) ? true : false,
              draggable : false,
              leaf : (children && children.length > 0) ? false : true,
              icon : (type == 'role') ? (Sonatype.config.extPath + '/resources/images/default/tree/folder.gif') : (Sonatype.config.extPath + '/resources/images/default/tree/leaf.gif')
            });

        parentNode.appendChild(childNode);

        if (children)
        {
          for (var i = 0; i < children.length; i++)
          {
            this.loadItemIntoTree(children[i].name, children[i].type, childNode, children[i].children)
          }
        }
      }
    });

Sonatype.Events.addListener('userViewInit', function(cardPanel, rec, gridPanel) {
      if (rec.data.resourceURI)
      {
        var parentName = null;

        if (rec.data.firstName)
        {
          parentName = rec.data.firstName;
        }

        if (rec.data.lastName)
        {
          parentName += ' ' + rec.data.lastName;
        }

        if (parentName == null)
        {
          parentName = rec.data.userId;
        }
        cardPanel.add(new Sonatype.repoServer.UserBrowsePanel({
              payload : rec,
              tabTitle : 'Role Tree',
              parentId : rec.data.userId,
              parentName : parentName
            }));
      }
    });

Sonatype.Events.addListener('roleViewInit', function(cardPanel, rec, gridPanel) {
      if (rec.data.resourceURI)
      {
        cardPanel.add(new Sonatype.repoServer.UserBrowsePanel({
              payload : rec,
              tabTitle : 'Role Tree',
              parentId : rec.data.id,
              isRole : true,
              parentName : rec.data.name
            }));
      }
    });
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/

/*
 * Privilege Edit/Create panel layout and controller
 */
define('repoServer/PrivilegeEditPanel',['extjs', 'Sonatype/all'], function(Ext, Sonatype){
Sonatype.repoServer.PrivilegeEditPanel = function(config) {
  Ext.apply(this, config || {}, {
    title : 'Privileges'
  });

  this.sp = Sonatype.lib.Permissions;

  this.propertyTypeStore = new Ext.data.SimpleStore({
        fields : [{
              name : 'type'
            }, {
              name : 'converter'
            }],
        data : [['repository', this.convertRepositoryProperty.createDelegate(this)], ['repogroup', this.convertRepoGroupProperty.createDelegate(this)], ['repotarget', this.convertRepoTargetProperty.createDelegate(this)],
            ['string', this.convertStringProperty.createDelegate(this)], ['repoOrGroup', this.convertRepoOrGroupProperty.createDelegate(this)]]
      });

  this.privilegeTypeStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        fields : [{
              name : 'id'
            }, {
              name : 'name'
            }, {
              name : 'properties'
            }],
        url : Sonatype.config.repos.urls.privilegeTypes
      });

  this.groupStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        fields : [{
              name : 'id'
            }, {
              name : 'format'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }],
        sortInfo : {
          field : 'name',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.groups
      });

  this.repoStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        fields : [{
              name : 'id'
            }, {
              name : 'format'
            }, {
              name : 'repoType'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }],
        sortInfo : {
          field : 'name',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.allRepositories
      });

  this.targetStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        fields : [{
              name : 'id'
            }, {
              name : 'contentClass'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }],
        sortInfo : {
          field : 'name',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.repoTargets
      });

  Sonatype.Events.on('privilegeAddMenuInit', this.onAddMenuInit, this);
  Sonatype.Events.on('privilegeViewInit', this.onViewInit, this);
  Sonatype.Events.on('privilegeClickedEvent', this.onPrivilegeClicked, this);

  Sonatype.repoServer.PrivilegeEditPanel.superclass.constructor.call(this, {
        addMenuInitEvent : 'privilegeAddMenuInit',
        rowClickEvent : 'privilegeViewInit',
        rowFocusChangedEvent : 'privilegeClickedEvent',
        deleteButton : this.sp.checkPermission('security:privileges', this.sp.DELETE),
        url : Sonatype.config.repos.urls.privileges,
        dataStores : [this.privilegeTypeStore, this.groupStore, this.repoStore, this.targetStore],
        columns : [{
              name : 'resourceURI'
            }, {
              name : 'id'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString,
              header : 'Name',
              autoExpand : true
            }, {
              name : 'userManaged',
              header : 'User Managed',
              width : 80
            }, {
              name : 'type'
            }, {
              name : 'sType',
              mapping : 'type',
              convert : this.convertType.createDelegate(this),
              header : 'Type',
              width : 100
            }, {
              name : 'description'
            }, {
              name : 'properties'
            }, {
              name : 'sTarget',
              mapping : 'properties',
              convert : this.convertTarget.createDelegate(this),
              header : 'Target',
              width : 120
            }, {
              name : 'sRepository',
              mapping : 'properties',
              convert : this.convertRepository.createDelegate(this),
              header : 'Repository',
              width : 150
            }, {
              name : 'sMethod',
              mapping : 'properties',
              convert : this.convertMethod.createDelegate(this),
              header : 'Method',
              width : 150
            }],
        listeners : {
          beforedestroy : {
            fn : function() {
              Sonatype.Events.un('privilegeAddMenuInit', this.onAddMenuInit, this);
              Sonatype.Events.un('privilegeViewInit', this.onViewInit, this);
            },
            scope : this
          }
        }
      });

  Sonatype.Events.fireEvent('privilegePanelInit', this);
};

Ext.extend(Sonatype.repoServer.PrivilegeEditPanel, Sonatype.panels.GridViewer, {
      emptyText: 'No privilege defined',
      emptyTextWhileFiltering: 'No privilege matched criteria: {criteria}',

      convertRepository : function(value, parent) {
        var i, targetPriv = false;
        for (i = 0; i < parent.properties.length; i+=1)
        {
          if (parent.properties[i].key === 'repositoryId' && !Ext.isEmpty(parent.properties[i].value))
          {
            return this.convertDataValue(parent.properties[i].value, this.repoStore, 'id', 'name');
          }
          else if (parent.properties[i].key === 'repositoryGroupId' && !Ext.isEmpty(parent.properties[i].value))
          {
            return this.convertDataValue(parent.properties[i].value, this.groupStore, 'id', 'name');
          }
          else if (parent.properties[i].key === 'repositoryTargetId')
          {
            targetPriv = true;
          }
        }

        if (targetPriv)
        {
          return 'All Repositories';
        }
        else
        {
          return '';
        }
      },

      convertTarget : function(value, parent) {
        var i;
        for (i = 0; i < parent.properties.length; i+=1)
        {
          if (parent.properties[i].key === 'repositoryTargetId' && !Ext.isEmpty(parent.properties[i].value))
          {
            return this.convertDataValue(parent.properties[i].value, this.targetStore, 'id', 'name');
          }
        }
        return '';
      },

      convertType : function(value, parent) {
        return this.convertDataValue(value, this.privilegeTypeStore, 'id', 'name');
      },

      convertMethod : function(value, parent) {
        var i;
        for (i = 0; i < parent.properties.length; i+=1)
        {
          if (parent.properties[i].key === 'method' && !Ext.isEmpty(parent.properties[i].value))
          {
            return parent.properties[i].value;
          }
        }
        return '';
      },

      convertStringProperty : function(value, parent) {
        return value;
      },

      convertRepositoryProperty : function(value, parent) {
        var i;
        if (Ext.isEmpty(value))
        {
          for (i = 0; i < parent.length; i+=1)
          {
            if (parent[i].key === 'repositoryGroupId' && !Ext.isEmpty(parent[i].value))
            {
              return '';
            }
          }
          return 'All Repositories';
        }
        return this.convertDataValue(value, this.repoStore, 'id', 'name');
      },

      convertRepoGroupProperty : function(value, parent) {
        if (!Ext.isEmpty(value))
        {
          return this.convertDataValue(value, this.groupStore, 'id', 'name');
        }
        return '';
      },

      convertRepoTargetProperty : function(value, parent) {
        return this.convertDataValue(value, this.targetStore, 'id', 'name');
      },

      convertRepoOrGroupProperty : function(value, parent) {
        var result = '';
        result = this.convertRepoGroupProperty(value, parent);
        if (!Ext.isEmpty(result))
        {
          return result;
        }
        result = this.convertRepositoryProperty(value, parent);
        return result;
      },

      onAddMenuInit : function(menu) {
        if (this.sp.checkPermission('security:privileges', this.sp.CREATE))
        {
          menu.add(['-', {
                text : 'Repository Target Privilege',
                autoCreateNewRecord : true,
                handler : function(container, rec, item, e) {
                  rec.beginEdit();
                  rec.set('type', 'target');
                  rec.set('sType', this.convertType('target'));
                  rec.commit();
                  rec.endEdit();
                },
                scope : this
              }]);
        }
      },

      onViewInit : function(cardPanel, rec) {
        var editor = new Sonatype.repoServer.PrivilegeEditor({
              payload : rec,
              privilegeTypeStore : this.privilegeTypeStore,
              repoStore : this.repoStore,
              targetStore : this.targetStore,
              groupStore : this.groupStore,
              propertyTypeStore : this.propertyTypeStore
            });
        editor.on('submit', this.submitHandler, this);

        Sonatype.Events.fireEvent('privilegeEditorInit', editor);

        cardPanel.add(editor);
      },

      onPrivilegeClicked : function(cardPanel, rec) {
        this.toolbarDeleteButton.setVisible(rec.data.userManaged);
      },

      submitHandler : function(form, action, receivedData) {
        var i, r;
        if (Ext.isArray(receivedData))
        {
          for (i = 0; i < receivedData.length; i+=1)
          {
            r = receivedData[i];
            r.sTarget = this.convertTarget(r.repositoryTargetId, r);
            r.sType = this.convertType(r.type, r);
            r.sRepository = this.convertRepository(r.repositoryId, r);
            r.sResourceURI = r.resourceURI;
            r.resourceURI = Sonatype.config.host + Sonatype.config.repos.urls.privileges + '/' + r.id;
          }
        }
      }
    });

Sonatype.repoServer.PrivilegeEditor = function(config) {
  var i, items, targetStore2, typeRec,
        ht = Sonatype.repoServer.resources.help.privileges,
        defaultConfig = {
    uri : Sonatype.config.repos.urls.privileges + '_target',
    dataModifiers : {
      load : {
        properties : function(value, parent, fpanel) {
          for (i = 0; i < value.length; i+=1)
          {
            var field = fpanel.form.findField(value[i].key);
            field.setValue(fpanel.propertyTypeStore.getAt(fpanel.propertyTypeStore.find('type', field.fieldConverterType)).data.converter(value[i].value, value));
          }
        },
        type : function(value, parent, fpanel) {
          return fpanel.convertDataValue(value, fpanel.privilegeTypeStore, 'id', 'name');
        }
      },
      submit : {
        method : function(val, fpanel) {
          return ['create', 'read', 'update', 'delete'];
        },
        repositoryId : function(val, fpanel) {
          var v = fpanel.form.findField('repositoryOrGroup').getValue();
          return v.indexOf('repo_') === 0 ? v.substring('repo_'.length) : '';
        },
        repositoryGroupId : function(val, fpanel) {
          var v = fpanel.form.findField('repositoryOrGroup').getValue();
          return v.indexOf('group_') === 0 ? v.substring('group_'.length) : '';
        },
        type : function(val, fpanel) {
          return 'target';
        }
      }
    },
    validationModifiers : {
      repositoryId : "repositoryOrGroup",
      repositoryGroupId : "repositoryOrGroup"
    },
    referenceData : Sonatype.repoServer.referenceData.privileges.target
  };
  Ext.apply(this, config || {}, defaultConfig);

  this.COMBO_WIDTH = 300;
  this.sp = Sonatype.lib.Permissions;

  this.combinedStore = new Ext.data.JsonStore({
    root: 'data',
    id: 'id',
    fields: [
      { name: 'id' },
      { name: 'format' },
      { name: 'name', sortType: Ext.data.SortTypes.asUCString }
    ],
    sortInfo: {
      field: 'name',
      direction: 'ASC'
    },
    url: Sonatype.config.repos.urls.repositories,
  });
  this.initCombinedStore();

  this.checkPayload();
  if (!(this.sp.checkPermission('security:privileges', this.sp.UPDATE) || this.isNew))
  {
    this.readOnly = true;
  }

  items = [{
        xtype : 'hidden',
        name : 'id'
      }, {
        xtype : 'textfield',
        fieldLabel : 'Name',
        itemCls : this.readOnly ? '' : 'required-field',
        helpText : ht.name,
        name : 'name',
        allowBlank : false,
        width : this.COMBO_WIDTH,
        disabled : this.readOnly
      }, {
        xtype : 'textfield',
        fieldLabel : 'Description',
        itemCls : this.readOnly ? '' : 'required-field',
        helpText : ht.description,
        name : 'description',
        allowBlank : false,
        width : this.COMBO_WIDTH,
        disabled : this.readOnly
      }];

  if (this.isNew)
  {
    // clone the target store
    targetStore2 = new Ext.data.JsonStore({
          root : 'data',
          id : 'id',
          fields : [{
                name : 'id'
              }, {
                name : 'contentClass'
              }, {
                name : 'name',
                sortType : Ext.data.SortTypes.asUCString
              }],
          url : Sonatype.config.repos.urls.repoTargets
        });
    targetStore2.add(this.targetStore.getRange());
    this.targetStore = targetStore2;

    items.push({
          xtype : 'combo',
          fieldLabel : 'Repository',
          itemCls : 'required-field',
          helpText : ht.repositoryOrGroup,
          name : 'repositoryOrGroup',
          store : this.combinedStore,
          displayField : 'name',
          valueField : 'id',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false,
          width : this.COMBO_WIDTH,
          minListWidth : this.COMBO_WIDTH,
          value : "all_repo",
          listeners : {
            select : {
              fn : this.repositorySelectHandler,
              scope : this
            }
          }
        });
    items.push({
          xtype : 'combo',
          fieldLabel : 'Repository Target',
          itemCls : 'required-field',
          helpText : ht.repositoryTarget,
          name : 'repositoryTargetId',
          store : this.targetStore,
          displayField : 'name',
          valueField : 'id',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          width : this.COMBO_WIDTH
        });
  }
  else
  {
    items.push({
          xtype : 'textfield',
          fieldLabel : 'Type',
          helpText : ht.type,
          name : 'type',
          width : this.COMBO_WIDTH,
          disabled : true
        });

    typeRec = this.privilegeTypeStore.getById(this.payload.data.type);

    if (!Ext.isEmpty(typeRec))
    {
      for (i = 0; i < typeRec.data.properties.length; i+=1)
      {
        items.push({
              xtype : 'textfield',
              fieldConverterType : typeRec.data.properties[i].type,
              fieldLabel : typeRec.data.properties[i].name,
              helpText : typeRec.data.properties[i].helpText,
              name : typeRec.data.properties[i].id,
              width : this.COMBO_WIDTH,
              disabled : true
            });
      }
    }
  }

  Sonatype.repoServer.PrivilegeEditor.superclass.constructor.call(this, {
        labelWidth : 120,
        items : items
      });
};

Ext.extend(Sonatype.repoServer.PrivilegeEditor, Sonatype.ext.FormPanel, {
      initCombinedRecord : function(rec) {
        var isGroup = rec.data.repoType === null || rec.data.repoType === undefined;
        return {
          id : (isGroup ? 'group_' : 'repo_') + rec.data.id,
          name : rec.data.name + (isGroup ? ' (Group)' : ' (Repo)'),
          format : rec.data.format
        };
      },

      initCombinedStore : function() {
        var data = [{
              id : 'all_repo',
              name : 'All Repositories'
            }];

        this.repoStore.each(function(rec) {
              data.push(this.initCombinedRecord(rec));
            }, this);
        this.groupStore.each(function(rec) {
              data.push(this.initCombinedRecord(rec));
            }, this);

        this.combinedStore.loadData({
              data : data
            });
      },

      repositorySelectHandler : function(combo, rec, index) {
        var
              targetCombo = this.form.findField('repositoryTargetId'),
              previousValue = targetCombo.getValue(),

        filterValue = rec.data.format;
        targetCombo.setValue(null);
        targetCombo.store.clearFilter();

        if (filterValue)
        {
          targetCombo.store.filter('contentClass', filterValue);
        }

        if (targetCombo.store.getById(previousValue))
        {
          targetCombo.setValue(previousValue);
        }
      }
    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('repoServer/UserPrivilegeBrowserPanel',['Sonatype/all'], function(){
Sonatype.repoServer.UserPrivilegeBrowsePanel = function(config) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  this.appliedPrivilegesDataStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        url : Sonatype.config.servicePath + '/assigned_privileges/' + this.payload.get('userId'),
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true,
        fields : [{
              name : 'id'
            }, {
              name : 'name'
            }, {
              name : 'parents'
            }],
        listeners : {
          load : {
            fn : function(store, records, options) {
              this.loadPrivilegeList();
            },
            scope : this
          }
        }
      });

  Sonatype.repoServer.UserPrivilegeBrowsePanel.superclass.constructor.call(this, {
        region : 'center',
        width : '100%',
        height : '100%',
        autoScroll : true,
        border : false,
        frame : false,
        collapsible : false,
        collapsed : false,
        tbar : [{
              text : 'Refresh',
              iconCls : 'st-icon-refresh',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.refreshHandler
            }],
        items : [{
              xtype : 'panel',
              layout : 'table',
              layoutConfig : {
                columns : 2
              },
              frame : true,
              items : [{
                    xtype : 'panel',
                    style : 'padding: 10px; font-size: 11px;',
                    html : 'Select a privilege to view the role(s) in the user<br>that grant the privilege.'
                  }, {
                    xtype : 'panel',
                    style : 'padding: 10px; font-size: 11px;',
                    html : 'List of roles in the user that grant the selected privilege.<br>Expand the role to find nested role(s) that contain<br>the privilege.'
                  }, {
                    xtype : 'treepanel',
                    name : 'privilege-list',
                    title : 'Privileges',
                    border : true,
                    bodyBorder : true,
                    bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
                    style : 'padding: 0 20px 0 0',
                    width : 325,
                    height : 275,
                    animate : true,
                    lines : false,
                    autoScroll : true,
                    containerScroll : true,
                    rootVisible : false,
                    ddScroll : false,
                    enableDD : false,
                    root : new Ext.tree.TreeNode({
                          text : 'root',
                          draggable : false
                        })
                  }, {
                    xtype : 'treepanel',
                    name : 'role-tree',
                    title : 'Role Containment',
                    border : true,
                    bodyBorder : true,
                    bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
                    width : 325,
                    height : 275,
                    animate : true,
                    lines : false,
                    autoScroll : true,
                    containerScroll : true,
                    rootVisible : false,
                    ddScroll : false,
                    enableDD : false,
                    root : new Ext.tree.TreeNode({
                          text : 'root',
                          draggable : false
                        })
                  }]
            }]
      });
};

Ext.extend(Sonatype.repoServer.UserPrivilegeBrowsePanel, Ext.FormPanel, {
      refreshHandler : function(button, e) {
        // the load listener on these stores will reload the privileges
        this.appliedPrivilegesDataStore.reload();
        var tree = this.find('name', 'role-tree')[0];
        while (tree.root.lastChild)
        {
          tree.root.removeChild(tree.root.lastChild);
        }
      },
      clickHandler : function(node, event) {
        var tree = this.find('name', 'role-tree')[0];
        while (tree.root.lastChild)
        {
          tree.root.removeChild(tree.root.lastChild);
        }

        this.idCounter = 0;

        for (var i = 0; i < node.attributes.payload.data.parents.length; i++)
        {
          this.addRoleToTree(tree.root, node.attributes.payload.data.parents[i]);
        }
      },
      addRoleToTree : function(rootNode, childData) {
        var childNode = new Ext.tree.TreeNode({
              id : this.idCounter++,
              text : childData.name,
              payload : childData,
              allowChildren : (childData.parents && childData.parents.length > 0) ? true : false,
              draggable : false,
              leaf : (childData.parents && childData.parents.length > 0) ? false : true,
              icon : Sonatype.config.extPath + '/resources/images/default/tree/folder.gif'
            });

        rootNode.appendChild(childNode);

        if (childData.parents)
        {
          for (var i = 0; i < childData.parents.length; i++)
          {
            this.addRoleToTree(childNode, childData.parents[i]);
          }
        }
      },
      loadPrivilegeList : function() {
        var privilegeList = this.find('name', 'privilege-list')[0];
        if (!privilegeList) {
          // not displayed yet
          return;
        }
        while (privilegeList.root.lastChild)
        {
          privilegeList.root.removeChild(privilegeList.root.lastChild);
        }
        var privilegeRecs = this.appliedPrivilegesDataStore.getRange();
        for (var i = 0; i < privilegeRecs.length; i++)
        {
          privilegeList.root.appendChild(new Ext.tree.TreeNode({
                id : privilegeRecs[i].data.id,
                text : privilegeRecs[i].data.name,
                payload : privilegeRecs[i], // sonatype added
                allowChildren : false,
                draggable : false,
                leaf : true,
                listeners : {
                  click : {
                    fn : this.clickHandler,
                    scope : this
                  }
                }
              }));
        }
      }
    });

Sonatype.Events.addListener('userViewInit', function(cardPanel, rec, gridPanel) {
      if (rec.data.resourceURI)
      {
        cardPanel.add(new Sonatype.repoServer.UserPrivilegeBrowsePanel({
              payload : rec,
              tabTitle : 'Privilege Trace'
            }));
      }
    });
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Service Schedule Edit/Create panel layout and controller
 */

define('repoServer/SchedulesEditPanel',['Sonatype/all', 'Sonatype/strings','Nexus/ext/GridFilterBox'], function(Sonatype, Strings){
Sonatype.repoServer.SchedulesEditPanel = function(config) {
  var config = config || {},
      defaultConfig = {};

  Ext.apply(this, config, defaultConfig);

  var ht = Sonatype.repoServer.resources.help.schedules;

  // List of schedule types
  // None removed for the time being
  this.scheduleTypeStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['Manual'], ['Once'], ['Hourly'], ['Daily'], ['Weekly'], ['Monthly'], ['Advanced']]
      });
  // List of weekdays
  this.weekdaysStore = new Ext.data.SimpleStore({
        fields : ['name', 'id'],
        data : [['Sunday', 'sunday'], ['Monday', 'monday'], ['Tuesday', 'tuesday'], ['Wednesday', 'wednesday'], ['Thursday', 'thursday'], ['Friday', 'friday'], ['Saturday', 'saturday']]
      });

  this.customTypes = {};

  this.stopButton = new Ext.Button({
    id : 'schedule-stop-btn',
    text : 'Cancel',
    icon : Sonatype.config.resourcePath + '/static/images/icons/time_delete.png',
    cls : 'x-btn-text-icon',
    scope : this,
    handler : this.stopHandler,
    disabled : true
      });

  this.runButton = new Ext.Button({
        id : 'schedule-run-btn',
        text : 'Run',
        icon : Sonatype.config.resourcePath + '/static/images/icons/time_go.png',
        cls : 'x-btn-text-icon',
        scope : this,
        handler : this.runHandler,
        disabled : true
      });

  this.deleteButton = new Ext.Button({
        id : 'schedule-delete-btn',
        text : 'Delete',
        icon : Sonatype.config.resourcePath + '/static/images/icons/delete.png',
        cls : 'x-btn-text-icon',
        scope : this,
        handler : this.deleteHandler,
        disabled : true
      });

  this.disableEditingHeader = new Ext.Panel({
        id : 'disablingMsg',
        name : 'disablingMsg',
        layout : 'table',
        hidden : true,
        style : 'font-size: 18px; padding: 5px 0px 5px 15px',
        items : [{
              html : '<b>This is scheduled to be run.  It can\'t be edited or deleted.</b><br><hr/>'
            }]
      });

  // Methods that will take the incoming json data and map over to the ui
  // controls
  this.loadDataModFuncs = {
    internal : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this)
    },
    manual : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this)
    },
    once : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this),
      startDate : this.importStartDateHelper.createDelegate(this),
      startTime : this.importStartTimeHelper.createDelegate(this)
    },
    hourly : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this),
      startDate : this.importStartDateHelper.createDelegate(this),
      startTime : this.importStartTimeHelper.createDelegate(this)
    },
    daily : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this),
      startDate : this.importStartDateHelper.createDelegate(this),
      recurringTime : this.importRecurringTimeHelper.createDelegate(this)
    },
    weekly : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this),
      startDate : this.importStartDateHelper.createDelegate(this),
      recurringTime : this.importRecurringTimeHelper.createDelegate(this),
      recurringDay : this.importRecurringDayHelper.createDelegate(this)
    },
    monthly : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this),
      startDate : this.importStartDateHelper.createDelegate(this),
      recurringTime : this.importRecurringTimeHelper.createDelegate(this),
      recurringDay : this.importMonthlyRecurringDayHelper.createDelegate(this)
    },
    advanced : {
      schedule : Strings.capitalize,
      properties : this.importServicePropertiesHelper.createDelegate(this)
    }
  };

  // Methods that will take the data from the ui controls and map over to json
  this.submitDataModFuncs = {
    manual : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this)
    },
    once : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this),
      startDate : this.exportStartDateHelper.createDelegate(this),
      startTime : this.exportStartTimeHelper.createDelegate(this)
    },
    hourly : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this),
      startDate : this.exportStartDateHelper.createDelegate(this),
      startTime : this.exportStartTimeHelper.createDelegate(this)
    },
    daily : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this),
      startDate : this.exportStartDateHelper.createDelegate(this),
      recurringTime : this.exportRecurringTimeHelper.createDelegate(this)
    },
    weekly : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this),
      startDate : this.exportStartDateHelper.createDelegate(this),
      recurringTime : this.exportRecurringTimeHelper.createDelegate(this),
      recurringDay : this.exportRecurringDayHelper.createDelegate(this)
    },
    monthly : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this),
      startDate : this.exportStartDateHelper.createDelegate(this),
      recurringTime : this.exportRecurringTimeHelper.createDelegate(this),
      recurringDay : this.exportMonthlyRecurringDayHelper.createDelegate(this)
    },
    advanced : {
      schedule : Strings.lowercase,
      properties : this.exportServicePropertiesHelper.createDelegate(this)
    }
  };

  // A record to hold the name and id of a repository
  this.repositoryRecordConstructor = Ext.data.Record.create([{
        name : 'id'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }]);

  // A record to hold the name and id of a repository group
  this.repositoryGroupRecordConstructor = Ext.data.Record.create([{
        name : 'id'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }]);

  this.repositoryOrGroupRecordConstructor = Ext.data.Record.create([{
        name : 'id'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }]);

  // Simply a record to hold details of each service type
  this.serviceTypeRecordConstructor = Ext.data.Record.create([{
        name : 'id',
        sortType : Ext.data.SortTypes.asUCString
      }, {
        name : 'name'
      }, {
        name : 'formFields'
      }]);

  // A record that holds the data for each service in the system
  this.scheduleRecordConstructor = Ext.data.Record.create([{
        name : 'resourceURI'
      }, {
        name : 'enabled'
      }, {
        name : 'id'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }, {
        name : 'typeName'
      }, {
        name : 'typeId'
      }, {
        name : 'readableStatus'
      }, {
        name : 'alertEmail'
      }, {
        name : 'schedule'
      }, {
        name : 'nextRunTimeInMillis',
        convert : function(v) {
          return v?new Date(v):'n/a';
        }
      }, {
        name : 'lastRunTimeInMillis',
        convert : function(v) {
          return v?new Date(v):'n/a';
        }
      }, {
        name : 'lastRunResult'
      }]);

  // Datastore that will hold both repos and repogroups
  this.repoOrGroupDataStore = new Ext.data.SimpleStore({
        fields : ['id', 'name'],
        id : 'id'
      });

  // Reader and datastore that queries the server for the list of repositories
  this.repositoryReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'id'
      }, this.repositoryRecordConstructor);
  this.repositoryDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.repositories,
        reader : this.repositoryReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true,
        listeners : {
          'load' : {
            fn : function() {
              this.repositoryDataStore.each(function(item, i, len) {
                    var newRec = new this.repositoryOrGroupRecordConstructor({
                          id : item.data.id,
                          name : item.data.name + ' (Repo)'
                        }, item.id);
                    this.repoOrGroupDataStore.add([newRec]);
                  }, this);
              var allRec = new this.repositoryRecordConstructor({
                    id : 'all_repo',
                    name : 'All Repositories'
                  }, 'all_repo');
              this.repoOrGroupDataStore.insert(0, allRec);
            },
            scope : this
          }
        }
      });
  this.repositoryTargetDataStore = new Ext.data.JsonStore({
    url : Sonatype.config.repos.urls.repoTargets,
    root : 'data',
    id : 'id',
    fields : ['id', 'name', 'resourceURI', 'contentClass'],
    autoLoad: true
  });

  // Reader and datastore that queries the server for the list of repository
  // groups
  this.repositoryGroupReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'id'
      }, this.repositoryGroupRecordConstructor);
  this.repositoryGroupDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.groups,
        reader : this.repositoryGroupReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true,
        listeners : {
          'load' : {
            fn : function() {
              this.repositoryGroupDataStore.each(function(item, i, len) {
                    var newRec = new this.repositoryOrGroupRecordConstructor({
                          id : item.data.id,
                          name : item.data.name + ' (Group)'
                        }, item.id);
                    this.repoOrGroupDataStore.add([newRec]);
                  }, this);
            },
            scope : this
          }
        }
      });

  // Reader and datastore that queries the server for the list of service types
  this.serviceTypeReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'id'
      }, this.serviceTypeRecordConstructor);
  this.serviceTypeDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.scheduleTypes,
        reader : this.serviceTypeReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true
      });

  // Reader and datastore that queries the server for the list of currently
  // defined services
  this.schedulesReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'resourceURI'
      }, this.scheduleRecordConstructor);
  this.schedulesDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.schedules,
        reader : this.schedulesReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true
      });

  this.COMBO_WIDTH = 300;
  
  this.timeZone = new Date().toTimeString();
  this.timeZone = this.timeZone.substring(this.timeZone.indexOf(" "));

  // Build the form
  this.formConfig = {};
  this.formConfig.schedule = {
    region : 'center',
    width : '100%',
    height : '100%',
    autoScroll : true,
    border : false,
    frame : true,
    collapsible : false,
    collapsed : false,
    labelWidth : 200,
    layoutConfig : {
      labelSeparator : ''
    },

    items : [/* this.disableEditingHeader, */{
          xtype : 'hidden',
          name : 'id'
        }, {
          xtype : 'checkbox',
          fieldLabel : 'Enabled',
          labelStyle : 'margin-left: 15px; width: 185px;',
          helpText : ht.enabled,
          name : 'enabled',
          allowBlank : false,
          checked : true
        }, {
          xtype : 'textfield',
          fieldLabel : 'Name',
          labelStyle : 'margin-left: 15px; width: 185px;',
          itemCls : 'required-field',
          helpText : ht.name,
          name : 'name',
          width : this.COMBO_WIDTH,
          allowBlank : false
        }, {
          xtype : 'combo',
          fieldLabel : 'Task Type',
          labelStyle : 'margin-left: 15px; width: 185px;',
          itemCls : 'required-field',
          helpText : ht.serviceType,
          name : 'typeId',
          store : this.serviceTypeDataStore,
          displayField : 'name',
          valueField : 'id',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false,
          width : this.COMBO_WIDTH
        }, {
          xtype : 'panel',
          id : 'service-type-config-card-panel',
          header : false,
          layout : 'card',
          region : 'center',
          activeItem : 0,
          bodyStyle : 'padding:15px',
          deferredRender : false,
          autoScroll : false,
          frame : false,
          items : []
        }, {
          xtype : 'textfield',
          fieldLabel : 'Alert Email',
          labelStyle : 'margin-left: 15px; width: 185px;',
          helpText : ht.alertEmail,
          name : 'alertEmail',
          width : this.COMBO_WIDTH,
          allowBlank : true
        }, {
          xtype : 'combo',
          fieldLabel : 'Recurrence',
          labelStyle : 'margin-left: 15px; width: 185px;',
          itemCls : 'required-field',
          helpText : ht.serviceSchedule,
          name : 'schedule',
          store : this.scheduleTypeStore,
          displayField : 'value',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false,
          width : this.COMBO_WIDTH
        }, {
          xtype : 'panel',
          id : 'schedule-config-card-panel',
          header : false,
          layout : 'card',
          region : 'center',
          activeItem : 0,
          bodyStyle : 'padding:15px',
          deferredRender : false,
          autoScroll : false,
          frame : false,
          items : [{
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'label',
                      text : 'Without recurrence, this service can only be run manually.'
                    }]
              }, {
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'datefield',
                      fieldLabel : 'Start Date',
                      itemCls : 'required-field',
                      helpText : ht.startDate,
                      name : 'startDate',
                      disabled : true,
                      allowBlank : false,
                      value : new Date()
                    }, {
                      xtype : 'timefield',
                      fieldLabel : 'Start Time',
                      itemCls : 'required-field',
                      afterText : this.timeZone,
                      helpText : ht.startTime,
                      name : 'startTime',
                      width : 75,
                      disabled : true,
                      allowBlank : false,
                      format : 'H:i'
                    }]
              }, {
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'datefield',
                      fieldLabel : 'Start Date',
                      itemCls : 'required-field',
                      helpText : ht.startDate,
                      name : 'startDate',
                      disabled : true,
                      allowBlank : false,
                      value : new Date()
                    }, {
                      xtype : 'timefield',
                      fieldLabel : 'Start Time',
                      itemCls : 'required-field',
                      afterText : this.timeZone,
                      helpText : ht.startTime,
                      name : 'startTime',
                      width : 75,
                      disabled : true,
                      allowBlank : false,
                      format : 'H:i'
                    }]
              }, {
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'datefield',
                      fieldLabel : 'Start Date',
                      itemCls : 'required-field',
                      helpText : ht.startDate,
                      name : 'startDate',
                      disabled : true,
                      allowBlank : false,
                      value : new Date()
                    }, {
                      xtype : 'timefield',
                      fieldLabel : 'Recurring Time',
                      itemCls : 'required-field',
                      afterText : this.timeZone,
                      helpText : ht.recurringTime,
                      name : 'recurringTime',
                      width : 75,
                      disabled : true,
                      allowBlank : false,
                      format : 'H:i'
                    }]
              }, {
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'datefield',
                      fieldLabel : 'Start Date',
                      itemCls : 'required-field',
                      helpText : ht.startDate,
                      name : 'startDate',
                      disabled : true,
                      allowBlank : false,
                      value : new Date()
                    }, {
                      xtype : 'timefield',
                      fieldLabel : 'Recurring Time',
                      itemCls : 'required-field',
                      afterText : this.timeZone,
                      helpText : ht.recurringTime,
                      name : 'recurringTime',
                      width : 75,
                      disabled : true,
                      allowBlank : false,
                      format : 'H:i'
                    }, {
                      xtype : 'twinpanelchooser',
                      titleLeft : 'Selected Days',
                      titleRight : 'Available Days',
                      name : 'weekdays',
                      displayField : 'name',
                      valueField : 'id',
                      store : this.weekdaysStore,
                      required : true,
                      halfSize : true
                    }]
              }, {
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'datefield',
                      fieldLabel : 'Start Date',
                      itemCls : 'required-field',
                      helpText : ht.startDate,
                      name : 'startDate',
                      disabled : true,
                      allowBlank : false,
                      value : new Date()
                    }, {
                      xtype : 'timefield',
                      fieldLabel : 'Recurring Time',
                      itemCls : 'required-field',
                      afterText : this.timeZone,
                      helpText : ht.recurringTime,
                      name : 'recurringTime',
                      width : 75,
                      disabled : true,
                      allowBlank : false,
                      format : 'H:i'
                    }, {
                      xtype : 'panel',
                      layout : 'column',
                      items : [{
                            width : 180,
                            xtype : 'label',
                            text : 'Days'
                          }, {
                            xtype : 'panel',
                            layout : 'column',
                            items : [{
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '1',
                                        name : 'day1'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '8',
                                        name : 'day8'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '15',
                                        name : 'day15'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '22',
                                        name : 'day22'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '29',
                                        name : 'day29'
                                      }]
                                }, {
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '2',
                                        name : 'day2'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '9',
                                        name : 'day9'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '16',
                                        name : 'day16'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '23',
                                        name : 'day23'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '30',
                                        name : 'day30'
                                      }]
                                }, {
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '3',
                                        name : 'day3'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '10',
                                        name : 'day10'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '17',
                                        name : 'day17'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '24',
                                        name : 'day24'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '31',
                                        name : 'day31'
                                      }]
                                }, {
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '4',
                                        name : 'day4'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '11',
                                        name : 'day11'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '18',
                                        name : 'day18'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '25',
                                        name : 'day25'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : 'Last',
                                        name : 'dayLast'
                                      }]
                                }, {
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '5',
                                        name : 'day5'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '12',
                                        name : 'day12'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '19',
                                        name : 'day19'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '26',
                                        name : 'day26'
                                      }]
                                }, {
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '6',
                                        name : 'day6'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '13',
                                        name : 'day13'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '20',
                                        name : 'day20'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '27',
                                        name : 'day27'
                                      }]
                                }, {
                                  xtype : 'panel',
                                  width : 50,
                                  items : [{
                                        xtype : 'checkbox',
                                        boxLabel : '7',
                                        name : 'day7'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '14',
                                        name : 'day14'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '21',
                                        name : 'day21'
                                      }, {
                                        xtype : 'checkbox',
                                        boxLabel : '28',
                                        name : 'day28'
                                      }]
                                }]
                          }]
                    }]
              }, {
                xtype : 'fieldset',
                checkboxToggle : false,
                title : 'Schedule Settings',
                anchor : Sonatype.view.FIELDSET_OFFSET,
                collapsible : false,
                autoHeight : true,
                labelWidth : 175,
                layoutConfig : {
                  labelSeparator : ''
                },
                items : [{
                      xtype : 'textfield',
                      fieldLabel : 'CRON expression',
                      itemCls : 'required-field',
                      name : 'cronCommand',
                      helpText : ht.cronCommand,
                      disabled : true,
                      allowBlank : false,
                      width : this.COMBO_WIDTH
                    }, {
                      xtype : 'panel',
                      layout : 'fit',
                      html : Sonatype.repoServer.resources.help.cronBriefHelp.text
                    }, {
                      xtype : 'button',
                      text : 'Show More Help >>',
                      hideLabel : true,
                      listeners : {
                        click : {
                          fn : function(target, event) {
                            var helpIndex = target.ownerCt.items.indexOf(target);
                            var helpPanel = target.ownerCt.items.itemAt(helpIndex + 1);
                            if (helpPanel.collapsed)
                            {
                              helpPanel.expand(false);
                              target.setText("Hide Extra Help <<");
                            }
                            else
                            {
                              helpPanel.collapse(false);
                              target.setText("Show More Help >>");
                            }
                          },
                          scope : this
                        }
                      }
                    }, {
                      xtype : 'panel',
                      collapsible : true,
                      collapsed : true,
                      layout : 'fit',
                      html : Sonatype.repoServer.resources.help.cronBigHelp.text
                    }]
              }]
        }],
    buttons : [{
          text : 'Save',
          disabled : true
        }, {
          text : 'Cancel'
        }]
  };

  this.sp = Sonatype.lib.Permissions;

  this.schedulesGridPanel = new Ext.grid.GridPanel({
        id : 'st-schedules-grid',
        region : 'north',
        layout : 'fit',
        split : true,
        height : 200,
        minHeight : 150,
        maxHeight : 400,
        frame : false,
        autoScroll : true,
        tbar : [{
              id : 'schedule-refresh-btn',
              text : 'Refresh',
              iconCls : 'st-icon-refresh',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.reloadAll
            }, {
              id : 'schedule-add-btn',
              text : 'Add',
              icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.addResourceHandler,
              disabled : !this.sp.checkPermission('nexus:tasks', this.sp.CREATE)
            },
            this.runButton,
            this.stopButton,
            this.deleteButton
        ],

        // grid view options
        ds : this.schedulesDataStore,
        sortInfo : {
          field : 'name',
          direction : "ASC"
        },
        loadMask : true,
        deferredRender : false,
        columns : [{
              header : 'Enabled',
              dataIndex : 'enabled',
              width : 50,
              id : 'schedule-config-enabled-col'
            }, {
              header : 'Name',
              dataIndex : 'name',
              width : 175,
              id : 'schedule-config-name-col'
            }, {
              header : 'Type',
              dataIndex : 'typeName',
              width : 175,
              id : 'schedule-config-service-type-col'
            }, {
              header : 'Status',
              dataIndex : 'readableStatus',
              width : 100,
              id : 'schedule-config-service-status-col'
            }, {
              header : 'Schedule',
              dataIndex : 'schedule',
              width : 100,
              id : 'schedule-config-service-schedule-col'
            }, {
              header : 'Next Run',
              dataIndex : 'nextRunTimeInMillis',
              width : 250,
              id : 'schedule-config-service-next-run-col'
            }, {
              header : 'Last Run',
              dataIndex : 'lastRunTimeInMillis',
              width : 250,
              id : 'schedule-config-service-last-run-col'
            }, {
              header : 'Last Result',
              dataIndex : 'lastRunResult',
              width : 175,
              id : 'schedule-config-service-last-result-col'
            }],
        autoExpandColumn : 'schedule-config-service-last-result-col',
        disableSelection : false,
        viewConfig : {
          deferEmptyText: false,
          emptyText: 'No scheduled tasks defined',
          emptyTextWhileFiltering: 'No scheduled tasks matched criteria: {criteria}'
        }
      });
  this.schedulesGridPanel.getSelectionModel().on('rowselect', this.rowSelect, this);

  this.schedulesGridPanel.getTopToolbar().add([
    '->',
    NX.create('Nexus.ext.GridFilterBox', {
      filteredGrid: this.schedulesGridPanel
    })
  ]);

  Sonatype.repoServer.SchedulesEditPanel.superclass.constructor.call(this, {
        layout : 'border',
        autoScroll : false,
        width : '100%',
        height : '100%',
        items : [this.schedulesGridPanel, {
              xtype : 'panel',
              id : 'schedule-config-forms',
              title : 'Scheduled Task Configuration',
              layout : 'card',
              region : 'center',
              activeItem : 0,
              deferredRender : false,
              autoScroll : false,
              frame : false,
              items : [{
                    xtype : 'panel',
                    layout : 'fit',
                    html : '<div class="little-padding">Select a scheduled task to edit it, or click "Add" to create a new one.</div>'
                  }]
            }]
      });

  this.formCards = this.findById('schedule-config-forms');

  this.on('render', this.initializeCustomTypes, this);
};

Ext.extend(Sonatype.repoServer.SchedulesEditPanel, Ext.Panel, {
      initializeCustomTypes : function() {
        Sonatype.Events.fireEvent('initializeCustomTypes', this.customTypes);
      },

      // Dump the currently stored data and requery for everything
      reloadAll : function() {
        var gridSelectModel = this.schedulesGridPanel.getSelectionModel();
        gridSelectModel.clearSelections();
        this.formCards.getLayout().setActiveItem(0);

        this.runButton.disable();
        this.stopButton.disable();
        this.deleteButton.disable();

        this.schedulesDataStore.removeAll();
        this.schedulesDataStore.reload();
        this.repoOrGroupDataStore.removeAll();
        this.repositoryDataStore.reload();
        this.repositoryTargetDataStore.reload();
        this.repositoryGroupDataStore.reload();
        this.serviceTypeDataStore.reload();
        this.formCards.items.each(function(item, i, len) {
              if (i > 0)
              {
                this.remove(item, true);
              }
            }, this.formCards);

        // //Enable add button on refresh
        // this.schedulesGridPanel.getTopToolbar().items.get('schedule-add-btn').enable();
      },

      markTreeInvalid : function(tree) {
        var elp = tree.getEl();

        if (!tree.errorEl)
        {
          tree.errorEl = elp.createChild({
                cls : 'x-form-invalid-msg'
              });
          tree.errorEl.setWidth(elp.getWidth(true)); // note removed -20 like
          // on form fields
        }
        tree.invalid = true;
        tree.errorEl.update(tree.invalidText);
        elp.child('.x-panel-body').setStyle({
              'background-color' : '#fee',
              border : '1px solid #dd7870'
            });
        Ext.form.Field.msgFx['normal'].show(tree.errorEl, tree);
      },

      saveHandler : function(formInfoObj) {
        var allValid = false;
        allValid = formInfoObj.formPanel.form.isValid();

        // form validation for weekdays tree, only when it's shown
        var scheduleWeekConfig = Ext.getCmp(formInfoObj.formPanel.id + '_schedule-config-card-panel').items.itemAt(4);

        if (scheduleWeekConfig.isVisible())
        {
          allValid = (allValid && formInfoObj.formPanel.find('name', 'weekdays')[0].validate());
        }

        if (allValid)
        {
          var isNew = formInfoObj.isNew;
          var serviceSchedule = formInfoObj.formPanel.find('name', 'schedule')[0].getValue().toLowerCase();
          var createUri = Sonatype.config.repos.urls.schedules;
          var updateUri = (formInfoObj.resourceURI) ? formInfoObj.resourceURI : '';
          var form = formInfoObj.formPanel.form;

          form.doAction('sonatypeSubmit', {
            method : (isNew) ? 'POST' : 'PUT',
            url : isNew ? createUri : updateUri,
            waitMsg : isNew ? 'Creating scheduled task...' : 'Updating scheduled task configuration...',
            fpanel : formInfoObj.formPanel,
            dataModifiers : this.submitDataModFuncs[serviceSchedule],
            serviceDataObj : Sonatype.repoServer.referenceData.schedule[serviceSchedule],
            isNew : isNew,
            scope : this,
            success : function() {
              if (this.sp.checkPermission('nexus:tasksrun', this.sp.READ))
              {
                this.runButton.enable();
              }
              else
              {
                this.runButton.disable();
              }
              this.stopButton.disable();
            }
              // extra option to send to callback, instead of conditioning on
              // method
            });
        }
      },

      cancelHandler : function(formInfoObj) {
        var formLayout = this.formCards.getLayout();
        var gridSelectModel = this.schedulesGridPanel.getSelectionModel();
        var store = this.schedulesGridPanel.getStore();

        this.formCards.remove(formInfoObj.formPanel.id, true);

        if (this.formCards.items.length > 1)
        {
          formLayout.setActiveItem(this.formCards.items.length - 1);
          // select the coordinating row in the grid, or none if back to default
          var i = store.indexOfId(formLayout.activeItem.id);
          if (i >= 0)
          {
            gridSelectModel.selectRow(i);
          }
          else
          {
            gridSelectModel.clearSelections();
          }
        }
        else
        {
          formLayout.setActiveItem(0);
          gridSelectModel.clearSelections();
        }

        // delete row from grid if cancelling a new repo form
        if (formInfoObj.isNew)
        {
          store.remove(store.getById(formInfoObj.formPanel.id));
        }
      },

      addResourceHandler : function() {
        // first disable the add button, at least until
        // save/cancel/delete/refresh
        // this.schedulesGridPanel.getTopToolbar().items.get('schedule-add-btn').disable();
        var id = 'new_schedule_' + new Date().getTime();

        var config = Ext.apply({}, this.formConfig.schedule, {
              id : id
            });
        config = this.configUniqueIdHelper(id, config);
        Ext.apply(config.items[4].items, FormFieldGenerator(id, 'Task Settings', 'serviceProperties_', this.serviceTypeDataStore, this.repositoryDataStore, this.repositoryGroupDataStore, this.repoOrGroupDataStore, this.customTypes, this.COMBO_WIDTH, this.repositoryTargetDataStore));
        var formPanel = new Ext.FormPanel(config);

        formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
        formPanel.form.on('actionfailed', this.actionFailedHandler, this);
        formPanel.on('afterlayout', this.afterLayoutFormHandler, this, {
              single : true
            });

        var serviceTypeField = formPanel.find('name', 'typeId')[0];
        serviceTypeField.on('select', this.serviceTypeSelectHandler, formPanel);

        var serviceScheduleField = formPanel.find('name', 'schedule')[0];
        serviceScheduleField.on('select', this.serviceScheduleSelectHandler, formPanel);

        var buttonInfoObj = {
          formPanel : formPanel,
          isNew : true
        };

        // save button event handler
        formPanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
        // cancel button event handler
        formPanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

        var sp = Sonatype.lib.Permissions;
        if (sp.checkPermission('nexus:tasks', sp.EDIT))
        {
          formPanel.buttons[0].disabled = false;
        }

        this.importRecurringDayHelper([], {}, formPanel);

        // add new form
        this.formCards.add(formPanel);

        // add place holder to grid
        var newRec = new this.scheduleRecordConstructor({
              name : 'New Scheduled Task',
              resourceURI : 'new'
            }, id); // use "new_schedule_" id instead of resourceURI like the
        // reader does
        this.schedulesDataStore.insert(0, [newRec]);
        this.schedulesGridPanel.getSelectionModel().selectRow(0);
      },

      afterLayoutFormHandler : function(formPanel, fLayout) {
        // register required field quicktip, but have to wait for elements to
        // show up in DOM
        var temp = function() {
          var els = Ext.select('.required-field .x-form-item-label, .required-field .x-panel-header-text', this.getEl());
          els.each(function(el, els, i) {
                Ext.QuickTips.register({
                      target : el,
                      cls : 'required-field',
                      title : '',
                      text : 'Required Field',
                      enabled : true
                    });
              });
        }.defer(300, formPanel);

      },

      deleteHandler : function() {
        if (this.ctxRecord || this.schedulesGridPanel.getSelectionModel().hasSelection())
        {
          var rec = this.ctxRecord ? this.ctxRecord : this.schedulesGridPanel.getSelectionModel().getSelected();

          if (rec.data.resourceURI == 'new')
          {
            this.cancelHandler({
                  formPanel : Ext.getCmp(rec.id),
                  isNew : true
                });
          }
          else
          {
            // @note: this handler selects the "No" button as the default
            // @todo: could extend Sonatype.MessageBox to take the button to
            // select as a param
            Sonatype.MessageBox.getDialog().on('show', function() {
                  this.focusEl = this.buttons[2]; // ack! we're offset dependent
                  // here
                  this.focus();
                }, Sonatype.MessageBox.getDialog(), {
                  single : true
                });

            Sonatype.MessageBox.show({
                  animEl : this.schedulesGridPanel.getEl(),
                  title : 'Delete Scheduled Task?',
                  msg : 'Delete the ' + rec.get('name') + ' scheduled task?',
                  buttons : Sonatype.MessageBox.YESNO,
                  scope : this,
                  icon : Sonatype.MessageBox.QUESTION,
                  fn : function(btnName) {
                    if (btnName == 'yes' || btnName == 'ok')
                    {
                      Ext.Ajax.request({
                            callback : this.deleteCallback,
                            cbPassThru : {
                              resourceId : rec.id
                            },
                            scope : this,
                            method : 'DELETE',
                            url : rec.data.resourceURI
                          });
                    }
                  }
                });
          }
        }
      },

      deleteCallback : function(options, isSuccess, response) {
        if (isSuccess)
        {
          this.reloadAll();
        }
        else
        {
          Sonatype.utils.connectionError(response, 'The server did not delete the task.', null, null, true);
        }
      },

      stopHandler : function() {
        if (this.ctxRecord || this.schedulesGridPanel.getSelectionModel().hasSelection())
        {
          var rec = this.ctxRecord ? this.ctxRecord : this.schedulesGridPanel.getSelectionModel().getSelected();

          Sonatype.MessageBox.show({
                animEl : this.schedulesGridPanel.getEl(),
                title : 'Cancel Scheduled Task?',
                msg : 'Cancel the ' + rec.get('name') + ' scheduled task?',
                buttons : Sonatype.MessageBox.YESNO,
                scope : this,
                icon : Sonatype.MessageBox.QUESTION,
                fn : function(btnName) {
                  if (btnName == 'yes' || btnName == 'ok')
                  {
                    Ext.Ajax.request({
                          callback : this.stopCallback,
                          cbPassThru : {
                            resourceId : rec.id
                          },
                          scope : this,
                          method : 'DELETE',
                          url : Sonatype.config.repos.urls.scheduleRun + '/' + rec.data.id + '?cancelOnly=true'
                        });
                  }
                }
              });
        }
      },

      stopCallback : function(options, isSuccess, response) {
        if (isSuccess)
        {
          this.reloadAll();
        }
        else
        {
          Sonatype.utils.connectionError(response, 'The server did not delete the task.', null, null, true);
        }
      },

      runHandler : function() {
        if (this.ctxRecord && this.ctxRecord.data.resourceURI != 'new')
        {
          var rec = this.ctxRecord;
          Sonatype.MessageBox.getDialog().on('show', function() {
                this.focusEl = this.buttons[2]; // ack! we're offset dependent
                // here
                this.focus();
              }, Sonatype.MessageBox.getDialog(), {
                single : true
              });

          Sonatype.MessageBox.show({
                animEl : this.schedulesGridPanel.getEl(),
                title : 'Run Scheduled Task?',
                msg : 'Run the ' + rec.get('name') + ' scheduled task?',
                buttons : Sonatype.MessageBox.YESNO,
                scope : this,
                icon : Sonatype.MessageBox.QUESTION,
                fn : function(btnName) {
                  if (btnName == 'yes' || btnName == 'ok')
                  {
                    this.alreadyDeferred = false;
                    Ext.Ajax.request({
                          callback : this.runCallback,
                          cbPassThru : {
                            resourceId : rec.id
                          },
                          scope : this,
                          method : 'GET',
                          url : Sonatype.config.repos.urls.scheduleRun + '/' + rec.data.id
                        });
                  }
                }
              });
        }
      },

      runCallback : function(options, isSuccess, response) {
        if (!isSuccess)
        {
          Sonatype.utils.connectionError(response, 'The server did not run the scheduled task.');
        }
        else if (!this.alreadyDeferred)
        {
          var receivedData = Ext.decode(response.responseText).data;
          var i = this.schedulesDataStore.indexOfId(options.cbPassThru.resourceId);
          var rec = this.schedulesDataStore.getAt(i);

          this.updateServiceRecord(rec, receivedData);

          this.schedulesDataStore.commitChanges();

          var sortState = this.schedulesDataStore.getSortState();
          this.schedulesDataStore.sort(sortState.field, sortState.direction);
          this.alreadyDeferred = true;
          this.runCallback.defer(500, this, [options, isSuccess, response]);
        }
        else
        {
          this.reloadAll();
        }
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionCompleteHandler : function(form, action) {
        // @todo: handle server error response here!!

        if (action.type == 'sonatypeSubmit')
        {
          var isNew = action.options.isNew;
          var receivedData = action.handleResponse(action.response).data;
          if (isNew)
          {
            // successful create
            var sentData = action.output.data;

            var dataObj = {
              name : receivedData.resource.name,
              enabled : receivedData.resource.enabled,
              id : receivedData.resource.id,
              resourceURI : receivedData.resourceURI,
              typeName : this.serviceTypeDataStore.getAt(this.serviceTypeDataStore.findBy(function(record, id) {
                    if (record.data.id == receivedData.resource.typeId)
                    {
                      return true;
                    }
                    return false;
                  })).data.name,
              typeId : receivedData.resource.typeId,
              readableStatus : receivedData.readableStatus,
              alertEmail : receivedData.resource.alertEmail,
              schedule : receivedData.resource.schedule,
              nextRunTimeInMillis : receivedData.nextRunTimeInMillis?new Date(receivedData.nextRunTimeInMillis):'n/a',
              lastRunTimeInMillis : receivedData.lastRunTimeInMillis?new Date(receivedData.lastRunTimeInMillis):'n/a',
              lastRunResult : receivedData.lastRunResult
            };

            var newRec = new this.scheduleRecordConstructor(dataObj, action.options.fpanel.id);

            this.schedulesDataStore.remove(this.schedulesDataStore.getById(action.options.fpanel.id)); // remove
            // old
            // one
            this.schedulesDataStore.addSorted(newRec);
            this.schedulesGridPanel.getSelectionModel().selectRecords([newRec], false);

            // set the hidden id field in the form for subsequent updates
            action.options.fpanel.find('name', 'id')[0].setValue(receivedData.resourceURI);
            // remove button click listeners
            action.options.fpanel.buttons[0].purgeListeners();
            action.options.fpanel.buttons[1].purgeListeners();

            var buttonInfoObj = {
              formPanel : action.options.fpanel,
              isNew : false,
              resourceURI : dataObj.resourceURI
            };

            if (dataObj.schedule == 'once')
            {
              action.options.fpanel.buttons[0].disable();
            }
            else
            {
              // save button event handler
              action.options.fpanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
            }

            // cancel button event handler
            action.options.fpanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

            // disable the service type, only avaiable on add
            action.options.fpanel.find('name', 'typeId')[0].disable();

            // //Enable add button on save complete
            // this.schedulesGridPanel.getTopToolbar().items.get('schedule-add-btn').enable();
          }
          else
          {
            var sentData = action.output.data;

            var i = this.schedulesDataStore.indexOfId(action.options.fpanel.id);
            var rec = this.schedulesDataStore.getAt(i);

            this.updateServiceRecord(rec, receivedData);

            var sortState = this.schedulesDataStore.getSortState();
            this.schedulesDataStore.sort(sortState.field, sortState.direction);
          }
          
        // NEXUS-4365: after a save action, panel state is broken: fields are enabled but empty, combo box values are missing.
        // -> rebuild panel after save action
        var formLayout = this.formCards.getLayout();
        var fp = action.options.fpanel;
        this.formCards.remove(fp.id, true);

        // switch to empty
        formLayout.setActiveItem(0);
        
        var store = this.schedulesGridPanel.getStore();
        this.schedulesGridPanel.getSelectionModel().selectRow(store.indexOfId(fp.id));
        // END NEXUS-4365
    }
    else if ( action.type == 'sonatypeLoad' )
    {
        // NEXUS-4363 disable the south panel after service-type-config-card values are loaded
        var formPanel = action.options.fpanel;
        var readableStatus = formPanel.readableStatus;
        if (!readableStatus || readableStatus == 'Waiting' || readableStatus == '')
        {
        //this.disableEditingHeader.setVisible(false);
        // only layout if change is needed
        if ( formPanel.disabled )
        {
            formPanel.enable();
            formPanel.doLayout();
        }
        }
        else if (!formPanel.disabled)
        {
        //this.disableEditingHeader.setVisible(true);
        formPanel.disable();
        formPanel.doLayout();
        }
        }
      },

      updateServiceRecord : function(rec, receivedData) {
        rec.beginEdit();
        rec.set('name', receivedData.resource.name);
        rec.set('enabled', receivedData.resource.enabled);
        rec.set('typeId', receivedData.resource.typeId);
        rec.set('typeName', this.serviceTypeDataStore.getAt(this.serviceTypeDataStore.findBy(function(record, id) {
                  if (record.data.id == receivedData.resource.typeId)
                  {
                    return true;
                  }
                  return false;
                })).data.name);
        rec.set('alertEmail', receivedData.resource.alertEmail);
        rec.set('schedule', receivedData.resource.schedule);
        rec.set('readableStatus', receivedData.readableStatus);
        rec.set('nextRunTimeInMillis', receivedData.nextRunTimeInMillis?new Date(receivedData.nextRunTimeInMillis):'n/a');
        rec.set('lastRunTimeInMillis', receivedData.lastRunTimeInMillis?new Date(receivedData.lastRunTimeInMillis):'n/a');
        rec.set('lastRunResult', receivedData.lastRunResult);
        rec.commit();
        rec.endEdit();
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionFailedHandler : function(form, action) {
        if (action.failureType == Ext.form.Action.CLIENT_INVALID)
        {
          Sonatype.MessageBox.alert('Missing or Invalid Fields', 'Please change the missing or invalid fields.').setIcon(Sonatype.MessageBox.WARNING);
        }
        // @note: server validation error are now handled just like client
        // validation errors by marking the field invalid
        // else if(action.failureType == Ext.form.Action.SERVER_INVALID){
        // Sonatype.MessageBox.alert('Invalid Fields', 'The server identified
        // invalid fields.').setIcon(Sonatype.MessageBox.ERROR);
        // }
        else if (action.failureType == Ext.form.Action.CONNECT_FAILURE)
        {
          if (action.response.responseText.indexOf("There is no task with ID=") > -1 )
          {
            Sonatype.MessageBox.alert('Selected task was removed', 'The selected task was removed by another process and cannot be displayed.');
            this.reloadAll();
          }
          else
          {
            Sonatype.utils.connectionError(action.response, 'There is an error communicating with the server.');
          }
        }
        else if (action.failureType == Ext.form.Action.LOAD_FAILURE)
        {
          Sonatype.MessageBox.alert('Load Failure', 'The data failed to load from the server.').setIcon(Sonatype.MessageBox.ERROR);
        }

        // @todo: need global alert mechanism for fatal errors.
      },

      formDataLoader : function(formPanel, resourceURI, modFuncs) {
        formPanel.getForm().doAction('sonatypeLoad', {
              url : resourceURI,
              method : 'GET',
              fpanel : formPanel,
              dataModifiers : modFuncs,
              scope : this
            });

      },

      rowSelect : function(selectionModel, index, rec) {
        this.ctxRow = this.schedulesGridPanel.view.getRow(index);
        this.ctxRecord = this.schedulesGridPanel.store.getAt(index);

        this.runButton.disable();
        this.stopButton.disable();
        if (this.sp.checkPermission('nexus:tasks', this.sp.DELETE))
        {
            this.deleteButton.enable();
        }
        else
        {
            this.deleteButton.disable();
        }

        var readableStatus = rec.data.readableStatus;

        if (rec.data.name.substring(0, 4) == 'New ')
        {
          this.runButton.disable();
          this.stopButton.disable();
        }
        else if (readableStatus == 'Cancelling')
        {
          this.stopButton.disable();
          this.runButton.disable();
          this.deleteButton.disable();
        }
        else if (readableStatus == 'Waiting')
        {
          if (this.sp.checkPermission('nexus:tasksrun', this.sp.READ))
          {
            this.runButton.enable();
          }
          else
          {
            this.runButton.disable();
          }
          this.stopButton.disable();
        }
        else
        {
          if (this.sp.checkPermission('nexus:tasksrun', this.sp.READ))
          {
            this.stopButton.enable();
          }
          else
          {
            this.stopButton.disable();
          }
          this.runButton.disable();
        }

        var id = rec.id; // note: rec.id is unique for new resources and equal
        // to resourceURI for existing ones
        var formPanel = this.formCards.findById(id);
        var schedulePanel = null;

        // assumption: new route forms already exist in formCards, so they won't
        // get into this case
        if (!formPanel)
        { // create form and populate current data
          var config = Ext.apply({}, this.formConfig.schedule, {
                id : id
              });
          config = this.configUniqueIdHelper(id, config);
          Ext.apply(config.items[4].items, FormFieldGenerator(id, 'Task Settings', 'serviceProperties_', this.serviceTypeDataStore, this.repositoryDataStore, this.repositoryGroupDataStore, this.repoOrGroupDataStore, this.customTypes, this.COMBO_WIDTH, this.repositoryTargetDataStore));
          formPanel = new Ext.FormPanel(config);

          formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
          formPanel.form.on('actionfailed', this.actionFailedHandler, this);
          formPanel.on('afterlayout', this.afterLayoutFormHandler, this, {
                single : true
              });

          // On load need to make sure and set the proper schedule type card as
          // active
          schedulePanel = formPanel.findById(formPanel.id + '_schedule-config-card-panel');
          if (rec.data.schedule == 'once')
          {
            schedulePanel.activeItem = 1;
          }
          else if (rec.data.schedule == 'hourly')
          {
            schedulePanel.activeItem = 2;
          }
          else if (rec.data.schedule == 'daily')
          {
            schedulePanel.activeItem = 3;
          }
          else if (rec.data.schedule == 'weekly')
          {
            schedulePanel.activeItem = 4;
          }
          else if (rec.data.schedule == 'monthly')
          {
            schedulePanel.activeItem = 5;
          }
          else if (rec.data.schedule == 'advanced')
          {
            schedulePanel.activeItem = 6;
          }
          // Then enable each field in the active card (after this point, select
          // handler takes care of all
          // enable/disable transitions
          if (schedulePanel.items.items[schedulePanel.activeItem].items)
          {
            schedulePanel.items.items[schedulePanel.activeItem].items.each(function(item) {
                  item.disabled = false;
                });
          }

          // Need to do the same w/ service type and make sure the correct card
          // is active. Dynamic cards, so little more generic
          var serviceTypePanel = formPanel.findById(formPanel.id + '_service-type-config-card-panel');
          serviceTypePanel.hide();
          serviceTypePanel.items.each(function(item, i, len) {
                if (item.id === id + '_' + rec.data.typeId)
                {
                  serviceTypePanel.activeItem = i;
                  if (item.items && item.items.length !== 0)
                  {
                    serviceTypePanel.show();
                    item.items.each(function(item) {
                          item.disabled = false;
                        });
                  }
                }
              });

          formPanel.find('name', 'typeId')[0].disable();
          formPanel.find('name', 'schedule')[0].on('select', this.serviceScheduleSelectHandler, formPanel);

          var buttonInfoObj = {
            formPanel : formPanel,
            isNew : false, // not a new route form, see assumption
            resourceURI : rec.data.resourceURI
          };

          formPanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
          formPanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

          var sp = Sonatype.lib.Permissions;
          if (sp.checkPermission('nexus:tasks', sp.EDIT) && rec.data.schedule != 'internal')
          {
            formPanel.buttons[0].disabled = false;
          }

          this.importRecurringDayHelper([], {}, formPanel);

          this.formDataLoader(formPanel, rec.data.resourceURI, this.loadDataModFuncs[rec.data.schedule]);

          this.formCards.add(formPanel);
        }
        // save readable status
        formPanel.readableStatus = readableStatus;

        // always set active
        this.formCards.getLayout().setActiveItem(formPanel);
        formPanel.doLayout();
      },

      serviceTypeSelectHandler : function(combo, record, index) {
        var serviceTypePanel = this.findById(this.id + '_service-type-config-card-panel');
        // First disable all the items currently on screen, so they wont be
        // validated/submitted etc
        serviceTypePanel.getLayout().activeItem.items.each(function(item) {
              item.disable();
            });
        // Then find the proper card to activate (based upon id of the
        // serviceType)
        // Then enable the fields in that card
        var formId = this.id;
        serviceTypePanel.items.each(function(item, i, len) {
              if (item.id === formId + '_' + record.data.id)
              {
                serviceTypePanel.getLayout().setActiveItem(item);
                if (item.items.length === 0) {
                  serviceTypePanel.hide();
                } else  {
                  serviceTypePanel.show();
                  item.items.each(function(item) {
                    item.enable();
                  });
                }
              }
            }, serviceTypePanel);

        serviceTypePanel.doLayout();
      },

      serviceScheduleSelectHandler : function(combo, record, index) {
        var schedulePanel = this.findById(this.id + '_schedule-config-card-panel');
        // First disable all the items currently on screen, so they wont be
        // validated/submitted etc
        schedulePanel.getLayout().activeItem.items.each(function(item) {
              item.disable();
            });
        // Then find the proper card to activate (based upon the selected
        // schedule type)
        if (record.data.value == 'Once')
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(1));
        }
        else if (record.data.value == 'Hourly')
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(2));
        }
        else if (record.data.value == 'Daily')
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(3));
        }
        else if (record.data.value == 'Weekly')
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(4));
        }
        else if (record.data.value == 'Monthly')
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(5));
        }
        else if (record.data.value == 'Advanced')
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(6));
        }
        else
        {
          schedulePanel.getLayout().setActiveItem(schedulePanel.items.itemAt(0));
        }
        // Then enable the fields in that card
        schedulePanel.getLayout().activeItem.items.each(function(item) {
              item.enable();
              if (item.xtype == 'datefield')
              {
                item.setValue(item.value);
              }
              else if (item.xtype == 'timefield')
              {
                if (item.value == null)
                {
                  item.setValue(new Date().format('H:i'));
                }
                else
                {
                  item.setValue(item.value);
                }
              }
            });
        schedulePanel.doLayout();
      },

      // creates a unique config object with specific IDs on the two grid item
      configUniqueIdHelper : function(id, config) {
        // @note: there has to be a better way to do this. Depending on offsets
        // is very error prone
        var newConfig = config;

        this.assignItemIds(id, newConfig.items);

        return newConfig;
      },

      assignItemIds : function(id, items) {
        for (var i = 0; i < items.length; i++)
        {
          var item = items[i];
          if (item.id)
          {
            if (!item.originalId)
            {
              item.originalId = item.id;
            }
            item.id = id + '_' + item.originalId;
          }
          if (item.items)
          {
            this.assignItemIds(id, item.items);
          }
        }
      },

      exportStartDateHelper : function(val, fpanel) {
        // as there is no Date transfer in json, i pass the long representation
        // of the date.
        var selectedStartDate = "";

        var startDateFields = fpanel.find('name', 'startDate');
        // Need to find the startDate that is currently enabled, as multiple
        // cards will have a startDate field
        for (var i = 0; i < startDateFields.length; i++)
        {
          if (!startDateFields[i].disabled)
          {
            selectedStartDate = startDateFields[i];
            break;
          }
        }

        return '' + selectedStartDate.getValue().getTime();
      },
      exportStartTimeHelper : function(val, fpanel) {
        var selectedStartTime = "";

        var startTimeFields = fpanel.find('name', 'startTime');
        // Need to find the startTime that is currently enabled, as multiple
        // cards will have a startDate field
        for (var i = 0; i < startTimeFields.length; i++)
        {
          if (!startTimeFields[i].disabled)
          {
            selectedStartTime = startTimeFields[i];
            break;
          }
        }

        // rest api is using 24 hour clock
        var hours = selectedStartTime.getValue().substring(0, selectedStartTime.getValue().indexOf(':'));
        var minutes = selectedStartTime.getValue().substring(selectedStartTime.getValue().indexOf(':') + 1, selectedStartTime.getValue().indexOf(':') + 3);

        return hours + ':' + minutes;
      },
      exportRecurringTimeHelper : function(val, fpanel) {
        var selectedRecurringTime = "";

        var recurringTimeFields = fpanel.find('name', 'recurringTime');
        // Need to find the recurringTime that is currently enabled, as multiple
        // cards will have a startDate field
        for (var i = 0; i < recurringTimeFields.length; i++)
        {
          if (!recurringTimeFields[i].disabled)
          {
            selectedRecurringTime = recurringTimeFields[i];
            break;
          }
        }

        // rest api is using 24 hour clock
        var hours = parseInt(selectedRecurringTime.getValue().substring(0, selectedRecurringTime.getValue().indexOf(':')), 10);
        var minutes = selectedRecurringTime.getValue().substring(selectedRecurringTime.getValue().indexOf(':') + 1, selectedRecurringTime.getValue().indexOf(':') + 3);

        return hours + ':' + minutes;
      },
      exportRecurringDayHelper : function(val, fpanel) {
        var weekdaysBox = fpanel.find('name', 'weekdays')[0];
        var weekdaysIds = weekdaysBox.getValue();

        var response = [];
        for (var i = 0; i < weekdaysIds.length; i++)
        {
          response[i] = Strings.lowercase(weekdaysIds[i]);
        }

        return response;
      },
      exportMonthlyRecurringDayHelper : function(val, fpanel) {
        var outputArr = [];
        var j = 0;
        // Another pretty simple conversion, just find all selected check boxes
        // and send along the day number
        for (var i = 1; i <= 31; i++)
        {
          if (fpanel.find('name', 'day' + i)[0].getValue())
          {
            outputArr[j++] = '' + i;
          }
        }
        // and last if necessary
        if (fpanel.find('name', 'dayLast')[0].getValue())
        {
          outputArr[j] = 'last';
        }
        return outputArr;
      },
      exportServicePropertiesHelper : function(val, fpanel) {
        return FormFieldExporter(fpanel, '_service-type-config-card-panel', 'serviceProperties_', this.customTypes);
      },
      importStartDateHelper : function(val, srcObj, fpanel) {
        var selectedStartDate = "";

        var startDateFields = fpanel.find('name', 'startDate');
        // Find the correct startDate field, as their will be multiples, all but
        // 1 disabled
        for (var i = 0; i < startDateFields.length; i++)
        {
          if (!startDateFields[i].disabled)
          {
            selectedStartDate = startDateFields[i];
            break;
          }
        }

        // translate the long representation back into date
        var importedDate = new Date(Number(val));
        selectedStartDate.setValue(importedDate);
        return importedDate;
      },
      importStartTimeHelper : function(val, srcObj, fpanel) {
        var selectedStartTime = "";

        var startTimeFields = fpanel.find('name', 'startTime');
        // Find the correct startTime field, as their will be multiples, all but
        // 1 disabled
        for (var i = 0; i < startTimeFields.length; i++)
        {
          if (!startTimeFields[i].disabled)
          {
            selectedStartTime = startTimeFields[i];
            break;
          }
        }

        var startDate = new Date(Number(srcObj.startDate));
        var hours = startDate.getHours();
        var minutes = startDate.getMinutes();

        var importedTime = (hours<10?'0':'') + hours + ':' + (minutes<10?'0':'') + minutes;
        selectedStartTime.setValue(importedTime);
        return importedTime;
      },
      importRecurringTimeHelper : function(val, srcObj, fpanel) {
        var selectedRecurringTime = "";

        var recurringTimeFields = fpanel.find('name', 'recurringTime');
        // Find the correct recurringTime field, as their will be multiples, all
        // but 1 disabled
        for (var i = 0; i < recurringTimeFields.length; i++)
        {
          if (!recurringTimeFields[i].disabled)
          {
            selectedRecurringTime = recurringTimeFields[i];
            break;
          }
        }

        var startDate = new Date(Number(srcObj.startDate));
        var hours = startDate.getHours();
        var minutes = startDate.getMinutes();

        var importedTime = (hours<10?'0':'') + hours + ':' + (minutes<10?'0':'') + minutes;
        selectedRecurringTime.setValue(importedTime);
        return importedTime;
      },
      importRecurringDayHelper : function(arr, srcObj, fpanel) {
        var weekdayBox = fpanel.find('name', 'weekdays')[0];
        weekdayBox.setValue(arr);

        return arr; // return arr, even if empty to comply with sonatypeLoad
        // data modifier requirement
      },
      importMonthlyRecurringDayHelper : function(arr, srcObj, fpanel) {
        // simply look at each item, and select the proper checkbox
        for (var i = 0; i < arr.length; i++)
        {
          var checkbox = fpanel.find('name', 'day' + arr[i])[0];
          if (checkbox == null)
          {
            checkbox = fpanel.find('name', 'dayLast')[0];
          }
          checkbox.setValue(true);
        }

        return arr;
      },
      importServicePropertiesHelper : function(val, srcObj, fpanel) {
        FormFieldImporter(srcObj, fpanel, 'serviceProperties_', this.customTypes);
        return val;
      }
    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Repository Routes Edit/Creat panel layout and controller
 */

define('repoServer/RoutesEditPanel',['Sonatype/all', 'Sonatype/strings','Nexus/ext/GridFilterBox'], function(Sonatype, Strings){
Sonatype.repoServer.RoutesEditPanel = function(config) {
  var config = config || {},
      defaultConfig = {};

  Ext.apply(this, config, defaultConfig);

  this.repoDataStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        url : Sonatype.config.repos.urls.allRepositories,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        fields : [{
              name : 'id'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }],
        autoLoad : true
      });

  var tfStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['True'], ['False']]
      });
  this.ruleTypeStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['Blocking'], ['Exclusive'], ['Inclusive']]
      });
  this.BLOCKING_TYPE_INDEX = 0;
  this.BLOCKING = 'Blocking';

  // A record to hold the name and id of a repository group
  this.repositoryGroupRecordConstructor = Ext.data.Record.create([{
        name : 'id'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }]);

  // Reader and datastore that queries the server for the list of repository
  // groups
  this.repositoryGroupReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'id'
      }, this.repositoryGroupRecordConstructor);
  this.repositoryGroupDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.groups,
        reader : this.repositoryGroupReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true,
        listeners : {
          'load' : {
            fn : function() {
              var allRec = new this.repositoryGroupRecordConstructor({
                    id : '*',
                    name : 'All Repository Groups'
                  }, '*');
              this.repositoryGroupDataStore.insert(0, allRec);
            },
            scope : this
          }
        }
      });

  var ht = Sonatype.repoServer.resources.help.routes;

  this.loadDataModFuncs = {
    route : {
      ruleType : Strings.capitalize,
      repositories : this.loadRepoListHelper.createDelegate(this)
    }
  };

  this.submitDataModFuncs = {
    route : {
      ruleType : Strings.lowercase,
      repositories : this.exportRepoListHelper.createDelegate(this)
    }
  };

  this.formConfig = {};
  this.formConfig.route = {
    region : 'center',
    width : '100%',
    height : '100%',
    autoScroll : true,
    border : false,
    frame : true,
    collapsible : false,
    collapsed : false,
    labelWidth : 175,
    layoutConfig : {
      labelSeparator : ''
    },

    items : [{
          xtype : 'hidden',
          name : 'id'
        }, {
          xtype : 'textfield',
          fieldLabel : 'URL Pattern',
          itemCls : 'required-field',
          helpText : ht.pattern,
          name : 'pattern',
          width : 300,
          allowBlank : false
        }, {
          xtype : 'combo',
          fieldLabel : 'Rule Type',
          itemCls : 'required-field',
          helpText : ht.ruleType,
          name : 'ruleType',
          // hiddenName: 'connectionTimeout',
          width : 100,
          store : this.ruleTypeStore,
          displayField : 'value',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false,
          listeners : {
            'select' : {
              fn : function(combo, record, index) {
                this.updateTreePanel(index == this.BLOCKING_TYPE_INDEX, combo.findParentByType('form').id);
              },
              scope : this
            }
          }
        }, {
          xtype : 'combo',
          fieldLabel : 'Repository Group',
          itemCls : 'required-field',
          helpText : ht.group,
          name : 'groupId',
          width : 200,
          minWidth : 200,
          store : this.repositoryGroupDataStore,
          displayField : 'name',
          valueField : 'id',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false
        }, {
          xtype : 'twinpanelchooser',
          titleLeft : 'Ordered Route Repositories',
          titleRight : 'Available Repositories',
          name : 'repositories',
          valueField : 'id',
          store : this.repoDataStore,
          required : true
        }],
    buttons : [{
          text : 'Save',
          disabled : true
        }, {
          text : 'Cancel'
        }]
  };

  this.nameConcatinator = function(val, parent) {
    return Sonatype.utils.joinArrayObject(val, 'name');
  };

  this.nameFinder = function(val, parent) {
    var repos = [];
    for (var i = 0; i < val.length; i++)
    {
      repos[i] = this.repoDataStore.getById(val[i].id).data;
    }

    return Sonatype.utils.joinArrayObject(repos, 'name');
  };

  // START: Repo list ******************************************************
  this.routeRecordConstructor = Ext.data.Record.create([{
        name : 'resourceURI'
      }, {
        name : 'pattern',
        sortType : Ext.data.SortTypes.asUCString
      }, {
        name : 'ruleType'
      }, {
        name : 'groupId'
      }, {
        name : 'repositories'
      }, {
        name : 'sRepositories',
        mapping : 'repositories',
        convert : this.nameConcatinator
      }]);

  this.routesReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'resourceURI'
      }, this.routeRecordConstructor);

  // @ext: must use data.Store (not JsonStore) to pass in reader instead of
  // using fields config array
  this.routesDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.routes,
        reader : this.routesReader,
        sortInfo : {
          field : 'pattern',
          direction : 'ASC'
        },
        autoLoad : true
      });

  this.sp = Sonatype.lib.Permissions;

  this.routesGridPanel = new Ext.grid.GridPanel({
        id : 'st-routes-grid',
        region : 'north',
        layout : 'fit',
        split : true,
        height : 200,
        minHeight : 150,
        maxHeight : 400,
        frame : false,
        autoScroll : true,
        tbar : [{
              id : 'route-refresh-btn',
              text : 'Refresh',
              iconCls : 'st-icon-refresh',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.reloadAll
            }, {
              id : 'route-add-btn',
              text : 'Add',
              icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.addResourceHandler,
              disabled : !this.sp.checkPermission('nexus:routes', this.sp.CREATE)
            }, {
              id : 'route-delete-btn',
              text : 'Delete',
              icon : Sonatype.config.resourcePath + '/static/images/icons/delete.png',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.deleteResourceHandler,
              disabled : !this.sp.checkPermission('nexus:routes', this.sp.DELETE)
            }
        ],

        // grid view options
        ds : this.routesDataStore,
        sortInfo : {
          field : 'name',
          direction : "ASC"
        },
        loadMask : true,
        deferredRender : false,
        columns : [{
              header : 'Route',
              dataIndex : 'pattern',
              width : 200
            }, {
              header : 'Rule Type',
              dataIndex : 'ruleType',
              width : 100
            }, {
              header : 'Group',
              dataIndex : 'groupId',
              width : 100
            }, {
              header : 'Repositories',
              dataIndex : 'sRepositories',
              id : 'routes-config-repos-col',
              width : 300
            }],
        autoExpandColumn : 'routes-config-repos-col',
        disableSelection : false,
        viewConfig : {
          deferEmptyText: false,
          emptyText: 'No routes defined',
          emptyTextWhileFiltering: 'No routes matched criteria: {criteria}'
        }
      });
  this.routesGridPanel.getSelectionModel().on('rowselect', this.rowSelect, this);

  this.routesGridPanel.getTopToolbar().add([
    '->',
    NX.create('Nexus.ext.GridFilterBox', {
      filteredGrid: this.routesGridPanel
    })
  ]);

  // END: Repo List ******************************************************
  // *********************************************************************

  Sonatype.repoServer.RoutesEditPanel.superclass.constructor.call(this, {
        layout : 'border',
        autoScroll : false,
        width : '100%',
        height : '100%',
        items : [this.routesGridPanel, {
              xtype : 'panel',
              id : 'route-config-forms',
              title : 'Repository Route Configuration',
              layout : 'card',
              region : 'center',
              activeItem : 0,
              deferredRender : false,
              autoScroll : false,
              frame : false,
              items : [{
                    xtype : 'panel',
                    layout : 'fit',
                    id : 'route-no-form',
                    html : '<div class="little-padding">Select a Repository Route to edit it, or click "Add" to create a new one.</div>'
                  }]
            }]
      });

  this.formCards = this.findById('route-config-forms');
};

Ext.extend(Sonatype.repoServer.RoutesEditPanel, Ext.Panel, {
      reloadAll : function() {
        this.repoDataStore.removeAll();
        this.repoDataStore.reload();
        this.routesDataStore.removeAll();
        this.routesDataStore.reload();
        this.formCards.items.each(function(item, i, len) {
              if (i > 0)
              {
                this.remove(item, true);
              }
            }, this.formCards);

        this.formCards.getLayout().setActiveItem(0);
      },

      // formInfoObj : {formPanel, isNew, [resourceURI]}
      saveHandler : function(formInfoObj) {
        var allValid = false;
        allValid = formInfoObj.formPanel.form.isValid() && formInfoObj.formPanel.find('name', 'repositories')[0].validate();

        if (allValid)
        {
          var isNew = formInfoObj.isNew;
          var uri = (isNew) ? Sonatype.config.repos.urls.routes : formInfoObj.resourceURI;
          var form = formInfoObj.formPanel.form;

          form.doAction('sonatypeSubmit', {
            method : isNew ? 'POST' : 'PUT',
            url : uri,
            waitMsg : isNew ? 'Creating repository route...' : 'Updating repository route configuration...',
            fpanel : formInfoObj.formPanel,
            dataModifiers : this.submitDataModFuncs.route,
            serviceDataObj : Sonatype.repoServer.referenceData.route,
            isNew : isNew
              // extra option to send to callback, instead of conditioning on
              // HTTP method
            });
        }
      },

      // formInfoObj : {formPanel, isNew, [resourceURI]}
      cancelHandler : function(formInfoObj) {
        var formLayout = this.formCards.getLayout();
        var gridSelectModel = this.routesGridPanel.getSelectionModel();
        var store = this.routesGridPanel.getStore();

        this.formCards.remove(formInfoObj.formPanel.id, true);
        // select previously selected form, or the default view (index == 0)
        var newIndex = this.formCards.items.length - 1;
        newIndex = (newIndex >= 0) ? newIndex : 0;
        formLayout.setActiveItem(newIndex);

        // delete row from grid if canceling a new repo form
        if (formInfoObj.isNew)
        {
          store.remove(store.getById(formInfoObj.formPanel.id));
        }

        // select the coordinating row in the grid, or none if back to default
        var i = store.indexOfId(formLayout.activeItem.id);
        if (i >= 0)
        {
          gridSelectModel.selectRow(i);
        }
        else
        {
          gridSelectModel.clearSelections();
        }
      },

      addResourceHandler : function() {
        var id = 'new_route_' + new Date().getTime();

        var config = Ext.apply({}, this.formConfig.route, {
              id : id
            });
        var formPanel = new Ext.FormPanel(config);

        formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
        formPanel.form.on('actionfailed', this.actionFailedHandler, this);
        formPanel.on('beforerender', this.beforeFormRenderHandler, this);
        formPanel.on('afterlayout', this.afterLayoutFormHandler, this, {
              single : true
            });

        var buttonInfoObj = {
          formPanel : formPanel,
          isNew : true
        };

        // save button event handler
        formPanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
        // cancel button event handler
        formPanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

        // add place holder to grid
        var newRec = new this.routeRecordConstructor({
              pattern : 'New Route',
              ruleType : '-',
              resourceURI : 'new',
              repositories : []
            }, id); // use "new_route_" id instead of resourceURI like the
                    // reader does
        this.routesDataStore.insert(0, [newRec]);

        // add new form
        this.formCards.add(formPanel);

        // always set active and re-layout
        this.formCards.getLayout().setActiveItem(formPanel);
        formPanel.doLayout();

        this.routesGridPanel.getSelectionModel().selectRow(0);
      },

      afterLayoutFormHandler : function(formPanel, fLayout) {
        // register required field quicktip, but have to wait for elements to
        // show up in DOM
        var temp = function() {
          var els = Ext.select('.required-field .x-form-item-label, .required-field .x-panel-header-text', this.getEl());
          els.each(function(el, els, i) {
                Ext.QuickTips.register({
                      target : el,
                      cls : 'required-field',
                      title : '',
                      text : 'Required Field',
                      enabled : true
                    });
              });
        }.defer(300, formPanel);

      },

      deleteResourceHandler : function() {
        if (this.routesGridPanel.getSelectionModel().hasSelection())
        {
          var selections = this.routesGridPanel.getSelectionModel().getSelections();

          if (selections.length == 1)
          {
            var rec = this.routesGridPanel.getSelectionModel().getSelected();

            if (rec.data.resourceURI == 'new')
            {
              this.cancelHandler({
                    formPanel : Ext.getCmp(rec.id),
                    isNew : true
                  });
            }
            else
            {
              Sonatype.utils.defaultToNo();
              Sonatype.MessageBox.show({
                    animEl : this.routesGridPanel.getEl(),
                    title : 'Delete Route',
                    msg : 'Delete the ' + rec.get('pattern') + ' route?',
                    buttons : Sonatype.MessageBox.YESNO,
                    scope : this,
                    icon : Sonatype.MessageBox.QUESTION,
                    fn : function(btnName) {
                      if (btnName == 'yes' || btnName == 'ok')
                      {
                        Ext.Ajax.request({
                              callback : this.deleteCallback,
                              cbPassThru : {
                                resourceId : rec.id
                              },
                              scope : this,
                              method : 'DELETE',
                              url : rec.data.resourceURI
                            });
                      }
                    }
                  });
            }
          }
          else
          {
            Sonatype.utils.defaultToNo();
            Sonatype.MessageBox.show({
                  animEl : this.routesGridPanel.getEl(),
                  title : 'Delete Routes',
                  msg : 'Delete ' + selections.length + ' routes?',
                  buttons : Sonatype.MessageBox.YESNO,
                  scope : this,
                  icon : Sonatype.MessageBox.QUESTION,
                  fn : function(btnName) {
                    Ext.each(selections, function(rec) {
                          if (rec.data.resourceURI == 'new')
                          {
                            this.cancelHandler({
                                  formPanel : Ext.getCmp(rec.id),
                                  isNew : true
                                });
                          }
                          else
                          {
                            Ext.Ajax.request({
                                  callback : this.deleteCallback,
                                  cbPassThru : {
                                    resourceId : rec.id
                                  },
                                  scope : this,
                                  method : 'DELETE',
                                  url : rec.data.resourceURI
                                });
                          }
                        }, this);
                  }
                });
          }
        }
      },

      deleteCallback : function(options, isSuccess, response) {
        if (isSuccess)
        {
          var resourceId = options.cbPassThru.resourceId;
          var formLayout = this.formCards.getLayout();
          var gridSelectModel = this.routesGridPanel.getSelectionModel();
          var store = this.routesGridPanel.getStore();

          if (formLayout.activeItem.id == resourceId)
          {
            this.formCards.remove(resourceId, true);
            // select previously selected form, or the default view (index == 0)
            var newIndex = this.formCards.items.length - 1;
            newIndex = (newIndex >= 0) ? newIndex : 0;
            formLayout.setActiveItem(newIndex);
          }
          else
          {
            this.formCards.remove(resourceId, true);
          }

          store.remove(store.getById(resourceId));

          // select the coordinating row in the grid, or none if back to default
          var i = store.indexOfId(formLayout.activeItem.id);
          if (i >= 0)
          {
            gridSelectModel.selectRow(i);
          }
          else
          {
            gridSelectModel.clearSelections();
          }
        }
        else
        {
          Sonatype.MessageBox.alert('The server did not delete the route.');
        }
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionCompleteHandler : function(form, action) {
        // @todo: handle server error response here!!

        if (action.type == 'sonatypeSubmit')
        {
          var isNew = action.options.isNew;

          if (isNew)
          {
            // successful create
            var respData = (Ext.decode(action.response.responseText)).data;
            // route state data doesn't have resourceURI in it like the list
            // data
            respData.resourceURI = action.getUrl() + '/' + respData.id; // add
                                                                        // this
                                                                        // to
                                                                        // match
                                                                        // the
                                                                        // list
                                                                        // data
                                                                        // field
                                                                        // to
                                                                        // create
                                                                        // the
                                                                        // record
            // @ext: this application of the convert function is needed because
            // Ext only
            // applies the convert functions in
            // Ext.data.XXXReader.readRecords(),
            // not in the constructor yielded from Ext.data.Record.create()
            respData.sRepositories = this.nameFinder(respData.repositories);

            var newRec = new this.routeRecordConstructor(respData, action.options.fpanel.id); // form
                                                                                              // and
                                                                                              // grid
                                                                                              // data
                                                                                              // id
                                                                                              // match,
                                                                                              // keep
                                                                                              // the
                                                                                              // new
                                                                                              // id

            this.routesDataStore.remove(this.routesDataStore.getById(action.options.fpanel.id)); // remove
                                                                                                  // old
                                                                                                  // one
            this.routesDataStore.addSorted(newRec);
            this.routesGridPanel.getSelectionModel().selectRecords([newRec], false);

            // set the hidden id field in the form for subsequent updates
            action.options.fpanel.find('name', 'id')[0].setValue(respData.id);
            // remove button click listeners
            action.options.fpanel.buttons[0].purgeListeners();
            action.options.fpanel.buttons[1].purgeListeners();

            var buttonInfoObj = {
              formPanel : action.options.fpanel,
              isNew : false,
              resourceURI : respData.resourceURI
            };

            // save button event handler
            action.options.fpanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
            // cancel button event handler
            action.options.fpanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));
          }
          else
          {
            var sentData = action.output.data;

            var i = this.routesDataStore.indexOfId(action.options.fpanel.id);
            var rec = this.routesDataStore.getAt(i);

            rec.beginEdit();
            rec.set('pattern', sentData.pattern);
            rec.set('ruleType', sentData.ruleType);
            rec.set('groupId', sentData.groupId);
            rec.set('repositories', sentData.repositories);
            rec.set('sRepositories', this.nameFinder(sentData.repositories));
            rec.commit();
            rec.endEdit();

            var sortState = this.routesDataStore.getSortState();
            this.routesDataStore.sort(sortState.field, sortState.direction);
          }
        }
        else if (action.type == 'sonatypeLoad')
        {
          var value = action.options.fpanel.find('name', 'ruleType')[0].getValue();
          this.updateTreePanel(value == this.BLOCKING, action.options.fpanel.id);
        }
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionFailedHandler : function(form, action) {
        if (action.failureType == Ext.form.Action.CLIENT_INVALID)
        {
          Sonatype.MessageBox.alert('Missing or Invalid Fields', 'Please change the missing or invalid fields.').setIcon(Sonatype.MessageBox.WARNING);
        }
        // @note: server validation error are now handled just like client
        // validation errors by marking the field invalid
        // else if(action.failureType == Ext.form.Action.SERVER_INVALID){
        // Sonatype.MessageBox.alert('Invalid Fields', 'The server identified
        // invalid fields.').setIcon(Sonatype.MessageBox.ERROR);
        // }
        else if (action.failureType == Ext.form.Action.CONNECT_FAILURE)
        {
          Sonatype.utils.connectionError(action.response, 'There is an error communicating with the server.')
        }
        else if (action.failureType == Ext.form.Action.LOAD_FAILURE)
        {
          Sonatype.MessageBox.alert('Load Failure', 'The data failed to load from the server.').setIcon(Sonatype.MessageBox.ERROR);
        }

        // @todo: need global alert mechanism for fatal errors.
      },

      beforeFormRenderHandler : function(component) {
        var sp = Sonatype.lib.Permissions;
        if (sp.checkPermission('nexus:routes', sp.EDIT))
        {
          component.buttons[0].disabled = false;
        }
      },

      formDataLoader : function(formPanel, resourceURI, modFuncs) {
        formPanel.getForm().doAction('sonatypeLoad', {
              url : resourceURI,
              method : 'GET',
              fpanel : formPanel,
              dataModifiers : modFuncs,
              scope : this
            });
      },

      rowSelect : function(selectionModel, index, rec) {
        var id = rec.id; // note: rec.id is unique for new resources and equal
                          // to resourceURI for existing ones
        var formPanel = this.formCards.findById(id);

        // assumption: new route forms already exist in formCards, so they won't
        // get into this case
        if (!formPanel)
        { // create form and populate current data
          var config = Ext.apply({}, this.formConfig.route, {
                id : id
              });

          formPanel = new Ext.FormPanel(config);
          formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
          formPanel.form.on('actionfailed', this.actionFailedHandler, this);
          formPanel.on('beforerender', this.beforeFormRenderHandler, this);
          formPanel.on('afterlayout', this.afterLayoutFormHandler, this, {
                single : true
              });

          var buttonInfoObj = {
            formPanel : formPanel,
            isNew : false, // not a new route form, see assumption
            resourceURI : rec.data.resourceURI
          };

          // save button event handler
          formPanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
          // cancel button event handler
          formPanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

          this.formDataLoader(formPanel, rec.data.resourceURI, this.loadDataModFuncs.route);
          this.formCards.add(formPanel);
        }

        // always set active and re-layout
        this.formCards.getLayout().setActiveItem(formPanel);
        formPanel.doLayout();
      },

      // requires being scoped to RoutesConfigPanel to get shared reposList data
      loadRepoListHelper : function(arr, srcObj, fpanel) {
        var tree = fpanel.find('name', 'repositories')[0];

        tree.setValue(arr);

        return arr; // return arr, even if empty to comply with sonatypeLoad
                    // data modifier requirement
      },

      exportRepoListHelper : function(val, fpanel) {
        var tree = fpanel.find('name', 'repositories')[0];

        var outputArr = [];
        var items = tree.getValue();

        for (var i = 0; i < items.length; i++)
        {
          outputArr[i] = {
            id : items[i]
          };
        }

        return outputArr;
      },

      updateTreePanel : function(blockingType, id) {
        var p = this.formCards.getLayout().activeItem.find('name', 'repositories')[0];
        p.setVisible(!blockingType);
        p.required = !blockingType;
      }
    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Repository Edit/Create panel layout and controller
 */

define('repoServer/RepoEditPanel',['Sonatype/all', 'Sonatype/strings'], function(Sonatype, Strings){
var REPO_REMOTE_STORAGE_REGEXP = /^(?:http|https|ftp):\/\//i;

Sonatype.repoServer.AbstractRepositoryEditor = function(config) {
  var config = config || {};
  var defaultConfig = {
    uri : Sonatype.config.repos.urls.repositories,
    resetButton : true,
    defaultTimeoutValues : {}
  };
  Ext.apply(this, config, defaultConfig);

  Sonatype.repoServer.AbstractRepositoryEditor.superclass.constructor.call(this, {
        listeners : {
          submit : {
            fn : this.submitHandler,
            scope : this
          },
          load : {
              fn : this.loadHandler,
              scope : this
          }
        }
      });

  this.on('show', this.showHandler, this);
};

Ext.extend(Sonatype.repoServer.AbstractRepositoryEditor, Sonatype.ext.FormPanel, {
      loadData : function() {
        if (this.isNew)
        {
          var templateModifiers = Ext.apply({}, {
                id : Strings.returnEmptyStr,
                name : Strings.returnEmptyStr
              }, this.dataModifiers.load);

          this.form.on('actioncomplete', function(form, action) {
                form.clearInvalid();
              }, {
                single : true
              });
          this.form.doAction('sonatypeLoad', {
                url : Sonatype.config.repos.urls.repoTemplate[this.payload.data.repoType],
                method : 'GET',
                fpanel : this,
                dataModifiers : templateModifiers,
                scope : this,
                success : this.templateLoadSuccess.createDelegate(this)
              });
        }
        else
        {
          Sonatype.repoServer.AbstractRepositoryEditor.superclass.loadData.call(this);
        }
      },

      /* For plugin contributions:

      contributions added here are valid for hosted and proxy repository editors.

      When a form is build or it's values reset (e.g. load/select provider),
      contributions are selected according to their conditions:
      the condition objects's properties will be checked for equality with
      properties from the data set.
        e.g. var condition = { provider: 'maven2' }
      If all keys match,  the 'enter' method will be called with the form data and the form itself.

      The 'leave' method is called only when the provider selection changed (for new repositories).
      The data given is already changed and does not resemble the data used to select contributions.
      This method will only be called if the contribution's 'enter' method was called before.

       */
      contributions : {
        add : function(condition, enter, leave) {
          this.list.push(
            {
              condition: condition,
              enter: enter,
              leave: leave
            }
          )
        },
        list : new Array(),
        lastPlugins : new Array()
      },

      contribute : function( data, action)
      {
          if ( action === 'leave' && this.contributions.lastPlugins.length != 0 )
          {
              for ( i = 0; i < this.contributions.lastPlugins.length; i++ )
              {
                  contrib = this.contributions.lastPlugins[i];
                  contrib.leave( data, this.form );
              }
              this.contributions.lastPlugins = new Array();
          } else {
              plugins:
              for ( i = 0; i < this.contributions.list.length; i++ )
              {
                  contrib = this.contributions.list[i];
                  for ( key in contrib.condition )
                  {
                      if ( contrib.condition.hasOwnProperty( key ) && data[key] !== contrib.condition[key] )
                      {
                          continue plugins;
                      }
                  }
                  this.contributions.lastPlugins.push(contrib);
                  contrib[action]( data, this.form );
              }
          }
      },

      providerSelectHandler : function(combo, rec, index) {
          this.form.findField( 'format' ).setValue( rec.data.format );
          this.form.findField( 'providerRole' ).setValue( rec.data.providerRole );
          this.afterProviderSelectHandler( combo, rec, index );

          // let plugins change UI
          this.contribute( rec.data, 'enter' )
      },

      templateLoadSuccess : function(form, action) {},


      repoPolicySelectHandler : function(combo, rec, index) {
        var repoPolicy = rec.data.value.toLowerCase();
        var fields = ['notFoundCacheTTL', 'artifactMaxAge', 'metadataMaxAge', 'itemMaxAge'];

        if (this.lastPolicy != repoPolicy)
        {
          if (this.setBackExpValsFunc)
          {
            this.setBackExpValsFunc();
          }
          else
          {
            var oldValues = {};
            for (var i = fields.length - 1; i >= 0; i--)
            {
              var formField = this.form.findField(fields[i]);
              if (formField)
              {
                oldValues[fields[i]] = formField.getValue();
              }
            }

            this.setBackExpValsFunc = function() {
              this.form.setValues(oldValues);
              this.setBackExpValsFunc = null;
            }.createDelegate(this);

            var repoType = this.form.findField('repoType').getValue();

            if (this.defaultTimeoutValues[repoPolicy])
            {
              this.form.setValues(this.defaultTimeoutValues[repoPolicy]);
            }
            else
            {
              Ext.Ajax.request({
                    scope : this,
                    url : Sonatype.config.repos.urls.repoTemplate[repoType + '_' + repoPolicy],
                    callback : function(options, success, response) {
                      if (success)
                      {
                        var templateData = Ext.decode(response.responseText);
                        this.defaultTimeoutValues[repoPolicy] = {};
                        for (var i = fields.length - 1; i >= 0; i--)
                        {
                          var formField = this.form.findField(fields[i]);
                          if (formField)
                          {
                            this.defaultTimeoutValues[repoPolicy][fields[i]] = templateData.data[fields[i]];
                          }
                        }

                        this.form.setValues(this.defaultTimeoutValues[repoPolicy]);
                      }
                    }
                  });
            }

          }

          // filter the Deploy Policy combo
          this.updateWritePolicy();

          this.lastPolicy = repoPolicy;
        }
      },

      repoPolicySubmitDataModifier : function() {

      },
      updateWritePolicy : function() {

      },

      submitHandler : function(form, action, receivedData) {
        if (this.isNew)
        {
          if (!receivedData.resourceURI)
          {
            receivedData.resourceURI = Sonatype.config.host + Sonatype.config.repos.urls.repositories + '/' + receivedData.id;
            if (receivedData.exposed == null)
            {
              receivedData.exposed = true;
            }
            if (receivedData.userManaged == null)
            {
              receivedData.userManaged = true;
            }
          }

          var repoPanel = Ext.getCmp('view-repositories');
          repoPanel.statusStart();

          // convert case
          receivedData.repoPolicy = Strings.upperFirstCharLowerRest(receivedData.repoPolicy);

          return;
        }

        var rec = this.payload;
        rec.beginEdit();
        rec.set('name', receivedData.name);
        rec.set('repoType', receivedData.repoType);
        rec.set('format', receivedData.format);
        rec.set('repoPolicy', Strings.upperFirstCharLowerRest(receivedData.repoPolicy));

        // give subclasses the opportunity to update data as well (ATM only for proxy repo URLs)
        if (this.updateRecord) {
          this.updateRecord(rec, receivedData);
        }

        rec.commit();
        rec.endEdit();

      },

      // @override
      addSorted : function(store, rec) {
        // make sure listing group first, then the repositories sorted
        var insertIndex = store.getCount();
        for (var i = 0; i < store.getCount(); i++)
        {
          var tempRec = store.getAt(i);
          if (tempRec.get('repoType') == 'group')
          {
            continue;
          }
          if (tempRec.get('name').toLowerCase() > rec.get('name').toLowerCase())
          {
            insertIndex = i;
            break;
          }
        }
        // hack the policy
        if (rec.get('repoPolicy') && rec.get('repoPolicy') == 'MIXED')
        {
          rec.beginEdit();
          rec.set('repoPolicy', '');
          rec.commit();
          rec.endEdit();
        }

        store.insert(insertIndex, [rec]);
      },
      isMavenFormat : function(format) {
        return format.indexOf('maven') == 0;
      },
      updateRepoPolicyField : function(form, format, repoPolicy) {
        var repoPolicyField = form.findField('repoPolicy');
        if (this.isMavenFormat(format))
        {
          repoPolicyField.enable();
          repoPolicyField.setValue(Strings.upperFirstCharLowerRest(repoPolicy || 'RELEASE'));
        } else {
          repoPolicyField.setValue('Mixed');
          repoPolicyField.disable();
        }
      },
      updateIndexableCombo : function(form, format) {
        var indexableCombo = form.findField('indexable');
        if (this.isMavenFormat(format))
        {
          indexableCombo.enable();
        }
        else
        {
          indexableCombo.setValue('False');
          indexableCombo.disable();
        }
      },
      updateDownloadRemoteIndexCombo : function(form, format) {
        var downloadRemoteIndexCombo = form.findField('downloadRemoteIndexes');

        if (downloadRemoteIndexCombo)
        {
          if (this.isMavenFormat(format))
          {
            downloadRemoteIndexCombo.enable();
          }
          else
          {
            downloadRemoteIndexCombo.setValue('False');
            downloadRemoteIndexCombo.disable();
          }
        }
      },
      updateFields : function(form, data) {
        this.updateIndexableCombo(form, data.format);
        this.updateRepoPolicyField(form, data.format, data.repoPolicy)
        this.updateDownloadRemoteIndexCombo(form, data.format);
        this.contribute(data, 'enter')
      },
      afterProviderSelectHandler : function(combo, rec, index) {
        this.updateFields(this.form, rec.data);
      },
      showHandler : function() {
        this.updateWritePolicy();
      },
      loadHandler : function(form, action, receivedData) {
        this.updateFields(form, receivedData)
      }
    });

Sonatype.repoServer.HostedRepositoryEditor = function(config) {
  var config = config || {};
  var defaultConfig = {
    dataModifiers : {
      load : {
        repoPolicy : Strings.upperFirstCharLowerRest,
        browseable : Strings.capitalize,
        indexable : Strings.capitalize,
        exposed : Strings.capitalize
      },
      submit : {
        repoPolicy : Strings.uppercase,
        browseable : Strings.convert.stringContextToBool,
        indexable : Strings.convert.stringContextToBool,
        exposed : Strings.convert.stringContextToBool,
        downloadRemoteIndexes : function() {
          return false;
        },
        autoBlockActive : function() {
          return false;
        },
        checksumPolicy : function() {
          return 'IGNORE';
        }
      }
    },
    referenceData : Sonatype.repoServer.referenceData.repositoryState.hosted
  };
  Ext.apply(this, config, defaultConfig);

  var ht = Sonatype.repoServer.resources.help.repos;

  this.tfStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['True'], ['False']]
      });

  this.policyStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['Release'], ['Snapshot']]
      });

  this.writePolicyStore = new Ext.data.SimpleStore({
        fields : ['value', 'display'],
        data : [['ALLOW_WRITE', 'Allow Redeploy'], ['ALLOW_WRITE_ONCE', 'Disable Redeploy'], ['READ_ONLY', 'Read Only']]
      });

  this.providerStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'provider',
        fields : [{
              name : 'description',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'format'
            }, {
              name : 'providerRole'
            }, {
              name : 'provider'
            }],
        sortInfo : {
          field : 'description',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.repoTypes + '?repoType=hosted',
        autoLoad : true
      });

  this.checkPayload();

  Sonatype.repoServer.HostedRepositoryEditor.superclass.constructor.call(this, {
        dataStores : [this.providerStore],
        items : [{
              xtype : 'textfield',
              fieldLabel : 'Repository ID',
              itemCls : 'required-field',
              helpText : ht.id,
              name : 'id',
              width : 200,
              allowBlank : false,
              disabled : !this.isNew,
              validator : Strings.validateId
            }, {
              xtype : 'textfield',
              fieldLabel : 'Repository Name',
              itemCls : 'required-field',
              helpText : ht.name,
              name : 'name',
              width : 200,
              htmlDecode : true,
              allowBlank : false
            }, {
              xtype : 'textfield',
              fieldLabel : 'Repository Type',
              itemCls : 'required-field',
              helpText : ht.repoType,
              name : 'repoType',
              width : 100,
              disabled : true,
              allowBlank : false
            }, {
              xtype : 'combo',
              fieldLabel : 'Provider',
              itemCls : 'required-field',
              helpText : ht.provider,
              name : 'provider',
              width : 150,
              store : this.providerStore,
              displayField : 'description',
              valueField : 'provider',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              disabled : !this.isNew,
              listeners : {
                select : this.providerSelectHandler,
                beforeselect : function(combo, rec, index) { this.contribute(rec.data, 'leave'); return true },
                scope : this
              }
            }, {
              xtype : 'hidden',
              name : 'providerRole'
            }, {
              xtype : 'textfield',
              fieldLabel : 'Format',
              itemCls : 'required-field',
              helpText : ht.format,
              name : 'format',
              width : 100,
              disabled : true,
              allowBlank : false
            }, {
              xtype : 'combo',
              fieldLabel : 'Repository Policy',
              itemCls : 'required-field',
              helpText : ht.repoPolicy,
              name : 'repoPolicy',
              width : 80,
              store : this.policyStore,
              displayField : 'value',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : this.repoPolicySelectHandler,
                scope : this
              }
            }, {
              xtype : 'textfield',
              fieldLabel : 'Default Local Storage Location',
              helpText : ht.defaultLocalStorageUrl,
              name : 'defaultLocalStorageUrl',
              anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
              allowBlank : true,
              disabled : true
            }, {
              xtype : 'textfield',
              fieldLabel : 'Override Local Storage Location',
              helpText : ht.overrideLocalStorageUrl,
              name : 'overrideLocalStorageUrl',
              anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
              allowBlank : true
            }, {
              xtype : 'fieldset',
              checkboxToggle : false,
              title : 'Access Settings',
              anchor : Sonatype.view.FIELDSET_OFFSET,
              collapsible : true,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              defaults : {
                xtype : 'combo',
                fieldLabel : 'default',
                itemCls : 'required-field',
                name : 'default',
                width : 75,
                store : this.tfStore,
                displayField : 'value',
                editable : false,
                forceSelection : true,
                mode : 'local',
                triggerAction : 'all',
                emptyText : 'Select...',
                selectOnFocus : true,
                allowBlank : false
              },
              items : [{
                    fieldLabel : 'Deployment Policy',
                    helpText : ht.writePolicy,
                    name : 'writePolicy',
                    store : this.writePolicyStore,
                    displayField : 'display',
                    valueField : 'value',
                    width : 120,
                    listWidth : 120,
                    lastQuery : ''
                  }, {
                    fieldLabel : 'Allow File Browsing',
                    helpText : ht.browseable,
                    name : 'browseable'
                  }, {
                    fieldLabel : 'Include in Search',
                    helpText : ht.indexable,
                    name : 'indexable'
                  }, {
                    fieldLabel : 'Publish URL',
                    helpText : ht.exposed,
                    name : 'exposed'
                  }]
            }, {
              xtype : 'fieldset',
              checkboxToggle : false,
              title : 'Expiration Settings',
              anchor : Sonatype.view.FIELDSET_OFFSET,
              collapsible : true,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              defaults : {
                xtype : 'numberfield',
                fieldLabel : 'default',
                itemCls : 'required-field',
                afterText : 'minutes',
                name : 'default',
                width : 50,
                allowBlank : false,
                allowDecimals : false,
                allowNegative : true,
                minValue : -1,
                maxValue : 511000
              },
              items : [{
                    fieldLabel : 'Not Found Cache TTL',
                    helpText : ht.notFoundCacheTTL,
                    name : 'notFoundCacheTTL'
                  }]
            }]
      });

};

Ext.extend(Sonatype.repoServer.HostedRepositoryEditor, Sonatype.repoServer.AbstractRepositoryEditor, {
      updateWritePolicy : function() {

        var repoPolicyField = this.find('name', 'repoPolicy')[0];
        var repoPolicy = repoPolicyField.getValue();

        var writePolicyField = this.find('name', 'writePolicy')[0];

        // filter out the redeploy option for SNAPSHOT repos
        if (("" + repoPolicy).toLowerCase() == 'snapshot')
        {
          // first change the value if it is ALLOW_WRITE_ONCE
          if (writePolicyField.getValue() != 'READ_ONLY')
          {
            writePolicyField.setValue('ALLOW_WRITE');
          }

          this.writePolicyStore.filterBy(function(rec, id) {
                return rec.data.value != 'ALLOW_WRITE_ONCE';
              });
        }
        else
        {
          writePolicyField.store.clearFilter();

          if (writePolicyField.getValue() == null || writePolicyField.getValue() == '')
          {
            writePolicyField.setValue('ALLOW_WRITE_ONCE');
          }
        }
      }
    });

Sonatype.repoServer.ProxyRepositoryEditor = function(config) {
  var config = config || {};
  var defaultConfig = {
    dataModifiers : {
      load : {
        repoPolicy : Strings.upperFirstCharLowerRest,
        browseable : Strings.capitalize,
        indexable : Strings.capitalize,
        exposed : Strings.capitalize,
        downloadRemoteIndexes : Strings.capitalize,
        autoBlockActive : Strings.capitalize,
        fileTypeValidation : Strings.capitalize,
        checksumPolicy : Strings.upperFirstCharLowerRest
      },
      submit : {
        repoPolicy : Strings.uppercase,
        browseable : Strings.convert.stringContextToBool,
        indexable : Strings.convert.stringContextToBool,
        exposed : Strings.convert.stringContextToBool,
        downloadRemoteIndexes : Strings.convert.stringContextToBool,
        autoBlockActive : Strings.convert.stringContextToBool,
        fileTypeValidation : Strings.convert.stringContextToBool,
        checksumPolicy : Strings.uppercase
      }
    },
    validationModifiers : {
      remoteStorageUrl : 'remoteStorage.remoteStorageUrl'
    },
    referenceData : Sonatype.repoServer.referenceData.repositoryState.proxy
  };
  Ext.apply(this, config, defaultConfig);

  var ht = Sonatype.repoServer.resources.help.repos;

  this.tfStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['True'], ['False']]
      });

  this.policyStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['Release'], ['Snapshot']]
      });

  this.checksumPolicyStore = new Ext.data.SimpleStore({
        fields : ['label', 'value'],
        data : [['Ignore', 'IGNORE'], ['Warn', 'WARN'], ['StrictIfExists', 'STRICT_IF_EXISTS'], ['Strict', 'STRICT']]
      });

  this.providerStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'provider',
        fields : [{
              name : 'description',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'format'
            }, {
              name : 'providerRole'
            }, {
              name : 'provider'
            }],
        sortInfo : {
          field : 'description',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.repoTypes + '?repoType=proxy',
        autoLoad : true
      });

  this.checkPayload();

  Sonatype.repoServer.ProxyRepositoryEditor.superclass.constructor.call(this, {
        dataStores : [this.providerStore],
        items : [{
              xtype : 'textfield',
              fieldLabel : 'Repository ID',
              itemCls : 'required-field',
              helpText : ht.id,
              name : 'id',
              width : 200,
              allowBlank : false,
              disabled : !this.isNew,
              validator : Strings.validateId
            }, {
              xtype : 'textfield',
              fieldLabel : 'Repository Name',
              itemCls : 'required-field',
              helpText : ht.name,
              name : 'name',
              width : 200,
              allowBlank : false
            }, {
              xtype : 'textfield',
              fieldLabel : 'Repository Type',
              itemCls : 'required-field',
              helpText : ht.repoType,
              name : 'repoType',
              width : 100,
              disabled : true,
              allowBlank : false
            }, {
              xtype : 'combo',
              fieldLabel : 'Provider',
              itemCls : 'required-field',
              helpText : ht.provider,
              name : 'provider',
              width : 150,
              store : this.providerStore,
              displayField : 'description',
              valueField : 'provider',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              disabled : !this.isNew,
              listeners : {
                select : this.providerSelectHandler,
                beforeselect : function(combo, rec, index) { this.contribute(rec.data, 'leave'); return true },
                scope : this
              }
            }, {
              xtype : 'hidden',
              name : 'providerRole'
            }, {
              xtype : 'textfield',
              fieldLabel : 'Format',
              itemCls : 'required-field',
              helpText : ht.format,
              name : 'format',
              width : 100,
              disabled : true,
              allowBlank : false
            }, {
              xtype : 'combo',
              fieldLabel : 'Repository Policy',
              itemCls : 'required-field',
              helpText : ht.repoPolicy,
              name : 'repoPolicy',
              width : 80,
              store : this.policyStore,
              displayField : 'value',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : this.repoPolicySelectHandler,
                scope : this
              }
            }, {
              xtype : 'textfield',
              fieldLabel : 'Default Local Storage Location',
              helpText : ht.defaultLocalStorageUrl,
              name : 'defaultLocalStorageUrl',
              anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
              allowBlank : true,
              disabled : true
            }, {
              xtype : 'textfield',
              fieldLabel : 'Override Local Storage Location',
              helpText : ht.overrideLocalStorageUrl,
              name : 'overrideLocalStorageUrl',
              anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
              allowBlank : true
            }, {
              xtype : 'fieldset',
              checkboxToggle : false,
              title : 'Remote Repository Access',
              anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
              collapsible : true,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              items : [{
                    xtype : 'textfield',
                    fieldLabel : 'Remote Storage Location',
                    itemCls : 'required-field',
                    helpText : ht.remoteStorageUrl,
                    emptyText : 'http://some-remote-repository/repo-root',
                    name : 'remoteStorage.remoteStorageUrl',
                    anchor : Sonatype.view.FIELD_OFFSET,
                    allowBlank : false,
                    validator : function(v) {
                      if (v.match(REPO_REMOTE_STORAGE_REGEXP))
                      {
                        return true;
                      }
                      else
                      {
                        return 'Protocol must be http://, https:// or ftp://';
                      }
                    }
                  }, {
                    xtype : 'combo',
                    fieldLabel : 'Download Remote Indexes',
                    helpText : ht.downloadRemoteIndexes,
                    name : 'downloadRemoteIndexes',
                    itemCls : 'required-field',
                    width : 75,
                    store : this.tfStore,
                    displayField : 'value',
                    editable : false,
                    forceSelection : true,
                    mode : 'local',
                    triggerAction : 'all',
                    emptyText : 'Select...',
                    selectOnFocus : true,
                    allowBlank : false
                  }, {}, {
                    xtype : 'combo',
                    fieldLabel : 'Auto Blocking Enabled',
                    helpText : ht.autoBlockActive,
                    name : 'autoBlockActive',
                    itemCls : 'required-field',
                    width : 75,
                    store : this.tfStore,
                    displayField : 'value',
                    editable : false,
                    forceSelection : true,
                    mode : 'local',
                    triggerAction : 'all',
                    emptyText : 'Select...',
                    selectOnFocus : true,
                    allowBlank : false
                  }, {
                    xtype : 'combo',
                    fieldLabel : 'File Content Validation',
                    helpText : ht.fileTypeValidation,
                    name : 'fileTypeValidation',
                    itemCls : 'required-field',
                    width : 75,
                    store : this.tfStore,
                    displayField : 'value',
                    editable : false,
                    forceSelection : true,
                    mode : 'local',
                    triggerAction : 'all',
                    emptyText : 'Select...',
                    selectOnFocus : true,
                    allowBlank : false
                  }, {
                    xtype : 'combo',
                    fieldLabel : 'Checksum Policy',
                    itemCls : 'required-field',
                    helpText : ht.checksumPolicy,
                    name : 'checksumPolicy',
                    width : 95,
                    store : this.checksumPolicyStore,
                    displayField : 'label',
                    valueField : 'value',
                    editable : false,
                    forceSelection : true,
                    mode : 'local',
                    triggerAction : 'all',
                    emptyText : 'Select...',
                    selectOnFocus : true,
                    allowBlank : false
                  }, {
                    xtype : 'fieldset',
                    checkboxToggle : true,
                    title : 'Authentication (optional)',
                    name : 'fieldset_remoteStorage.authentication',
                    collapsed : true,
                    autoHeight : true,
                    layoutConfig : {
                      labelSeparator : ''
                    },
                    items : [{
                          xtype : 'textfield',
                          fieldLabel : 'Username',
                          helpText : ht.remoteUsername,
                          name : 'remoteStorage.authentication.username',
                          width : 100,
                          allowBlank : true
                        }, {
                          xtype : 'textfield',
                          fieldLabel : 'Password',
                          helpText : ht.remotePassword,
                          inputType : 'password',
                          name : 'remoteStorage.authentication.password',
                          width : 100,
                          allowBlank : true
                        }, {
                          xtype : 'textfield',
                          fieldLabel : 'NT LAN Host',
                          helpText : ht.remoteNtlmHost,
                          name : 'remoteStorage.authentication.ntlmHost',
                          anchor : Sonatype.view.FIELD_OFFSET,
                          allowBlank : true
                        }, {
                          xtype : 'textfield',
                          fieldLabel : 'NT LAN Manager Domain',
                          helpText : ht.remoteNtlmDomain,
                          name : 'remoteStorage.authentication.ntlmDomain',
                          anchor : Sonatype.view.FIELD_OFFSET,
                          allowBlank : true
                        }]
                  }]
            }, // end remote storage)
            {
              xtype : 'fieldset',
              checkboxToggle : false,
              title : 'Access Settings',
              anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
              collapsible : true,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              defaults : {
                xtype : 'combo',
                fieldLabel : 'default',
                itemCls : 'required-field',
                name : 'default',
                width : 75,
                store : this.tfStore,
                displayField : 'value',
                editable : false,
                forceSelection : true,
                mode : 'local',
                triggerAction : 'all',
                emptyText : 'Select...',
                selectOnFocus : true,
                allowBlank : false
              },

              items : [{
                    fieldLabel : 'Allow File Browsing',
                    helpText : ht.browseable,
                    name : 'browseable'
                  }, {
                    fieldLabel : 'Include in Search',
                    helpText : ht.indexable,
                    name : 'indexable'
                  }, {
                    fieldLabel : 'Publish URL',
                    helpText : ht.exposed,
                    name : 'exposed'
                  }]
            }, {
              xtype : 'fieldset',
              checkboxToggle : false,
              title : 'Expiration Settings',
              anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
              collapsible : true,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              defaults : {
                xtype : 'numberfield',
                fieldLabel : 'default',
                itemCls : 'required-field',
                afterText : 'minutes',
                name : 'default',
                width : 50,
                allowBlank : false,
                allowDecimals : false,
                allowNegative : true,
                minValue : -1,
                maxValue : 511000
              },

              items : [{
                    fieldLabel : 'Not Found Cache TTL',
                    helpText : ht.notFoundCacheTTL,
                    name : 'notFoundCacheTTL'
                  }, {
                    fieldLabel : 'Artifact Max Age',
                    helpText : ht.artifactMaxAge,
                    name : 'artifactMaxAge'
                  }, {
                    fieldLabel : 'Metadata Max Age',
                    helpText : ht.metadataMaxAge,
                    name : 'metadataMaxAge'
                  }, {
                    fieldLabel : 'Item Max Age',
                    helpText : ht.itemMaxAge,
                    name : 'itemMaxAge'
                  }]
            },

            {
              xtype : 'fieldset',
              checkboxToggle : true,
              title : 'HTTP Request Settings (optional)',
              anchor : Sonatype.view.FIELDSET_OFFSET_WITH_SCROLL,
              name : 'fieldset_remoteStorage.connectionSettings',
              collapsed : true,
              autoHeight : true,
              layoutConfig : {
                labelSeparator : ''
              },
              listeners : {
                'expand' : {
                  fn : this.optionalFieldsetExpandHandler,
                  scope : this
                },
                'collapse' : {
                  fn : this.optionalFieldsetCollapseHandler,
                  scope : this,
                  delay : 100
                }
              },
              items : [{
                    xtype : 'textfield',
                    fieldLabel : 'User Agent',
                    helpText : ht.userAgentString,
                    name : 'remoteStorage.connectionSettings.userAgentString',
                    anchor : Sonatype.view.FIELD_OFFSET,
                    allowBlank : true
                  }, {
                    xtype : 'textfield',
                    fieldLabel : 'Additional URL Parameters',
                    helpText : ht.queryString,
                    name : 'remoteStorage.connectionSettings.queryString',
                    anchor : Sonatype.view.FIELD_OFFSET,
                    allowBlank : true
                  }, {
                    xtype : 'numberfield',
                    fieldLabel : 'Request Timeout',
                    itemCls : 'required-field',
                    helpText : ht.connectionTimeout,
                    afterText : 'seconds',
                    name : 'remoteStorage.connectionSettings.connectionTimeout',
                    width : 75,
                    allowBlank : true,
                    allowDecimals : false,
                    allowNegative : false,
                    maxValue : 36000
                  }, {
                    xtype : 'numberfield',
                    fieldLabel : 'Request Retry Attempts',
                    itemCls : 'required-field',
                    helpText : ht.retrievalRetryCount,
                    name : 'remoteStorage.connectionSettings.retrievalRetryCount',
                    width : 50,
                    allowBlank : true,
                    allowDecimals : false,
                    allowNegative : false,
                    maxValue : 10
                  }]
            } // end http conn
        ]
      });
};

Ext.extend(Sonatype.repoServer.ProxyRepositoryEditor, Sonatype.repoServer.AbstractRepositoryEditor, {
      loadHandler : function(form, action, receivedData) {
        Sonatype.repoServer.AbstractRepositoryEditor.prototype.loadHandler.apply(this, arguments);

        var repoType = receivedData.repoType;

        if (repoType == 'proxy' && receivedData.remoteStorage && !receivedData.remoteStorage.remoteStorageUrl.match(REPO_REMOTE_STORAGE_REGEXP))
        {
          var rsUrl = this.form.findField('remoteStorage.remoteStorageUrl');
          rsUrl.disable();
          rsUrl.clearInvalid();

          // Disable the editor - this is a temporary measure,
          // until we find a better solution for procurement repos
          this.buttons[0].disable();
        }
      },

      updateRecord : function(rec, receivedData) {
        rec.set('remoteUri', receivedData.remoteStorage.remoteStorageUrl);
      }

    });

Sonatype.repoServer.VirtualRepositoryEditor = function(config) {
  var config = config || {};
  var defaultConfig = {
    dataModifiers : {
      load : {
        syncAtStartup : Strings.capitalize
      },
      submit : {
        syncAtStartup : Strings.convert.stringContextToBool,
        exposed : function() {
          return true;
        }
      }
    },
    referenceData : Sonatype.repoServer.referenceData.repositoryState.virtual
  };
  Ext.apply(this, config, defaultConfig);

  var ht = Sonatype.repoServer.resources.help.repos;

  this.COMBO_WIDTH = 300;

  this.tfStore = new Ext.data.SimpleStore({
        fields : ['value'],
        data : [['True'], ['False']]
      });

  this.repoStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'id',
        fields : [{
              name : 'id'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'format'
            }, {
              name : 'repoType'
            }],
        sortInfo : {
          field : 'name',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.repositories,
        autoLoad : true
      });

  this.providerStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'provider',
        fields : [{
              name : 'description',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'format'
            }, {
              name : 'providerRole'
            }, {
              name : 'provider'
            }],
        sortInfo : {
          field : 'description',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.repoTypes + '?repoType=shadow',
        autoLoad : true
      });

  this.checkPayload();

  Sonatype.repoServer.VirtualRepositoryEditor.superclass.constructor.call(this, {
        dataStores : [this.providerStore, this.repoStore],
        items : [{
              xtype : 'textfield',
              fieldLabel : 'Repository ID',
              itemCls : 'required-field',
              helpText : ht.id,
              name : 'id',
              width : 200,
              allowBlank : false,
              disabled : !this.isNew,
              validator : Strings.validateId
            }, {
              xtype : 'textfield',
              fieldLabel : 'Repository Name',
              itemCls : 'required-field',
              helpText : ht.name,
              name : 'name',
              width : 200,
              allowBlank : false
            }, {
              xtype : 'textfield',
              fieldLabel : 'Repository Type',
              itemCls : 'required-field',
              helpText : ht.repoType,
              name : 'repoType',
              width : 100,
              disabled : true,
              allowBlank : false
            }, {
              xtype : 'combo',
              fieldLabel : 'Provider',
              itemCls : 'required-field',
              helpText : ht.provider,
              name : 'provider',
              width : 150,
              store : this.providerStore,
              displayField : 'description',
              valueField : 'provider',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              disabled : !this.isNew,
              listeners : {
                select : this.providerSelectHandler,
                beforeselect : function(combo, rec, index) { this.contribute(rec.data, 'leave'); return true },
                scope : this
              }
            }, {
              xtype : 'hidden',
              name : 'providerRole'
            }, {
              xtype : 'textfield',
              fieldLabel : 'Format',
              itemCls : 'required-field',
              helpText : ht.format,
              name : 'format',
              width : 100,
              disabled : true,
              allowBlank : false
            }, {
              xtype : 'combo',
              fieldLabel : 'Source Nexus Repository ID',
              itemCls : 'required-field',
              helpText : ht.shadowOf,
              name : 'shadowOf',
              width : 200,
              midWidth : 200,
              store : this.repoStore,
              displayField : 'name',
              valueField : 'id',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              // this config can solve the problem of 'filter is not applied the
              // first time'
              lastQuery : ''
            }, {
              xtype : 'combo',
              fieldLabel : 'Synchronize on Startup',
              itemCls : 'required-field',
              helpText : ht.syncAtStartup,
              name : 'syncAtStartup',
              width : 75,
              store : this.tfStore,
              displayField : 'value',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false
            }]
      });
};

Ext.extend(Sonatype.repoServer.VirtualRepositoryEditor, Sonatype.repoServer.AbstractRepositoryEditor, {

      templateLoadSuccess : function(form, action) {
        var rec = {
          data : {
            provider : this.find('name', 'provider')[0].getValue()
          }
        };

        this.afterProviderSelectHandler(null, rec, null);
      },
      afterProviderSelectHandler : function(combo, rec, index) {
        var provider = rec.data.provider;
        var sourceRepoCombo = this.form.findField('shadowOf');
        sourceRepoCombo.clearValue();
        sourceRepoCombo.clearInvalid();
        //sourceRepoCombo.focus();
        if (provider == 'm1-m2-shadow')
        {
          sourceRepoCombo.store.filterBy(function fn(rec, id) {
                if (rec.data.repoType != 'virtual' && rec.data.format == 'maven1')
                {
                  return true;
                }
                return false;
              });
        }
        else if (provider == 'm2-m1-shadow')
        {
          sourceRepoCombo.store.filterBy(function fn(rec, id) {
                if (rec.data.repoType != 'virtual' && rec.data.format == 'maven2')
                {
                  return true;
                }
                return false;
              });
        }
      },
      loadHandler : function(form, action, receivedData) {}
    });

Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec) {
  var sp = Sonatype.lib.Permissions;

  var repoEditors = {
    hosted : Sonatype.repoServer.HostedRepositoryEditor,
    proxy : Sonatype.repoServer.ProxyRepositoryEditor,
    virtual : Sonatype.repoServer.VirtualRepositoryEditor
  };

  var editor = repoEditors[rec.data.repoType];

  if (editor && rec.data.userManaged && sp.checkPermission('nexus:repositories', sp.READ) && (sp.checkPermission('nexus:repositories', sp.CREATE) || sp.checkPermission('nexus:repositories', sp.DELETE) || sp.checkPermission('nexus:repositories', sp.EDIT)))
  {
    cardPanel.add(new editor({
          tabTitle : 'Configuration',
          name : 'configuration',
          payload : rec
        }));
  }
});

Sonatype.Events.addListener('repositoryAddMenuInit', function(menu) {
      var sp = Sonatype.lib.Permissions;

      if (sp.checkPermission('nexus:repositories', sp.CREATE))
      {
        var createRepoFunc = function(container, rec, item, e) {
          rec.beginEdit();
          rec.set('repoType', item.value);
          rec.set('exposed', true);
          rec.set('userManaged', true);
          rec.commit();
          rec.endEdit();
        };

        menu.add(['-', {
              text : 'Hosted Repository',
              value : 'hosted',
              autoCreateNewRecord : true,
              handler : createRepoFunc
            }, {
              text : 'Proxy Repository',
              value : 'proxy',
              autoCreateNewRecord : true,
              handler : createRepoFunc
            }, {
              text : 'Virtual Repository',
              value : 'virtual',
              autoCreateNewRecord : true,
              handler : createRepoFunc
            }]);
      }
    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Role Edit/Create panel layout and controller
 */

define('repoServer/RoleEditPanel',['Sonatype/all'], function(){
Sonatype.repoServer.RoleEditPanel = function(config) {
  var config = config || {};
  var defaultConfig = {
    title : 'Roles'
  };
  Ext.apply(this, config, defaultConfig);

  this.sp = Sonatype.lib.Permissions;

  this.externalMappingStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'defaultRole.roleId',
        fields : [{
              name : 'defaultRole'
            }, {
              name : 'mappedRoles'
            }],
        url : Sonatype.config.repos.urls.externalRolesAll,
        autoLoad : false
      });

  this.sourceStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'roleHint',
        autoLoad : false,
        url : Sonatype.config.repos.urls.userLocators,
        sortInfo : {
          field : 'description',
          direction : 'ASC'
        },
        fields : [{
              name : 'roleHint'
            }, {
              name : 'description',
              sortType : Ext.data.SortTypes.asUCString
            }],
        listeners : {
          load : {
            fn : function(store, records, options) {
              for (var i = 0; i < records.length; i++)
              {
                var rec = records[i];
                var v = rec.data.roleHint;
                if (v == 'allConfigured' || v == 'mappedExternal' || v == 'default')
                {
                  store.remove(rec);
                }
              }

              this.userLocatorsLoaded = true;
            },
            scope : this
          }
        }
      });

  Sonatype.Events.on('roleAddMenuInit', this.onAddMenuInit, this);

  Sonatype.repoServer.RoleEditPanel.superclass.constructor.call(this, {
        addMenuInitEvent : 'roleAddMenuInit',
        deleteButton : this.sp.checkPermission('security:roles', this.sp.DELETE),
        rowClickEvent : 'roleViewInit',
        url : Sonatype.config.repos.urls.roles,
        dataAutoLoad : true,
        dataId : 'id',
        dataBookmark : 'id',
        dataStores : [this.sourceStore, this.externalMappingStore],
        columns : [{
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString,
              header : 'Name',
              width : 200,
              renderer : function(value, meta, rec, index) {
                return rec.data.mapping ? ('<b>' + value + '</b') : value;
              }
            }, {
              name : 'id'
            }, {
              name : 'resourceURI'
            }, {
              name : 'mapping',
              header : 'Realm',
              width : 100,
              mapping : 'id',
              convert : this.convertMapping.createDelegate(this)
            }, {
              name : 'description',
              header : 'Description',
              width : 175,
              autoExpand : true
            }, {
              name : 'roles'
            }, {
              name : 'privileges'
            }, {
              name : 'userManaged'
            }],
        listeners : {
          beforedestroy : {
            fn : function() {
              Sonatype.Events.un('roleAddMenuInit', this.onAddMenuInit, this);
            },
            scope : this
          }
        }
      });
};

Ext.extend(Sonatype.repoServer.RoleEditPanel, Sonatype.panels.GridViewer, {
      emptyText: 'No roles defined',
      emptyTextWhileFiltering: 'No roles matched criteria: {criteria}',

      convertMapping : function(value, parent) {
        var mappingRec = this.externalMappingStore.getById(value);
        if (mappingRec)
        {
          var mappings = mappingRec.data.mappedRoles;
          var s = '';
          for (var i = 0; i < mappings.length; i++)
          {
            if (s)
              s += ', ';
            s += mappings[i].source;
          }
          return s;
        }
        else
        {
          return 'nexus';
        }
      },
      onAddMenuInit : function(menu) {
        menu.add('-');
        if (this.sp.checkPermission('security:roles', this.sp.CREATE))
        {
          menu.add({
                text : 'Nexus Role',
                autoCreateNewRecord : true,
                handler : function(container, rec, item, e) {
                  rec.beginEdit();
                  rec.set('source', 'default');
                  rec.commit();
                  rec.endEdit();
                },
                scope : this
              });
        }

        var
              self = this,
              addExternalMappingMenuItem = function() {
                if (self.sp.checkPermission('security:roles', self.sp.CREATE) && self.sourceStore.getCount() > 0 ) {
                  self.toolbarAddButton.menu.add({
                    text : 'External Role Mapping',
                    handler : self.mapExternalRoles,
                    scope : self
                  });

                  self.externalMappingMenuItemAdded = true;
                }
              };

        if (self.externalMappingMenuItemAdded) {
          return;
        }

        if (self.userLocatorsLoaded) {
          addExternalMappingMenuItem();
        } else {
          this.sourceStore.on('load', addExternalMappingMenuItem, this, {single : true});
        }
      },

      mapExternalRoles : function() {
        new Sonatype.repoServer.ExternapRoleMappingPopup({
              hostPanel : this,
              sourceStore : this.sourceStore
            }).show();
      }
    });

Sonatype.repoServer.ExternapRoleMappingPopup = function(config) {
  var config = config || {};
  var defaultConfig = {
    title : 'Map External Role'
  };
  Ext.apply(this, config, defaultConfig);

  this.roleStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'roleId',
        fields : [{
              name : 'roleId'
            }, {
              name : 'source'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }],
        sortInfo : {
          field : 'name',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.plexusRolesAll,
        autoLoad : true
      });

  Sonatype.repoServer.ExternapRoleMappingPopup.superclass.constructor.call(this, {
        closable : true,
        autoWidth : false,
        width : 400,
        autoHeight : true,
        modal : true,
        constrain : true,
        resizable : false,
        draggable : false,
        items : [{
              xtype : 'form',
              layoutConfig : {
                labelSeparator : ''
              },
              labelWidth : 60,
              frame : true,
              defaultType : 'textfield',
              monitorValid : true,
              items : [{
                    xtype : 'combo',
                    fieldLabel : 'Realm',
                    itemCls : 'required-field',
                    helpText : 'Security realm to select roles from.',
                    name : 'source',
                    anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                    width : 200,
                    store : this.sourceStore,
                    displayField : 'description',
                    valueField : 'roleHint',
                    editable : false,
                    forceSelection : true,
                    mode : 'local',
                    triggerAction : 'all',
                    emptyText : 'Select...',
                    selectOnFocus : true,
                    allowBlank : false,
                    listeners : {
                      select : {
                        fn : this.onSourceSelect,
                        scope : this
                      }
                    }
                  }, {
                    xtype : 'combo',
                    fieldLabel : 'Role',
                    itemCls : 'required-field',
                    helpText : 'External role to map.',
                    name : 'roleId',
                    anchor : Sonatype.view.FIELD_OFFSET_WITH_SCROLL,
                    width : 200,
                    store : this.roleStore,
                    displayField : 'name',
                    valueField : 'roleId',
                    editable : true,
                    forceSelection : false,
                    mode : 'local',
                    triggerAction : 'all',
                    lastQuery : '',
                    emptyText : 'Select...',
                    selectOnFocus : true,
                    allowBlank : false
                  }],
              buttons : [{
                    text : 'Create Mapping',
                    formBind : true,
                    handler : this.validateRoleMapping,
                    scope : this,
                    disabled : true
                  }, {
                    text : 'Cancel',
                    formBind : false,
                    handler : function(button, e) {
                      this.close();
                    },
                    scope : this
                  }]
            }]
      });
};

Ext.extend(Sonatype.repoServer.ExternapRoleMappingPopup, Ext.Window, {
      onSourceSelect : function(combo, rec, index) {
        var roleCombo = this.find('name', 'roleId')[0];
        roleCombo.clearValue();
        roleCombo.store.filter('source', rec.data.roleHint);
      },

      validateRoleMapping : function(button, e) {
        var roleId = this.find('name', 'roleId')[0].getValue();
        var sourceId = this.find('name', 'source')[0].getValue();

        Ext.Ajax.request({
              url : Sonatype.config.servicePath + '/external_role_map/' + sourceId + '/' + roleId,
              callback : function(options, isSuccess, response) {
                if (isSuccess)
                {
                  this.createRoleMapping();
                }
                else
                {
                  this.find('name', 'roleId')[0].markInvalid('Role not found!');
                }
              },
              scope : this,
              method : 'GET',
              suppressStatus : '404'
            });
      },
      createRoleMapping : function() {
        if (this.hostPanel)
        {
          var roleId = this.find('name', 'roleId')[0].getValue();
          var roleRec = this.roleStore.getById(roleId);
          var sourceId = this.find('name', 'source')[0].getValue();
          var handler = this.hostPanel.addActionHandler.createDelegate(this.hostPanel, [function(rec, item, e) {
                    rec.beginEdit();
                    rec.set('source', sourceId);
                    rec.set('mapping', sourceId);
                    rec.commit();
                    rec.endEdit();
                  }, {
                    autoCreateNewRecord : true,
                    text : "Role Mapping"
                  }], 0);
          handler();

          var name = roleRec == null ? roleId :roleRec.data.name; 
          var defaultData = {
            id : roleId,
            name : name,
            description : 'External mapping for ' + name + ' (' + sourceId + ')'
          };

          this.hostPanel.cardPanel.getLayout().activeItem.find('name', 'id')[0].disable();
          this.hostPanel.cardPanel.getLayout().activeItem.items.get(0).presetData = defaultData;
          this.hostPanel.cardPanel.getLayout().activeItem.items.get(0).resetHandler();

          this.close();
        }
      }
    });

Sonatype.repoServer.DefaultRoleEditor = function(config) {
  var config = config || {};
  var defaultConfig = {
    uri : Sonatype.config.repos.urls.roles,
    labelWidth : 100,
    referenceData : Sonatype.repoServer.referenceData.roles,
    dataModifiers : {
      load : {
        id : function(value, srcObj, fpanel) {
          fpanel.find('name', 'roleManager')[0].setHiddenRoleIds(value, true);
          return value;
        },
        roles : function(arr, srcObj, fpanel) {
          fpanel.find('name', 'roleManager')[0].setSelectedRoleIds(arr, true);
          return arr;
        },
        privileges : function(arr, srcObj, fpanel) {
          fpanel.find('name', 'roleManager')[0].setSelectedPrivilegeIds(arr, true);
          return arr;
        }
      },
      submit : {
        roles : function(value, fpanel) {
          return fpanel.find('name', 'roleManager')[0].getSelectedRoleIds();
        },
        privileges : function(value, fpanel) {
          return fpanel.find('name', 'roleManager')[0].getSelectedPrivilegeIds();
        },
        sessionTimeout : function() {
          return 60;
        }
      }
    }
  };
  Ext.apply(this, config, defaultConfig);

  var ht = Sonatype.repoServer.resources.help.roles;

  this.COMBO_WIDTH = 300;

  this.checkPayload();

  var items = [{
        name : 'internalResourceHeader',
        xtype : 'panel',
        layout : 'table',
        hidden : true,
        style : 'font-size: 18px; padding: 5px 0px 5px 15px',
        items : [{
              html : '<b>This is an internal Nexus resource which cannot be edited or deleted.</b><br><hr/>'
            }]
      }, {
        xtype : 'textfield',
        fieldLabel : 'Role Id',
        itemCls : 'required-field',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.id,
        name : 'id',
        readOnly : !this.isNew,
        allowBlank : false,
        width : this.COMBO_WIDTH
      }, {
        xtype : 'textfield',
        fieldLabel : 'Name',
        itemCls : 'required-field',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.name,
        name : 'name',
        allowBlank : false,
        htmlDecode : true,
        width : this.COMBO_WIDTH
      }, {
        xtype : 'textfield',
        fieldLabel : 'Description',
        labelStyle : 'margin-left: 15px; width: 185px;',
        helpText : ht.description,
        name : 'description',
        allowBlank : true,
        htmlDecode : true,
        width : this.COMBO_WIDTH
      },{
        xtype : 'rolemanager',
        name : 'roleManager',
        height : 200,
        width : 490,
        style : 'margin-left: 15px;margin-top: 10px;border: 1px solid #B5B8C8;'
      }];

  Sonatype.repoServer.DefaultUserEditor.superclass.constructor.call(this, {
        items : items,
        listeners : {
          submit : {
            fn : this.submitHandler,
            scope : this
          },
          load : {
            fn : this.loadHandler,
            scope : this
          }
        }
      });
};

Ext.extend(Sonatype.repoServer.DefaultRoleEditor, Sonatype.ext.FormPanel, {
      resetHandler : function(button, event) {
        this.el.mask('Loading...', 'x-mask-loading');
        Sonatype.repoServer.DefaultRoleEditor.superclass.resetHandler.call(this, button, event);

        if (this.presetData)
        {
          this.getForm().setValues(this.presetData);
        }

        this.el.unmask();
      },
      loadHandler : function() {
        if (!this.payload.data.userManaged)
        {
          this.find('name', 'internalResourceHeader')[0].setVisible(true);
          this.find('name', 'id')[0].disable();
          this.find('name', 'name')[0].disable();
          this.find('name', 'description')[0].disable();

          var roleManager = this.find('name', 'roleManager')[0];
          roleManager.disable();
          roleManager.clearValidation();

          for (var i = 0; i < this.buttons.length; i++)
          {
            this.buttons[i].disable();
          }
        }
      },
      loadData : function(form, action, receivedData) {
        Sonatype.repoServer.DefaultRoleEditor.superclass.loadData.call(this);

        if (this.presetData)
        {
          this.getForm().setValues(this.presetData);
        }
      },
      isValid : function() {
        return this.form.isValid() && this.find('name', 'roleManager')[0].validate();
      },
      submitHandler : function(form, action, receivedData) {
        receivedData.mapping = this.payload.data.mapping;
      },
      validationModifiers : { 'roles' : function(error,panel) { panel.find('name', 'roleManager')[0].markInvalid(error.msg); } }
    });

Sonatype.Events.addListener('roleViewInit', function(cardPanel, rec, gridPanel) {
      var config = {
        payload : rec,
        tabTitle : 'Configuration'
      };

      if (rec.data.userManaged == false)
      {}
      else
      {}

      cardPanel.add(new Sonatype.repoServer.DefaultRoleEditor(config));
    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Repository Maintenance panel layout and controller
 */

define('repoServer/RepoMaintPanel',['Sonatype/all', 'Nexus/repository/AbstractRepoPanel', 'Sonatype/strings'], function(Sonatype, AbstractRepoPanel, Strings){
/*
 * config options: { id: the is of this panel instance [required] title: title
 * of this panel (shows in tab) }
 */

Sonatype.repoServer.RepositoryPanel = function(config) {
  var config = config || {};
  var defaultConfig = {
    title : 'Repositories'
  };
  Ext.apply(this, config, defaultConfig);

  var sp = Sonatype.lib.Permissions;

  this.repoStatusTask = {
    run : function() {
      if (sp.checkPermission('nexus:repostatus', sp.READ))
      {
        Ext.Ajax.request({
              url : Sonatype.config.repos.urls.repositoryStatuses + (this.forceStatuses ? '?forceCheck' : ''),
              callback : this.statusCallback,
              scope : this
            });
      }
      this.forceStatuses = false;
    },
    interval : 5000, // poll every 5 seconds
    scope : this
  };

  this.browseTypeButton = new Ext.Button({
        text : 'User Managed Repositories',
        icon : Sonatype.config.resourcePath + '/static/images/icons/page_white_stack.png',
        cls : 'x-btn-text-icon',
        value : 'user',
        tooltip : 'Click to browse other types of repositories.',
        scope : this,
        menu : {
          items : [{
                text : 'User Managed Repositories',
                value : 'user',
                checked : true,
                group : 'repo-view-selector',
                scope : this,
                handler : this.switchBrowseType
              }, {
                text : 'Nexus Managed Repositories',
                value : 'nexus',
                checked : false,
                group : 'repo-view-selector',
                scope : this,
                handler : this.switchBrowseType
              }]
        }
      });

  this.groupStore = new Ext.data.JsonStore({
        root : 'data',
        id : 'resourceURI',
        fields : [{
              name : 'id'
            }, {
              name : 'name',
              sortType : Ext.data.SortTypes.asUCString
            }, {
              name : 'repoType',
              defaultValue : 'group'
            }, {
              name : 'exposed',
              type : 'boolean',
              defaultValue : true
            }, {
              name : 'userManaged',
              type : 'boolean',
              defaultValue : true
            }, {
              name : 'resourceURI'
            }, {
              name : 'format'
            }, {
              name : 'policy'
            }, {
              name : 'status'
            }, {
              name : 'repositories'
            }, {
              name : 'displayStatus',
              mapping : 'repositories',
              convert : function(v) {
                return Sonatype.utils.joinArrayObject(v, 'name');
              }
            }, {
              name : 'contentResourceURI'
            }],
        sortInfo : {
          field : 'name',
          direction : 'asc'
        },
        url : Sonatype.config.repos.urls.groups,
        listeners : {
          load : function(store, records, options) {
            switch (this.browseTypeButton.value)
            {
              case 'nexus' :
                store.filterBy(function(rec, id) {
                      return !rec.get("userManaged");
                    }, this);
                break;
              case 'user' :
                store.filterBy(function(rec, id) {
                      return rec.get("userManaged");
                    }, this);
                break;
            }
            this.dataStore.insert(0, store.data.items);
          },
          scope : this
        }
      });

  var toolbar = [];
  if (sp.checkPermission('nexus:wastebasket', sp.DELETE))
  {
    toolbar.push({
          id : 'repo-trash-btn',
          text : 'Trash...',
          icon : Sonatype.config.resourcePath + '/static/images/icons/user-trash.png',
          cls : 'x-btn-text-icon',
          tooltip : {
            title : 'Trash',
            text : 'Manage the Trash contents'
          },
          menu : {
            width : 125,
            items : [{
                  text : 'Empty Trash',
                  handler : this.deleteTrashHandler,
                  scope : this
                }]
          }
        });
  }
  toolbar.push(this.browseTypeButton);

  Sonatype.Events.addListener('nexusRepositoryStatus', this.statusStart, this);
  
  var columns = [{
      name : 'resourceURI'
  }, {
    name : 'remoteUri'
  }, {
    name : 'id'
  }, {
    name : 'exposed'
  }, {
    name : 'userManaged'
  }, {
    name : 'status'
  }, {
    name : 'name',
    sortType : Ext.data.SortTypes.asUCString,
    header : 'Repository',
    width : 175,
    sortable : true,
    renderer : function(value, metadata, record, rowIndex, colIndex, store) {
      return record.get('repoType') == 'group' ? ('<b>' + value + '</b>') : value;
    }
  }, {
    name : 'repoType',
    sortable : true,
    header : 'Type',
    width : 50
  }, {
    name : 'format',
    sortable : true,
    header : 'Format',
    width : 70
  }, {
    name : 'repoPolicy',
    sortable : true,
    header : 'Policy',
    width : 70,
    convert : Strings.upperFirstCharLowerRest
  }, {
    name : 'displayStatus',
    sortable : true,
    header : 'Repository Status',
    mapping : 'status',
    convert : Sonatype.repoServer.DefaultRepoHandler.statusConverter,
    width : 200
  }, {
    name : 'contentResourceURI',
    header : 'Repository Path',
    autoExpand : true,
    renderer : function(s) {
      if ( typeof s !== 'string' ) {
        return '';
      } else if (/^http(s)?:/.test(s)) {
        return '<a href="' + s + ((s != null && (s.charAt(s.length)) == '/') ? '' : '/') + '" target="_blank">' + s + '</a>';
      } else {
        return '' + s;
      }
    }
  }];
  
  //allow plugins to contribute to the columns and toolbar
  Sonatype.Events.fireEvent('repositoryGridInit', this, columns, toolbar);

  Sonatype.repoServer.RepositoryPanel.superclass.constructor.call(this, {
        emptyText: 'No repositories defined',
        emptyTextWhileFiltering: 'No repositories matched criteria: {criteria}',
        addMenuInitEvent : 'repositoryAddMenuInit',
        deleteButton : sp.checkPermission('nexus:repositories', sp.DELETE),
        rowClickEvent : 'repositoryViewInit',
        rowContextClickEvent : 'repositoryMenuInit',
        url : Sonatype.config.repos.urls.repositories,
        dataAutoLoad : false,
        tabbedChildren : true,
        tbar : toolbar,
        columns : columns
      });
  
  //allow plugins to access the properly built panel object
  Sonatype.Events.fireEvent('repositoryPanelInit', this);

  this.addListener('beforedestroy', function() {
        Ext.TaskMgr.stop(this.repoStatusTask);
        Sonatype.Events.removeListener('nexusRepositoryStatus', this.statusStart, this);
      }, this);
  this.dataStore.addListener('load', this.onRepoStoreLoad, this);
  this.dataStore.addListener('loadexception', this.onRepoStoreLoadException, this);
  this.dataStore.load();

  this.updatingBookmark = false;
  this.updating = new Ext.util.DelayedTask(function() {
        this.updatingBookmark = false;
      }, this, [this]);
  this.currentBookmark = [];
  this.cardPanel.on('afterlayout', function() {
        var tpanel = this.cardPanel.getLayout().activeItem.tabPanel;
        if (tpanel)
        {
          tpanel.on('tabchange', function(panel, tab) {
                if (!this.updatingBookmark)
                {
                  this.currentBookmark[1] = tab.name;
                  if (tab.name != 'browsestorage')
                  {
                    this.currentBookmark[2] = null;
                  }
                  Sonatype.utils.updateHistory(this);
                }
              }, this);
        }
      }, this);
  Sonatype.Events.addListener('repoBrowserStartSearch', function(text) {
        if (!this.updatingBookmark)
        {
          this.currentBookmark[2] = text;
          Sonatype.utils.updateHistory(this);
        }
      }, this);

  Sonatype.Events.addListener('repoBrowserStopSearch', function() {
        if (!this.updatingBookmark)
        {
          this.currentBookmark[2] = null;
          Sonatype.utils.updateHistory(this);
        }
      }, this);

};

Ext.extend(Sonatype.repoServer.RepositoryPanel, Sonatype.panels.GridViewer, {

	      deleteRecord : function(rec) {
        Ext.Ajax.request({
              callback : function(options, success, response) {
                if (success)
                {
                  this.dataStore.remove(rec);
                }
                else
                {
                	var options = {
                		hideErrorStatus : true
                	};
                  Sonatype.utils.connectionError(response, 'Delete request could not be completed!', false, options);
                }
              },
              scope : this,
              method : 'DELETE',
              url : rec.data.resourceURI
            });
      },

      applyBookmark : function(bookmark) {
        this.updatingBookmark = true;

        if (!this.groupStore.lastOptions)
        {
          this.groupStore.on('load', function(store, recs, options) {
                this.selectBookmarkedItem(bookmark);
              }, this, {
                single : true
              });

          this.groupStore.on('loadexception', function(store, recs, options) {
                this.selectBookmarkedItem(bookmark);
              }, this, {
                single : true
              });
        }
        else
        {
          this.selectBookmarkedItem(bookmark);
        }
      },

      selectBookmarkedItem : function(bookmark) {
        var parts = decodeURIComponent(bookmark).split('~');
        this.currentBookmark = parts;

        if (parts && parts.length > 0)
        {
          var recIndex = this.dataStore.findBy(function(rec, id) {
                return rec.data[this.dataBookmark] == parts[0];
              }, this);

          var selected = null;

          if (recIndex >= 0)
          {
            var rec = this.gridPanel.getSelectionModel().getSelected();
            var toSelect = this.dataStore.getAt(recIndex);
            if (rec == null || rec.id != toSelect.id)
            {
              selected = toSelect;
              this.gridPanel.getSelectionModel().selectRecords([toSelect]);
            }
          }

          if (parts.length == 1)
          {
            parts[1] = 'browsestorage';
            Sonatype.utils.replaceHistory(this);
          }

          if (parts.length > 1)
          {
            var tab, panel = this.cardPanel.getLayout().activeItem.tabPanel;

            if ( panel ) {
              tab = panel.find('name', parts[1])[0];

              if (tab)
              {
                panel.setActiveTab(tab);
              }
              else if (Sonatype.user.curr.isLoggedIn)
              {
                this.currentBookmark[1] = panel.getActiveTab().name;
                Sonatype.utils.replaceHistory(this);
              }
              else
              {
                var token = Ext.History.getToken();
                Sonatype.view.historyDisabled = false;
                Sonatype.view.afterLoginToken = token;
                Sonatype.repoServer.RepoServer.loginHandler();
                this.currentBookmark[1] = panel.getActiveTab().name;
                return;
              }
            }

            if (parts && parts.length > 2 && parts[1] == 'browsestorage' && toSelect != null)
            {
              var repoBrowser = panel.find('name', 'repositoryBrowser')[0];
              repoBrowser.selectPath('/' + toSelect.data.name + parts[2], 'text', function(success, node) {
                    if (success)
                    {
                      if (node.ownerTree.nodeClickEvent)
                      {
                        Sonatype.Events.fireEvent(node.ownerTree.nodeClickEvent, node, node.ownerTree.nodeClickPassthru);
                      }
                    }
                  });
            }
          }
        }
        this.updating.delay(200);
      },

      getBookmark : function() {
        var bookmark = Sonatype.repoServer.RepositoryPanel.superclass.getBookmark.call(this);
        if (bookmark == null)
        {
          return bookmark;
        }

        this.currentBookmark[0] = bookmark;

        bookmark = '';

        for (var i = 0; i < this.currentBookmark.length; i++)
        {
          if (!this.currentBookmark[i])
          {
            break;
          }

          if (i > 0)
          {
            bookmark += '~';
          }

          bookmark += this.currentBookmark[i];
        }

        return bookmark;
      },

      deleteTrashHandler : function(button, e) {
        Sonatype.utils.defaultToNo();

        Sonatype.MessageBox.show({
              animEl : this.gridPanel,
              title : 'Empty Trash',
              msg : 'Delete the entire contents of the Trash?<br><br>This operation cannot be undone!',
              buttons : Sonatype.MessageBox.YESNO,
              scope : this,
              icon : Sonatype.MessageBox.QUESTION,
              fn : function(btnName) {
                if (btnName == 'yes' || btnName == 'ok')
                {
                  Ext.Ajax.request({
                        callback : function(options, success, response) {
                          if (!success)
                          {
                            Sonatype.utils.connectionError(response, 'Error emptying the trash!');
                          }
                        },
                        scope : this,
                        method : 'DELETE',
                        url : Sonatype.config.repos.urls.trash
                      });
                }
              }
            });
      },

      onRepoStoreLoadException : function() {
        this.onRepoStoreLoad(null, null, null);
      },

      onRepoStoreLoad : function(store, records, options) {
        switch (this.browseTypeButton.value)
        {
          case 'nexus' :
            if (records == null)
            {
              this.dataStore.removeAll();
            }
            else
            {
              for (var i = 0; i < records.length; i++)
              {
                if (records[i].data.userManaged)
                {
                  store.remove(records[i]);
                }
              }
            }
            break;
          case 'user' :
            if (records == null)
            {
              this.dataStore.removeAll();
            }
            this.groupStore.reload();
            break;
        }
        this.statusStart();
      },

      refreshHandler : function(button, e) {
        if (button == this.refreshButton)
        {
          this.forceStatuses = true;
        }
        Sonatype.repoServer.RepositoryPanel.superclass.refreshHandler.call(this, button, e);
      },

      statusCallback : function(options, success, response) {
        if (response.status != 202)
        {
          Ext.TaskMgr.stop(this.repoStatusTask);
        }

        if (success)
        {
          var statusResp = Ext.decode(response.responseText);
          if (statusResp.data)
          {
            var data = statusResp.data;
            for (var i = data.length - 1; i >= 0; i--)
            {
              var item = data[i];
              var rec = this.dataStore.getById(item.resourceURI.replace(Sonatype.config.repos.urls.repositoryStatuses, Sonatype.config.repos.urls.repositories));
              if (rec)
              {
                rec.beginEdit();
                rec.set('status', item.status);
                rec.set('displayStatus', Sonatype.repoServer.DefaultRepoHandler.statusConverter(item.status, item));
                rec.commit(true);
                rec.endEdit();
              }
            }
          }
          else
          {
            // no content, clear display statuses
            this.dataStore.each(function(rec) {
              if (rec.get('repoType') !== 'group') {
                rec.beginEdit();
                rec.set('displayStatus', '');
                rec.commit(true);
                rec.endEdit();
              }
            });
          }
        }
        else
        {
          this.dataStore.each(function(rec) {
            if ( rec.get('repoType') !== 'group' ) {
              rec.beginEdit();
              rec.set('status', {localStatus : 'IN_SERVICE' , proxyMode : 'ALLOW', remoteStatus:'UNKNOWN'});
              rec.set('displayStatus', '<i>Status retrieval failed</i>');
              rec.commit(true);
              rec.endEdit();
            }
          });
        }

        this.gridPanel.getView().refresh();
      },

      statusStart : function() {
        Ext.TaskMgr.start(this.repoStatusTask);
      },

      switchBrowseType : function(button, e) {
        this.browseTypeButton.setText(button.text);
        this.browseTypeButton.value = button.value;

        switch (button.value)
        {
          case 'nexus' :
            if (this.toolbarAddButton)
              this.toolbarAddButton.disable();
            if (this.toolbarDeleteButton)
              this.toolbarDeleteButton.disable();
            this.dataStore.proxy.setApi(Ext.data.Api.actions.read, Sonatype.config.repos.urls.allRepositories);
            break;
          case 'user' :
            if (this.toolbarAddButton)
              this.toolbarAddButton.enable();
            if (this.toolbarDeleteButton)
              this.toolbarDeleteButton.enable();
            this.dataStore.proxy.setApi(Ext.data.Api.actions.read, Sonatype.config.repos.urls.repositories);
            break;
        }
        this.refreshHandler(button, e);
      }

    });

Sonatype.repoServer.RepositoryBrowsePanel = function(config) {
  var config = config || {};
  var defaultConfig = {
    titleColumn : 'name'
  };
  Ext.apply(this, config, defaultConfig);

  this.oldSearchText = '';
  this.searchTask = new Ext.util.DelayedTask(this.startSearch, this, [this]);
  this.nodeContextMenuEvent = 'repositoryContentMenuInit';

  this.searchField = new Ext.app.SearchField({
        xtype : 'nexussearchfield',
        searchPanel : this,
        width : 400,
        enableKeyEvents : true,
        listeners : {
          'keyup' : {
            fn : function(field, event) {
              var key = event.getKey();
              if (!event.isNavKeyPress())
              {
                this.searchTask.delay(200);
              }
            },
            scope : this
          },
          'render' : function(c) {
            Ext.QuickTips.register({
                  target : c.getEl(),
                  text : 'Enter a complete path to lookup, for example org/sonatype/nexus'
                });
          }
        }
      });

  Sonatype.repoServer.RepositoryBrowsePanel.superclass.constructor.call(this, {
        anchor : '0 -2',
        bodyStyle : 'background-color:#FFFFFF',
        animate : true,
        lines : false,
        autoScroll : true,
        containerScroll : true,
        rootVisible : true,
        enableDD : false,
        tbar : [{
              text : 'Refresh',
              iconCls : 'st-icon-refresh',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.refreshHandler
            }, ' ', 'Path Lookup:', this.searchField],
        loader : new Ext.tree.SonatypeTreeLoader({
              url : '',
              listeners : {
                loadexception : this.treeLoadExceptionHandler,
                scope : this
              }
            }),
        listeners : {
          click : this.nodeClickHandler,
          contextMenu : this.nodeContextMenuHandler,
          scope : this
        }
      });

  new Ext.tree.TreeSorter(this, {
        folderSort : true
      });

  this.refreshHandler();
};

Ext.extend(Sonatype.repoServer.RepositoryBrowsePanel, Ext.tree.TreePanel, {
      getBrowsePath : function(baseUrl) {
        return baseUrl + this.getBrowsePathSnippet() + '/';
      },

      getBrowsePathSnippet : function() {
        return Sonatype.config.browsePathSnippet;
      },

      nodeClickHandler : function(node, e) {
        if (e.target.nodeName == 'A')
          return; // no menu on links

        if (this.nodeClickEvent)
        {
          Sonatype.Events.fireEvent(this.nodeClickEvent, node, this.nodeClickPassthru);
        }
      },

      nodeContextMenuHandler : function(node, e) {
        if (e.target.nodeName == 'A')
          return; // no menu on links

        if (!this.payload.data.showCtx)
        {
          return;
        }

        if (this.nodeContextMenuEvent)
        {

          node.attributes.repoRecord = this.payload;
          node.data = node.attributes;

          var menu = new Sonatype.menu.Menu({
                payload : node,
                scope : this,
                items : []
              });

          Sonatype.Events.fireEvent(this.nodeContextMenuEvent, menu, this.payload, node);

          var item;
          while ((item = menu.items.first()) && !item.text)
          {
            menu.remove(item); // clean up if the first element is a separator
          }
          while ((item = menu.items.last()) && !item.text)
          {
            menu.remove(item); // clean up if the last element is a separator
          }
          if (!menu.items.first())
            return;

          e.stopEvent();
          menu.showAt(e.getXY());
        }
      },

      refreshHandler : function(button, e) {
        if (this.payload)
        {
          this.setRootNode(new Ext.tree.AsyncTreeNode({
                text : this.payload.data[this.titleColumn],
                id : this.getBrowsePath(this.payload.data.resourceURI),
                singleClickExpand : true,
                expanded : true,
                listeners : {
                  expand : {
                    fn : function() {
                      if (this.payload.data.expandPath)
                      {
                        this.selectPath(this.payload.data.expandPath, 'text', function(success, node) {
                              if (success)
                              {
                                if (node.ownerTree.nodeClickEvent)
                                {
                                  Sonatype.Events.fireEvent(node.ownerTree.nodeClickEvent, node, node.ownerTree.nodeClickPassthru);
                                }
                              }
                            });
                      }
                    },
                    scope : this
                  }
                }
              }));
        }
        else
        {
          this.setRootNode(new Ext.tree.TreeNode({
                text : '(Not Available)',
                id : '/',
                singleClickExpand : true,
                expanded : true
              }));
        }

        if (this.innerCt)
        {
          // need to check if root node is reloadable and not 'Not Available'
          // (may happen on closing staging repositories, which use this view)
          if ( this.root.reload ) {
            this.root.reload();
          }
          this.afterRender();
        }
      },

      updatePayload : function(payload) {
        this.payload = payload;
        this.refreshHandler();
      },

      startSearch : function(p) {
        var field = p.searchField;
        var searchText = field.getRawValue();

        Sonatype.Events.fireEvent('repoBrowserStartSearch', searchText);

        if (searchText.charAt(0) == '/')
        {
          searchText = searchText.slice(1, searchText.length);
        }

        var treePanel = p;
        if (searchText)
        {
          field.triggers[0].show();
          var justEdited = p.oldSearchText.length > searchText.length;

          var findMatchingNodes = function(root, textToMatch) {
            var n = textToMatch.indexOf('/');
            var remainder = '';
            if (n > -1)
            {
              remainder = textToMatch.substring(n + 1);
              textToMatch = textToMatch.substring(0, n);
            }

            var matchingNodes = [];
            var found = false;
            for (var i = 0; i < root.childNodes.length; i++)
            {
              var node = root.childNodes[i];

              var text = node.text;
              if (text == textToMatch)
              {
                node.enable();
                node.ensureVisible();
                node.expand();
                found = true;
                if (!node.isLeaf())
                {
                  var autoComplete = false;
                  if (!remainder && node.childNodes.length == 1)
                  {
                    remainder = node.firstChild.text;
                    autoComplete = true;
                  }
                  if (remainder)
                  {
                    if (node.expanded)
                    {
                      var s = findMatchingNodes(node, remainder);
                      if (autoComplete || (s && s != remainder))
                      {
                        return textToMatch + '/' + (s ? s : remainder);
                      }
                    }
                    else
                    {
                      node.on('expand', function(node) {
                            findMatchingNodes(node, remainder)
                          }, this);

                    }

                  }
                }
              }
              else if (text.substring(0, textToMatch.length) == textToMatch)
              {
                matchingNodes[matchingNodes.length] = node;
                node.enable();
                if (matchingNodes.length == 1)
                {
                  node.ensureVisible();
                }
              }
              else
              {
                node.disable();
                node.collapse(false, false);
              }
            }

            // if only one non-exact match found, suggest the name
            return !found && matchingNodes.length == 1 ? matchingNodes[0].text + '/' : null;
          };

          var s = findMatchingNodes(treePanel.root, searchText);

          p.oldSearchText = searchText;

          // if auto-complete is suggested, and the user hasn't just started
          // deleting
          // their own typing, try the suggestion
          if (s && !justEdited && s != searchText)
          {
            field.setRawValue(s);
            p.startSearch(p);
          }

        }
        else
        {
          p.stopSearch(p);
        }
      },

      stopSearch : function(p) {
        p.searchField.triggers[0].hide();
        p.oldSearchText = '';

        Sonatype.Events.fireEvent('repoBrowserStopSearch');

        var treePanel = p;

        var enableAll = function(root) {
          for (var i = 0; i < root.childNodes.length; i++)
          {
            var node = root.childNodes[i];
            node.enable();
            node.collapse(false, false);
            enableAll(node);
          }
        };
        enableAll(treePanel.root);
      },

      treeLoadExceptionHandler : function(treeLoader, node, response) {
        if (response.status == 503)
        {
          if (Sonatype.MessageBox.isVisible())
          {
            Sonatype.MessageBox.hide();
          }
          node.setText(node.text + ' (Out of Service)');
        }
        else if (response.status == 404 || response.status == 400)
        {
          if (Sonatype.MessageBox.isVisible())
          {
            Sonatype.MessageBox.hide();
          }
          node.setText(node.text + (node.isRoot ? ' (Not Available)' : ' (Not Found)'));
        }
        else if (response.status == 401 || response.status == 403)
        {
          if (Sonatype.MessageBox.isVisible())
          {
            Sonatype.MessageBox.hide();
          }
          node.setText(node.text + ' (Access Denied)');
        }
      }

    });
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('repoServer/RepoSummaryPanel',['Sonatype/all'], function(){
/*global Ext,Sonatype,Nexus*/
Sonatype.repoServer.AbstractRepositorySummaryPanel = function(cfg) {
  var config = cfg || {};
  var defaultConfig = {};

  Ext.apply(this, config, defaultConfig);

  var abstractItems = [{
        xtype : 'textfield',
        fieldLabel : 'Repository Information',
        hidden : true
      }, {
        xtype : 'textarea',
        name : 'informationField',
        anchor : Sonatype.view.FIELD_OFFSET,
        readOnly : true,
        hideLabel : true,
        height : 150
      }];

  Sonatype.repoServer.AbstractRepositorySummaryPanel.superclass.constructor.call(this, {
        uri : this.payload.data.resourceURI + '/meta',
        readOnly : true,
        dataModifiers : {
          load : {
            'rootData' : this.populateFields.createDelegate(this)
          },
          save : {}
        },
        items : abstractItems.concat(this.items)
      });
};

Ext.extend(Sonatype.repoServer.AbstractRepositorySummaryPanel, Sonatype.ext.FormPanel, {
      getActionURL : function() {
        return this.uri;
      },
      populateFields : function(arr, srcObj, fpanel) {
        this.populateInformationField('Repository ID: ' + srcObj.id + '\n');
        this.populateInformationField('Repository Name: ' + this.payload.data.name + '\n');
        this.populateInformationField('Repository Type: ' + this.payload.data.repoType + '\n');
        this.populateInformationField('Repository Policy: ' + this.payload.data.repoPolicy + '\n');
        this.populateInformationField('Repository Format: ' + this.payload.data.format + '\n');
        this.populateInformationField('Contained in groups: ' + '\n' + this.combineGroups(srcObj.groups) + '\n');
      },
      populateInformationField : function(text) {
        var infoText = this.find('name', 'informationField')[0].getValue();
        infoText += text;
        this.find('name', 'informationField')[0].setRawValue(infoText);
      },
      combineGroups : function(groups) {
        var combinedGroups = '';
        if (groups != undefined && groups.length > 0)
        {
          for (var i = 0; i < groups.length; i++)
          {
            var group = this.groupStore.getAt(this.groupStore.findBy(function(rec, recid) {
                  return rec.data.id == groups[i];
                }, this));

            if (group)
            {
              if (combinedGroups.length > 0)
              {
                combinedGroups += '\n';
              }

              combinedGroups += '   ' + group.data.name;
            }
          }
        }

        return combinedGroups;
      }
    });

Sonatype.repoServer.HostedRepositorySummaryPanel = function(config) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  Sonatype.repoServer.HostedRepositorySummaryPanel.superclass.constructor.call(this, {
        items : [{
              xtype : 'textfield',
              fieldLabel : 'Distribution Management',
              hidden : true
            }, {
              xtype : 'textarea',
              name : 'distMgmtField',
              anchor : Sonatype.view.FIELD_OFFSET,
              readOnly : true,
              hideLabel : true,
              height : 100
            }]
      });
};

Ext.extend(Sonatype.repoServer.HostedRepositorySummaryPanel, Sonatype.repoServer.AbstractRepositorySummaryPanel, {
      populateFields : function(arr, srcObj, fpanel) {
        Sonatype.repoServer.HostedRepositorySummaryPanel.superclass.populateFields.call(this, arr, srcObj, fpanel);

        this.populateDistributionManagementField(this.payload.data.id, this.payload.data.repoPolicy, this.payload.data.contentResourceURI, this.payload.data.format);
      },
      populateDistributionManagementField : function(id, policy, uri, format) {
        if (['maven1', 'maven2'].indexOf(format) > -1) {
          var distMgmtString = '<distributionManagement>\n  <${repositoryType}>\n    <id>${repositoryId}</id>\n    <url>${repositoryUrl}</url>\n  </${repositoryType}>\n</distributionManagement>';

          distMgmtString = distMgmtString.replaceAll('${repositoryType}',
              policy == 'Release' ? 'repository' : 'snapshotRepository');
          distMgmtString = distMgmtString.replaceAll('${repositoryId}', id);
          distMgmtString = distMgmtString.replaceAll('${repositoryUrl}', uri);

          this.find('name', 'distMgmtField')[0].setRawValue(distMgmtString);
          this.find('name', 'distMgmtField')[0].show();
        }
        else {
          this.find('name', 'distMgmtField')[0].hide();
        }
      }
    });

Sonatype.repoServer.ProxyRepositorySummaryPanel = function(config) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  Sonatype.repoServer.ProxyRepositorySummaryPanel.superclass.constructor.call(this, {
        items : []
      });
};

Ext.extend(Sonatype.repoServer.ProxyRepositorySummaryPanel, Sonatype.repoServer.AbstractRepositorySummaryPanel, {
      populateFields : function(arr, srcObj, fpanel) {
        Sonatype.repoServer.ProxyRepositorySummaryPanel.superclass.populateFields.call(this, arr, srcObj, fpanel);

        var remoteUri = this.payload.data.remoteUri;

        if (remoteUri == undefined)
        {
          remoteUri = this.payload.data.remoteStorage.remoteStorageUrl;
        }

        this.populateInformationField('Remote URL: ' + remoteUri + '\n');
      }
    });

Sonatype.repoServer.VirtualRepositorySummaryPanel = function(config) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  Sonatype.repoServer.VirtualRepositorySummaryPanel.superclass.constructor.call(this, {
        items : []
      });
};

Ext.extend(Sonatype.repoServer.VirtualRepositorySummaryPanel, Sonatype.repoServer.AbstractRepositorySummaryPanel, {});

Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec, gridPanel) {
      var sp = Sonatype.lib.Permissions;

      var repoPanels = {
        hosted : Sonatype.repoServer.HostedRepositorySummaryPanel,
        proxy : Sonatype.repoServer.ProxyRepositorySummaryPanel,
        virtual : Sonatype.repoServer.VirtualRepositorySummaryPanel
      };

      var panel = repoPanels[rec.data.repoType];

      if (panel && rec.data.resourceURI && sp.checkPermission('nexus:repometa', sp.READ))
      {
        cardPanel.add(new panel({
              tabTitle : 'Summary',
              name : 'summary',
              payload : rec,
              groupStore : gridPanel.groupStore
            }));
      }
    });
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('repoServer/RepositoryBrowserContainer',['Sonatype/repoServer/ArtifactContainer'], function(){
// This container will host both the repository browser and the artifact
// information panel
Sonatype.repoServer.RepositoryBrowserContainer = function(config) {
  var config = config || {};
  var defaultConfig = {
    artifactContainerInitEvent : 'fileContainerInit',
    artifactContainerUpdateEvent : 'fileContainerUpdate'
  };
  Ext.apply(this, config, defaultConfig);

  var items = [];

  this.repositoryBrowser = new Sonatype.repoServer.RepositoryBrowsePanel({
        name : 'repositoryBrowser',
        payload : this.payload,
        tabTitle : this.tabTitle,
        browseIndex : false,
        region : 'center',
        nodeClickEvent : 'fileNodeClickedEvent',
        nodeClickPassthru : {
          container : this
        }
      });

  this.artifactContainer = new Sonatype.repoServer.ArtifactContainer({
        collapsible : true,
        collapsed : true,
        region : 'east',
        split : true,
        width : '50%',
        initEventName : this.artifactContainerInitEvent,
        updateEventName : this.artifactContainerUpdateEvent
      });

  items.push(this.repositoryBrowser);
  items.push(this.artifactContainer);

  Sonatype.repoServer.RepositoryBrowserContainer.superclass.constructor.call(this, {
        layout : 'border',
        // this hideMode causes the tab to properly render when coming back from
        // hidden
        hideMode : 'offsets',
        items : items
      });
};

Ext.extend(Sonatype.repoServer.RepositoryBrowserContainer, Ext.Panel, {
      updatePayload : function(payload) {

        if (payload == null)
        {
          this.collapse();
          this.repositoryBrowser.updatePayload(null);
          this.artifactContainer.collapsePanel();
        }
        else
        {
          this.expand();
          this.repositoryBrowser.updatePayload(payload);
          this.artifactContainer.updateArtifact(payload);
        }

      }
    });

// Add the browse storage and browse index panels to the repo
Sonatype.Events.addListener('repositoryViewInit', function(cardPanel, rec) {
      if (rec.data.resourceURI
          && (['npm'].indexOf(rec.data.format) == -1))
      {
        cardPanel.add(new Sonatype.repoServer.RepositoryBrowserContainer({
              payload : rec,
              name : 'browsestorage',
              tabTitle : 'Browse Storage'
            }));
      }
    });

Sonatype.Events.addListener('fileNodeClickedEvent', function(node, passthru) {
      if (passthru && passthru.container && passthru.container.artifactContainer.items.getCount() > 0)
      {
        if (node && node.isLeaf())
        {
          var payload = passthru.container.payload;
          
          passthru.container.artifactContainer.updateArtifact({
                format : (payload && payload.data) ? payload.data.format : null,
                repoId : (payload && payload.data) ? payload.data.id : null,
                text : node.attributes.text,
                leaf : node.attributes.leaf,
                resourceURI : node.attributes.resourceURI
              });
        }
        else
        {
          passthru.container.artifactContainer.collapse();
        }
      }
    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Help text resource file for Nexus resource forms Dependencies:
 * Ext.form.Field.AfterRender override Instructions: To render the "?" next to
 * field label give tha associated value in this config a non-empty string
 * value. Fields with empty strings will not have the help "?" rendered next to
 * the field label.
 */

define('repoServer/resources',['Sonatype/all'], function(){
(function() {

  // Nexus default help text values for re-use in child forms
  var userAgentString = 'A custom fragment to add to the "user-agent" string used in HTTP requests.';
  var queryString = 'These are additional parameters sent along with the HTTP request. They are appended to the url along with a \'?\'. So \'foo=bar&foo2=bar2\' becomes \'HTTP://myurl?foo=bar&foo2=bar2\'';
  var connectionTimeout = 'Time Nexus will wait for a successful connection before retrying. (seconds)';
  var retrievalRetryCount = 'Nexus will make this many connection attempts before giving up.';

  var proxyHostname = 'This is the host name of the HTTP proxy used for remote connections. (no HTTP/HTTPs required...just the host or ip)';
  var proxyPort = 'This is the port number of the HTTP proxy used for remote connections.';
  var nonProxyHosts = 'List of host names to exclude from http proxy.<br/>Regular expressions are supported; example: <code>.*\\.somecompany\\.com</code>';

  var username = 'The username used for authentication to the HTTP proxy.';
  var password = 'The password used for authentication to the HTTP proxy.';
  var privateKey = 'The ssl private key used for authentication to the HTTP proxy.';
  var passphrase = 'The passphrase for the private key.';
  var ntlmHost = 'The Windows NT Lan Manager for authentication.';
  var ntlmDomain = 'The Windows NT Lan Manager domain for authentication.';

  Sonatype.repoServer.resources.help = {

    // Server Config help text
    server : {
      anonUsername : 'The username that will be used to authenticate anonymous users against the security realm.',
      anonPassword : 'The password that will be used to authenticate anonymous users against the security realm.',
      anonymousAccess : 'These fields are used to authenticate anonymous requests. When a request comes in without credentials, Nexus uses the Anonymous Username and Anonymous Password field as a substitute and pass it through the security subsystem. This is used in conjunction with third party security realm integration. For example if you were authenticating against Active Directory, the username might be "Guest" instead of anonymous. If you are using the default Nexus security realm, then these shouldn\'t be changed.',
      systemNotification: 'Nexus can be used to notify users (by email address, or users that are part of a specified role) when one of the following actions is performed:<br><ul><li>&nbsp;&nbsp;Nexus automatically blocks a proxy repository because the remote is unreachable.</li><li>&nbsp;&nbsp;Nexus automatically unblocks a proxy repository because the remote is again reachable.</li></ul>',
      baseUrl : 'This is the Base URL of the Nexus web application.  i.e. http://localhost:8081/nexus',
      forceBaseUrl : 'If checked, this will force all URLs to be built with a Base URL. Otherwise, Base URL will only be used in emails and rss feeds, with other URLs built based on the incoming request.',

      // use default nexus text
      userAgentString : userAgentString,
      queryString : queryString,
      connectionTimeout : connectionTimeout,
      retrievalRetryCount : retrievalRetryCount,

      proxyHostname : proxyHostname,
      proxyPort : proxyPort,
      nonProxyHosts : nonProxyHosts,

      username : username,
      password : password,
      privateKey : privateKey,
      passphrase : passphrase,
      ntlmHost : ntlmHost,
      ntlmDomain : ntlmDomain,

      smtphost : 'The host name of an SMTP server.',
      smtpport : 'The port the SMTP server is listening on.',
      smtpuser : 'The username used to access the SMTP server.',
      smtppass : 'The password used to access the SMTP server.',
      connection : 'Connection level security to be used with SMTP server. Use any of the SSL/TLS provided solutions for greater security.',
      smtpsysemail : 'Default System email address.  This is who the "From" address will be.',
      jiraUsername : 'The Username of a JIRA account.  If left empty default username and password will be used.',
      jiraPassword : 'The Password of a JIRA account.  If left empty default username and password will be used.',
      notificationEmailAddresses : 'A comma seperated list of email addresses to notify.',
      notificationsEnabled : 'Enable the system to send notification messages to the receipients defined below.'
    },

    // Groups Config help text
    groups : {
      id : 'The unique id for the group. This id will become part of the url so it should not contain spaces.',
      name : 'The Group Name which is referenced in the UI and Logs.',
      exposed : 'This controls if the group is published on a URL, if this field is false you will not be able to access this group remotely.'
    },

    // Routes Config help text
    routes : {
      pattern : 'A regular expression used to match the artifact path. ".*" is used to specify all paths. ".*/com/some/company/.*" will match any artifact with "com.some.company" as the group id or artifact id. "^/com/some/company/.*" will match any artifact starting with com/some/company.',
      ruleType : 'There are three types of rules: Inclusive (if the pattern matches, only use the repositories listed below), Exclusive (exclude the repositories listed below) and Blocking (block URLs matching the pattern).',
      group : 'A repository group this route applies to.'
    },

    // Scheduled Services Config help text
    schedules : {
      enabled : 'This flag determines if the task is currently active.  To disable this task for a period of time, de-select this checkbox.',
      name : 'A name for the scheduled task.',
      serviceType : 'The type of service that will be scheduled to run.',
      alertEmail : 'The email address where an email will be sent in case that task execution will fail.',
      serviceSchedule : 'The frequency this task will run.  Manual - this task can only be run manually. Once - run the task once at the specified date/time. Daily - run the task every day at the specified time. Weekly - run the task every week on the specified day at the specified time. Monthly - run the task every month on the specified day(s) and time. Advanced - run the task using the supplied cron string.',
      startDate : 'The date this task should start running.',
      startTime : 'The time this task should start running.',
      recurringTime : 'The time this task should start on days it will run.',
      cronCommand : 'A cron expression that will control the running of the task.'
    },

    // Users help
    users : {
      userId : 'The ID assigned to this user, will be used as the username.',
      firstName : 'The first name of the user.',
      lastName : 'The last name of the user.',
      email : 'Email address, to notify user when necessary.',
      status : 'The current status of the user.',
      roles : 'The roles assigned to this user.',
      password : 'The password required to log the user into the system.',
      reenterPassword : 'Re-enter the password to validate entry.'
    },

    // Roles help
    roles : {
      id : 'The id of this role.',
      name : 'The name of this role.',
      description : 'The description of this role.',
      sessionTimeout : 'The number of minutes to wait before timing out an idle user session.',
      rolesAndPrivileges : 'Roles and privileges contained in this Role.'
    },

    // Privileges help
    privileges : {
      name : 'The name of this privilege.',
      description : 'The description of this privilege.',
      type : 'The type of privilege.  Only "Repository Target" type privileges can be managed by the user.',
      repositoryOrGroup : 'The repository or repository group this privilege will be associated with.',
      repositoryTarget : 'The Repository Target that will be applied with this privilege.'
    },

    // Repositories Config help text
    repos : {
      // shared across types of repositories
      id : 'The unique id for the repository. This id will become part of the url so it should not contain spaces.',
      name : 'The Repository Name which is referenced in the UI and Logs.',
      repoType : 'Nexus supports 3 repository types: Hosted = Normal repository owned by this Nexus instance, Proxy = Retrieve artifacts from the remote repository and store them locally, Virtual = A logical view of another repository configured in Nexus (For example, to provide a Maven 1 view of an existing Maven 2 repository)',
      repoPolicy : 'Repositories can store either all Release artifacts or all Snapshot artifacts.',
      defaultLocalStorageUrl : 'This is the location on the file system used to host the artifacts. It is contained by the Working Directory set in the Server configuration.',
      overrideLocalStorageUrl : 'This is used to override the default local storage. Leave it blank to use the default. Note, file:/{drive-letter}:/ urls are supported in windows.  All other operating systems will use file:// .',
      writePolicy : 'This controls if users are allowed to deploy and/or update artifacts in this repository. (Hosted repositories only)',
      browseable : 'This controls if users can browse the contents of the repository via their web browser.',
      indexable : 'This controls if the artifacts contained by this repository are indexed and thus searchable.',
      exposed : 'This controls if the repository is published on a URL, if this field is false you will not be able to access this repository remotely.',
      notFoundCacheTTL : 'This controls how long to cache the fact that a file was not found in the repository.',
      artifactMaxAge : 'This controls how long to cache the artifacts in the repository before rechecking the remote repository. In a release repository, this value should be -1 (infinite) as release artifacts shouldn\'t change.',
      metadataMaxAge : 'This controls how long to cache the metadata in the repository before rechecking the remote repository. Unlike artifact max age, this value should not be infinite or Maven won\'t discover new artifact releases.',
      itemMaxAge : 'Repositories may contain resources that are neither artifacts identified by GAV coordinates or metadata. This value controls how long to cache such items in the repository before rechecking the remote repository.',
      provider : 'This is the content provider of the repository.',
      format : 'This is the format of the repository.  Maven1 = A Maven 1.x formatted view of the repository.  Maven2 = A Maven 2.x formatted view of the repository.',

      // virtual
      shadowOf : 'This is the id of the physical repository being presented as a logical view by this proxy.',
      syncAtStartup : 'The links are normally updated as changes are made to the repository, if changes may be made while Nexus is offline, the virtual repo should be synchronized at startup.',

      // proxy
      remoteStorageUrl : 'This is the location of the remote repository being proxied. Only HTTP/HTTPs urls are currently supported.',
      downloadRemoteIndexes : 'Indicates if the index stored on the remote repository should be downloaded and used for local searches.',
      autoBlockActive : 'Flag to enable Auto Blocking for this proxy repository. If enabled, Nexus will auto-block outbound connections on this repository if remote peer is detected as unreachable/unresponsive. Auto-blocked repositories will still try to detect remote peer availability, and will auto-unblock the proxy if remote peer detected as reachable/healthy. Auto-blocked repositories behaves exactly the same as user blocked proxy repositories, except they will auto-unblock themselves too.',
      checksumPolicy : 'The checksum policy for this repository: Ignore: Don\'t check remote checksums. Warn: Log a warning if the checksum is bad but serve the artifact anyway. (Default...there are currently known checksum errors on Central). StrictIfExists: Do not serve the artifact if the checksum exists but is invalid. Strict: Require that a checksum exists on the remote repository and that it is valid.',
      fileTypeValidation : 'Flag to check the remote file\'s content to see if it is valid. (e.g. not html error page), handy when you cannot enable strict checksum checking.',
      // remote
      remoteUsername : 'The username used for authentication to the remote repository.',
      remotePassword : 'The password used for authentication to the remote repository.',
      remotePrivateKey : 'The ssl private key used for authentication to the remote repository.',
      remotePassphrase : 'The passphase for the private key.',
      remoteNtlmHost : 'The Windows NT Lan Manager for authentication to the remote repository.',
      remoteNtlmDomain : 'The Windows NT Lan Manager domain for authentication to the remote repository.',

      // proxy override fields
      userAgentString : userAgentString,
      queryString : queryString,
      connectionTimeout : connectionTimeout,
      retrievalRetryCount : retrievalRetryCount,

      proxyHostname : proxyHostname,
      proxyPort : proxyPort,

      username : username,
      password : password,
      privateKey : privateKey,
      passphrase : passphrase,
      ntlmHost : ntlmHost,
      ntlmDomain : ntlmDomain
    },

    // artifact upload help text
    artifact : {
      groupId : 'Group ID',
      artifactId : 'Maven artifact ID',
      version : 'Artifact version',
      packaging : 'Packaging type',
      classifier : 'Classifier of the uploaded artifact.  If not supplied, no classifier will be appended to the artifact.',
      extension : 'Extension of the uploaded artifact.  If not supplied, the default extension (derived from the packaging type) will be used.',
      autoguess : 'Automatically guess the attributes by parsing the artifact file name.',
      description : 'A description of the artifact(s) that will be uploaded, be as descriptive as necessary.',
      gavDefinition : 'How the GAV parameters will be retrieved, either from a POM file or manually entered'
    },

    cronBriefHelp : {
      text : '<p>Provides a parser and evaluator for unix-like cron expressions.</p>' + '<table style="font-size: 11px">' + '<tr><td><b>Example</b></td><td><b>Description</b></td></tr>'
          + '<tr><td><code>0 0 12 * * ?</code></td><td>Fire at 12pm (noon) every day</td></tr>' + '<tr><td><code>0 15 10 ? * *</code></td><td>Fire at 10:15am every day</td></tr>'
          + '<tr><td><code>0 15 10 * * ?</code></td><td>Fire at 10:15am every day</td></tr>' + '<tr><td><code>0 15 10 * * ? *</code></td><td>Fire at 10:15am every day</td></tr>'
          + '<tr><td><code>0 15 10 * * ? 2005</code></td><td>Fire at 10:15am every day during the year 2005</td></tr>' + '<tr><td><code>0 * 14 * * ?</code></td><td>Fire every minute starting at 2pm and ending at 2:59pm, every day</td></tr>'
          + '<tr><td><code>0 0/5 14 * * ?</code></td><td>Fire every 5 minutes starting at 2pm and ending at 2:55pm, every day</td></tr>'
          + '<tr valign="top"><td><code>0 0/5 14,18 * * ?</code></td><td>Fire every 5 minutes starting at 2pm and ending at 2:55pm, AND fire every 5 minutes starting at 6pm and ending at 6:55pm, every day</td></tr>'
          + '<tr><td><code>0 0-5 14 * * ?</code></td><td>Fire every minute starting at 2pm and ending at 2:05pm, every day</td></tr>'
          + '<tr><td><code>0 10,44 14 ? 3 WED</code></td><td>Fire at 2:10pm and at 2:44pm every Wednesday in the month of March.</td></tr>'
          + '<tr><td><code>0 15 10 ? * MON-FRI</code></td><td>Fire at 10:15am every Monday, Tuesday, Wednesday, Thursday and Friday</td></tr>' + '<tr><td><code>0 15 10 15 * ?</code></td><td>Fire at 10:15am on the 15th day of every month</td></tr>'
          + '<tr><td><code>0 15 10 L * ?</code></td><td>Fire at 10:15am on the last day of every month</td></tr>' + '<tr><td><code>0 15 10 ? * 6L</code></td><td>Fire at 10:15am on the last Friday of every month</td></tr>'
          + '<tr valign="top"><td nowrap><code>0 15 10 ? * 6L 2002-2005 &nbsp; &nbsp;</code></td><td>Fire at 10:15am on every last friday of every month during the years 2002, 2003, 2004 and 2005</td></tr>'
          + '<tr><td><code>0 15 10 ? * 6#3</code></td><td>Fire at 10:15am on the third Friday of every month</td></tr>' + '</table>'
    },

    cronBigHelp : {
      text : '<br>Cron expressions provide the ability to specify complex time combinations such as - At 8:00am every Monday through Friday - or - At 1:30am every last Friday of the month.<br><br>Cron expressions are comprised of 6 required fields and one optional field separated by white space. The fields respectively are described as follows: <table cellspacing=&quot;8&quot;><tr><th align=&quot;left&quot;>Field Name</th><th align=&quot;left&quot;>&nbsp;</th><th align=&quot;left&quot;>Allowed Values</th><th align=&quot;left&quot;>&nbsp;</th><th align=&quot;left&quot;>Allowed Special Characters</th></tr><tr><td align=&quot;left&quot;><code>Seconds</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>0-59</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * /</code></td></tr><tr><td align=&quot;left&quot;><code>Minutes</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>0-59</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * /</code></td></tr><tr><td align=&quot;left&quot;><code>Hours</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>0-23</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * /</code></td></tr><tr><td align=&quot;left&quot;><code>Day-of-month</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>1-31</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * ? / L W</code></td></tr><tr><td align=&quot;left&quot;><code>Month</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>1-12 or JAN-DEC</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * /</code></td></tr><tr><td align=&quot;left&quot;><code>Day-of-Week</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>1-7 or SUN-SAT</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * ? / L #</code></td></tr><tr><td align=&quot;left&quot;><code>Year (Optional)</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>empty, 1970-2099</code></td><td align=&quot;left&quot;>&nbsp;</th><td align=&quot;left&quot;><code>, - * /</code></td></tr></table><br><br>The * character is used to specify all values. For example, * in the minute field means every minute.<br><br>The ? character is allowed for the day-of-month and day-of-week fields. It is used to specify no specific value.This is useful when you need to specify something in one of the two fields, but not the other.<br><br>The - character is used to specify ranges For example &quot;10-12&quot; in the hour field means &quot;the hours 10,11 and 12&quot;.<br><br>The , character is used to specify additional values. For example &quot;MON,WED,FRI&quot; in the day-of-week field means &quot;the days Monday, Wednesday, and Friday&quot;.<br><br>The / character is used to specify increments. For example &quot;0/15&quot; in the seconds field means &quot;the seconds 0, 15, 30, and 45&quot;. And &quot;5/15&quot; in the seconds field means &quot;the seconds 5, 20, 35, and 50&quot;. Specifying * before the / is equivalent to specifying 0 is the value to start with. Essentially, for each field in the expression, there is a set of numbers that can be turned on or off. For seconds and minutes, the numbers range from 0 to 59. For hours 0 to 23, for days of the month 0 to 31, and for months 1 to 12. The &quot;/&quot; character simply helps you turn on every &quot;nth&quot; value in the given set. Thus &quot;7/6&quot; in the month field only turns on month &quot;7&quot;, it does NOT mean every 6th month, please note that subtlety.<br><br>The L character is allowed for the day-of-month and day-of-week fields. This character is short-hand for &quot;last&quot;, but it has different meaning in each of the two fields. For example, the value &quot;L&quot; in the day-of-month field means &quot;the last day of the month&quot; - day 31 for January, day 28 for February on non-leap years. If used in the day-of-week field by itself, it simply means &quot;7&quot; or &quot;SAT&quot;. But if used in the day-of-week field after another value, it means &quot;the last xxx day of the month&quot; - for example &quot;6L&quot; means &quot;the last friday of the month&quot;. When using the L option, it is important not to specify lists, or ranges of values, as you will get confusing results.<br><br>The W character is allowed for the day-of-month field. This character is used to specify the weekday (Monday-Friday) nearest the given day. As an example, if you were to specify &quot;15W&quot; as the value for the day-of-month field, the meaning is: &quot;the nearest weekday to the 15th of the month&quot;. So if the 15th is a Saturday, the trigger will fire on Friday the 14th. If the 15th is a Sunday, the trigger will fire on Monday the 16th. If the 15th is a Tuesday, then it will fire on Tuesday the 15th. However if you specify &quot;1W&quot; as the value for day-of-month, and the 1st is a Saturday, the trigger will fire on Monday the 3rd, as it will not jump over the boundary of a months days. The W character can only be specified when the day-of-month is a single day, not a range or list of days.<br><br>The L and W characters can also be combined for the day-of-month expression to yield LW, which translates to &quot;last weekday of the month&quot;.<br><br>The # character is allowed for the day-of-week field. This character is used to specify &quot;the nth&quot; XXX day of the month. For example, the value of &quot;6#3&quot; in the day-of-week field means the third Friday of the month (day 6 = Friday and &quot;#3&quot; = the 3rd one in the month). Other examples: &quot;2#1&quot; = the first Monday of the month and &quot;4#5&quot; = the fifth Wednesday of the month. Note that if you specify &quot;#5&quot; and there is not 5 of the given day-of-week in the month, then no firing will occur that month.<br><br>The legal characters and the names of months and days of the week are not case sensitive.<br><br><b>NOTES:</b><ul><li>Support for specifying both a day-of-week and a day-of-month value is not complete (you will need to use the ? character in one of these fields). </li><li>Overflowing ranges is supported - that is, having a larger number on the left hand side than the right. You might do 22-2 to catch 10 o clock at night until 2 o clock in the morning, or you might have NOV-FEB. It is very important to note that overuse of overflowing ranges creates ranges that do not make sense and no effort has been made to determine which interpretation CronExpression chooses. An example would be 0 0 14-6 ? * FRI-MON. </li></ul></p>'
    },

    repoTargets : {
      name : 'The name of the repository target.',
      contentClass : 'The content class of the repository target. It will be matched only against repositories with the same content class.',
      pattern : 'Enter a pattern expression and click "Add" to add it to the list. Regular expressions are used to match the artifact path. ".*" is used to specify all paths. ".*/com/some/company/.*" will match any artifact with "com.some.company" as the group id or artifact id. "^/com/some/company/.*" will match any artifact starting with com/some/company.'
    },

    log : {
      rootLoggerLevel : 'The root logger level for your logging configuration.',
      rootLoggerAppenders : 'The root logger appenders for your logging configuration.',
      fileAppenderLocation : 'The file appender location for your logging configuration.',
      fileAppenderPattern : 'The file appender pattern for your logging configuration.'
    },

    repoMirrors : {
      mirrorUrl : 'A URL that acts as a mirror for this repository.'
    }
  };

})();
});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * Target Edit/Create panel layout and controller
 */
define('repoServer/RepoTargetEditPanel',['Sonatype/all','Nexus/ext/GridFilterBox'], function(){

Sonatype.repoServer.RepoTargetEditPanel = function(config) {
  var config = config || {},
      defaultConfig = {};

  Ext.apply(this, config, defaultConfig);

  var ht = Sonatype.repoServer.resources.help.repoTargets;

  this.actions = {
    refresh : new Ext.Action({
          text : 'Refresh',
          iconCls : 'st-icon-refresh',
          scope : this,
          handler : this.reloadAll
        }),
    deleteAction : new Ext.Action({
          text : 'Delete',
          scope : this,
          handler : this.deleteHandler
        })
  };

  // Methods that will take the incoming json data and map over to the ui
  // controls
  this.loadDataModFunc = {
    "patterns" : this.loadPatternsTreeHelper.createDelegate(this)
  };

  // Methods that will take the data from the ui controls and map over to json
  this.submitDataModFunc = {
    "patterns" : this.exportPatternsTreeHelper.createDelegate(this)
  };

  // A record to hold the name and id of a repository
  this.repoTargetRecordConstructor = Ext.data.Record.create([{
        name : 'resourceURI'
      }, {
        name : 'id'
      }, {
        name : 'contentClass'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }, {
        name : 'patterns'
      }]);

  // A record to hold the contentClasses
  this.contentClassRecordConstructor = Ext.data.Record.create([{
        name : 'contentClass'
      }, {
        name : 'name',
        sortType : Ext.data.SortTypes.asUCString
      }]);

  // Reader and datastore that queries the server for the list of repo targets
  this.repoTargetsReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'resourceURI'
      }, this.repoTargetRecordConstructor);
  this.repoTargetsDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.repoTargets,
        reader : this.repoTargetsReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : true
      });

  // Reader and datastore that queries the server for the list of content
  // classes
  this.contentClassesReader = new Ext.data.JsonReader({
        root : 'data',
        id : 'contentClass'
      }, this.contentClassRecordConstructor);
  this.contentClassesDataStore = new Ext.data.Store({
        url : Sonatype.config.repos.urls.repoContentClasses,
        reader : this.contentClassesReader,
        sortInfo : {
          field : 'name',
          direction : 'ASC'
        },
        autoLoad : false
      });

  this.COMBO_WIDTH = 300;

  // Build the form
  this.formConfig = {
    region : 'center',
    width : '100%',
    height : '100%',
    autoScroll : true,
    border : false,
    frame : true,
    collapsible : false,
    collapsed : false,
    labelWidth : 150,
    layoutConfig : {
      labelSeparator : ''
    },

    items : [{
          xtype : 'textfield',
          fieldLabel : 'Name',
          itemCls : 'required-field',
          helpText : ht.name,
          name : 'name',
          allowBlank : false,
          width : this.COMBO_WIDTH,
          validator : this.checkForDuplicates.createDelegate(this)
        }, {
          xtype : 'combo',
          fieldLabel : 'Repository Type',
          itemCls : 'required-field',
          helpText : ht.contentClass,
          name : 'contentClass',
          width : this.COMBO_WIDTH,
          store : this.contentClassesDataStore,
          displayField : 'name',
          valueField : 'contentClass',
          editable : false,
          forceSelection : true,
          mode : 'local',
          triggerAction : 'all',
          emptyText : 'Select...',
          selectOnFocus : true,
          allowBlank : false,
          disabled : true,
          listeners : {
            'select' : {
              fn : function(combo, record, index) {},
              scope : this
            }
          }
        }, {
          xtype : 'panel',
          style : 'padding-top: 20px',
          layout : 'column',
          items : [{
                xtype : 'panel',
                layout : 'form',
                width : 475,
                items : [{
                      xtype : 'textfield',
                      fieldLabel : 'Pattern Expression',
                      helpText : ht.pattern,
                      name : 'pattern',
                      width : 300
                    }]
              }, {
                xtype : 'panel',
                width : 120,
                items : [{
                      xtype : 'button',
                      text : 'Add',
                      style : 'padding-left: 7px',
                      minWidth : 100,
                      id : 'button-add',
                      handler : this.addNewPattern,
                      scope : this
                    }]
              }]
        }, {
          xtype : 'panel',
          layout : 'column',
          autoHeight : true,
          style : 'padding-left: 155px',
          items : [{
                xtype : 'treepanel',
                id : 'repoTargets-pattern-list', // note: unique ID is assinged
                                                  // before instantiation
                name : 'repoTargets-pattern-list',
                title : 'Patterns',
                cls : 'required-field',
                border : true, // note: this seem to have no effect w/in form
                                // panel
                bodyBorder : true, // note: this seem to have no effect w/in
                                    // form panel
                // note: this style matches the expected behavior
                bodyStyle : 'background-color:#FFFFFF; border: 1px solid #B5B8C8',
                style : 'padding: 0 20px 0 0',
                width : 320,
                height : 150,
                animate : true,
                lines : false,
                autoScroll : true,
                containerScroll : true,
                // @note: root node must be instantiated uniquely for each
                // instance of treepanel
                // @ext: can TreeNode be registerd as a component with an xtype
                // so this new root node
                // may be instantiated uniquely for each form panel that uses
                // this config?
                rootVisible : false,
                enableDD : false
              }, {
                xtype : 'panel',
                width : 120,
                items : [{
                      xtype : 'button',
                      text : 'Remove',
                      style : 'padding-left: 6px',
                      minWidth : 100,
                      id : 'button-remove',
                      handler : this.removePattern,
                      scope : this
                    }, {
                      xtype : 'button',
                      text : 'Remove All',
                      style : 'padding-left: 6px; margin-top: 5px',
                      minWidth : 100,
                      id : 'button-remove-all',
                      handler : this.removeAllPatterns,
                      scope : this
                    }]
              }]
        }, {
          xtype : 'hidden',
          name : 'id'
        }],
    buttons : [{
          text : 'Save',
          disabled : true
        }, {
          text : 'Cancel'
        }]
  };

  this.sp = Sonatype.lib.Permissions;

  this.repoTargetsGridPanel = new Ext.grid.GridPanel({
        id : 'st-repoTargets-grid',
        region : 'north',
        layout : 'fit',
        split : true,
        height : 200,
        minHeight : 150,
        maxHeight : 400,
        frame : false,
        autoScroll : true,
        tbar : [{
              id : 'repoTarget-refresh-btn',
              text : 'Refresh',
              iconCls : 'st-icon-refresh',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.reloadAll
            }, {
              id : 'repoTarget-add-btn',
              text : 'Add',
              icon : Sonatype.config.resourcePath + '/static/images/icons/add.png',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.addResourceHandler,
              disabled : !this.sp.checkPermission('nexus:targets', this.sp.CREATE)
            }, {
              id : 'repoTarget-delete-btn',
              text : 'Delete',
              icon : Sonatype.config.resourcePath + '/static/images/icons/delete.png',
              cls : 'x-btn-text-icon',
              scope : this,
              handler : this.deleteHandler,
              disabled : !this.sp.checkPermission('nexus:targets', this.sp.DELETE)
            }
        ],

        // grid view options
        ds : this.repoTargetsDataStore,
        sortInfo : {
          field : 'name',
          direction : "ASC"
        },
        loadMask : true,
        deferredRender : false,
        columns : [{
              header : 'Name',
              dataIndex : 'name',
              width : 200
            }, {
              header : 'Repository Type',
              dataIndex : 'contentClass',
              width : 200
            }, {
              header : 'Patterns',
              dataIndex : 'patterns',
              width : 200,
              id : 'patterns-expandable-col'
            }],
        autoExpandColumn : 'patterns-expandable-col',
        disableSelection : false,
        viewConfig : {
          deferEmptyText: false,
          emptyText: 'No repository targets defined',
          emptyTextWhileFiltering: 'No repository targets matched criteria: {criteria}'
        }
      });
  this.repoTargetsGridPanel.getSelectionModel().on('rowselect', this.rowSelect, this);
  this.repoTargetsGridPanel.on('rowcontextmenu', this.contextClick, this);

  this.repoTargetsGridPanel.getTopToolbar().add([
      '->',
      NX.create('Nexus.ext.GridFilterBox', {
        filteredGrid: this.repoTargetsGridPanel
      })
  ]);

  Sonatype.repoServer.RepoTargetEditPanel.superclass.constructor.call(this, {
        layout : 'border',
        autoScroll : false,
        width : '100%',
        height : '100%',
        items : [this.repoTargetsGridPanel, {
              xtype : 'panel',
              id : 'repoTarget-config-forms',
              title : 'Repository Target Configuration',
              layout : 'card',
              region : 'center',
              activeItem : 0,
              deferredRender : false,
              autoScroll : false,
              frame : false,
              items : [{
                    xtype : 'panel',
                    layout : 'fit',
                    html : '<div class="little-padding">Select a target to edit it, or click "Add" to create a new one.</div>'
                  }]
            }]
      });

  this.formCards = this.findById('repoTarget-config-forms');
};

Ext.extend(Sonatype.repoServer.RepoTargetEditPanel, Ext.Panel, {
      // Dump the currently stored data and requery for everything
      reloadAll : function() {
        this.repoTargetsDataStore.reload();
        this.formCards.items.each(function(item, i, len) {
              if (i > 0)
              {
                this.remove(item, true);
              }
            }, this.formCards);

        this.formCards.getLayout().setActiveItem(0);
      },

      saveHandler : function(formInfoObj) {
        // validate the form to make sure all the error messages are present
        var formIsValid = formInfoObj.formPanel.form.isValid();

        var fpanel = formInfoObj.formPanel;
        var treePanel = fpanel.findById(fpanel.id + '_repoTargets-pattern-list');
        var patternField = fpanel.find('name', 'pattern')[0];
        if (!treePanel.root.hasChildNodes())
        {
          patternField.markInvalid('The target should have at least one pattern.');
          return;
        }

        if (formIsValid)
        {
          var isNew = formInfoObj.isNew;
          var createUri = Sonatype.config.repos.urls.repoTargets;
          var updateUri = (formInfoObj.resourceURI) ? formInfoObj.resourceURI : '';
          var form = formInfoObj.formPanel.form;

          form.doAction('sonatypeSubmit', {
            method : (isNew) ? 'POST' : 'PUT',
            url : isNew ? createUri : updateUri,
            waitMsg : isNew ? 'Creating Target...' : 'Updating Target...',
            fpanel : formInfoObj.formPanel,
            dataModifiers : this.submitDataModFunc,
            serviceDataObj : Sonatype.repoServer.referenceData.repoTargets,
            isNew : isNew
              // extra option to send to callback, instead of conditioning on
              // method
            });
        }
      },

      cancelHandler : function(formInfoObj) {
        var formLayout = this.formCards.getLayout();
        var gridSelectModel = this.repoTargetsGridPanel.getSelectionModel();
        var store = this.repoTargetsGridPanel.getStore();

        this.formCards.remove(formInfoObj.formPanel.id, true);
        // select previously selected form, or the default view (index == 0)
        var newIndex = this.formCards.items.length - 1;
        newIndex = (newIndex >= 0) ? newIndex : 0;
        formLayout.setActiveItem(newIndex);

        // delete row from grid if canceling a new repo form
        if (formInfoObj.isNew)
        {
          store.remove(store.getById(formInfoObj.formPanel.id));
        }

        // select the coordinating row in the grid, or none if back to default
        var i = store.indexOfId(formLayout.activeItem.id);
        if (i >= 0)
        {
          gridSelectModel.selectRow(i);
        }
        else
        {
          gridSelectModel.clearSelections();
        }
      },

      addResourceHandler : function() {
        var id = 'new_target_' + new Date().getTime();

        var config = Ext.apply({}, this.formConfig, {
              id : id
            });

        config = this.initializeTreeRoots(id, config);

        var formPanel = new Ext.FormPanel(config);

        formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
        formPanel.form.on('actionfailed', this.actionFailedHandler, this);
        formPanel.on('beforerender', this.beforeFormRenderHandler, this);
        formPanel.on('afterlayout', this.afterLayoutFormHandler, this, {
              single : true
            });

        var buttonInfoObj = {
          formPanel : formPanel,
          isNew : true
        };

        // save button event handler
        formPanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
        // cancel button event handler
        formPanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

        // add new form
        this.formCards.add(formPanel);

        // add place holder to grid
        var newRec = new this.repoTargetRecordConstructor({
              name : 'New Target',
              resourceURI : 'new'
            }, id); // use "new_user_" id instead of resourceURI like the reader
                    // does
        this.repoTargetsDataStore.insert(0, [newRec]);
        this.repoTargetsGridPanel.getSelectionModel().selectRow(0);

        // always set active and re-layout
        formPanel.doLayout();
      },

      afterLayoutFormHandler : function(formPanel, fLayout) {
        // register required field quicktip, but have to wait for elements to
        // show up in DOM
        var temp = function() {
          var els = Ext.select('.required-field .x-form-item-label, .required-field .x-panel-header-text', this.getEl());
          els.each(function(el, els, i) {
                Ext.QuickTips.register({
                      target : el,
                      cls : 'required-field',
                      title : '',
                      text : 'Required Field',
                      enabled : true
                    });
              });
        }.defer(300, formPanel);
      },

      deleteHandler : function() {
        if (this.ctxRecord || this.repoTargetsGridPanel.getSelectionModel().hasSelection())
        {
          var rec = this.ctxRecord ? this.ctxRecord : this.repoTargetsGridPanel.getSelectionModel().getSelected();

          if (rec.data.resourceURI == 'new')
          {
            this.cancelHandler({
                  formPanel : Ext.getCmp(rec.id),
                  isNew : true
                });
          }
          else
          {
            // @note: this handler selects the "No" button as the default
            // @todo: could extend Sonatype.MessageBox to take the button to
            // select as a param
            Sonatype.MessageBox.getDialog().on('show', function() {
                  this.focusEl = this.buttons[2]; // ack! we're offset dependent
                                                  // here
                  this.focus();
                }, Sonatype.MessageBox.getDialog(), {
                  single : true
                });

            Sonatype.MessageBox.show({
                  animEl : this.repoTargetsGridPanel.getEl(),
                  title : 'Delete Target?',
                  msg : 'Delete the ' + rec.get('name') + ' target?',
                  buttons : Sonatype.MessageBox.YESNO,
                  scope : this,
                  icon : Sonatype.MessageBox.QUESTION,
                  fn : function(btnName) {
                    if (btnName == 'yes' || btnName == 'ok')
                    {
                      Ext.Ajax.request({
                            callback : this.deleteCallback,
                            cbPassThru : {
                              resourceId : rec.id
                            },
                            scope : this,
                            method : 'DELETE',
                            url : rec.data.resourceURI
                          });
                    }
                  }
                });
          }
        }
      },

      deleteCallback : function(options, isSuccess, response) {
        if (isSuccess)
        {
          var resourceId = options.cbPassThru.resourceId;
          var formLayout = this.formCards.getLayout();
          var gridSelectModel = this.repoTargetsGridPanel.getSelectionModel();
          var store = this.repoTargetsGridPanel.getStore();

          if (formLayout.activeItem.id == resourceId)
          {
            this.formCards.remove(resourceId, true);
            // select previously selected form, or the default view (index == 0)
            var newIndex = this.formCards.items.length - 1;
            newIndex = (newIndex >= 0) ? newIndex : 0;
            formLayout.setActiveItem(newIndex);
          }
          else
          {
            this.formCards.remove(resourceId, true);
          }

          store.remove(store.getById(resourceId));

          // select the coordinating row in the grid, or none if back to default
          var i = store.indexOfId(formLayout.activeItem.id);
          if (i >= 0)
          {
            gridSelectModel.selectRow(i);
          }
          else
          {
            gridSelectModel.clearSelections();
          }
        }
        else
        {
          Sonatype.MessageBox.alert('The server did not delete the target.');
        }
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionCompleteHandler : function(form, action) {
        // @todo: handle server error response here!!

        if (action.type == 'sonatypeSubmit')
        {
          var isNew = action.options.isNew;
          var receivedData = action.handleResponse(action.response).data;
          if (isNew)
          {

            var dataObj = {
              id : receivedData.id,
              name : receivedData.name,
              resourceURI : action.getUrl() + '/' + receivedData.id,
              contentClass : receivedData.contentClass,
              patterns : receivedData.patterns
            };

            var newRec = new this.repoTargetRecordConstructor(dataObj, action.options.fpanel.id);

            this.repoTargetsDataStore.remove(this.repoTargetsDataStore.getById(action.options.fpanel.id)); // remove
                                                                                                            // old
                                                                                                            // one
            this.repoTargetsDataStore.addSorted(newRec);
            this.repoTargetsGridPanel.getSelectionModel().selectRecords([newRec], false);

            // set the hidden id field in the form for subsequent updates
            action.options.fpanel.find('name', 'id')[0].setValue(receivedData.id);

            // remove button click listeners
            action.options.fpanel.buttons[0].purgeListeners();
            action.options.fpanel.buttons[1].purgeListeners();

            var buttonInfoObj = {
              formPanel : action.options.fpanel,
              isNew : false,
              resourceURI : dataObj.resourceURI
            };

            // save button event handler
            action.options.fpanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));

            // cancel button event handler
            action.options.fpanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));
          }
          else
          {
            var sentData = action.output.data;

            var i = this.repoTargetsDataStore.indexOfId(action.options.fpanel.id);
            var rec = this.repoTargetsDataStore.getAt(i);

            this.updateRepoTargetRecord(rec, sentData);

            var sortState = this.repoTargetsDataStore.getSortState();
            this.repoTargetsDataStore.sort(sortState.field, sortState.direction);
          }
        }
      },

      updateRepoTargetRecord : function(rec, receivedData) {
        rec.beginEdit();
        rec.set('name', receivedData.name);
        rec.set('id', receivedData.id);
        rec.set('contentClass', receivedData.contentClass);
        rec.set('patterns', receivedData.patterns);
        rec.commit();
        rec.endEdit();
      },

      // (Ext.form.BasicForm, Ext.form.Action)
      actionFailedHandler : function(form, action) {
        if (action.failureType == Ext.form.Action.CLIENT_INVALID)
        {
          Sonatype.MessageBox.alert('Missing or Invalid Fields', 'Please change the missing or invalid fields.').setIcon(Sonatype.MessageBox.WARNING);
        }
        // @note: server validation error are now handled just like client
        // validation errors by marking the field invalid
        // else if(action.failureType == Ext.form.Action.SERVER_INVALID){
        // Sonatype.MessageBox.alert('Invalid Fields', 'The server identified
        // invalid fields.').setIcon(Sonatype.MessageBox.ERROR);
        // }
        else if (action.failureType == Ext.form.Action.CONNECT_FAILURE)
        {
          Sonatype.utils.connectionError(action.response, 'There is an error communicating with the server.')
        }
        else if (action.failureType == Ext.form.Action.LOAD_FAILURE)
        {
          Sonatype.MessageBox.alert('Load Failure', 'The data failed to load from the server.').setIcon(Sonatype.MessageBox.ERROR);
        }

        // @todo: need global alert mechanism for fatal errors.
      },

      beforeFormRenderHandler : function(component) {
        var sp = Sonatype.lib.Permissions;
        // TODO
        if (sp.checkPermission('nexus:targets', sp.EDIT))
        {
          component.buttons[0].disabled = false;
        }
        if (sp.checkPermission('nexus:componentscontentclasses', sp.READ))
        {
          this.contentClassesDataStore.load();
          var contentClassCombo = component.items.get(1);
          contentClassCombo.disabled = false;
        }
      },

      formDataLoader : function(formPanel, resourceURI, modFuncs) {
        formPanel.getForm().doAction('sonatypeLoad', {
              url : resourceURI,
              method : 'GET',
              fpanel : formPanel,
              dataModifiers : modFuncs,
              scope : this
            });
      },

      rowSelect : function(selectionModel, index, rec) {
        var id = rec.id; // note: rec.id is unique for new resources and equal
                          // to resourceURI for existing ones
        var formPanel = this.formCards.findById(id);

        // assumption: new route forms already exist in formCards, so they won't
        // get into this case
        if (!formPanel)
        { // create form and populate current data
          var config = Ext.apply({}, this.formConfig, {
                id : id
              });

          config = this.initializeTreeRoots(id, config);

          formPanel = new Ext.FormPanel(config);
          formPanel.form.on('actioncomplete', this.actionCompleteHandler, this);
          formPanel.form.on('actionfailed', this.actionFailedHandler, this);
          formPanel.on('beforerender', this.beforeFormRenderHandler, this);
          formPanel.on('afterlayout', this.afterLayoutFormHandler, this, {
                single : true
              });

          var buttonInfoObj = {
            formPanel : formPanel,
            isNew : false, // not a new route form, see assumption
            resourceURI : rec.data.resourceURI
          };

          formPanel.buttons[0].on('click', this.saveHandler.createDelegate(this, [buttonInfoObj]));
          formPanel.buttons[1].on('click', this.cancelHandler.createDelegate(this, [buttonInfoObj]));

          this.formDataLoader(formPanel, rec.data.resourceURI, this.loadDataModFunc);

          this.formCards.add(formPanel);
          this.formCards.getLayout().setActiveItem(formPanel);
          formPanel.doLayout();
        }
        else
        {
          // always set active
          this.formCards.getLayout().setActiveItem(formPanel);
        }
      },

      contextClick : function(grid, index, e) {
        this.contextHide();

        if (e.target.nodeName == 'A')
          return; // no menu on links

        this.ctxRow = this.repoTargetsGridPanel.view.getRow(index);
        this.ctxRecord = this.repoTargetsGridPanel.store.getAt(index);
        Ext.fly(this.ctxRow).addClass('x-node-ctx');

        // @todo: would be faster to pre-render the six variations of the menu
        // for whole instance
        var menu = new Ext.menu.Menu({
              items : [this.actions.refresh]
            });

        if (this.sp.checkPermission('nexus:targets', this.sp.DELETE))
        {
          menu.add(this.actions.deleteAction);
        }

        // TODO: Add additional menu items

        menu.on('hide', this.contextHide, this);
        e.stopEvent();
        menu.showAt(e.getXY());
      },

      contextHide : function() {
        if (this.ctxRow)
        {
          Ext.fly(this.ctxRow).removeClass('x-node-ctx');
          this.ctxRow = null;
          this.ctxRecord = null;
        }
      },

      initializeTreeRoots : function(id, config) {
        // @note: there has to be a better way to do this. Depending on offsets
        // is very error prone
        var newConfig = config;

        newConfig.items[3].items[0].id = id + '_repoTargets-pattern-list';
        newConfig.items[3].items[0].root = new Ext.tree.TreeNode({
              text : 'root'
            });

        return newConfig;
      },

      loadPatternsTreeHelper : function(arr, srcObj, fpanel) {
        var treePanel = fpanel.findById(fpanel.id + '_repoTargets-pattern-list');

        var pattern;

        for (var i = 0; i < arr.length; i++)
        {
          pattern = arr[i];
          this.addPatternNode(treePanel, pattern);
        }

        return arr; // return arr, even if empty to comply with sonatypeLoad
                    // data modifier requirement
      },

      exportPatternsTreeHelper : function(val, fpanel) {
        var treePanel = fpanel.findById(fpanel.id + '_repoTargets-pattern-list');

        var outputArr = [];
        var nodes = treePanel.root.childNodes;

        for (var i = 0; i < nodes.length; i++)
        {
          outputArr[i] = nodes[i].attributes.payload;
        }

        return outputArr;
      },

      addPatternNode : function(treePanel, pattern) {
        var id = Ext.id();

        treePanel.root.appendChild(new Ext.tree.TreeNode({
                  id : id,
                  text : pattern,
                  payload : pattern,
                  allowChildren : false,
                  draggable : false,
                  leaf : true,
                  nodeType : 'pattern',
                  icon : Sonatype.config.extPath + '/resources/images/default/tree/leaf.gif'
                }));
      },

      addNewPattern : function() {
        var fpanel = this.formCards.getLayout().activeItem;
        var treePanel = fpanel.findById(fpanel.id + '_repoTargets-pattern-list');
        var patternField = fpanel.find('name', 'pattern')[0];
        var pattern = patternField.getRawValue();

        if (pattern)
        {
          var nodes = treePanel.root.childNodes;
          for (var i = 0; i < nodes.length; i++)
          {
            if (pattern == nodes[i].attributes.payload)
            {
              patternField.markInvalid('This pattern already exists');
              return;
            }
          }

          this.addPatternNode(treePanel, pattern);
          patternField.setRawValue('');
        }
      },

      removePattern : function() {
        var fpanel = this.formCards.getLayout().activeItem;
        var treePanel = fpanel.findById(fpanel.id + '_repoTargets-pattern-list');

        var selectedNode = treePanel.getSelectionModel().getSelectedNode();
        if (selectedNode)
        {
          treePanel.root.removeChild(selectedNode);
        }
      },

      removeAllPatterns : function() {
        var fpanel = this.formCards.getLayout().activeItem;
        var treePanel = fpanel.findById(fpanel.id + '_repoTargets-pattern-list');
        var treeRoot = treePanel.root;

        while (treeRoot.lastChild)
        {
          treeRoot.removeChild(treeRoot.lastChild);
        }
      },

      checkForDuplicates : function(value) {
        var selectedRec = this.repoTargetsGridPanel.getSelectionModel().getSelected();
        var result = this.repoTargetsDataStore.findBy(function(rec, id) {
              return selectedRec != rec && rec.get('name') == value;
            });
        if (result >= 0)
        {
          return 'A target with this name already exists.';
        }
        return true;
      }

    });

});


/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('repoServer/RepoServer',['extjs', 'sonatype', 'Sonatype/lib', 'Nexus/config', 'Sonatype/utils', 'Sonatype/view', 'Nexus/navigation'], function(Ext, Sonatype){
  // Repository main Controller(conglomerate) Singleton
  Sonatype.repoServer.RepoServer = (function() {
    var cfg = Sonatype.config.repos,
        sp = Sonatype.lib.Permissions,
          lastFormHeight = -1;

    // ************************************
    return {
      pomDepTmpl : new Ext.XTemplate('<dependency><groupId>{groupId}</groupId><artifactId>{artifactId}</artifactId><version>{version}</version></dependency>'),

      loginFormConfig : {
        labelAlign : 'right',
        labelWidth : 60,
        frame : true,

        defaultType : 'textfield',
        monitorValid : true,
        listeners : {
          clientvalidation : function(form, valid) {
            // IE does not resize the window when invalid messages are shown
            var window, formHeight = form.getHeight();

            if ( lastFormHeight !== formHeight ) {
              window = form.findParentByType('window');
              if ( window !== undefined ) {
                lastFormHeight = formHeight;
                window.setHeight(formHeight + form.getToolbarHeight());
              }
            }
          },
          scope : this
        },

        items : [{
              id : 'usernamefield',
              fieldLabel : 'Username',
              name : 'username',
              tabIndex : 1,
              width : 200,
              allowBlank : false
            }, {
              id : 'passwordfield',
              fieldLabel : 'Password',
              name : 'password',
              tabIndex : 2,
              inputType : 'password',
              width : 200,
              allowBlank : false
            }]
        // buttons added later to provide scope to handler

      },

      statusComplete : function(statusResponse) {
        this.resetMainTabPanel();

        this.createSubComponents(); // update left panel
      },

      // Each Sonatype server will need one of these
      initServerTab : function() {

        Sonatype.Events.addListener('nexusNavigationInit', this.addNexusNavigationItems, this);

        Sonatype.Events.addListener('nexusStatus', this.nexusStatusEvent, this);

        // Left Panel
        this.nexusPanel = new Sonatype.navigation.NavigationPanel({
              id : 'st-nexus-tab',
              title : 'Nexus Repository Manager'
            });

        this.createSubComponents();

        Sonatype.view.serverTabPanel.add(this.nexusPanel);
        Sonatype.view.serverTabPanel.hideTabStripItem('st-nexus-tab');

        this.loginFormConfig.buttons = [{
              id : 'loginbutton',
              text : 'Log In',
              tabIndex : 3,
              formBind : true,
              scope : this,
              handler : function() {
                var usernameField = this.loginForm.find('name', 'username')[0],
                    passwordField = this.loginForm.find('name', 'password')[0];

                if (usernameField.isValid() && passwordField.isValid())
                {
                  Sonatype.utils.doLogin(this.loginWindow, usernameField.getValue(), passwordField.getValue());
                }
              }
            }];

        this.loginFormConfig.keys = {
          key : Ext.EventObject.ENTER,
          fn : this.loginFormConfig.buttons[0].handler,
          scope : this
        };

        this.loginForm = new Ext.form.FormPanel(this.loginFormConfig);
        this.loginWindow = new Ext.Window({
              id : 'login-window',
              title : 'Nexus Repository Manager Log In',
              animateTarget : Ext.get('head-link-r'),
              closable : true,
              closeAction : 'hide',
              onEsc : function() {
                if ( this.closable ) {
                  Ext.Window.prototype.onEsc.apply(this, arguments);
                }
              },
              autoWidth : false,
              width : 300,
              modal : true,
              constrain : true,
              resizable : false,
              draggable : false,
              items : [this.loginForm]
            });

        this.loginWindow.on('show', function() {
              var
                    panel, field,
                    // NEXUS-5108 hide close button when anon access is disabled
                    anonDisabled = Sonatype.utils.anonDisabled || false; // if anonDisabled is undefined -> boolean

              this.loginWindow.closable = !anonDisabled;
              this.loginWindow.tools.close.setVisible(!anonDisabled);

              field = this.loginForm.find('name', 'username')[0];
              if (field.getRawValue())
              {
                field = this.loginForm.find('name', 'password')[0];
              }
              field.focus(true, 100);
            }, this);

        this.loginWindow.on('close', function() {
              this.loginForm.getForm().reset();
            }, this);

        this.loginWindow.on('hide', function() {
              this.loginForm.getForm().reset();
              Sonatype.view.afterLoginToken = null;
            }, this);
      },

      // Add/Replace Nexus left hand components
      createSubComponents : function() {
        var wasRendered = this.nexusPanel.rendered;

        if (wasRendered)
        {
          this.nexusPanel.getEl().mask('Updating...', 'loading-indicator');
          this.nexusPanel.items.each(function(item, i, len) {
                this.remove(item, true);
              }, this.nexusPanel);
        }

        Sonatype.Events.fireEvent('nexusNavigationInit', this.nexusPanel);

        // fire second event so plugins can contribute navigation items, and set the order
        Sonatype.Events.fireEvent('nexusNavigationPostInit', this.nexusPanel);

        if (wasRendered)
        {
          this.nexusPanel.doLayout();
          this.nexusPanel.getEl().unmask();
          // this.nexusPanel.enable();
        }

        Sonatype.Events.fireEvent('nexusNavigationInitComplete', this.nexusPanel);

      },

      nexusStatusEvent : function() {

        // check the user status, if it is not set, then reset the panels
        if (!Sonatype.user.curr.repoServer)
        {
          this.resetMainTabPanel();
        }
      },

      addNexusNavigationItems : function(nexusPanel) {

        // Views Group **************************************************
        nexusPanel.add({
              title : 'Views/Repositories',
              id : 'st-nexus-views',
              items : [{
                    enabled : sp.checkPermission('nexus:repostatus', sp.READ),
                    title : 'Repositories',
                    tabId : 'view-repositories',
                    tabCode : Sonatype.repoServer.RepositoryPanel,
                    tabTitle : 'Repositories'
                  }, {
                    enabled : sp.checkPermission('nexus:targets', sp.READ) && (sp.checkPermission('nexus:targets', sp.CREATE) || sp.checkPermission('nexus:targets', sp.DELETE) || sp.checkPermission('nexus:targets', sp.EDIT)),
                    title : 'Repository Targets',
                    tabId : 'targets-config',
                    tabCode : Sonatype.repoServer.RepoTargetEditPanel
                  }, {
                    enabled : sp.checkPermission('nexus:routes', sp.READ) && (sp.checkPermission('nexus:routes', sp.CREATE) || sp.checkPermission('nexus:routes', sp.DELETE) || sp.checkPermission('nexus:routes', sp.EDIT)),
                    title : 'Routing',
                    tabId : 'routes-config',
                    tabCode : Sonatype.repoServer.RoutesEditPanel
                  }]
            });

        // Config Group **************************************************
        nexusPanel.add({
              title : 'Enterprise',
              id : 'st-nexus-enterprise'
            });

        // Security Group **************************************************
        nexusPanel.add({
              title : 'Security',
              id : 'st-nexus-security',
              collapsed : true,
              items : [
                  {
                    enabled : sp.checkPermission('security:users', sp.READ) && (sp.checkPermission('security:users', sp.CREATE) || sp.checkPermission('security:users', sp.DELETE) || sp.checkPermission('security:users', sp.EDIT)),
                    title : 'Users',
                    tabId : 'security-users',
                    tabCode : Sonatype.repoServer.UserEditPanel
                  }, {
                    enabled : sp.checkPermission('security:roles', sp.READ) && (sp.checkPermission('security:roles', sp.CREATE) || sp.checkPermission('security:roles', sp.DELETE) || sp.checkPermission('security:roles', sp.EDIT)),
                    title : 'Roles',
                    tabId : 'security-roles',
                    tabCode : Sonatype.repoServer.RoleEditPanel
                  }, {
                    enabled : sp.checkPermission('security:privileges', sp.READ) && (sp.checkPermission('security:privileges', sp.CREATE) || sp.checkPermission('security:privileges', sp.DELETE) || sp.checkPermission('security:privileges', sp.EDIT)),
                    title : 'Privileges',
                    tabId : 'security-privileges',
                    tabCode : Sonatype.repoServer.PrivilegeEditPanel
                  }]
            });

        // Config Group **************************************************
        nexusPanel.add({
              title : 'Administration',
              id : 'st-nexus-config',
              collapsed : true,
              items : [{
                    enabled : sp.checkPermission('nexus:settings', sp.READ) && (sp.checkPermission('nexus:settings', sp.CREATE) || sp.checkPermission('nexus:settings', sp.DELETE) || sp.checkPermission('nexus:settings', sp.EDIT)),
                    title : 'Server',
                    tabId : 'nexus-config',
                    tabCode : Sonatype.repoServer.ServerEditPanel,
                    tabTitle : 'nexus'
                  }, {
                    enabled : sp.checkPermission('nexus:tasks', sp.READ),
                    title : 'Scheduled Tasks',
                    tabId : 'schedules-config',
                    tabCode : Sonatype.repoServer.SchedulesEditPanel
                  }]
            });

        nexusPanel.add({
              title : 'Help',
              id : 'st-nexus-docs',
              collapsible : true,
              collapsed : true,
              items : [{
                    title : 'Welcome',
                    tabId : 'welcome',
                    tabCode : Sonatype.view.welcomePanel
                  },
                  {
                    title : 'About',
                    tabId : 'AboutNexus',
                    tabCode : Sonatype.repoServer.HelpAboutPanel
                  }, {
                    title : 'Browse Issue Tracker',
                    href : 'http://links.sonatype.com/products/nexus/oss/issues'
                  }, {
                    title : 'Documentation',
                    tabId : 'Documentation',
                    tabCode : Sonatype.repoServer.Documentation
                  }
              ]
            });
      },

      logout : function() {
        Sonatype.utils.refreshTask.stop();

        // do logout
        Ext.Ajax.request({
          scope : this,
          method : 'GET',
          url : Sonatype.config.repos.urls.logout,
          callback : function(options, success, response) {
            Sonatype.view.justLoggedOut = true;
            Sonatype.utils.loadNexusStatus();
            window.location.hash = 'welcome';
          }
        });
      },

      loginHandler : function() {
        var cp, username;

        if (Sonatype.user.curr.isLoggedIn)
        {
          this.logout();
        }
        else
        {
          this.loginForm.getForm().clearInvalid();
          cp = Sonatype.state.CookieProvider;
          username = cp.get('username', null);
          if (username)
          {
            this.loginForm.find('name', 'username')[0].setValue(username);
          }
          this.loginWindow.show();
        }
      },
      profileMenu : new Ext.menu.Menu({
        cls : 'user-profile-menu',
        shadow: false,
        items : [
          {
            text : 'Profile',
            handler : function() {
              Sonatype.view.mainTabPanel.addOrShowTab('profile', Nexus.profile.UserProfile);
            },
            listeners : {
              render : function(cmp) {
                cmp.setVisible(Sonatype.utils.editionShort === "OSS" || ( Sonatype.utils.licenseInstalled && !Sonatype.utils.licenseExpired));
              }
            }
          },
          {
            text : 'Logout',
            handler : function() {
              Sonatype.repoServer.RepoServer.loginHandler();
            }
          }
        ],
        listeners : {
          show : function() {
            Ext.get('head-link-r').addClass('profile-menu-visible')
          },
          hide : function() {
            Ext.get('head-link-r').removeClass('profile-menu-visible')
          }
        }
      }),

      /**
       * Shows the profile menu at the position of the scope of this function.
       */
      showProfileMenu : function() {
        Sonatype.repoServer.RepoServer.profileMenu.show(this);
      },

      resetMainTabPanel : function() {
        Sonatype.view.mainTabPanel.items.each(function(item, i, len) {
              this.remove(item, true);
            }, Sonatype.view.mainTabPanel);
        Sonatype.view.mainTabPanel.activeTab = null;
        Sonatype.view.supportedNexusTabs = {};

        var welcomePanelConfig = {
          layout : 'auto',
          width : 670,
          closable : false,
          items : []
        };
        var welcomeTabConfig = {
          title : 'Welcome',
          id : 'welcome',
          closable : false,
          items : [{
                layout : 'column',
                border : false,
                defaults : {
                  border : false,
                  style : 'padding-top: 30px;'
                },
                items : [{
                      columnWidth : .5,
                      html : '&nbsp;'
                    }, welcomePanelConfig, {
                      columnWidth : .5,
                      html : '&nbsp;'
                    }]
              }],
          listeners : {
            render : {
              fn : function() {
                Sonatype.Events.fireEvent('welcomeTabRender');
              },
              single : true,
              delay : 300
            }
          }
        };

        var welcomeMsg = '<p style="text-align:center;"><a href="http://nexus.sonatype.org" target="new">' + '<img src="images/nexus650x55.png" border="0" alt="Welcome to the Nexus Repository Manager"></a>' + '</p>';

        var statusEnabled = sp.checkPermission('nexus:status', sp.READ);
        if (Sonatype.utils.anonDisabled && (!Sonatype.user.curr.isLoggedIn)) {
          // show login window if anonymous access is disabled (unauthorized for status resource)
          Sonatype.repoServer.RepoServer.loginHandler();
        } else if (!statusEnabled) {
          // other error, could not retrieve status. Backend down.
          welcomeMsg += '</br><p style="color:red">Warning: Could not connect to Nexus.</p>';
        }

        welcomePanelConfig.items.push({
              border : false,
              html : '<div class="little-padding">' + welcomeMsg + '</div>'
            });

        var itemCount = welcomePanelConfig.items.length;

        Sonatype.Events.fireEvent('welcomePanelInit', this, welcomePanelConfig);
        Sonatype.Events.fireEvent('welcomeTabInit', this, welcomeTabConfig);

        // If nothing was added, then add the default blurb, if perm'd of course
        if (welcomePanelConfig.items.length <= itemCount && sp.checkPermission('nexus:repostatus', sp.READ))
        {
          welcomePanelConfig.items.push({
                layout : 'form',
                border : false,
                frame : false,
                labelWidth : 10,
                items : [{
                      border : false,
                      html : '<div class="little-padding">' + '<br/><p>You may browse the repositories using the options on the left.</p>' + '</div>'
                    }]
              });
        }

        Sonatype.view.welcomePanel = function(config) {
          var cfg = config || {},
              defaultConfig = {};

          Ext.apply(this, cfg, defaultConfig);

          Sonatype.view.welcomePanel.superclass.constructor.call(this, welcomeTabConfig );
        };

        Ext.extend(Sonatype.view.welcomePanel, Ext.Panel);

        Sonatype.view.welcomeTab = new Sonatype.view.welcomePanel(welcomeTabConfig);
        Sonatype.view.mainTabPanel.add(Sonatype.view.welcomeTab);

        // set closable to false again, mainTabPanel.defaults contains 'closable : true'
        Sonatype.view.welcomeTab.closable = false;

        // HACK: close is not a tool button, so we need to hide it manually
        Ext.get(Sonatype.view.welcomeTab.tabEl).down('a.x-tab-strip-close').toggleClass('x-hidden');

        Sonatype.view.mainTabPanel.setActiveTab(Sonatype.view.welcomeTab);
      },

    };
  })();

});

/*
 * Sonatype Nexus (TM) Open Source Version
 * Copyright (c) 2008-present Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/oss/attributions.
 *
 * This program and the accompanying materials are made available under the terms of the Eclipse Public License Version 1.0,
 * which accompanies this distribution and is available at http://www.eclipse.org/legal/epl-v10.html.
 *
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc. Apache Maven is a trademark of the Apache Software Foundation. M2eclipse is a trademark of the
 * Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global define*/
define('repoServer',
  [
    'Nexus/repository/AbstractRepoPanel',
    'Sonatype/repoServer/ArtifactContainer',
    'repoServer/PrivilegeEditPanel',
    'repoServer/RepoEditPanel',
    'repoServer/RepoMaintPanel',
    'repoServer/RepoServer',
    'repoServer/RepositoryBrowserContainer',
    'repoServer/RepoSummaryPanel',
    'repoServer/RepoTargetEditPanel',
    'repoServer/resources',
    'repoServer/RoleEditPanel',
    'repoServer/RoutesEditPanel',
    'repoServer/SchedulesEditPanel',
    'repoServer/UserBrowserPanel',
    'repoServer/UserPrivilegeBrowserPanel',
    'Sonatype/repoServer/ArtifactContainer',
    'Sonatype/repoServer/ArtifactUploadPanel',
    'Sonatype/repoServer/ConfigViewPanel',
    'Sonatype/repoServer/DefaultUserEditor',
    'Sonatype/repoServer/Documentation',
    'Sonatype/repoServer/HelpAboutPanel',
    'Sonatype/repoServer/Maven2InformationPanel',
    'Sonatype/repoServer/referenceData',
    'Sonatype/repoServer/RepositoryGroupEditor',
    'Sonatype/repoServer/RepositoryRoutingPanel',
    'Sonatype/repoServer/ServerEditPanel',
    'Sonatype/repoServer/UserEditPanel',
    'Sonatype/repoServer/UserMappingEditor'
  ], function() {

      }
);

